#! /usr/bin/env python

"""
SYNOPSIS
    Asset Register Masterfile Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import Sql, TartanDialog

class ar1010:
    def __init__(self, mf, coy, nam, usr):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.user = usr
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["chglog", "assgrp", "assmst", "assdep",
            "asstrn"], prog=__name__)
        if self.sql.error:
            return "no"

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
                "Asset Register Masterfile Maintenance (%s)" % __name__)
        grp = {
            "stype": "R",
            "tables": ("assgrp",),
            "cols": (
                ("asg_group", "", 0, "Grp"),
                ("asg_desc", "", 0, "Description", "Y")),
            "where": [("asg_cono", "=", self.conum)]}
        cod = {
            "stype": "R",
            "tables": ("assmst",),
            "cols": (
                ("asm_code", "", 0, "Code"),
                ("asm_desc", "", 0, "Description", "Y")),
            "where": [("asm_cono", "=", self.conum)],
            "whera": (("T", "asm_group", 0, 0),)}
        dep = {
            "stype": "R",
            "tables": ("assdep",),
            "cols": (
                ("asd_code", "", 0, "Cod"),
                ("asd_desc", "", 0, "Description", "Y")),
            "where": [("asd_cono", "=", self.conum)]}
        r1s = (("Straight Line","S"),("Diminishing Balance","D"))
        self.fld = (
            (("T",0,0,0),"IUA",3,"Group","Asset Group",
                "","Y",self.doGroup,grp,None,("notblank",)),
            (("T",0,1,0),"INa",7,"Code","Asset Code",
                "","N",self.doCode,cod,None,("notblank",)),
            (("T",0,2,0),"INA",30,"Description","Description",
                "","N",None,None,self.doDelete,("notblank",)),
            (("T",0,3,0),"INa",3,"Depreciation Code","Depreciation Code",
                "","N",self.doDepCode,dep,None,None),
            (("T",0,3,0),"ONA",34,""))
        but = (
            ("Accept",None,self.doAccept,0,("T",0,2),("T",0,0)),
            ("Cancel",None,self.doCancel,0,("T",0,2),("T",0,0)),
            ("Quit",None,self.doExit,1,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        abt = None
        self.df = TartanDialog(self.mf, title=tit, eflds=self.fld, butt=but,
            tend=tnd, txit=txt)

    def doGroup(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="assgrp", cols=["asg_desc"],
            where=[("asg_cono", "=", self.conum), ("asg_group", "=",w)],
            limit=1)
        if not acc:
            return "Invalid Asset Group"
        self.group = w

    def doCode(self, frt, pag, r, c, p, i, w):
        self.code = w
        self.old = self.sql.getRec(tables="assmst", where=[("asm_cono", "=",
            self.conum), ("asm_group", "=", self.group), ("asm_code", "=",
            self.code)], limit=1)
        if not self.old:
            return "Asset Does Not Exist"
        for x in range(1, len(self.old)):
            self.df.loadEntry(frt, pag, x-1, data=self.old[x])
        dep = self.sql.getRec(tables="assdep", cols=["asd_desc"],
            where=[("asd_cono", "=", self.conum), ("asd_code", "=",
            self.old[self.sql.assmst_col.index("asm_depcod")])], limit=1)
        self.df.loadEntry(frt, pag, 4, data=dep[0])

    def doDepCode(self, frt, pag, r, c, p, i, w):
        chk = self.sql.getRec(tables="assdep", cols=["asd_desc"],
            where=[("asd_cono", "=", self.conum), ("asd_code", "=", w)],
            limit=1)
        if not chk:
            return "Invalid Depreciation Code"
        self.df.loadEntry(frt, pag, p+1, data=chk[0])

    def doDelete(self):
        trs = self.sql.getRec(tables="asstrn", cols=["count(*)"],
            where=[("ast_cono", "=", self.conum), ("ast_group", "=",
            self.group), ("ast_code", "=", self.code)], limit=1)
        if trs[0]:
            return "Transactions Exist, Not Deleted"
        self.sql.delRec("assmst", where=[("asm_cono", "=", self.conum),
            ("asm_group", "=", self.group), ("asm_code", "=", self.code)])
        dte = long("%04i%02i%02i%02i%02i%02i" % time.localtime()[:-3])
        self.sql.insRec("chglog", data=["assmst", "D", "%03i%-3s%-7s" % \
            (self.conum, self.group, self.code), "", dte, self.user, "", "", 0])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doEnd(self):
        data = [self.conum]
        for x in range(0, len(self.df.t_work[0][0]) - 1):
            data.append(self.df.t_work[0][0][x])
        if data != self.old:
            self.sql.updRec("assmst", where=[("asm_cono", "=", self.conum),
                ("asm_group", "=", self.group), ("asm_code", "=", self.code)],
                data=data)
            dte = long("%04i%02i%02i%02i%02i%02i" % time.localtime()[:-3])
            for num, dat in enumerate(self.old):
                if dat != data[num]:
                    self.sql.insRec("chglog", data=["assmst", "U",
                        "%03i%-3s%-7s" % (self.conum, self.group, self.code),
                        self.sql.assmst_col[num], dte, self.user, str(dat),
                        str(data[num]), 0])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doAccept(self, widget=None):
        frt, pag, col, mes = self.df.doCheckFields()
        if mes:
            self.df.focusField(frt, pag, (col+1), err=mes)
        else:
            self.df.doEndFrame("T", 0, cnf="N")

    def doCancel(self, widget=None):
        self.mf.dbm.rollbackDbase()
        self.df.focusField("T", 0, 1)

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()
