#! /usr/bin/env python

"""
SYNOPSIS
    Asset Register Account Statements.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, Balances, CCD, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, getModName, doPrinter, showError
from tartanWork import artrtp, armvtp

class ar3030:
    def __init__(self, mf, coy, nam, period):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.period = period
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdtd = "%i/%02i/%02i" % (t[0], t[1], t[2])
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.t = None
        self.sql = Sql(self.mf.dbm, ["assgrp", "assmst"], prog=__name__)
        self.sper = self.period[1][0] / 100
        self.eper = self.period[2][0] / 100

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Asset Account Statements (%s)" % __name__)
        eper = self.sysdtw / 100
        if eper > self.eper:
            eper = self.eper
        asg = {
            "stype": "R",
            "tables": ("assgrp",),
            "cols": (
                ("asg_group", "", 0, "Grp"),
                ("asg_desc", "", 0, "Description", "Y")),
            "where": [("asg_cono", "=", self.conum)]}
        r1s = (("Yes","Y"),("No","N"))
        fld = (
            (("T",0,0,0),"ID2",7,"Start Period","Start Period",
                self.sper,"Y",self.doSdate,None,None,("efld",)),
            (("T",0,1,0),"ID2",7,"End Period","End Period",
                eper,"Y",self.doEdate,None,None,("efld",)),
            (("T",0,2,0),"IUA",3,"Product Group","Product Group",
                "","Y",self.doGroup,asg,None,None),
            (("T",0,3,0),("IRB",r1s),0,"Asset per Page","Asset per Page",
                "N","Y",self.doPage,None,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doSdate(self, frt, pag, r, c, p, i, w):
        if w < self.sper or w > self.eper:
            return "Invalid Period"
        self.sdate = w

    def doEdate(self, frt, pag, r, c, p, i, w):
        if w < self.sdate or w > self.eper:
            return "Invalid Period"
        self.edate = w

    def doGroup(self, frt, pag, r, c, p, i, w):
        if w:
            acc = self.sql.getRec(tables="assgrp", cols=["asg_desc"],
                where=[("asg_cono", "=", self.conum), ("asg_group", "=", w)],
                limit=1)
            if not acc:
                return "Invalid Group"
        self.group = w

    def doPage(self, frt, pag, r, c, p, i, w):
        self.npag = w

    def doEnd(self):
        self.df.closeProcess()
        if not self.group:
            self.sgrp = ""; self.egrp = "zzz"
        else:
            self.sgrp = self.group; self.egrp = self.group
        all = self.sql.getRec(tables="assmst", where=[("asm_cono", "=",
            self.conum), ("asm_group", ">=", self.sgrp), ("asm_group", "<=",
            self.egrp)], order="asm_group, asm_code")
        if not all:
            showError(self.mf.body, "Processing Error", "No Records Selected")
        else:
            self.printReport(all)
        self.closeProcess()

    def printReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        self.head = ("%03u %-30s %22s %33s %23s %6s" % \
            (self.conum, self.conam, "", self.sysdttm, "", __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        f = CCD(all[0][0], "UA", 3)
        oldgrp = ""
        oldcod = ""
        self.pgnum = 0
        self.pglin = 99
        mc = self.sql.assmst_col
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            self.grp = CCD(dat[mc.index("asm_group")], "UA", 3)
            self.cod = CCD(dat[mc.index("asm_code")], "Na", 7)
            self.desc = CCD(dat[mc.index("asm_desc")], "NA", 30)
            bals = Balances(self.mf, "ASS", self.conum, self.sper,  \
                keys=(self.grp.work, self.cod.work))
            asset = bals.doAssBals(start=self.sdate, end=self.edate, trans="C")
            if not asset:
                continue
            cap, cdp, rdp, cbl, rbl, mov, trn = asset
            self.bal1 = cbl
            self.bal2 = rbl
            if trn[0]:
                for z in trn[0]:
                    dt = CCD(z[trn[1].index("ast_date")], "d1", 10)
                    rf = CCD(z[trn[1].index("ast_refno")], "Na", 9)
                    tp = CCD(z[trn[1].index("ast_type")], "UI", 1)
                    bt = CCD(z[trn[1].index("ast_batch")], "Na", 7)
                    mv = CCD(z[trn[1].index("ast_mtyp")], "UI", 1)
                    m1 = CCD(z[trn[1].index("ast_amt1")], "SD", 13.2)
                    m2 = CCD(z[trn[1].index("ast_amt2")], "SD", 13.2)
                    ds = CCD(z[trn[1].index("ast_desc")], "NA", 30)
                    if self.pglin > (self.lpp - 10):
                        if self.pgnum != 0:
                            self.c.drawText(self.t)
                            self.c.showPage()
                        self.pageHeading()
                    if (oldgrp and self.grp.work != oldgrp) or \
                            (oldcod and self.cod.work != oldcod):
                        if self.npag == "Y":
                            self.c.drawText(self.t)
                            self.c.showPage()
                            self.pageHeading()
                        else:
                            self.accountHeading()
                    self.bal1 = float(ASD(self.bal1) + ASD(m1.work))
                    self.bal2 = float(ASD(self.bal2) + ASD(m2.work))
                    b1 = CCD(self.bal1, "SD", 13.2)
                    b2 = CCD(self.bal2, "SD", 13.2)
                    self.t.textLine("%s %s %s %s %s %s %s %s %s %s" % \
                        (dt.disp, rf.disp, artrtp[tp.work-1][0], bt.disp,
                        armvtp[mv.work-1][0], m1.disp, b1.disp, m2.disp,
                        b2.disp, ds.disp))
                    self.pglin = self.pglin + 1
                    oldgrp = self.grp.work
                    oldcod = self.cod.work
        p.closeProgress()
        if self.t and can != "yes":
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
            head=self.tit, view=self.df.repprt[0][1],
            prtnam=self.df.repprt[0][2], mail=self.df.repprt[1][2])
            
    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-22s %-10s %82s %5s" % \
            ("Asset Statements as at", self.sysdtd, "Page", self.pgnum))
        self.t.textLine()
        self.t.textLine("(Options: From Period %7s to Period %7s)" % \
            (self.df.t_disp[0][0][0], self.df.t_disp[0][0][1]))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 5
        self.accountHeading()

    def accountHeading(self):
        if self.pglin > (self.lpp - 10):
            self.c.drawText(self.t)
            self.c.showPage()
            self.pageHeading()
        else:
            acc = self.sql.getRec(tables="assgrp", cols=["asg_desc"],
                where=[("asg_cono", "=", self.conum), ("asg_group",
                "=", self.grp.work)], limit=1)
            if not acc:
                acc = [""]
            self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
            self.t.textLine("%s" % (self.suc * len(self.head)))
            self.t.textLine("Group %3s %s    Code %-7s %s" % (self.grp.disp,
                acc[0], self.cod.disp, self.desc.disp))
            self.t.textLine()
            self.t.textLine("%-10s %-9s %-3s %-7s %-3s %13s %13s %13s "\
                "%13s %-30s" % ("   Date", "Reference", "Typ", "BatchNo",
                "Mov", "Amount-C ", "Balance-C ", "Amount-R ", "Balance-R ",
                "Description"))
            self.t.textLine("%s" % (self.suc * len(self.head)))
            self.t.setFont("Tartan", self.cpi, self.ldg)
            b1 = CCD(self.bal1, "SD", 13.2)
            b2 = CCD(self.bal2, "SD", 13.2)
            self.t.textLine("%-50s %-13s %-13s %-13s %-30s" % \
                ("", b1.disp, "", b2.disp, "Balance Forward"))
            self.pglin = self.pglin + 6

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()
