#! /usr/bin/python

"""
SYNOPSIS
    Asset's Register Key Change.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    is granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN; if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import GetCtl, Sql, TartanDialog

class ar6010:
    def __init__(self, mf, conum, conam):
        self.mf = mf
        self.conum = conum
        self.conam = conam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.tables = (
            ("ctlnot", "not_key"),
            ("ctlvtf", "vtt_cono", "vtt_acno", "vtt_styp"),
            ("assmst", "asm_cono", "asm_group", "asm_code"),
            ("asstrn", "ast_cono", "ast_group", "ast_code"))
        tabs = ["assgrp"]
        for tab in self.tables:
            if not tabs.count(tab[0]):
                tabs.append(tab[0])
        self.sql = Sql(self.mf.dbm, tabs, prog=__name__)
        gc = GetCtl(self.mf)
        assctl = gc.getCtl("assctl", self.conum)
        if not assctl:
            return "no"

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "Asset's Register Key Changes")
        grp = {
            "stype": "R",
            "tables": ("assgrp",),
            "cols": (
                ("asg_group", "", 0, "Grp"),
                ("asg_desc", "", 0, "Description", "Y")),
            "where": [("asg_cono", "=", self.conum)]}
        asm = {
            "stype": "R",
            "tables": ("assmst",),
            "cols": (
                ("asm_code", "", 0, "Code"),
                ("asm_desc", "", 0, "Description", "Y")),
            "where": [("asm_cono", "=", self.conum)],
            "whera": [["T", "asm_group", 0]]}
        fld = [
            (["T",0,0,0],"I@asm_group",0,"Old Group","Old Group",
                "","Y",self.doOldGrp,grp,None,("notblank",)),
            (["T",0,0,12],"I@asm_code",0,"Old Code","Old Product Code",
                "","Y",self.doOldCod,asm,None,("notblank",)),
            (["T",0,0,27],"O@asm_desc",0,""),
            (["T",0,1,0],"I@asm_group",0,"New Group","New Group",
                "","Y",self.doNewGrp,grp,None,("notblank",)),
            (["T",0,1,12],"I@asm_code",0,"New Code","New Product Code",
                "","Y",self.doNewCod,None,None,("notblank",))]
        tnd = ((self.doProcess,"y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tend=tnd,
            txit=txt)

    def doOldGrp(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="assgrp", where=[("asg_cono", "=",
            self.conum), ("asg_group", "=", w)], limit=1)
        if not acc:
            return "Invalid Group, Does Not exist"
        self.oldgrp = w

    def doOldCod(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="assmst", where=[("asm_cono", "=",
            self.conum), ("asm_group", "=", self.oldgrp), ("asm_code", "=",
            w)], limit=1)
        if not acc:
            return "Invalid Code, Does Not exist"
        self.oldcod = w
        self.oldnot = "%3s%s" % (self.oldgrp, w)
        self.df.loadEntry(frt, pag, p+1, data=acc[3])

    def doNewGrp(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="assgrp", where=[("asg_cono", "=",
            self.conum), ("asg_group", "=", w)], limit=1)
        if not acc:
            return "Invalid Group, Does Not exist"
        self.newgrp = w

    def doNewCod(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="assmst", where=[("asm_cono", "=",
            self.conum), ("asm_group", "=", self.newgrp), ("asm_code", "=",
            w)], limit=1)
        if acc:
            return "Invalid Code, Already Exists"
        self.newcod = w
        self.newnot = "%3s%s" % (self.newgrp, w)

    def doProcess(self):
        for tab in self.tables:
            if tab[0] == "ctlnot":
                whr = [("not_cono", "=", self.conum), ("not_sys", "=", "ASS"),
                    (tab[1], "=", self.oldnot)]
                dat = [self.newnot]
                col = [tab[1]]
            elif tab[0] == "ctlvtf":
                whr = [(tab[1], "=", self.conum), (tab[2], "=", self.oldcod),
                    (tab[3], "=", "A")]
                dat = [self.newcod]
                col = [tab[2]]
            else:
                whr = [(tab[1], "=", self.conum), (tab[2], "=", self.oldgrp),
                    (tab[3], "=", self.oldcod)]
                dat = [self.newgrp, self.newcod]
                col = [tab[2], tab[3]]
            self.sql.updRec(tab[0], where=whr, data=dat, cols=col)
        self.df.focusField("T", 0, 1)

    def doExit(self):
        self.df.closeProcess()
        self.mf.dbm.commitDbase(ask=True)
        self.mf.closeLoop()

