#! /usr/bin/env python

"""
SYNOPSIS
    Asset Groups File Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import GetCtl, Sql, TartanDialog

class arc310:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["assdep", "assgrp", "genmst"],
            prog=__name__)
        if self.sql.error:
            return "no"
        gc = GetCtl(self.mf)
        assctl = gc.getCtl("assctl", self.conum)
        if not assctl:
            return "no"
        self.glint = assctl["cta_glint"]

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
                "Asset Groups File Maintenance (%s)" % __name__)
        grp = {
            "stype": "R",
            "tables": ("assgrp",),
            "cols": (
                ("asg_group", "", 0, "Grp"),
                ("asg_desc", "", 0, "Description", "Y")),
            "where": [("asg_cono", "=", self.conum)]}
        dep = {
            "stype": "R",
            "tables": ("assdep",),
            "cols": (
                ("asd_code", "", 0, "Cod"),
                ("asd_desc", "", 0, "Description", "Y")),
            "where": [("asd_cono", "=", self.conum)]}
        glm = {
            "stype": "R",
            "tables": ("genmst",),
            "cols": (
                ("glm_acno", "", 0, "Acc-Num"),
                ("glm_desc", "", 0, "Description", "Y")),
            "where": [("glm_cono", "=", self.conum)],
            "order": "glm_acno"}
        self.fld = [
            (("T",0,0,0),"IUA",3,"Asset Group","Asset Group",
                "","N",self.doGroup,grp,None,("notblank",)),
            (("T",0,1,0),"INA",30,"Description","Description",
                "","N",None,None,self.doDelete,("notblank",)),
            (("T",0,2,0),"INa",3,"Depreciation Code","Depreciation Code",
                "","N",self.doDepCode,dep,None,None),
            (("T",0,2,0),"ONA",34,"")]
        if self.glint == "Y":
            self.fld.append((("T",0,4,0),"IUI",7,"Asset Account",
                "Asset Account","","N",self.doAsset,glm,None,("notzero",)))
            self.fld.append((("T",0,4,0),"ONA",30,""))
            self.fld.append((("T",0,5,0),"IUI",7,"Accum Account",
                "Accum Account","","N",self.doAccum,glm,None,("notzero",)))
            self.fld.append((("T",0,5,0),"ONA",30,""))
            self.fld.append((("T",0,6,0),"IUI",7,"Expense Account",
                "Expense Account","","N",self.doExpense,glm,None,("notzero",)))
            self.fld.append((("T",0,6,0),"ONA",30,""))
        but = (
            ("Accept",None,self.doAccept,0,("T",0,2),("T",0,0)),
            ("Cancel",None,self.doCancel,0,("T",0,2),("T",0,0)),
            ("Quit",None,self.doExit,1,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        abt = None
        self.df = TartanDialog(self.mf, title=tit, eflds=self.fld, butt=but,
            tend=tnd, txit=txt)

    def doGroup(self, frt, pag, r, c, p, i, w):
        self.group = w
        acc = self.sql.getRec(tables="assgrp", where=[("asg_cono", "=",
            self.conum), ("asg_group", "=", self.group)], limit=1)
        if not acc:
            self.new = "Y"
        else:
            self.new = "N"
            self.df.loadEntry(frt, pag, p+1, data=acc[2])
            self.df.loadEntry(frt, pag, p+2, data=acc[3])
            dep = self.sql.getRec(tables="assdep", cols=["asd_desc"],
                where=[("asd_cono", "=", self.conum), ("asd_code", "=",
                acc[3])], limit=1)
            self.df.loadEntry(frt, pag, p+3, data=dep[0])
            if self.glint == "N":
                return
            self.df.loadEntry(frt, pag, p+4, data=acc[4])
            des = self.sql.getRec(tables="genmst", cols=["glm_desc"],
                where=[("glm_cono", "=", self.conum), ("glm_acno", "=",
                acc[4])], limit=1)
            if des:
                self.df.loadEntry("T", pag, p+5, data=des[0])
            self.df.loadEntry(frt, pag, p+6, data=acc[4])
            des = self.sql.getRec(tables="genmst", cols=["glm_desc"],
                where=[("glm_cono", "=", self.conum), ("glm_acno", "=",
                acc[5])], limit=1)
            if des:
                self.df.loadEntry("T", pag, p+7, data=des[0])
            self.df.loadEntry(frt, pag, p+8, data=acc[4])
            des = self.sql.getRec(tables="genmst", cols=["glm_desc"],
                where=[("glm_cono", "=", self.conum), ("glm_acno", "=",
                acc[6])], limit=1)
            if des:
                self.df.loadEntry("T", pag, p+9, data=des[0])

    def doDepCode(self, frt, pag, r, c, p, i, w):
        chk = self.sql.getRec(tables="assdep", cols=["asd_desc"],
            where=[("asd_cono", "=", self.conum), ("asd_code", "=", w)],
            limit=1)
        if not chk:
            return "Invalid Depreciation Code"
        self.df.loadEntry(frt, pag, p+1, data=chk[0])

    def doAsset(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="genmst", cols=["glm_desc"],
            where=[("glm_cono", "=", self.conum), ("glm_acno", "=", w)],
            limit=1)
        if not acc:
            return "Invalid G/L Account"
        self.df.loadEntry("T", pag, p+1, data=acc[0])

    def doAccum(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="genmst", cols=["glm_desc"],
            where=[("glm_cono", "=", self.conum), ("glm_acno", "=", w)],
            limit=1)
        if not acc:
            return "Invalid G/L Account"
        self.df.loadEntry("T", pag, p+1, data=acc[0])

    def doExpense(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="genmst", cols=["glm_desc"],
            where=[("glm_cono", "=", self.conum), ("glm_acno", "=", w)],
            limit=1)
        if not acc:
            return "Invalid G/L Account"
        self.df.loadEntry("T", pag, p+1, data=acc[0])

    def doDelete(self):
        chk = self.sql.getRec(tables="assmst", where=[("asm_cono", "=",
            self.conum), ("asm_group", "=", self.group)])
        if chk:
            return "Group in Use, Not Deleted"
        self.sql.delRec("assgrp", where=[("asg_cono", "=", self.conum),
            ("asg_group", "=", group)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doEnd(self):
        data = [self.conum]
        for x in range(0, len(self.df.t_work[0][0])):
            if x == 3 or (self.glint == "Y" and x in (5, 7, 9)):
                continue
            data.append(self.df.t_work[0][0][x])
        if self.glint == "N":
            data.extend([0, 0, 0])
        if self.new == "Y":
            self.sql.insRec("assgrp", data=data)
        else:
            self.sql.updRec("assgrp", data=data, where=[("asg_cono", "=",
            self.conum), ("asg_group", "=", self.group)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doAccept(self, widget=None):
        frt, pag, col, mes = self.df.doCheckFields()
        if mes:
            self.df.focusField(frt, pag, (col+1), err=mes)
        else:
            self.df.doEndFrame("T", 0, cnf="N")

    def doCancel(self, widget=None):
        self.mf.dbm.rollbackDbase()
        self.df.focusField("T", 0, 1)

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()
