#! /usr/bin/python

"""
SYNOPSIS
    Competition Types Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import TartanDialog, Sql
from tartanFunctions import getNextCode

class bc1030:
    def __init__(self, mf):
        self.mf = mf
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["bwltyp"], prog=__name__)

    def mainProcess(self):
        tit = ("Competition Types Maintenance",)
        typ = {
            "stype": "R",
            "tables": ("bwltyp",),
            "cols": (
                ("bct_code", "", 0, "Cod"),
                ("bct_desc", "", 0, "Description", "Y"))}
        r1s = (("Yes", "Y"), ("No", "N"))
        fld = (
            (("T",0,0,0),"I@bct_code",0,"","",
                0,"Y",self.doTypCod,typ,None,("efld",)),
            (("T",0,1,0),"I@bct_desc",0,"","",
                "","N",None,None,self.doDelete,("notblank",)),
            (("T",0,2,0),"I@bct_games",0,"","",
                0,"N",None,None,None,("notzero",)),
            (("T",0,3,0),"I@bct_ends",0,"","",
                0,"N",self.doEnds,None,None,("notzero",)),
            (("T",0,4,0),("IRB",r1s),0,"Groups","Groups",
                "N","N",self.doGroups,None,None,None),
            (("T",0,5,0),"I@bct_grgame",0,"","",
                0,"N",self.doGrGame,None,None,("efld",)),
            (("T",0,6,0),("IRB",r1s),0,"Adjust Scores","Adjust Scores",
                "N","N",self.doAdjust,None,None,None),
            (("T",0,7,0),"I@bct_expunge",0,"","",
                0,"N",None,None,None,("efld",)),
            (("T",0,8,0),"I@bct_percent",0,"","",
                100,"N",None,None,None,("efld",)),
            (("T",0,9,0),"I@bct_drawn",0,"","",
                1,"N",None,None,None,("efld",)),
            (("T",0,10,0),("IRB",r1s),0,"Skins","Skins",
                "N","N",self.doSkins,None,None,None),
            (("T",0,11,0),"I@bct_sends",0,"","",
                0,"N",self.doEndsPerSkin,None,None,("efld",)),
            (("T",0,12,0),"I@bct_e_points",0,"","",
                0,"N",self.doPoints,None,None,("efld",)),
            (("T",0,13,0),"I@bct_s_points",0,"","",
                0,"N",None,None,None,("efld",)),
            (("T",0,14,0),"I@bct_g_points",0,"","",
                0,"N",None,None,None,("efld",)),
            (("T",0,15,0),("IRB",r1s),0,"Bonus Points","Bonus Points",
                "N","N",self.doBonus,None,None,None),
            (("T",0,16,0),"I@bct_win_by",0,"","",
                0,"N",None,None,None,("efld",)),
            (("T",0,17,0),"I@bct_lose_by",0,"","",
                0,"N",None,None,None,("efld",)))
        tnd = ((self.doEnd,"y"),)
        txt = (self.doExit,)
        self.df = TartanDialog(self.mf, tops=False, title=tit, eflds=fld,
            tend=tnd, txit=txt, clicks=self.doClick)

    def doClick(self, *opts):
        if self.df.col == 1:
            return
        if self.click:
            self.df.loadEntry("T", self.click[0], self.click[1],
                data=self.df.t_work[self.click[0]][0][self.click[1]])
        self.click = opts[2]
        self.df.focusField("T", opts[2][0], opts[2][1] + 1)

    def doTypCod(self, frt, pag, r, c, p, i, w):
        self.click = []
        if not w:
            self.ctyp = getNextCode(self.sql, "bwltyp", "bct_code", last=99)
            self.df.loadEntry(frt, pag, p, data=self.ctyp)
        else:
            self.ctyp = w
        chk = self.sql.getRec(tables="bwltyp", where=[("bct_code", "=",
            self.ctyp)], limit=1)
        if not chk:
            self.newtyp = True
        else:
            self.newtyp = False
            for num, fld in enumerate(chk):
                self.df.loadEntry(frt, pag, num, data=fld)

    def doDelete(self, widget=None):
        if self.newtyp:
            return
        error = False
        for tab in (("bwlcmp", "bcm_type"),):
            if self.sql.getRec(tables=tab[0], where=[(tab[1], "=", self.ctyp)], limit=1):
                error = True
                break
        if error:
            return "There are Movements for this Type, Not Deleted"
        self.sql.delRec("bwltyp", where=[("bct_code", "=", self.ctyp)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doEnds(self, frt, pag, r, c, p, i, w):
        self.ends = w

    def doGroups(self, frt, pag, r, c, p, i, w):
        self.groups = w
        if self.groups == "N":
            self.adjust = "N"
            self.df.loadEntry(frt, pag, p+1, data="")
            self.df.loadEntry(frt, pag, p+2, data=self.adjust)
            self.df.loadEntry(frt, pag, p+3, data="")
            self.df.loadEntry(frt, pag, p+4, data="")
            return "sk4"

    def doGrGame(self, frt, pag, r, c, p, i, w):
        if not w:
            return "Invalid Game Number"

    def doAdjust(self, frt, pag, r, c, p, i, w):
        self.adjust = w
        if self.adjust == "N":
            self.df.loadEntry(frt, pag, p+1, data=0)
            self.df.loadEntry(frt, pag, p+2, data=0)
            return "sk2"

    def doSkins(self, frt, pag, r, c, p, i, w):
        self.skins = w
        if self.skins == "N":
            self.df.loadEntry(frt, pag, p+1, data=0)
            return "sk1"

    def doEndsPerSkin(self, frt, pag, r, c, p, i, w):
        if self.ends % w:
            return "Invalid Ends per Skin, Not Divisible"

    def doPoints(self, frt, pag, r, c, p, i, w):
        if self.skins == "N":
            self.df.loadEntry(frt, pag, p+1, data=0)
            return "sk1"

    def doBonus(self, frt, pag, r, c, p, i, w):
        if w == "N":
            self.df.loadEntry(frt, pag, p+1, data=0)
            self.df.loadEntry(frt, pag, p+2, data=0)
            return "nd"

    def doEnd(self, widget=None):
        if widget:
            frt, pag, col, mes = self.df.doCheckFields()
            if mes:
                self.df.focusField(frt, pag, (col+1), err=mes)
                return
        data = self.df.t_work[0][0]
        if self.newtyp:
            self.sql.insRec("bwltyp", data=data)
        else:
            self.sql.updRec("bwltyp", data=data,
                where=[("bct_code", "=", self.ctyp)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()
