#! /usr/bin/python

"""
SYNOPSIS
    Bowls Competition Entry Forms.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import RepPrt, Sql, TartanDialog
from tartanFunctions import askChoice, askQuestion, getNextCode

class bc2010:
    def __init__(self, mf):
        self.mf = mf
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["bwlcmp", "bwlskp", "bwlclb", "bwlent",
            "bwltyp", "bwlgme"], prog=__name__)

    def mainProcess(self):
        tit = ("Competition Entry Forms",)
        cmp = {
            "stype": "R",
            "tables": ("bwlcmp",),
            "cols": (
                ("bcm_code", "", 0, "Cod"),
                ("bcm_name", "", 0, "Name", "Y"),
                ("bcm_date", "", 0, "Date"))}
        typ = {
            "stype": "R",
            "tables": ("bwltyp",),
            "cols": (
                ("bct_code", "", 0, "Cod"),
                ("bct_desc", "", 0, "Description", "Y"))}
        sk1 = {
            "stype": "R",
            "tables": ("bwlskp",),
            "cols": (
                ("bcs_code", "", 0, "Cod"),
                ("bcs_surname", "", 0, "Surname", "Y"),
                ("bcs_names", "", 0, "Names")),
            "order": "bcs_surname"}
        sk2 = {
            "stype": "R",
            "tables": ("bwlent", "bwlskp"),
            "cols": (
                ("bcs_code", "", 0, "Cod"),
                ("bcs_surname", "", 0, "Surame", "Y"),
                ("bcs_names", "", 0, "Names"),
                ("bce_paid", "", 0, "P")),
            "where": [("bcs_code=bce_scod",)],
            "whera": [("T", "bce_ccod", 0, 0)],
            "order": "bcs_surname",
            "butt": [("Print", self.doPrint)]}
        clb = {
            "stype": "R",
            "tables": ("bwlclb",),
            "cols": (
                ("bcc_code", "", 0, "Cod"),
                ("bcc_name", "", 0, "Name", "Y")),
            "butt": [("New Club", self.doNewClub)],
            "order": "bcc_name"}
        r1s = (("Male", "M"), ("Female", "F"))
        r2s = (("Yes", "Y"), ("No", "N"))
        tag = (("Skips",None,("T",0,0),("T",0,1)),)
        fld = (
            (("T",0,0,0),"I@bcm_code",0,"","",
                "","Y",self.doCmpCod,cmp,None,("efld",)),
            (("T",0,0,0),"I@bcm_name",0,"Name","",
                "","N",self.doCmpNam,None,None,("notblank",)),
            (("T",0,0,0),"I@bcm_date",0,"Date","",
                "","N",None,None,None,("efld",)),
            (("T",0,0,0),"I@bcm_type",0,"Type","",
                "","N",self.doCmpTyp,typ,None,("efld",)),
            (("T",1,0,0),"I@bcs_code",0,"","",
                "","Y",self.doSkpCod,sk1,None,("efld",)),
            (("T",1,1,0),"I@bcs_title",0,"","",
                "","N",None,None,None,("notblank",)),
            (("T",1,2,0),"I@bcs_surname",0,"","",
                "","N",None,None,None,("notblank",)),
            (("T",1,3,0),"I@bcs_names",0,"","",
                "","N",None,None,None,("notblank",)),
            (("T",1,4,0),("IRB",r1s),0,"Gender","Gender",
                "M","N",None,None,None,None),
            (("T",1,5,0),"I@bcs_club",0,"","",
                "","N",self.doClbCod,clb,None,("efld",)),
            (("T",1,5,0),"INA",30,"","Club Name",
                "","N",self.doClbNam,None,None,("notblank",)),
            (("T",1,6,0),"I@bcs_add1",0,"","",
                "","N",None,None,None,("notblank",)),
            (("T",1,7,0),"I@bcs_add2",0,"","",
                "","N",None,None,None,("efld",)),
            (("T",1,8,0),"I@bcs_add3",0,"","",
                "","N",None,None,None,("efld",)),
            (("T",1,9,0),"I@bcs_pcod",0,"","",
                "","N",None,None,None,("notblank",)),
            (("T",1,10,0),"I@bcs_email",0,"","",
                "","N",None,None,None,("efld",)),
            (("T",1,11,0),"I@bcs_phone",0,"","",
                "","N",None,None,None,("efld",)),
            (("T",1,12,0),("IRB",r2s),0,"Paid","Paid",
                "N","N",None,None,None,None))
        but = (
            ("Accept",None,self.doEnd,0,("T",1,2),("T",1,0)),
            ("Entered Skips",sk2,None,0,("T",1,1),("T",0,1)))
        tnd = ((self.doEnd,"y"), (self.doEnd,"y"))
        txt = (self.doExit, self.doExit)
        self.df = TartanDialog(self.mf, tops=False, title=tit, tags=tag,
            eflds=fld, butt=but,tend=tnd, txit=txt)

    def doCmpCod(self, frt, pag, r, c, p, i, w):
        if not w:
            self.ccod = getNextCode(self.sql, "bwlcmp", "bcm_code", last=99)
            self.df.loadEntry(frt, pag, p, data=self.ccod)
        else:
            self.ccod = w
        chk = self.sql.getRec(tables="bwlcmp", where=[("bcm_code", "=",
            self.ccod)], limit=1)
        if not chk:
            ok = askQuestion(self.mf.body, head="New Competition",
                mess="Is This a New Competition?", default="no")
            if ok == "no":
                return "rf"
            self.newcmp = True
        else:
            game = self.sql.getRec(tables="bwlgme", cols=["count(*)"],
                where=[("bcg_ccod", "=", self.ccod), ("bcg_ocod", "<>", 0)],
                limit=1)
            if game[0]:
                return "This Competition Has Already Started"
            self.newcmp = False
            self.typ = self.sql.getRec(tables="bwltyp", where=[("bct_code",
                "=", chk[3])], limit=1)
            for num, fld in enumerate(chk):
                self.df.loadEntry(frt, pag, p+num, data=fld)

    def doCmpNam(self, frt, pag, r, c, p, i, w):
        self.cmpnam = w

    def doCmpTyp(self, frt, pag, r, c, p, i, w):
        self.typ = self.sql.getRec(tables="bwltyp", where=[("bct_code", "=",
            w)], limit=1)
        if not self.typ:
            return "Invalid Competition Type"

    def doSkpCod(self, frt, pag, r, c, p, i, w):
        if not w:
            self.scod = getNextCode(self.sql, "bwlskp", "bcs_code", last=999)
            self.df.loadEntry(frt, pag, p, data=self.scod)
        else:
            self.scod = w
        # Initialise Club Variables
        self.newclb = False
        self.club = None
        #
        chk = self.sql.getRec(tables="bwlskp", where=[("bcs_code", "=",
            self.scod)], limit=1)
        if not chk:
            self.newskp = True
            self.newent = True
        else:
            self.newskp = False
            ent = self.sql.getRec(tables="bwlent", where=[("bce_ccod", "=",
                self.ccod), ("bce_scod", "=", self.scod)], limit=1)
            if ent:
                bt = (("Delete", 1), ("Amend", 2), ("Ignore", 3))
                ok = askChoice(self.mf.body, head="Skip Already Entered",
                    mess="This Skip, %s %s %s, Has Already Been Entered!" % \
                    (chk[1], chk[3], chk[2]), butt=bt, default=3)
                if ok not in (1, 2):
                    return "rf"
                if ok == 1:
                    self.sql.delRec("bwlent", where=[("bce_ccod", "=",
                        self.ccod), ("bce_scod", "=", self.scod)])
                    self.sql.delRec("bwlgme", where=[("bcg_ccod", "=",
                        self.ccod), ("bcg_scod", "=", self.scod)])
                    self.mf.dbm.commitDbase()
                    self.df.clearFrame("T", 1)
                    return "ff1"
                self.newent = False
            else:
                self.newent = True
            for num, fld in enumerate(chk):
                if num > 5:
                    pos = num + 1
                else:
                    pos = num
                self.df.loadEntry(frt, pag, pos, data=fld)
                if num == 5:
                    self.club = fld
                    clb = self.sql.getRec(tables="bwlclb",
                        where=[("bcc_code", "=", fld)], limit=1)
                    self.df.loadEntry(frt, pag, pos+1, data=clb[1])
            if not self.newent:
                self.df.loadEntry("T", 1, 13, data=ent[2])
                return "ff14"
            if not self.newskp:
                self.df.loadEntry("T", 1, 13, data="N")
                return "ff14"

    def doClbCod(self, frt, pag, r, c, p, i, w):
        if not w:
            self.club = getNextCode(self.sql, "bwlclb", "bcc_code", last=999)
            self.df.loadEntry(frt, pag, p, data=self.club)
        else:
            self.club = w
        chk = self.sql.getRec(tables="bwlclb", where=[("bcc_code", "=",
            self.club)], limit=1)
        if not chk:
            ok = askQuestion(self.mf.body, head="New Club",
                mess="This Club Code Does Not Exist, Do You Want To Create It?")
            if ok == "no":
                return "rf"
            self.doNewClub(self.club)
        else:
            self.newclb = False
            self.df.loadEntry(frt, pag, p+1, data=chk[1])
            return "sk1"

    def doNewClub(self, club=None):
        self.newclb = True
        if not club:
            self.club = getNextCode(self.sql, "bwlclb", "bcc_code", last=999)
        else:
            self.club = club

    def doClbNam(self, frt, pag, r, c, p, i, w):
        if not self.club:
            return "ff6|Invalid Club Code"

    def doEnd(self, widget=None):
        if widget:
            frt, pag, col, mes = self.df.doCheckFields()
            if mes:
                if col == 6:
                    self.df.focusField(frt, pag, (col), err=mes)
                else:
                    self.df.focusField(frt, pag, (col+1), err=mes)
                return
        if self.df.pag == 0:
            if self.newcmp:
                self.sql.insRec("bwlcmp", data=self.df.t_work[0][0])
            else:
                self.sql.updRec("bwlcmp", data=self.df.t_work[0][0],
                    where=[("bcm_code", "=", self.ccod)])
        elif self.df.pag == 1:
            data = self.df.t_work[1][0][:6]
            data.extend(self.df.t_work[1][0][7:-1])
            if self.newskp:
                self.sql.insRec("bwlskp", data=data)
            else:
                self.sql.updRec("bwlskp", data=data,
                    where=[("bcs_code", "=", self.scod)])
            if self.newclb:
                self.sql.insRec("bwlclb", data=[self.club,
                    self.df.t_work[1][0][6]])
            if self.newent:
                self.sql.insRec("bwlent", data=[self.ccod, self.scod,
                    self.df.t_work[1][0][13]])
                # Populate bwlgme records
                num = self.typ[self.sql.bwltyp_col.index("bct_games")]
                dgm = self.typ[self.sql.bwltyp_col.index("bct_drawn")]
                data = [self.ccod, self.scod, 0, "", 0, 0, "", 0, 0,0,0, 0,0,0]
                for x in range(0, dgm):
                    data[2] = x + 1
                    data[3] = "D"
                    self.sql.insRec("bwlgme", data=data)
                for x in range(dgm, num):
                    data[2] = x + 1
                    data[3] = "S"
                    self.sql.insRec("bwlgme", data=data)
            else:
                self.sql.updRec("bwlent", cols=["bce_paid"],
                    data=[self.df.t_work[1][0][13]], where=[("bce_ccod", "=",
                    self.ccod), ("bce_scod", "=", self.scod)])
        self.mf.dbm.commitDbase()
        self.df.clearFrame("T", 1)
        self.df.focusField("T", 1, 1)

    def doPrint(self, widget=None):
        self.df.mstAlw.hide()
        hdr = ["Skips Entered for the %s" % self.cmpnam]
        tab = ["bwlent", "bwlskp"]
        col = ["bcs_code", "bcs_title", "bcs_surname", "bcs_names",
            "bcs_phone", ["bcs_email", "TX", 30, "Email-Address"], "bce_paid"]
        whr = [("bce_ccod", "=", self.ccod), ("bcs_code=bce_scod",)]
        odr = "bcs_surname"
        prt = RepPrt(self.mf, name=__name__, heads=hdr, tables=tab, cols=col,
            where=whr, order=odr, prtdia=(("Y","V"),("Y","N")))
        self.df.mstAlw.show()

    def doExit(self):
        if self.df.pag == 0:
            self.df.closeProcess()
            self.mf.closeLoop()
        elif self.df.pag == 1:
            self.df.clearFrame("T", 1)
            self.df.focusField("T", 0, 1)
