#! /usr/bin/python

"""
SYNOPSIS
    Bowls Competition Draw and Match Cards.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import copy, os, random, time
from pyPdf import PdfFileWriter, PdfFileReader
from TartanClasses import CCD, RepPrt, TartanDialog, Sql
from tartanFunctions import askQuestion, callModule, doPrinter, doReportlab
from tartanFunctions import getModName, getSingleRecords, showError, showWarning

class bc2020:
    def __init__(self, mf):
        self.mf = mf
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        t = time.localtime()
        self.today = ((t[0] * 10000) + (t[1] * 100) + t[2])
        self.sql = Sql(self.mf.dbm, ["bwlcmp", "bwlgme", "bwlskp", "bwltyp",
            "bwlent"], prog=__name__)
        self.game = 0
        random.seed()

    def mainProcess(self):
        tit = ("Bowls Competition Match Cards",)
        cmp = {
            "stype": "R",
            "tables": ("bwlcmp",),
            "cols": (
                ("bcm_code", "", 0, "Cod"),
                ("bcm_name", "", 0, "Name", "Y"))}
        gme = {
            "stype": "R",
            "tables": ("bwlgme",),
            "cols": (
                ("bcg_game", "", 0, "Cod"),
                ("bcg_type", "", 0, "T"),
                ("bcg_date", "", 0, "Date")),
            "whera": [("T", "bcg_ccod", 0, 0)],
            "group": "bcg_game, bcg_type, bcg_date",
            "order": "bcg_game"}
        r1s = (("No", "N"),("Yes","Y"),("Only", "O"))
        r2s = (("Yes","Y"),("No", "N"))
        fld = (
            (("T",0,0,0),"I@bcm_code",0,"","",
                0,"N",self.doCmpCod,cmp,None,None),
            (("T",0,1,0),"I@bcg_game",0,"","",
                self.game,"N",self.doGamNum,gme,None,("efld",)),
            (("T",0,2,0),"I@bcg_date",0,"","",
                self.today,"N",self.doGamDte,None,None,("efld",)),
            (("T",0,3,0),"IUI",1,"Number of Groups","Number of Groups",
                0,"N",self.doGrpNum,None,None,("in", (0, 2, 3))),
            (("T",0,4,0),"IUA",30,"Greens","Greens (A,B,C)",
                "","N",self.doGreens,None,None,("efld",),None,"Available "\
                "Greens in the format A,B or A,B345 showing Green Code and "\
                "Rinks. If the Rinks are Not Entered they will Default to 6."),
            (("T",0,5,0),("IRB",r1s),0,"Print Cards","Print Cards",
                "N","N",self.doPrtCards,None,None,None),
            (("T",0,6,0),("IRB",r2s),0,"All Cards","All Cards",
                "Y","N",self.doAllCards,None,None,None))
        tnd = ((self.doEnd,"y"),)
        txt = (self.doExit,)
        self.df = TartanDialog(self.mf, tops=False, title=tit, eflds=fld,
            tend=tnd, txit=txt, view=("N","V"))

    def doCmpCod(self, frt, pag, r, c, p, i, w):
        cmp = self.sql.getRec(tables="bwlcmp", where=[("bcm_code", "=", w)],
            limit=1)
        if not cmp:
            return "Invalid Competition Code"
        self.totskp = self.sql.getRec(tables="bwlent", cols=["count(*)"],
            where=[("bce_ccod", "=", w)], limit=1)[0]
        if self.totskp % 2:
            showError(self.mf.body, "Mismatch",
                "There is an Uneven Number of Teams (%s)" % self.totskp)
            return "rf"
        self.ccod = w
        self.reprint = False
        self.cdes = cmp[self.sql.bwlcmp_col.index("bcm_name")]
        self.sdat = cmp[self.sql.bwlcmp_col.index("bcm_date")]
        ctyp = cmp[self.sql.bwlcmp_col.index("bcm_type")]
        cmp = self.sql.getRec(tables="bwltyp", where=[("bct_code", "=", ctyp)],
            limit=1)
        self.games = cmp[self.sql.bwltyp_col.index("bct_games")]
        self.ends = cmp[self.sql.bwltyp_col.index("bct_ends")]
        self.groups = cmp[self.sql.bwltyp_col.index("bct_groups")]
        self.grgame = cmp[self.sql.bwltyp_col.index("bct_grgame")]
        self.adjust = cmp[self.sql.bwltyp_col.index("bct_adjust")]
        expunge = cmp[self.sql.bwltyp_col.index("bct_expunge")].split(",")
        self.expunge = []
        for ex in expunge:
            if ex:
                self.expunge.append(int(ex))
        self.percent = cmp[self.sql.bwltyp_col.index("bct_percent")]
        self.skins = cmp[self.sql.bwltyp_col.index("bct_skins")]
        self.sends = cmp[self.sql.bwltyp_col.index("bct_sends")]
        gme = self.sql.getRec(tables="bwlgme", cols=["bcg_game", "bcg_type",
            "sum(bcg_points)", "sum(bcg_ocod)"], where=[("bcg_ccod", "=",
            self.ccod)], group="bcg_game, bcg_type", order="bcg_game")
        for game in gme:
            self.game = game[0]
            self.gtyp = game[1]
            if not game[2]:
                self.df.loadEntry(frt, pag, p+1, data=self.game)
                return
        ok = askQuestion(self.mf.body, "All Printed",
            "All Matches are Already Printed, Do You Want to Reprint?")
        if ok == "yes":
            self.reprint = True
        else:
            return "rf"

    def doGamNum(self, frt, pag, r, c, p, i, w):
        if w > self.game:
            return "Invalid Game Number, Previous Matches Unfinished"
        chk = self.sql.getRec(tables="bwlgme", cols=["bcg_game", "bcg_type",
            "bcg_date", "sum(bcg_ocod)"], where=[("bcg_ccod", "=", self.ccod),
            ("bcg_game", "=", w)], limit=1)
        if not chk[0]:
            return "Invalid Game Number, Does Not Exist"
        if not self.reprint and chk[3]:
            ok = askQuestion(self.mf.body, "Match Printed",
                "This Match is Already Printed, Do You Want to Reprint?")
            if ok == "yes":
                self.reprint = True
            else:
                return "Invalid Game Number, Already Printed"
        if self.reprint:
            self.game = chk[0]
            self.gtyp = chk[1]
            self.date = chk[2]
            self.datd = CCD(self.date, "D1", 10).disp
            self.df.loadEntry(frt, pag, p+1, data=self.date)
            self.df.topEntry[0][5][2][0].show()
            return "sk3"
        self.df.topEntry[0][5][2][0].hide()

    def doGamDte(self, frt, pag, r, c, p, i, w):
        if w < self.today or w < self.sdat:
            return "Invalid Date, in the Past or Before the Starting Date"
        self.date = w
        self.datd = self.df.t_disp[pag][0][i]
        if self.groups == "Y" and self.grgame == (self.game - 1):
            self.grpsel = True
        else:
            self.grpsel = False
            return "sk1"

    def doGrpNum(self, frt, pag, r, c, p, i, w):
        if not w:
            return "Invalid Number, Must Be Two or More"
        self.grpnum = self.totskp / w
        if self.grpnum % 2:
            self.grpnum += 1

    def doGreens(self, frt, pag, r, c, p, i, w):
        if not w:
            return "Invalid Greens"
        self.greens = {}
        rinks = 0
        grns = w.split(",")
        for n, g in enumerate(grns):
            if not n:
                # First Green
                self.first = g[0]
            self.greens[g[0]] = []
            if len(g) == 1:
                for x in range(1, 7):
                    self.greens[g[0]].append(x)
                rinks += 6
            else:
                for r in g[1:]:
                    self.greens[g[0]].append(r)
                    rinks += 1
        if (self.totskp / 2) > rinks:
            return "Not Enough Rinks"

    def doPrtCards(self, frt, pag, r, c, p, i, w):
        self.prtcards = w
        if not self.reprint or self.prtcards == "N":
            self.allcards = "Y"
            return "sk1"

    def doAllCards(self, frt, pag, r, c, p, i, w):
        self.allcards = w

    def doEnd(self):
        self.df.closeProcess()
        if not self.reprint:
            if self.pairSkips():
                self.mf.closeLoop()
                self.doExit()
                return
            self.allocateRinks()
        if self.reprint and self.prtcards == "O":
            pass
        else:
            self.printGame()
        if not self.reprint:
            ok = askQuestion(self.mf.body, "Summary",
                "Do You Want to Print a Cards Summary?", default="no")
            if ok == "yes":
                callModule(self.mf, None, "bc3010", args=[self.ccod,
                    self.df.repprt[0][1], self.df.repprt[0][2],
                    self.df.repprt[1][2]])
            self.mf.dbm.commitDbase(ask=True, rback=False)
        if self.prtcards in ("Y", "O"):
            showWarning(self.mf.body, "Change Paper", "In Order to Print "\
                "Cards You Need to Change the Paper to A6 Cards. Please "\
                "Do So Now Before Continuing")
            self.printCards()
        self.mf.closeLoop()

    def pairSkips(self):
        if self.gtyp == "D":
            # Drawn Games
            chk = self.sql.getRec(tables="bwlgme", cols=["bcg_scod"],
                where=[("bcg_ccod", "=", self.ccod), ("bcg_game", "=",
                self.game)], order="bcg_scod")
            if len(chk) % 2:
                showError(self.mf.body, "Mismatch",
                    "There is an Uneven Number of Teams (%s)" % len(chk))
                return True
            self.skips = []
            for c in chk:
                self.skips.append(c[0])
            chk = True
            while chk:
                random.shuffle(self.skips)
                for x in range(0, len(self.skips), 2):
                    one = self.skips[x]
                    two = self.skips[x+1]
                    chk = self.sql.getRec(tables="bwlgme", where=[("bcg_ccod",
                        "=", self.ccod), ("bcg_scod", "=", one), ("bcg_type",
                        "=", "D"), ("bcg_ocod", "=", two)], limit=1)
                    if chk:
                        break
            return
        # Strength versus Strength
        if self.grpsel:
            # Allocate Into Groups
            grpcnt = 0
            grpcod = 0
            chk = self.sql.getRec(tables="bwlgme", cols=["bcg_scod",
                "sum(bcg_points) as pts",
                "sum(bcg_shots_for - bcg_shots_agt) as diff"],
                where=[("bcg_ccod", "=", self.ccod)], group="bcg_scod",
                order="pts desc, diff desc, bcg_shots_agt asc")
            for skp in chk:
                if not grpcnt:
                    grpcnt = self.grpnum
                    grpcod += 1
                self.sql.updRec("bwlgme", cols=["bcg_group"],
                    data=[grpcod], where=[("bcg_ccod", "=",
                    self.ccod), ("bcg_scod", "=", skp[0])])
                grpcnt -= 1
        groups = self.sql.getRec(tables="bwlgme", cols=["bcg_group"],
            where=[("bcg_ccod", "=", self.ccod), ("bcg_game", "<",
            self.game)], group="bcg_group", order="bcg_group")
        self.skips = []
        for grp in groups:
            chk = self.sql.getRec(tables="bwlgme", cols=["bcg_scod",
                "sum(bcg_a_points) as pts", "sum(bcg_a_shots_agt) as agt",
                "sum(bcg_a_shots_for - bcg_a_shots_agt) as diff"],
                where=[("bcg_ccod", "=", self.ccod), ("bcg_group",
                "in", (0, grp[0]))], group="bcg_scod",
                order="pts desc, diff desc, agt asc")
            if self.game == self.games:
                # Last game, straight Strength verse Strength
                for x in range(0, len(chk), 2):
                    self.skips.extend([chk[x][0], chk[x+1][0]])
                continue
            dun = []
            while len(dun) != len(chk):
                for x in range(0, len(chk)):
                    if chk[x][0] in dun:
                        continue
                    one = chk[x][0]
                    done = False
                    for s in chk:
                        if s[0] == one or s[0] in dun:
                            continue
                        if not self.checkSkip(one, s[0]):
                            dun.extend([one, s[0]])
                            done = True
                            break
                    if not done:
                        for s in chk:
                            if s[0] == one or s[0] in dun:
                                continue
                            dun.extend([one, s[0]])
                            break
            for x in range(0, len(dun), 2):
                self.skips.extend([dun[x], dun[x+1]])

    def checkSkip(self, one, two):
        return self.sql.getRec(tables="bwlgme", where=[("bcg_ccod", "=",
            self.ccod), ("bcg_scod", "=", one), ("bcg_ocod", "=", two),
            ("bcg_game", "<", self.game)])

    def allocateRinks(self):
        allrnk = []
        for grn in self.greens:
            for rnk in self.greens[grn]:
                allrnk.append("%s%s" % (grn, rnk))
        skpdic = {}
        if self.game == self.games:
            # If Final, Allocate First Green to Leaders
            groups = self.sql.getRec(tables="bwlgme", cols=["bcg_group"],
                where=[("bcg_ccod", "=", self.ccod), ("bcg_group", "<>", 0)],
                group="bcg_group")
            num = len(groups)
            lst = None
            cnt = 0
            for x in range(0, len(self.skips), 2):
                one = self.skips[x]
                two = self.skips[x+1]
                grp = self.sql.getRec(tables="bwlgme", cols=["bcg_group"],
                    where=[("bcg_ccod", "=", self.ccod), ("bcg_scod", "=",
                    one), ("bcg_game", "=", self.game)], limit=1)[0]
                if lst != grp:
                    lst = grp
                    pos = 0
                rnk = None
                if grp == 0 and pos < 6:
                    rnk = "%s%s" % (self.first, [4, 3, 5, 2, 1, 6][pos])
                elif grp == 1 and pos < 4:
                    rnk = "%s%s" % (self.first, [4, 3, 5, 2][pos])
                elif grp == 2 and num == 2 and pos < 2:
                    rnk = "%s%s" % (self.first, [1, 6][pos])
                elif grp == 2 and num == 3 and pos == 0:
                    rnk = "%s1" % self.first
                elif grp == 3 and num == 3 and pos == 0:
                    rnk = "%s6" % self.first
                if rnk and rnk in allrnk:
                    skpdic[one] = (two, rnk)
                    allrnk.remove(rnk)
                    cnt += 1
                    if cnt == 6:
                        break
                pos += 1
        # Allocate Balance of Greens/Rinks to End Rinkers
        for y in range(0, len(self.skips), 2):
            one = self.skips[y]
            if one in skpdic:
                continue
            two = self.skips[y+1]
            if not self.checkEnds(one, two):
                continue
            random.shuffle(allrnk)
            done = False
            for rk in allrnk:
                if rk[1] in ("1", "6"):
                    continue
                if not self.checkRink(one, two, rk):
                    skpdic[one] = (two, rk)
                    allrnk.remove(rk)
                    done = True
                    break
            if not done:
                random.shuffle(allrnk)
                for rk in allrnk:
                    if rk[1] in ("1", "6") and rk != allrnk[-1]:
                        continue
                skpdic[one] = (two, rk)
                allrnk.remove(rk)
        # Allocate Balance of Greens/Rinks to Rest
        for x in range(100):
            again = False
            cpydic = copy.copy(skpdic)
            cpyrnk = copy.copy(allrnk)
            for y in range(0, len(self.skips), 2):
                one = self.skips[y]
                if one in cpydic:
                    continue
                two = self.skips[y+1]
                random.shuffle(cpyrnk)
                done = False
                for rk in cpyrnk:
                    if not self.checkRink(one, two, rk):
                        cpydic[one] = (two, rk)
                        cpyrnk.remove(rk)
                        done = True
                        break
                if not done:
                    again = True
                    random.shuffle(cpyrnk)
                    cpydic[one] = (two, cpyrnk.pop())
            if x < 99 and not again:
                break
        skpdic = cpydic
        # Update Tables with Oponents and Rinks
        col = ["bcg_date", "bcg_ocod", "bcg_rink"]
        for one in skpdic:
            two, rnk = skpdic[one]
            self.sql.updRec("bwlgme", cols=col, data=[self.date, two, rnk],
                where=[("bcg_ccod", "=", self.ccod), ("bcg_scod", "=", one),
                ("bcg_game", "=", self.game)])
            self.sql.updRec("bwlgme", cols=col, data=[self.date, one, rnk],
                where=[("bcg_ccod", "=", self.ccod), ("bcg_scod", "=", two),
                ("bcg_game", "=", self.game)])
        # Adjust scores
        if self.adjust == "Y":
            chk = self.sql.getRec(tables="bwlgme", where=[("bcg_ccod",
                "=", self.ccod), ("bcg_group", ">", 1)],
                order="bcg_game")
            for gam in chk:
                skp = gam[self.sql.bwlgme_col.index("bcg_scod")]
                gme = gam[self.sql.bwlgme_col.index("bcg_game")]
                shf = gam[self.sql.bwlgme_col.index("bcg_shots_for")]
                sha = gam[self.sql.bwlgme_col.index("bcg_shots_agt")]
                pts = gam[self.sql.bwlgme_col.index("bcg_points")]
                if gme in self.expunge:
                    shf = 0
                    sha = 0
                    pts = 0
                elif gme <= self.grgame:
                    shf = int(round(shf * self.percent / 100.0, 0))
                    sha = int(round(sha * self.percent / 100.0, 0))
                    pts = int(round(pts * self.percent / 100.0, 0))
                self.sql.updRec("bwlgme", cols=["bcg_a_shots_for",
                    "bcg_a_shots_agt", "bcg_a_points"], data=[shf,
                    sha, pts], where=[("bcg_ccod", "=", self.ccod),
                    ("bcg_scod", "=", skp), ("bcg_game", "=", gme)])

    def checkEnds(self, one, two):
        check = round(self.games / 3.0, 0) - 1
        for sk in (one, two):
            ends = 0
            rnks = self.sql.getRec(tables="bwlgme", cols=["bcg_rink"],
                where=[("bcg_ccod", "=", self.ccod), ("bcg_scod", "=",
                sk), ("bcg_rink", "<>", "")])
            for rk in rnks:
                if rk[0][1] in ("1", "6"):
                    ends += 1
            if ends > check:
                return True

    def checkRink(self, one, two, rnk):
        return self.sql.getRec(tables="bwlgme", where=[("bcg_ccod", "=",
            self.ccod), ("bcg_scod", "in", (one, two)), ("bcg_rink", "=",
            rnk)])

    def printGame(self):
        groups = ["All", "A", "B", "C"]
        if self.reprint:
            chk = self.sql.getRec(tables="bwlcmp", where=[("bcm_code", "=",
                self.ccod)], limit=1)
            self.cdes = chk[self.sql.bwlcmp_col.index("bcm_name")]
            dte = self.sql.getRec(tables="bwlgme", cols=["bcg_date"],
                where=[("bcg_ccod", "=", self.ccod), ("bcg_game", "=",
                self.game)], limit=1)
        if self.game <= self.grgame:
            grp = [[0]]
        else:
            grp = self.sql.getRec(tables="bwlgme", cols=["bcg_group"],
                where=[("bcg_ccod", "=", self.ccod)], group="bcg_group",
                order="bcg_group")
        cols = [
            ["a", "UI", 3,  "Skp",  "y"],
            ["b", "NA", 40, "Name", "y"],
            ["c", "UA", 2,  "GR",   "y"],
            ["d", "UI", 3,  "Skp",  "y"],
            ["e", "NA", 40, "Name", "y"]]
        whr = [("bcg_ccod", "=", self.ccod), ("bcg_game", "=", self.game),
            ("bcs_code=bcg_scod",)]
        out = PdfFileWriter()
        for g in grp:
            where = whr[:]
            if self.game <= self.grgame:
                all = self.sql.getRec(tables=["bwlgme", "bwlskp"],
                    cols=["bcs_code", "bcs_surname", "bcs_names", "bcg_ocod",
                    "bcg_rink"], where=where, order="bcs_surname, bcs_names")
            else:
                where.append(("bcg_group", "=", g[0]))
                all = self.sql.getRec(tables=["bwlgme", "bwlskp"],
                    cols=["bcs_code", "bcs_surname", "bcs_names", "bcg_ocod",
                    "bcg_rink"], where=where, order="bcs_surname, bcs_names")
            head = ["", (self.cdes, 30), "Game Number: %s  Group: %s  "\
                "Date: %s" % (self.game, groups[g[0]], self.datd)]
            data = []
            for skp in all:
                opp = self.sql.getRec(tables="bwlskp", cols=["bcs_code",
                    "bcs_surname", "bcs_names"], where=[("bcs_code", "=",
                    skp[3])], limit=1)
                one = "%s, %s" % (skp[1], skp[2])
                two = "%s, %s" % (opp[1], opp[2])
                data.append([skp[0], one, skp[4], opp[0], two])
            pdffl = getModName(self.mf.wrkdir, __name__, g[0])
            rpt = RepPrt(self.mf, name=__name__, tables=data, heads=head,
                cols=cols, ttype="D", pages=False, pdffl=pdffl, view="N")
            inp = PdfFileReader(file(rpt.pdfnam, "rb"))
            for page in range(inp.getNumPages()):
                out.addPage(inp.getPage(page))
        pdfnam = getModName(self.mf.wrkdir, __name__, "report", ext="pdf")
        outputStream = file(pdfnam, "wb")
        out.write(outputStream)
        outputStream.close()
        doPrinter(mf=self.mf, conum=1, rep=pdfnam, view=self.df.repprt[0][1],
            prtnam=self.df.repprt[0][2])

    def printCards(self):
        pdfnam = getModName(self.mf.wrkdir, __name__, "cards", ext="pdf")
        c, cpi, inch, ss, dd = doReportlab(self.mf.prgdir, pdfnam, orient="A6")
        cw = c.stringWidth(("X"), "Tartan", cpi)    # character width
        ld = inch / 6                               # line depth
        mg = ((10.5, 36, 46.5), (.5, 6.5, 8.5))     # margins
        skips = []
        all = []
        chk = []
        if self.allcards == "N":
            gme = getSingleRecords(self.mf, "bwlgme", ("bcg_rink", "bcg_scod",
                "bcg_ocod"), where=[("bcg_ccod", "=", self.ccod), ("bcg_game",
                "=", self.game)], group="bcg_rink", order="bcg_rink")
            if not gme:
                return
            for g in  gme:
                all.append([g[1], g[5], g[6]])
        else:
            all = self.sql.getRec(tables="bwlgme", cols=["bcg_scod", "bcg_ocod",
                "bcg_rink"], where=[("bcg_ccod", "=", self.ccod), ("bcg_game",
                "=", self.game)], order="bcg_rink")
        for rec in all:
            if rec[0] in chk:
                continue
            skips.append(rec)
            chk.extend([rec[0], rec[1]])
        scales = (0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,1.40,16,17,18,19,20,1.10)
        for rec in skips:
            skp = self.sql.getRec(tables="bwlskp", cols=["bcs_code",
                "bcs_surname", "bcs_names"], where=[("bcs_code", "=",
                rec[0])], limit=1)
            opp = self.sql.getRec(tables="bwlskp", cols=["bcs_code",
                "bcs_surname", "bcs_names"], where=[("bcs_code", "=",
                rec[1])], limit=1)
            grn = rec[2]
            c.translate(-65, 5)
            # scale drops by .05 per end therefore 21 * -.05 = -1.05
            c.scale(1.25, ((-1.05 / 21) * self.ends) + 2.15)
            c.rect(cw * mg[0][0], (mg[1][0]*ld), cw*mg[0][1], (mg[1][1]*ld))
            c.setFont("Tartan-Bold", 14)
            c.drawCentredString(cw*28.5, 1.5 * ld, self.cdes)
            c.setFont("Tartan-Bold", 10)
            c.setFillGray(.9)
            c.rect(cw * mg[0][0], (mg[1][1]*ld), cw * mg[0][1], (1*ld), fill=1)
            c.setFillGray(0)
            for x in range(self.ends + 1):
                if (self.skins=="Y" and not (x+1) % self.sends) or x==self.ends:
                    c.setFillGray(.9)
                    c.rect(cw * mg[0][0], (mg[1][1] + 1 + x) * ld, \
                        cw * mg[0][1], ld, fill=1)
                    c.setFillGray(0)
                else:
                    c.line(cw * mg[0][0], (mg[1][2] + x) * ld, \
                        cw * mg[0][2], (mg[1][2] + x) * ld)
                c.setFont("Tartan-Bold", 10)
                if x == self.ends:
                    txt = "TOT"
                else:
                    txt = "%s" % (x+1)
                c.drawCentredString(cw * 28.5, (8.2 + x) * ld, txt)
                c.setFont("Tartan", 10)
                for y in (mg[0][0],16.5,22.5,26.5,30.5,36.5,42.5,mg[0][2]):
                    if x == self.ends:
                        z = x + 1
                    else:
                        z = x
                    c.line(cw * y, ((mg[1][1] + x) * ld), cw * y, \
                        ((7.5 + z) * ld))
            t = c.beginText()
            t.setTextOrigin(60, 10)
            t.setFont("Tartan-Bold", 10)
            t.textLine(" ")
            t.setFont("Tartan-Bold", 14)
            t.textLine(" ")
            t.setFont("Tartan-Bold", 10)
            t.textLine(" Game:  %3s   %s   Date: %10s" % (self.game, rec[2],
                self.datd))
            t.textLine(" ")
            t.textLine(" %s (%s)" % (self.getName(skp), skp[0]))
            t.textLine(" %30s (%s)" % (self.getName(opp), opp[0]))
            t.textLine(" Shots Total Pts End Shots Total Pts")
            c.drawText(t)
            c.showPage()
        c.save()
        doPrinter(mf=self.mf, rep=pdfnam, view=self.df.repprt[0][1],
            prtnam=self.df.repprt[0][2])

    def getName(self, data):
        name = ""
        init = data[2].split()
        for i in init:
            name = i[0].capitalize() + ". "
        return name + data[1]

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()
