#! /usr/bin/python

"""
SYNOPSIS
    Bowls Competition Capture Match Results.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import ASD, TartanDialog, Sql
from tartanFunctions import askQuestion

class bc2040:
    def __init__(self, mf):
        self.mf = mf
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["bwlcmp", "bwlskp", "bwlclb", "bwlent",
            "bwltyp", "bwlgme"], prog=__name__)

    def mainProcess(self):
        tit = ("Competition Match Cards",)
        cmp = {
            "stype": "R",
            "tables": ("bwlcmp",),
            "cols": (
                ("bcm_code", "", 0, "Cod"),
                ("bcm_name", "", 0, "Name", "Y"),
                ("bcm_date", "", 0, "Date"))}
        gme = {
            "stype": "R",
            "tables": ("bwlgme",),
            "cols": (("bcg_game", "", 0, "GC"),),
            "whera": [("T", "bcg_ccod", 0, 0)],
            "group": "bcg_game"}
        skp = {
            "stype": "R",
            "tables": ("bwlent", "bwlskp", "bwlgme"),
            "cols": (
                ("bcs_code", "", 0, "Cod"),
                ("bcs_surname", "", 0, "Surame", "Y"),
                ("bcs_names", "", 0, "Names"),
                ("bcg_shots_for", "", 0, "For"),
                ("bcg_shots_agt", "", 0, "Agt"),
                ("bcg_points", "", 0, "Pts")),
            "where": [
                ("bcs_code=bce_scod",),
                ("bcg_scod=bce_scod",)],
            "whera": [
                ("T", "bce_ccod", 0, 0),
                ("T", "bcg_ccod", 0, 0),
                ("T", "bcg_game", 2, 0)]}
        fld = (
            (("T",0,0,0),"I@bcm_code",0,"","",
                "","Y",self.doCmpCod,cmp,None,("notzero",)),
            (("T",0,0,0),"O@bcm_name",0,"Name"),
            (("T",0,0,0),"I@bcg_game",0,"Game Number","",
                "","N",self.doGamCod,gme,None,("efld",)),
            (("C",0,0,0),"I@bcg_scod",0,"","",
                "","N",self.doSkpCod,skp,None,("efld",)),
            (("C",0,0,0),"ONA",30,"Skp-Name"),
            (("C",0,0,0),"I@bcg_shots_for",0,"","",
                "","N",self.doShots,None,None,("efld",)),
            (("C",0,0,0),"I@bcg_points",0,"","",
                "","N",self.doPoints,None,None,("efld",)),
            (("C",0,0,0),"O@bcg_ocod",0,""),
            (("C",0,0,0),"ONA",30,"Opp-Name"),
            (("C",0,0,0),"I@bcg_shots_agt",0,"","",
                "","N",self.doShots,None,None,("efld",)),
            (("C",0,0,0),"I@bcg_points",0,"","",
                "","N",self.doPoints,None,None,("efld",)))
        tnd = ((self.doEnd,"y"),)
        txt = (self.doExit,)
        cnd = ((self.doEnd,"y"),)
        cxt = (self.doExit,)
        self.df = TartanDialog(self.mf, tops=False, title=tit, eflds=fld,
            tend=tnd, txit=txt, cend=cnd, cxit=cxt)

    def doCmpCod(self, frt, pag, r, c, p, i, w):
        cmp = self.sql.getRec(tables="bwlcmp", cols=["bcm_name", "bcm_type"],
            where=[("bcm_code", "=", w)], limit=1)
        if not cmp:
            return "Invalid Competition Code"
        self.ccod = w
        self.df.loadEntry(frt, pag, p+1, data=cmp[0])
        self.typ = self.sql.getRec(tables="bwltyp", where=[("bct_code",
            "=", cmp[1])], limit=1)
        self.games = self.typ[self.sql.bwltyp_col.index("bct_games")]
        ends = self.typ[self.sql.bwltyp_col.index("bct_ends")]
        epts = self.typ[self.sql.bwltyp_col.index("bct_e_points")]
        gpts = self.typ[self.sql.bwltyp_col.index("bct_g_points")]
        self.pts = (ends * epts) + gpts
        if self.typ[self.sql.bwltyp_col.index("bct_skins")] == "Y":
            snds = self.typ[self.sql.bwltyp_col.index("bct_sends")]
            spts = self.typ[self.sql.bwltyp_col.index("bct_s_points")]
            self.pts = self.pts + ((ends / snds) * spts)
        self.bpts = self.typ[self.sql.bwltyp_col.index("bct_bonus")]
        dif = self.typ[self.sql.bwltyp_col.index("bct_win_by")]
        self.win_by = [dif, dif * -1]
        dif = self.typ[self.sql.bwltyp_col.index("bct_lose_by")]
        self.lose_by = [dif, dif * -1]
        chk = self.sql.getRec(tables="bwlgme", cols=["bcg_game",
            "sum(bcg_points)"], where=[("bcg_ccod", "=", self.ccod)],
            group="bcg_game", order="bcg_game")
        for game in chk:
            self.gcod = game[0]
            if not game[1]:
                self.df.loadEntry(frt, pag, p+2, data=self.gcod)
                return

    def doGamCod(self, frt, pag, r, c, p, i, w):
        chk = self.sql.getRec(tables="bwlgme", cols=["sum(bcg_ocod)"],
            where=[("bcg_ccod", "=", self.ccod), ("bcg_game", "=", w)],
            limit=1)
        if not chk[0]:
            return "Invalid Game Code or Game Not Yet Drawn"
        if w != self.games:
            chk = self.sql.getRec(tables="bwlgme", cols=["sum(bcg_ocod)"],
                where=[("bcg_ccod", "=", self.ccod), ("bcg_game", "=", w+1)],
                limit=1)
            if chk[0]:
                return "Invalid Game Code, Next Game Already Drawn"
        self.gcod = w

    def doSkpCod(self, frt, pag, r, c, p, i, w):
        chk = self.sql.getRec(tables=["bwlgme", "bwlskp"], cols=["bcs_surname",
            "bcs_names", "bcg_ocod", "bcg_shots_for", "bcg_shots_agt",
            "bcg_points"], where=[("bcg_ccod", "=", self.ccod), ("bcg_scod",
            "=", w), ("bcg_game", "=", self.gcod), ("bcs_code=bcg_scod",)],
            limit=1)
        if not chk:
            return "Invalid Skip Code"
        if chk[3] or chk[4]:
            ok = askQuestion(self.mf.body, head="Already Entered",
                mess="This Card Has Already Been Entered, Re-Enter?",
                default="no")
            if ok == "no":
                return "rf"
        self.skp = w
        self.opp = chk[2]
        name = "%s, %s" % tuple(chk[:2])
        self.df.loadEntry(frt, pag, p+1, data=name)
        self.df.loadEntry(frt, pag, p+2, data=chk[3])
        self.df.loadEntry(frt, pag, p+3, data=chk[5])
        self.df.loadEntry(frt, pag, p+4, data=self.opp)
        chk = self.sql.getRec(tables=["bwlgme", "bwlskp"], cols=["bcs_surname",
            "bcs_names", "bcg_shots_for", "bcg_shots_agt", "bcg_points"],
            where=[("bcg_ccod", "=", self.ccod), ("bcg_scod", "=", self.opp),
            ("bcg_game", "=", self.gcod), ("bcs_code=bcg_scod",)], limit=1)
        name = "%s, %s" % (chk[0], chk[1])
        self.df.loadEntry(frt, pag, p+5, data=name)
        self.df.loadEntry(frt, pag, p+6, data=chk[2])
        self.df.loadEntry(frt, pag, p+7, data=chk[4])

    def doShots(self, frt, pag, r, c, p, i, w):
        if i == 2:
            self.s_for = w
        else:
            self.s_agt = w
            self.mpts = self.pts
            if self.bpts == "N":
                return
            diff = float(ASD(self.s_for) - ASD(self.s_agt))
            if not diff:
                return
            if diff > self.win_by[0] or diff < self.win_by[1]:
                self.mpts = float(ASD(self.mpts) + ASD(1))
            elif diff < self.lose_by[0] and diff > self.lose_by[1]:
                self.mpts = float(ASD(self.mpts) + ASD(1))

    def doPoints(self, frt, pag, r, c, p, i, w):
        if w and w % 1 not in (0.0, 0.5):
            return "Invalid Decimal in Points"
        if i == 3:
            self.p_for = w
            self.df.loadEntry(frt, pag, p+1, data=self.opp)
            chk = self.sql.getRec(tables=["bwlskp"], cols=["bcs_surname",
                "bcs_names"], where=[("bcs_code", "=", self.opp)], limit=1)
            name = "%s, %s" % tuple(chk)
            self.df.loadEntry(frt, pag, p+2, data=name)
            return "sk2"
        if not w:
            self.p_agt = float(ASD(self.mpts) - ASD(self.p_for))
            self.df.loadEntry(frt, pag, p, data=self.p_agt)
        else:
            self.p_agt = w
        tot = float(ASD(self.p_for) + ASD(self.p_agt))
        if tot != self.mpts:
            return "Invalid Total Points (%s s/b %s)" % (tot, self.mpts)

    def doEnd(self, widget=None):
        if self.df.frt == "T":
            self.df.focusField("C", 0, 1)
        else:
            self.sql.updRec("bwlgme", cols=["bcg_shots_for", "bcg_shots_agt",
                "bcg_points", "bcg_a_shots_for", "bcg_a_shots_agt",
                "bcg_a_points"], data=[self.s_for, self.s_agt, self.p_for,
                self.s_for, self.s_agt, self.p_for], where=[("bcg_ccod", "=",
                self.ccod), ("bcg_scod", "=", self.skp), ("bcg_game", "=",
                self.gcod)])
            self.sql.updRec("bwlgme", cols=["bcg_shots_for", "bcg_shots_agt",
                "bcg_points", "bcg_a_shots_for", "bcg_a_shots_agt",
                "bcg_a_points"], data=[self.s_agt, self.s_for, self.p_agt,
                self.s_agt, self.s_for, self.p_agt], where=[("bcg_ccod", "=",
                self.ccod), ("bcg_scod", "=", self.opp), ("bcg_game", "=",
                self.gcod)])
            self.mf.dbm.commitDbase()
            self.df.advanceLine(0)

    def doExit(self, widget=None):
        if self.df.frt == "C" and self.df.col != 1:
            chk = self.sql.getRec(tables="bwlgme", where=[("bcg_ccod", "=",
                self.ccod), ("bcg_game", "=", self.gcod)])
            for c in chk:
                scod = c[self.sql.bwlgme_col.index("bcg_scod")]
                ocod = c[self.sql.bwlgme_col.index("bcg_ocod")]
                sfor = c[self.sql.bwlgme_col.index("bcg_shots_for")]
                sagt = c[self.sql.bwlgme_col.index("bcg_shots_agt")]
                if not sfor and not sagt:
                    self.df.focusField(self.df.frt, self.df.pag, self.df.col,
                        err="Missing Score Card for Skips %s and %s" % (scod,
                        ocod))
                    return
        self.df.closeProcess()
        self.mf.closeLoop()
