#! /usr/bin/python

"""
SYNOPSIS
    Bowls Competition Match Results.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import os
from pyPdf import PdfFileWriter, PdfFileReader
from TartanClasses import RepPrt, TartanDialog, Sql
from tartanFunctions import doPrinter, getModName

class bc3030:
    def __init__(self, mf):
        self.mf = mf
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["bwlcmp", "bwlgme", "bwlskp", "bwltyp",
            "bwlent"], prog=__name__)

    def mainProcess(self):
        tit = ("Bowls Competition Match Results",)
        cmp = {
            "stype": "R",
            "tables": ("bwlcmp",),
            "cols": (
                ("bcm_code", "", 0, "Cod"),
                ("bcm_name", "", 0, "Name", "Y"))}
        gme = {
            "stype": "R",
            "tables": ("bwlgme",),
            "cols": (("bcg_game", "", 0, "GC"),),
            "whera": [("T", "bcg_ccod", 0, 0)],
            "group": "bcg_game"}
        fld = (
            (("T",0,0,0),"I@bcm_code",0,"","",
                0,"N",self.doCmpCod,cmp,None,None),
            (("T",0,1,0),"IUI",2,"Last Game","Last Game",
                0,"N",self.doLstGam,gme,None,("efld",)))
        tnd = ((self.doEnd,"y"),)
        txt = (self.doExit,)
        self.df = TartanDialog(self.mf, tops=False, title=tit, eflds=fld,
            tend=tnd, txit=txt, view=("N","V"))

    def doCmpCod(self, frt, pag, r, c, p, i, w):
        chk = self.sql.getRec(tables="bwlcmp", where=[("bcm_code", "=", w)],
            limit=1)
        if not chk:
            return "Invalid Competition Code"
        self.ccod = w
        self.cdes = chk[self.sql.bwlcmp_col.index("bcm_name")]
        ctyp = chk[self.sql.bwlcmp_col.index("bcm_type")]
        chk = self.sql.getRec(tables="bwltyp", where=[("bct_code", "=", ctyp)],
            limit=1)
        self.grgame = chk[self.sql.bwltyp_col.index("bct_grgame")]
        game = self.sql.getRec(tables="bwlgme", cols=["bcg_game",
            "sum(bcg_points)"], where=[("bcg_ccod", "=", self.ccod)],
            group="bcg_game", order="bcg_game")
        self.game = 0
        for g in game:
            if not g[1]:
                break
            self.game = g[0]
        if not self.game:
            return "No Completed Games to Print"
        self.df.loadEntry(frt, pag, p+1, data=self.game)

    def doLstGam(self, frt, pag, r, c, p, i, w):
        if not w:
            chk = self.sql.getRec(tables="bwlgme", cols=["bcg_game",
                "sum(bcg_points)"], where=[("bcg_ccod", "=", self.ccod)],
                group="bcg_game", order="bcg_game")
            self.lastgam = 0
            for g in chk:
                if g[1]:
                    self.lastgam = g[0]
                else:
                    break
            if not self.lastgam:
                return "No Completed Games"
        else:
            chk = self.sql.getRec(tables="bwlgme", cols=["sum(bcg_points)"],
                where=[("bcg_ccod", "=", self.ccod), ("bcg_game", "=", w)],
                group="bcg_game", limit=1)
            if not chk[0]:
                return "Game Results Not Yet Entered"
            self.lastgam = w

    def doEnd(self):
        self.df.closeProcess()
        groups = ["None", "A", "B", "C", "D", "E"]
        chk = self.sql.getRec(tables="bwlgme", cols=["bcg_group", "bcg_scod"],
            where=[("bcg_ccod", "=", self.ccod), ("bcg_game", "=",
            self.lastgam)], order="bcg_group")
        grps = {}
        for rec in chk:
            if self.lastgam <= self.grgame:
                rec[0] = 0
            if rec[0] not in grps:
                grps[rec[0]] = [rec[1]]
            else:
                grps[rec[0]].append(rec[1])
        keys = grps.keys()
        keys.sort()
        cols = [
            ["a", "UI",  3,   "Pos",          "y"],
            ["b", "UI",  3,   "Skp",          "y"],
            ["c", "NA", 40,   "Name",         "y"],
            ["d", "UL", 11,   "        For",  "y"],
            ["e", "UL", 11,   "    Against",  "y"],
            ["f", "SL", 11,   "Difference ",  "y"],
            ["g", "UD", 9.1, "   Points",  "y"]]
        where = [("bcg_ccod", "=", self.ccod), ("bcg_game", "<=", self.lastgam),
            ("bcs_code=bcg_scod",)]
        pdfnam = None
        out = PdfFileWriter()
        for g in keys:
            whr = where[:]
            whr.append(("bcg_scod", "in", grps[g]))
            col = ["bcs_code", "bcs_surname", "bcs_names"]
            if self.lastgam <= self.grgame:
                col.extend(["sum(bcg_shots_for) as sfor",
                    "sum(bcg_shots_agt) as sagt",
                    "sum(bcg_shots_for - bcg_shots_agt) as agg",
                    "sum(bcg_points) as pts"])
            else:
                col.extend(["sum(bcg_a_shots_for) as sfor",
                    "sum(bcg_a_shots_agt) as sagt",
                    "sum(bcg_a_shots_for - bcg_a_shots_agt) as agg",
                    "sum(bcg_a_points) as pts"])
            all = self.sql.getRec(tables=["bwlgme", "bwlskp"], cols=col,
                where=whr, group="bcs_code, bcs_surname, bcs_names",
                order="pts desc, agg desc, sagt asc")
            if not g:
                head = ["", (self.cdes, 30), "Results After Game Number: %s" % \
                    self.lastgam]
            else:
                head = ["", (self.cdes, 30), "Results After Game Number: %s  "\
                    "for Group: %s" % (self.lastgam, groups[g])]
            data = []
            for num, (skp, snam, fnam, sfor, sagt, agg, pts) in enumerate(all):
                nam = "%s, %s" % (snam, fnam)
                data.append([num+1, skp, nam, sfor, sagt, agg, pts])
            pdffl = getModName(self.mf.wrkdir, __name__, g)
            rpt = RepPrt(self.mf, name=__name__, tables=data, heads=head,
                cols=cols, ttype="D", pages=False, pdffl=pdffl, view="N")
            inp = PdfFileReader(file(rpt.pdfnam, "rb"))
            for page in range(inp.getNumPages()):
                out.addPage(inp.getPage(page))
        pdfnam = getModName(self.mf.wrkdir, __name__, "report", ext="pdf")
        outputStream = file(pdfnam, "wb")
        out.write(outputStream)
        outputStream.close()
        doPrinter(mf=self.mf, conum=1, rep=pdfnam, view=self.df.repprt[0][1],
            prtnam=self.df.repprt[0][2])
        self.mf.closeLoop()

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()
