#! /usr/bin/python

"""
SYNOPSIS
    Bowls League Match Declaration Forms.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import os, sys, time
from PIL import Image
from TartanClasses import Sql, TartanDialog
from tartanFunctions import getModName, doDrawTable, doPrinter, doReportlab

class bc3050:
    def __init__(self, mf):
        self.mf = mf
        if self.setVariables():
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, tables=["ctlmst", "bwlflf", "bwlflm",
            "bwlflo", "bwlflp", "bwlfls", "bwlflt"], prog=__name__)
        acc = self.sql.getRec(tables="ctlmst", cols=["ctm_name"],
            where=[("ctm_cono", "=", 1)], limit=1)
        if not acc:
            showError(self.mf.body, "Missing Record",
                "The Club Record is Missing, Please Create it and The Retry")
            return
        self.conam = acc[0]
        return True

    def mainProcess(self):
        self.tit = "Bowls League Match Declaration Forms"
        fmt = {
            "stype": "R",
            "tables": ("bwlflf",),
            "cols": (
                ("bff_code", "", 0, "Cod"),
                ("bff_desc", "", 0, "Description", "Y"))}
        mch = {
            "stype": "R",
            "tables": ("bwlflm",),
            "cols": (
                ("bfm_date", "", 0, "Match-Date"),
                ("bfm_round", "", 0, "RN")),
            "whera": [("T", "bfm_fmat", 0, 0)],
            "group": "bfm_date, bfm_round"}
        fld = (
            (("T",0,0,0),"I@bff_code",0,"","",
                0,"Y",self.doFmat,fmt,None,("efld",)),
            (("T",0,0,0),"ONA",30,""),
            (("T",0,1,0),"I@bfm_date",0,"","",
                0,"Y",self.doDate,mch,None,("efld",)))
        tnd = ((self.doEnd,"y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doFmat(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="bwlflf", where=[("bff_code", "=", w)],
            limit=1)
        if not acc:
            return "Invalid Format"
        self.fmat = w
        self.fdsc = acc[1].upper()
        self.logo = acc[2]
        if not os.path.isfile(self.logo):
            self.logo = None
        else:
            image = Image.open(self.logo)
            self.logo = os.path.join(self.mf.wrkdir, "logo.jpg")
            image.transpose(Image.FLIP_TOP_BOTTOM).save(self.logo)
        self.df.loadEntry(frt, pag, p+1, data=acc[1])

    def doDate(self, frt, pag, r, c, p, i, w):
        self.date = w
        self.disp = self.df.t_disp[0][0][2]

    def doEnd(self):
        self.df.closeProcess()
        pdfnam = getModName(self.mf.wrkdir, __name__, "declare_%s" % self.date,
            ext="pdf")
        c, cpi, inch, ss, dd = doReportlab(self.mf.prgdir, pdfnam, orient="A4")
        cw = c.stringWidth(("X"), "Tartan", cpi)    # character width
        ld = inch / 6                               # line depth
        r1 = {
            "margins": ((25, 85), (10, 16)),
            "repeat": (1, 1),
            "rows": (
                (25, 10, 50, 1.5, .8),
                (25, 13, 50, 1.5, .8),
                (25, 15, (
                    (11, 1.5, .8, "Division"), (4, 1.5),
                    (11, 1.5, .8, "Round"), (4, 1.5),
                    (6, 1.5, .8, "Date"), (14, 1.5))),
                (25, 17, ((10, 1.5, .8, "Played at"), (40, 1.5))))}
        r2 = {
            "margins": ((10, 85), (19.5, 27)),
            "repeat": (1, 3),
            "rows": (
                (10, 19.5, (
                    (10, 1.5, .8, "Position"),
                    (10, 1.5, .8, "Initials"),
                    (50, 1.5, .8, "Surname"),
                    (10, 1.5, .8, "BSA No."))),
                (10, 21, (
                    (10, 1.5, .9, ("Skip", "Third", "Second", "Lead")),
                        (10, 1.5), (50, 1.5), (10, 1.5)), 4))}
        r3 = {
            "margins": ((10, 85), (46, 50.5)),
            "repeat": (1, 1),
            "rows": (
                (10, 46, (
                    (10, 1.5, .8, "Position"),
                    (10, 1.5, .8, "Initials"),
                    (50, 1.5, .8, "Surname"),
                    (10, 1.5, .8, "BSA No."))),
                (10, 47.5, (
                    (10, 1.5, .9, "Reserve"), (10, 1.5), (50, 1.5), (10, 1.5))),
                (10, 49, (
                    (10, 1.5, .9, "Captain"), (10, 1.5), (50, 1.5))))}
        all = self.sql.getRec(tables="bwlflm", where=[("bfm_fmat", "=",
            self.fmat), ("bfm_date", "=", self.date)], order="bfm_team")
        for cnt, match in enumerate(all):
            side = self.sql.getRec(tables="bwlfls", where=[("bfs_fmat", "=",
                self.fmat), ("bfs_code", "=", match[3])], limit=1)
            if side[3] == "F":
                continue
            if self.logo:
                c.drawImage(self.logo, 5*cw, 1*ld, 100, 113.25, None)
                c.drawImage(self.logo, 77*cw, 1*ld, 100, 113.25, None)
            c.setFont("Helvetica-Bold", 16)
            c.drawCentredString(50*cw, 3*ld, "WESTERN PROVINCE BOWLS")
            c.drawCentredString(50*cw, 5*ld, "%s CHAMPIONSHIP" % self.fdsc)
            c.drawCentredString(50*cw, 7*ld, "MATCH DECLARATION FORM")
            c.setFont("Helvetica-Bold", 14)
            c.drawCentredString(50*cw, 9*ld, "The following players will "\
                "represent")
            c.drawCentredString(50*cw, 12.5*ld, "vs")
            c.setFont("Helvetica-Bold", 12)
            last, tab1 = doDrawTable(c, cw, ld, r1, ppad=1, font=False)
            last, tab2 = doDrawTable(c, cw, ld, r2, ppad=1, font=False)
            last, tab3 = doDrawTable(c, cw, ld, r3, ppad=1, font=False)
            c.drawCentredString(50*cw, 11*ld, "%s Bowling Club" % self.conam)
            opp = self.sql.getRec(tables=["bwlflo", "bwlclb"],
                cols=["bcc_name", "bfo_desc"], where=[("bfo_fmat", "=",
                self.fmat), ("bfo_code", "=", match[4]),
                ("bcc_code=bfo_club",)], limit=1)
            c.drawCentredString(50*cw, 14*ld, "%s Bowling Club" % opp[0])
            c.drawString(36.5*cw, 16*ld, side[4])
            c.drawString(52.5*cw, 16*ld, str(match[2]))
            c.drawString(63*cw, 16*ld, self.disp)
            if match[5] == "H":
                venue = self.conam
            else:
                venue = opp[0]
            c.drawString(36*cw, 18*ld, "%s Bowling Club" % venue)
            teams = self.sql.getRec(tables=["bwlflt", "bwlflp"],
                cols=["bfp_fnam", "bfp_snam", "bfp_bsano"],
                where=[("bft_fmat", "=", self.fmat), ("bft_date", "=",
                self.date), ("bft_team", "=", match[3]),
                ("bfp_code=bft_player",)], order="bft_skip, bft_position")
            for num, player in enumerate(teams):
                init = ""
                for i in player[0].split():
                    init = init + i[0].capitalize() + " "
                pos = tab2[((num%4)+1)*4][1][num/4]+1
                c.drawString(21*cw, pos*ld, init.strip())
                c.drawString(31*cw, pos*ld, player[1])
                c.drawString(82*cw, pos*ld, str(player[2]))
            captain = self.sql.getRec(tables="bwlflp", cols=["bfp_fnam",
                "bfp_snam", "bfp_bsano"], where=[("bfp_code", "=", match[8])],
                limit=1)
            init = ""
            for i in captain[0].split():
                init = init + i[0].capitalize() + " "
            c.drawString(21*cw, (tab3[8][1][0]+1)*ld, init.strip())
            c.drawString(31*cw, (tab3[8][1][0]+1)*ld, captain[1])
            c.drawString(10*cw, 55*ld, "Declaration")
            c.drawString(10*cw, 57*ld, "I certify that the above players "\
                "are registered with WP Bowls as playing members")
            c.drawString(10*cw, 58*ld, "of the Club and that the names are "\
                "without exception the same as those entered")
            c.drawString(10*cw, 59*ld, "on the scorecards relevant to this "\
                "match")
            c.drawString(10*cw, 63*ld, "CAPTAIN:   __________________")
            c.drawString(26*cw, 64*ld, "(Signature)")
            c.showPage()
        c.save()
        head = "Match Declaration Forms for %s" % self.disp
        doPrinter(mf=self.mf, head=head, rep=pdfnam, view=self.df.repprt[0][1],
            prtnam=self.df.repprt[0][2], mail=self.df.repprt[1][2])
        self.closeProcess()

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()
