#! /usr/bin/python

"""
SYNOPSIS
    Bowls League Assessment Summary.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import TartanDialog, RepPrt, Sql

class bc3070:
    def __init__(self, mf):
        self.mf = mf
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["bwlflf", "bwlflm", "bwlflt", "bwlflp"],
            prog=__name__)

    def mainProcess(self):
        tit = "Assessment Summary Report"
        fmt = {
            "stype": "R",
            "tables": ("bwlflf",),
            "cols": (
                ("bff_code", "", 0, "Cod"),
                ("bff_desc", "", 0, "Description", "Y"))}
        dte = {
            "stype": "R",
            "tables": ("bwlflm",),
            "cols": (("bfm_date", "", 0, "Match-Date"),),
            "where": [("bfm_round", "=", 1)],
            "whera": [("T", "bfm_fmat", 0, 0)],
            "group": "bfm_date"}
        fld = (
            (("T",0,0,0),"I@bff_code",0,"","",
                0,"N",self.doFmat,fmt,None,("efld",)),
            (("T",0,0,0),"ONA",30,""),
            (("T",0,1,0),"I@bfm_date",0,"First Round Date","",
                "","N",self.doDate,dte,None,("efld",)))
        tnd = ((self.doEnd,"y"),)
        txt = (self.doExit,)
        self.df = TartanDialog(self.mf, tops=False, title=tit, eflds=fld,
            tend=tnd, txit=txt, view=("Y","V"), mail=("Y","N"))

    def doFmat(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="bwlflf", where=[("bff_code", "=", w)],
            limit=1)
        if not acc:
            return "Invalid Format"
        self.fmat = w
        self.fnam = acc[1]
        self.df.loadEntry(frt, pag, p+1, data=acc[1])
        acc = self.sql.getRec(tables="bwlflm", where=[("bfm_fmat", "=",
            self.fmat), ("bfm_round", "=", 1)], order="bfm_date")
        if acc:
            self.df.loadEntry(frt, pag, p+2, data=acc[-1][1])

    def doDate(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="bwlflm", where=[("bfm_fmat", "=",
            self.fmat), ("bfm_date", "=", w), ("bfm_round", "=", 1)], limit=1)
        if not acc:
            return "Invalid 1st Round Date"
        self.sdate = w
        self.ddisp = self.df.t_disp[0][0][2]
        acc = self.sql.getRec(tables="bwlflm", where=[("bfm_fmat", "=",
            self.fmat), ("bfm_date", ">", self.sdate), ("bfm_round", "=", 1)],
            limit=1)
        if not acc:
            self.edate = 99999999
        else:
            self.edate = acc[1]
        self.rounds = self.sql.getRec(tables="bwlflm", cols=["bfm_round",
            "bfm_date", "bfm_forms"], where=[("bfm_fmat", "=", self.fmat),
            ("bfm_date", ">=", self.sdate, "and", "bfm_date", "<",
            self.edate)], group="bfm_date", order="bfm_round")
        self.forms = self.rounds[-1][2]

    def doEnd(self):
        self.df.closeProcess()
        data = []
        dat = self.sql.getRec(tables=["bwlflt", "bwlflp"], cols=["bft_player",
            "bfp_snam", "bfp_fnam"], where=[("bft_fmat", "=", self.fmat),
            ("bft_date", ">=", self.sdate, "and", "bft_date", "<", self.edate),
            ("bfp_code=bft_player",)], group="bft_player, bfp_snam, bfp_fnam",
            order="bfp_snam, bfp_fnam")
        for plr in dat:
            values = [self.getName(plr[1], plr[2])]
            if self.forms == 1:
                values = values + [0] * 10
            else:
                values = values + [0] * 12
            his = 0
            tot = 0
            cnt = 0
            for rnd in self.rounds:
                pl = self.sql.getRec(tables=["bwlfls", "bwlflt"],
                    cols=["bft_position", "bft_rating1", "bft_rating2",
                    "bft_rating3", "bft_rating4", "bft_skip"],
                    where=[("bfs_fmat", "=", self.fmat), ("bfs_league", "=",
                    "M"), ("bft_date", "=", rnd[1]), ("bft_player", "=",
                    plr[0]), ("bft_fmat=bfs_fmat",), ("bft_team=bfs_code",)],
                    limit=1)
                if not pl:
                    # Did not play
                    continue
                if not pl[1] and not pl[2] and not pl[3] and not pl[4]:
                    # Not yet assessed
                    continue
                cnt += 1.0
                his += pl[pl[0]]
                team = self.sql.getRec(tables="bwlflt", where=[("bft_fmat",
                    "=", self.fmat), ("bft_date", "=", rnd[1]), ("bft_skip",
                    "=", pl[5])])
                tmp = 0
                for tm in team:
                    tmp += tm[pl[0]+7]
                    tot += tm[pl[0]+7]
                avg = round(tmp / 4.0, 1)
                values[rnd[0]] = avg
            if not tot:
                # No assessments
                continue
            values[10] = round(tot / 4.0 / cnt, 1)
            if self.forms == 4:
                values[11] = round(his / cnt)
                values[12] = round((tot - his) / 3.0 / cnt, 1)
            data.append(values)
        head = ("Assessments Summary for %s Season Commencing on the %s" % \
            (self.fnam, self.ddisp),)
        if self.forms == 1:
            cols = [["a", "NA",  40, "Player", "y"]]
        else:
            cols = [["a", "NA",  30, "Player", "y"]]
        cols.extend([
            ["b", "UD", 4.1, "R1",     "y"],
            ["c", "UD", 4.1, "R2",     "y"],
            ["d", "UD", 4.1, "R3",     "y"],
            ["e", "UD", 4.1, "R4",     "y"],
            ["f", "UD", 4.1, "R5",     "y"],
            ["g", "UD", 4.1, "R6",     "y"],
            ["h", "UD", 4.1, "R7",     "y"],
            ["i", "UD", 4.1, "R8",     "y"],
            ["j", "UD", 4.1, "R9",     "y"],
            ["k", "UD", 4.1, "AV",     "y"]])
        if self.forms == 4:
            cols.extend([
                ["l", "UD", 4.1, "HA",     "y"],
                ["m", "UD", 4.1, "NA",     "y"]])
        RepPrt(self.mf, name=__name__, tables=data, heads=head, cols=cols,
            ttype="D", blank=True, view=self.df.repprt[0][1],
            prtnam=self.df.repprt[0][2], mail=self.df.repprt[1][2])
        self.mf.closeLoop()

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()

    def getName(self, snam, fnam):
        for i in fnam.split():
            snam = "%s %s" % (snam, i[0].capitalize())
        return snam
