#! /usr/bin/env python

"""
SYNOPSIS
    Creditors Ledger Master Listing.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import CCD, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, getModName, doPrinter, showError

class cr3060:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, "crsmst", prog=__name__)
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdtd = "%i/%02i/%02i" % (t[0], t[1], t[2])
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Creditors Master Listing (%s)" % __name__)
        r1s = (("Number","N"),("Name","M"))
        r2s = (("Yes","Y"),("No","N"))
        r3s = (("Manager","M"),("Accounts","A"),("Orders","O"))
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"Sort Order","Sort Order",
                "N","Y",self.doSort,None,None,None),
            (("T",0,1,0),("IRB",r2s),0,"Use Full Address","Use Full Address",
                "N","Y",self.doAddress,None,None,None),
            (("T",0,2,0),("IRB",r3s),0,"Contact Details","Contact Details",
                "M","Y",self.doContact,None,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doSort(self, frt, pag, r, c, p, i, w):
        self.sort = w

    def doAddress(self, frt, pag, r, c, p, i, w):
        self.address = w

    def doContact(self, frt, pag, r, c, p, i, w):
        self.condet = w

    def doEnd(self):
        self.df.closeProcess()
        if self.sort == "N":
            ord = "crm_acno"
        else:
            ord = "crm_name"
        all = self.sql.getRec(tables="crsmst", where=[("crm_cono", "=",
            self.conum)], order=ord)
        if not all:
            showError(self.mf.body, "Selection Error",
            "No Records Selected")
        else:
            self.printReport(all)
        self.closeProcess()

    def printReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        self.head = ("%03u %-30s %128s %10s" % \
            (self.conum, self.conam, self.sysdttm, __name__))
        pdfnam = getModName(self.mf.wrkdir, __name__, self.conum, ext="pdf")
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, pdfnam, self.head)
        self.pgnum = 0
        self.pglin = 99
        col = self.sql.crsmst_col
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            acno = CCD(dat[col.index("crm_acno")], "NA", 7)
            name = CCD(dat[col.index("crm_name")], "NA", 30)
            add1 = CCD(dat[col.index("crm_add1")], "NA", 30)
            add2 = CCD(dat[col.index("crm_add2")], "NA", 30)
            add3 = CCD(dat[col.index("crm_add3")], "NA", 30)
            pcod = CCD(dat[col.index("crm_pcod")], "NA", 4)
            tel = CCD(dat[col.index("crm_tel")], "NA", 12)
            fax = CCD(dat[col.index("crm_fax")], "NA", 12)
            if self.condet == "M":
                contact = dat[col.index("crm_mgr")]
                email = dat[col.index("crm_mgr_email")].split(",")
            elif self.condet == "A":
                contact = dat[col.index("crm_acc")]
                email = dat[col.index("crm_acc_email")].split(",")
            else:
                contact = dat[col.index("crm_ord")]
                email = dat[col.index("crm_ord_email")].split(",")
            nameml = CCD("%s <%s>" % (contact, email[0]), "NA", 40)
            pyind = CCD(dat[col.index("crm_pyind")], "NA", 1)
            terms = CCD(dat[col.index("crm_terms")], "UI", 3)
            termsb = CCD(dat[col.index("crm_termsb")], "NA", 1)
            stday = CCD(dat[col.index("crm_stday")], "UI", 2)
            limit = CCD(dat[col.index("crm_limit")], "UI", 7)
            trdis = CCD(dat[col.index("crm_trdis")], "UD", 6.2)
            pydis = CCD(dat[col.index("crm_pydis")], "UD", 6.2)
            if self.pglin > self.lpp:
                if self.pgnum:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
            self.t.textLine("%s %s %s %s %s %s %s %s %s %s %s %s %s %s" % \
                (acno.disp, name.disp, add1.disp, pcod.disp, tel.disp, fax.disp,
                nameml.disp, pyind.disp, terms.disp, termsb.disp, stday.disp,
                limit.disp, trdis.disp, pydis.disp))
            if self.address == "Y" and add2.work:
                self.t.textLine("%38s %-30s" % ("", add2.disp))
                self.pglin = self.pglin + 1
            if self.address == "Y" and add3.work:
                self.t.textLine("%38s %-30s" % ("", add3.disp))
                self.pglin = self.pglin + 1
            self.pglin = self.pglin + 1
        p.closeProgress()
        if self.t:
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=pdfnam, head=self.tit,
            view=self.df.repprt[0][1], prtnam=self.df.repprt[0][2],
            mail=self.df.repprt[1][2])
        self.closeProcess()

    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-34s %-10s %122s %5s" % \
            ("Creditors Ledger Master List as at", self.sysdtd,
                "Page", self.pgnum))
        self.t.textLine()
        self.t.textLine("%-15s%-1s%-1s" % ("(Options: Sort-",
            self.df.t_disp[0][0][0], ")"))
        self.t.textLine()
        if self.condet == "M":
            det = "Manager's Contact Details"
        elif self.condet == "A":
            det = "Accounts Contact Details"
        else:
            det = "Orders Contact Details"
        self.t.textLine("%-7s %-30s %-30s %-4s %-12s %-12s %-40s %-1s %-3s "\
            "%-1s %-2s %-7s %-6s %-6s" % ("Acc-Num", "Name", "Address", "PCod",
            "Tel-Number", "Fax-Number", "%s" % det, "P", "Trm", "B", "St",
            "CrLimit", "Tr-Dis", "Py-Dis"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 8

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

