#! /usr/bin/env python

"""
SYNOPSIS
    Creditors Ledger Remittance Advices.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import os, time
from PIL import Image
from TartanClasses import ASD, CCD, GetCtl, ProgressBar, Sql, TartanDialog
from tartanFunctions import doPrinter, doReportlab, flipImage, getModName
from tartanFunctions import getTrn, getSingleRecords, showError
from tartanWork import crtrtp

class cr3080:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sql = Sql(self.mf.dbm, ["ctlctl", "ctlmst", "crsctl", "crsmst",
            "crstrn", "crsage", "gentrn"], prog=__name__)
        if self.sql.error:
            return self.sql.error
        gc = GetCtl(self.mf)
        crsctl = gc.getCtl("crsctl", self.conum)
        if not crsctl:
            return "no"
        self.glint = crsctl["ctc_glint"]
        self.bestac = crsctl["ctc_bestac"]
        self.besttp = crsctl["ctc_besttp"]
        self.bankac = crsctl["ctc_bankac"]
        if self.glint == "Y":
            ctlctl = gc.getCtl("ctlctl", self.conum)
            if gc.chkRec(self.conum, ctlctl, ["crs_ctl"]):
                return "no"
            self.crsctl = ctlctl["crs_ctl"]
        acc = self.sql.getRec(tables="crstrn", cols=["max(crt_ref1)"],
            where=[("crt_cono", "=", self.conum), ("crt_type", "=", 5),
            ("crt_ref1", "like", "EFT______")], limit=1)
        try:
            self.cats = int(acc[0][3:])
            self.refs = int(acc[0][3:]) + 1
        except:
            self.cats = 0
            self.refs = 1
        self.etotal = 0
        self.t = None
        self.maxlines = 40
        self.buildHead()

    def buildHead(self):
        self.head = []
        ctl = self.sql.getRec(tables=["ctlmst"], cols=["ctm_name", "ctm_add1",
            "ctm_add2", "ctm_add3", "ctm_pcode", "ctm_regno", "ctm_taxno",
            "ctm_tel", "ctm_fax", "ctm_logo", "ctm_flip"], where=[("ctm_cono",
            "=", self.conum)], limit=1)
        self.logo = ctl[9]
        self.flip = ctl[10]
        if not self.logo or not os.path.exists(self.logo):
            self.logo = None
        elif self.flip == "N":
            self.logo = flipImage(self.logo)
        if self.logo:
            self.head.append("%34s%10s" % ("", "REMITTANCE"))
            self.head.append(" ")
            self.head.append(" ")
            self.head.append(" ")
            self.head.append(" ")
        else:
            mess = "%1s%-30s%3s%10s" % ("", ctl[0], "", "REMITTANCE")
            self.head.append(mess)
            mess = "%2s%-30s%2s%-30s" % ("", ctl[1], "", ("RegNo: " + ctl[5]))
            self.head.append(mess)
            mess = "%2s%-30s%2s%-30s" % ("", ctl[2], "", ("TaxNo: " + ctl[6]))
            self.head.append(mess)
            mess = "%2s%-30s%2s%-30s" % ("", ctl[3], "", ("TelNo: " + ctl[7]))
            self.head.append(mess)
            mess = "%2s%-30s%2s%-30s" % ("", ctl[4], "", ("FaxNo: " + ctl[8]))
            self.head.append(mess)

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Creditors Remittance Advices (%s)" % __name__)
        data = (("Account Number", "N"), ("Postal Code", "P"))
        crm = {
            "stype": "R",
            "tables": ("crsmst",),
            "cols": (
                ("crm_acno", "", 0, "Acc-Num"),
                ("crm_name", "", 0, "Name", "Y")),
            "where": [("crm_cono", "=", self.conum)]}
        r1s = (("Yes","Y"),("Range","R"),("Singles", "S"))
        r2s = (("Yes","Y"),("No","N"))
        r3s = (("Number","N"),("Name","M"),("Postal Code","P"))
        r4s = (("Monthly","M"),("Daily","D"))
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"Whole File","Whole File",
                "S","Y",self.doWhole,None,None,None),
            [["T",0,1,0],"INA",7,"From Account","From Account",
                "","Y",self.doAcc,crm,None,None],
            [["T",0,2,0],"INA",7,"To   Account","To Account",
                "","Y",self.doAcc,crm,None,None],
            (("T",0,3,0),("IRB",r3s),0,"Sort Order","Sort Order",
                "N","Y",self.doSort,None,None,None),
            (("T",0,4,0),("IRB",r4s),0,"Terms Base","Terms Base",
                "M","Y",self.doFrequency,None,None,None),
            (("T",0,5,0),("IRB",r2s),0,"Exceptions","Exceptions",
                "N","Y",self.doExcepts,None,None,None),
            (("T",0,6,0),"ID1",10,"Due Date","Due Date",
                self.sysdtw,"Y",self.doDuedat,None,None,("efld",)),
            (("T",0,7,0),"ID1",10,"Payment Date","Payment Date",
                self.sysdtw,"Y",self.doPaydat,None,None,("efld",)))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("N","N"), save="N")

    def doWhole(self, frt, pag, r, c, p, i, w):
        self.whole = w
        if self.whole in ("Y", "S"):
            self.sacc = ""
            self.eacc = ""
            self.df.loadEntry("T", 0, 1, data=self.sacc)
            self.df.loadEntry("T", 0, 2, data=self.eacc)
            return "sk2"

    def doAcc(self, frt, pag, r, c, p, i, w):
        chk = self.sql.getRec(tables=["crsmst"], cols=["crm_name"],
            where=[("crm_cono", "=", self.conum), ("crm_acno", "=", w)],
            limit=1)
        if not chk:
            return "Invalid Creditors Account"
        if c == 2:
            self.sacc = w
        else:
            self.eacc = w

    def doSort(self, frt, pag, r, c, p, i, w):
        self.order = w

    def doFrequency(self, frt, pag, r, c, p, i, w):
        self.freq = w

    def doExcepts(self, frt, pag, r, c, p, i, w):
        self.excepts = w

    def doDuedat(self, frt, pag, r, c, p, i, w):
        self.duedtw = w
        self.duedtd = self.df.t_disp[pag][0][p]

    def doPaydat(self, frt, pag, r, c, p, i, w):
        self.paydtw = w
        self.paydtd = self.df.t_disp[pag][0][p]
        self.curdt = self.paydtw / 100
        self.batno = "E%s" % self.curdt

    def doEnd(self):
        self.df.closeProcess()
        if self.excepts == "Y":
            self.doExceptions()
        self.prtnam = self.df.repprt[0][2]
        self.email = self.df.repprt[1][1]
        self.archive = self.df.repprt[2][1]
        self.pdfnam = None
        if self.bestac:
            self.export = open(os.path.join(self.mf.wrkdir,
                "best%03d_%s.txt" % (self.conum, self.paydtw)), "w")
            # Header for BEST
            self.export.write("%1s%4s%-40s%8s%1s%8s%-15s%1s%2s%1s%9s%2s%4s"\
                "\r\n" % ("*", self.bestac, self.conam, self.paydtw, "Y", "",
                "CREDITORS EFT", "+", self.besttp, 0, "", "01", "LIVE"))
        else:
            self.export = None
        if self.email == "N":
            self.printSetup()
        if self.whole == "S":
            all = getSingleRecords(self.mf, "crsmst",
                ("crm_acno", "crm_name"), where=[("crm_cono", "=", self.conum),
                ("crm_termsb", "=", self.freq), ("crm_pyind", "<>", "N")])
        else:
            if not self.eacc:
                self.eacc = "zzzzzzz"
            whr = [("crm_cono", "=", self.conum), ("crm_acno", "between",
                self.sacc, self.eacc), ("crm_termsb", "=", self.freq),
                ("crm_pyind", "<>", "N")]
            if self.order == "N":
                order = "crm_acno"
            elif self.sort == "M":
                order = "crm_name"
            else:
                order = "crm_pcod"
            all = self.sql.getRec(tables=["crsmst"], where=whr, order=order)
            if not all:
                showError(self.mf.body, "Error", "No Accounts Selected")
        if all:
            p = ProgressBar(self.mf.body, max=len(all), esc=True)
            for num, rec in enumerate(all):
                can = p.displayProgress(num)
                if can == "yes":
                    self.mf.dbm.rollbackDbase()
                    break
                self.doProcess(rec)
            if self.email == "N":
                self.c.save()
                self.doPrint()
            if self.bestac:
                # Trailer for BEST
                value = long(round((self.etotal * 100), 0))
                self.export.write("%1s%4s%1s%30s%013u%47s\r\n" % \
                    (2, self.bestac, "T", "", value, ""))
                self.export.close()
                if self.glint == "Y" and self.etotal:
                    # Create total transactions in GL
                    data = [self.conum, self.crsctl, self.curdt, self.paydtw,
                        2, self.refno, self.batno, self.etotal, 0.0,
                        "Payment EFT%06i to EFT%06i" % (self.refs, self.cats),
                        "", "", 0, self.sysdtw, 0]
                    self.sql.insRec("gentrn", data=data)
                    data[1] = self.bankac
                    data[7] = float(ASD(0) - ASD(self.etotal))
                    self.sql.insRec("gentrn", data=data)
                self.mf.dbm.commitDbase(ask=True, mess="""Would you like to commit all elecronic payments?
                
If you decide to do this, you must remember to upload the BEST file to the Bank otherwise you are NOT going to Reconcile!""", default="no")
            p.closeProgress()
        self.closeProcess()

    def doExceptions(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "Payment Exceptions (%s)" % __name__)
        crm = {
            "stype": "R",
            "tables": ("crsmst",),
            "cols": (
                ("crm_acno", "", 0, "Acc-Num"),
                ("crm_name", "", 0, "Name", "Y")),
            "where": [("crm_cono", "=", self.conum), ("crm_termsb", "=",
                self.freq)]}
        crt = {
            "stype": "R",
            "tables": ("crstrn",),
            "cols": (
                ("crt_ref1", "", 0, "Reference", "Y"),
                ("crt_type", ("XX", crtrtp), 3, "Typ"),
                ("crt_trdt", "", 0, "Date"),
                ("crt_tramt", "", 0, "    Amount"),
                ("paid", "SD", 11.2, "      Paid"),
                ("balance", "SD", 11.2, "   Balance"),
                ("crt_paydt", "", 0, "Pay-Date"),
                ("crt_payind", "", 0, "I"),
                ("crt_payamt", "", 0, "  Pay-Amnt")),
            "wtype": "D",
            "where": [],
            "order": "crt_ref1"}
        types = []
        for x in range(1, len(crtrtp)+1):
            types.append((x, crtrtp[x-1][1]))
        typ = {
            "stype": "C",
            "titl": "Select the Required Type",
            "data": types}
        fld = (
            (("T",0,0,0),"I@crm_acno",0,"","",
                "","N",self.doExAcNo,crm,None,("notblank",)),
            (("T",0,0,0),"O@crm_name",0,""),
            (("C",0,0,1),"I@crt_type",0,"", "",
                "","N",self.doExTrnTyp,typ,None,("in",(1,2,3,4,5))),
            (("C",0,0,0),"I@crt_ref1",0,"","",
                "","N",self.doExTrnRef,crt,None,("notblank",)),
            (("C",0,0,2),"O@crt_trdt",0,""),
            (("C",0,0,3),"OSD",13.2,"Balance"),
            (("C",0,0,4),"I@crt_payind",0,"","",
                "","N",self.doExInd,None,None,("in", ("Y", "N"))),
            (("C",0,0,5),"I@crt_paydt",0,"","",
                "","N",self.doExDte,None,None,("efld",)),
            (("C",0,0,6),"I@crt_payamt",0,"","",
                "","N",self.doExAmt,None,None,("efld",)))
        tnd = ((self.doExEndTop,"n"),)
        txt = (self.doExExitTop,)
        cnd = ((self.doExEndCol,"y"),)
        cxt = (self.doExExitCol,)
        self.ex = TartanDialog(self.mf, title=tit, eflds=fld, tend=tnd,
            txit=txt, cend=cnd, cxit=cxt)
        self.mf.startLoop()

    def doExAcNo(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables=["crsmst"], cols=["crm_name",
            "crm_termsb", "crm_pyind"], where=[("crm_cono", "=", self.conum),
            ("crm_acno", "=", w)], limit=1)
        if not acc:
            return "Invalid Account Number"
        if acc[1] != self.freq:
            return "Invalid Terms Base"
        if acc[2] == "N":
            return "Invalid Payment Indicator"
        self.exacc = w
        self.ex.loadEntry(frt, pag, p+1, data=acc[0])

    def doExEndTop(self):
        self.ex.focusField("C", 0, 1)

    def doExExitTop(self):
        self.mf.dbm.commitDbase(ask=True, mess="Would you like to commit "\
            "these exceptions?")
        self.ex.closeProcess()
        self.mf.closeLoop()

    def doExTrnTyp(self, frt, pag, r, c, p, i, w):
        self.extyp = w
        data = []
        # Build the data for the F1 choice selection
        col, dat = getTrn(self.mf.dbm, "crs", whr=[("crt_cono", "=",
            self.conum), ("crt_acno", "=", self.exacc), ("crt_type", "=",
            w)], zer="N")
        if dat:
            cols = ("crt_ref1", "crt_type", "crt_trdt", "crt_tramt", "paid",
                "balance", "crt_paydt", "crt_payind", "crt_payamt")
            for d in dat:
                rec = []
                for c in cols:
                    rec.append(d[col.index(c)])
                data.append(rec)
        self.ex.colf[0][1][8]["where"] = data

    def doExTrnRef(self, frt, pag, r, c, p, i, w):
        col = ["crt_trdt", "balance", "crt_payind", "crt_paydt", "crt_payamt"]
        whr = [("crt_cono", "=", self.conum), ("crt_acno", "=", self.exacc),
            ("crt_type", "=", self.extyp), ("crt_ref1", "=", w)]
        c, d = getTrn(self.mf.dbm, "crs", whr=whr, lim=1)
        if not d:
            return "Invalid Transaction Number"
        if not d[0][c.index("balance")]:
            return "Transaction Has No Balance"
        self.exref = w
        for pos, fld in enumerate(col):
            self.ex.loadEntry(frt, pag, p+1+pos, data=d[0][c.index(fld)])

    def doExInd(self, frt, pag, r, c, p, i, w):
        self.exind = w

    def doExDte(self, frt, pag, r, c, p, i, w):
        self.exdte = w

    def doExAmt(self, frt, pag, r, c, p, i, w):
        self.examt = w

    def doExEndCol(self):
        # Update Transaction"
        self.sql.updRec("crstrn", cols=["crt_payind", "crt_paydt",
            "crt_payamt"], data=[self.exind, self.exdte, self.examt],
            where=[("crt_cono", "=", self.conum), ("crt_acno", "=", self.exacc),
                ("crt_type", "=", self.extyp), ("crt_ref1", "=", self.exref)])
        self.ex.advanceLine(0)

    def doExExitCol(self):
        self.ex.focusField("T", 0, 1)
         
    def doProcess(self, crm):
        self.pgnum = 0
        self.totbal = 0
        cmc = self.sql.crsmst_col
        self.acno = CCD(crm[cmc.index("crm_acno")], "NA", 7)
        jon = "cra_curdt <= %s" % self.curdt
        whr = [("crt_cono", "=", self.conum), ("crt_acno", "=", self.acno.work),
            ("crt_payind", "=", "Y"), ("crt_paydt", "<=", self.duedtw),
            ("crt_payamt", "<>", 0)]
        self.ct, crt = getTrn(self.mf.dbm, "crs", jon=jon, whr=whr, zer="N")
        if not crt:
            return
        bal = 0
        pay = 0
        for d in crt:
            bal = float(ASD(bal) + ASD(d[self.ct.index("balance")]))
            pay = float(ASD(pay) + ASD(d[self.ct.index("crt_payamt")]))
        if pay > bal:
            pay = bal
        if pay > 0:
            if self.email == "Y":
                self.printSetup()
            self.name = CCD(crm[cmc.index("crm_name")], "NA", 30)
            self.add1 = CCD(crm[cmc.index("crm_add1")], "NA", 30)
            self.add2 = CCD(crm[cmc.index("crm_add2")], "NA", 30)
            self.add3 = CCD(crm[cmc.index("crm_add3")], "NA", 30)
            self.pcod = CCD(crm[cmc.index("crm_pcod")], "NA", 4)
            self.emadd = CCD(crm[cmc.index("crm_acc_email")], "TX", 0)
            self.bname = crm[cmc.index("crm_bname")]
            self.bibt = crm[cmc.index("crm_bibt")]
            self.bacc = crm[cmc.index("crm_bacc")]
            if self.bname and self.bibt and self.bacc:
                self.ptype = "E"                        # Electronic
                test = False
                while not test:
                    self.cats += 1
                    self.refno = "EFT%06i" % self.cats
                    # Check if Reference Number Already Exists
                    chk = self.sql.getRec(tables="crstrn", where=[("crt_cono",
                        "=", self.conum), ("crt_acno", "=", self.acno.work),
                        ("crt_type", "=", 5), ("crt_ref1", "=", self.refno)])
                    if not chk:
                        test = True
            else:
                self.ptype = "C"                        # Cheque
            self.printHeader()
            self.printBody(crt)
            self.printTrailer()
            if self.email == "Y":
                self.c.save()
                self.doPrint()

    def printSetup(self):
        if self.archive == "Y":
            dir = self.mf.arcdir
        else:
            dir = self.mf.wrkdir
        if self.email == "Y":
            key = "%s_%s" % (self.conum, self.acno.work)
            self.pdfnam = getModName(dir, __name__, key, ext="pdf")
        elif not self.pdfnam:
            self.pdfnam = getModName(dir, __name__, self.conum, ext="pdf")
        self.c, self.cpi, self.inch, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam)
        self.cwidth = self.c.stringWidth(("*"), "Tartan", self.cpi)

    def doPrint(self):
        if self.prtnam == "view":
            view = "Y"
        else:
            view = "N"
        if self.email == "N" or not self.emadd.work:
            mail = "N"
        else:
            mail = self.emadd.work
        doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam, head=self.tit,
            view=view, prtnam = self.prtnam, mail=mail)

    def printHeader(self):
        self.drawDocument()
        self.pgnum = self.pgnum + 1
        self.t.textLine()
        self.t.textLine()
        self.t.setFont("Tartan-Bold", 18)
        self.t.textLine(self.head[0])
        self.t.setFont("Tartan-Bold", 10)
        self.t.textLine(self.head[1])
        self.t.textLine(self.head[2])
        self.t.textLine(self.head[3])
        self.t.textLine(self.head[4])
        self.t.setFont("Tartan", 10)
        self.t.textLine()
        self.t.textLine()
        mess = "%1s%-30s" % ("", self.name.disp)
        self.t.textLine(mess)
        mess = "%1s%-30s%40s%7s" % ("", self.add1.disp, "", self.acno.disp)
        self.t.textLine(mess)
        mess = "%1s%-30s" % ("", self.add2.disp)
        self.t.textLine(mess)
        mess = "%1s%-30s%37s%10s" % ("", self.add3.disp, "", self.pgnum)
        self.t.textLine(mess)
        mess = "%1s%-30s" % ("", self.pcod.disp)
        self.t.textLine(mess)
        mess = "%68s%-10s" % ("", self.paydtd)
        self.t.textLine(mess)
        self.t.textLine()
        self.t.textLine()
        self.t.setFont("Tartan", 10)
        self.pglin = 0

    def printBody(self, crt):
        for trn in crt:
            trtp = CCD(trn[self.ct.index("crt_type")], "UI", 1)
            ref1 = CCD(trn[self.ct.index("crt_ref1")], "Na", 9)
            ref2 = CCD(trn[self.ct.index("crt_ref2")], "Na", 9)
            trdt = CCD(trn[self.ct.index("crt_trdt")], "D1", 10)
            tramt = CCD(trn[self.ct.index("crt_tramt")], "SD", 13.2)
            trbal = CCD(trn[self.ct.index("balance")], "SD", 13.2)
            payamt = trn[self.ct.index("crt_payamt")]
            if trbal.work > 0 and payamt > trbal.work:
                payamt = trbal.work
            elif trbal.work < 0 and payamt < trbal.work:
                payamt = trbal.work
            payamt = CCD(payamt, "SD", 13.2)
            paid = CCD(float(ASD(tramt.work) - ASD(trbal.work)), "SD", 13.2)
            if self.pglin > (self.maxlines - 1):
                self.printTrailer(fwd="Y")
                self.printHeader()
            self.t.textLine("%-2s%s %s %s %s %s %s %s" % ("", trdt.disp,
                ref1.disp, ref2.disp, crtrtp[trtp.work - 1][0], tramt.disp,
                paid.disp, payamt.disp))
            if self.ptype == "E" and self.bestac:
                # Create Ageing Transaction
                self.sql.insRec("crsage", data=[self.conum, self.acno.work,
                    trtp.work, ref1.work, self.curdt, 5, self.refno,
                    payamt.work, 0])
            self.pglin = self.pglin + 1
            self.totbal = float(ASD(self.totbal) + ASD(payamt.work))

    def printTrailer(self, fwd="N"):
        for z in range(0, (self.maxlines + 2 - self.pglin)):
            self.t.textLine()
        if fwd == "Y":
            mess = "%-66s%-12s" % ("", "Continued...")
            self.t.textLine(mess)
        else:
            j = CCD(self.totbal, "SD", 13.2)
            self.t.textLine("%-64s %13s" % ("", j.disp))
            if self.ptype == "E" and self.bestac:
                value = long(round((self.totbal * 100), 0))
                self.export.write("%1s%4s%06u%-7s%019u%1s%1s%011u%-20s%10s"\
                    "%-15s%1s\r\n" % (2, self.bestac, self.bibt,
                    self.acno.disp, long(self.bacc), "", "1",
                    value, self.bname, "", self.conam[:15], ""))
                self.etotal = float(ASD(self.etotal) + ASD(self.totbal))
                self.t.textLine()
                self.t.textLine("%5sELECTRONIC TRANSFER AT %s "\
                    "REFERENCE %s" % ("", self.paydtd, self.refno))
                # Create Payment and Ageing Transaction
                p = float(ASD(0) - ASD(self.totbal))
                self.sql.insRec("crstrn", data=[self.conum, self.acno.work, 5,
                    self.refno, self.batno, self.paydtw, "", p, 0.0, 0.0,
                    self.curdt, 0, "", 0.0, "Electronic Payment", "", "",
                    self.sysdtw, 0])
                self.sql.insRec("crsage", data=[self.conum, self.acno.work, 5,
                    self.refno, self.curdt, 5, self.refno, p, 0])
        self.c.drawText(self.t)
        self.c.showPage()

    def drawDocument(self):
        self.c.translate(self.inch*0.4, self.inch)
        self.t = self.c.beginText()
        self.t.setFont("Tartan-Bold", self.cpi)
        self.t.setTextOrigin(0, 0)
        # If logo, draw logo
        if self.logo:
            self.c.drawImage(self.logo, 0, 3, 350, 70, None)
        # Draw Account Rectangles
        self.c.setFillGray(.8)
        x = self.cwidth*.5
        w = self.cwidth*32.5
        self.c.rect(x, (7.75*(self.inch/6)), w, (1*(self.inch/6)), fill=1)
        self.c.rect(x, (7.75*(self.inch/6)), w, (7.25*(self.inch/6)), fill=0)
        self.c.setFillGray(0)
        self.c.setFont("Tartan-Bold", 8)
        self.c.drawString(self.cwidth*1, (8.45*(self.inch/6)), "Account")
        self.c.setFont("Tartan", 10)
        # Draw Page Rectangles
        self.c.setFillGray(.8)
        x = self.cwidth*64.5
        w = self.cwidth*14
        for q in (9, 11, 13):
            self.c.rect(x, (q*(self.inch/6)), w, (1*(self.inch/6)), fill=1)
            self.c.rect(x, (q*(self.inch/6)), w, (2*(self.inch/6)), fill=0)
        self.c.setFillGray(0)
        self.c.setFont("Tartan-Bold", 8)
        self.c.drawCentredString((self.cwidth*71.5),
            (9.80*(self.inch/6)), "Acc-Num")
        self.c.drawCentredString((self.cwidth*71.5),
            (11.80*(self.inch/6)), "Page")
        self.c.drawCentredString((self.cwidth*71.5),
            (13.80*(self.inch/6)), "Date")
        self.c.setFont("Tartan", 10)
        # Draw Ribbon Rectangles
        self.c.setFillGray(.8)
        x = self.cwidth*.5
        w = self.cwidth*78
        self.c.rect(x, (15.50*(self.inch/6)), w, ( 1.25*(self.inch/6)), fill=1)
        self.c.rect(x, (15.50*(self.inch/6)), w, (41.75*(self.inch/6)), fill=0)
        self.c.setFillGray(0)
        self.c.setFont("Tartan-Bold", 10)
        for q, r in ((2, "   Date"), (13, "Reference"), (23, "Ref-Num-2"),
            (33, "Typ"), (37, " Orig-Amount"), (51, " Paid-Amount"),
            (65, "  Bal-Amount")):
            self.c.drawString(self.cwidth*q, (16.40*(self.inch/6)), r)
        self.c.setFont("Tartan", 10)
        for q in (12.5, 22.5, 32.5, 36.5, 50.5, 64.5):
            self.c.line(self.cwidth*q, (15.50*(self.inch/6)), self.cwidth*q,
            (57.25*(self.inch/6)))
        # Draw Total Rectangles
        self.c.setFillGray(.8)
        x = self.cwidth*64.5
        w = self.cwidth*14
        self.c.rect(x, (57.50*(self.inch/6)), w, (1.25*(self.inch/6)), fill=1)
        self.c.rect(x, (57.50*(self.inch/6)), w, (2.75*(self.inch/6)), fill=0)
        self.c.setFillGray(0)
        self.c.setFont("Tartan-Bold", 10)
        self.c.drawCentredString((self.cwidth*71.5),
            (58.40*(self.inch/6)), "Total Payment")
        self.c.setFont("Tartan", 10)

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

