#! /usr/bin/python

"""
SYNOPSIS
    Debtors Ledger - Raising Recurring Charges.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, Batches, GetCtl, Sql, TartanDialog
from tartanFunctions import getSingleRecords, mthendDate

class dr2030:
    def __init__(self, mf, coy, nam, period, rtn, usr):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.period = period
        self.rtn = rtn
        self.capnm = usr
        no = self.checkControls()
        if not no:
            self.setVariables()
            self.doProcess()
            self.mf.startLoop()

    def checkControls(self):
        self.sql = Sql(self.mf.dbm, ["ctlvmf", "ctlvtf", "drsmst", "drstrn",
            "drsrcm", "drsrct", "genmst", "gentrn"], prog=__name__)
        self.gc = GetCtl(self.mf)
        drsctl = self.gc.getCtl("drsctl", self.conum)
        if not drsctl:
            return "no"
        self.glint = drsctl["ctd_glint"]
        if self.glint == "Y":
            ctlctl = self.gc.getCtl("ctlctl", self.conum)
            if not ctlctl:
                return "no"
            if self.gc.chkRec(self.conum, ctlctl, ["drs_ctl", "vat_ctl"]):
                return "no"
            self.drsctl = ctlctl["drs_ctl"]
            self.vatctl = ctlctl["vat_ctl"]
        self.batchHeader()
        if not self.bh.batno:
            return "no"

    def setVariables(self):
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]

    def batchHeader(self):
        self.bh = Batches(self.mf, self.conum, self.conam, self.period,
            "DRS", self.rtn, multi="N", glint=self.glint)
        self.bh.doBatchHeader()
        if not self.bh.batno:
            return

    def doProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "Debtors Ledger Recurring Charges (%s)" % __name__)
        r1s = (
            ("Monthly","M"),
            ("Quarterly","3"),
            ("Bi-Annually","6"),
            ("Annually","Y"))
        r2s = (("Yes", "Y"), ("No", "N"))
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"Frequency","",
                "M","N",self.doFreq,None,None,None),
            (("T",0,1,0),("IRB",r2s),1,"All Charges","",
                "N","N",self.doAll,None,None,None),
            (("T",0,2,0),"INA",9,"2nd Reference","",
                "","N",self.doRef2,None,None,None))
        tnd = ((self.endPage,"y"),)
        txt = (self.doExit,)
        self.df = TartanDialog(self.mf, title=tit, eflds=fld,
            tend=tnd, txit=txt)

    def doFreq(self, frt, pag, r, c, p, i, w):
        self.freq = w
        self.wher = [("dcm_cono", "=", self.conum), ("dcm_freq", "=",
            self.freq), ("dcm_last", "<", self.bh.curdt)]
        data = self.sql.getRec(tables="drsrcm", where=self.wher)
        if not data:
            return "No Valid Recurring Charges"
        if self.freq == "M":
            self.mths = 1
        elif self.freq == "3":
            self.mths = 3
        elif self.freq == "6":
            self.mths = 6
        else:
            self.mths = 12

    def doAll(self, frt, pag, r, c, p, i, w):
        self.all = w

    def doRef2(self, frt, pag, r, c, p, i, w):
        self.ref2 = w

    def endPage(self):
        self.df.closeProcess()
        if self.all == "N":
            all = getSingleRecords(self.mf, "drsrcm", ("dcm_num", "dcm_desc"),
                where=self.wher)
        else:
            all = self.sql.getRec(tables="drsrcm", where=self.wher)
        if all:
            for dcm in all:
                num = dcm[self.sql.drsrcm_col.index("dcm_num")]
                desc = dcm[self.sql.drsrcm_col.index("dcm_desc")]
                day = dcm[self.sql.drsrcm_col.index("dcm_day")]
                if day == 30:
                    trdt = mthendDate((self.bh.curdt * 100) + 1)
                else:
                    trdt = (self.bh.curdt * 100) + day
                vat = dcm[self.sql.drsrcm_col.index("dcm_vat")]
                rte = self.sql.getRec(tables="ctlvmf", cols=["vtm_new"],
                    where=[("vtm_cono", "=", self.conum), ("vtm_code",
                    "=", vat)], limit=1)[0]
                glac = dcm[self.sql.drsrcm_col.index("dcm_glac")]
                nxt = self.sql.getRec(tables="drstrn", cols=["max(drt_ref1)"],
                    where=[("drt_cono", "=", self.conum), ("drt_ref1", "like",
                    "RC%03i%s" % (num, "%"))], limit=1)
                if not nxt[0]:
                    nxt = 0
                else:
                    nxt = int(nxt[0][5:])
                tot_val = 0
                tot_vat = 0
                rec = self.sql.getRec(tables="drsrct", where=[("dct_cono", "=",
                    self.conum), ("dct_num", "=", num), ("dct_start", "<=",
                    self.bh.curdt), ("dct_end", ">=", self.bh.curdt)])
                for dct in rec:
                    # Check for Valid Period
                    charge = False
                    start = dct[self.sql.drsrct_col.index("dct_start")]
                    year = start / 100
                    month = start % 100
                    while not start > self.bh.curdt:
                        if start == self.bh.curdt:
                            charge = True
                            break
                        month += self.mths
                        if month > 12:
                            year += 1
                            month -= 12
                        start = (year * 100) + month
                    if not charge:
                        continue
                    # Create Transactions
                    seq = dct[self.sql.drsrct_col.index("dct_seq")]
                    chain = dct[self.sql.drsrct_col.index("dct_chain")]
                    acno = dct[self.sql.drsrct_col.index("dct_acno")]
                    name = self.sql.getRec(tables="drsmst", cols=["drm_name"],
                        where=[("drm_cono", "=", self.conum), ("drm_chain",
                        "=", chain), ("drm_acno", "=", acno)], limit=1)[0]
                    nxt += 1
                    ref = "RC%03i%04i" % (num, nxt)
                    detail = dct[self.sql.drsrct_col.index("dct_detail")]
                    amnt = dct[self.sql.drsrct_col.index("dct_amnt")]
                    vmnt = round(amnt * rte / 100, 2)
                    tmnt = float(ASD(amnt) + ASD(vmnt))
                    tot_val = float(ASD(tot_val) + ASD(amnt))
                    tot_vat = float(ASD(tot_vat) + ASD(vmnt))
                    # Debtors (drstrn)
                    self.sql.insRec("drstrn", data=[self.conum, chain, acno,
                        1, ref, self.bh.batno, trdt, self.ref2, tmnt, vmnt,
                        self.bh.curdt, detail, vat, "Y", self.sysdtw, 0])
                    # VAT (ctlvtf)
                    amnt = float(ASD(0) - ASD(amnt))
                    vmnt = float(ASD(0) - ASD(vmnt))
                    data = [self.conum, vat, "O", self.bh.curdt, "D", 1,
                        self.bh.batno, ref, trdt, acno, detail, amnt,
                        vmnt, self.sysdtw, 0, 0]
                    self.sql.insRec("ctlvtf", data=data)
                if self.glint == "Y":
                    ref = "RC%07i" % num
                    # Update Debtors Control
                    amnt = float(ASD(tot_val) + ASD(tot_vat))
                    data = (self.conum, self.drsctl, self.bh.curdt, trdt, 1,
                        ref, self.bh.batno, amnt, 0, desc, "", "", 0,
                        self.sysdtw, 0)
                    self.sql.insRec("gentrn", data=data)
                    # Update Sales Account
                    amnt = float(ASD(0) - ASD(tot_val))
                    data = (self.conum, glac, self.bh.curdt, trdt, 1, ref,
                        self.bh.batno, amnt, 0, desc, "", "", 0,
                        self.sysdtw, 0)
                    self.sql.insRec("gentrn", data=data)
                    amnt = float(ASD(0) - ASD(tot_vat))
                    if amnt:
                        # Update VAT Control
                        data = (self.conum, self.vatctl, self.bh.curdt, trdt,
                            1, ref, self.bh.batno, amnt, 0, desc, "", "", 0,
                            self.sysdtw, 0)
                        self.sql.insRec("gentrn", data=data)
                # Update Recurring Charge (drsrcm)
                self.sql.updRec("drsrcm", cols=["dcm_last"],
                    data=[self.bh.curdt], where=[("dcm_cono", "=", self.conum),
                    ("dcm_num", "=", num), ("dcm_freq", "=", self.freq)])
            self.mf.dbm.commitDbase()
        self.mf.closeLoop()

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()
