#! /usr/bin/env python

"""
SYNOPSIS
    Debtors Ledger Outstanding Transactions.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, CCD, GetCtl, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, doPrinter, getTrn, getModName
from tartanFunctions import showError
from tartanWork import drtrtp

class dr3040:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["drsmst", "drstrn"], prog=__name__)
        gc = GetCtl(self.mf)
        drsctl = gc.getCtl("drsctl", self.conum)
        if not drsctl:
            return "no"
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.curdt = self.sysdtw / 100
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Debtors Outstanding Transactions (%s)" % __name__)
        r1s = (("Yes","Y"),("No","N"))
        fld = (
            (("T",0,0,0),"ID2",7,"Report Period","Report Period",
                self.curdt,"Y",self.doRepPer,None,None,("efld",)),
            (("T",0,1,0),("IRB",r1s),0,"Account per Page","Account per Page",
                "N","Y",self.doNewPage,None,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doRepPer(self, frt, pag, r, c, p, i, w):
        self.curdt = w
        self.curdd = self.df.t_disp[0][0][0]

    def doNewPage(self, frt, pag, r, c, p, i, w):
        self.npag = w

    def doEnd(self):
        self.df.closeProcess()
        mst = self.sql.getRec(tables="drsmst", cols=["drm_chain", "drm_acno",
            "drm_name"], where=[("drm_cono", "=", self.conum)],
            order="drm_chain, drm_acno")
        if not mst:
            showError(self.mf.body, "Selection Error",
            "No Accounts Selected")
        else:
            self.printReport(mst)
        self.closeProcess()

    def printReport(self, mst):
        p = ProgressBar(self.mf.body, max=len(mst), esc=True)
        self.head = ("%03u %-30s %13s %33s %13s %10s" % \
            (self.conum, self.conam, "", self.sysdttm, "", __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        self.ctots = [0,0,0,0]
        self.gtots = [0,0,0,0]
        self.pgnum = 0
        self.pglin = 99
        for num, rec in enumerate(mst):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            chn = CCD(rec[0], "UI", 3)
            acno = CCD(rec[1], "NA", 7)
            name = CCD(rec[2], "NA", 30)
            col, all = getTrn(self.mf.dbm, "drs", dte=self.curdt,
                whr=[("drt_cono", "=", self.conum), ("drt_chain", "=",
                chn.work), ("drt_acno", "=", acno.work)], zer="N")
            if not all:
                continue
            if not self.pgnum:
                self.pageHeading(chn.disp, acno.disp, name.disp)
            else:
                self.newAccount(chn.disp, acno.disp, name.disp)
            for dat in all:
                ref1 = CCD(dat[col.index("drt_ref1")], "Na", 9)
                ref2 = CCD(dat[col.index("drt_ref2")], "Na", 9)
                trtp = CCD(dat[col.index("drt_type")], "UI", 2)
                trdt = CCD(dat[col.index("drt_trdt")], "d1", 10)
                if dat[col.index("drt_desc")] == name.work:
                    desc = CCD("", "NA", 30)
                else:
                    desc = CCD(dat[col.index("drt_desc")], "NA", 30)
                tramt = CCD(dat[col.index("drt_tramt")], "SD", 13.2)
                paid = CCD(dat[col.index("paid")], "SD", 13.2)
                trbal = CCD(dat[col.index("balance")], "SD", 13.2)
                if self.pglin > self.lpp:
                    if self.pgnum:
                        self.c.drawText(self.t)
                        self.c.showPage()
                    self.pageHeading(chn.disp, acno.disp, name.disp)
                self.t.textLine("%s %s %s %s %s %s %s %s" % (ref1.disp,
                    ref2.disp, drtrtp[trtp.work-1][0], trdt.disp,
                    desc.disp, tramt.disp, paid.disp, trbal.disp))
                self.pglin = self.pglin + 1
                self.ctots[0] = self.ctots[0] + 1
                self.ctots[1] = float(ASD(self.ctots[1]) + ASD(tramt.work))
                self.ctots[2] = float(ASD(self.ctots[2]) + ASD(paid.work))
                self.ctots[3] = float(ASD(self.ctots[3]) + ASD(trbal.work))
                self.gtots[0] = self.gtots[0] + 1
                self.gtots[1] = float(ASD(self.gtots[1]) + ASD(tramt.work))
                self.gtots[2] = float(ASD(self.gtots[2]) + ASD(paid.work))
                self.gtots[3] = float(ASD(self.gtots[3]) + ASD(trbal.work))
            if self.t:
                self.accountTotal()
            if  self.npag == "Y":
                self.pglin = 99
        p.closeProgress()
        if self.t:
            if can != "yes":
                self.grandTotal()
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
            head=self.tit, view=self.df.repprt[0][1],
            prtnam=self.df.repprt[0][2], mail=self.df.repprt[1][2])
            
    def pageHeading(self, chn, acno, name):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-40s %-7s %52s %5s" % \
            ("Debtors Outstanding Transactions as at", self.curdd,
            "Page", self.pgnum))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 3
        self.newAccount(chn, acno, name)

    def newAccount(self, chn, acno, name):
        if self.pglin > (self.lpp - 10):
            self.c.drawText(self.t)
            self.c.showPage()
            self.pageHeading(chn, acno, name)
        else:
            self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
            self.t.textLine("%s" % (self.suc * len(self.head)))
            self.t.textLine("%-7s %-3s %-7s %-30s" % \
                ("Account", chn, acno, name))
            self.t.textLine()
            self.t.textLine("%-9s %-9s %-3s %-10s %-30s %-13s %-13s %-13s" % \
                ("Reference", "Ref-Num-2", "Typ", "   Date", "Details",
                "      Amount", "        Paid", "     Balance"))
            self.t.textLine("%s" % (self.suc * len(self.head)))
            self.t.setFont("Tartan", self.cpi, self.ldg)
            self.pglin = self.pglin + 5

    def accountTotal(self):
        j = CCD(self.ctots[0], "SI", 7)
        k = CCD(self.ctots[1], "SD", 13.2)
        l = CCD(self.ctots[2], "SD", 13.2)
        m = CCD(self.ctots[3], "SD", 13.2)
        self.t.textLine()
        self.t.textLine("%-57s %7s %13s %13s %13s" % \
            ("Account Totals", j.disp, k.disp, l.disp, m.disp))
        self.ctots = [0,0,0,0]
        self.pglin = self.pglin + 2
    
    def grandTotal(self):
        j = CCD(self.gtots[0], "SI", 7)
        k = CCD(self.gtots[1], "SD", 13.2)
        l = CCD(self.gtots[2], "SD", 13.2)
        m = CCD(self.gtots[3], "SD", 13.2)
        self.t.textLine()
        self.t.textLine("%-57s %7s %13s %13s %13s" % \
            ("Grand Totals", j.disp, k.disp, l.disp, m.disp))
        self.t.textLine()

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

