#! /usr/bin/env python

"""
SYNOPSIS
    Debtors Ledger Age Analysis.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, Balances, CCD, GetCtl, ProgressBar, Sql
from TartanClasses import TartanDialog
from tartanFunctions import doReportlab, doWriteExport, getModName, doPrinter
from tartanFunctions import showError

class dr3050:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["drsact", "drstyp", "drsmst", "drstrn"],
            prog=__name__)
        gc = GetCtl(self.mf)
        drsctl = gc.getCtl("drsctl", self.conum)
        if not drsctl:
            return "no"
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i) %6s" % (t[0],
            t[1], t[2], t[3], t[4], __name__)
        self.head = ("%03u %-30s %s" % (self.conum, self.conam, "%s"))
        self.colsh = ["Chn", "Acc-Num", "Name", "Cr-Balance", "Tot-Balance",
            "Current", "30-Days", "60-Days", "90-Days", "Over-90-Days"]
        self.forms = [("UI", 3), ("NA", 7), ("NA", 30)] + [("SD", 13.2)] * 7
        self.gtots = [0] * 7
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Debtors Age Analysis (%s)" % __name__)
        dra = {
            "stype": "R",
            "tables": ("drsact",),
            "cols": (
                ("dac_code", "", 0, "Cod"),
                ("dac_desc", "", 0, "Description", "Y"))}
        drt = {
            "stype": "R",
            "tables": ("drstyp",),
            "cols": (
                ("dtp_code", "", 0, "Cod"),
                ("dtp_desc", "", 0, "Description", "Y"))}
        r1s = (("Yes","Y"),("No","N"))
        fld = (
            (("T",0,0,0),"ID2",7,"Cut Off Period","Cut Off Period",
                (self.sysdtw/100),"Y",self.doCutOff,None,None,None),
            (("T",0,1,0),("IRB",r1s),0,"Totals Only","Totals Only",
                "N","Y",self.doTots,None,None,None),
            (("T",0,2,0),"IUA",3,"Business Activity","Business Activity",
                "","Y",self.doDrsAct,dra,None,None),
            (("T",0,3,0),"IUA",3,"Business Type","Business Type",
                "","Y",self.doDrsTyp,drt,None,None),
            (("T",0,4,0),"IUI",7,"Lower Balance Limit","Lower Balance Limit",
                0,"Y",self.doLimit,None,None,None),
            (("T",0,5,0),("IRB",r1s),0,"Include Credit Balances",
                "Include Credit Balances","Y","Y",self.doCredit,None,None,None),
            (("T",0,6,0),("IRB",r1s),0,"Credit Balances Only",
                "Credit Balances Only","N","Y",self.doOnlyCr,None,None,None),
            (("T",0,7,0),("IRB",r1s),0,"Ignore Zero Balances",
                "Ignore Zero Balances","Y","Y",self.doZero,None,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("Y","V"), mail=("Y","N"), save="N")

    def doCutOff(self, frt, pag, r, c, p, i, w):
        self.cutoff = w
        self.period = self.df.t_disp[pag][r][p]

    def doTots(self, frt, pag, r, c, p, i, w):
        self.totsonly = w
        if self.totsonly == "Y":
            self.df.topEntry[0][8][2][0].hide()
            self.df.topEntry[0][8][3][0].hide()
            self.drsact = "All"
            self.df.loadEntry(frt, pag, p+1, data=self.drsact)
            self.drstyp = "All"
            self.df.loadEntry(frt, pag, p+2, data=self.drstyp)
            self.limit = 0
            self.df.loadEntry(frt, pag, p+3, data=self.limit)
            self.credit = "Y"
            self.df.loadEntry(frt, pag, p+4, data=self.credit)
            self.onlycr = "N"
            self.df.loadEntry(frt, pag, p+5, data=self.onlycr)
            self.zero = "Y"
            self.df.loadEntry(frt, pag, p+6, data=self.zero)
            return "sk6"
        else:
            self.df.topEntry[0][8][2][0].show()
            self.df.topEntry[0][8][3][0].show()

    def doDrsAct(self, frt, pag, r, c, p, i, w):
        if w:
            acc = self.sql.getRec(tables="drsact", where=[("dac_code", "=", w)],
                limit=1)
            if not acc:
                return "Invalid Account Activity"
            self.drsact = w
        else:
            self.drsact = "All"

    def doDrsTyp(self, frt, pag, r, c, p, i, w):
        if w:
            acc = self.sql.getRec(tables="drstyp", where=[("dtp_code", "=", w)],
                limit=1)
            if not acc:
                return "Invalid Account Activity"
            self.drstyp = w
        else:
            self.drstyp = "All"

    def doLimit(self, frt, pag, r, c, p, i, w):
        self.limit = w
        if self.limit:
            self.credit = "N"
            self.df.loadEntry(frt, pag, p+1, data=self.limit)
            self.onlycr = "N"
            self.df.loadEntry(frt, pag, p+2, data=self.onlycr)
            self.zero = "Y"
            self.df.loadEntry(frt, pag, p+3, data=self.zero)
            return "sk3"

    def doCredit(self, frt, pag, r, c, p, i, w):
        self.credit = w
        if self.credit == "N":
            self.onlycr = "N"
            self.df.loadEntry(frt, pag, p+1, data=self.limit)
            return "sk1"

    def doOnlyCr(self, frt, pag, r, c, p, i, w):
        self.onlycr = w
        if self.onlycr == "Y":
            self.zero = "Y"
            self.df.loadEntry(frt, pag, p+1, data=self.zero)
            return "sk1"

    def doZero(self, frt, pag, r, c, p, i, w):
        self.zero = w

    def doEnd(self):
        self.df.closeProcess()
        whr = [("drm_cono", "=", self.conum)]
        if self.drsact != "All":
            whr.append(("drm_bus_activity", "=", self.drsact))
        if self.drstyp != "All":
            whr.append(("drm_bus_type", "=", self.drstyp))
        ord = "drm_chain, drm_acno"
        all = self.sql.getRec(tables="drsmst", cols=["drm_chain", "drm_acno",
            "drm_name"], where=whr, order=ord)
        if not all:
            showError(self.mf.body, "Selection Error",
                "No Records Selected")
        elif self.df.repprt[0][2] == "export":
            self.exportReport(all)
        else:
            self.printReport(all)
        self.closeProcess()

    def exportReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        expnam = getModName(self.mf.wrkdir, __name__, self.conum)
        self.expheads = [self.head % self.sysdttm]
        self.expheads.append("Debtor's Age Analaysis up to %s" % self.period)
        self.expheads.append("Options: Activity %s Type %s Low-Bal %s "\
            "Cr-Bal-Only %s Ignore-Zeros %s" % (self.drsact, self.drstyp,
            self.limit, self.onlycr, self.zero))
        self.expcolsh = [self.colsh]
        self.expforms = self.forms
        self.expdatas = []
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                p.closeProgress()
                return
            vals = self.getValues(dat)
            if not vals:
                continue
            line = ["BODY", [vals[0].work, vals[1].work, vals[2].work]]
            for val in vals[3:]:
                line[1].append(val.work)
            self.expdatas.append(line)
        p.closeProgress()
        self.grandTotal()
        doWriteExport(xtype=self.df.repprt[0][1], name=expnam, sheet=__name__,
            heads=self.expheads, colsh=self.expcolsh, forms=self.expforms,
            datas=self.expdatas, rcdic=self.mf.rcdic)

    def printReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        chrs = len(self.colsh)
        for f in self.forms:
            chrs += int(f[1])
        if f[0][0].lower() == "s":
            chrs -= 2
        else:
            chrs -= 1
        pad = chrs - 35 - len(self.sysdttm)
        self.head1 = self.head % (" " * pad + self.sysdttm)
        self.head2 = "Debtor's Age Analysis up to %s%s" % (self.period,"%s%s")
        pad = chrs - len(self.head2) + 4 - 11 # %s%s and ' Page     1'
        self.head2 = self.head2 % (" " * pad, " Page %5s")
        if self.df.repprt[2][1] == "Y":
            dir = self.mf.arcdir
        else:
            dir = self.mf.wrkdir
        pdfnam = getModName(dir, __name__, self.conum, ext="pdf")
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, pdfnam, self.head1)
        f = CCD(all[0][0], "UI", 3)
        self.pgnum = 0
        self.pglin = 99
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            vals = self.getValues(dat)
            if not vals:
                continue
            if self.totsonly != "Y":
                if self.pglin > self.lpp:
                    if self.pgnum:
                        self.c.drawText(self.t)
                        self.c.showPage()
                    self.pageHeading()
                self.t.textLine("%s %s %s %s %s %s %s %s %s %s" % (vals[0].disp,
                    vals[1].disp, vals[2].disp, vals[3].disp, vals[4].disp,
                    vals[5].disp, vals[6].disp, vals[7].disp, vals[8].disp,
                    vals[9].disp))
                self.pglin = self.pglin + 1
        p.closeProgress()
        if self.totsonly == "Y" and can != "yes":
            self.pageHeading()
            self.grandTotal()
            self.c.drawText(self.t)
        elif self.t:
            if can != "yes":
                self.grandTotal()
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=pdfnam, head=self.tit,
            view=self.df.repprt[0][1], prtnam=self.df.repprt[0][2],
            mail=self.df.repprt[1][2])

    def getValues(self, data):
        chn = CCD(data[0], "UI", 3)
        acc = CCD(data[1], "NA", 7)
        name = CCD(data[2], "NA", 30)
        bals = Balances(self.mf, "DRS", self.conum, self.cutoff,
            keys=(data[0], data[1]))
        obal, cbal, ages = bals.doAllBals()
        bal = CCD(cbal, "SD", 13.2)
        if self.limit and bal.work < self.limit:
            return
        if self.zero == "Y" and not bal.work:
            return
        if self.credit == "N" and bal.work < 0:
            return
        if self.onlycr == "Y" and not bal.work < 0:
            return
        cur = CCD(ages[0], "SD", 13.2)
        d30 = CCD(ages[1], "SD", 13.2)
        d60 = CCD(ages[2], "SD", 13.2)
        d90 = CCD(ages[3], "SD", 13.2)
        d120 = CCD(ages[4], "SD", 13.2)
        if bal.work < 0:
            deb = CCD(bal.work, "SD", 13.2)
        else:
            deb = CCD(0, "SD", 13.2)
        self.gtots[0] = float(ASD(self.gtots[0]) + ASD(deb.work))
        self.gtots[1] = float(ASD(self.gtots[1]) + ASD(bal.work))
        self.gtots[2] = float(ASD(self.gtots[2]) + ASD(cur.work))
        self.gtots[3] = float(ASD(self.gtots[3]) + ASD(d30.work))
        self.gtots[4] = float(ASD(self.gtots[4]) + ASD(d60.work))
        self.gtots[5] = float(ASD(self.gtots[5]) + ASD(d90.work))
        self.gtots[6] = float(ASD(self.gtots[6]) + ASD(d120.work))
        return (chn, acc, name, deb, bal, cur, d30, d60, d90, d120)
            
    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head1)
        self.t.textLine()
        self.t.textLine(self.head2 % self.pgnum)
        self.t.textLine()
        self.t.textLine("(%s%-3s%3s%6s%-3s%3s%9s%-8s%3s%13s%1s%3s%17s%s)" % \
            ("Options: Activity: ", self.drsact, "", "Type: ", self.drstyp, "",
            "Low-Bal: ", self.limit, "", "Cr-Bal-Only: ", self.onlycr, "",
            "Ignore-Zero-Bal: ", self.zero))
        self.t.textLine()
        self.t.textLine("%-3s %-7s %-30s %-13s %-13s %-13s %-13s %-13s %-13s "\
            "%-13s" % ("Chn", "Acc-Num", "Name", "  Cr-Balance", " Tot-Balance",
            "     Current", "     30-Days", "     60-Days", "     90-Days",
            "Over-90-Days"))
        self.t.textLine("%s" % (self.suc * len(self.head1)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 8

    def grandTotal(self):
        if self.df.repprt[0][2] == "export":
            self.expdatas.append(["ULINES"])
            self.expdatas.append(["TOTAL", ["","","Grand Totals"] + self.gtots])
            self.expdatas.append(["ULINED"])
            return
        for x in range(0, 7):
            exec "t%s = CCD(self.gtots[x], 'SD', 13.2)" % x
            if self.gtots[1] == 0:
                exec "p%s = CCD(0, 'SD', 13.2)" % x
            else:
                exec "p%s = CCD(round((self.gtots[x]/self.gtots[1]*100),2), "\
                "'OSD', 13.2)" % x
        self.t.textLine()
        self.t.textLine("%-11s %-30s %13s %13s %13s %13s %13s %13s %13s" % \
            (" ", "Grand Totals", t0.disp, t1.disp, t2.disp, t3.disp,
            t4.disp, t5.disp, t6.disp))
        self.t.textLine()
        self.t.textLine("%-11s %-30s %13s %13s %13s %13s %13s %13s %13s" % \
            (" ", "Percentages", p0.disp, p1.disp, p2.disp, p3.disp,
            p4.disp, p5.disp, p6.disp))

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

