#! /usr/bin/env python

"""
SYNOPSIS
    Debtors Ledger Statements.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import os, time
from PIL import Image
from TartanClasses import ASD, Balances, CCD, GetCtl, ProgressBar, Sql
from TartanClasses import TartanDialog
from tartanFunctions import doPrinter, doReportlab, flipImage, getModName
from tartanFunctions import getSingleRecords, showError, textFormat
from tartanWork import drtrtp

class dr3080:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        gc = GetCtl(self.mf)
        drsctl = gc.getCtl("drsctl", self.conum)
        if not drsctl:
            return "no"
        self.sql = Sql(self.mf.dbm, ["ctlmes", "ctlmst", "drschn", "drsmst",
            "ctlmes"], prog=__name__)
        self.chains = drsctl["ctd_chain"]
        self.sttype = drsctl["ctd_stype"]
        self.sinv = drsctl["ctd_sinv"]
        self.sage = drsctl["ctd_sage"]
        if not self.sttype:
            self.sttype = "N"
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.t = None
        if self.chains == "N":
            self.schn = 0
            self.echn = 0
        self.trtypes = []
        for q in range(0, len(drtrtp)):
            self.trtypes.append(drtrtp[q][0])
        self.trtypes.append("Dsc")

    def buildHead(self):
        ctl = self.sql.getRec(tables="ctlmst", where=[("ctm_cono", "=",
            self.conum)], limit=1)
        for col in ("ctm_name", "ctm_add1", "ctm_add2", "ctm_add3",
            "ctm_pcode", "ctm_regno", "ctm_taxno", "ctm_tel", "ctm_fax",
            "ctm_b_name", "ctm_b_ibt", "ctm_b_acno", "ctm_logo", "ctm_flip"):
            exec "self.%s = ctl[self.sql.ctlmst_col.index('%s')]" % (col, col)
        if not self.ctm_logo or not os.path.exists(self.ctm_logo):
            self.ctm_logo = None
        elif self.ctm_flip == "N":
            self.ctm_logo = flipImage(self.ctm_logo)
        self.head = []
        if self.sttype == "N":
            if self.ctm_logo:
                mess = "%64s" % ""
                if self.remits == "Y":
                    mess = mess + "%-17s%-30s" % ("", self.ctm_add1)
                self.head.append(mess)
                mess = "%64s" % ""
                if self.remits == "Y":
                    mess = mess + "%-17s%-30s" % ("", self.ctm_add2)
                self.head.append(mess)
                mess = "%64s" % ""
                if self.remits == "Y":
                    mess = mess + "%-17s%-30s" % ("", self.ctm_add3)
                self.head.append(mess)
                mess = "%64s" % ""
                if self.remits == "Y":
                    mess = mess + "%-17s%-30s" % ("", self.ctm_pcode)
                self.head.append(mess)
            else:
                mess = "%2s%-30s%2s%-30s" % ("", self.ctm_add1, "",
                    ("RegNo: " + self.ctm_regno))
                if self.remits == "Y":
                    mess = mess + "%-17s%-30s" % ("", self.ctm_add1)
                self.head.append(mess)
                mess = "%2s%-30s%2s%-30s" % ("", self.ctm_add2, "",
                    ("TaxNo: " + self.ctm_taxno))
                if self.remits == "Y":
                    mess = mess + "%-17s%-30s" % ("", self.ctm_add2)
                self.head.append(mess)
                mess = "%2s%-30s%2s%-30s" % ("", self.ctm_add3, "",
                    ("TelNo: " + self.ctm_tel))
                if self.remits == "Y":
                    mess = mess + "%-17s%-30s" % ("", self.ctm_add3)
                self.head.append(mess)
                mess = "%2s%-30s%2s%-30s" % ("", self.ctm_pcode, "",
                    ("FaxNo: " + self.ctm_fax))
                if self.remits == "Y":
                    mess = mess + "%-17s%-30s" % ("", self.ctm_pcode)
                self.head.append(mess)
        elif self.ctm_logo:
            self.head.append(" ")
            self.head.append(" ")
            self.head.append(" ")
            self.head.append(" ")
        else:
            mess = "%2s%-30s%2s%-30s" % \
                ("", self.ctm_add1, "", ("RegNo: " + self.ctm_regno))
            self.head.append(mess)
            mess = "%2s%-30s%2s%-30s" % \
                ("", self.ctm_add2, "", ("TaxNo: " + self.ctm_taxno))
            self.head.append(mess)
            mess = "%2s%-30s%2s%-30s" % \
                ("", self.ctm_add3, "", ("TelNo: " + self.ctm_tel))
            self.head.append(mess)
            mess = "%2s%-30s%2s%-30s" % \
                ("", self.ctm_pcode, "", ("FaxNo: " + self.ctm_fax))
            self.head.append(mess)

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Debtors Statements (%s)" % __name__)
        drc = {
            "stype": "R",
            "tables": ("drschn",),
            "cols": (
                ("chm_chain", "", 0, "Num"),
                ("chm_name", "", 0, "Name", "Y")),
            "where": [("chm_cono", "=", self.conum)]}
        drm = {
            "stype": "R",
            "tables": ("drsmst",),
            "cols": (
                ("drm_acno", "", 0, "Acc-Num"),
                ("drm_name", "", 0, "Name", "Y"),
                ("drm_add1", "", 0, "Address Line 1"))}
        if self.chains == "Y":
            drm["where"] = [("drm_cono", "=", self.conum)]
            drm["whera"] = [["C", "drm_chain", 0, 1]]
        else:
            drm["where"] = [
                ("drm_cono", "=", self.conum),
                ("drm_chain", "=", 0)]
        mss = {
            "stype": "R",
            "tables": ("ctlmes",),
            "cols": (
                ("mss_message", "", 0, "Num"),
                ("mss_detail", "NA", 50, "Details")),
            "where": [("mss_system", "=", "STA")]}
        r1s = (("Normal","N"),("Other","O"))
        r2s = (("Yes","Y"),("Range","R"),("Singles", "S"))
        r3s = (("Number","N"),("Name","M"),("Postal Code","P"))
        r4s = (("Yes","Y"),("No","N"))
        fld = [
            [["T",0,0,0],("IRB",r1s),0,"Statement Type","",
                self.sttype,"Y",self.doType,None,None,None],
            [["T",0,1,0],"IUI",2,"Maximum Pages","",
                1,"Y",self.doPages,None,None,None],
            [["T",0,2,0],("IRB",r2s),0,"Whole File","",
                "S","Y",self.doWhole,None,None,None],
            [["T",0,3,0],"IUI",3,"From Chain","",
                "","Y",self.doChn,drc,None,None],
            [["T",0,4,0],"INA",7,"From Account","",
                "","Y",self.doAcc,drm,None,None],
            [["T",0,5,0],"INA",3,"To Chain","",
                "","Y",self.doChn,drc,None,None],
            [["T",0,6,0],"INA",7,"To Account","",
                "","Y",self.doAcc,drm,None,None],
            [["T",0,7,0],("IRB",r3s),0,"Sort Order","",
                "N","Y",self.doSort,None,None,None],
            [["T",0,8,0],("IRB",r4s),0,"Print Remittances","",
                "N","Y",self.doRemits,None,None,None],
            [["T",0,9,0],("IRB",r4s),0,"Include Zero Balances","",
                "N","Y",self.doZeros,None,None,None],
            [["T",0,10,0],("IRB",r4s),0,"Include Negative Balances","",
                "N","Y",self.doMinus,None,None,None],
            [["T",0,11,0],("IRB",r4s),0,"Include Stopped Accounts","",
                "N","Y",self.doStops,None,None, None],
            [["T",0,12,0],"IUI",3,"Message Number","",
                "","Y",self.doMessno,mss,None,("efld",)],
            [["T",0,13,0],"ID1",10,"Statement Date","",
                self.sysdtw,"Y",self.doDat,None,None,("efld",)]]
        if self.chains != "Y":
            del fld[5]
            del fld[3]
            for n, f in enumerate(fld):
                fld[n][0][2] = n
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("N","Y"), save="N")

    def doType(self, frt, pag, r, c, p, i, w):
        self.sttype = w
        if self.sttype == "O":
            self.maxlines = 25
            self.buildHead()
        else:
            self.pages = 0
            self.df.loadEntry(frt, pag, p+1, data=self.pages)
            return "sk1"

    def doPages(self, frt, pag, r, c, p, i, w):
        self.pages = w

    def doWhole(self, frt, pag, r, c, p, i, w):
        self.whole = w
        if self.whole in ("Y", "S"):
            self.schn = 0
            self.echn = 0
            self.sacc = ""
            self.eacc = ""
            if self.chains == "Y":
                self.df.loadEntry("T", 0, p+1, data=self.schn)
                self.df.loadEntry("T", 0, p+2, data=self.sacc)
                self.df.loadEntry("T", 0, p+3, data=self.echn)
                self.df.loadEntry("T", 0, p+4, data=self.eacc)
                return "sk4"
            else:
                self.df.loadEntry("T", 0, p+1, data=self.sacc)
                self.df.loadEntry("T", 0, p+2, data=self.eacc)
                return "sk2"

    def doChn(self, frt, pag, r, c, p, i, w):
        chk = self.sql.getRec(tables="drschn", cols=["chm_name"],
            where=[("chm_chain", "=", w)], limit=1)
        if not chk:
            return "Invalid Chain Store"
        if c == 4:
            self.schn = w
        else:
            self.echn = w

    def doAcc(self, frt, pag, r, c, p, i, w):
        if self.chains == "Y" and c == 5:
            chk = self.schn
            self.sacc = w
        elif self.chains == "Y" and c == 7:
            chk = self.echn
            self.eacc = w
        elif c == 4:
            chk = self.schn
            self.sacc = w
        else:
            chk = self.echn
            self.eacc = w
        chk = self.sql.getRec(tables="drsmst", cols=["drm_name"],
            where=[("drm_cono", "=", self.conum), ("drm_chain", "=", chk),
            ("drm_acno", "=", w)], limit=1)
        if not chk:
            return "Invalid Debtors Account"

    def doSort(self, frt, pag, r, c, p, i, w):
        self.sort = w
        if self.sttype == "O":
            self.remits = "N"
            self.df.loadEntry(frt, pag, p+1, data=self.remits)
            return "sk1"

    def doRemits(self, frt, pag, r, c, p, i, w):
        self.remits = w
        if self.remits == "Y":
            self.maxlines = 15
        else:
            self.maxlines = 35
        self.buildHead()

    def doZeros(self, frt, pag, r, c, p, i, w):
        self.zeros = w

    def doMinus(self, frt, pag, r, c, p, i, w):
        self.minus = w

    def doStops(self, frt, pag, r, c, p, i, w):
        self.stops = w

    def doMessno(self, frt, pag, r, c, p, i, w):
        if w:
            self.mes = self.sql.getRec(tables="ctlmes", cols=["mss_detail"],
                where=[("mss_system", "=", "STA"), ("mss_message", "=", w)],
                limit=1)
            if not self.mes:
                return "Invalid Message Number"
        else:
            self.mes = None

    def doDat(self, frt, pag, r, c, p, i, w):
        self.datew = w
        self.dated = self.df.t_disp[pag][0][p]
        self.curdt = w / 100

    def doEnd(self):
        self.df.closeProcess()
        self.prtnam = self.df.repprt[0][2]
        self.email = self.df.repprt[1][1]
        self.printm = self.df.repprt[1][3]
        self.archive = self.df.repprt[2][1]
        self.pdfnam = None
        if self.email == "N":
            self.key = "%s_all_all" % self.conum
            self.printSetup()
        if self.whole == "S":
            all = getSingleRecords(self.mf, "drsmst",
            ("drm_chain", "drm_acno", "drm_name"), where=[("drm_cono",
            "=", self.conum)])
        else:
            if self.echn == 0:
                self.echn = 999
            if not self.eacc:
                self.eacc = "zzzzzzz"
            whr = [("drm_cono", "=", self.conum), ("drm_chain", ">=",
                self.schn), ("drm_acno", ">=", self.sacc), ("drm_chain", "<=",
                self.echn), ("drm_acno", "<=", self.eacc)]
            if self.sort == "N":
                ord = "drm_chain, drm_acno"
            elif self.sort == "M":
                ord = "drm_name"
            else:
                ord = "drm_pcod"
            all = self.sql.getRec(tables="drsmst", where=whr, order=ord)
            if not all:
                showError(self.mf.body, "Error", "No Accounts Selected")
        if all:
            num = len(all)
            p = ProgressBar(self.mf.body, max=num, esc=True)
            for x in range(0, num):
                can = p.displayProgress(x)
                if can == "yes":
                    if self.t:
                        self.t.textLine()
                        self.t.textLine("Print Job ABORTED")
                    break
                self.doProcess(all[x])
            p.closeProgress()
            if self.email == "N":
                self.c.save()
                self.doPrint()
        self.mf.closeLoop()
         
    def doProcess(self, drm):
        self.pgnum = 0
        dmc = self.sql.drsmst_col
        self.chn = CCD(drm[dmc.index("drm_chain")], "UI", 3)
        self.acc = CCD(drm[dmc.index("drm_acno")], "NA", 7)
        if self.sttype == "N":
            bals = Balances(self.mf, "DRS", self.conum, self.curdt,
                (self.chn.work, self.acc.work))
            if self.sttype == "O":
                trans = "A"
            else:
                trans = "Y"
            self.obal,self.tbal,self.ages,self.trns=bals.doAllBals(trans=trans)
            self.dtc = self.trns[0]
            self.drt = self.trns[1]
            if not self.drt:
                return
            if self.zeros == "N" and not self.tbal:
                return
            if self.minus == "N" and self.tbal < 0:
                return
        else:
            self.tots = [0.0, 0.0, 0.0, 0.0]
            self.drt = self.sql.getRec(tables="drstrn", cols=["drt_trdt",
                "drt_ref1", "drt_type", "drt_desc", "drt_tramt", "drt_taxamt",
                "drt_curdt"], where=[("drt_cono", "=", self.conum),
                ("drt_chain", "=", self.chn.work), ("drt_acno", "=",
                self.acc.work), ("drt_trdt", "<=", self.datew)],
                order="drt_trdt, drt_ref1")
            if not self.drt:
                return
            cmth = False
            for t in self.drt:
                if t[6] == self.curdt:
                    cmth = True
                self.tots[3] = float(ASD(self.tots[3]) + ASD(t[4]))
                if t[2] not in (2, 6) and t[6] == self.curdt:
                    self.tots[1] = float(ASD(self.tots[1]) + ASD(t[4]))
                    if t[5]:
                        self.tots[1] = float(ASD(self.tots[1]) - ASD(t[5]))
                        self.tots[2] = float(ASD(self.tots[2]) + ASD(t[5]))
                else:
                    self.tots[0] = float(ASD(self.tots[0]) + ASD(t[4]))
            if self.zeros == "N" and not self.tots[3] and not cmth:
                return
            if self.minus == "N" and self.tots[3] < 0:
                return
        if self.email == "Y":
            self.key = "%s_%s_%s" % (self.conum, self.chn.work, self.acc.work)
            self.printSetup()
        self.name = CCD(drm[dmc.index("drm_name")], "NA", 30)
        self.add1 = CCD(drm[dmc.index("drm_add1")], "NA", 30)
        self.add2 = CCD(drm[dmc.index("drm_add2")], "NA", 30)
        self.add3 = CCD(drm[dmc.index("drm_add3")], "NA", 30)
        self.pcod = CCD(drm[dmc.index("drm_pcod")], "NA", 4)
        self.emadd = CCD(drm[dmc.index("drm_acc_email")], "TX")
        self.drmes = CCD(drm[dmc.index("drm_stames")], "UI", 3)
        self.printHeader()
        if self.sttype == "N":
            self.printNormalBody()
        else:
            self.printOtherBody()
        self.printTrailer()
        if self.email == "Y":
            self.c.save()
            self.doPrint()
         
    def printSetup(self):
        if self.email == "Y" or not self.pdfnam:
            if self.archive == "Y":
                dir = self.mf.arcdir
            else:
                dir = self.mf.wrkdir
            self.pdfnam = getModName(dir, __name__, self.key, ext="pdf")
        if self.remits == "Y":
            orient = "L"
        else:
            orient = "P"
        self.c, self.cpi, self.inch, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, orient=orient)
        # Width in pixels of each character
        self.cwidth = self.c.stringWidth(("*"), "Tartan", self.cpi)

    def doPrint(self):
        if self.prtnam == "view":
            view = "Y"
        else:
            view = "N"
        if self.email == "N" or not self.emadd.work:
            mail = "N"
        else:
            mail = self.emadd.work
        doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam, head=self.tit,
            view=view, prtnam=self.prtnam, mail=mail, printm=self.printm)

    def printHeader(self):
        self.drawDocument()
        self.pgnum = self.pgnum + 1
        if self.sttype == "N":
            if not self.ctm_logo:
                self.c.setFont("Tartan-Bold", 18)
                self.c.drawString(self.cwidth*2, 2*(self.inch/6),
                    self.ctm_name)
            if self.remits == "Y":
                self.c.setFont("Tartan-Bold", 15)
                self.c.drawString(self.cwidth*80.5, 2*(self.inch/6),
                    self.ctm_name)
        elif not self.ctm_logo:
            self.c.setFont("Tartan-Bold", 18)
            self.c.drawString(self.cwidth*2, 2*(self.inch/6), self.ctm_name)
        self.c.setFont("Tartan", 10)
        self.t.setTextOrigin(0, 45)
        self.t.textLine(self.head[0])
        self.t.textLine(self.head[1])
        self.t.textLine(self.head[2])
        self.t.textLine(self.head[3])
        self.t.setFont("Tartan", 10)
        self.t.textLine()
        mess = "%-74s%-3s" % ("", self.chn.disp)
        if self.remits == "Y":
            mess = mess + "%-44s%-3s" % ("", self.chn.disp)
        self.t.textLine(mess)
        mess = "%1s%-30s" % ("", self.name.disp)
        if self.remits == "Y":
            mess = mess + "%-50s%-30s" % ("", self.name.disp[0:28])
        self.t.textLine(mess)
        mess = "%1s%-30s%39s%7s" % ("", self.add1.disp, "", self.acc.disp)
        if self.remits == "Y":
            mess = mess + "%4s%-30s%6s%7s" % ("", self.add1.disp[0:28], "",
            self.acc.disp)
        self.t.textLine(mess)
        mess = "%1s%-30s" % ("", self.add2.disp)
        if self.remits == "Y":
            mess = mess + "%-50s%-30s" % ("", self.add2.disp[0:28])
        self.t.textLine(mess)
        mess = "%1s%-30s%36s%10s" % ("", self.add3.disp, "", self.pgnum)
        if self.remits == "Y":
            mess = mess + "%-4s%-30s%3s%10s" % ("", self.add3.disp[0:28],
                "", self.pgnum)
        self.t.textLine(mess)
        mess = "%1s%-30s" % ("", self.pcod.disp)
        if self.remits == "Y":
            mess = mess + "%50s%-30s" % ("", self.pcod.disp)
        self.t.textLine(mess)
        mess = "%67s%-10s" % ("", self.dated)
        if self.remits == "Y":
            mess = mess + "%37s%-10s" % ("", self.dated)
        self.t.textLine(mess)
        self.t.textLine()
        self.t.textLine()
        self.t.setFont("Tartan", 10)
        self.pglin = 0

    def printNormalBody(self):
        for drt in self.drt:
            trtp = CCD(drt[self.dtc.index("drt_type")], "UI", 1)
            ref1 = CCD(drt[self.dtc.index("drt_ref1")], "Na", 9)
            ref2 = CCD(drt[self.dtc.index("drt_ref2")], "Na", 9)
            trdt = CCD(drt[self.dtc.index("drt_trdt")], "d1", 10)
            tramt = CCD(drt[self.dtc.index("drt_tramt")], "SD", 13.2)
            pay = CCD(drt[self.dtc.index("paid")], "SD", 13.2)
            bal = CCD(drt[self.dtc.index("balance")], "SD", 13.2)
            if self.pglin > (self.maxlines - 1):
                self.printTrailer(fwd="Y")
                self.printHeader()
            if self.remits == "Y" and bal.work:
                mess = "%2s%s %s %s %s %s %s %s   %s %s       %s %s" % \
                    ("", trdt.disp, ref1.disp, ref2.disp,
                    self.trtypes[trtp.work - 1], tramt.disp, pay.disp,
                    bal.disp, trdt.disp, ref1.disp,
                    self.trtypes[trtp.work -1], bal.disp)
            else:
                mess = "%2s%s %s %s %s %s %s %s" % \
                    ("", trdt.disp, ref1.disp, ref2.disp,
                    self.trtypes[trtp.work - 1], tramt.disp, pay.disp,
                    bal.disp)
            self.t.textLine(mess)
            self.pglin = self.pglin + 1

    def printOtherBody(self):
        obal = 0.0
        if self.pages and len(self.drt) > ((self.pages * self.maxlines) - 1):
            chk = len(self.drt) - ((self.pages * self.maxlines) - 1)
        else:
            chk = 0
        for num, drt in enumerate(self.drt):
            trdt = CCD(drt[0], "d1", 10)
            ref1 = CCD(drt[1], "Na", 9)
            trtp = CCD(drt[2], "UI", 1)
            desc = CCD(drt[3], "NA", 39)
            tramt = CCD(drt[4], "SD", 11.2)
            if self.pglin > (self.maxlines - 1):
                self.printTrailer(fwd="Y")
                self.printHeader()
            if chk:
                if num < chk:
                    obal = float(ASD(obal) + ASD(tramt.work))
                    continue
                elif num == chk:
                    obal = CCD(obal, "SD", 11.2)
                    mess = "%2s%s %s %-3s %-39s %s" % ("", trdt.disp,
                        "Bal/B-Fwd", "B/F", "BALANCE BROUGHT FORWARD",
                        obal.disp)
                    self.t.textLine(mess)
                    self.pglin = self.pglin + 1
            mess = "%2s%s %s %s %s %s" % ("", trdt.disp, ref1.disp,
                self.trtypes[trtp.work - 1], desc.disp, tramt.disp)
            self.t.textLine(mess)
            self.pglin = self.pglin + 1

    def printTrailer(self, fwd="N"):
        if self.sttype == "N":
            tots = [CCD(self.tbal, "SD", 11.2).disp]
            for x in range(0, 5):
                if self.sage == "N":
                    y = CCD("", "NA", 11)
                else:
                    y = CCD(self.ages[x], "SD", 11.2)
                tots.append(y.disp)
            if fwd == "Y":
                mess = "%-66s%-11s" % ("", "Continued..")
                if self.remits == "Y":
                    mess = mess + "%-36s%-11s" % ("", "Continued..")
            else:
                mess = "%2s%s %s  %s  %s  %s   %s" % \
                    ("", tots[5], tots[4], tots[3], tots[2], tots[1], tots[0])
                if self.remits == "Y":
                    mess = mess + "%-35s %s" % ("", tots[0])
            for z in range(0, (self.maxlines + 2 - self.pglin)):
                self.t.textLine()
            self.t.textLine(mess)
            self.t.textLine()
            self.t.textLine()
            if fwd == "Y":
                mes2 = None
            elif self.mes:
                mes2 = self.mes
            elif self.drmes.work:
                mes2 = self.sql.getRec(tables="ctlmes",
                    cols=["mss_detail"], where=[("mss_system", "=", "STA"),
                    ("mss_message", "=", self.drmes.work)], limit=1)
            else:
                mes2 = None
            if mes2:
                # Draw Message Rectangles
                self.c.setFillGray(.8)
                x = self.cwidth*.5
                w = self.cwidth*74
                if self.remits == "Y":
                    b1 = 36.50
                    b2 = 37.30
                else:
                    b1 = 56.50
                    b2 = 57.30
                self.c.rect(x,(b1*(self.inch/6)),w,(1.25*(self.inch/6)),fill=1)
                self.c.rect(x,(b1*(self.inch/6)),w,(4.75*(self.inch/6)),fill=0)
                self.c.setFillGray(0)
                self.c.setFont("Tartan-Bold", 10)
                self.c.drawString(self.cwidth*1,(b2*(self.inch/6)),"Message")
                self.c.setFont("Tartan", 10)
                self.t.textLine()
                mes3 = textFormat(mes2[0], width=75)
                for mess in mes3:
                    self.t.textLine(" " + mess)
        else: 
            tots = [""]
            for s, t in enumerate(self.tots):
                if self.sage == "N" and s < 3:
                    y = CCD("", "NA", 11)
                else:
                    y = CCD(t, "SD", 11.2)
                tots.append(y.disp)
            if fwd == "Y":
                mess = "%-66s%-11s" % ("", "Continued..")
            else:
                mess = "%18s %s     %s     %s     %s" % tuple(tots)
            for z in range(0, (self.maxlines + 2 - self.pglin)):
                self.t.textLine()
            self.t.textLine(mess)
            # Draw Banking Details Block
            self.c.setFillGray(.8)
            x = self.cwidth*.5
            w = self.cwidth*78
            b1 = self.maxlines + 21.50
            b2 = self.maxlines + 22.40
            b3 = self.maxlines + 24
            self.c.rect(x, (b1*(self.inch/6)), w, (1.25*(self.inch/6)), fill=1)
            self.c.rect(x, (b1*(self.inch/6)), w, (3.25*(self.inch/6)), fill=0)
            self.c.setFillGray(0)
            self.c.setFont("Tartan-Bold", 10)
            self.c.drawString(self.cwidth*1, (b2*(self.inch/6)),
                "Banking Details")
            self.c.setFont("Tartan", 10)
            self.c.setFont("Tartan-Bold", 10)
            self.c.drawString((self.cwidth*1), (b3*(self.inch/6)),
             "Bank:                                 Branch:           Account:")
            self.c.setFont("Tartan", 10)
            self.c.drawString((self.cwidth*1), (b3*(self.inch/6)),
                "      %-30s          %-8s           %-16s" % (self.ctm_b_name,
                self.ctm_b_ibt, self.ctm_b_acno))
            # Check for Message
            if self.mes:
                mes2 = self.mes[:]
            else:
                mes2 = None
            if mes2:
                # Draw Message Rectangles
                self.c.setFillGray(.8)
                x = self.cwidth*.5
                w = self.cwidth*78
                b1 = self.maxlines + 25.5
                b2 = self.maxlines + 26.3
                self.c.rect(x,(b1*(self.inch/6)),w,(1.25*(self.inch/6)),fill=1)
                self.c.rect(x,(b1*(self.inch/6)),w,(4.75*(self.inch/6)),fill=0)
                self.c.setFillGray(0)
                self.c.setFont("Tartan-Bold", 10)
                self.c.drawString(self.cwidth*1,(b2*(self.inch/6)),"Message")
                self.c.setFont("Tartan", 10)
                for x in range(7):
                    self.t.textLine()
                mes3 = textFormat(mes2[0], width=75)
                for mess in mes3[:3]:
                    self.t.textLine(" " + mess)
        self.c.drawText(self.t)
        self.c.showPage()

    def drawDocument(self):
        self.c.translate(self.inch, self.inch) # Set x and y margins
        self.t = self.c.beginText()
        self.t.setFont("Tartan-Bold", self.cpi)
        self.t.setTextOrigin(0, 0)
        # If logo, draw logo
        if self.ctm_logo:
            self.c.drawImage(self.ctm_logo, 0, 3, 350, 70, None)
        # Draw Title
        self.c.setFont("Tartan-Bold", 18)
        self.c.drawString(self.cwidth*62, 2*(self.inch/6), "STATEMENT")
        if self.sttype == "O" and self.sinv == "Y":
            self.c.drawString(self.cwidth*62, 3*(self.inch/6), "& INVOICE")
        self.c.setFont("Tartan-Bold", self.cpi)
        # Draw Name and Address Rectangle
        self.c.setFillGray(.8)
        x = self.cwidth*.5
        w = self.cwidth*32.5
        self.c.rect(x, (7.75*(self.inch/6)), w, (1*(self.inch/6)), fill=1)
        self.c.rect(x, (7.75*(self.inch/6)), w, (7.25*(self.inch/6)), fill=0)
        self.c.setFillGray(0)
        self.c.setFont("Tartan-Bold", 8)
        self.c.drawString(self.cwidth*1, (8.45*(self.inch/6)), "Account")
        self.c.setFont("Tartan", 10)
        if self.remits == "Y":
            self.c.setFillGray(.8)
            x = self.cwidth*80.5
            w = self.cwidth*32.5
            self.c.rect(x, (7.75*(self.inch/6)), w, (1*(self.inch/6)), fill=1)
            self.c.rect(x, (7.75*(self.inch/6)), w, (7.25*(self.inch/6)),fill=0)
            self.c.setFillGray(0)
            self.c.setFont("Tartan-Bold", 8)
            self.c.drawString(self.cwidth*90, (8.45*(self.inch/6)), "Account")
            self.c.setFont("Tartan", 10)
        # Draw Chain, Acc-Num, Page and Date Rectangles
        self.c.setFillGray(.8)
        x = self.cwidth*64.5
        w = self.cwidth*14
        nums = ((7, "Chain"), (9, "Acc-Num"), (11, "Page"), (13, "Date"))
        for q in nums:
            self.c.rect(x, (q[0]*(self.inch/6)), w, (1*(self.inch/6)), fill=1)
            self.c.rect(x, (q[0]*(self.inch/6)), w, (2*(self.inch/6)), fill=0)
        self.c.setFillGray(0)
        self.c.setFont("Tartan-Bold", 8)
        for q in nums:
            self.c.drawCentredString((self.cwidth*71.5),
                ((q[0]+.80)*(self.inch/6)), q[1])
        self.c.setFont("Tartan", 10)
        if self.remits == "Y":
            self.c.setFillGray(.8)
            x = self.cwidth*113.5
            w = self.cwidth*12
            for q in (7, 9, 11, 13):
                self.c.rect(x, (q*(self.inch/6)), w, (1*(self.inch/6)), fill=1)
                self.c.rect(x, (q*(self.inch/6)), w, (2*(self.inch/6)), fill=0)
            self.c.setFillGray(0)
            self.c.setFont("Tartan-Bold", 8)
            self.c.drawCentredString((self.cwidth*120),
                (7.80*(self.inch/6)), "Chain")
            self.c.drawCentredString((self.cwidth*120),
                (9.80*(self.inch/6)), "Acc-Num")
            self.c.drawCentredString((self.cwidth*120),
                (11.80*(self.inch/6)), "Page")
            self.c.drawCentredString((self.cwidth*120),
                (13.80*(self.inch/6)), "Date")
            self.c.setFont("Tartan", 10)
        # Draw Ribbon Rectangles
        self.c.setFillGray(.8)
        x = self.cwidth*.5
        w = self.cwidth*78
        if self.sttype == "N":
            if self.remits == "Y":
                b1 = 16.75
                b2 = 32.25
            else:
                b1 = 36.75
                b2 = 52.25
        else:
            b1 = self.maxlines + 1.75
            b2 = self.maxlines + 17.25
        b3 = 15.50
        self.c.rect(x, (b3*(self.inch/6)), w, (1.25*(self.inch/6)), fill=1)
        self.c.rect(x, (b3*(self.inch/6)), w, (b1*(self.inch/6)), fill=0)
        self.c.setFillGray(0)
        self.c.setFont("Tartan-Bold", 10)
        if self.sttype == "N":
            for q, r in ((2, "   Date"), (13, "Reference"), (23, "Ref-Num-2"),
                (33, "Typ"), (37, " Orig-Amount"), (51, " Paid-Amount"),
                (65, "  Bal-Amount")):
                self.c.drawString(self.cwidth*q, (16.40*(self.inch/6)), r)
            self.c.setFont("Tartan", 10)
            for q in (12.5, 22.5, 32.5, 36.5, 50.5, 64.5):
                self.c.line(self.cwidth*q, (b3*(self.inch/6)), self.cwidth*q,
                    (b2*(self.inch/6)))
            if self.remits == "Y":
                self.c.setFillGray(.8)
                x = self.cwidth*80.5
                w = self.cwidth*45
                self.c.rect(x, (b3*(self.inch/6)), w, (1.25*(self.inch/6)),
                    fill=1)
                self.c.rect(x, (b3*(self.inch/6)), w, (b1*(self.inch/6)),
                    fill=0)
                self.c.setFillGray(0)
                self.c.setFont("Tartan-Bold", 10)
                for q, r in ((81, "   Date"), (92, "Reference"), (108, "Typ"),
                                                    (112, " Bal-Amount")):
                    self.c.drawString(self.cwidth*q, (16.40*(self.inch/6)), r)
                self.c.setFont("Tartan", 10)
                for q in (80.5, 91.5, 107.5, 111.5):
                    self.c.line(self.cwidth*q, (b3*(self.inch/6)),
                        self.cwidth*q, (b2*(self.inch/6)))
            # Draw Ageing Rectangles
            self.c.setFillGray(.8)
            x = self.cwidth*.5
            w = self.cwidth*78
            if self.remits == "Y":
                b1 = 32.50
                b2 = 35.25
                b3 = 33.40
            else:
                b1 = 52.50
                b2 = 55.25
                b3 = 53.40
            self.c.rect(x, (b1*(self.inch/6)), w, (1.25*(self.inch/6)), fill=1)
            self.c.rect(x, (b1*(self.inch/6)), w, (2.75*(self.inch/6)), fill=0)
            self.c.setFillGray(0)
            self.c.setFont("Tartan-Bold", 10)
            for q, r in ((2, " 120 Days"), (14, "  90 Days"), (26, "  60 Days"),
                (39, "  30 Days"), (52, "  Current"), (65, "    Total")):
                if self.sage == "N" and q < 65:
                    continue
                self.c.drawString(self.cwidth*q, (b3*(self.inch/6)), r)
            self.c.setFont("Tartan", 10)
            for q in (13.5, 25.5, 38.5, 51.5, 64.5):
                if self.sage == "N" and q < 64.5:
                    continue
                self.c.line(self.cwidth*q, (b1*(self.inch/6)), self.cwidth*q,
                    (b2*(self.inch/6)))
            if self.remits == "Y":
                self.c.setFillGray(.8)
                x = self.cwidth*111.5
                w = self.cwidth*14
                self.c.rect(x,(b1*(self.inch/6)),w,(1.25*(self.inch/6)),fill=1)
                self.c.rect(x,(b1*(self.inch/6)),w,(2.75*(self.inch/6)),fill=0)
                self.c.setFillGray(0)
                self.c.setFont("Tartan-Bold", 10)
                q = 113
                r = "   Total"
                self.c.drawString(self.cwidth*q,(b3*(self.inch/6)), r)
                self.c.setFont("Tartan", 10)
        else:
            for q, r in ((2, "   Date"), (13, "Reference"), (23, "Typ"),
                (27, "Details"), (63, "    Amount")):
                self.c.drawString(self.cwidth*q, (16.40*(self.inch/6)), r)
            self.c.setFont("Tartan", 10)
            for q in (12.5, 22.5, 26.5, 62.5):
                self.c.line(self.cwidth*q, (b3*(self.inch/6)), self.cwidth*q,
                    (b2*(self.inch/6)))
            # Draw Total Rectangles
            self.c.setFillGray(.8)
            x = self.cwidth*.5
            w = self.cwidth*78
            b1 = self.maxlines + 17.5
            b2 = self.maxlines + 20.25
            b3 = self.maxlines + 18.4
            self.c.rect(x, (b1*(self.inch/6)), w, (1.25*(self.inch/6)), fill=1)
            self.c.rect(x, (b1*(self.inch/6)), w, (2.75*(self.inch/6)), fill=0)
            self.c.setFillGray(0)
            self.c.setFont("Tartan-Bold", 10)
            for q, r in (
                (17,"     Arrears"),
                (33,"  Month-Excl"),
                (49,"   Month-VAT"),
                (65, "   Total-Due")):
                if self.sage == "N" and q < 65:
                    continue
                self.c.drawString(self.cwidth*q, (b3*(self.inch/6)), r)
            self.c.setFont("Tartan", 10)
            for q in (30.5, 46.5, 62.5,):
                if self.sage == "N" and q < 62.5:
                    continue
                self.c.line(self.cwidth*q, (b1*(self.inch/6)), self.cwidth*q,
                    (b2*(self.inch/6)))

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()

