#! /usr/bin/env python

"""
SYNOPSIS
    Debtors Ledger Sales History.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import Balances, CCD, CreateChart, ProgressBar, Sql
from TartanClasses import TartanDialog
from tartanFunctions import doReportlab, doWriteExport, getModName, doPrinter
from tartanFunctions import showError
from tartanWork import mthnam

class dr3110:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, "drsmst", prog=__name__)
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i) %6s" % (t[0],
            t[1], t[2], t[3], t[4], __name__)
        self.head = ("%03u %-30s %s" % (self.conum, self.conam, "%s"))
        self.forms = [("UI",3), ("NA",7), ("NA",30), ("Na",3), ("UA",3),
            ("UA",3)] + [("SL",10)] * 12
        self.stots = [0] * 12
        self.gtots = [0] * 12
        self.mchart = []
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Debtor's Sales History (%s)" % __name__)
        r1s = (("Yes","Y"),("No","N"))
        fld = (
            (("T",0,0,0),"ID2",7,"Ending Period","Ending Period",
                (self.sysdtw/100),"Y",self.doDat,None,None,("efld",)),
            (("T",0,1,0),("IRB",r1s),0,"Ignore Zero Sales","Ignore Zero Sales",
                "Y","Y",self.doZer,None,None,None),
            (("T",0,2,0),("IRB",r1s),0,"Report By Salesman","",
                "N","Y",self.doRep,None,None,None),
            (("T",0,3,0),("IRB",r1s),0,"Report By Activity","",
                "N","Y",self.doAct,None,None,None),
            (("T",0,4,0),("IRB",r1s),0,"Report By Type","",
                "N","Y",self.doTyp,None,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("Y","V"), mail=("Y","N"))

    def doDat(self, frt, pag, r, c, p, i, w):
        self.coffw = w
        self.coffd = self.df.t_disp[pag][r][p]

    def doZer(self, frt, pag, r, c, p, i, w):
        self.zer = w
    
    def doRep(self, frt, pag, r, c, p, i, w):
        self.rep = w
        if self.rep == "Y":
            self.act = "N"
            self.typ = "N"
            return "sk2"
    
    def doAct(self, frt, pag, r, c, p, i, w):
        self.act = w
        if self.act == "Y":
            self.typ = "N"
            return "sk1"
    
    def doTyp(self, frt, pag, r, c, p, i, w):
        self.typ = w

    def doEnd(self):
        self.df.closeProcess()
        if self.rep == "Y":
            odr = "drm_rep, drm_chain, drm_acno"
        elif self.act == "Y":
            odr = "drm_bus_activity, drm_chain, drm_acno"
        elif self.typ == "Y":
            odr = "drm_bus_type, drm_chain, drm_acno"
        else:
            odr = "drm_chain, drm_acno"
        acs = self.sql.getRec(tables="drsmst", cols=["drm_chain", "drm_acno",
            "drm_name", "drm_rep", "drm_bus_activity", "drm_bus_type"],
            where=[("drm_cono", "=", self.conum)], order=odr)
        if not acs:
            showError(self.mf.body, "Selection Error", "No Accounts Selected")
        else:
            self.start = self.coffw - 99
            if self.start % 100 == 13:
                self.start = self.start + 88
            s = self.start % 100
            self.mthhead = ""
            self.colsh = ["Chn", "Acc-Num", "Name", "Rep", "Act", "Typ"]
            for x in range(12):
                if self.mthhead:
                    self.mthhead = "%s %9s " % (self.mthhead, mthnam[s][1])
                else:
                    self.mthhead = "%9s " % mthnam[s][1]
                self.colsh.append(mthnam[s][1])
                s += 1
                if s == 13:
                     s = 1
            if self.df.repprt[0][2] == "export":
                self.exportReport(acs)
            else:
                self.printReport(acs)
        self.closeProcess()

    def exportReport(self, acs):
        p = ProgressBar(self.mf.body, max=len(acs), esc=True)
        expnam = getModName(self.mf.wrkdir, __name__, self.conum)
        self.expheads = [self.head % self.sysdttm]
        self.expheads.append("Debtor's Sales History for the 12 Months "\
            "Period to %s" % self.coffd)
        self.expheads.append("Options:- Ignore-Zeros: %s" % self.zer)
        self.expcolsh = [self.colsh]
        self.expforms = self.forms
        self.expdatas = []
        for num, dat in enumerate(acs):
            can = p.displayProgress(num)
            if can == "yes":
                p.closeProgress()
                return
            vals = self.getValues(dat)
            if not vals:
                continue
            line = ["BODY", [vals[0].work, vals[1].work, vals[2].work,
                vals[3].work, vals[4].work, vals[5].work]]
            for val in vals[7]:
                line[1].append(val)
            self.expdatas.append(line)
        p.closeProgress()
        self.doTotals()
        doWriteExport(xtype=self.df.repprt[0][1], name=expnam, sheet=__name__,
            heads=self.expheads, colsh=self.expcolsh, forms=self.expforms,
            datas=self.expdatas, rcdic=self.mf.rcdic)

    def printReport(self, acs):
        p = ProgressBar(self.mf.body, max=len(acs), esc=True)
        chrs = len(self.colsh)
        if self.rep == "Y" or self.act == "Y" or self.typ == "Y":
            chrs -= 1
        for n, f in enumerate(self.forms):
            if self.rep == "Y" and n == 3:
                continue
            elif self.act == "Y" and n == 4:
                continue
            elif self.typ == "Y" and n == 5:
                continue
            chrs += int(f[1])
        if f[0][0].lower() == "s":
            chrs -= 2
        else:
            chrs -= 1
        pad = chrs - 35 - len(self.sysdttm)
        self.head1 = self.head % (" " * pad + self.sysdttm)
        self.head2 = "Debtor's Sales History for the 12 Months "\
            "Period to %s%s" % (self.coffd, "%s%s")
        pad = chrs - len(self.head2) + 4 - 11 # %s%s and ' Page     1'
        self.head2 = self.head2 % (" " * pad, " Page %5s")
        pdfnam = getModName(self.mf.wrkdir, __name__, self.conum, ext="pdf")
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, pdfnam, self.head1)
        self.pgnum = 0
        self.pglin = 99
        self.repno = None
        self.actno = None
        self.typno = None
        for num, dat in enumerate(acs):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            vals = self.getValues(dat)
            if not vals:
                continue
            if self.pglin > self.lpp:
                if self.pgnum:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
            if self.rep == "Y":
                self.t.textLine("%s %s %s %s %s %s" % (vals[0].disp,
                    vals[1].disp, vals[2].disp, vals[4].disp, vals[5].disp,
                    vals[6]))
            elif self.act == "Y":
                self.t.textLine("%s %s %s %s %s %s" % (vals[0].disp,
                    vals[1].disp, vals[2].disp, vals[3].disp, vals[5].disp,
                    vals[6]))
            elif self.typ == "Y":
                self.t.textLine("%s %s %s %s %s %s" % (vals[0].disp,
                    vals[1].disp, vals[2].disp, vals[3].disp, vals[4].disp,
                    vals[6]))
            else:
                self.t.textLine("%s %s %s %s %s %s %s" % (vals[0].disp,
                    vals[1].disp, vals[2].disp, vals[3].disp, vals[4].disp,
                    vals[5].disp, vals[6]))
            self.pglin = self.pglin + 1
        p.closeProgress()
        if self.t:
            if can != "yes":
                if self.rep == "Y" or self.act == "Y" or self.typ == "Y":
                    self.doTotals(ttype="S")
                self.doTotals(ttype="G")
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=pdfnam, head=self.tit,
            view=self.df.repprt[0][1], prtnam=self.df.repprt[0][2],
            mail=self.df.repprt[1][2])
        if self.df.repprt[0][1] == "X":
            return
        CreateChart(self.mf, self.conum, self.conam, [self.start, self.coffw],
            [self.tit, "Values"], None, self.mchart, ylab="Values")

    def getValues(self, data):
        chn = CCD(data[0], "UI", 3)
        acc = CCD(data[1], "NA", 7)
        name = CCD(data[2], "NA", 30)
        rep = CCD(data[3], "Na", 3)
        act = CCD(data[4], "UA", 3)
        typ = CCD(data[5], "UA", 3)
        if self.df.repprt[0][2] != "export":
            if self.rep == "Y":
                if rep.work != self.repno:
                    if self.repno != None:
                        self.doTotals(ttype="S")
                        self.c.drawText(self.t)
                        self.c.showPage()
                    self.repno = rep.work
                    self.pageHeading()
            elif self.act == "Y":
                if act.work != self.actno:
                    if self.actno != None:
                        self.doTotals(ttype="S")
                        self.c.drawText(self.t)
                        self.c.showPage()
                    self.actno = act.work
                    self.pageHeading()
            elif self.typ == "Y":
                if typ.work != self.typno:
                    if self.typno != None:
                        self.doTotals(ttype="S")
                        self.c.drawText(self.t)
                        self.c.showPage()
                    self.typno = typ.work
                    self.pageHeading()
        bals = Balances(self.mf, "DRS", self.conum, self.coffw,
            (chn.work, acc.work))
        this, hist = bals.doCrsDrsHist()
        if not this:
            return
        salesd = ""
        salesw = []
        purtot = 0
        for x in range(11, -1, -1):
            amt = CCD(round(hist[0][x], 0), "SL", 10)
            self.stots[x] = self.stots[x] + amt.work
            self.gtots[x] = self.gtots[x] + amt.work
            purtot = purtot + amt.work
            if salesd:
                salesd = "%s %9s" % (salesd, amt.disp)
            else:
                salesd = "%9s" % amt.disp
            salesw.append(amt.work)
        if self.zer == "Y" and purtot == 0:
            return
        return chn, acc, name, rep, act, typ, salesd, salesw
            
    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head1)
        self.t.textLine()
        self.t.textLine(self.head2 % self.pgnum)
        self.t.textLine()
        self.t.textLine("(%-23s %1s %-12s %1s)" % ("Options:- Ignore Zeros:",
            self.zer, "By Salesman:", self.rep))
        self.t.textLine()
        if self.rep == "Y":
            acc = self.sql.getRec(tables="ctlrep", cols=["rpm_name"],
                where=[("rpm_cono", "=", self.conum), ("rpm_rep", "=",
                self.repno)], limit=1)
            if acc:
                self.repnam = acc[0]
            else:
                self.repnam = "Unallocated Accounts"
            self.t.textLine("Saleman %-3s %-30s" % (self.repno, self.repnam))
            self.t.textLine()
            self.t.textLine("%-3s %-7s %-30s %3s %3s %s" % ("Chn", "Acc-Num",
                "Name", "Act", "Typ", self.mthhead))
        elif self.act == "Y":
            acc = self.sql.getRec(tables="drsact", cols=["dac_desc"],
                where=[("dac_code", "=", self.actno)], limit=1)
            if acc:
                self.actnam = acc[0]
            else:
                self.actnam = "Unallocated Accounts"
            self.t.textLine("Activity %-3s %-30s" % (self.actno, self.actnam))
            self.t.textLine()
            self.t.textLine("%-3s %-7s %-30s %3s %3s %s" % ("Chn", "Acc-Num",
                "Name", "Rep", "Typ", self.mthhead))
        elif self.typ == "Y":
            acc = self.sql.getRec(tables="drstyp", cols=["dtp_desc"],
                where=[("dtp_code", "=", self.typno)], limit=1)
            if acc:
                self.typnam = acc[0]
            else:
                self.typnam = "Unallocated Accounts"
            self.t.textLine("Type %-3s %-30s" % (self.typno, self.typnam))
            self.t.textLine()
            self.t.textLine("%-3s %-7s %-30s %3s %3s %s" % ("Chn", "Acc-Num",
                "Name", "Rep", "Act", self.mthhead))
        else:
            self.t.textLine("%-3s %-7s %-30s %-3s %3s %3s %s" % ("Chn",
                "Acc-Num", "Name", "Rep", "Act", "Typ", self.mthhead))
        self.t.textLine("%s" % (self.suc * len(self.head1)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 8

    def doTotals(self, ttype="G"):
        if self.df.repprt[0][2] == "export":
            self.expdatas.append(["ULINES"])
            self.expdatas.append(["TOTAL", ["", "",
                "Grand Totals", "", "", ""] + self.gtots])
            self.expdatas.append(["ULINED"])
            return
        if ttype == "G":
            desc = "Grand Total"
        elif self.rep == "Y":
            desc = "%s Total" % self.repnam
        elif self.act == "Y":
            desc = "%s Total" % self.actnam
        else:
            desc = "%s Total" % self.typnam
        mchart = ["T", desc]
        salesd = ""
        for x in range(11,-1,-1):
            if ttype == "G":
                a = CCD(self.gtots[x], "SL", 10)
            else:
                a = CCD(self.stots[x], "SL", 10)
                self.stots[x] = 0
            if salesd:
                salesd = "%s %s" % (salesd, a.disp)
            else:
                salesd = "%s" % a.disp
            mchart.append(a.work)
        self.mchart.append(mchart)
        self.t.textLine("%s" % (self.suc * len(self.head1)))
        if self.rep == "Y":
            self.t.textLine("%7s %3s %-30s %-3s %-3s %s" % ("", "",
                desc, "", "", salesd))
        elif self.act == "Y":
            self.t.textLine("%7s %3s %-30s %-3s %-3s %s" % ("", "",
                desc, "", "", salesd))
        elif self.typ == "Y":
            self.t.textLine("%7s %3s %-30s %-3s %-3s %s" % ("", "",
                desc, "", "", salesd))
        else:
            self.t.textLine("%7s %3s %-30s %-3s %-3s %-3s %s" % ("", "",
                desc, "", "", "", salesd))

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

