#! /usr/bin/env python

"""
SYNOPSIS
    Debtors Ledger Interest Chargeable.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, Balances, CCD, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, doPrinter, getModName, projectDate
from tartanFunctions import showError

class dr3120:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, "drsmst", prog=__name__)
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Debtors Interest Chargeable Report (%s)" % __name__)
        fld = (
            (("T",0,0,0),"ID1",10,"Cut-Off Date","Cut-Off Date",
                self.sysdtw,"Y",self.doCutoff,None,None,("efld",)),
            (("T",0,1,0),"IUI",2,"Days Leeway","Days Leeway",
                "","Y",self.doLeeway,None,None,("between",0,31)),
            (("T",0,2,0),"IUD",12.2,"Minimum Balance","Minimum Balance",
                "","Y",self.doMinBal,None,None,("efld",)))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doCutoff(self, frt, pag, r, c, p, i, w):
        self.cutoff = w

    def doLeeway(self, frt, pag, r, c, p, i, w):
        self.lway = w
        self.validw = projectDate(self.cutoff, self.lway)
        f = CCD(self.validw, "D1", 10)
        self.repdate = f.disp

    def doMinBal(self, frt, pag, r, c, p, i, w):
        self.minbal = w

    def doEnd(self):
        self.df.closeProcess()
        self.col = ["drm_chain", "drm_acno", "drm_name", "drm_rfterms",
            "drm_int_per"]
        all = self.sql.getRec(tables="drsmst", cols=self.col,
            where=[("drm_cono", "=", self.conum)], order="drm_chain, drm_acno")
        if not all:
            showError(self.mf.body, "Transaction Error",
            "No Transactions Selected")
        else:
            self.printReport(all)
        self.closeProcess()

    def printReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        self.head = ("%03u %-30s %11s %34s %10s %6s" % \
            (self.conum, self.conam, "", self.sysdttm, "", __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        self.tots = [0,0]
        self.pgnum = 0
        self.pglin = 99
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            chn = CCD(dat[self.col.index("drm_chain")], "UI", 3)
            acno = CCD(dat[self.col.index("drm_acno")], "NA", 7)
            name = CCD(dat[self.col.index("drm_name")], "NA", 30)
            rft = CCD(dat[self.col.index("drm_rfterms")], "UI", 3)
            per = CCD(dat[self.col.index("drm_int_per")], "UD", 11.2)
            if not per.work:
                continue
            bals = Balances(self.mf, "DRS", self.conum, (self.cutoff/100),
                (chn.work, acno.work))
            obal, tbal, ages = bals.doAllBals()
            if tbal <= 0:
                continue
            bal = CCD(tbal, "SD", 13.2)
            cur = CCD(ages[0], "SD", 13.2)
            d30 = CCD(ages[1], "SD", 13.2)
            d60 = CCD(ages[2], "SD", 13.2)
            d90 = CCD(ages[3], "SD", 13.2)
            d120 = CCD(ages[4], "SD", 13.2)
            tm = int(rft.work / 30)
            if tm > 4:
                tm = 4
            od = 0
            for x in range(tm, 5):
                od = float(ASD(od) + ASD(ages[x]))
            odu = CCD(od, "SD", 13.2)
            if odu.work <= 0 or (self.minbal and odu.work < self.minbal):
                continue
            b = round((odu.work * per.work / 1200), 2)
            amt = CCD(b, "SD", 13.2)
            if self.pglin > self.lpp:
                if self.pgnum:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
            self.t.textLine("%s %s %s %s %s %s %s %s" % \
                (chn.disp, acno.disp, name.disp, rft.disp, bal.disp,
                odu.disp, per.disp, amt.disp))
            self.pglin = self.pglin + 1
            self.tots[0] = float(ASD(self.tots[0]) + ASD(odu.work))
            self.tots[1] = float(ASD(self.tots[1]) + ASD(amt.work))
        p.closeProgress()
        if self.t:
            if can != "yes":
                self.grandTotal()
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
            head=self.tit, view=self.df.repprt[0][1],
            prtnam=self.df.repprt[0][2], mail=self.df.repprt[1][2])
            
    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-36s %-7s %45s %5s" % \
            ("Debtors Interest to be Charged up to",
            self.df.t_disp[0][0][0], "Page", self.pgnum))
        self.t.textLine()
        self.t.textLine("%-18s%-10s%-3s%-7s%-2s%-1s" % \
            ("(Options: Cut-Off-", self.df.t_disp[0][0][0], "",
                "Leeway-", self.df.t_disp[0][0][1], ")"))
        self.t.textLine()
        self.t.textLine("%3s %7s %-28s %5s %12s %13s %12s %12s" % \
            ("Chn", "Acc-Num", "Debtor", "Terms", "Tot-Balance",
            "Overdue", "Rate", "Interest"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 8

    def grandTotal(self):
        a = CCD(self.tots[0], "SD", 13.2)
        b = CCD(self.tots[1], "SD", 13.2)
        self.t.textLine()
        self.t.textLine("%-11s %-48s %13s %-11s %13s" % \
            (" ", "Grand Totals", a.disp, "", b.disp))
        self.t.textLine()

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()
            
    def closeProcess(self):
        self.mf.closeLoop()

