#! /usr/bin/env python

"""
SYNOPSIS
    Debtors Messages Listing.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import CCD, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, getModName, textFormat, doPrinter
from tartanFunctions import showError

class drc620:
    def __init__(self, mf):
        self.mf = mf
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, "ctlmes", prog=__name__)
        self.pdfnam = getModName(self.mf.wrkdir, __name__, 0, ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdtd = "%i/%02i/%02i" % (t[0], t[1], t[2])
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.mtyp = "S"
        self.t = None

    def mainProcess(self):
        self.tit = ("Debtors Messages Master Listing (%s)" % __name__,)
        r1s = (("All Types","A"),("Invoice","I"),("Statement","S"))
        r2s = (("Yes","Y"),("No","N"))
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"Message Type","Message Type",
                "I","Y",self.doType,None,None,None),)
        tnd = ((self.doEnd,"Y"), )
        txt = (self.closeProcess, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"))

    def doType(self, frt, pag, r, c, p, i, w):
        if w == "A":
            self.mtyp = "ALL"
        elif w == "I":
            self.mtyp = "INV"
        else:
            self.mtyp = "STA"

    def doEnd(self):
        if self.mtyp != "ALL":
            all = self.sql.getRec(tables="ctlmes", where=[("mss_system",
                "=", self.mtyp)], order="mss_system, mss_message")
        else:
            all = self.sql.getRec(tables="ctlmes", where=[("mss_system",
                "in", ("INV", "STA"))], order="mss_system, mss_message")
        if not all:
            showError(self.mf.body, "Transaction Error",
            "No Transactions Selected")
        else:
            self.printReport(all)
        self.closeProcess()

    def printReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        self.head = ("%-24s %37s %5s %6s" % \
            ("Tartan Systems", self.sysdttm, "", __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        self.pgnum = 0
        self.pglin = 99
        old_sys = all[0][0]
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            self.sys = CCD(dat[0], "NA", 3)
            mes = CCD(dat[1], "UI", 3)
            det = CCD(dat[2], "NA", 150)
            fmt = textFormat(det.work, width=30)
            if len(fmt) > 5:
                l = 5
            else:
                l = len(fmt)
            if old_sys != self.sys.work:
                old_sys = self.sys.work
                self.pglin = self.lpp
            if self.pglin > (self.lpp - 5):
                if self.pgnum:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
            self.t.textLine("%-14s %s %-10s %s" % ("", mes.disp, "", fmt[0]))
            self.pglin = self.pglin + 1
            for q in range(1, len(fmt)):
                self.t.textLine("%-29s %s" % ("", fmt[q]))
                self.pglin = self.pglin + 1
            self.t.textLine()
            for r in range(l, 5):
                fmt.append("")
            self.pglin = self.pglin + 1
        p.closeProgress()
        old_sys = self.sys.work
        if self.t:
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, rep=self.pdfnam, head=self.tit,
            view=self.df.repprt[0][1], prtnam=self.df.repprt[0][2],
            mail=self.df.repprt[1][2])
        self.closeProcess()
            
    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        if self.sys.work == "INV":
            desc = "Debtors  Invoice  Messages as at" 
        else:
            desc = "Debtors Statement Messages as at" 
        self.t.textLine("%-32s %-10s %25s %5s" % \
            (desc, self.sysdtd, "Page", self.pgnum))
        self.t.textLine()
        self.t.textLine("%-14s %-3s %-10s %-30s" % ("", "Num", "", "Message"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 6

    def closeProcess(self):
        self.df.closeProcess()
        self.mf.closeLoop()

