#! /usr/bin/python2

"""
SYNOPSIS
    Financial Loan Calculator.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import TartanDialog

class fc1010:
    def __init__(self, mf):
        self.mf = mf
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        pass

    def mainProcess(self):
        tit = ("Loans and Leases",)
        fld = (
            (("T",0,0,0),"IUD",13.2,"Capital Amount","Capital Amount",
                "","Y",None,None,None,("notzero",)),
            (("T",0,1,0),"IUD",13.2,"Residual Value","Residual Value",
                "","N",None,None,None,("efld",)),
            (("T",0,2,0),"IUI",13.2,"Number of Months","Number of Months",
                "","N",None,None,None,("notzero",)),
            (("T",0,3,0),"IUD",13.2,"Repay Amount","Repay Amount",
                "","N",self.doRep,None,None,("efld",)),
            (("T",0,4,0),"IUD",13.4,"Interest Rate","Interest Rate",
                "","N",self.doInt,None,None,("efld",)))
        tnd = ((self.doEnd, "y"), )
        txt = (self.doExit, )
        but = (
            ("New", None, self.doMore, 1, None, None),
            ("Quit", None, self.doExit, 1, None, None))
        self.df = TartanDialog(self.mf, tops=True, title=tit, eflds=fld,
            tend=tnd, txit=txt, butt=but)

    def doRep(self, frt, pag, r, c, p, i, w):
        if w != 0:
            return "nd"

    def doInt(self, frt, pag, r, c, p, i, w):
        if w != 0:
            return "nd"
        else:
            return "Invalid Rate"

    def doEnd(self):
        cap, res, mth, rep, int = self.df.t_work[0][0]
        if rep != 0:
            self.doRate(cap, res, mth, rep)
        else:
            self.doRepay(cap, res, mth, int)

    def doMore(self, widget=None):
        self.df.setWidget(self.df.topEntry[0][3], state="normal",
            bg="white", fg="black")
        self.df.setWidget(self.df.topEntry[0][4], state="normal",
            bg="white", fg="black")
        self.df.setWidget(self.df.B0, "normal")
        self.df.focusField("T", 0, 1)

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

    def doRepay(self, cap, res, mth, int):
        rate = (int / 1200.0)
        rep = (((cap*rate)*((1+rate)**mth))-(res*rate)) / \
            (((1+rate)**mth)-1)
        self.df.loadEntry("T", 0, 3, data=round(rep, 2))
        self.df.setWidget(self.df.topEntry[0][3], state="normal",
            bg="red", fg="yellow")

    def doRate(self, cap, res, mth, rep):
        int = 0.0000
        pay = 0.0
        while pay < rep:
            int += .0001
            rate = (int / 1200.0)
            pay = (((cap*rate)*((1+rate)**mth))-(res*rate)) / \
                (((1+rate)**mth)-1)
        self.df.loadEntry("T", 0, 4, data=round(int, 4))
        self.df.setWidget(self.df.topEntry[0][4], "normal",
            bg="red", fg="yellow")

