#! /usr/bin/python

"""
SYNOPSIS
    General Ledger Detail Records Creation, Deletion and Amendments.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import Sql, TartanDialog

class gl1040:
    def __init__(self, mf, conum, conam, period):
        self.mf = mf
        self.conum = conum
        self.conam = conam
        self.period = period
        self.setVariables()
        self.buildScreen()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["gendtm", "gendtt"], prog=__name__)
        self.title = ("%-3d %s" % (self.conum, self.conam),
            "General Ledger Details Maintenance (%s)" % __name__)
        self.sper = self.period[1][0] / 100
        self.eper = self.period[2][0] / 100

    def buildScreen(self):
        cod_sel = {
            "stype": "R",
            "tables": ("gendtm",),
            "cols": (("gdm_code", "", 0, "Cod"),
                    ("gdm_desc", "", 0, "Description", "Y")),
            "where": [("gdm_cono", "=", self.conum)]}
        per_sel = {
            "stype": "R",
            "tables": ("gendtt",),
            "cols": (("gdt_curdt", "", 0, "Period"),
                    ("gdt_value", "", 0, "Value")),
            "where": [
                ("gdt_cono", "=", self.conum),
                ("gdt_curdt", "between", self.sper, self.eper)],
            "whera": (("T","gdt_code", 0),)}
        flds = (
            (("T",0,0,0,4),"INA",2,"Code","Code",
                "","Y",self.doCode,cod_sel,None,("notblank",)),
            (("T",0,0,0),"INA",30,"Description","Description",
                "","N",self.doDesc,None,self.doDelCode,("notblank",)),
            (("C",0,0,0),"ID2",7,"Period","Financial Period",
                "p","N",self.doPeriod,per_sel,None,None),
            (("C",0,0,1),"ISD",13.2,"Value","Period Value",
                "","N",self.doValue,None,self.doDelPeriod,None))
        row = (17,)
        but = (
            ("Cancel",None,self.doCancel,0,("C",0,1),("T",0,1)),
            ("Quit",None,self.exitTops,1,None,None))
        txt = (self.exitTops,)
        tnd = ((self.endTops,"y"),)
        cnd = ((self.endData,"y"),)
        cxt = (self.exitData,)
        self.df = TartanDialog(self.mf, sc=None, title=self.title, eflds=flds,
            rows=row, butt=but, tend=tnd, cend=cnd, txit=txt, cxit=cxt)

    def doCode(self, frt, pag, r, c, p, i, code):
        self.code = code
        cod = self.sql.getRec(tables="gendtm", cols=["gdm_desc"],
            where=[("gdm_cono", "=", self.conum), ("gdm_code", "=", self.code)],
            limit=1)
        if not cod:
            self.newcod = "y"
        else:
            self.newcod = "n"
            self.df.loadEntry(frt, pag, p+1, data=cod[0])

    def doDelCode(self):
        self.sql.delRec("gendtm", where=[("gdm_cono", "=", self.conum),
            ("gdm_code", "=", self.code)])
        self.sql.delRec("gendtt", where=[("gdt_cono", "=", self.conum),
            ("gdt_code", "=", self.code)])

    def doDesc(self, frt, pag, r, c, p, i, desc):
        self.desc = desc

    def endTops(self):
        data = [self.conum, self.code, self.desc]
        if self.newcod == "y":
            self.sql.insRec("gendtm", data=data)
        else:
            self.sql.updRec("gendtm", data=data, where=[("gdm_cono", "=",
            self.conum), ("gdm_code", "=", self.code)])
        self.df.focusField("C", 0, 1)

    def doPeriod(self, frt, pag, r, c, p, i, w):
        if w < self.sper or w > self.eper:
            return "Invalid Period"
        self.per = w
        val = self.sql.getRec(tables="gendtt", cols=["gdt_value"],
            where=[("gdt_cono", "=", self.conum), ("gdt_code", "=", self.code),
            ("gdt_curdt", "=", self.per)], limit=1)
        if not val:
            self.newper = "y"
        else:
            self.newper = "n"
            self.df.loadEntry(frt, pag, p+1, data=val[0])

    def doDelPeriod(self):
        if self.newper == "y":
            return
        self.sql.delRec("gendtt", where=[("gdt_cono", "=", self.conum),
            ("gdt_code", "=", self.code), ("gdt_curdt", "=", self.per)])

    def doValue(self, frt, pag, r, c, p, i, val):
        self.val = val

    def endData(self):
        data = [self.conum, self.code, self.per, self.val]
        if self.newper == "y":
            self.sql.insRec("gendtt", data=data)
        else:
            self.sql.updRec("gendtt", data=data, where=[("gdt_cono", "=",
            self.conum), ("gdt_code", "=", self.code), ("gdt_curdt", "=",
            self.per)])
        self.df.advanceLine(0)

    def doCancel(self, widget=None):
        self.mf.dbm.rollbackDbase()
        self.df.focusField("T", 0, 1)

    def exitTops(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

    def exitData(self):
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

