#! /usr/bin/python

"""
SYNOPSIS
    General Ledger - Capturing Stream Records.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import Sql, TartanDialog
from tartanFunctions import showPrinters

class gl1050:
    def __init__(self, mf, conum, conam):
        self.mf = mf
        self.conum = conum
        self.conam = conam
        no = self.doVariables()
        if not no:
            self.doProcess()
            self.mf.startLoop()

    def doVariables(self):
        self.sql = Sql(self.mf.dbm, ["genrpc", "genstr", "gendtm", "genrpt"],
            prog=__name__)
        if self.sql.error:
            return self.sql.error
        self.strm = 0

    def doProcess(self):
        tit = ("%-3d %s" % (self.conum, self.conam),
            "General Ledger Report Stream Records (%s)" % __name__)
        st1 = {
            "stype": "R",
            "tables": ("genstr",),
            "cols": (
                ("gls_strm", "", 0, "Str"),
                ("gls_desc", "", 0, "Description", "Y")),
            "where": [("gls_cono", "=", self.conum)],
            "group": "gls_strm, gls_desc"}
        st2 = {
            "stype": "R",
            "tables": ("genstr",),
            "cols": (
                ("gls_seq", "", 0, "Seq"),
                ("gls_typ", "", 0, "T"),
                ("gls_cno", "", 0, "CN"),
                ("gls_con", "", 0, "C"),
                ("gls_rep", "", 0, "Rep"),
                ("gls_gen", "", 0, "G"),
                ("gls_val", "", 0, "V"),
                ("gls_det", "", 0, "Det"),
                ("gls_zer", "", 0, "Z"),
                ("gls_opt", "", 0, "O"),
                ("gls_num", "", 0, "N"),
                ("gls_prnt", "", 20, "Printer-Name"),
                ("gls_mail", "", 20, "E-Mail-Address")),
            "where": [("gls_cono", "=", self.conum)],
            "whera": (("T", "gls_strm", 0, 0),),
            "comnd": self.doSelTyp}
        typ = {
            "stype": "C",
            "title": "Available Types",
            "data": (
                ("S", "YTD Report"),
                ("L", "MTD and YTD Report"),
                ("M", "Monthly Report"),
                ("C", "Customised Report"))}
        rpc = {
            "stype": "R",
            "tables": ("genrpc",),
            "cols": (
                ("glc_cusno", "", 0, "CN"),
                ("glc_head1", "", 0, "Heading-1"),
                ("glc_head2", "", 0, "Heading-2"),
                ("glc_head3", "", 0, "Heading-3"),
                ("glc_head4", "", 0, "Heading-4")),
            "where": [("glc_cono", "=", self.conum)]}
        rpt = {
            "stype": "R",
            "tables": ("genrpt",),
            "cols": (
                ("glr_cono", "", 0, "Coy"),
                ("glr_repno", "", 0, "Rep"),
                ("glr_type", "", 0, "T"),
                ("glr_desc", "", 0, "Description", "Y")),
            "where": [("(", "glr_cono", "=", 0, "or", "glr_cono", "=",
                self.conum, ")"), ("glr_seq", "=", 0)],
            "group": "glr_cono, glr_repno, glr_type, glr_desc",
            "index": 1}
        con = {
            "stype": "C",
            "title": "Available Options",
            "data": (
                ("V", "Actuals"),
                ("B", "Budgets"),
                ("C", "Actuals and Budgets"),
                ("X", "Variance to Budget"),
                ("D", "Detail"))}
        self.printers = showPrinters()
        self.printers.insert(1, "Export")
        prt = {
            "stype": "C",
            "titl": "Select Required Printer",
            "head": ("Name","Description"),
            "data": self.printers}
        fld = (
            (("T",0,0,0),"IUI",3,"Stream-Number","Stream Number",
                0,"Y",self.doStr,st1,None,("notzero",)),
            (("T",0,0,0),"INA",30,"Description","Description",
                "","N",self.doDsc,None,self.doDelStr,("notblank",)),
            (("C",0,0,0),"IUI",3,"Seq","Sequence Number",
                "i","N",self.doSeq,st2,None,("efld",)),
            (("C",0,0,1),"IUA",1,"T","Report Type",
                "S","N",self.doTyp,typ,self.doDelLin,("in",("S","L","M","C"))),
            (("C",0,0,2),"IUI",2,"CN","Custom Number",
                0,"N",self.doCus,rpc,None,("efld",)),
            (("C",0,0,3),"IUA",1,"C","Consolidate (Y/N)",
                "N","N",self.doCon,None,None,("in",("Y","N"))),
            (("C",0,0,4),"IUI",3,"Rep","Report Number",
                0,"N",self.doRep,rpt,None,("notzero",)),
            (("C",0,0,5),"IUA",1,"G","General Report (Y/N)",
                "N","N",self.doGen,None,None,("in",("Y","N"))),
            (("C",0,0,6),"IUA",1,"V","Report Contents",
                "V","N",self.doContent,con,None,("in",("V","B","C","X","D"))),
            (("C",0,0,7),"IUI",3,"Cod","Details Code",
                "","N",self.doCod,None,None,None),
            (("C",0,0,8),"IUA",1,"Z","Ignore Zeros (Y/N)",
                "N","N",None,None,None,("in",("Y","N"))),
            (("C",0,0,9),"IUA",1,"O","Print Options (Y/N)",
                "N","N",None,None,None,("in",("Y","N"))),
            (("C",0,0,10),"IUA",1,"N","Print Numbers (Y/N)",
                "N","N",None,None,None,("in",("Y","N"))),
            (("C",0,0,11),"ITX",30,"Printer Name","Printer-Name",
                "Default","N",self.doPrt,prt,None,None),
            (("C",0,0,12),"ITX",30,"E-Mail Address","E-Mail-Address",
                "","N",None,None,None,("email",)))
        but = (
            ("Cancel",None,self.doCancel,0,("C",0,1),("T",0,1)),
            ("Quit",None,self.exitTops,1,None,None))
        self.df = TartanDialog(self.mf, title=tit, eflds=fld,
            tend=((self.endTops,"n"),), txit=(self.exitTops,),
            cend=((self.endData,"y"),), cxit=(self.exitData,), butt=but)
        self.df.focusField("T", 0, 1)

    def doStr(self, frt, pag, r, c, p, i, w):
        self.strm = w
        acc = self.sql.getRec(tables="genstr", cols=["gls_desc"],
            where=[("gls_cono", "=", self.conum), ("gls_strm", "=",
            self.strm)], limit=1)
        if not acc:
            self.new = "y"
        else:
            self.new = "n"
            self.desc = acc[0]
            self.df.loadEntry(frt, pag, p+1, self.desc)

    def doDelStr(self):
        if self.new == "y":
            return
        self.sql.delRec("genstr", where=[("gls_cono", "=", self.conum),
            ("gls_strm", "=", self.strm)])

    def doDsc(self, frt, pag, r, c, p, i, w):
        self.desc = w

    def doSeq(self, frt, pag, r, c, p, i, w):
        self.seq = w
        dt = self.sql.getRec(tables="genstr", where=[("gls_cono", "=",
            self.conum), ("gls_strm", "=", self.strm), ("gls_seq", "=",
            self.seq)], limit=1)
        if not dt:
            self.newlin = "y"
        else:
            self.newlin = "n"
            for n, d in enumerate(dt[3:]):
                self.df.loadEntry(frt, pag, p+n, data=d)

    def doTyp(self, frt, pag, r, c, p, i, w):
        self.typ = w
        if self.typ != "C":
            self.cno = 0
            return "sk1"

    def doCus(self, frt, pag, r, c, p, i, w):
        cn = self.sql.getRec(tables="genrpc", where=[("glc_cono", "=",
            self.conum), ("glc_cusno", "=", w)], limit=1)
        if not cn:
            return "Invalid Custom Report Number"
        self.cno = w

    def doCon(self, frt, pag, r, c, p, i, w):
        self.con = w

    def doRep(self, frt, pag, r, c, p, i, w):
        rp = self.sql.getRec(tables="genrpt", cols=["glr_cono", "glr_repno"],
            where=[("(", "glr_cono", "=", 0, "or", "glr_cono", "=", self.conum,
            ")"), ("glr_repno", "=", w)], limit=1)
        if not rp:
            return "No Such Report"
        self.rep = w

    def doGen(self, frt, pag, r, c, p, i, w):
        if w == "Y":
            c = 0
        else:
            c = self.conum
        rp = self.sql.getRec(tables="genrpt", cols=["glr_cono", "glr_repno"],
            where=[("glr_cono", "=", c), ("glr_repno", "=", self.rep)], limit=1)
        if not rp:
            return "No Such Report"
        self.gen = w

    def doContent(self, frt, pag, r, c, p, i, w):
        if w in ("B", "C", "X") and self.typ in ("S", "L", "C"):
            return "Invalid Choice for this Report Type"
        if w != "D":
            self.df.loadEntry(frt, pag, p+1, data="")
            return "sk1"

    def doCod(self, frt, pag, r, c, p, i, w):
        d = self.sql.getRec(tables="gendtm", cols=["gdm_desc"],
            where=[("gdm_cono", "=", self.conum), ("gdm_code", "=", w)],
            limit=1)
        if not d:
            return "Invalid Detail Code"

    def doPrt(self, frt, pag, r, c, p, i, w):
        if w not in self.printers:
            return "Invalid Printer"
        if w == "Export":
            self.df.loadEntry(frt, pag, p+1, data="")
            return "sk1"

    def endTops(self):
        if self.new == "y":
            self.df.focusField("C", 0, 1)
        else:
            self.doLoadLines()

    def doCancel(self, widget=None):
        self.mf.dbm.rollbackDbase()
        self.df.focusField("T", 0, 1)

    def exitTops(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

    def doSelTyp(self, frt, pag, r, c, p, i, w):
        self.newlin = "n"
        self.typ = w[0]
        self.cno = w[1]
        self.con = w[2]
        self.rep = w[3]
        self.gen = w[4]
        for x in range(0, len(w)):
            self.df.loadEntry(frt, pag, p+x, data=w[x])
        self.df.col = c + 5
        self.df.skip[pag] = [2,3,4,5]

    def doDelLin(self):
        if self.newlin == "y":
            return
        self.sql.delRec("genstr", where=[("gls_cono", "=", self.conum),
            ("gls_strm", "=", self.strm), ("gls_seq", "=", self.seq)])
        self.doReSeq()

    def endData(self):
        data = [self.conum, self.strm, self.desc, self.seq, self.typ,
            self.cno, self.con, self.rep, self.gen]
        for x in range(6, 13):
            data.append(self.df.c_work[self.df.pag][self.df.row][x])
        if self.newlin == "y":
            self.sql.insRec("genstr", data=data)
        else:
            self.sql.updRec("genstr", data=data, where=[("gls_cono", "=",
            self.conum), ("gls_strm", "=", self.strm), ("gls_seq", "=",
            self.seq)])
        self.doLoadLines()

    def exitData(self):
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doReSeq(self):
        all = self.sql.getRec(tables="genstr", where=[("gls_cono", "=",
            self.conum), ("gls_strm", "=", self.strm)], order="gls_seq")
        if not all:
            self.newlin = "y"
            self.df.focusField("C", 0, 1)
            return
        self.sql.delRec("genstr", where=[("gls_cono", "=", self.conum),
            ("gls_strm", "=", self.strm)])
        for seq, acc in enumerate(all):
            acc[3] = seq
            self.sql.insRec("genstr", data=acc)
        self.doLoadLines()

    def doLoadLines(self):
        self.df.clearFrame("C", 0)
        pos = 0
        all = self.sql.getRec(tables="genstr", where=[("gls_cono", "=",
            self.conum), ("gls_strm", "=", self.strm)])
        for row, rec in enumerate(all):
            for num, dat in enumerate(rec):
                if num < 3:
                    continue
                self.df.loadEntry("C", 0, pos, data=dat)
                pos += 1
                if pos == (self.df.rows[0] * self.df.colq[0]):
                    self.df.scrollScreen(0)
                    pos = pos - self.df.colq[0]
        pos += 1
        self.df.focusField("C", 0, pos)

