#! /usr/bin/python

"""
SYNOPSIS
    General Ledger - Capturing Opening Balances.

    These are various classes used in TARTAN.

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import Sql, TartanDialog
from tartanFunctions import showError

class gl2010:
    def __init__(self, mf, conum, conam):
        self.mf = mf
        self.conum = conum
        self.conam = conam
        no = self.setVariables()
        if not no:
            self.drawDialog()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlynd", "genbal"], prog=__name__)
        if self.sql.error:
            return self.sql.error
        per = self.sql.getRec(tables="ctlynd", where=[("cye_cono", "=",
            self.conum), ("cye_period", "=", 0)], limit=1)
        if not per:
            showError(self.mf.body, "Period Error", """No Period 0!\n\nPlease Contact your Software Support and Quote this Message as this is a Serious Error!""")
            return "No Period 0"
        self.start = per[self.sql.ctlynd_col.index("cye_start")]

    def drawDialog(self):
        sel_acc = {
            "stype": "R",
            "tables": ("genmst",),
            "cols": (("glm_acno", "", 0, "Acc-Num"),
                    ("glm_desc", "", 0, "Description", "Y")),
            "where": [("glm_cono", "=", self.conum)]}
        sel_all = {
            "stype": "R",
            "tables": ("genbal","genmst"),
            "cols": (
                ("glo_acno", "", 0, "Acc-Num"),
                ("glm_desc", "", 0, "Description"),
                ("glo_trdt", "", 0, "Date"),
                ("glo_cyr", "", 0, "Balance")),
            "where": [("glo_cono", "=", self.conum), ("glm_cono=glo_cono",),
                ("glm_acno=glo_acno",), ("glo_trdt", "=", self.start)]}
        tit = ("%-3d %s" % (self.conum, self.conam),
            "General Ledger Opening Balances Data Capture (%s)" % __name__)
        fld = (
            (("C",0,0,0),"IUI",7,"Acc-Num","Account Number",
                "","N",self.doAcc,sel_acc,None,None),
            (("C",0,0,1),"ONA",30,"Description"),
            (("C",0,0,2),"ISD",13.2,"Balance","Balance Value",
                "","N",self.doBal,None,self.doDelAcc,None))
        but = (("All Entries", sel_all, None, 1, ("C", 0, 1), None),)
        row = (20,)
        self.df = TartanDialog(self.mf, title=tit, rows=row, eflds=fld,
            cend=((self.endData,"y"),), cxit=(self.exitData,), butt=but)

    def doAcc(self, frt, pag, r, c, p, i, w):
        desc = self.sql.getRec(tables="genmst", cols=["glm_desc"],
            where=[("glm_cono", "=", self.conum), ("glm_acno", "=", w)],
            limit=1)
        if not desc:
            return "Invalid Account Number"
        self.acc = w
        self.df.loadEntry(frt, pag, p+1, data=desc[0])
        bals = self.sql.getRec(tables="genbal", cols=["glo_cyr"],
            where=[("glo_cono", "=", self.conum), ("glo_acno", "=", self.acc),
            ("glo_trdt", "=", self.start)], limit=1)
        if not bals:
            self.new_acc = "y"
        else:
            self.new_acc = "n"
            self.df.loadEntry(frt, pag, p+2, bals[0])

    def doDelAcc(self):
        if self.new_acc == "y":
            return
        self.sql.delRec("genbal", where=[("glo_cono", "=", self.conum),
            ("glo_acno", "=", self.acc), ("glo_trdt", "=", self.start)])
        self.mf.dbm.commitDbase()

    def doBal(self, frt, pag, r, c, p, i, w):
        self.bal = w

    def endData(self):
        data = [self.conum, self.acc, self.start, self.bal]
        if self.new_acc == "y":
            self.sql.insRec("genbal", data=data)
        else:
            self.sql.updRec("genbal", data=data, where=[("glo_cono", "=",
                self.conum), ("glo_acno", "=", self.acc), ("glo_trdt", "=",
                self.start)])
        self.mf.dbm.commitDbase()
        next = self.sql.getRec(tables="genmst", cols=["glm_acno"],
            where=[("glm_cono", "=", self.conum), ("glm_acno", ">", self.acc)],
            limit=1)
        if next:
            self.df.colf[0][0][5] = next[0]
        else:
            self.df.colf[0][0][5] = 0
        self.df.advanceLine(0)

    def exitData(self):
        self.df.closeProcess()
        self.mf.closeLoop()

