#! /usr/bin/python

"""
SYNOPSIS
    General Ledger Imported Bank Statements..

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import RepPrt, Sql, TartanDialog

class gl3090:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlctl", "genmst"], prog=__name__)

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "General Ledger Imported Bank Statements (%s)" % __name__)
        glm = {
            "stype": "R",
            "tables": ("ctlctl", "genmst"),
            "cols": (
                ("ctl_conacc", "", 0, "Acc-Num"),
                ("glm_desc", "", 0, "Description", "Y")),
            "where": [
                ("ctl_cono", "=", self.conum),
                ("ctl_code", "like", "bank_%"),
                ("glm_cono=ctl_cono",),
                ("glm_acno=ctl_conacc",)]}
        r1s = (("Yes","Y"), ("No","N"))
        fld = (
            (("T",0,0,0),"IUI",7,"Bank Account","Bank Account",
                "","Y",self.doBankAcc,glm,None,("efld",)),
            (("T",0,1,0),"Id1",10,"From Date","From Date (0 for Beginning)",
                0,"Y",self.doFrom,None,None,("efld",)),
            (("T",0,2,0),"Id1",10,"To   Date","To Date (0 for End)",
                0,"Y",self.doTo,None,None,("efld",)),
            (("T",0,3,0),("IRB",r1s),0,"Unallocated Only","Unallocated Only",
                "Y","Y",self.doUnall,None,None,None))
        tnd = ((self.doEnd,"y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("Y","V"), mail=("Y","N"))

    def doBankAcc(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables=["ctlctl", "genmst"], cols=["glm_desc"],
            where=[("glm_cono", "=", self.conum), ("glm_acno", "=", w),
            ("ctl_cono=glm_cono",), ("ctl_conacc=glm_acno",), ("ctl_code",
            "like", "bank_%")], limit=1)
        if not acc:
            return "Invalid Bank Account Number"
        self.acnow = w
        self.acnod = self.df.t_disp[pag][r][p]
        self.name = acc[0]

    def doFrom(self, frt, pag, r, c, p, i, w):
        self.start = w

    def doTo(self, frt, pag, r, c, p, i, w):
        if w and self.start and w < self.start:
            return "Invalid Date, Earlier than From Date"
        self.to = w

    def doUnall(self, frt, pag, r, c, p, i, w):
        self.unall = w

    def doEnd(self):
        self.df.closeProcess()
        if self.unall == "Y":
            hds = ["General Ledger Bank Statement Records (Unallocated Only)"]
        else:
            hds = ["General Ledger Bank Statement Records (All)"]
        des = "Bank Account Number %s %s" % (self.acnod, self.name)
        if self.start:
            des = "%s from %s" % (des, self.df.t_disp[0][0][1])
        if self.to:
            des = "%s to %s" % (des, self.df.t_disp[0][0][2])
        hds.append(des)
        col = ["grt_date", "grt_memo", "grt_refno", "grt_payee", "grt_amount"]
        whr = [("grt_cono", "=", self.conum), ("grt_acno", "=", self.acnow)]
        if self.start:
            whr.append(("grt_date", ">=", self.start))
        if self.to:
            whr.append(("grt_date", "<=", self.to))
        if self.unall == "Y":
            whr.append(("grt_flag", "=", "N"))
        ord = "grt_date, grt_refno"
        tot = ["grt_amount"]
        rpt = RepPrt(self.mf, conum=self.conum, conam=self.conam, name=__name__,
            tables=["genrct"], heads=hds, cols=col, where=whr, order=ord,
            view=self.df.repprt[0][1], gtots=tot, prtnam=self.df.repprt[0][2],
            mail=self.df.repprt[1][2])
        self.closeProcess()

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

