#! /usr/bin/python

"""
SYNOPSIS
    General Ledger Financial Statement Viewer.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, ProgressBar, ScrollGrid, Sql, SRec, TartanDialog
from tartanFunctions import getPeriods, makeArray
from tartanWork import gltrtp, mthnam

class gl4020:
    def __init__(self, mf, coy, nam, per):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.period = per
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlmst", "genctl", "genbal", "genbud",
            "genmst", "genrpt", "gentrn"], prog=__name__)
        t = time.localtime()
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.s_per = self.period[1][0] / 100
        self.e_per = self.period[2][0] / 100
        if self.period[0] > 0:
            self.s_lyr, e_lyr, fin = getPeriods(self.mf, self.conum,
                (self.period[0] - 1))

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "General Ledger Financial Viewer (%s)" % __name__)
        rpt = {
            "stype": "R",
            "tables": ("genrpt",),
            "cols": (
                ("glr_repno", "", 0, "Num"),
                ("glr_type", "", 0, "T"),
                ("glr_desc", "", 0, "Description", "Y")),
            "where": [
                ("glr_cono", "in", (0, self.conum)),
                ("glr_seq", "=", 0),
                ("glr_type", "in", ("P", "B", "O"))],
            "group": "glr_repno, glr_type, glr_desc"}
        r1s = (("Yes","Y"),("No","N"))
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"Consolidate Companies",
                "Consolidate Companies","N","Y",self.doCons,None,None,None),
            (("T",0,1,0),"IUI",3,"Report Number","Report Number",
                1,"Y",self.doRepNum,rpt,None,("notzero",)),
            (("T",0,2,0),("IRB",r1s),0,"General Report","General Report",
                "N","N",self.doRepGen,None,None,None),
            (("T",0,3,0),("IRB",r1s),0,"Ignore Zeros","Ignore Zeros",
                "Y","Y",self.doIgnore,None,None,None))
        tnd = ((self.doMainEnd,"y"), )
        txt = (self.doMainExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, focus=False)
        chk = self.sql.getRec(tables="ctlmst", cols=["count(*)"], limit=1)
        if chk[0] == 1:
            self.con = "N"
            self.df.loadEntry("T", 0, 0, data=self.con)
            self.df.focusField("T", 0, 2)
        else:
            self.df.focusField("T", 0, 1)

    def doCons(self, frt, pag, r, c, p, i, w):
        self.con = w

    def doRepNum(self, frt, pag, r, c, p, i, w):
        if self.con == "Y":
            whr = [("glr_cono", "=", 0), ("glr_repno", "=", w)]
        else:
            whr = [("glr_cono", "in", (0, self.conum)), ("glr_repno", "=", w)]
        whr.append(("glr_seq", "=", 0))
        chk = self.sql.getRec(tables="genrpt", cols=["glr_cono"], where=whr,
            order="glr_cono")
        if not chk:
            return "Invalid Report Number"
        self.rep = w
        self.gen = "N"
        for c in chk:
            if c[0] == 0:
                self.gen = "Y"
                if self.con == "Y":
                    break
            else:
                self.gen = "N"
        if self.gen == "Y":
            err = self.getRepDetails()
            if err:
                return err
            else:
                self.df.loadEntry(frt, pag, p+1, data=self.gen)
                return "sk1"
        elif self.con == "Y":
            return "Invalid Report, Not A General Report"

    def doRepGen(self, frt, pag, r, c, p, i, w):
        if self.con == "Y" and w == "N":
            return "Invalid"
        self.gen = w
        return self.getRepDetails()

    def doIgnore(self, frt, pag, r, c, p, i, w):
        self.zero = w

    def doMainEnd(self):
        self.df.closeProcess()
        if self.con == "Y":
            self.doConCoy()
            if self.con == "X":
                self.mf.closeLoop()
                return
        self.doGenerateReport()
        self.doDisplayReport()
        self.mf.updateStatus("")

    def doConCoy(self):
        tit = ("Companies to Consolidate",)
        self.coys = self.sql.getRec(tables="ctlmst", cols=["ctm_cono",
            "ctm_name"], order="ctm_cono")
        coy = {
            "stype": "C",
            "titl": "",
            "data": self.coys,
            "mode": "M",
            "comnd": self.doCoyCmd}
        r1s = (("Yes","Y"),("Include","I"),("Exclude","E"))
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"All Companies","All Companies",
                "Y","N",self.doAllCoy,None,None,None,None),
            (("T",0,1,0),"INA",(30,100),"Companies","Companies",
                "","N",self.doCoySel,coy,None,None,None))
        state = self.df.disableButtons()
        self.cf = TartanDialog(self.mf, screen=self.mf.body, tops=True,
            title=tit, eflds=fld, tend=((self.doCoyEnd, "y"),),
            txit=(self.doCoyExit,))
        self.mf.startLoop()
        self.df.enableButtons(state=state)

    def doAllCoy(self, frt, pag, r, c, p, i, w):
        self.con = w
        if self.con == "Y":
            return "nd"
        if self.con == "I":
            self.cf.topf[pag][1][8]["titl"] = "Select Companies to Include"
        else:
            self.cf.topf[pag][1][8]["titl"] = "Select Companies to Exclude"

    def doCoyCmd(self, frt, pag, r, c, p, i, w):
        c = ""
        for co in w:
            if int(co) != 0:
                c = c + str(int(co)) + ","
        if len(c) > 1:
            c = c[:-1]
        self.cf.loadEntry(frt, pag, p, data=c)

    def doCoySel(self, frt, pag, r, c, p, i, w):
        if not w:
            return "Invalid List of Companies"
        if w[-1:] == ",":
            w = w[:-1]
        self.coy = w.split(",")

    def doCoyEnd(self):
        if self.con == "I":
            self.con = self.coy
        elif self.con == "E":
            self.con = []
            for co in self.coys:
                self.con.append(int(co[0]))
            for co in self.coy:
                del self.con[self.con.index(int(co))]
            self.con.sort()
        self.doCoyClose()

    def doCoyExit(self):
        self.con = "X"
        self.doCoyClose()

    def doCoyClose(self):
        self.cf.closeProcess()
        self.mf.closeLoop()

    def getRepDetails(self):
        if self.gen == "Y":
            self.rco = 0
        else:
            self.rco = self.conum
        chk = self.sql.getRec(tables="genrpt", cols=["glr_type", "glr_desc"],
            where=[("glr_cono", "=", self.rco), ("glr_repno", "=", self.rep),
            ("glr_seq", "=", 0)], limit=1)
        if not chk:
            return "Invalid Report Number or Type"
        self.rptyp, self.rpdes = chk

    def doGenerateReport(self):
        self.mf.updateStatus("Generating the Report, Please Wait....")
        self.pb = ProgressBar(self.mf.body, typ="G")
        if self.gen == "Y":
            self.repco = 0
        else:
            self.repco = self.conum
        self.grpind = "n"
        self.exit = "n"
        self.clearData()
        self.allFields = []
        self.mth_tot = makeArray(14, 10, 1)
        self.mth_str = makeArray(14, 100, 1)
        self.lyr_tot = makeArray(14, 10, 1)
        self.lyr_str = makeArray(14, 100, 1)
        self.bud_tot = makeArray(14, 10, 1)
        self.bud_str = makeArray(14, 100, 1)
        self.var_tot = makeArray(14, 10, 1)
        self.var_str = makeArray(14, 100, 1)
        self.row = 0
        self.trn = {}
        for x in range(14):
            self.trn[x] = {}
        if self.con == "N":
            self.con = None
        elif self.con == "Y":
            con = "("
            coy = self.sql.getRec(tables="ctlmst", cols=["ctm_cono"],
                order="ctm_cono")
            if not coy:
                return "no"
            self.con = []
            for c in coy:
                self.con.append(c[0])
        self.rep_num = 0
        self.rpt_lst = self.sql.getRec(tables="genrpt", where=[("glr_cono",
            "=", self.repco), ("glr_repno", "=", self.rep)], order="glr_seq")
        if self.rpt_lst:
            self.s_type = self.rpt_lst[0][3]
            self.atype = self.s_type
            if self.s_type == "P":
                self.stanam = "Profit and Loss"
            else:
                self.stanam = self.rpt_lst[self.rep_num][4]
            while self.exit != "y":
                self.processRecords()

    def processRecords(self):
        if self.grpind == "y":
            self.grpind = "n"
        else:
            self.nextRepRecord()
            self.storeRepRecord()
            if self.rpt_dic == {}:
                self.exit = "y"
        if self.exit != "y":
            if self.s_type == "H":
                self.doHead()
            elif self.s_type == "L":
                self.doLedger()
            elif self.s_type == "G":
                self.doGroup()
            elif self.s_type == "T":
                self.doTotal()
            elif self.s_type == "S":
                self.doStore()
            elif self.s_type == "U":
                self.doUnderline()

    def doHead(self):
        self.pb.displayProgress()
        self.printLine()
        self.clearData()

    def doLedger(self):
        accs = self.readGenmst()
        for acc in accs:
            self.pb.displayProgress()
            self.lstdicGenmst(acc)
            self.s_cono = self.gen_dic["glm_cono"]
            self.s_acno = self.gen_dic["glm_acno"]
            self.s_desc = self.gen_dic["glm_desc"]
            trans = self.accumData()
            amt = self.mth[13]
            if self.s_print == "+" and amt < 0:
                self.clearData()
                continue
            elif self.s_print == "-" and amt >= 0:
                self.clearData()
                continue
            elif self.s_print == "N":
                pass
            else:
                if not self.row in self.trn[13]:
                    self.trn[13][self.row] = []
                self.trn[13][self.row].extend(trans)
                self.printLine()
            self.accumTotals(self.s_acbal)
            if self.s_store == "Y":
                self.storeBalances()
            self.clearData()

    def doGroup(self):
        self.grpind = "y"
        trans = []
        while self.rpt_dic and self.rpt_dic["glr_type"] == "G" and \
                        self.rpt_dic["glr_group"] == self.s_group:
            accs = self.readGenmst()
            for acc in accs:
                self.pb.displayProgress()
                self.lstdicGenmst(acc)
                self.s_cono = self.gen_dic["glm_cono"]
                self.s_acno = self.gen_dic["glm_acno"]
                trans.extend(self.accumData())
            self.nextRepRecord()
        amt = self.mth[13]
        if self.s_print == "+" and amt < 0:
            self.clearData()
            return
        elif self.s_print == "-" and amt >= 0:
            self.clearData()
            return
        elif self.s_print == "N":
            pass
        else:
            if not self.row in self.trn[13]:
                self.trn[13][self.row] = []
            self.trn[13][self.row].extend(trans)
            self.printLine()
        self.accumTotals(self.s_acbal)
        if self.s_store == "Y":
            self.storeBalances()
        self.clearData()
        self.storeRepRecord()

    def doTotal(self):
        self.pb.displayProgress()
        for x in range(0, 14):
            self.mth[x] = self.mth_tot[self.s_total][x]
            self.lyr[x] = self.lyr_tot[self.s_total][x]
            self.bud[x] = self.bud_tot[self.s_total][x]
            self.var[x] = self.var_tot[self.s_total][x]
        amt = self.mth[13]
        if self.s_print == "+" and amt < 0:
            self.clearData()
            return
        elif self.s_print == "-" and amt >= 0:
            self.clearData()
            return
        elif self.s_print == "N":
            pass
        else:
            self.printLine()
        if self.s_store == "Y":
            self.storeBalances()
        self.clearData()
        if self.s_clear == "Y":
            for x in range(0, self.s_total+1):
                for y in range(0, 14):
                    self.mth_tot[x][y] = 0
                    self.lyr_tot[x][y] = 0
                    self.bud_tot[x][y] = 0
                    self.var_tot[x][y] = 0

    def doStore(self):
        self.pb.displayProgress()
        for x in range(0, 13):
            if self.mth_str[self.s_snum1][x] == 0:
                self.mth[x] = 0
            else:
                self.mth[x] = round((self.mth_str[self.s_snum1][x] *
                    self.s_strper / 100.0), 2)
                self.mth[13] = float(ASD(self.mth[13]) + ASD(self.mth[x]))
            if self.lyr_str[self.s_snum1][x] == 0:
                self.lyr[x] = 0
            else:
                self.lyr[x] = round((self.lyr_str[self.s_snum1][x] *
                    self.s_strper / 100.0), 2)
                self.lyr[13] = float(ASD(self.lyr[13]) + ASD(self.lyr[x]))
            if self.bud_str[self.s_snum1][x] == 0:
                self.bud[x] = 0
            else:
                self.bud[x] = round((self.bud_str[self.s_snum1][x] *
                    self.s_strper / 100.0), 2)
                self.bud[13] = float(ASD(self.bud[13]) + ASD(self.bud[x]))
        amt = self.mth[13]
        if self.s_print == "+" and amt < 0:
            self.clearData()
            return
        elif self.s_print == "-" and amt >= 0:
            self.clearData()
            return
        elif self.s_print == "N":
            pass
        else:
            self.printLine()
        self.accumTotals(self.s_acbal)
        self.clearData()
        if self.s_clear == "Y":
            for x in range(0, 14):
                self.mth_str[self.s_snum1][x] = 0
                self.lyr_str[self.s_snum1][x] = 0
                self.bud_str[self.s_snum1][x] = 0

    def doUnderline(self):
        self.pb.displayProgress()
        if self.rpt_dic["glr_uline"] == "B":
            self.s_desc = "Blank"
        elif self.rpt_dic["glr_uline"] == "S":
            self.s_desc = "Single"
        else:
            self.s_desc = "Double"
        self.printLine()
        self.clearData()

    def readGenmst(self):
        if not self.con:
            if not self.rpt_dic["glr_to"]:
                accs = self.sql.getRec(tables="genmst", where=[("glm_cono", "=",
                    self.conum), ("glm_acno", "=", self.rpt_dic["glr_from"])])
            else:
                accs = self.sql.getRec(tables="genmst", where=[("glm_cono", "=",
                    self.conum), ("glm_acno", ">=", self.rpt_dic["glr_from"]),
                    ("glm_acno", "<=", self.rpt_dic["glr_to"])],
                    order="glm_acno")
            if not accs:
                accs = []
        else:
            accs = []
            if not self.rpt_dic["glr_to"]:
                acs = self.sql.getRec(tables="genmst", cols=["glm_acno"],
                    where=[("glm_cono", "in", self.con), ("glm_acno", "=",
                    self.rpt_dic["glr_from"])], group="glm_acno")
            else:
                acs = self.sql.getRec(tables="genmst", cols=["glm_acno"],
                    where=[("glm_cono", "in", self.con), ("glm_acno", ">=",
                    self.rpt_dic["glr_from"]), ("glm_acno", "<=",
                    self.rpt_dic["glr_to"])], group="glm_acno",
                    order="glm_acno")
            if acs:
                for ac in acs:
                    accs.append(self.sql.getRec(tables="genmst",
                    where=[("glm_cono", "in", self.con),
                    ("glm_acno", "=", ac[0])], limit=1))
        return accs

    def lstdicGenmst(self, acc):
        self.gen_lst = acc
        self.gen_dic = {}
        num = 0
        for fld in self.sql.genmst_col:
            self.gen_dic[fld] = acc[num]
            num += 1

    def clearData(self):
        self.o_cyr = 0
        self.mth = [0,0,0,0,0,0,0,0,0,0,0,0,0,0]
        self.lyr = [0,0,0,0,0,0,0,0,0,0,0,0,0,0]
        self.bud = [0,0,0,0,0,0,0,0,0,0,0,0,0,0]
        self.var = [0,0,0,0,0,0,0,0,0,0,0,0,0,0]

    def accumData(self):
        if self.rpt_dic["glr_obal"] == "Y":
            o_lyr, self.o_cyr = self.getObal()
            self.mth[1] = float(ASD(self.mth[1]) + ASD(self.o_cyr))
            self.mth[13] = float(ASD(self.mth[13]) + ASD(self.o_cyr))
            self.lyr[1] = float(ASD(self.lyr[1]) + ASD(o_lyr))
            self.lyr[13] = float(ASD(self.lyr[13]) + ASD(o_lyr))
            self.var[1] = float(ASD(self.var[1]) - ASD(self.o_cyr))
            self.var[13] = float(ASD(self.var[13]) - ASD(self.o_cyr))
        curdt = self.s_per
        trans = []
        for mth in range(1, 13):
            lyr, cyr, bud, trn = self.getMbal(curdt)
            self.mth[mth] = float(ASD(self.mth[mth]) + ASD(cyr))
            self.mth[13] = float(ASD(self.mth[13]) + ASD(cyr))
            self.lyr[mth] = float(ASD(self.lyr[mth]) + ASD(lyr))
            self.lyr[13] = float(ASD(self.lyr[13]) + ASD(lyr))
            self.bud[mth] = float(ASD(self.bud[mth]) + ASD(bud))
            self.bud[13] = float(ASD(self.bud[13]) + ASD(bud))
            self.var[mth] = float(ASD(self.var[mth]) + ASD(bud) - ASD(cyr))
            self.var[13] = float(ASD(self.var[13]) + ASD(bud) - ASD(cyr))
            if not self.row in self.trn[mth]:
                self.trn[mth][self.row] = []
            self.trn[mth][self.row] = trn
            trans.extend(trn)
            y = curdt / 100
            m = (curdt % 100) + 1
            if m > 12:
                y += 1
                m -= 12
            curdt = (y * 100) + m
        return trans

    def getObal(self):
        if not self.con:
            if not self.period[0]:
                lyr = None
            else:
                lyr = self.sql.getRec(tables="genbal",
                    cols=["round(sum(glo_cyr), 2)"], where=[("glo_cono", "=",
                    self.s_cono), ("glo_acno", "=", self.s_acno), ("glo_trdt",
                    "=", self.s_lyr.work)], limit=1)[0]
            cyr = self.sql.getRec(tables="genbal",
                cols=["round(sum(glo_cyr), 2)"], where=[("glo_cono", "=",
                self.s_cono), ("glo_acno", "=", self.s_acno), ("glo_trdt",
                "=", self.period[1][0])], limit=1)[0]
        else:
            if not self.period[0]:
                lyr = None
            else:
                lyr = self.sql.getRec(tables="genbal",
                    cols=["round(sum(glo_cyr), 2)"], where=[("glo_cono", "in",
                    self.con), ("glo_acno", "=", self.s_acno), ("glo_trdt",
                    "=", self.s_lyr.work)], limit=1)[0]
            cyr = self.sql.getRec(tables="genbal",
                cols=["round(sum(glo_cyr), 2)"], where=[("glo_cono", "in",
                self.con), ("glo_acno", "=", self.s_acno), ("glo_trdt",
                "=", self.period[1][0])], limit=1)[0]
        if not lyr:
            lyr = 0
        else:
            lyr = float(lyr)
        if not cyr:
            cyr = 0
        else:
            cyr = float(cyr)
        return (lyr, cyr)

    def getMbal(self, curdt):
        if not self.con:
            lyr = self.sql.getRec(tables="gentrn",
                cols=["round(sum(glt_tramt), 2)"], where=[("glt_cono", "=",
                self.s_cono), ("glt_acno", "=", self.s_acno), ("glt_curdt",
                "=", (curdt - 100))], limit=1)[0]
            cyr = self.sql.getRec(tables="gentrn",
                cols=["round(sum(glt_tramt), 2)"], where=[("glt_cono", "=",
                self.s_cono), ("glt_acno", "=", self.s_acno), ("glt_curdt",
                "=", curdt)], limit=1)[0]
            bud = self.sql.getRec(tables="genbud", cols=["sum(glb_tramt)"],
                where=[("glb_cono", "=", self.s_cono), ("glb_acno", "=",
                self.s_acno), ("glb_curdt", "=", curdt)], limit=1)[0]
            trn = self.sql.getRec(tables="gentrn", where=[("glt_cono", "=",
                self.s_cono), ("glt_acno", "=", self.s_acno), ("glt_curdt",
                "=", curdt)], order="glt_trdt")
        else:
            lyr = self.sql.getRec(tables="gentrn",
                cols=["round(sum(glt_tramt), 2)"], where=[("glt_cono", "in",
                self.con), ("glt_acno", "=", self.s_acno), ("glt_curdt",
                "=", (curdt - 100))], limit=1)[0]
            cyr = self.sql.getRec(tables="gentrn",
                cols=["round(sum(glt_tramt), 2)"], where=[("glt_cono", "in",
                self.con), ("glt_acno", "=", self.s_acno), ("glt_curdt",
                "=", curdt)], limit=1)[0]
            bud = self.sql.getRec(tables="genbud", cols=["sum(glb_tramt)"],
                where=[("glb_cono", "in", self.con), ("glb_acno", "=",
                self.s_acno), ("glb_curdt", "=", curdt)], limit=1)[0]
            trn = self.sql.getRec(tables="gentrn", where=[("glt_cono", "in",
                self.con), ("glt_acno", "=", self.s_acno), ("glt_curdt",
                "=", curdt)], order="glt_trdt")
        if not lyr:
            lyr = 0
        else:
            lyr = float(lyr)
        if not cyr:
            cyr = 0
        else:
            cyr = float(cyr)
        if not bud:
            bud = 0
        else:
            bud = int(bud)
        return (lyr, cyr, bud, trn)

    def printLine(self):
        if self.o_cyr:
            trn = [0, 0, self.period[1][0] / 100, self.period[1][0], 4,
                "O/Balance", "B/Fwd", self.o_cyr, 0, "Opening Balance",
                "", "", 0, 0, 0]
            self.trn[1][self.row].insert(0, trn)
            self.trn[13][self.row].insert(0, trn)
        mth = self.mth[:]
        lyr = self.lyr[:]
        bud = self.bud[:]
        var = self.var[:]
        if not self.s_type == "L":
            self.s_acno = 0
        self.allFields.append([self.s_type, self.s_desc, mth, lyr, bud, var,
            self.stanam, self.s_norm])
        self.row += 1

    def accumTotals(self, acbal):
        for x in range(1, 10):
            if acbal == "A":
                for y in range(0, 14):
                    self.mth_tot[x][y] = \
                        float(ASD(self.mth_tot[x][y]) + ASD(self.mth[y]))
                    self.lyr_tot[x][y] = \
                        float(ASD(self.lyr_tot[x][y]) + ASD(self.lyr[y]))
                    self.bud_tot[x][y] = \
                        float(ASD(self.bud_tot[x][y]) + ASD(self.bud[y]))
                    self.var_tot[x][y] = \
                        float(ASD(self.var_tot[x][y]) + ASD(self.var[y]))
            elif acbal == "S":
                for y in range(0, 14):
                    self.mth_tot[x][y] = \
                        float(ASD(self.mth_tot[x][y]) - ASD(self.mth[y]))
                    self.lyr_tot[x][y] = \
                        float(ASD(self.lyr_tot[x][y]) - ASD(self.lyr[y]))
                    self.bud_tot[x][y] = \
                        float(ASD(self.bud_tot[x][y]) - ASD(self.bud[y]))
                    self.var_tot[x][y] = \
                        float(ASD(self.var_tot[x][y]) - ASD(self.var[y]))

    def storeBalances(self):
        if self.s_acstr == "A":
            for x in range(0, 14):
                self.mth_str[self.s_snum1][x] = \
                    float(ASD(self.mth_str[self.s_snum1][x]) + ASD(self.mth[x]))
                self.lyr_str[self.s_snum1][x] = \
                    float(ASD(self.lyr_str[self.s_snum1][x]) + ASD(self.lyr[x]))
                self.bud_str[self.s_snum1][x] = \
                    float(ASD(self.bud_str[self.s_snum1][x]) + ASD(self.bud[x]))
                self.var_str[self.s_snum1][x] = \
                    float(ASD(self.var_str[self.s_snum1][x]) + ASD(self.var[x]))
        elif self.s_acstr == "S":
            for x in range(0, 14):
                self.mth_str[self.s_snum1][x] = \
                    float(ASD(self.mth_str[self.s_snum1][x]) - ASD(self.mth[x]))
                self.lyr_str[self.s_snum1][x] = \
                    float(ASD(self.lyr_str[self.s_snum1][x]) - ASD(self.lyr[x]))
                self.bud_str[self.s_snum1][x] = \
                    float(ASD(self.bud_str[self.s_snum1][x]) - ASD(self.bud[x]))
                self.var_str[self.s_snum1][x] = \
                    float(ASD(self.var_str[self.s_snum1][x]) - ASD(self.var[x]))

    def nextRepRecord(self):
        self.rpt_dic = {}
        if self.rep_num == len(self.rpt_lst) - 1:
            self.exit = "y"
        else:
            self.rep_num += 1
            num = 0
            for fld in self.sql.genrpt_col:
                self.rpt_dic[fld] = self.rpt_lst[self.rep_num][num]
                num += 1

    def storeRepRecord(self):
        self.s_acno = 0
        for key in self.rpt_dic:
            k = key.split("_")
            exec "self.s_%s = self.rpt_dic[key]" % k[1]

    def doDisplayReport(self):
        if not self.allFields:
            return
        self.pb.mess = "Displaying the Report"
        self.mf.updateStatus("%s, Please Wait ....." % self.pb.mess)
        head = []
        head.append("%03u %-s" % (self.conum, self.conam))
        self.stanam = self.allFields[0][6]
        head.append("%-s for the period %s to %s" % (self.stanam,
            self.period[1][1], self.period[2][1]))
        if not self.con:
            pass
        elif self.con == "Y":
            head.append("Consolidated Companies (All)")
        else:
            con = str(tuple(self.con)).replace("'", "").replace(" ", "")
            head.append("Consolidated Companies (%s)" % con)
        head.append("")
        cols = [[""]]
        m = self.s_per % 100
        for x in range(0, 12):
            cols[0].append([mthnam[m][1], (x*3)+1, (x*3)+3])
            m += 1
            if  m > 12:
                m = 1
        cols[0].append(["Year-Total", 37, 39])
        cols.append(["Description"])
        for x in range(0, 13):
            cols[1].append("%9s " % "Curr-Year")
            cols[1].append("%9s " % "Budget")
            cols[1].append("%9s " % "Variance")
        form = (
            ("NA", 30, False),
            ("SI", 10, True), ("SI", 10, False), ("SI", 10, False, "red"),
            ("SI", 10, True), ("SI", 10, False), ("SI", 10, False, "red"),
            ("SI", 10, True), ("SI", 10, False), ("SI", 10, False, "red"),
            ("SI", 10, True), ("SI", 10, False), ("SI", 10, False, "red"),
            ("SI", 10, True), ("SI", 10, False), ("SI", 10, False, "red"),
            ("SI", 10, True), ("SI", 10, False), ("SI", 10, False, "red"),
            ("SI", 10, True), ("SI", 10, False), ("SI", 10, False, "red"),
            ("SI", 10, True), ("SI", 10, False), ("SI", 10, False, "red"),
            ("SI", 10, True), ("SI", 10, False), ("SI", 10, False, "red"),
            ("SI", 10, True), ("SI", 10, False), ("SI", 10, False, "red"),
            ("SI", 10, True), ("SI", 10, False), ("SI", 10, False, "red"),
            ("SI", 10, True), ("SI", 10, False), ("SI", 10, False, "red"),
            ("SI", 10, True), ("SI", 10, False), ("SI", 10, False, "red"))
        self.data = []
        self.count = 0
        for fld in self.allFields:
            if fld[0] == "H":
                self.doHeading(fld)
            elif fld[0] in ("C","G","L","P","S","T"):
                self.doValues(fld)
            elif fld[0] == "U":
                self.doDisplayUnderline(fld)
            self.count += 1
        width, height = self.mf.geom
        if self.mf.fixed:
            height = int(height * .95)
        self.swin = ScrollGrid(scrn=self.mf.body, width=width, height=height,
            name=__name__, head=head, cols=cols, form=form, data=self.data,
            event=("button_release_event", self.doTrans), butt=[("Periods",
            self.doPeriods)], pbar=self.pb, rcdic=self.mf.rcdic)

    def doPeriods(self, widget=None):
        tit = ("Range of Periods to Display",)
        fld = (
            (("T",0,0,0),"IUI",2,"Starting Period","Starting Period",
                1,"N",self.doStartMth,None,None,None,("between",1,12)),
            (("T",0,1,0),"IUI",2,"Ending Period","Ending Period",
                12,"N",self.doEndMth,None,None,None,("between",1,12)))
        self.sm = TartanDialog(self.mf, newscn=(0,0,1), tops=True,
            title=tit, eflds=fld, tend=((self.doMthEnd, "n"),),
            txit=(self.doMthExit,))
        self.mf.startLoop()

    def doStartMth(self, frt, pag, r, c, p, i, w):
        self.startmth = w

    def doEndMth(self, frt, pag, r, c, p, i, w):
        if w < self.startmth:
            return "Invalid Month, Less Than Starting Month"
        self.endmth = w

    def doMthEnd(self, widget=None):
        self.sm.closeProcess()
        for mth in range(1, 13):
            num = (mth*3) - 2
            if mth >= self.startmth and mth <= self.endmth:
                for x in range(num, num + 3):
                    self.swin.showCol(x)
            else:
                for x in range(num, num + 3):
                    self.swin.hideCol(x)
        self.mf.closeLoop()

    def doMthExit(self, widget=None):
        self.sm.closeProcess()
        self.mf.closeLoop()

    def doHeading(self, line):
        self.data.append(["HEAD", [line[1]] + [""] * 39, 0])

    def doValues(self, line):
        if self.zero == "Y":
            p = False
            for x in range(0, 14):
                if line[2][x] or line[4][x] or line[5][x]:
                    p = True
        else:
            p = True
        if not p:
            return
        typ = line[0]
        if typ == "T":
            data = ["TOTAL", [line[1]]]
        else:
            data = ["BODY", [line[1]]]
        for x in range(1, 14):
            data[1].append(line[2][x])
            data[1].append(line[4][x])
            data[1].append(line[5][x])
        data.append(self.count)
        self.data.append(data)

    def doDisplayUnderline(self, line):
        if line[1] == "Blank":
            self.data.append(["BLANK", [""] * 40, 0])
        elif line[1] == "Single":
            self.data.append(["ULINES", [""] + ["------------ "] * 39, 0])
        elif line[1] == "Double":
            self.data.append(["ULINED", [""] + ["------------ "] * 39, 0])

    def doTrans(self, *args):
        try:
            trans = self.trn[(args[3] / 3) + 1][args[2]]
            if not trans:
                raise Exception
        except:
            return
        tit = "Transactions for Selection"
        gtt = ["gentrn"]
        col = self.sql.gentrn_col
        gtc = (
            ("glt_trdt", "", 0, "   Date"),
            ("glt_refno", "", 0, "Reference", "Y"),
            ("glt_type", ("XX", gltrtp), 3, "Typ"),
            ("glt_batch", "", 0, "Batch"),
            ("glt_tramt", "", 0, "       Debit"),
            ("glt_tramt", "", 0, "      Credit"),
            ("balance", "SD", 13.2, "     Balance"),
            ("glt_desc", "", 0, "Remarks"))
        data = []
        acctot = 0
        for rec in trans:
            acctot = float(ASD(acctot) + ASD(rec[col.index("glt_tramt")]))
            if rec[col.index("glt_tramt")] < 0:
                dr = 0
                cr = rec[col.index("glt_tramt")]
            else:
                dr = rec[col.index("glt_tramt")]
                cr = 0
            data.append([
                rec[col.index("glt_trdt")],
                rec[col.index("glt_refno")],
                rec[col.index("glt_type")],
                rec[col.index("glt_batch")],
                dr, cr, acctot,
                rec[col.index("glt_desc")]])
        rec = SRec(self.mf, screen=None, title=tit, tables=gtt, cols=gtc,
            where=data, wtype="D", sort=False, rows=15)

    def doMainExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()
