#! /usr/bin/python

"""
SYNOPSIS
    General Ledger Key Change.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    is granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN; if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import CCD, TartanDialog, Sql
from tartanFunctions import askQuestion

class gl6010:
    def __init__(self, mf, conum, conam):
        self.mf = mf
        self.conum = conum
        self.conam = conam
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        sql = Sql(self.mf.dbm, "ftable", prog=__name__)
        self.tables = [
            ("assgrp", "asg_cono", "asg_assacc"),
            ("assgrp", "asg_cono", "asg_depacc"),
            ("assgrp", "asg_cono", "asg_expacc"),
            ("crsctl", "ctc_cono", "ctc_bankac"),
            ("crsmst", "crm_cono", "crm_glac"),
            ("ctlctl", "ctl_cono", "ctl_conacc"),
            ("ctldep", "dep_dr_coy", "dep_dr_sal"),
            ("ctldep", "dep_dr_coy", "dep_dr_com"),
            ("ctldep", "dep_cr_coy", "dep_cr_sal"),
            ("ctldep", "dep_cr_coy", "dep_cr_com"),
            ("ctlnot", "not_key"),
            ("ctlvtf", "vtt_cono", "vtt_acno", "vtt_styp"),
            ("genbal", "glo_cono", "glo_acno"),
            ("genbud", "glb_cono", "glb_acno"),
            ("genint", "cti_cono", "cti_acno"),
            ("genjlt", "gjt_cono", "gjt_acno"),
            ("genmst", "glm_cono", "glm_acno"),
            ("genrcc", "grc_cono", "grc_acno"),
            ("genrcc", "grc_acoy", "grc_aacc"),
            ("genrct", "grt_cono", "grt_acno"),
            ("genrpt", "glr_cono", "glr_from"),
            ("genrpt", "glr_cono", "glr_to"),
            ("gentrn", "glt_cono", "glt_acno"),
            ("memctc", "mcc_cono", "mcc_glac"),
            ("rcactl", "cte_cono", "cte_glbnk"),
            ("rtlprm", "rtp_cono", "rtp_rtlacc"),
            ("rtlprm", "rtp_cono", "rtp_incacc"),
            ("strgrp", "gpm_cono", "gpm_sales"),
            ("strgrp", "gpm_cono", "gpm_costs"),
            ("strmf1", "st1_cono", "st1_sls"),
            ("strmf1", "st1_cono", "st1_cos"),
            ("wagedc", "ced_eglco", "ced_eglno"),
            ("wagedc", "ced_rglco", "ced_rglno"),
            ("wagtf2", "wt2_gl_econo", "wt2_gl_eacno"),
            ("wagtf2", "wt2_gl_rcono", "wt2_gl_racno")]
        tabs = []
        tables = self.tables[:]
        for tab in tables:
            chk = sql.getRec(tables="ftable", where=[("ft_tabl", "=", tab[0])])
            if not chk:
                self.tables.remove(tab)
                continue
            if not tabs.count(tab[0]):
                tabs.append(tab[0])
        self.sql = Sql(self.mf.dbm, tabs, prog=__name__)

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "General Ledger Key Changes")
        glm = {
            "stype": "R",
            "tables": ("genmst",),
            "cols": (
                ("glm_acno", "", 0, "Acc-Num"),
                ("glm_desc", "", 0, "Description", "Y")),
            "where": [("glm_cono", "=", self.conum)]}
        fld = (
            (("T",0,0,0),"I@glm_acno",0,"Old Number","Old Account Number",
                "","Y",self.doOldAcc,glm,None,("notzero",)),
            (("T",0,0,0),"O@glm_desc",0,""),
            (("T",0,1,0),"I@glm_acno",0,"New Number","New Account Number",
                "","Y",self.doNewAcc,None,None,("notzero",)))
        tnd = ((self.doEnd,"y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tend=tnd,
            txit=txt)

    def doOldAcc(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="genmst", where=[("glm_cono", "=",
            self.conum), ("glm_acno", "=", w)], limit=1)
        if not acc:
            return "Invalid Account Number, Does Not exist"
        self.oldacc = w
        self.oldnot = str(w)
        self.df.loadEntry(frt, pag, p+1, data=acc[3])

    def doNewAcc(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="genmst", where=[("glm_cono", "=",
            self.conum), ("glm_acno", "=", w)], limit=1)
        if acc:
            return "Invalid Account Number, Already Exists"
        self.newacc = w
        self.newnot = str(w)

    def doEnd(self):
        genrpt = askQuestion(self.mf.body, head="Report Generator",
            mess="Apply this Change to the Report Generator as well?",
            default="no")
        for tab in self.tables:
            if tab[0] == "ctlnot":
                whr = [("not_cono", "=", self.conum), ("not_sys", "=", "GEN"),
                    (tab[1], "=", self.oldnot)]
                dat = [self.newnot]
                col = [tab[1]]
            elif tab[0] == "ctlvtf":
                oldacc = CCD(self.oldacc, "Na", 7).work
                newacc = CCD(self.newacc, "Na", 7).work
                whr = [(tab[1], "=", self.conum), (tab[2], "=", oldacc),
                    (tab[3], "=", "G")]
                dat = [newacc]
                col = [tab[2]]
            elif tab[0] == "genrpt" and genrpt == "yes":
                whr = [(tab[1], "in", (0, self.conum)), (tab[2], "=",
                    self.oldacc)]
                dat = [self.newacc]
                col = [tab[2]]
            else:
                whr = [(tab[1], "=", self.conum), (tab[2], "=", self.oldacc)]
                dat = [self.newacc]
                col = [tab[2]]
            self.sql.updRec(tab[0], where=whr, data=dat, cols=col)
        self.df.focusField("T", 0, 1)

    def doExit(self):
        self.df.closeProcess()
        self.mf.dbm.commitDbase(ask=True)
        self.mf.closeLoop()

