#! /usr/bin/python

"""
SYNOPSIS
    Members Ledger Data Capture (Invoices, Payments and Journals).

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import AgeAll, ASD, Batches, GetCtl, PwdConfirm, Sql
from TartanClasses import TartanDialog
from tartanFunctions import askQuestion, chkCtlAcc, mthendDate, showError
from tartanWork import mltrtp

class ml2010:
    def __init__(self, mf, coy, nam, period, rtn, usr):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.period = period
        self.rtn = rtn
        self.capnm = usr
        no = self.checkControls()
        if not no:
            self.setVariables()
            self.drawDialog()
            self.mf.startLoop()

    def checkControls(self):
        # Check for Valid Posting Routine
        if self.rtn not in (1, 2, 3, 4):
            showError(self.mf.body, "Control Error", "Invalid Routine %s, "\
                "Only 1-4 Are Allowed" % str(self.rtn))
            return "no"
        # Create SQL Object
        self.sql = Sql(self.mf.dbm, ["ctlmst", "ctlvmf", "ctlvtf", "memmst",
            "memtrn", "genint", "genmst", "gentrn"], prog=__name__)
        #
        self.gc = GetCtl(self.mf)
        memctl = self.gc.getCtl("memctl", self.conum)
        if not memctl:
            return "no"
        self.glint = memctl["mcm_glint"]
        if self.glint == "Y":
            ctlctl = self.gc.getCtl("ctlctl", self.conum)
            if not ctlctl:
                return "no"
            ctls = ["mem_ctl", "vat_ctl", "dis_all"]
            if self.gc.chkRec(self.conum, ctlctl, ctls):
                return "no"
            self.memctl = ctlctl["mem_ctl"]
            self.dis_all = ctlctl["dis_all"]
        self.batchHeader()
        if not self.bh.batno:
            return "no"

    def setVariables(self):
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.taxdf = self.gc.getCtl("ctlmst", self.conum)["ctm_taxdf"]
        if self.rtn == 1:
            self.glt = 1
        elif self.rtn == 2:
            self.glt = 2
        elif self.rtn == 3:
            self.glt = 4
        elif self.rtn == 4:
            self.glt = 1

    def batchHeader(self):
        self.bh = Batches(self.mf, self.conum, self.conam, self.period,
            "MEM", self.rtn, glint=self.glint)
        self.bh.doBatchHeader()
        if not self.bh.batno:
            return

    def drawDialog(self):
        tit = ("%03i %s" % (self.conum, self.conam), "Members " + \
            mltrtp[self.rtn - 1][1] + " Data Capture (%s)" % __name__)
        mlm = {
            "stype": "R",
            "tables": ("memmst",),
            "cols": (
                ("mlm_memno", "", 0, "Mem-No"),
                ("mlm_oldno", "", 0, "Old-No"),
                ("mlm_gender", "", 0, "G"),
                ("mlm_state", "", 0, "S"),
                ("mlm_surname", "", 0, "Surname", "Y"),
                ("mlm_names", "", 0, "Names", "F")),
            "where": [("mlm_cono", "=", self.conum)],
            "order": "mlm_surname, mlm_names",
            "sort": False}
        coy = {
            "stype": "R",
            "tables": ("ctlmst",),
            "cols": (
                ("ctm_cono", "", 0, "Coy"),
                ("ctm_name", "", 0, "Name", "Y"))}
        glm = {
            "stype": "R",
            "tables": ("genmst",),
            "cols": (
                ("glm_acno", "", 0, "Acc-Num"),
                ("glm_desc", "", 0, "Description", "Y")),
            "where": [("glm_cono", "=", self.conum)]}
        vtm = {
            "stype": "R",
            "tables": ("ctlvmf",),
            "cols": (
                ("vtm_code", "", 0, "C"),
                ("vtm_desc", "", 0, "Description", "Y")),
            "where": [("vtm_cono", "=", self.conum)]}
        fld = [
            (("T",0,0,0),"OUI",9,"Batch %s Quantity" % self.bh.batno),
            (("T",0,0,0),"OSD",13.2,"Value"),
            (("C",1,0,0),"IUI",6,"Mem-No","Member Number",
                "r","N",self.doMemAcc,mlm,None,("notzero",)),
            [("C",1,0,1),"ONA",10,"Name"],
            (("C",1,0,2),"INa",9,"Reference","Reference Number One",
                "i","N",self.doRefno,None,None,("notblank",)),
            (("C",1,0,3),"ID1",10,"Date","Transaction Date",
                "r","N",self.doTrnDat,None,None,("efld",)),
            (("C",1,0,4),"ISD",13.2,"Amount","Transaction Amount",
                "","N",self.doTrnAmt,None,None,("efld",))]
        if self.rtn == 2:
            fld[3][2] = 20
            fld.extend([
                (("C",1,0,5),"ISD",13.2,"Discount",
                    "Discount Amount","","N",self.doDisAmt,None,None,None),
                (("C",1,0,6),"INA",(20,30),"Details","Transaction Details",
                    "","N",self.doTrnDet,None,None,None)])
        elif self.glint == "Y":
            fld[3][2] = 24
            fld.extend([
                (("C",1,0,5),"INA",30,"Details","Transaction Details",
                    "","N",self.doTrnDet,None,None,None),
                (("T",2,0,0),"OSD",13.2,"Unallocated Balance"),
                (("C",2,0,0),"IUI",7,"Acc-Num","Account Number",
                    "","N",self.doGenAcc,glm,None,None),
                (("C",2,0,1),"ONA",19,"Description"),
                (("C",2,0,2),"IUA",1,"V","V.A.T Code",
                    self.taxdf,"N",self.doVatCode2,vtm,None,("notblank",)),
                (("C",2,0,3),"ISD",13.2,"All-Amt","Allocation Amount",
                    "","N",self.doAllAmt,None,None,("efld",)),
                (("C",2,0,4),"ISD",11.2,"V.A.T","V.A.T Amount",
                    "","N",self.doAllVat,None,None,None),
                (("C",2,0,5),"INA",(26,30),"Details","Details",
                    "","N",self.doAllDet,None,None,("notblank",))])
        else:
            fld[3][2] = 22
            fld.extend([
                (("C",1,0,6),"IUA",1,"V","V.A.T Code",
                    self.taxdf,"N",self.doVatCode1,vtm,None,("notblank",)),
                (("C",1,0,7),"ISD",11.2,"V.A.T","V.A.T Amount",
                    "","N",self.doVatAmt,None,None,None),
                (("C",1,0,8),"INA",(20,30),"Details","Transaction Details",
                    "","N",self.doTrnDet,None,None,None)])
        but = (
            ("Age _Normal",None,self.doAgeNormal,0,None,None,
                "Only Show Unallocated Transactions",1),
            ("Age _History",None,self.doAgeHistory,0,None,None,
                "Show All Transactions, Including Already Allocated",1),
            ("Age _Automatic",None,self.doAgeAuto,0,None,None,
                "Automatically Allocate the Amount Starting With the "\
                "Oldest Unallocated One",1),
            ("Age _Current",None,self.doAgeCurrent,0,None,None,
                "Leave the Transaction Unallocated",1),
            ("Cancel",None,self.doCancel,0,("C",2,1),("C",1,1),"",1))
        tag = [("Transaction", None, None, None)]
        cnd = [(None,"n"), (self.endPage1,"y")]
        cxt = [None, self.exitPage1]
        if self.rtn != 2 and self.glint == "Y":
            tag.append(("Allocation", None, None, None))
            cnd.append((self.endPage2,"y"))
            cxt.append(self.exitPage2)
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tags=tag,
            cend=cnd, cxit=cxt, butt=but)
        self.df.loadEntry("T", 0, 0, data=self.bh.batqty)
        self.df.loadEntry("T", 0, 1, data=self.bh.batval)
        self.df.focusField("C", 1, 1)

    def doMemAcc(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="memmst", cols=["mlm_title", "mlm_initial",
            "mlm_surname"], where=[("mlm_cono", "=", self.conum), ("mlm_memno",
            "=", w)], limit=1)
        if not acc:
            return "Invalid Member Number"
        self.memno = w
        self.name = "%s, %s %s" % (acc[2], acc[0], acc[1])
        self.df.loadEntry("C", pag, p+1, data=self.name)

    def doRefno(self, frt, pag, r, c, p, i, w):
        self.refno = w
        acc = self.sql.getRec(tables="memtrn", cols=["mlt_batch"],
            where=[("mlt_cono", "=", self.conum), ("mlt_memno", "=",
            self.memno), ("mlt_refno", "=", self.refno)])
        if acc:
            return "Transaction Already Exists"

    def doTrnDat(self, frt, pag, r, c, p, i, w):
        if w < self.period[1][0] or w > self.period[2][0]:
            return "Invalid Date, Not in Financial Period"
        if w > mthendDate(self.bh.curdt * 100):
            return "Invalid Date, After Batch Period"
        self.trndat = w

    def doTrnAmt(self, frt, pag, r, c, p, i, w):
        self.trnamt = w
        if self.glint == "Y" or self.rtn == 2:
            self.vatcode = "N"
            self.vatamt = 0

    def doVatCode1(self, frt, pag, r, c, p, i, w):
        vat = self.sql.getRec(tables="ctlvmf", cols=["vtm_new"],
            where=[("vtm_cono", "=", self.conum), ("vtm_code", "=", w)],
            limit=1)
        if not vat:
            return "Invalid V.A.T Code"
        self.vatcode = w
        rate = vat[0]
        self.vatamt = round((self.trnamt * rate / (rate + 100)), 2)
        self.df.loadEntry(frt, pag, p+1, self.vatamt)
        if self.vatamt == 0:
            return "sk"

    def doVatAmt(self, frt, pag, r, c, p, i, w):
        if self.trnamt < 0 and w > 0:
            self.vatamt = float(ASD(0) - ASD(w))
        elif self.trnamt > 0 and w < 0:
            self.vatamt = float(ASD(0) - ASD(w))
        else:
            self.vatamt = w
        self.df.loadEntry(frt, pag, p, data=self.vatamt)

    def doDisAmt(self, frt, pag, r, c, p, i, w):
        if self.trnamt < 0 and w > 0:
            self.disamt = float(ASD(0) - ASD(w))
        elif self.trnamt > 0 and w < 0:
            self.disamt = float(ASD(0) - ASD(w))
        else:
            self.disamt = w
        self.df.loadEntry(frt, pag, p, data=self.disamt)

    def doTrnDet(self, frt, pag, r, c, p, i, w):
        self.trndet = w
        if self.rtn != 2 and self.glint == "Y":
            self.df.colf[2][5][5] = w
        self.allocated = float(0.00)

    def endPage1(self):
        self.cancel = False
        self.agecan = False
        self.batupd = False
        self.updateTables1()
        if self.cancel:
            return
        else:
            self.updateBatch()
            if self.trnamt == 0:
                self.df.advanceLine(1)
            else:
                self.allocated = float(0.0)
                if self.rtn == 2 or self.glint == "N":
                    self.doMemTrn()
                    self.mf.dbm.commitDbase()
                    self.df.selPage("Transaction")
                    self.df.advanceLine(1)
                else:
                    self.df.selPage("Allocation")
                    self.df.loadEntry("T", 2, 0, data=self.trnamt)
                    self.df.focusField("C", 2, 1)

    def updateTables1(self):
        if self.bh.multi == "Y":
            self.curdt = self.trndat / 100
        else:
            self.curdt = self.bh.curdt
        self.amt = self.trnamt
        self.vat = self.vatamt
        if self.rtn == 1:                          # Invoices
            recon = 0
            self.dis = 0.00
        elif self.rtn == 2:                        # Payment
            if self.trnamt == 0:
                recon = self.curdt
            else:
                recon = 0
            self.vat = 0.00
            self.dis = self.disamt
        elif self.rtn == 3:                        # Journal
            recon = 0
            self.dis = 0.00
        elif self.rtn == 4:                        # Journal
            recon = 0
            self.amt = float(ASD(0) - ASD(self.amt))
            self.dis = 0.00
        if self.rtn == 1:
            self.ageloop = False
            self.doAgeCurrent()
        else:
            state = self.df.disableButtons()
            self.mf.updateStatus("Choose an Ageing Option")
            for x in range(0, 4):
                exec "self.df.setWidget(self.df.B%s, 'normal')" % x
            self.ageloop = True
            self.mf.startLoop()
            self.df.enableButtons(state=state)
            if self.agecan:
                self.doCancel()
                return
        if self.glint == "N":
            return
        # General Ledger Control Transaction (Members)
        val = float(ASD(self.amt) + ASD(self.dis))
        data = (self.conum, self.memctl, self.curdt, self.trndat,
            self.glt, self.refno, self.bh.batno, val, 0.00, self.trndet,
            "N", "", recon, self.sysdtw, 0)
        self.sql.insRec("gentrn", data=data)
        # General Ledger Control Transaction (Bank)
        if self.rtn == 2:
            val = float(ASD(0) - ASD(self.amt))
            data = (self.conum, self.bh.acc, self.curdt, self.trndat,
                self.glt, self.refno, self.bh.batno, val, 0.00,
                self.trndet, "N", "", recon, self.sysdtw, 0)
            self.sql.insRec("gentrn", data=data)
            # General Ledger Control Transaction (Discount)
            if self.dis:
                val = float(ASD(0) - ASD(self.dis))
                data = (self.conum, self.dis_all, self.curdt, self.trndat,
                    self.glt, self.refno, self.bh.batno, val, 0.00,
                    self.trndet, "N", "", recon, self.sysdtw, 0)
                self.sql.insRec("gentrn", data=data)

    def doAgeNormal(self, widget=None):
        self.doAgeing("N")

    def doAgeHistory(self, widget=None):
        self.doAgeing("H")

    def doAgeAuto(self, widget=None):
        self.doAgeing("O")

    def doAgeCurrent(self, widget=None):
        self.doAgeing("C")

    def doAgeing(self, atype):
        # Disable All Ageing Buttons
        for x in range(0, 4):
            exec "self.df.setWidget(self.df.B%s, 'disabled')" % x
        self.mf.updateStatus("Select a Transaction to Allocate Against")
        age = AgeAll(self.mf, system="mem", agetyp=atype, agekey=[self.conum,
            self.memno, self.rtn, self.refno, self.curdt, self.amt, self.dis])
        self.agecan = age.cancel
        if self.ageloop:
            self.mf.closeLoop()

    def exitPage1(self):
        self.df.closeProcess()
        self.bh.doBatchTotal()
        self.mf.closeLoop()

    def doGenAcc(self, frt, pag, r, c, p, i, w):
        self.genacc = w
        acc = self.sql.getRec(tables="genmst", cols=["glm_desc", "glm_ind",
            "glm_vat"], where=[("glm_cono", "=", self.conum), ("glm_acno",
            "=", self.genacc)], limit=1)
        if not acc:
            return "Invalid G/L Account"
        if acc[1] == "N":
            cf = PwdConfirm(self.mf, conum=self.conum, system="GEN",
                code="NoPost")
            if not cf.pwd or cf.flag == "no":
                return "Postings Disallowed"
        else:
            chk = chkCtlAcc(self.mf, self.conum, self.genacc)
            if chk:
                return chk
        if not acc[2]:
            self.taxgl = self.taxdf
        else:
            self.taxgl = acc[2]
        self.df.loadEntry("C", pag, p+1, acc[0])
        self.df.loadEntry("C", pag, p+2, data=self.taxgl)

    def doVatCode2(self, frt, pag, r, c, p, i, w):
        ctlctl = self.gc.getCtl("ctlctl", self.conum, error=False)
        if not ctlctl:
            return "Missing ctlctl Record for Company"
        if self.gc.chkRec(self.conum, ctlctl, ["vat_ctl"]):
            return "Missing or Invalid Control Record"
        self.convat = ctlctl["vat_ctl"]
        acc = self.sql.getRec(tables="ctlvmf", cols=["vtm_new"],
            where=[("vtm_cono", "=", self.conum), ("vtm_code", "=", w)],
            limit=1)
        if not acc:
            return "Invalid V.A.T Code"
        self.vatcode = w
        self.vatrat = acc[0]

    def doAllAmt(self, frt, pag, r, c, p, i, w):
        if w == 0:
            allamt = float(ASD(self.trnamt) - ASD(self.allocated))
            self.allamt = round((allamt * 100 / (100 + self.vatrat)), 2)
            self.df.loadEntry(frt, pag, p, data=self.allamt)
            self.allvat = float(ASD(allamt) - ASD(self.allamt))
        else:
            self.allamt = w
            self.allvat = round((self.allamt * self.vatrat / 100), 2)
        self.df.loadEntry(frt, pag, p+1, data=self.allvat)

    def doAllVat(self, frt, pag, r, c, p, i, w):
        if self.allamt < 0 and w > 0:
            self.allvat = float(ASD(0) - ASD(w))
        elif self.allamt > 0 and w < 0:
            self.allvat = float(ASD(0) - ASD(w))
        else:
            self.allvat = w
        self.df.loadEntry(frt, pag, p, data=self.allvat)
        self.df.loadEntry(frt, pag, p+1, data=self.name)

    def doAllDet(self, frt, pag, r, c, p, i, w):
        self.alldet = w

    def doCancel(self, widget=None):
        if self.agecan:
            ok = "yes"
        else:
            ok = askQuestion(self.mf.body, head="Cancel",
                mess="Are You Certain You Want to Cancel This Entry?")
        if ok == "yes":
            self.cancel = True
            self.mf.dbm.rollbackDbase()
            if self.batupd:
                self.updateBatch(rev=True)
            if self.glint == "Y":
                self.df.clearFrame("C", 2)
            row = int((self.df.last[1][1] - 1) / self.df.colq[1])
            col = (row * self.df.colq[1]) + 1
            self.df.selPage("Transaction")
            self.df.focusField("C", 1, col)

    def endPage2(self):
        self.updateTables2()
        self.allocated = float(ASD(self.allocated) + \
            ASD(self.allamt) + ASD(self.allvat))
        if self.allocated == self.trnamt:
            self.doMemTrn()
            self.mf.dbm.commitDbase()
            self.df.clearFrame("C", 2)
            self.df.selPage("Transaction")
            self.df.advanceLine(1)
        else:
            bal = float(ASD(self.trnamt) - ASD(self.allocated))
            self.df.loadEntry("T", 2, 0, data=bal)
            self.df.advanceLine(2)

    def updateTables2(self):
        if self.bh.multi == "Y":
            self.curdt = self.trndat / 100
        else:
            self.curdt = self.bh.curdt
        if self.rtn == 1:                          # Invoices
            self.credit()
        elif self.rtn == 3:                        # Journal Entry
            self.credit()
        elif self.rtn == 4:                        # Credit Notes
            self.debit()

    def debit(self):
        amt = self.allamt
        vat = self.allvat
        self.restDebitCredit(amt, vat)

    def credit(self):
        amt = float(ASD(0) - ASD(self.allamt))
        vat = float(ASD(0) - ASD(self.allvat))
        self.restDebitCredit(amt, vat)

    def restDebitCredit(self, amt, vat):
        # VAT Transaction (ctlvtf)
        data = (self.conum, self.vatcode, "O", self.curdt, "D", self.glt,
            self.bh.batno, self.refno, self.trndat, self.memno, self.alldet,
            amt, vat, self.sysdtw, 0, 0)
        self.sql.insRec("ctlvtf", data=data)
        if self.glint == "N":
            return
        # General Ledger Transaction (Expense)
        data = (self.conum, self.genacc, self.curdt, self.trndat,
            self.glt, self.refno, self.bh.batno, amt, vat, self.alldet,
            self.vatcode, "", 0, self.sysdtw, 0)
        self.sql.insRec("gentrn", data=data)
        if vat:
            # General Ledger Control Transaction (V.A.T.)
            data = (self.conum, self.convat, self.curdt, self.trndat,
                self.glt, self.refno, self.bh.batno, vat, 0.00, self.alldet,
                "N", "", 0, self.sysdtw, 0)
            self.sql.insRec("gentrn", data=data)
        vat = float(ASD(0) - ASD(vat))

    def exitPage2(self):
        self.df.focusField("C", 2, self.df.col)

    def doMemTrn(self):
        # Members Ledger Transaction
        data = [self.conum, self.memno, self.rtn, self.refno, self.bh.batno,
            self.trndat, self.amt, self.vat, self.curdt, "", 0, self.trndet,
            self.vatcode, "N", self.sysdtw, 0]
        self.sql.insRec("memtrn", data=data)
        if self.dis:
            data = [self.conum, self.memno, 6, self.refno, self.bh.batno,
                self.trndat, self.dis, self.vat, self.curdt, "", 0,
                self.trndet, self.vatcode, "N", self.sysdtw, 0]
            self.sql.insRec("memtrn", data=data, unique="mlt_refno")

    def updateBatch(self, rev=False):
        if rev:
            self.bh.batqty = self.bh.batqty - 1
            self.bh.batval = float(ASD(self.bh.batval) - ASD(self.trnamt))
        else:
            self.batupd = True
            self.bh.batqty = self.bh.batqty + 1
            self.bh.batval = float(ASD(self.bh.batval) + ASD(self.trnamt))
        self.df.loadEntry("T", 0, 0, data=self.bh.batqty)
        self.df.loadEntry("T", 0, 1, data=self.bh.batval)
