#! /usr/bin/python

"""
SYNOPSIS
    Member's Ledger Transaction Audit Trail.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, CCD, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, doWriteExport, getModName, doPrinter
from tartanFunctions import showError
from tartanWork import mltrtp

class ml3020:
    def __init__(self, mf, coy, nam, per=None, args=None):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            if not args:
                self.sper = per[1][0] / 100
                self.eper = per[2][0] / 100
                self.mainProcess()
                self.mf.startLoop()
            else:
                self.ptyp, self.sperw, self.eperw, self.btyp, self.batch = args
                self.sperd = CCD(self.sperw, "d2", 7).disp
                self.eperd = CCD(self.eperw, "d2", 7).disp
                self.totsonly = "N"
                self.view = "V"
                self.prtnam = "view"
                self.mail = "N"
                self.whr = [("mlt_cono", "=", self.conum)]
                if self.ptyp == "F":
                    self.whr.append(("mlt_curdt", "between", self.sperw,
                        self.eperw))
                else:
                    self.whr.append(("mlt_capdt", "between", self.sdatw,
                        self.edatw))
                if self.btyp:
                    self.whr.append(("mlt_type", "=", self.btyp))
                self.whr.append(("mlt_batch", "=", self.batch))
                allrec = self.getRecords()
                if allrec:
                    self.printReport(allrec)

    def setVariables(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Member's Ledger Audit Trail (%s)" % __name__)
        self.sql = Sql(self.mf.dbm, ["ctlmst", "memmst", "memtrn"],
            prog=__name__)
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i) %6s" % (t[0],
            t[1], t[2], t[3], t[4], __name__)
        self.head = ("%03u %-30s %s" % (self.conum, self.conam, "%s"))
        self.colsh = ["TP", "BatchNo", "Mem-No", "Name", "Reference",
            "Date", "Debits", "Credits", "Tax-Amount", "T", "Remarks"]
        self.forms = [("UI", 2, False, False, True), ("Na", 7),
            ("UI", 6, False, False, True), ("NA", 30), ("Na", 9), ("D1", 10),
            ("SD", 13.2), ("SD", 13.2), ("SD", 13.2), ("UA", 1), ("NA", 30)]
        self.gqt = [0] * (len(mltrtp) + 1)
        self.gdr = [0] * (len(mltrtp) + 1)
        self.gcr = [0] * (len(mltrtp) + 1)
        self.gvt = [0] * (len(mltrtp) + 1)
        self.totind = "N"
        self.t = None

    def mainProcess(self):
        data = ["All Types"]
        for x in range(1, len(mltrtp)+1):
            data.append(mltrtp[x-1][1])
        btt = {
            "stype": "C",
            "titl": "Valid Types",
            "data": data,
            "retn": "I"}
        btm = {
            "stype": "R",
            "tables": ("memtrn",),
            "cols": (
                ("mlt_batch", "", 0, "Bat-Num"),
                ("mlt_type", ("xx", mltrtp), 20, "Type"),
                ("mlt_curdt", "", 0, "Cur-Dat")),
            "where": [],
            "group": "mlt_batch, mlt_type, mlt_curdt",
            "order": "mlt_type, mlt_curdt, mlt_batch"}
        r1s = (("Financial","F"),("Capture","C"))
        r2s = (("Yes","Y"),("No","N"))
        fld = [
            (("T",0,0,0),("IRB",r1s),0,"Period Type","",
                "F","Y",self.doPtyp,None,None,None),
            (("T",0,1,0),"Id2",7,"Starting Period","",
                self.sper,"Y",self.doStartPer,None,None,("efld",)),
            (("T",0,2,0),"Id2",7,"Ending Period","",
                self.eper,"Y",self.doEndPer,None,None,("efld",)),
            (("T",0,3,0),"Id1",10,"Starting Date","",
                self.sysdtw,"Y",self.doStartDat,None,None,("efld",)),
            (("T",0,4,0),"Id1",10,"Ending Date","",
                self.sysdtw,"Y",self.doEndDat,None,None,("efld",)),
            (("T",0,5,0),"IUI",1,"Type","Transaction Type",
                "","Y",self.doBatTyp,btt,None,None),
            (("T",0,6,0),"INa",7,"Batch Number","",
                "","Y",self.doBatNum,btm,None,None),
            (("T",0,7,0),("IRB",r2s),0,"Totals Only","",
                "Y","Y",self.doTots,None,None,None)]
        tnd = ((self.doEnd,"y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("Y","V"), mail=("Y","N"))

    def doPtyp(self, frt, pag, r, c, p, i, w):
        self.ptyp = w
        if self.ptyp == "C":
            return "sk2"

    def doStartPer(self, frt, pag, r, c, p, i, w):
        if w == 0:
            w = self.sper
        elif w < self.sper or w > self.eper:
            return "Invalid Period"
        else:
            self.sperw = w
        self.sperd = self.df.t_disp[pag][0][i]
        self.df.loadEntry(frt, pag, p, data=self.sperw)

    def doEndPer(self, frt, pag, r, c, p, i, w):
        if w == 0:
            w = self.eper
        elif w < self.sperw or w > self.eper:
            return "Invalid Period"
        else:
            self.eperw = w
        self.eperd = self.df.t_disp[pag][0][i]
        self.df.loadEntry(frt, pag, p, self.eperw)
        return "sk2"

    def doStartDat(self, frt, pag, r, c, p, i, w):
        self.sdatw = w
        self.sperd = self.df.t_disp[pag][0][i]
        self.df.loadEntry(frt, pag, p, data=self.sdatw)

    def doEndDat(self, frt, pag, r, c, p, i, w):
        if w < self.sdatw:
            return "Invalid Date"
        self.edatw = w
        self.eperd = self.df.t_disp[pag][0][i]
        self.df.loadEntry(frt, pag, p, self.edatw)

    def doBatTyp(self, frt, pag, r, c, p, i, w):
        if w > len(mltrtp):
            return "Invalid Batch Type"
        self.btyp = w
        self.whr = [("mlt_cono", "=", self.conum)]
        if self.ptyp == "F":
            self.whr.append(("mlt_curdt", "between", self.sperw, self.eperw))
        else:
            self.whr.append(("mlt_capdt", "between", self.sdatw, self.edatw))
        if self.btyp:
            self.whr.append(("mlt_type", "=", self.btyp))
        self.df.topf[pag][i+1][8]["where"] = self.whr

    def doBatNum(self, frt, pag, r, c, p, i, w):
        self.batch = w
        if self.batch:
            self.whr.append(("mlt_batch", "=", self.batch))

    def doTots(self, frt, pag, r, c, p, i, w):
        self.totsonly = w
        if self.totsonly == "Y":
            self.df.topEntry[0][8][2][0].hide()
            self.df.topEntry[0][8][3][0].hide()
        else:
            self.df.topEntry[0][8][2][0].show()
            self.df.topEntry[0][8][3][0].show()

    def doEnd(self):
        self.df.closeProcess()
        allrec = self.getRecords()
        if allrec:
            self.view = self.df.repprt[0][1]
            self.prtnam = self.df.repprt[0][2]
            self.mail = self.df.repprt[1][2]
            if self.prtnam == "export":
                self.exportReport(allrec)
            else:
                self.printReport(allrec)
        self.closeProcess()

    def getRecords(self):
        col = ["mlt_memno", "mlt_trdt", "mlt_type", "mlt_refno", "mlt_batch",
            "mlt_tramt", "mlt_taxamt", "mlt_desc", "mlt_taxind", "mlm_surname",
            "mlm_names"]
        self.whr.extend([("mlt_cono=mlm_cono",), ("mlt_memno=mlm_memno",)])
        odr = "mlt_type, mlt_batch, mlt_refno"
        allrec = self.sql.getRec(tables=["memmst", "memtrn"], cols=col,
            where=self.whr, order=odr)
        if not allrec:
            showError(self.mf.body, "Transaction Error",
                "No Transactions Selected")
        else:
            return allrec

    def exportReport(self, allrec):
        p = ProgressBar(self.mf.body, max=len(allrec), esc=True)
        expnam = getModName(self.mf.wrkdir, __name__, self.conum)
        self.expheads = [self.head % self.sysdttm]
        self.expheads.append("Member's Ledger Audit Trail for Period "\
            "%s to %s" % (self.sperd, self.eperd))
        self.expcolsh = [self.colsh]
        self.expforms = self.forms
        self.expdatas = []
        for num, dat in enumerate(allrec):
            can = p.displayProgress(num)
            if can == "yes":
                p.closeProgress()
                return
            vals = self.getValues(dat)
            if not vals:
                continue
            memno, trdt, trtp, refno, batch, debit, credit, taxamt, \
                detail, taxind, name = vals
            line = ["BODY", [trtp.work, batch.work, memno.work, name.work,
                refno.work, trdt.work, debit.work, credit.work, taxamt.work,
                taxind.work, detail.work]]
            self.expdatas.append(line)
        p.closeProgress()
        self.grandTotal()
        doWriteExport(xtype=self.view, name=expnam, sheet=__name__,
            heads=self.expheads, colsh=self.expcolsh, forms=self.expforms,
            datas=self.expdatas, rcdic=self.mf.rcdic)

    def printReport(self, allrec):
        p = ProgressBar(self.mf.body, max=len(allrec), esc=True)
        if self.totsonly == "Y":
            self.head = ("%03u %-30s %49s %6s" % \
                (self.conum, self.conam, self.sysdttm, __name__))
        else:
            self.head = ("%03u %-30s %91s %10s" % \
                (self.conum, self.conam, self.sysdttm, __name__))
        pdfnam = getModName(self.mf.wrkdir, __name__, self.conum, ext="pdf")
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, pdfnam, self.head)
        self.bqty = 0
        self.bdrs = 0
        self.bcrs = 0
        self.bvat = 0
        self.tqty = 0
        self.tdrs = 0
        self.tcrs = 0
        self.tvat = 0
        self.trtp = 0
        self.pgnum = 0
        self.pglin = 99
        for num, dat in enumerate(allrec):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            vals = self.getValues(dat)
            if not vals:
                continue
            memno, trdt, trtp, refno, batch, debit, credit, taxamt, \
                detail, taxind, name = vals
            if self.trtp == 0:
                self.trtp = trtp.work
                self.batch = batch.work
            if trtp.work != self.trtp:
                self.batchTotal()
                self.typeTotal()
                self.trtp = trtp.work
                self.batch = batch.work
                self.pglin = 99
            if batch.work != self.batch:
                self.batchTotal()
                self.batch = batch.work
                if self.totsonly != "Y":
                    self.typeHeading()
            if self.pglin > self.lpp:
                if self.pgnum:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
            if self.totsonly != "Y":
                self.t.textLine("%s %s %s %s %s %s %s %s %s" % (memno.disp,
                    name.disp, refno.disp, trdt.disp, debit.disp, credit.disp,
                    taxamt.disp, taxind.disp, detail.disp))
                self.pglin = self.pglin + 1
            # Batch Totals
            self.bqty += 1
            self.bdrs = float(ASD(self.bdrs) + ASD(debit.work))
            self.bcrs = float(ASD(self.bcrs) + ASD(credit.work))
            self.bvat = float(ASD(self.bvat) + ASD(taxamt.work))
            # Type Totals
            self.tqty += 1
            self.tdrs = float(ASD(self.tdrs) + ASD(debit.work))
            self.tcrs = float(ASD(self.tcrs) + ASD(credit.work))
            self.tvat = float(ASD(self.tvat) + ASD(taxamt.work))
        p.closeProgress()
        if self.t:
            if can != "yes":
                self.batchTotal()
                self.typeTotal()
                self.grandTotal()
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=pdfnam, head=self.tit,
            view=self.view, prtnam=self.prtnam, mail=self.mail)

    def getValues(self, data):
        memno = CCD(data[0], "UI", 6)
        trdt = CCD(data[1], "D1", 10)
        trtp = CCD(data[2], "UI", 2)
        refno = CCD(data[3], "Na", 9)
        batch = CCD(data[4], "Na", 7)
        if data[5] < 0:
            debit = CCD(0, "SD", 13.2)
            credit = CCD(data[5], "SD", 13.2)
        else:
            debit = CCD(data[5], "SD", 13.2)
            credit = CCD(0, "SD", 13.2)
        taxamt = CCD(data[6], "SD", 13.2)
        detail = CCD(data[7], "NA", 30)
        taxind = CCD(data[8], "NA", 1)
        name = CCD("%s, %s" % (data[9], data[10]), "NA", 30)
        self.gqt[trtp.work - 1] = self.gqt[trtp.work - 1] + 1
        self.gdr[trtp.work - 1] = float(ASD(self.gdr[trtp.work - 1]) + \
            ASD(debit.work))
        self.gcr[trtp.work - 1] = float(ASD(self.gcr[trtp.work - 1]) + \
            ASD(credit.work))
        self.gvt[trtp.work - 1] = float(ASD(self.gvt[trtp.work - 1]) + \
            ASD(taxamt.work))
        return (memno, trdt, trtp, refno, batch, debit, credit, taxamt,
            detail, taxind, name)

    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        if self.totsonly == "Y":
            self.t.textLine("%-38s %-10s %-2s %-10s %21s %5s" % \
                ("Member's Ledger Audit Trail for Period",
                self.sperd, "to", self.eperd, "Page", self.pgnum))
        else:
            self.t.textLine("%-38s %-10s %-2s %-10s %67s %5s" % \
                ("Member's Ledger Audit Trail for Period",
                self.sperd, "to", self.eperd, "Page", self.pgnum))
        self.t.textLine()
        self.pglin = 4
        if self.totind == "N":
            self.typeHeading()
        else:
            self.t.textLine("%-14s" % "Totals Summary")
            self.t.textLine()
            if self.totsonly == "Y":
                self.t.textLine("%-27s%-8s  %-13s %-13s %-13s %-13s" % \
                    ("Document Type", "Quantity", "      Debits",
                    "     Credits", "  Difference", "      V.A.T."))
            else:
                self.t.textLine("%-34s%-8s  %-13s %-13s %-13s %-13s" % \
                    ("Document Type", "Quantity", "      Debits",
                    "     Credits", "  Difference", "      V.A.T."))
            self.t.textLine("%s" % (self.suc * len(self.head)))
            self.t.setFont("Tartan", self.cpi, self.ldg)
            self.pglin = self.pglin + 4

    def typeHeading(self):
        if self.totsonly != "Y":
            batch = self.batch
        else:
            batch = "Various"
        if self.lpp - self.pglin < 7:
            self.c.drawText(self.t)
            self.c.showPage()
            self.pageHeading()
            return
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.t.textLine("%-7s %7s %-10s %3s" % \
            ("Batch", batch, "    Type", mltrtp[self.trtp - 1][1]))
        self.t.textLine()
        if self.totsonly == "Y":
            self.t.textLine("%-36s %-13s %-13s %-13s %-13s" % \
                ("Details", "      Debits", "     Credits", "  Difference",
                "      V.A.T."))
        else:
            self.t.textLine("%-6s %-30s %-9s %-10s %-13s %-13s %-13s "\
                "%-1s %-30s" % ("Mem-No", "Name", "Reference",
                "   Date", "      Debits", "     Credits", "  Tax-Amount",
                "T", "Remarks"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = self.pglin + 4

    def batchTotal(self):
        j = CCD(self.bdrs, "SD", 13.2)
        k = CCD(self.bcrs, "SD", 13.2)
        l = CCD(float(ASD(j.work) + ASD(k.work)), "SD", 13.2)
        m = CCD(self.bvat, "SD", 13.2)
        if self.totsonly == "Y":
            self.t.textLine("%-36s %13s %13s %13s %13s" % \
                ("Batch " + self.batch + " Totals", j.disp,
                k.disp, l.disp, m.disp))
            self.pglin = self.pglin + 1
        else:
            self.t.textLine()
            self.pglin = self.pglin + 1
            self.t.textLine("%-6s %-51s %13s %13s %13s" % (" ",
                "Batch " + self.batch + " Totals", j.disp, k.disp, m.disp))
            self.t.textLine()
            self.pglin = self.pglin + 1
        self.bqty = 0
        self.bcrs = 0
        self.bdrs = 0
        self.bvat = 0

    def typeTotal(self):
        j = CCD(self.tdrs, "SD", 13.2)
        k = CCD(self.tcrs, "SD", 13.2)
        l = CCD(float(ASD(j.work) + ASD(k.work)), "SD", 13.2)
        m = CCD(self.tvat, "SD", 13.2)
        if self.totsonly == "Y":
            self.t.textLine()
            self.t.textLine("%-36s %13s %13s %13s %13s" % \
                ("Type Totals", j.disp, k.disp, l.disp, m.disp))
        else:
            self.t.textLine("%-6s %-51s %13s %13s %13s" % \
            (" ", "Type-Totals", j.disp, k.disp, m.disp))
        self.t.textLine()
        self.tqty = 0
        self.tcrs = 0
        self.tdrs = 0
        self.tvat = 0
    
    def grandTotal(self):
        tot = [0, 0, 0, 0, 0]
        if self.prtnam == "export":
            for x in range(0, len(mltrtp)):
                tot[0] = float(ASD(tot[0]) + ASD(self.gdr[x]))
                tot[1] = float(ASD(tot[1]) + ASD(self.gcr[x]))
                tot[2] = float(ASD(tot[2]) + ASD(self.gvt[x]))
            self.expdatas.append(["ULINES"])
            self.expdatas.append(["TOTAL", ["", "", "", "", "Grand Totals", "",
                "", tot[0], tot[1], tot[2], ""]])
            self.expdatas.append(["ULINED"])
            return
        self.c.drawText(self.t)
        self.c.showPage()
        self.totind = "Y"
        self.pageHeading()
        for x in range(0, len(mltrtp)):
            qt = CCD(self.gqt[x], "SI", 8)
            dr = CCD(self.gdr[x], "SD", 13.2)
            cr = CCD(self.gcr[x], "SD", 13.2)
            df = float(ASD(dr.work) + ASD(cr.work))
            df = CCD(df, "SD", 13.2)
            vt = CCD(self.gvt[x], "SD", 13.2)
            if self.totsonly == "Y":
                self.t.textLine("%-27s %s %s %s %s %s" % \
                (mltrtp[x][1], qt.disp, dr.disp, cr.disp, df.disp, vt.disp))
            else:
                self.t.textLine("%-34s %s %s %s %s %s" % \
                (mltrtp[x][1], qt.disp, dr.disp, cr.disp, df.disp, vt.disp))
            tot[0] = tot[0] + qt.work
            tot[1] = float(ASD(tot[1]) + ASD(dr.work))
            tot[2] = float(ASD(tot[2]) + ASD(cr.work))
            tot[3] = float(ASD(tot[3]) + ASD(df.work))
            tot[4] = float(ASD(tot[4]) + ASD(vt.work))
        self.t.textLine()
        qt = CCD(tot[0], "SI", 8)
        dr = CCD(tot[1], "SD", 13.2)
        cr = CCD(tot[2], "SD", 13.2)
        df = CCD(tot[3], "SD", 13.2)
        vt = CCD(tot[4], "SD", 13.2)
        if self.totsonly == "Y":
            self.t.textLine("%-27s %s %s %s %s %s" % \
            ("Grand Totals", qt.disp, dr.disp, cr.disp, df.disp, vt.disp))
        else:
            self.t.textLine("%-34s %s %s %s %s %s" % \
            ("Grand Totals", qt.disp, dr.disp, cr.disp, df.disp, vt.disp))
        self.t.textLine()

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

