#! /usr/bin/env python

"""
SYNOPSIS
    Members Ledger Notes Listing.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import CCD, NotesPrint, ProgressBar, RepPrt, Sql
from tartanFunctions import textFormat

class ml3050:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, "memmst", prog=__name__)
        self.notes = NotesPrint(self.mf, self.conum, self.conam, "MEM")
        if not self.notes.data:
            return "no"

    def mainProcess(self):
        data = []
        p = ProgressBar(self.mf.body, typ="Generating the Report",
            max=len(self.notes.data))
        for num, dat in enumerate(self.notes.data):
            can = p.displayProgress(num)
            if can == "yes":
                break
            desc = textFormat(dat[5], width=50)
            for n, d in enumerate(desc):
                if not n:
                    mem = int(dat[2].strip())
                    des = self.sql.getRec(tables="memmst", cols=["mlm_title",
                        "mlm_initial", "mlm_surname"], where=[("mlm_cono", "=",
                        self.conum), ("mlm_memno", "=", mem)], limit=1)
                    data.append([mem, "%s %s %s" % tuple(des), dat[4],
                        CCD(dat[3], "d1", 10).disp, dat[6],
                        CCD(dat[7], "d1", 10).disp, d])
                else:
                    data.append(["", "", "", "", "", "", d])
        p.closeProgress()
        if can != "yes":
            name = __name__
            head = ["Member's Ledger Notes Listing"]
            cols = [
                ["a", "NA",  7, "Acc-Num",     "y"],
                ["b", "NA", 30, "Description", "y"],
                ["c", "NA", 20, "User-Name",   "y"],
                ["d", "NA", 10, "Cap-Date",    "y"],
                ["e", "UA",  1, "F",           "y"],
                ["f", "NA", 10, "Act-Date",    "y"],
                ["g", "NA", 50, "Details",     "y"]]
            RepPrt(self.mf, conum=self.conum, conam=self.conam, name=name,
                tables=data, heads=head, cols=cols, ttype="D",
                view=self.notes.view, prtnam=self.notes.pnam,
                mail=self.notes.mail)

