#! /usr/bin/env python

"""
SYNOPSIS
    Members Ledger Update Request Email.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import CCD, GetCtl, Sql, TartanDialog
from tartanFunctions import sendMail, showError
from tartanWork import countries

class ml3090:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        gc = GetCtl(self.mf)
        memctl = gc.getCtl("memctl", self.conum)
        if not memctl:
            return "no"
        ctlsys = gc.getCtl("ctlsys")
        self.smtp = ctlsys["sys_smtp"]
        if not self.smtp:
            showError(self.mf.body, "SMTP Error",
                "There is NO SMTP Server in the System Record!")
            return "no"
        ctlmst = gc.getCtl("ctlmst", self.conum)
        self.fadd = ctlmst["ctm_email"]
        if not self.fadd:
            showError(self.mf.body, "From Error",
                "There is NO Email Address on the Company Record!")
            return "no"
        self.sql = Sql(self.mf.dbm, ["ctlmst", "memctk", "memmst", "memadd",
            "memkon"], prog=__name__)
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i) %6s" % (t[0],
            t[1], t[2], t[3], t[4], __name__)
        self.head = ("%03u %-30s %s" % (self.conum, self.conam, "%s"))
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Members Ledger Update Request (%s)" % __name__)
        cod = {
            "stype": "R",
            "tables": ("memctc",),
            "cols": (
                ("mcc_code", "", 0, "Cd"),
                ("mcc_desc", "", 0, "Description", "Y")),
            "where": [("mcc_cono", "=", self.conum)],
            "whera": [["T", "mcc_type", 2]],
            "order": "mcc_code",
            "size": (400, 600)}
        mlm = {
            "stype": "R",
            "tables": ("memmst",),
            "cols": (
                ("mlm_memno", "", 0, "Mem-No"),
                ("mlm_oldno", "", 0, "Old-No"),
                ("mlm_gender", "", 0, "G"),
                ("mlm_state", "", 0, "S"),
                ("mlm_surname", "", 0, "Surname", "Y"),
                ("mlm_names", "", 0, "Names", "F")),
            "where": [("mlm_cono", "=", self.conum)],
            "order": "mlm_surname, mlm_names",
            "sort": False}
        r1s = (
            ("All", "A"),
            ("Main","B"),
            ("Sports","C"),
            ("Debentures","D"))
        r3s = (("Yes","Y"),("No","N"))
        r4s = (("Number","N"),("Surname","M"))
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"Category","Category",
                "A","Y",self.doCat,None,None,None),
            (("T",0,1,0),"IUI",2,"Code","Code",
                0,"Y",self.doCod,cod,None,None),
            (("T",0,1,0),"ONA",30,""),
            (("T",0,2,0),"IUI",6,"First Member Number","First Member Number",
                "","Y",self.doAcc1,mlm,None,None),
            (("T",0,3,0),"IUI",6,"Last Member Number","Last Member Number",
                "","Y",self.doAcc2,mlm,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt)

    def doCat(self, frt, pag, r, c, p, i, w):
        if w == "A":
            self.cat = ""
            self.cod = ""
            self.df.loadEntry(frt, pag, p+2, data="All Categories and Codes")
            return "sk2"
        self.cat = w

    def doCod(self, frt, pag, r, c, p, i, w):
        self.cod = w
        if self.cod:
            self.chk = self.sql.getRec(tables="memctc", cols=["mcc_desc"],
                where=[("mcc_cono", "=", self.conum), ("mcc_type", "=",
                self.cat), ("mcc_code", "=", self.cod)], limit=1)
            if not self.chk:
                return "Invalid Category Code"
            self.df.loadEntry(frt, pag, p+1, data=self.chk[0])
        else:
            self.df.loadEntry(frt, pag, p+1, "All Codes")

    def doAcc1(self, frt, pag, r, c, p, i, w):
        self.acc1 = w

    def doAcc2(self, frt, pag, r, c, p, i, w):
        if w:
            if w < self.acc1:
                return "Invalid Last Member < First Member"
            self.acc2 = w
        else:
            self.acc2 = 999999

    def doEnd(self):
        self.df.closeProcess()
        tab = ["memmst"]
        whr = [
            ("mlm_cono", "=", self.conum),
            ("mlm_memno", ">=", self.acc1),
            ("mlm_memno", "<=", self.acc2),
            ("mlm_state", "=", "A")]
        if self.cat:
            tab.append("memcat")
            whr.append(("mlc_cono=mlm_cono",))
            whr.append(("mlc_memno=mlm_memno",))
            whr.append(("mlc_type", "=", self.cat))
            if self.cod:
                whr.append(("mlc_code", "=", self.cod))
        ord = "mlm_memno"
        all = self.sql.getRec(tables=tab, cols=["mlm_memno", "mlm_title",
            "mlm_initial", "mlm_surname", "mlm_names", "mlm_nation",
            "mlm_dob"], where=whr, order=ord)
        if not all:
            showError(self.mf.body, "Selection Error",
                "No Members Selected")
        self.printReport(all)
        self.closeProcess()

    def printReport(self, all):
        ktypes = {}
        ctk = self.sql.getRec(tables="memctk")
        for c in ctk:
            ktypes[c[1]] = c[2]
        for mem in all:
            text = """Dear %s %s %s

Here are your details as stored on the club's database. Please could you be so kind as to check them and let us have any changes by return email so that we can update our records.

%-30s%s
%-30s%s
%-30s%s
%-30s%s""" % (mem[1], mem[2], mem[3], "Membership Number", mem[0], "Names", mem[4], "Nationality", countries[mem[5]][1], "Date of Birth", CCD(mem[6], "D1", 10).disp)
            add = self.sql.getRec(tables="memadd", cols=["mla_type", "mla_add1",
                "mla_add2", "mla_add3", "mla_city", "mla_code"],
                where=[("mla_cono", "=", self.conum), ("mla_memno", "=",
                mem[0])])
            for a in add:
                if a[0] == "P":
                    text = """%s
%-30s%s""" % (text, "Postal Address", a[1])
                else:
                    text = """%s
%-30s%s""" % (text, "Street Address", a[1])
                for n in a[2:]:
                    if n:
                        text = """%s
%-30s%s""" % (text, "", n)
            kon = self.sql.getRec(tables=["memctk", "memkon"],
                cols=["mck_code", "mck_type", "mlk_detail"],
                where=[("mlk_cono", "=", self.conum), ("mlk_memno",
                "=", mem[0]), ("mck_code=mlk_code",)])
            mail = []
            for e in kon:
                if e[0] == 5:
                    ktyp = "E-Mail Address"
                else:
                    ktyp = ktypes[e[1]]
                text = """%s
%-30s%s""" % (text, ktyp, e[2])
                if e[1] == "E" and e[2][:2] != "08":
                    mail.append(e[2])
            text = """%s

With Thanks

Membership Secretary
%s""" % (text, self.conam)
            sendMail(self.smtp, self.fadd, mail, "Member's Details", mess=text)

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

