#! /usr/bin/env python

"""
SYNOPSIS
    Members Ledger Birthday Report.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import GetCtl, RepPrt, Sql, SplashScreen, TartanDialog

class ml3100:
    def __init__(self, mf, conum, conam):
        self.mf = mf
        self.conum = conum
        self.conam = conam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        mc = GetCtl(self.mf)
        memctl = mc.getCtl("memctl", self.conum)
        if not memctl:
           return "no"
        self.sql = Sql(self.mf.dbm, "memmst", prog=__name__)
        t = time.localtime()
        self.frdt = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.todt = self.frdt

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "Member's Birthday Report")
        cod = {
            "stype": "R",
            "tables": ("memctc",),
            "cols": (
                ("mcc_code", "", 0, "Cd"),
                ("mcc_desc", "", 0, "Description", "Y")),
            "where": [("mcc_cono", "=", self.conum)],
            "whera": [["T", "mcc_type", 0]],
            "order": "mcc_code",
            "size": (400, 600)}
        r1s = (
             ("All", "A"),
             ("Main","B"),
             ("Sports","C"),
             ("Debentures","D"))
        r2s = (("Number", "N"), ("Surname", "M"))
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"Category","Category",
                "A","Y",self.doCat,None,None,None),
            (("T",0,1,0),"IUI",2,"Code","Code",
                0,"Y",self.doCod,cod,None,None),
            (("T",0,1,0),"ONA",30,""),
            (("T",0,2,0),("IRB",r2s),0,"Sort Order","Sort Order",
                "N","Y",self.doSort,None,None,None),
            (("T",0,3,0),"ID1",10,"From Date","From Date",
                self.frdt,"Y",self.doDate,None,None,("efld",)),
            (("T",0,4,0),"ID1",10,"To   Date","To   Date",
                self.todt,"Y",self.doDate,None,None,("efld",)))
        tnd = ((self.doEnd,"y"),)
        txt = (self.doExit,)
        self.df = TartanDialog(self.mf, tops=False, title=tit, eflds=fld,
            tend=tnd, txit=txt, view=("Y","V"), mail=("Y","N"))

    def doCat(self, frt, pag, r, c, p, i, w):
        if w == "A":
            self.cat = ""
            self.cod = ""
            self.df.loadEntry(frt, pag, p+2, data="All Categories and Codes")
            return "sk2"
        self.cat = w

    def doCod(self, frt, pag, r, c, p, i, w):
        self.cod = w
        if self.cod:
            self.chk = self.sql.getRec(tables="memctc", cols=["mcc_desc"],
                where=[("mcc_cono", "=", self.conum), ("mcc_type", "=",
                self.cat), ("mcc_code", "=", self.cod)], limit=1)
            if not self.chk:
                return "Invalid Category Code"
            self.df.loadEntry(frt, pag, p+1, data=self.chk[0])
        else:
            self.df.loadEntry(frt, pag, p+1, "All Codes")

    def doSort(self, frt, pag, r, c, p, i, w):
        self.sort = w

    def doDate(self, frt, pag, r, c, p, i, w):
        if i == 4:
            self.frdt = w
        elif w < self.frdt:
            return "Invalid To Date, Before From Date"
        else:
            self.todt = w

    def doEnd(self):
        self.df.closeProcess()
        tab = ["memmst"]
        col = ["mlm_memno", "mlm_surname", "mlm_names", "mlm_dob"]
        whr = [
            ("mlm_cono", "=", self.conum),
            ("mlm_state", "=", "A"),
            ("mlm_dob % 10000", "between", self.frdt%10000, self.todt%10000)]
        if self.cat:
            tab.append("memcat")
            whr.append(("mlc_cono=mlm_cono",))
            whr.append(("mlc_memno=mlm_memno",))
            whr.append(("mlc_type", "=", self.cat))
            if self.cod:
                whr.append(("mlc_code", "=", self.cod))
        if self.sort == "N":
            ord = "mlm_memno"
        else:
            ord = "mlm_surname"
        sp = SplashScreen(self.mf.body, "Preparing Report ... Please Wait")
        all = self.sql.getRec(tables=tab, cols=col, where=whr, order=ord)
        sp.closeSplash()
        if not all:
            self.mf.closeLoop()
            return
        name = __name__
        head = ["Members Birthday Report"]
        cols = [
            ["a", "NA",  7, "Mem-No", "y"],
            ["b", "NA", 50, "Surame", "y"],
            ["c", "NA", 50, "Names",  "y"],
            ["d", "D1", 10, "Birthday", "y"]]
        RepPrt(self.mf, conum=self.conum, conam=self.conam, name=name,
            tables=all, heads=head, cols=cols, ttype="D",
            view=self.df.repprt[0][1], prtnam=self.df.repprt[0][2],
            mail=self.df.repprt[1][2])
        self.mf.closeLoop()

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()

if __name__ == "__main__":
    from TartanClasses import MainFrame
    mf = MainFrame(fixed=False)
    mf.dbm.openDbase()
    rpt = ml3100(mf, 1, "Company One")
