#! /usr/bin/python

"""
SYNOPSIS
    Members Ledger Change Transaction Allocations.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import AgeTrans, Sql, TartanDialog
from tartanFunctions import getTrn

class ml6010:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.dataHeader()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["memmst", "memtrn", "memage"],
            prog=__name__)
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.curdt = self.sysdtw / 100

    def dataHeader(self):
        tit = ("%03i %s" % (self.conum, self.conam),
                "Members Change Allocations (%s)" % __name__)
        mlm = {
            "stype": "R",
            "tables": ("memmst",),
            "cols": (
                ("mlm_memno", "", 0, "Mem-No"),
                ("mlm_oldno", "", 0, "Old-No"),
                ("mlm_gender", "", 0, "G"),
                ("mlm_state", "", 0, "S"),
                ("mlm_surname", "", 0, "Surname", "Y"),
                ("mlm_names", "", 0, "Names", "F")),
            "where": [("mlm_cono", "=", self.conum)],
            "order": "mlm_surname, mlm_names",
            "sort": False}
        fld = (
            (("T",0,0,0),"IUI",6,"Mem-No","Member Number",
                "","N",self.doMemNo,mlm,None,("notblank",)),
            (("T",0,0,0),"ONA",30,"Member"))
        tnd = ((self.endTop, "n"),)
        txt = (self.exitTop,)
        self.but = (
            ("Normal", None, self.doReAgeNormal, 0, None, None,
                "Only Show Unallocated Transactions"),
            ("History", None, self.doReAgeHistory, 0, None, None,
                "Show All Transactions, Including Already Allocated"))
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tend=tnd,
            txit=txt, butt=self.but)

    def doMemNo(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="memmst", cols=["mlm_title", "mlm_initial",
            "mlm_surname"], where=[("mlm_cono", "=", self.conum), ("mlm_memno",
            "=", w)], limit=1)
        if not acc:
            return "Invalid Member Number"
        self.memno = w
        self.key = None
        name = "%s, %s %s" % (acc[2], acc[0], acc[1])
        self.df.loadEntry(frt, pag, p+1, data=name)
        self.mf.mess.set_text("Select Routine")
        for b in range(2):
            exec "self.df.setWidget(self.df.B%s, 'normal')" % b
        self.mf.startLoop()
        if self.key in ("normal", "history"):
            return "nd"
        elif self.key == "cancel":
            return "nc"

    def doCancel(self, widget=None):
        if self.key == "move":
            self.mf.dbm.rollbackDbase()
            self.df.focusField("T", 0, 1)
        else:
            self.key = "cancel"
            self.mf.closeLoop()

    def doReAgeNormal(self, widget=None):
        self.key = "normal"
        self.doReAge()

    def doReAgeHistory(self, widget=None):
        self.key = "history"
        self.doReAge()

    def doReAge(self):
        self.mf.updateStatus("")
        for b in range(2):
            exec "self.df.setWidget(self.df.B%s, 'disabled')" % b
        col = ["mlt_trdt","mlt_refno","mlt_type","mlt_tramt","paid","balance"]
        whr = [("mlt_cono", "=", self.conum), ("mlt_memno", "=", self.memno)]
        if self.key == "normal":
            dtc, all = getTrn(self.mf.dbm, "mem", whr=whr, zer="N")
        else:
            dtc, all = getTrn(self.mf.dbm, "mem", whr=whr)
        if all:
            data = []
            for rec in all:
                dat = []
                for c in col:
                    dat.append(rec[dtc.index(c)])
                data.append(dat)
            age = AgeTrans(self.mf, "mem", data, 0, exit=False)
            if not age.ccl and age.data:
                if age.total.work:
                    totl = age.total.work
                    while not age.ccl:
                        age.doAllocate()
                        if age.ccl or not age.total.work:
                            break
                if age.ccl:
                    self.key = "cancel"
                    age.data = []
                for tr in age.data:
                    if tr[6]:
                        w = whr[:]
                        w.extend([("mlt_type", "=", tr[2]),
                            ("mlt_refno", "=", tr[1])])
                        self.sql.insRec("memage", data=[self.conum, self.memno,
                            tr[2], tr[1], self.curdt, tr[2], tr[1], tr[6], 0])
            else:
                self.key = "cancel"
        self.mf.closeLoop()

    def endTop(self):
        self.df.focusField("T", 0, 1)

    def exitTop(self):
        self.mf.dbm.commitDbase(ask=True)
        self.df.closeProcess()
        self.mf.closeLoop()

def runAll(rc=True):
    from TartanClasses import MainFrame
    if rc:
        from tartanFunctions import loadRcFile
        rc = loadRcFile(rcfile="C:\\Tartan\\memberrc")
        mf = MainFrame(rcdic=rc, display=False)
    else:
        mf = MainFrame(display=False)
    mf.dbm.openDbase()
    ml6010(mf, 1, "WPCC", all=True)
    mf.dbm.commitDbase()
    mf.dbm.closeDbase()

if __name__ == "__main__":
    runAll(rc=False)
