#! /usr/bin/python

"""
SYNOPSIS
    Members Ledger Synchronise Access Control System.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import Dbase, GetCtl, ProgressBar, Sql

class ml6020:
    def __init__(self, mf, conum, conam):
        self.mf = mf
        self.conum = conum
        self.conam = conam
        no = self.setVariables()
        if not no:
            self.mainProcess()

    def setVariables(self):
        mc = GetCtl(self.mf)
        memctl = mc.getCtl("memctl", self.conum)
        if not memctl or not memctl["mcm_acs"] == "Y":
            return "no"
        try:
            self.dba = Dbase(rcdic={
                "dbase": "FBird",
                "dbhost": "%s" % str(memctl["mcm_ahst"]),
                "dbname": "%s" % str(memctl["mcm_adbn"]),
                "dbuser": "%s" % str(memctl["mcm_ausr"]),
                "dbpwd": "%s" % str(memctl["mcm_apwd"]),
                "dbdir": "",
                "wrkdir": self.mf.wrkdir})
            self.dba.openDbase()
            self.sqa = Sql(self.dba, prog=__name__)
        except:
            return "no"
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sql = Sql(self.mf.dbm, ["memmst"], prog=__name__)

    def mainProcess(self):
        all = self.sql.getRec(tables="memmst", where=[("mlm_cono", "=",
            self.conum)], order="mlm_memno")
        p = ProgressBar(self.mf.body, typ="F", max=len(all), esc=True)
        can = None
        for num, mem in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                break
            memn = mem[self.sql.memmst_col.index("mlm_memno")]
            titl = mem[self.sql.memmst_col.index("mlm_title")]
            snam = mem[self.sql.memmst_col.index("mlm_surname")]
            fnam = mem[self.sql.memmst_col.index("mlm_names")].split()
            if not fnam:
                fnam = ["", ""]
            elif len(fnam) == 1:
                fnam.append("")
            gend = mem[self.sql.memmst_col.index("mlm_gender")]
            stat = mem[self.sql.memmst_col.index("mlm_state")]
            date = mem[self.sql.memmst_col.index("mlm_sdate")]
            seq = self.sqa.sqlRec(("Select mst_sq from employee where "\
                "emp_employeeno=?", (str(memn),)), limit=1)
            if not seq and stat == "A":
                cnt = self.sqa.sqlRec("Select max(mst_sq) from master", limit=1)
                if cnt:
                    cnt = cnt[0]
                else:
                    cnt = 0
                chk = True
                while chk:
                    cnt += 1
                    chk = self.sqa.sqlRec(("Select * from master where "\
                        "mst_id=?", (str(cnt),)), limit=1)
                self.sqa.sqlRec(("Insert into master values (?, ?, ?, ?, "\
                    "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)", (cnt, titl, fnam[0],
                    fnam[1], snam, "", str(cnt), gend, 0, 0, 1, "01000000",
                    None, None, 1)))
                try:
                    self.sqa.sqlRec(("Delete from employee where mst_sq=? "\
                        "or emp_employeeno=?", (cnt, str(memn))))
                except:
                    pass
                self.sqa.sqlRec(("Insert into employee values(?, ?, ?, "\
                    "?, ?, ?)", (cnt, str(memn), "", "", 1, "01000000")))
            elif seq:
                self.sqa.sqlRec(("Update master set mst_title=?, "\
                    "mst_firstname=?, mst_middlename=?, mst_lastname=?, "\
                    "mst_gender=? where mst_sq=?", (titl, fnam[0], fnam[1],
                    snam, gend, seq[0])))
            tag = self.sqa.sqlRec(("Select a.tag_code from tag a, employee b "\
                "where b.emp_employeeno=? and b.mst_sq = a.mst_sq",
                (str(memn),)), limit=1)
            if tag:
                if stat == "A":
                    self.sqa.sqlRec(("Update tag set tag_suspend=?, "\
                        "tag_blacklist=?, tag_expiry=?, tag_modified=? "\
                        "where tag_code=?", (0, 0, 0, 1, tag[0])))
                else:
                    if not date:
                        sdate = self.sysdtw
                    elif date > self.sysdtw:
                        sdate = self.sysdtw
                    else:
                        sdate = date
                    self.sqa.sqlRec(("Update tag set tag_suspend=?, "\
                        "tag_blacklist=?, tag_expiry=?, tag_modified=? "\
                        "where tag_code=?", (1, 0, sdate, 1, tag[0])))
                self.sqa.sqlRec(("Update tag_t_a_g set ttag_modified=? "\
                    "where tag_code=?", (1, tag[0])))
        p.closeProgress()
        if can and can == "yes":
            self.dba.rollbackDbase()
        else:
            self.dba.commitDbase(ask=True)
