#! /usr/bin/env python

"""
SYNOPSIS
    Members Control File Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import Dbase, Sql, TartanDialog

class mlc110:
    def __init__(self, mf, coy, nam, usr):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.user = usr
        no = self.setVariables()
        if not no:
            self.drawDialog()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["memctl", "chglog"], prog=__name__)
        if self.sql.error:
            return "no"
        self.acc = self.sql.getRec(tables="memctl", where=[("mcm_cono", "=",
            self.conum)], limit=1)
        if not self.acc:
            self.new = True
            self.acc = [self.conum, "N", 15, 0, "", "N","","","",""]
        else:
            self.new = False

    def drawDialog(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "Members Ledger Control File Maintenance (%s)" % __name__)
        dir = {
            "stype":  "F",
            "types":  "dir",
            "initd":  ""}
        r1s = (("Yes","Y"),("No","N"))
        self.fld = (
            (("T",0,0,0),("IRB",r1s),0,"G/L Integration","G/L Integration",
                self.acc[1],"N",None,None,None,None),
            (("T",0,1,0),"IUI",2,"Bonus Days","Bonus Days",
                self.acc[2],"N",None,None,None,("between",0,30)),
            (("T",0,2,0),"ID1",10,"Last Month End","Last Month End",
                self.acc[3],"N",None,None,None,("efld",)),
            (("T",0,3,0),"IFF",50,"Photo's Path","Photo's Path",
                self.acc[4],"N",None,dir,None,("dir",),None,
                "The Directory where the Photographs are Situated."),
            (("T",0,4,0),("IRB",r1s),0,"Access Control","Access Control",
                self.acc[5],"N",self.doAccess,None,None,None),
            (("T",0,5,0),"INA",20,"Access Host","Access Host",
                self.acc[6],"N",None,None,None,None),
            (("T",0,6,0),"ITX",50,"Access Database","Access Database",
                self.acc[7],"N",None,None,None,None),
            (("T",0,7,0),"INA",20,"Access Username","Access Username",
                self.acc[8],"N",None,None,None,None),
            (("T",0,8,0),"IHA",20,"Access Password","Access Password",
                self.acc[9],"N",None,None,None,None))
        but = (("Quit",None,self.doExit,1,None,None),)
        tnd = ((self.doEnd,"Y"),)
        txt = (self.doExit,)
        self.df = TartanDialog(self.mf, title=tit, eflds=self.fld, butt=but,
            tend=tnd, txit=txt)
        if not self.new:
            for n, f in enumerate(self.acc[1:]):
                self.df.loadEntry("T", 0, n, data=f)

    def doAccess(self, frt, pag, r, c, p, i, w):
        self.acs = w
        if self.acs == "N":
            self.df.loadEntry(frt, pag, p+1, data="")
            self.df.loadEntry(frt, pag, p+2, data="")
            self.df.loadEntry(frt, pag, p+3, data="")
            self.df.loadEntry(frt, pag, p+4, data="")
            return "nd"

    def doEnd(self):
        if self.acs == "Y":
            try:
                dbf = Dbase(rcdic={
                    "dbase": "FBird",
                    "dbhost": "%s" % self.df.t_work[0][0][5],
                    "dbname": "%s" % self.df.t_work[0][0][6],
                    "dbuser": "%s" % self.df.t_work[0][0][7],
                    "dbpwd": "%s" % self.df.t_work[0][0][8],
                    "dbdir": "",
                    "wrkdir": self.mf.wrkdir})
                if not dbf.checkDbase():
                    raise Exception
            except:
                self.df.focusField("T", 0, 5, err="Failed to Connect to Server")
                return
        data = [self.conum]
        for x in range(0, len(self.df.t_work[0][0])):
            data.append(self.df.t_work[0][0][x])
        if self.new:
            self.sql.insRec("memctl", data=data)
        elif data != self.acc:
            acc = self.sql.updRec("memctl", data=data,
                where=[("mcm_cono", "=", self.conum)])
            dte = long("%04i%02i%02i%02i%02i%02i" % time.localtime()[:-3])
            for num, dat in enumerate(self.acc):
                if dat != data[num]:
                    self.sql.insRec("chglog", data=["memctl", "U",
                        "%03i" % self.conum, self.sql.memctl_col[num],
                        dte, self.user, str(dat), str(data[num]), 0])
        self.mf.dbm.commitDbase()
        self.doExit()

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()
