#! /usr/bin/env python

"""
SYNOPSIS
    Members Messages File Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import Sql, TartanDialog
from tartanFunctions import textFormat

class mlc310:
    def __init__(self, mf):
        self.mf = mf
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, "ctlmes", prog=__name__)
        self.system = "MEM"

    def mainProcess(self):
        tit = ("%s" % "Members Messages File Maintenance (%s)" % __name__)
        mss = {
            "stype": "R",
            "tables": ("ctlmes",),
            "cols": (
                ("mss_message", "", 0, "Mss"),
                ("mss_detail", "NA", 50, "Details")),
            "where": [("mss_system", "=", self.system)],
            "order": "mss_message"}
        self.fld = [
            [("T",0,0,0),"IUI",3,"Message Number","Message Number",
                "","N",self.doMes,mss,None,("notzero",)],
            [("T",0,1,0),"ITv",(30,5),"Details","Details",
                "","N",self.doDetail,None,self.doDelete,("chars",150)],
            [("T",0,2,0),"ONA",30,"Sample Line 1"],
            [("T",0,3,0),"ONA",30,"       Line 2"],
            [("T",0,4,0),"ONA",30,"       Line 3"],
            [("T",0,5,0),"ONA",30,"       Line 4"],
            [("T",0,6,0),"ONA",30,"       Line 5"]]
        but = (
            ("Accept",None,self.doAccept,0,("T",0,3),("T",0,0)),
            ("Cancel",None,self.doCancel,0,("T",0,3),("T",0,0)),
            ("Quit",None,self.doExit,1,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=tit, eflds=self.fld, butt=but,
            tend=tnd, txit=txt)

    def doMes(self, frt, pag, r, c, p, i, w):
        self.message = w
        acc = self.sql.getRec(tables="ctlmes", cols=["mss_detail"],
            where=[("mss_system", "=", self.system), ("mss_message", "=",
            self.message)], limit=1)
        if not acc:
            self.new = "Y"
            self.df.loadEntry(frt, pag, p+1, data="")
            for x in range(0, 5):
                self.df.loadEntry("T", 0, 2+x, data="")
        else:
            self.new = "N"
            self.detail = acc[0]
            self.df.loadEntry(frt, pag, p+1, data=self.detail)
            self.splitMess(acc[0])

    def doDetail(self, frt, pag, r, c, p, i, w):
        self.detail = w
        self.splitMess(self.detail)

    def splitMess(self, mess):
        mes1 = textFormat(mess, width=30)
        if len(mes1) > 5:
            l = 5
        else:
            l = len(mes1)
        for x in range(0, l):
            self.df.loadEntry("T", 0, 2+x, data=mes1[x])

    def doDelete(self):
        self.sql.delRec("ctlmes", where=[("mss_system", "=", self.system),
            ("mss_message", "=", self.message)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doEnd(self):
        data = [self.system, self.message, self.detail]
        if self.new == "Y":
            self.sql.insRec("ctlmes", data=data)
        else:
            self.sql.updRec("ctlmes", data=data, where=[("mss_system", "=",
            self.system), ("mss_message", "=", self.message)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doAccept(self, widget=None):
        frt, pag, col, mes = self.df.doCheckFields()
        if mes:
            self.df.focusField(frt, pag, (col+1), err=mes)
        else:
            self.df.doEndFrame("T", 0, cnf="N")

    def doCancel(self, widget=None):
        self.mf.dbm.rollbackDbase()
        self.df.focusField("T", 0, 1)

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

