#! /usr/bin/python

"""
SYNOPSIS
    Create and maintain company records.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import os, time
from TartanClasses import TartanDialog, ShowImage, Sql, TabPrt
from tartanFunctions import askQuestion, showError

class ms1010:
    def __init__(self, mf, usr):
        self.mf = mf
        self.user = usr
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlmst", "ctlynd", "ctlvmf", "chglog"],
            prog=__name__)
        self.sys = ["GL","CR","DR","ST","SI","AR","RC","RT","WG","SL","ML","BC"]
        self.img = None

    def mainProcess(self):
        self.tit = ("Company Records File Maintenance",)
        ctm = {
            "stype": "R",
            "tables": ("ctlmst",),
            "cols": (
                ("ctm_cono", "", 0, "Coy"),
                ("ctm_name", "", 0, "Name", "Y"))}
        log = {
            "stype": "F",
            "types": "fle",
            "ftype": (("Image", "*"),)}
        r1s = (("Yes", "Y"), ("No", "N"))
        tag = (
            ("General",None,("T",1,2),("T",1,1)),
            ("Banking",None,("T",1,2),("T",1,1)),
            ("Systems",None,("T",1,2),("T",1,1)),
            ("Logo",None,("T",1,2),("T",1,1)))
        fld = (
            (("T",1,0,0),"IUI",3,"Company Number","Number",
                "","Y",self.doCoyNum,ctm,None,("notzero",)),
            (("T",1,1,0),"INA",30,"Name","Company Name",
                "","N",self.doCoyNam,None,self.doDelete,("notblank",)),
            (("T",1,2,0),"INA",30,"Postal Address Line-1","Address Line-1",
                "","N",None,None,None,None),
            (("T",1,3,0),"INA",30,"               Line-2","Address Line-2",
                "","N",None,None,None,None),
            (("T",1,4,0),"INA",30,"               Line-3","Address Line-3",
                "","N",None,None,None,None),
            (("T",1,5,0),"INA",4,"Postal Code","",
                "","N",None,None,None,None),
            (("T",1,6,0),"INA",30,"Street Address Line-1",
                "Street Address Line-1","","N",None,None,None,None),
            (("T",1,7,0),"INA",30,"               Line-2","Address Line-2",
                "","N",None,None,None,None),
            (("T",1,8,0),"INA",30,"               Line-3","Address Line-3",
                "","N",None,None,None,None),
            (("T",1,9,0),"INA",4,"Street Code","",
                "","N",None,None,None,None),
            (("T",1,10,0),"INA",30,"Contact Name","",
                "","N",None,None,None,None),
            (("T",1,11,0),"INA",15,"Telephone","Telephone No",
                "","N",None,None,None,None),
            (("T",1,12,0),"INA",15,"Facsimile","Facsimile No",
                "","N",None,None,None,None),
            (("T",1,13,0),"INA",15,"Mobile","Mobile Number",
                "","N",None,None,None,None),
            (("T",1,14,0),"ITX",50,"E-Mail Address","",
                "","N",None,None,None,("email",)),
            (("T",1,15,0),"INA",20,"Registration No","",
                "","N",None,None,None,None),
            (("T",1,16,0),"INA",20,"V.A.T. Number","",
                "","N",None,None,None,None),
            (("T",1,17,0),"IUA",1,"V.A.T. Default","",
                "","N",self.doVat,None,None,("notblank",)),
            (("T",2,0,0),"INA",30,"Bank Name","",
                "","N",None,None,None,None),
            (("T",2,1,0),"INA",30,"Bank Branch","",
                "","N",None,None,None,None),
            (("T",2,2,0),"INA",8,"Bank IBT","Bank IBT Number",
                "","N",None,None,None,None),
            (("T",2,3,0),"INA",16,"Bank Account","Bank Account Number",
                "","N",None,None,None,None),
            (("T",3,0,0),("IRB",r1s),0,"General Ledger","",
                "N","N",None,None,None,None),
            (("T",3,1,0),("IRB",r1s),0,"Creditor's Ledger","",
                "N","N",None,None,None,None),
            (("T",3,2,0),("IRB",r1s),0,"Debtor's Ledger","",
                "N","N",None,None,None,None),
            (("T",3,3,0),("IRB",r1s),0,"Store's Ledger","",
                "N","N",None,None,None,None),
            (("T",3,4,0),("IRB",r1s),0,"Sales Invoicing","",
                "N","N",None,None,None,None),
            (("T",3,5,0),("IRB",r1s),0,"Asset's Register","",
                "N","N",None,None,None,None),
            (("T",3,6,0),("IRB",r1s),0,"Rent Collections","",
                "N","N",self.doSys,None,None,None),
            (("T",3,7,0),("IRB",r1s),0,"Rental's Ledger","",
                "N","N",None,None,None,None),
            (("T",3,8,0),("IRB",r1s),0,"Salaries & Wages","",
                "N","N",None,None,None,None),
            (("T",3,9,0),("IRB",r1s),0,"Staff Loans","",
                "N","N",self.doSys,None,None,None),
            (("T",3,10,0),("IRB",r1s),0,"Member's Ledger","",
                "N","N",None,None,None,None),
            (("T",3,11,0),("IRB",r1s),0,"Bowling Club","",
                "N","N",None,None,None,None),
            (("T",4,0,0),"IFF",75,"Letterhead Image","",
                "","N",self.doLogo,log,None,("fle","blank")),
            (("T",4,1,0),("IRB",r1s),0,"Flip Logo","",
                "N","N",self.doFlip,None,None,None))
        but = (
            ("Accept",None,self.doAccept,0,("T",1,2),("T",2,0)),
            ("Print",None,self.doPrint,0,("T",1,2),("T",2,0)),
            ("Cancel",None,self.doCancel,0,("T",1,2),("T",2,0)),
            ("Quit",None,self.doExit1,1,None,None))
        tnd = (None, (self.doEnd, "n"), (self.doEnd, "n"), (self.doEnd, "n"),
            (self.doEnd, "y"))
        txt = (None, self.doExit1, self.doExit2, self.doExit3, self.doExit4)
        self.df = TartanDialog(self.mf, screen=self.mf.body, title=self.tit,
            tags=tag, eflds=fld, butt=but, tend=tnd, txit=txt,
            clicks=self.doClick)

    def doClick(self, *opts):
        if self.df.pag == 0:
            return
        if self.click:
            self.df.loadEntry("T", self.click[0], self.click[1],
                data=self.df.t_work[self.click[0]][0][self.click[1]])
        self.click = opts[2]
        self.df.focusField("T", opts[2][0], opts[2][1] + 1)

    def doCoyNum(self, frt, pag, r, c, p, i, w):
        if w == 0:
            return "Invalid Company Number"
        self.conum = w
        self.click = []
        self.old = self.sql.getRec(tables="ctlmst", where=[("ctm_cono", "=",
            self.conum)], limit=1)
        if not self.old:
            self.new = True
            if self.conum != 1:
                chk = self.sql.getRec(tables="ctlmst", where=[("ctm_cono",
                    "=", 1)], limit=1)
                if not chk:
                    return "Missing Company Number 1"
        else:
            self.new = False
            for x in range(1, self.df.topq[pag]):
                data = self.old[x]
                self.df.loadEntry(frt, pag, x, data=data)
            for x in range(0, self.df.topq[pag+1]):
                data = self.old[x+self.df.topq[pag]]
                self.df.loadEntry(frt, pag+1, x, data=data)
            for x in range(0, self.df.topq[pag+2]):
                self.df.loadEntry(frt, pag+2, x, data="N")
            for x in range(0, 20, 2):
                if self.old[22][x:x+2]:
                    idx = self.sys.index(self.old[22][x:x+2])
                    self.df.loadEntry(frt, pag+2, idx, data="Y")
            self.df.loadEntry(frt, pag+3, 0, data=self.old[-2])
            self.df.loadEntry(frt, pag+3, 1, data=self.old[-1])
            if self.old[-2]:
                self.logo = self.old[-2]
                self.displayLogo(self.old[-2], self.old[-1])

    def doDelete(self):
        err = False
        for tab in (
                ("assmst", "asm_cono"),
                ("crsmst", "crm_cono"),
                ("drsmst", "drm_cono"),
                ("genmst", "glm_cono"),
                ("memmst", "mlm_cono"),
                ("rtlmst", "rtm_cono"),
                ("strmf1", "st1_cono"),
                ("wagmst", "wgm_cono")):
            sql = Sql(self.mf.dbm, tab[0], prog=__name__)
            err = sql.getRec(tab[0], where=[(tab[1], "=", self.conum)])
            if err:
                break
        if err:
            showError(self.mf.body, "Deletion Error",
                "Records Exist for This Company, Cannot Delete!")
        else:
            self.sql.delRec("ctlmst", where=[("ctm_cono", "=", self.conum)])
            self.sql.delRec("ctlvmf", where=[("vtm_cono", "=", self.conum)])
            self.sql.delRec("ctlynd", where=[("cye_cono", "=", self.conum)])
            self.mf.dbm.commitDbase()

    def doCoyNam(self, frt, pag, r, c, p, i, w):
        self.name = w

    def doVat(self, frt, pag, r, c, p, i, w):
        self.vat = w
        acc = self.doReadVat(self.vat)
        if not acc:
            if w == "N":
                self.sql.insRec("ctlvmf", data=[self.conum, w, "No VAT",
                    "N", 0, 0, 0])
            else:
                ok = askQuestion(self.mf.body, "Invalid VAT Code",
                    "This Code Does Not Exist, Do You Want to Create It?")
                if ok == "no":
                    return "Invalid Code"
                state = self.df.disableButtons()
                self.df.mstAlw.hide()
                self.mf.gtkDispatch()
                import ms1030
                ms1030.ms1030(self.mf, self.conum, self.name, self.user, code=w)
                self.df.mstAlw.show()
                self.mf.gtkDispatch()
                self.df.enableButtons(state=state)
                acc = self.doReadVat(self.vat)
                if not acc:
                    self.df.loadEntry(frt, pag, p, data="")
                    return "Invalid Code"

    def doSys(self, frt, pag, r, c, p, i, w):
        if self.sys[i] == "RC" and w == "Y":
            self.df.loadEntry(frt, pag, p+1, data="N")
            return "sk1"
        if self.sys[i] == "SL" and w == "Y":
            self.df.loadEntry(frt, pag, p-1, data="Y")

    def doLogo(self, frt, pag, r, c, p, i, w):
        if not w:
            if self.img:
                self.img.image.destroy()
            self.df.loadEntry(frt, pag, p+1, data="N")
            return "sk1"
        self.logo = os.path.abspath(w)
        self.df.loadEntry(frt, pag, p, data=self.logo)

    def doFlip(self, frt, pag, r, c, p, i, w):
        if self.displayLogo(self.logo, w):
            return "ff1|Invalid Logo Image"

    def displayLogo(self, logo, flip):
        try:
            if self.img:
                self.img.image.destroy()
            self.img = ShowImage(self.df.nb.Page4, logo, msiz=640, flip=flip)
        except:
            return "error"

    def doReadVat(self, w):
        acc = self.sql.getRec(tables="ctlvmf", cols=["vtm_desc"],
            where=[("vtm_cono", "=", self.conum), ("vtm_code", "=", w)],
            limit=1)
        return acc

    def doEnd(self):
        if self.df.pag == 1:
            self.df.selPage("Banking")
            self.df.focusField("T", 2, 1)
        elif self.df.pag == 2:
            self.df.selPage("Systems")
            self.df.focusField("T", 3, 1)
        elif self.df.pag == 3:
            self.df.selPage("Logo")
            self.df.focusField("T", 4, 1)
        else:
            data = []
            mods = ""
            for x in range(0, len(self.df.t_work[1][0])):
                data.append(self.df.t_work[1][0][x])
            for x in range(0, len(self.df.t_work[2][0])):
                data.append(self.df.t_work[2][0][x])
            for x in range(0, len(self.df.t_work[3][0])):
                if self.df.t_work[3][0][x] == "Y":
                    mods = mods + self.sys[x]
            data.append(mods)
            data.extend(self.df.t_work[4][0][0:2])
            if self.new:
                self.sql.insRec("ctlmst", data=data)
            elif data != self.old:
                self.sql.updRec("ctlmst", data=data, where=[("ctm_cono",
                    "=", self.conum)])
                dte = long("%04i%02i%02i%02i%02i%02i" % time.localtime()[:-3])
                for num, dat in enumerate(self.old):
                    if dat != data[num]:
                        self.sql.insRec("chglog", data=["ctlmst", "U",
                            "%03i" % self.conum, self.sql.ctlmst_col[num],
                            dte, self.user, str(dat), str(data[num]), 0])
            fin = self.sql.getRec(tables="ctlynd", cols=["count(*)"],
                where=[("cye_cono", "=", self.conum)], limit=1)
            if not fin[0]:
                self.doFinPeriod()
                if self.exit == "y":
                    self.doCancel()
                    return True
            self.mf.dbm.commitDbase()
            self.df.selPage("General")
            self.df.focusField("T", 1, 1)

    def doFinPeriod(self):
        tit = ("Initial Financial Period",)
        fld = (
            (("T",0,0,0),"ID1",10,"Period Start Date","Period Start Date",
                "","N",self.doStartPer,None,None,("efld",)),
            (("T",0,1,0),"ID1",10,"Period End Date","Period End Date",
                "","N",self.doEndPer,None,None,("efld",)))
        tnd = ((self.doPerEnd,"y"), )
        txt = (self.doPerExit,)
        state = self.df.disableButtons()
        self.df.mstAlw.hide()
        self.pf = TartanDialog(self.mf, newscn=(0,0,0), tops=True, title=tit,
            eflds=fld, tend=tnd, txit=txt)
        self.mf.startLoop()
        self.df.mstAlw.show_all()
        self.df.enableButtons(state=state)
        self.mf.head.set_text(self.tit[0])

    def doStartPer(self, frt, pag, r, c, p, i, w):
        y = (w / 10000) - 1
        m = (w % 10000) / 100
        if m == 2:
            if (y % 4) == 0:
                d = 29
            else:
                d = 28
        else:
            d = w % 100
        self.s0 = (y*10000) + (m*100) + d
        self.s1 = w

    def doEndPer(self, frt, pag, r, c, p, i, w):
        if w <= self.s1:
            return "Invalid End Period"
        y = (w / 10000) - 1
        m = (w % 10000) / 100
        if m == 2:
            if (y % 4) == 0:
                d = 29
            else:
                d = 28
        else:
            d = w % 100
        self.e0 = (y*10000) + (m*100) + d
        self.e1 = w

    def doPerEnd(self):
        self.sql.insRec("ctlynd",data=[self.conum, 0, self.s0, self.e0, 0, "N"])
        self.sql.insRec("ctlynd",data=[self.conum, 1, self.s1, self.e1, 0, "N"])
        self.exit = "n"
        self.closeProcess()

    def doPerExit(self):
        self.exit = "y"
        self.closeProcess()

    def closeProcess(self):
        self.pf.closeProcess()
        self.mf.closeLoop()

    def doAccept(self, widget=None):
        mod = False
        frt, pag, col, mes = self.df.doCheckFields(("T",1,None))
        if not mes:
            frt, pag, col, mes = self.df.doCheckFields(("T",2,None))
            if not mes:
                for x in range(len(self.df.t_work[3][0])):
                    if self.df.t_work[3][0][x] == "Y":
                        mod = True
        if not mod:
            self.df.selPage("Systems")
            self.df.focusField("T", 3, 1, err="Missing System Module")
        else:
            frt, pag, col, mes = self.df.doCheckFields(("T",1,None))
            if mes:
                self.df.last[pag][0] = col+1
                self.df.selPage(self.df.tags[pag - 1][0])
                self.df.focusField(frt, pag, (col+1), err=mes)
            else:
                if self.img:
                    self.img.image.destroy()
                self.df.selPage("Logo")
                self.df.doEndFrame("T", 4, cnf="N")

    def doPrint(self, widget=None):
        state = self.df.disableButtons()
        self.df.mstAlw.hide()
        rp = TabPrt(self.mf, self.conum, name=__name__,
            tabs="ctlmst", wher=[("ctm_cono", "=", self.conum)])
        self.df.mstAlw.show()
        self.mf.gtkDispatch()
        self.df.enableButtons(state=state)
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def doCancel(self, widget=None):
        self.mf.dbm.rollbackDbase()
        self.df.selPage("General")
        self.df.focusField("T", 1, 1)

    def doExit1(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

    def doExit2(self, widget=None):
        self.df.selPage("General")

    def doExit3(self, widget=None):
        self.df.selPage("Banking")

    def doExit4(self, widget=None):
        self.df.selPage("Systems")
