#! /usr/bin/python

"""
SYNOPSIS
    Value Added Tax Records Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import TartanDialog, Sql

class ms1030:
    def __init__(self, mf, coy, nam, usr, code=None):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.user = usr
        self.setVariables()
        self.mainProcess()
        if code:
            self.df.doKeyPressed("T", 0, 0, code)
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlvmf", "ctlvtf", "chglog"],
            prog=__name__)

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "V.A.T. Records Maintenance")
        vtm = {
            "stype": "R",
            "tables": ("ctlvmf",),
            "cols": (
                ("vtm_code", "", 0, "C"),
                ("vtm_desc", "", 0, "Description", "Y"),
                ("vtm_cat", "", 0, "C")),
            "where": [("vtm_cono", "=", self.conum)]}
        vtt = {
            "stype": "C",
            "titl": "VAT Types",
            "data": [
                ["N", "None"],
                ["S", "Standard"],
                ["Z", "Zero Rated"],
                ["C", "Capital Item"],
                ["X", "Excluded Item"]]}
        fld = (
            (("T",0,0,0),"IUA",1,"V.A.T. Code","V.A.T. Code",
                "","Y",self.doCode,vtm,None,("notblank",)),
            (("T",0,1,0),"INA",30,"Description","Description",
                "","N",None,None,self.doDelete,("notblank",)),
            (("T",0,2,0),"IUA",1,"Category","Category",
                "S","N",self.doCat,vtt,None,("in", ("C","N","S","X","Z"))),
            (("T",0,3,0),"Id1",10,"Start Date","Start Date",
                "","N",self.doStart,None,None,("notzero",)),
            (("T",0,4,0),"IUD",6.2,"Old Rate","Old Rate",
                "","N",None,None,None,("notzero",)),
            (("T",0,5,0),"IUD",6.2,"New Rate","New Rate",
                "","N",None,None,None,("notzero",)))
        but = (
            ("Accept",None,self.doAccept,0,("T",0,2),("T",0,0)),
            ("Cancel",None,self.doCancel,0,("T",0,2),("T",0,0)),
            ("Quit",None,self.doExit,1,None,None))
        tnd = ((self.doEnd,"y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, butt=but,
            tend=tnd, txit=txt)

    def doCode(self, frt, pag, r, c, p, i, w):
        self.code = w
        self.old = self.sql.getRec(tables="ctlvmf", where=[("vtm_cono", "=",
            self.conum), ("vtm_code", "=", self.code)], limit=1)
        if not self.old:
            self.new = "y"
        else:
            self.new = "n"
            for x in range(0, self.df.topq[pag]):
                self.df.loadEntry(frt, pag, x, data=self.old[x+1])

    def doCat(self, frt, pag, r, c, p, i, w):
        self.cat = w
        if self.cat == "N":
            self.df.loadEntry(frt, pag, p+1, data=0)
            self.df.loadEntry(frt, pag, p+2, data=0)
            self.df.loadEntry(frt, pag, p+3, data=0)
            return "sk3"

    def doStart(self, frt, pag, r, c, p, i, w):
        self.start = w
        if self.cat in ("X", "Z"):
            self.df.loadEntry(frt, pag, p+1, data=0)
            self.df.loadEntry(frt, pag, p+2, data=0)
            return "sk2"

    def doDelete(self):
        t = self.sql.getRec(tables="ctlvtf", cols=["count(*)"],
            where=[("vtt_cono", "=", self.conum), ("vtt_code", "=", self.code)],
            limit=1)
        if t[0]:
            return "Transactions Exist, Not Deleted"
        self.sql.delRec("ctlvmf", where=[("vtm_cono", "=", self.conum),
            ("vtm_code", "=", self.code)])
        dte = long("%04i%02i%02i%02i%02i%02i" % time.localtime()[:-3])
        self.sql.insRec("chglog", data=["ctlvmf", "D", "%03i%-1s" % \
            (self.conum, self.code), "", dte, self.user, "", "", 0])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doEnd(self):
        data = [self.conum]
        for x in range(0, len(self.df.t_work[0][0])):
            data.append(self.df.t_work[0][0][x])
        if self.new == "y":
            self.sql.insRec("ctlvmf", data=data)
        elif data != self.old:
            self.sql.updRec("ctlvmf", data=data, where=[("vtm_cono", "=",
                self.conum), ("vtm_code", "=", self.code)])
            dte = long("%04i%02i%02i%02i%02i%02i" % time.localtime()[:-3])
            for num, dat in enumerate(self.old):
                if dat != data[num]:
                    self.sql.insRec("chglog", data=["ctlvmf",
                    "U", "%03i%-1s" % (self.conum, self.code),
                    self.sql.ctlvmf_col[num], dte, self.user,
                    str(dat), str(data[num]), 0])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doAccept(self, widget=None):
        frt, pag, col, mes = self.df.doCheckFields()
        if mes:
            self.df.focusField(frt, pag, (col+1), err=mes)
        else:
            self.df.doEndFrame("T", 0, cnf="N")

    def doCancel(self, widget=None):
        self.mf.dbm.rollbackDbase()
        self.df.focusField("T", 0, 1)

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

