#! /usr/bin/python

"""
SYNOPSIS
    Email Log.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import CCD, RepPrt, TartanDialog

class ms3030:
    def __init__(self, mf):
        self.mf = mf
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]

    def mainProcess(self):
        tit = ("Email Log Report",)
        r1s = (("Ascending", "A"), ("Descending","D"))
        fld = (
            (("T",0,0,0),"Id1",10,"Date From","",
                0,"N",self.doDate,None,None,("efld",)),
            (("T",0,1,0),"ID1",10,"Date To","",
                self.sysdtw,"N",self.doDate,None,None,("efld",)),
            (("T",0,2,0),"INA",50,"Recipient","",
                "","N",self.doRecpt,None,None,("efld",)),
            (("T",0,3,0),("IRB",r1s),0,"Date Order","",
                "A","N",self.doOrder,None,None,None))
        tnd = ((self.doEnd,"y"),)
        txt = (self.doExit,)
        self.df = TartanDialog(self.mf, tops=False, title=tit, eflds=fld,
            tend=tnd, txit=txt, view=("Y","V"), mail=("Y","N"))

    def doDate(self, frt, pag, r, c, p, i, w):
        if p == 0:
            self.frm = w
        elif w < self.frm:
            return "Invalid Date, Less than Start Date"
        else:
            self.too = w

    def doRecpt(self, frt, pag, r, c, p, i, w):
        self.recpt = w

    def doOrder(self, frt, pag, r, c, p, i, w):
        self.order = w

    def doEnd(self):
        self.df.closeProcess()
        frm = CCD(self.frm, "d1", 10).disp
        too = CCD(self.too, "D1", 10).disp
        hds = "Email Log Report from %s to %s" % (frm, too)
        col = ["eml_too", "eml_sub", "eml_dtt", "eml_sta"]
        whr = [
            ("eml_dtt", ">=", "%10s 00:00" % frm),
            ("eml_dtt", "<=", "%10s 99:99" % too)]
        if self.recpt:
            whr.append(("eml_too", "like", "%s%s%s" % ("%", self.recpt, "%")))
        if self.order == "A":
            ord = "eml_dtt asc"
        else:
            ord = "eml_dtt desc"
        rpt = RepPrt(self.mf, name=__name__, tables=["emllog"], heads=[hds],
            cols=col, where=whr, order=ord, view=self.df.repprt[0][1],
            prtnam=self.df.repprt[0][2], mail=self.df.repprt[1][2])
        self.mf.closeLoop()

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()

if __name__ == "__main__":
    from TartanClasses import MainFrame
    mf = MainFrame(fixed=False)
    mf.openDbase()
    rpt = ms3030(mf)
