#! /usr/bin/python2

"""
SYNOPSIS
    Password Manager

    This file is part of Tartan Systems (TARTAN).
    And pycrypto is a dependancy.

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    License (GPL) as published by the Free Software Foundation; either
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN; if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import os, shutil, sys
from Crypto.Cipher import AES
from TartanClasses import TartanDialog
from tartanFunctions import askQuestion

class pm1010:
    def __init__(self, mf, user):
        self.mf = mf
        self.user = user
        self.setVariables()
        self.getSecret()
        self.mf.startLoop()
        if self.pwd:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.pwd = None
        if sys.platform == "win32":
            self.file = "secrets.dat"
        else:
            self.file = ".secrets.dat"
        home = os.environ.get("HOME")
        if home:
            self.file = os.path.join(home, self.file)
        self.show = False

    def getSecret(self):
        tit = ("Secret Word",)
        fld = (
            (("T",0,0,0),"IHA",32,"Secret Word","Secret Word",
                "","N",self.doSecret,None,None,("notblank",)),)
        tnd = ((self.doEndSecret,"n"), )
        txt = (self.doExitSecret,)
        self.sw = TartanDialog(self.mf, tops=True, title=tit, eflds=fld,
            tend=tnd, txit=txt)

    def doSecret(self, frt, pag, r, c, p, i, w):
        siz = len(w)
        if siz not in (16, 24, 32):
            if siz < 16:
                key = w + "*" * (16 - siz)
            elif siz < 24:
                key = w + "*" * (24 - siz)
            elif siz < 32:
                key = w + "*" * (32 - siz)
        self.pwd = AES.new(key, AES.MODE_ECB)

    def doEndSecret(self, widget=None):
        self.data = {}
        if os.path.isfile(self.file):
            of = open(self.file, "rb")
            dt = of.readlines()
            for line in dt:
                data = []
                for codes in ("User", "Code", "Clear", "Fill", "Encrypt"):
                    f = len("Start%s" % codes) + line.find("Start%s" % codes)
                    t = line.find("End%s" % codes)
                    data.append(line[f:t])
                user = data[0]
                if not user in self.data:
                    self.data[user] = {}
                code = data[1]
                login = data[2]
                fill = int(data[3])
                chrs = data[4].split(",")
                text = ""
                if not chrs[0]:
                    encrypt = ""
                else:
                    for n, c in enumerate(chrs):
                        if not n:
                            text = chr(int(c))
                        else:
                            text = "%s%s" % (text, chr(int(c)))
                    encrypt = self.pwd.decrypt(text)
                    if fill:
                        encrypt = encrypt[:-fill]
                self.data[user][code] = [login, encrypt]
        if not self.user in self.data:
            self.data[self.user] = {}
        self.sw.closeProcess()
        self.mf.closeLoop()

    def doExitSecret(self, widget=None):
        self.sw.closeProcess()
        self.mf.closeLoop()

    def mainProcess(self):
        tit = ("Password Manager",)
        pwm = {
            "stype": "C",
            "titl": "Select the Secret to Change or <Esc> to Exit",
            "head": ("Code", "Clear-Text"),
            "data": self.doLoadCodes()}
        fld = (
            (("T",0,0,0),"INA",30,"Code","Code",
                "","Y",self.doCode,pwm,None,("notblank",)),
            (("T",0,1,0),"INA",50,"Clear-Text","Clear Text",
                "","N",self.doClear,None,self.doDelete,("efld",)),
            (("T",0,2,0),"IHA",50,"Encrypted-Text","Encrypted Text",
                "","N",self.doEncrypted,None,None,("efld",)))
        but = (
            ("Toggle Encryption",None,self.doShow,1,None,None),
            ("Quit",None,self.doQuit,1,None,None))
        tnd = ((self.doEnd,"y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, butt=but,
            tend=tnd, txit=txt)

    def doCode(self, frt, pag, r, c, p, i, w):
        self.code = w
        if not self.code in self.data[self.user]:
            self.data[self.user][self.code] = ["", ""]
        self.login = self.data[self.user][self.code][0]
        self.encrypt = self.data[self.user][self.code][1]
        self.df.loadEntry(frt, pag, p+1, data=self.login)
        self.df.topEntry[pag][p+1].set_sensitive(True)
        self.df.loadEntry(frt, pag, p+2, data=self.encrypt)
        self.df.topEntry[pag][p+2].set_sensitive(True)

    def doClear(self, frt, pag, r, c, p, i, w):
        self.login = w

    def doEncrypted(self, frt, pag, r, c, p, i, w):
        self.encrypt = w

    def doQuit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

    def doShow(self, widget=None):
        if self.show:
            self.show = False
        else:
            self.show = True
        self.df.topEntry[0][2].set_visibility(self.show)
        self.df.focusField("T", 0, self.df.col)

    def doDelete(self, widget=None):
        del self.data[self.user][self.code]
        self.df.topf[0][0][8]["data"] = self.doLoadCodes()
        self.df.focusField("T", 0, 1)

    def doEnd(self, widget=None):
        if self.show:
            self.doShow()
        self.data[self.user][self.code] = [self.login, self.encrypt]
        self.df.topf[0][0][8]["data"] = self.doLoadCodes()
        self.df.focusField("T", 0, 1)

    def doLoadCodes(self):
        data = []
        codes = self.data[self.user].keys()
        codes.sort()
        for code in codes:
            login, encrypt = self.data[self.user][code]
            data.append((code, login))
        return data

    def doExit(self, widget=None):
        ok = askQuestion(self.mf.body, "Save Changes",
            "Would you like to SAVE All Changes?", default="yes")
        if ok == "yes":
            if os.path.isfile(self.file):
                shutil.copy(self.file, self.file + "~")
            of = open(self.file, "wb")
            users = self.data.keys()
            users.sort()
            for user in users:
                for code in self.data[user]:
                    login = self.data[user][code][0]
                    encrypt = self.data[user][code][1]
                    fill = 16 - (len(encrypt) % 16)
                    if fill:
                        encrypt = "%s%s" % (encrypt, ("*" * fill))
                    hash = self.pwd.encrypt(encrypt)
                    text = ""
                    for n, c in enumerate(hash):
                        if not n:
                            text = ord(c)
                        else:
                            text = "%s,%s" % (text, ord(c))
                    of.write(
                        "StartUser%sEndUser"\
                        "StartCode%sEndCode"\
                        "StartClear%sEndClear"\
                        "StartFill%sEndFill"\
                        "StartEncrypt%sEndEncrypt\n" % \
                        (user, code, login, fill, text))
            of.close()
            if sys.platform == "win32":
                import win32api
                win32api.SetFileAttributes(self.file,
                    win32con.FILE_ATTRIBUTE_HIDDEN)
        self.df.closeProcess()
        self.mf.closeLoop()

if __name__ == "__main__":
    import getpass
    from TartanClasses import MainFrame
    from tartanFunctions import loadRcFile
    rcdic = loadRcFile(default=True)
    mf = MainFrame(rcdic=rcdic, fixed=False, dbobj=False)
    pm1010(mf, getpass.getuser())
