#! /usr/bin/env python

"""
SYNOPSIS
    Rental System Owners Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import GetCtl, Sql, TartanDialog

class rc1010:
    def __init__(self, mf, coy, nam, usr):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.user = usr
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlmst", "rcaowm", "rcaowt", "chglog"],
            prog=__name__)
        if self.sql.error:
            return "no"
        self.gc = GetCtl(self.mf)
        self.taxdf = self.gc.getCtl("ctlmst", self.conum)["ctm_taxdf"]

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
                "Rental System Owners Maintenance (%s)" % __name__)
        own = {
            "stype": "R",
            "tables": ("rcaowm",),
            "cols": (
                ("rom_acno", "", 0, "Acc-Num"),
                ("rom_name", "", 0, "Name", "Y")),
            "where": [("rom_cono", "=", self.conum)]}
        self.fld = (
            (("T",0,0,0),"INA",7,"Account Number","Account Number",
                "","Y",self.doOwner,own,None,("notblank",)),
            (("T",0,1,0),"INA",30,"Name","Name",
                "","N",None,None,self.doDelete,("notblank",)),
            (("T",0,2,0),"INA",30,"Address Line 1","Address Line 1",
                "","N",None,None,None,("notblank",)),
            (("T",0,3,0),"INA",30,"Address Line 2","Address Line 2",
                "","N",None,None,None,("efld",)),
            (("T",0,4,0),"INA",30,"Address Line 3","Address Line 3",
                "","N",None,None,None,("efld",)),
            (("T",0,5,0),"INA",4,"Postal Code","PCod",
                "","N",None,None,None,("notblank",)),
            (("T",0,6,0),"INA",20,"Home Number","Home Number",
                "","N",None,None,None,("efld",)),
            (("T",0,7,0),"INA",20,"Office Number","Office Number",
                "","N",None,None,None,("efld",)),
            (("T",0,8,0),"INA",20,"Mobile Number","Mobile Number",
                "","N",None,None,None,("efld",)),
            (("T",0,9,0),"INA",20,"Fax Number","Fax Number",
                "","N",None,None,None,("efld",)),
            (("T",0,10,0),"ITX",50,"E-Mail Address","E-Mail Address",
                "","N",None,None,None,("email",)),
            (("T",0,11,0),"INA",10,"VAT Number","VAT-Number",
                "","N",None,None,None,("efld",)),
            (("T",0,12,0),"IUA",1,"VAT Default","V",
                "","N",None,None,None,("efld",)),
            (("T",0,13,0),"INA",20,"Bank Name","Bank-Name",
                "","N",None,None,None,("efld",)),
            (("T",0,14,0),"IUI",8,"Bank Branch","Bank-IBT",
                "","N",None,None,None,("efld",)),
            (("T",0,15,0),"INA",16,"Bank Account","Bank-Account-Num",
                "","N",None,None,None,("efld",)))
        but = (
            ("Accept",None,self.doAccept,0,("T",0,2),("T",0,0)),
            ("Cancel",None,self.doCancel,0,("T",0,2),("T",0,0)),
            ("Quit",None,self.doExit,1,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        abt = None
        self.df = TartanDialog(self.mf, title=tit, eflds=self.fld, butt=but,
            tend=tnd, txit=txt)

    def doOwner(self, frt, pag, r, c, p, i, w):
        self.acno = w
        self.oldm = self.sql.getRec(tables="rcaowm", where=[("rom_cono", "=",
            self.conum), ("rom_acno", "=", self.acno)], limit=1)
        if not self.oldm:
            self.new = "y"
            for num in range(2, self.df.topq[0]):
                self.df.clearEntry(frt, pag, num+1)
        else:
            self.new = "n"
            for num, fld in enumerate(self.oldm[1:]):
                self.df.loadEntry(frt, pag, p+num, data=fld)
            trn = self.sql.getRec(tables="rcaowt", cols=["count(*)"],
                where=[("rot_cono", "=", self.conum), ("rot_acno", "=",
                self.acno)], limit=1)
            if trn[0]:
                self.trn = True
            else:
                self.trn = False

    def doName(self, frt, pag, r, c, p, i, w):
        pass

    def doTelno(self, frt, pag, r, c, p, i, w):
        pass

    def doEmail(self, frt, pag, r, c, p, i, w):
        pass

    def doVatInd(self, frt, pag, r, c, p, i, w):
        pass

    def doVatNum(self, frt, pag, r, c, p, i, w):
        if self.new == "n" and self.trn:
            return "sk1"

    def doStart(self, frt, pag, r, c, p, i, w):
        pass

    def doPeriod(self, frt, pag, r, c, p, i, w):
        pass

    def doPayInd(self, frt, pag, r, c, p, i, w):
        pass

    def doAmount(self, frt, pag, r, c, p, i, w):
        if self.new == "y":
            self.df.loadEntry(frt, pag, p+1, data="C")

    def doStatus(self, frt, pag, r, c, p, i, w):
        pass

    def doDelete(self):
        if self.trn:
            return "Transactions Exist, Not Deleted"
        self.sql.delRec("rcaowm", where=[("rom_cono", "=", self.conum),
            ("rom_acno", "=", self.acno)])
        dte = long("%04i%02i%02i%02i%02i%02i" % time.localtime()[:-3])
        self.sql.insRec("chglog", data=["rcaowm", "D", "%03i%-7s" % \
            (self.conum, self.acno), "", dte, self.user, "", "", 0])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doEnd(self):
        datm = [self.conum]
        for num, fld in enumerate(self.df.t_work[0][0]):
            datm.append(fld)
        if self.new == "y":
            self.sql.insRec("rcaowm", data=datm)
        else:
            dte = long("%04i%02i%02i%02i%02i%02i" % time.localtime()[:-3])
            if datm != self.oldm:
                self.sql.updRec("rcaowm", data=datm, where=[("rom_cono", "=",
                    self.conum), ("rom_acno", "=", self.acno)])
                for num, dat in enumerate(self.oldm):
                    if dat != datm[num]:
                        self.sql.insRec("chglog", data=["rcaowm", "U",
                            "%03i%-7s" % (self.conum, self.acno),
                            self.sql.rcaowm_col[num], dte, self.user,
                            str(dat), str(datm[num]), 0])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doAccept(self, widget=None):
        frt, pag, col, mes = self.df.doCheckFields()
        if mes:
            self.df.focusField(frt, pag, (col+1), err=mes)
        else:
            self.df.doEndFrame("T", 0, cnf="N")

    def doCancel(self, widget=None):
        self.mf.dbm.rollbackDbase()
        self.df.focusField("T", 0, 1)

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

