#! /usr/bin/env python

"""
SYNOPSIS
    Rental System Premises Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import Sql, TartanDialog

class rc1020:
    def __init__(self, mf, coy, nam, usr):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.user = usr
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["chglog", "rcaprm"], prog=__name__)
        if self.sql.error:
            return "no"

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
                "Rental System Premises Maintenance (%s)" % __name__)
        own = {
            "stype": "R",
            "tables": ("rcaowm",),
            "cols": (
                ("rom_acno", "", 0, "Own-Cod"),
                ("rom_name", "", 0, "Name", "Y")),
            "where": [("rom_cono", "=", self.conum)]}
        prm = {
            "stype": "R",
            "tables": ("rcaprm",),
            "cols": (
                ("rcp_code", "", 0, "Prm-Code"),
                ("rcp_addr1", "", 0, "Address-Line-1")),
            "where": [("rcp_cono", "=", self.conum)],
            "whera": [("T", "rcp_owner", 0, 0)]}
        self.fld = [
            (("T",0,0,0),"INA",7,"Owner Code","Owner",
                "","N",self.doOwnCod,own,None,("notblank",)),
            (("T",0,0,0),"ONA",30,"Name"),
            (("T",0,1,0),"INA",7,"Premises Code","Premises",
                "","N",self.doPrmCod,prm,None,("notblank",)),
            (("T",0,2,0),"INA",30,"Description","Description",
                "","N",None,None,self.doDelete,("notblank",)),
            (("T",0,3,0),"INA",30,"Address Line 1","Address Line 1",
                "","N",None,None,None,("notblank",)),
            (("T",0,4,0),"INA",30,"Address Line 2","Address Line 2",
                "","N",None,None,None,("efld",)),
            (("T",0,5,0),"INA",30,"Address Line 3","Address Line 3",
                "","N",None,None,None,("efld",)),
            (("T",0,6,0),"INA",4,"Postal Code","PCod",
                "","N",None,None,None,("notblank",)),
            (("T",0,7,0),"IUD",6.2,"Commission Rate","C-Rate",
                "","N",None,None,None,("efld",))]
        but = (
            ("Accept",None,self.doAccept,0,("T",0,2),("T",0,0)),
            ("Cancel",None,self.doCancel,0,("T",0,2),("T",0,0)),
            ("Quit",None,self.doExit,1,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        abt = None
        self.df = TartanDialog(self.mf, title=tit, eflds=self.fld, butt=but,
            tend=tnd, txit=txt)

    def doOwnCod(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="rcaowm", cols=["rom_name"],
            where=[("rom_cono", "=", self.conum), ("rom_acno", "=", w)],
            limit=1)
        if not acc:
            return "Invalid Owner"
        self.owner = w
        self.name = acc[0]
        self.df.loadEntry(frt, pag, p+1, data=self.name)

    def doPrmCod(self, frt, pag, r, c, p, i, w):
        self.code = w
        self.old = self.sql.getRec(tables="rcaprm", where=[("rcp_cono", "=",
            self.conum), ("rcp_owner", "=", self.owner), ("rcp_code", "=",
            self.code)], limit=1)
        if not self.old:
            self.new = "y"
        else:
            self.new = "n"
            acc = self.old[2:]
            for num, dat in enumerate(acc):
                self.df.loadEntry(frt, pag, p+num, data=dat)

    def doDelete(self):
        mst = self.sql.getRec(tables="rcatnm", cols=["count(*)"],
            where=[("rtn_cono", "=", self.conum), ("rtn_code", "=",
            self.code)], limit=1)
        if mst[0]:
            return "Accounts Exist, Not Deleted"
        self.sql.delRec("rcaprm", where=[("rcp_cono", "=", self.conum),
            ("rcp_code", "=", self.code)])
        dte = long("%04i%02i%02i%02i%02i%02i" % time.localtime()[:-3])
        self.sql.insRec("chglog", data=["rcaprm", "D", "%03i%-7s" % \
            (self.conum, self.code), "", dte, self.user, "", "", 0])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doEnd(self):
        data = [self.conum]
        for x in range(0, len(self.df.t_work[0][0])):
            if x == 1:
                continue
            data.append(self.df.t_work[0][0][x])
        if self.new == "y":
            self.sql.insRec("rcaprm", data=data)
        elif data != self.old:
            self.sql.updRec("rcaprm", data=data, where=[("rcp_cono", "=",
                self.conum), ("rcp_code", "=", self.code)])
            dte = long("%04i%02i%02i%02i%02i%02i" % time.localtime()[:-3])
            for num, dat in enumerate(self.old):
                if dat != data[num]:
                    self.sql.insRec("chglog", data=["rcaprm",
                        "U", "%03i%-7s" % (self.conum, self.code),
                        self.sql.rcaprm_col[num], dte, self.user,
                        str(dat), str(data[num]), 0])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doAccept(self, widget=None):
        frt, pag, col, mes = self.df.doCheckFields()
        if mes:
            self.df.focusField(frt, pag, (col+1), err=mes)
        else:
            self.df.doEndFrame("T", 0, cnf="N")

    def doCancel(self, widget=None):
        self.mf.dbm.rollbackDbase()
        self.df.focusField("T", 0, 1)

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

