#! /usr/bin/env python

"""
SYNOPSIS
    Rental System Tenants Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import Sql, TartanDialog

class rc1030:
    def __init__(self, mf, coy, nam, usr):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.user = usr
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlmst", "rcaprm", "rcatnm", "rcacon",
            "rcatnt", "chglog"], prog=__name__)
        if self.sql.error:
            return "no"

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
                "Rental System Masterfile Maintenance (%s)" % __name__)
        own = {
            "stype": "R",
            "tables": ("rcaowm",),
            "cols": (
                ("rom_acno", "", 0, "Acc-Num"),
                ("rom_name", "", 0, "Name", "Y")),
            "where": [("rom_cono", "=", self.conum)]}
        prm = {
            "stype": "R",
            "tables": ("rcaprm",),
            "cols": (
                ("rcp_code", "", 0, "Prm-Code"),
                ("rcp_addr1", "", 0, "Address-Line-1")),
            "where": [("rcp_cono", "=", self.conum)],
            "whera": (("T", "rcp_owner", 0, 0),)}
        acc = {
            "stype": "R",
            "tables": ("rcatnm",),
            "cols": (
                ("rtn_acno", "", 0, "Acc-Num"),
                ("rtn_name", "", 0, "Name", "Y")),
            "where": [("rtn_cono", "=", self.conum)],
            "whera": (("T", "rtn_owner", 0, 0), ("T", "rtn_code", 1, 0))}
        r1s = (
            ("Monthly","M"),
            ("Quarterly","3"),
            ("Bi-Annually", "6"),
            ("Annually","A"))
        self.fld = (
            (("T",0,0,0),"INA",7,"Owners Code","Owner",
                "","Y",self.doOwner,own,None,("notblank",)),
            (("T",0,1,0),"INA",7,"Premises Code","Premises",
                "","Y",self.doPremises,prm,None,("notblank",)),
            (("T",0,2,0),"INA",7,"Account Code","Acc-Cod",
                "","N",self.doAccount,acc,None,("notblank",)),
            (("T",0,3,0),"INA",30,"Tenant Name","Tenant Name",
                "","N",self.doName,None,self.doDelete,("notblank",)),
            (("T",0,4,0),"INA",30,"Address Line 1","Address Line 1",
                "","N",self.doAddr,None,None,("efld",)),
            (("T",0,5,0),"INA",30,"Address Line 2","Address Line 2",
                "","N",None,None,None,("efld",)),
            (("T",0,6,0),"INA",30,"Address Line 3","Address Line 3",
                "","N",None,None,None,("efld",)),
            (("T",0,7,0),"INA",4,"Postal Code","PCod",
                "","N",None,None,None,("efld",)),
            (("T",0,8,0),"INA",20,"Telephone Number","Telephone Number",
                "","N",self.doTelno,None,None,("efld",)),
            (("T",0,9,0),"ITX",50,"E-Mail Address","E-Mail Address",
                "","N",self.doEmail,None,None,("email",)),
            (("T",0,10,0),"INA",10,"VAT Number","VAT Number",
                "","N",self.doVatNum,None,None,("efld",)),
            (("T",0,11,0),("IRB",r1s),0,"Payment Frequency","Payment Frequency",
                "M","N",self.doPayInd,None,None,None),
            (("T",0,12,0),"ID1",10,"Start Date","Start Date",
                "","N",self.doStart,None,None,("notzero",)),
            (("T",0,13,0),"IUI",3,"Number of Periods","Number of Periods",
                "","N",self.doPeriod,None,None,("notzero",)),
            (("T",0,14,0),"IUD",12.2,"Rental Amount","Rental Amount",
                "","N",self.doAmount,None,None,("notzero",)),
            (("T",0,15,0),"IUD",12.2,"Deposit Amount","Deposit Amount",
                "","N",self.doAmount,None,None,("efld",)),
            (("T",0,16,0),"IUA",1,"Status","Status",
                "","N",self.doStatus,None,None,("in",("C","X"))))
        but = (
            ("Accept",None,self.doAccept,0,("T",0,2),("T",0,0)),
            ("Cancel",None,self.doCancel,0,("T",0,2),("T",0,0)),
            ("Quit",None,self.doExit,1,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        abt = None
        self.df = TartanDialog(self.mf, title=tit, eflds=self.fld, butt=but,
            tend=tnd, txit=txt)

    def doOwner(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="rcaowm", cols=["rom_name"],
            where=[("rom_cono", "=", self.conum), ("rom_acno", "=", w)],
            limit=1)
        if not acc:
            return "Invalid Owner"
        self.owner = w

    def doPremises(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="rcaprm", cols=["rcp_desc"],
            where=[("rcp_cono", "=", self.conum), ("rcp_owner", "=",
            self.owner), ("rcp_code", "=", w)], limit=1)
        if not acc:
            return "Invalid Premises"
        self.code = w

    def doAccount(self, frt, pag, r, c, p, i, w):
        self.acno = w
        self.oldm = self.sql.getRec(tables="rcatnm", where=[("rtn_cono", "=",
            self.conum), ("rtn_owner", "=", self.owner), ("rtn_code", "=",
            self.code), ("rtn_acno", "=", self.acno)], limit=1)
        if not self.oldm:
            self.new = "y"
            for num in range(4, self.df.topq[0]):
                self.df.clearEntry(frt, pag, num+1)
            con = self.sql.getRec(tables="rcacon", cols=["count(*)"],
                where=[("rcc_cono", "=", self.conum), ("rcc_owner", "=",
                self.owner), ("rcc_code", "=", self.code), ("rcc_acno",
                "=", self.acno)], limit=1)
            if not con[0]:
                self.cnum = 1
            else:
                self.cnum = con[0] + 1
        else:
            self.new = "n"
            for num, fld in enumerate(self.oldm[3:]):
                self.df.loadEntry(frt, pag, p+num, data=fld)
            self.oldc = self.sql.getRec(tables="rcacon", where=[("rcc_cono",
                "=", self.conum), ("rcc_owner", "=", self.owner), ("rcc_code",
                "=", self.code), ("rcc_acno", "=", self.acno)],
                order="rcc_cnum")
            self.cnum = self.oldc[-1:][0][4]
            for num, fld in enumerate(self.oldc[-1:][0][5:]):
                self.df.loadEntry(frt, pag, num+11, data=fld)
            trn = self.sql.getRec(tables="rcatnt", cols=["count(*)"],
                where=[("rtu_cono", "=", self.conum), ("rtu_owner", "=",
                self.owner), ("rtu_code", "=", self.code), ("rtu_acno", "=",
                self.acno)], limit=1)
            if trn[0]:
                self.trn = True
            else:
                self.trn = False

    def doName(self, frt, pag, r, c, p, i, w):
        pass

    def doAddr(self, frt, pag, r, c, p, i, w):
        if not w:
            self.df.loadEntry(frt, pag, p+1, data="")
            self.df.loadEntry(frt, pag, p+2, data="")
            self.df.loadEntry(frt, pag, p+3, data="")
            return "sk3"

    def doTelno(self, frt, pag, r, c, p, i, w):
        pass

    def doEmail(self, frt, pag, r, c, p, i, w):
        pass

    def doVatInd(self, frt, pag, r, c, p, i, w):
        pass

    def doVatNum(self, frt, pag, r, c, p, i, w):
        if self.new == "n" and self.trn:
            return "sk1"

    def doStart(self, frt, pag, r, c, p, i, w):
        pass

    def doPeriod(self, frt, pag, r, c, p, i, w):
        pass

    def doPayInd(self, frt, pag, r, c, p, i, w):
        pass

    def doAmount(self, frt, pag, r, c, p, i, w):
        if self.new == "y":
            self.df.loadEntry(frt, pag, p+1, data="C")

    def doStatus(self, frt, pag, r, c, p, i, w):
        pass

    def doDelete(self):
        if self.trn:
            return "Transactions Exist, Not Deleted"
        self.sql.delRec("rcatnm", where=[("rtn_cono", "=", self.conum),
            ("rtn_owner", "=", self.owner), ("rtn_code", "=", self.code),
            ("rtn_acno", "=", self.acno)])
        self.sql.delRec("rcacon", where=[("rcc_cono", "=", self.conum),
            ("rcc_owner", "=", self.owner), ("rcc_code", "=", self.code),
            ("rcc_acno", "=", self.acno)])
        dte = long("%04i%02i%02i%02i%02i%02i" % time.localtime()[:-3])
        self.sql.insRec("chglog", data=["rcatnm", "D", "%03i%-7s" % \
            (self.conum, self.code), "", dte, self.user, "", "", 0])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doEnd(self):
        datm = [self.conum]
        datc = [self.conum, self.owner, self.code, self.acno, self.cnum]
        for num, fld in enumerate(self.df.t_work[0][0]):
            if num < 11:
                datm.append(fld)
            if num > 10:
                datc.append(fld)
        if self.new == "y":
            self.sql.insRec("rcatnm", data=datm)
            self.sql.insRec("rcacon", data=datc)
        else:
            dte = long("%04i%02i%02i%02i%02i%02i" % time.localtime()[:-3])
            if datm != self.oldm:
                self.sql.updRec("rcatnm", data=datm, where=[("rtn_cono", "=",
                    self.conum), ("rtn_owner", "=", self.owner), ("rtn_code",
                    "=", self.code), ("rtn_acno", "=", self.acno)])
                for num, dat in enumerate(self.oldm):
                    if dat != datm[num]:
                        self.sql.insRec("chglog", data=["rcatnm", "U",
                            "%03i%-7s%-7s" % (self.conum, self.owner,
                            self.code), self.sql.rcatnm_col[num], dte,
                            self.user, str(dat), str(datm[num]), 0])
            if datc != self.oldc[-1:][0]:
                self.sql.updRec("rcacon", data=datc, where=[("rcc_cono", "=",
                    self.conum), ("rcc_owner", "=", self.owner), ("rcc_code",
                    "=", self.code), ("rcc_acno", "=", self.acno), ("rcc_cnum",
                    "=", self.cnum)])
                for num, dat in enumerate(self.oldc[-1:][0]):
                    if dat != datc[num]:
                        self.sql.insRec("chglog", data=["rcacon", "U",
                            "%03i%-7s%-7s%-7s%03i" % (self.conum, self.owner,
                            self.code, self.acno, self.cnum),
                            self.sql.rcacon_col[num], dte, self.user, str(dat),
                            str(datc[num]), 0])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doAccept(self, widget=None):
        frt, pag, col, mes = self.df.doCheckFields()
        if mes:
            self.df.focusField(frt, pag, (col+1), err=mes)
        else:
            self.df.doEndFrame("T", 0, cnf="N")

    def doCancel(self, widget=None):
        self.mf.dbm.rollbackDbase()
        self.df.focusField("T", 0, 1)

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

