#! /usr/bin/env python

"""
SYNOPSIS
    Rental Owners Statements.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import os, time
from PIL import Image
from TartanClasses import ASD, CCD, GetCtl, ProgressBar, Sql, TartanDialog
from tartanFunctions import doPrinter, doReportlab, flipImage, getModName
from tartanFunctions import getSingleRecords, showError, textFormat
from tartanWork import rttrtp

class rc3050:
    def __init__(self, mf, coy, nam, *args):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.buildHead()
            if not args:
                self.mainProcess()
                self.mf.startLoop()
            else:
                self.acc, self.datew, self.dated, self.prtnam, \
                    self.email = args[0]
                self.archive = "N"
                self.zeros = "Y"
                self.debit = "Y"
                self.mes = None
                self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
                    ext="pdf")
                if self.email == "N":
                    self.printSetup()
                whr = [
                    ("rom_cono", "=", self.conum),
                    ("rom_acno", "=", self.acc)]
                rtm = self.sql.getRec(tables="rcaowm", where=whr, limit=1)
                self.doProcess(rtm)
                if self.email == "N":
                    self.c.save()
                    self.doPrint()

    def setVariables(self):
        gc = GetCtl(self.mf)
        rcactl = gc.getCtl("rcactl", self.conum)
        if not rcactl:
            return "no"
        self.sql = Sql(self.mf.dbm, ["ctlmes", "ctlmst", "rcaowm", "rcaowt"],
            prog=__name__)
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.title = ("%03i %s" % (self.conum, self.conam),
            "Rental Owners Statements (%s)" % __name__)
        self.t = None
        self.trtypes = []
        for q in range(0, len(rttrtp)):
            self.trtypes.append(rttrtp[q][0])
        self.maxlines = 35

    def buildHead(self):
        self.head = []
        ctl = self.sql.getRec(tables="ctlmst", where=[("ctm_cono", "=",
            self.conum)], limit=1)
        for col in ("ctm_name", "ctm_add1", "ctm_add2", "ctm_add3",
            "ctm_pcode", "ctm_regno", "ctm_taxno", "ctm_tel", "ctm_fax",
            "ctm_logo", "ctm_flip"):
            exec "%s = ctl[self.sql.ctlmst_col.index('%s')]" % (col, col)
        if not ctm_logo or not os.path.exists(ctm_logo):
            self.ctm_logo = None
        elif ctm_flip == "N":
            self.ctm_logo = flipImage(ctm_logo)
        else:
            self.ctm_logo = ctm_logo
        if self.ctm_logo:
            self.head.append("%34s%9s" % ("", "STATEMENT"))
            self.head.append(" ")
            self.head.append(" ")
            self.head.append(" ")
            self.head.append(" ")
        else:
            mess = "%1s%-30s%3s%9s" % ("",ctm_name,"","STATEMENT")
            self.head.append(mess)
            mess = "%2s%-30s%2s%-30s" % ("",ctm_add1,"",("RegNo: " + ctm_regno))
            self.head.append(mess)
            mess = "%2s%-30s%2s%-30s" % ("",ctm_add2,"",("TaxNo: " + ctm_taxno))
            self.head.append(mess)
            mess = "%2s%-30s%2s%-30s" % ("",ctm_add3,"",("TelNo: " + ctm_tel))
            self.head.append(mess)
            mess = "%2s%-30s%2s%-30s" % ("",ctm_pcode,"",("FaxNo: " + ctm_fax))
            self.head.append(mess)

    def mainProcess(self):
        mss = {
            "stype": "R",
            "tables": ("ctlmes",),
            "cols": (
                ("mss_message", "", 0, "Num"),
                ("mss_detail", "NA", 50, "Details")),
            "where": [("mss_system", "=", "RCA")]}
        r1s = (("Yes","Y"),("No","N"))
        fld = (
            (("T",0,1,0),("IRB",r1s),0,"Whole File","Whole File",
                "Y","Y",self.doWhole,None,None,None),
            (("T",0,2,0),("IRB",r1s),0,"Zero Balances",
                "Include Zero Balances","N","Y",self.doZeros,None,None,None),
            (("T",0,3,0),("IRB",r1s),0,"Debit Balances",
                "Include Debit Balances","N","Y",self.doDebit,None,None,None),
            (("T",0,4,0),"IUI",3,"Message Number","Message Number",
                "","Y",self.doMessno,mss,None,("efld",)),
            (("T",0,5,0),"ID1",10,"Statement Date","Statement Date",
                self.sysdtw,"Y",self.doDat,None,None,("efld",)))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.title, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("N","Y"), save="N")

    def doWhole(self, frt, pag, r, c, p, i, w):
        self.whole = w

    def doZeros(self, frt, pag, r, c, p, i, w):
        self.zeros = w

    def doDebit(self, frt, pag, r, c, p, i, w):
        self.debit = w

    def doMessno(self, frt, pag, r, c, p, i, w):
        if w:
            self.mes = self.sql.getRec(tables="ctlmes", cols=["mss_detail"],
                where=[("mss_system", "=", "RCA"), ("mss_message", "=", w)],
                limit=1)
            if not self.mes:
                return "Invalid Message Number"
        else:
            self.mes = None

    def doDat(self, frt, pag, r, c, p, i, w):
        self.datew = w
        self.dated = self.df.t_disp[pag][0][p]

    def doEnd(self):
        self.df.closeProcess()
        self.prtnam = self.df.repprt[0][2]
        self.email = self.df.repprt[1][1]
        self.archive = self.df.repprt[2][1]
        if self.archive == "Y":
            dir = self.mf.arcdir
        else:
            dir = self.mf.wrkdir
        self.pdfnam = getModName(dir, __name__, self.conum, ext="pdf")
        if self.email == "N":
            self.printSetup()
        if self.whole == "N":
            all = getSingleRecords(self.mf, "rcaowm",
                ("rom_acno", "rom_name"), where=[("rom_cono", "=", self.conum)])
        else:
            whr = [("rom_cono", "=", self.conum)]
            ord = "rom_acno"
            all = self.sql.getRec(tables="rcaowm", where=whr, order=ord)
            if not all:
                showError(self.mf.body, "Error", "No Accounts Selected")
        if all:
            num = len(all)
            p = ProgressBar(self.mf.body, max=num, esc=True)
            for x in range(0, num):
                can = p.displayProgress(x)
                if can == "yes":
                    if self.t:
                        self.t.textLine()
                        self.t.textLine("Print Job ABORTED")
                    break
                self.doProcess(all[x])
            p.closeProgress()
            if self.email == "N":
                self.c.save()
                self.doPrint()
        self.mf.closeLoop()
         
    def doProcess(self, rtm):
        self.pgnum = 0
        tmc = self.sql.rcaowm_col
        self.acc = CCD(rtm[tmc.index("rom_acno")], "NA", 7)
        tots = self.sql.getRec(tables="rcaowt", cols=["rot_type",
            "round(sum(rot_tramt), 2)"], where=[("rot_cono", "=", self.conum),
            ("rot_acno", "=", self.acc.work), ("rot_trdt", "<=", self.datew)],
            group="rot_type")
        if not tots:
            return
        self.tots = [0, 0, 0, 0, 0, 0]
        for t in tots:
            self.tots[t[0]-1] = t[1]
            if t[0] != 5:
                self.tots[5] = float(ASD(self.tots[5]) + ASD(t[1]))
        if self.zeros == "N" and self.tots[5] == 0:
            return
        if self.debit == "N" and self.tots[5] > 0:
            return
        self.rtt = self.sql.getRec(tables="rcaowt", where=[("rot_cono", "=",
            self.conum), ("rot_acno", "=", self.acc.work), ("rot_type", "<>",
            5), ("rot_trdt", "<=", self.datew)], order="rot_trdt, rot_refno")
        if not self.rtt:
            return
        if self.email == "Y":
            self.printSetup()
        self.name = CCD(rtm[tmc.index("rom_name")], "NA", 30)
        self.add1 = CCD(rtm[tmc.index("rom_add1")], "NA", 30)
        if self.add1:
            self.add2 = CCD(rtm[tmc.index("rom_add2")], "NA", 30)
            self.add3 = CCD(rtm[tmc.index("rom_add3")], "NA", 30)
            self.pcod = CCD(rtm[tmc.index("rom_pcod")], "NA", 4)
        else:
            prm = self.sql.getRec(tables="rcaprm", cols=["rcp_addr1",
                "rcp_addr2", "rcp_addr3", "rcp_pcode"], where=[("rcp_cono",
                "=", self.conum), ("rcp_owner", "=", self.acc.work),
                ("rcp_code", "=", self.prm.work)], limit=1)
            if not prm:
                prm = ["", "", "", ""]
            self.add1 = CCD(prm[0], "NA", 30)
            self.add2 = CCD(prm[1], "NA", 30)
            self.add3 = CCD(prm[2], "NA", 30)
            self.pcod = CCD(prm[3], "NA", 4)
        self.emadd = CCD(rtm[tmc.index("rom_email")], "TX", 0)
        self.printHeader()
        self.printBody()
        self.printTrailer()
        if self.email == "Y":
            self.c.save()
            self.doPrint()

    def printSetup(self):
        self.c, self.cpi, self.inch, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam)
        # Width in pixels of each character
        self.cwidth = self.c.stringWidth(("*"), "Tartan", self.cpi)

    def doPrint(self):
        if self.prtnam == "view":
            view = "Y"
        else:
            view = "N"
        if self.email == "N" or not self.emadd.work:
            mail = "N"
        else:
            mail = self.emadd.work
        doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
            head=self.title, view=view, prtnam = self.prtnam, mail=mail)

    def printHeader(self):
        self.pageHeading()
        self.drawDocument()
        self.pgnum = self.pgnum + 1
        self.t.textLine()
        self.t.textLine()
        self.t.setFont("Tartan-Bold", 18)
        self.t.textLine(self.head[0])
        self.t.setFont("Tartan-Bold", 10)
        self.t.textLine(self.head[1])
        self.t.textLine(self.head[2])
        self.t.textLine(self.head[3])
        self.t.textLine(self.head[4])
        self.t.textLine()
        self.t.textLine()
        self.t.setFont("Tartan", 10)
        mess = "%1s%-30s" % ("", self.name.disp)
        self.t.textLine(mess)
        mess = "%1s%-30s" % ("", self.add1.disp)
        self.t.textLine(mess)
        mess = "%1s%-30s" % ("", self.add2.disp)
        self.t.textLine(mess)
        mess = "%1s%-30s%39s%7s" % ("", self.add3.disp, "", self.acc.disp)
        self.t.textLine(mess)
        mess = "%1s%-30s" % ("", self.pcod.disp)
        self.t.textLine(mess)
        mess = "%67s%-10s" % ("", self.dated)
        self.t.textLine(mess)
        self.t.textLine()
        self.t.textLine()
        self.t.setFont("Tartan", 10)
        self.pglin = 0

    def pageHeading(self):
        self.c.translate(self.inch, self.inch) # Set x and y margins
        self.t = self.c.beginText()
        self.t.setFont("Tartan-Bold", self.cpi)
        self.t.setTextOrigin(0, 0)

    def printBody(self):
        obal = 0.0
        if len(self.rtt) > (self.maxlines - 2):
            chk = len(self.rtt) - (self.maxlines - 2)
        else:
            chk = 0
        for num, rtt in enumerate(self.rtt):
            trdt = CCD(rtt[self.sql.rcaowt_col.index("rot_trdt")], "d1", 10)
            refno = CCD(rtt[self.sql.rcaowt_col.index("rot_refno")], "Na", 9)
            trtp = CCD(rtt[self.sql.rcaowt_col.index("rot_type")], "UI", 1)
            desc = CCD(rtt[self.sql.rcaowt_col.index("rot_desc")], "NA", 39)
            tramt = CCD(rtt[self.sql.rcaowt_col.index("rot_tramt")], "SD", 11.2)
            if chk:
                if num < chk:
                    obal = float(ASD(obal) + ASD(tramt.work))
                    continue
                if num == chk:
                    obal = CCD(obal, "SD", 11.2)
                    self.t.textLine("%2s%s %-9s %3s %-39s %s" % ("", trdt.disp,
                        "Bal/Bfwd", "", "Balance Brought Forward", obal.disp))
                    self.pglin = self.pglin + 1
            mess = "%2s%s %s %s %s %s" % ("", trdt.disp, refno.disp,
                self.trtypes[trtp.work - 1], desc.disp, tramt.disp)
            self.t.textLine(mess)
            self.pglin = self.pglin + 1

    def printTrailer(self):
        arr = self.sql.getRec(tables="rcatnt",
            cols=["round(sum(rtu_tramt), 2)"], where=[("rtu_cono", "=",
            self.conum), ("rtu_owner", "=", self.acc.work), ("rtu_trdt",
            "<=", self.datew), ("rtu_mtyp", "in", (1, 4))], limit=1)
        if arr and arr[0] > 0:
            trdt = CCD(self.datew, "d1", 10)
            refno = CCD("Arrears", "Na", 9)
            desc = CCD("Tenants In Arrear as per Schedule", "NA", 39)
            tramt = CCD(arr[0], "SD", 11.2)
            self.tots[3] = float(ASD(self.tots[3]) + ASD(tramt.work))
            self.tots[5] = float(ASD(self.tots[5]) + ASD(tramt.work))
            mess = "%2s%s %s %s %s %s" % ("", trdt.disp, refno.disp,
                self.trtypes[3], desc.disp, tramt.disp)
            self.t.textLine(mess)
            self.pglin = self.pglin + 1
        tots = []
        for t in self.tots:
            y = CCD(t, "SD", 11.2)
            tots.append(y.disp)
        mess = "%2s%s %s  %s  %s  %s   %s" % \
            ("", tots[0], tots[1], tots[2], tots[3], tots[4], tots[5])
        for z in range(0, ((self.maxlines + 2) - self.pglin)):
            self.t.textLine()
        self.t.textLine(mess)
        self.t.textLine()
        self.t.textLine()
        if self.mes:
            mes2 = self.mes
        else:
            mes2 = None
        if mes2:
            # Draw Message Rectangles
            self.c.setFillGray(.8)
            x = self.cwidth*.5
            w = self.cwidth*74
            b1 = 56.50
            b2 = 57.30
            self.c.rect(x, (b1*(self.inch/6)), w, (1.25*(self.inch/6)), fill=1)
            self.c.rect(x, (b1*(self.inch/6)), w, (4.75*(self.inch/6)), fill=0)
            self.c.setFillGray(0)
            self.c.setFont("Tartan-Bold", 10)
            self.c.drawString(self.cwidth*1,(b2*(self.inch/6)),"Message")
            self.c.setFont("Tartan", 10)
            self.t.textLine()
            mes3 = textFormat(mes2[0], width=75)
            for mess in mes3:
                self.t.textLine(" " + mess)
        self.c.drawText(self.t)
        self.c.showPage()

    def drawDocument(self):
        # If logo, draw logo
        if self.ctm_logo:
            self.c.drawImage(self.ctm_logo, 0, 3, 350, 70, None)
        # Draw Account Rectangles
        self.c.setFillGray(.8)
        x = self.cwidth*.5
        w = self.cwidth*32.5
        self.c.rect(x, (7.75*(self.inch/6)), w, (1*(self.inch/6)), fill=1)
        self.c.rect(x, (7.75*(self.inch/6)), w, (7.25*(self.inch/6)), fill=0)
        self.c.setFillGray(0)
        self.c.setFont("Tartan-Bold", 8)
        self.c.drawString(self.cwidth*1, (8.45*(self.inch/6)), "Account")
        self.c.setFont("Tartan", 10)
        # Draw Account Rectangles
        self.c.setFillGray(.8)
        x = self.cwidth*64.5
        w = self.cwidth*14
        for q in (11, 13):
            self.c.rect(x, (q*(self.inch/6)), w, (1*(self.inch/6)), fill=1)
            self.c.rect(x, (q*(self.inch/6)), w, (2*(self.inch/6)), fill=0)
        self.c.setFillGray(0)
        self.c.setFont("Tartan-Bold", 8)
        #self.c.drawCentredString((self.cwidth*71.5),
        #    (7.80*(self.inch/6)), "Owner")
        self.c.drawCentredString((self.cwidth*71.5),
            (11.80*(self.inch/6)), "Acc-Num")
        #self.c.drawCentredString((self.cwidth*71.5),
        #    (11.80*(self.inch/6)), "Page")
        self.c.drawCentredString((self.cwidth*71.5),
            (13.80*(self.inch/6)), "Date")
        self.c.setFont("Tartan", 10)
        # Draw Ribbon Rectangles
        self.c.setFillGray(.8)
        x = self.cwidth*.5
        w = self.cwidth*78
        b1 = 36.75
        b2 = 52.25
        self.c.rect(x, (15.50*(self.inch/6)), w, (1.25*(self.inch/6)), fill=1)
        self.c.rect(x, (15.50*(self.inch/6)), w, (b1*(self.inch/6)), fill=0)
        self.c.setFillGray(0)
        self.c.setFont("Tartan-Bold", 10)
        for q, r in ((2, "   Date"), (13, "Reference"), (23, "Typ"),
            (27, "Details"), (67, "    Amount")):
            self.c.drawString(self.cwidth*q, (16.40*(self.inch/6)), r)
        self.c.setFont("Tartan", 10)
        for q in (12.5, 22.5, 26.5, 66.5):
            self.c.line(self.cwidth*q, (15.50*(self.inch/6)), self.cwidth*q,
                (b2*(self.inch/6)))
        # Draw Ageing Rectangles
        self.c.setFillGray(.8)
        x = self.cwidth*.5
        w = self.cwidth*78
        b1 = 52.50
        b2 = 55.25
        b3 = 53.40
        self.c.rect(x, (b1*(self.inch/6)), w, (1.25*(self.inch/6)), fill=1)
        self.c.rect(x, (b1*(self.inch/6)), w, (2.75*(self.inch/6)), fill=0)
        self.c.setFillGray(0)
        self.c.setFont("Tartan-Bold", 10)
        for q, r in ((2, "   Rentals"), (14, "  Receipts"), (26, "   Payments"),
            (39, "   Journals"), (52, "    Deposit"), (65, "   Total-Due")):
            self.c.drawString(self.cwidth*q, (b3*(self.inch/6)), r)
        self.c.setFont("Tartan", 10)
        for q in (13.5, 25.5, 38.5, 51.5, 64.5):
            self.c.line(self.cwidth*q, (b1*(self.inch/6)), self.cwidth*q,
                (b2*(self.inch/6)))

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()

if __name__ == "__main__":
    from TartanClasses import MainFrame
    mf = MainFrame()
    mf.dbm.openDbase()
    rc3050(mf, 1, "Company 1")
