#! /usr/bin/env python

"""
SYNOPSIS
    Rental Tenants Statements.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import os, time
from PIL import Image
from TartanClasses import ASD, CCD, GetCtl, ProgressBar, Sql, TartanDialog
from tartanFunctions import doPrinter, doReportlab, flipImage, getModName
from tartanFunctions import getSingleRecords, showError, textFormat
from tartanWork import rcmvtp, rctrtp

class rc3060:
    def __init__(self, mf, coy, nam, *args):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.buildHead()
            if not args:
                self.mainProcess()
                self.mf.startLoop()
            else:
                self.own, self.prm, self.acc, self.datew, self.dated, \
                    self.prtnam, self.email = args[0]
                self.archive = "N"
                self.zeros = "Y"
                self.minus = "Y"
                self.mes = None
                self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
                    ext="pdf")
                if self.email == "N":
                    self.printSetup()
                whr = [
                    ("rtn_cono", "=", self.conum),
                    ("rtn_owner", "=", self.own),
                    ("rtn_code", "=", self.prm),
                    ("rtn_acno", "=", self.acc)]
                rtm = self.sql.getRec(tables="rcatnm", where=whr, limit=1)
                self.doProcess(rtm)
                if self.email == "N":
                    self.c.save()
                    self.doPrint()

    def setVariables(self):
        gc = GetCtl(self.mf)
        rcactl = gc.getCtl("rcactl", self.conum)
        if not rcactl:
            return "no"
        self.sql = Sql(self.mf.dbm, ["ctlmes", "ctlmst", "rcatnm", "rcatnt",
            "rcaprm", "rcacon"], prog=__name__)
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.title = ("%03i %s" % (self.conum, self.conam),
            "Rental Tenants Statements (%s)" % __name__)
        self.t = None
        self.maxlines = 21

    def buildHead(self):
        self.head = []
        ctl = self.sql.getRec(tables="ctlmst", where=[("ctm_cono", "=",
            self.conum)], limit=1)
        for col in ("ctm_name", "ctm_add1", "ctm_add2", "ctm_add3",
            "ctm_pcode", "ctm_regno", "ctm_taxno", "ctm_tel", "ctm_fax",
            "ctm_b_name", "ctm_b_ibt", "ctm_b_acno", "ctm_logo", "ctm_flip"):
            exec "self.%s = ctl[self.sql.ctlmst_col.index('%s')]" % (col, col)
        if not self.ctm_logo or not os.path.exists(self.ctm_logo):
            self.ctm_logo = None
        elif self.ctm_flip == "N":
            self.ctm_logo = flipImage(self.ctm_logo)
        if self.ctm_logo:
            self.head.append("%37s%9s" % ("", "STATEMENT"))
            self.head.append(" ")
            self.head.append(" ")
            self.head.append(" ")
            self.head.append(" ")
        else:
            mess = "%1s%-30s%6s%9s" % \
                ("", self.ctm_name, "", "STATEMENT")
            self.head.append(mess)
            mess = "%2s%-30s%2s%-30s" % \
                ("", self.ctm_add1, "", ("RegNo: " + self.ctm_regno))
            self.head.append(mess)
            mess = "%2s%-30s%2s%-30s" % \
                ("", self.ctm_add2, "", ("TaxNo: " + self.ctm_taxno))
            self.head.append(mess)
            mess = "%2s%-30s%2s%-30s" % \
                ("", self.ctm_add3, "", ("TelNo: " + self.ctm_tel))
            self.head.append(mess)
            mess = "%2s%-30s%2s%-30s" % \
                ("", self.ctm_pcode, "", ("FaxNo: " + self.ctm_fax))
            self.head.append(mess)

    def mainProcess(self):
        mss = {
            "stype": "R",
            "tables": ("ctlmes",),
            "cols": (
                ("mss_message", "", 0, "Num"),
                ("mss_detail", "NA", 50, "Details")),
            "where": [("mss_system", "=", "RCA")]}
        r1s = (("Yes","Y"),("No","N"))
        fld = [
            [["T",0,0,0],("IRB",r1s),0,"Whole File","Whole File",
                "Y","Y",self.doWhole,None,None,None],
            [["T",0,1,0],("IRB",r1s),0,"Zero Balances",
                "Include Zero Balances","N","Y",self.doZeros,None,None,None],
            [["T",0,2,0],("IRB",r1s),0,"Minus Balances",
                "Include Minus Balances","N","Y",self.doMinus,None,None,None],
            [["T",0,3,0],"IUI",3,"Message Number","Message Number",
                "","Y",self.doMessno,mss,None,("efld",)],
            [["T",0,4,0],"ID1",10,"Statement Date","Statement Date",
                self.sysdtw,"Y",self.doDat,None,None,("efld",)]]
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.title, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("N","Y"), save="N")

    def doWhole(self, frt, pag, r, c, p, i, w):
        self.whole = w

    def doZeros(self, frt, pag, r, c, p, i, w):
        self.zeros = w

    def doMinus(self, frt, pag, r, c, p, i, w):
        self.minus = w

    def doMessno(self, frt, pag, r, c, p, i, w):
        if w:
            self.mes = self.sql.getRec(tables="ctlmes", cols=["mss_detail"],
                where=[("mss_system", "=", "RCA"), ("mss_message", "=", w)],
                limit=1)
            if not self.mes:
                return "Invalid Message Number"
        else:
            self.mes = None

    def doDat(self, frt, pag, r, c, p, i, w):
        self.datew = w
        self.dated = self.df.t_disp[pag][0][p]

    def doEnd(self):
        self.df.closeProcess()
        self.prtnam = self.df.repprt[0][2]
        self.email = self.df.repprt[1][1]
        self.archive = self.df.repprt[2][1]
        if self.archive == "Y":
            dir = self.mf.arcdir
        else:
            dir = self.mf.wrkdir
        self.pdfnam = getModName(dir, __name__, self.conum, ext="pdf")
        if self.email == "N":
            self.printSetup()
        if self.whole == "N":
            all = getSingleRecords(self.mf, "rcatnm",
                ("rtn_owner", "rtn_code", "rtn_acno", "rtn_name"),
                where=[("rtn_cono", "=", self.conum)])
        else:
            whr = [("rtn_cono", "=", self.conum)]
            ord = "rtn_owner, rtn_code, rtn_acno"
            all = self.sql.getRec(tables="rcatnm", where=whr, order=ord)
            if not all:
                showError(self.mf.body, "Error", "No Accounts Selected")
        if all:
            num = len(all)
            p = ProgressBar(self.mf.body, max=num, esc=True)
            for x in range(0, num):
                can = p.displayProgress(x)
                if can == "yes":
                    if self.t:
                        self.t.textLine()
                        self.t.textLine("Print Job ABORTED")
                    break
                self.doProcess(all[x])
            p.closeProgress()
            if self.email == "N":
                self.c.save()
                self.doPrint()
        self.mf.closeLoop()
         
    def doProcess(self, rtm):
        tmc = self.sql.rcatnm_col
        self.own = CCD(rtm[tmc.index("rtn_owner")], "NA", 7)
        self.prm = CCD(rtm[tmc.index("rtn_code")], "NA", 7)
        self.acc = CCD(rtm[tmc.index("rtn_acno")], "NA", 7)
        acc = self.sql.getRec(tables="rcacon", cols=["rcc_cnum", "rcc_deposit",
            "rcc_status"], where=[("rcc_cono", "=", self.conum), ("rcc_owner",
            "=", self.own.work), ("rcc_code", "=", self.prm.work), ("rcc_acno",
            "=", self.acc.work)], order="rcc_cnum")
        if acc:
            dps, dep, sta = acc[-1:][0]
        else:
            dps, dep, sta = 0, 0.0, ""
        if not sta == "C":
            dep = 0.0
            dos = 0.0
        else:
            dos = dep
        self.tots = [0.0, dep, 0.0, 0.0, 0.0, dep]
        tots = self.sql.getRec(tables="rcatnt", cols=["rtu_type", "rtu_mtyp",
            "round(sum(rtu_tramt), 2)"], where=[("rtu_cono", "=", self.conum),
            ("rtu_owner", "=", self.own.work), ("rtu_code", "=", self.prm.work),
            ("rtu_acno", "=", self.acc.work), ("rtu_trdt", "<=", self.datew)],
            group="rtu_type, rtu_mtyp")
        self.dpp = 0.0
        for t in tots:
            if t[1] == 1:
                self.tots[0] = float(ASD(self.tots[0]) + ASD(t[2]))
                self.tots[5] = float(ASD(self.tots[5]) + ASD(t[2]))
            elif t[1] == 2:
                self.dpp = float(ASD(self.dpp) + ASD(t[2]))
                if dep:
                    dos = float(ASD(dos) + ASD(t[2]))
                    self.tots[1] = float(ASD(self.tots[1]) + ASD(t[2]))
                    self.tots[5] = float(ASD(self.tots[5]) + ASD(t[2]))
            elif t[1] == 3:
                self.tots[2] = float(ASD(self.tots[2]) + ASD(t[2]))
                self.tots[5] = float(ASD(self.tots[5]) + ASD(t[2]))
            elif t[1] in (4, 5):
                self.tots[3] = float(ASD(self.tots[3]) + ASD(t[2]))
                self.tots[5] = float(ASD(self.tots[5]) + ASD(t[2]))
            else:
                self.tots[4] = float(ASD(self.tots[4]) + ASD(t[2]))
                self.tots[5] = float(ASD(self.tots[5]) + ASD(t[2]))
        if not dos and not tots:
            return
        if self.zeros == "N" and not dos and not self.tots[5]:
            return
        if self.minus == "N" and not dos and self.tots[5] < 0:
            return
        self.rtt = self.sql.getRec(tables="rcatnt", cols=["rtu_trdt",
            "rtu_refno", "rtu_type", "rtu_desc", "rtu_cnum", "rtu_mtyp",
            "round(sum(rtu_tramt), 2)"], where=[("rtu_cono", "=", self.conum),
            ("rtu_owner", "=", self.own.work), ("rtu_code", "=", self.prm.work),
            ("rtu_acno", "=", self.acc.work), ("rtu_type", "<>", 5),
            ("rtu_trdt", "<=", self.datew)],
            group="rtu_trdt, rtu_refno, rtu_type, rtu_desc, rtu_cnum, rtu_mtyp",
            order="rtu_trdt, rtu_cnum, rtu_refno")
        if not self.rtt:
            self.rtt = []
        if dos > 0:
            self.rtt.append([self.datew, "Deposit", 4, "Outstanding Deposit",
                dps, 2, dos])
        elif dos < 0:
            self.rtt.append([self.datew, "Deposit", 4, "Overpaid Deposit",
                dps, 2, dos])
        if self.email == "Y":
            self.printSetup()
        self.name = CCD(rtm[tmc.index("rtn_name")], "NA", 30)
        self.add1 = CCD(rtm[tmc.index("rtn_addr1")], "NA", 30)
        if self.add1.work:
            self.add2 = CCD(rtm[tmc.index("rtn_addr2")], "NA", 30)
            self.add3 = CCD(rtm[tmc.index("rtn_addr3")], "NA", 30)
            self.pcod = CCD(rtm[tmc.index("rtn_pcode")], "NA", 4)
        else:
            prm = self.sql.getRec(tables="rcaprm", cols=["rcp_addr1",
                "rcp_addr2", "rcp_addr3", "rcp_pcode"], where=[("rcp_cono",
                "=", self.conum), ("rcp_owner", "=", self.own.work),
                ("rcp_code", "=", self.prm.work)], limit=1)
            if not prm:
                prm = ["", "", "", ""]
            self.add1 = CCD(prm[0], "NA", 30)
            self.add2 = CCD(prm[1], "NA", 30)
            self.add3 = CCD(prm[2], "NA", 30)
            self.pcod = CCD(prm[3], "NA", 4)
        self.emadd = CCD(rtm[tmc.index("rtn_email")], "TX", 0)
        self.printHeader()
        self.printBody()
        self.printTrailer()
        if self.email == "Y":
            self.c.save()
            self.doPrint()

    def printSetup(self):
        self.c, self.cpi, self.inch, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam)
        # Width in pixels of each character
        self.cwidth = self.c.stringWidth(("*"), "Tartan", self.cpi)

    def doPrint(self):
        if self.prtnam == "view":
            view = "Y"
        else:
            view = "N"
        if self.email == "N" or not self.emadd.work:
            mail = "N"
        else:
            mail = self.emadd.work
        doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
            head=self.title, view=view, prtnam = self.prtnam, mail=mail)

    def printHeader(self):
        self.pageHeading()
        self.drawDocument()
        self.t.textLine()
        self.t.textLine()
        self.t.setFont("Tartan-Bold", 18)
        self.t.textLine(self.head[0])
        self.t.setFont("Tartan-Bold", 10)
        self.t.textLine(self.head[1])
        self.t.textLine(self.head[2])
        self.t.textLine(self.head[3])
        self.t.textLine(self.head[4])
        self.t.textLine()
        self.t.setFont("Tartan", 10)
        mess = "%-74s%7s" % ("", self.own.disp)
        self.t.textLine(mess)
        mess = "%1s%-30s" % ("", self.name.disp)
        self.t.textLine(mess)
        mess = "%1s%-30s%43s%7s" % ("", self.add1.disp, "", self.prm.disp)
        self.t.textLine(mess)
        mess = "%1s%-30s" % ("", self.add2.disp)
        self.t.textLine(mess)
        mess = "%1s%-30s%43s%7s" % ("", self.add3.disp, "", self.acc.disp)
        self.t.textLine(mess)
        mess = "%1s%-30s" % ("", self.pcod.disp)
        self.t.textLine(mess)
        mess = "%71s%-10s" % ("", self.dated)
        self.t.textLine(mess)
        self.t.textLine()
        self.t.textLine()
        self.t.setFont("Tartan", 10)
        self.pglin = 0

    def pageHeading(self):
        self.c.translate(self.inch, self.inch) # Set x and y margins
        self.t = self.c.beginText()
        self.t.setFont("Tartan-Bold", self.cpi)
        self.t.setTextOrigin(0, 0)

    def printBody(self):
        obal = 0.0
        if len(self.rtt) > (self.maxlines - 1):
            chk = len(self.rtt) - (self.maxlines - 1)
        else:
            chk = 0
        for num, rtt in enumerate(self.rtt):
            trdt = CCD(rtt[0], "d1", 10)
            refno = CCD(rtt[1], "Na", 9)
            trtp = CCD(rtt[2], "UI", 1)
            desc = CCD(rtt[3], "NA", 35)
            cnum = CCD(rtt[4], "UI", 3)
            mtyp = CCD(rtt[5], "UI", 1)
            tramt = CCD(rtt[6], "SD", 11.2)
            if chk:
                if num < chk:
                    if mtyp.work != 2:
                        obal = float(ASD(obal) + ASD(tramt.work))
                    continue
                if num == chk:
                    obal = CCD(obal, "SD", 11.2)
                    self.t.textLine("%2s%s %-9s %3s %3s %-35s %3s %s" % ("",
                        trdt.disp, "Bal/Bfwd", "", "", "Balance Brought "\
                        "Forward", "", obal.disp))
                    self.pglin = self.pglin + 1
            mess = "%2s%s %s %s %s %s %s %s" % ("", trdt.disp, refno.disp,
                rctrtp[trtp.work - 1][0], rcmvtp[mtyp.work - 1][0], desc.disp,
                cnum.disp, tramt.disp)
            self.t.textLine(mess)
            self.pglin = self.pglin + 1

    def printTrailer(self):
        tots = []
        for t in self.tots:
            y = CCD(t, "SD", 11.2)
            tots.append(y.disp)
        mess = "%2s %s  %s  %s  %s  %s     %s" % \
            ("", tots[0], tots[1], tots[2], tots[3], tots[4], tots[5])
        for z in range(0, ((self.maxlines + 2) - self.pglin)):
            self.t.textLine()
        self.t.textLine(mess)
        # Draw Property and Banking Details Block
        self.c.setFillGray(.8)
        x = self.cwidth*.5
        w = self.cwidth*82
        b1 = 42.50
        self.c.rect(x, (b1*(self.inch/6)), w, (1.25*(self.inch/6)), fill=1)
        self.c.rect(x, (b1*(self.inch/6)), w, (4.75*(self.inch/6)), fill=0)
        self.c.setFillGray(0)
        self.c.setFont("Tartan-Bold", 10)
        self.c.drawString(self.cwidth*1, (43.40*(self.inch/6)),
            "Details")
        self.c.drawString(self.cwidth*1, (44.8*(self.inch/6)),
            "Property:%52sDeposit:" % "")
        self.c.drawString(self.cwidth*1, (46.8*(self.inch/6)),
            "Bank:%25sBranch:%19sAccount:" % ("", ""))
        self.c.setFont("Tartan", 10)
        self.t.textLine()
        self.t.textLine()
        self.t.textLine()
        dep = CCD(self.dpp, "SD", 11.2)
        self.t.textLine("%11s%-45s%15s%11s" % ("", self.add1.disp, "",
            dep.disp))
        self.t.textLine()
        self.t.textLine("%7s%-20s%12s%-8s%19s%-16s" % ("", self.ctm_b_name, "",
            self.ctm_b_ibt, "", self.ctm_b_acno))
        self.t.textLine()
        self.t.textLine()
        # Check for Message
        if self.mes:
            mes2 = self.mes[:]
        else:
            mes2 = None
        if mes2:
            # Draw Message Rectangles
            self.c.setFillGray(.8)
            x = self.cwidth*.5
            w = self.cwidth*78
            b1 = 48.50
            b2 = 49.30
            self.c.rect(x, (b1*(self.inch/6)), w, (1.25*(self.inch/6)), fill=1)
            self.c.rect(x, (b1*(self.inch/6)), w, (4.75*(self.inch/6)), fill=0)
            self.c.setFillGray(0)
            self.c.setFont("Tartan-Bold", 10)
            self.c.drawString(self.cwidth*1,(b2*(self.inch/6)),"Message")
            self.c.setFont("Tartan", 10)
            self.t.textLine()
            mes3 = textFormat(mes2[0], width=75)
            for mess in mes3[:3]:
                self.t.textLine(" " + mess)
        self.c.drawText(self.t)
        self.c.showPage()

    def drawDocument(self):
        # If logo, draw logo
        if self.ctm_logo:
            self.c.drawImage(self.ctm_logo, 0, 3, 350, 70, None)
        # Draw Account Rectangles
        self.c.setFillGray(.8)
        x = self.cwidth*.5
        w = self.cwidth*32.5
        self.c.rect(x, (7.75*(self.inch/6)), w, (1*(self.inch/6)), fill=1)
        self.c.rect(x, (7.75*(self.inch/6)), w, (7.25*(self.inch/6)), fill=0)
        self.c.setFillGray(0)
        self.c.setFont("Tartan-Bold", 8)
        self.c.drawString(self.cwidth*1, (8.45*(self.inch/6)), "Account")
        self.c.setFont("Tartan", 10)
        # Draw Account Rectangles
        self.c.setFillGray(.8)
        x = self.cwidth*68.5
        w = self.cwidth*14
        for q in (7, 9, 11, 13):
            self.c.rect(x, (q*(self.inch/6)), w, (1*(self.inch/6)), fill=1)
            self.c.rect(x, (q*(self.inch/6)), w, (2*(self.inch/6)), fill=0)
        self.c.setFillGray(0)
        self.c.setFont("Tartan-Bold", 8)
        self.c.drawCentredString((self.cwidth*75.5),
            (7.80*(self.inch/6)), "Owner")
        self.c.drawCentredString((self.cwidth*75.5),
            (9.80*(self.inch/6)), "Premises")
        self.c.drawCentredString((self.cwidth*75.5),
            (11.80*(self.inch/6)), "Tenant")
        self.c.drawCentredString((self.cwidth*75.5),
            (13.80*(self.inch/6)), "Date")
        self.c.setFont("Tartan", 10)
        # Draw Ribbon Rectangles
        self.c.setFillGray(.8)
        x = self.cwidth*.5
        w = self.cwidth*82
        b1 = 22.75
        b2 = 38.25
        self.c.rect(x, (15.50*(self.inch/6)), w, (1.25*(self.inch/6)), fill=1)
        self.c.rect(x, (15.50*(self.inch/6)), w, (b1*(self.inch/6)), fill=0)
        self.c.setFillGray(0)
        self.c.setFont("Tartan-Bold", 10)
        for q, r in ((2, "   Date"), (13, "Reference"), (23, "Typ"),
            (27, "Acn"), (31, "Details"), (67, "CCC"), (71, "    Amount")):
            self.c.drawString(self.cwidth*q, (16.40*(self.inch/6)), r)
        self.c.setFont("Tartan", 10)
        for q in (12.5, 22.5, 26.5, 30.5, 66.5, 70.5):
            self.c.line(self.cwidth*q, (15.50*(self.inch/6)), self.cwidth*q,
                (b2*(self.inch/6)))
        # Draw Ageing Rectangles
        self.c.setFillGray(.8)
        x = self.cwidth*.5
        w = self.cwidth*82
        b1 = 38.50
        b2 = 41.25
        b3 = 39.40
        self.c.rect(x, (b1*(self.inch/6)), w, (1.25*(self.inch/6)), fill=1)
        self.c.rect(x, (b1*(self.inch/6)), w, (2.75*(self.inch/6)), fill=0)
        self.c.setFillGray(0)
        self.c.setFont("Tartan-Bold", 10)
        for q, r in ((2, "     Rental"), (15, "    Deposit"),
            (28, "       Fees"), (41, "   Services"), (54, "    Repairs"),
            (70, "  Total-Due")):
                self.c.drawString(self.cwidth*q, (b3*(self.inch/6)), r)
        self.c.setFont("Tartan", 10)
        for q in (14.5, 27.5, 40.5, 53.5, 66.5):
            self.c.line(self.cwidth*q, (b1*(self.inch/6)), self.cwidth*q,
                (b2*(self.inch/6)))

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()

if __name__ == "__main__":
    from TartanClasses import MainFrame
    mf = MainFrame()
    mf.dbm.openDbase()
    rc3060(mf, 1, "Company 1")
