#! /usr/bin/env python

"""
SYNOPSIS
    Rentals Tenants Interrogation.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, Balances, CCD, NotesCreate, SChoice, Sql
from TartanClasses import SRec, TartanDialog
from tartanFunctions import dateDiff, doPrinter, getDeposit, getModName
from tartanWork import rcmvtp, rctrtp

class rc4020:
    def __init__(self, mf, coy, nam, usr):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.capnm = usr
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["rcaint", "rcaprm", "rcatnm", "rcacon",
            "rcatnt"], prog=__name__)
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) +  (t[1] * 100) + t[2]
        self.sysdtd = "%i/%02i/%02i" % (t[0], t[1], t[2])
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
                "Rental Tenants Interrogation (%s)" % __name__)
        prm = {
            "stype": "R",
            "tables": ("rcaprm",),
            "cols": (
                ("rcp_owner", "", 0, "Own-Cod"),
                ("rcp_code", "", 0, "Prm-Code"),
                ("rcp_addr1", "", 0, "Address-Line-1")),
            "where": [("rcp_cono", "=", self.conum)],
            "index": 1}
        acc = {
            "stype": "R",
            "tables": ("rcatnm",),
            "cols": (
                ("rtn_acno", "", 0, "Acc-Num"),
                ("rtn_name", "", 0, "Name", "Y")),
            "where": [("rtn_cono", "=", self.conum)],
            "whera": (("T", "rtn_code", 0, 0),)}
        seq = {
            "stype": "R",
            "tables": ("rcacon",),
            "cols": (
                ("rcc_cnum", "", 0, "Acc-Num"),
                ("rcc_payind", "", 0, "F"),
                ("rcc_start", "", 0, "Start-Date"),
                ("rcc_period", "", 0, "Per")),
            "where": [("rcc_cono", "=", self.conum)],
            "whera": [["T", "rcc_code", 0, 0], ["T", "rcc_acno", 1, 0]]}
        tag = (
            ("Premises", self.doTagSelect, ("T",0,2), ("T",0,1)),
            ("Tenant", self.doTagSelect, ("T",0,2), ("T",0,1)),
            ("Balances", self.doTagSelect, ("T",0,2), ("T",0,1)),
            ("Transactions", self.doTrans1, ("T",0,2), ("T",0,1)),
            ("Deposit", self.doDeposit1, ("T",0,2), ("T",0,1)))
        r1s = (
            ("Monthly","M"),
            ("Quarterly","3"),
            ("Bi-Annually", "6"),
            ("Annually","A"))
        fld = (
            (("T",0,0,0),"INA",7,"Premises","Premises Code",
                "","Y",self.doPremises,prm,None,("notblank",)),
            (("T",0,0,0),"INA",7,"Account","Account Code",
                "","N",self.doAccount,acc,None,("notblank",)),
            (("T",0,0,0),"ONA",30,"Name"),
            (("T",1,0,0),"ONA",30,"Description"),
            (("T",1,1,0),"ONA",30,"Address-1"),
            (("T",1,2,0),"ONA",30,"Address-2"),
            (("T",1,3,0),"ONA",30,"Address-3"),
            (("T",1,4,0),"ONA",4,"Postal Code"),
            (("T",1,5,0),"OUD",6.2,"Commission Rate"),
            (("T",2,0,0),"ONA",20,"Telephone Number"),
            (("T",2,1,0),"OTX",50,"E-Mail Address"),
            (("T",2,2,0),"OUA",1,"VAT Indicator"),
            (("T",2,3,0),"ONA",10,"VAT Number"),
            (("T",2,4,0),"OUA",1,"Payment Frequency"),
            (("T",2,5,0),"OD1",10,"Start Date"),
            (("T",2,6,0),"OUI",3,"Number of Periods"),
            (("T",2,7,0),"OUD",12.2,"Rental Amount"),
            (("T",2,8,0),"OUD",12.2,"Deposit Amount"),
            (("T",2,9,0),"OUA",1,"Status"),
            (("C",3,0,0),"OSD",13.2,"Value","",
                "","N",None,None,None,None,("Details",7)),
            (("T",4,0,0),"IUI",3,"Sequence Number","",
                "N","N",self.doTrans2,seq,None,None),
            (("T",5,0,0),"ID1",10,"Effective Date","",
                self.sysdtw,"N",self.doDeposit2,None,None,("efld",)))
        but = (
            ("Clear",None,self.doClear,0,("T",0,0),("T",0,1)),
            ("Notes",None,self.doNotes,0,("T",0,0),("T",0,1)),
            ("Print",None,self.doPrint,0,("T",0,0),("T",0,1)),
            ("Quit",None,self.doExit,1,None,None))
        tnd = ((self.doEndTop, "N"), None, None, None, None, None)
        txt = (self.doExit, None, None, None, None, None)
        self.df = TartanDialog(self.mf, title=self.tit, tags=tag, eflds=fld,
            butt=but, tend=tnd, txit=txt)
        self.df.colLabel[3][0].set_text("Deposit     ")
        self.df.colLabel[3][1].set_text("Rentals     ")
        self.df.colLabel[3][2].set_text("Fees        ")
        self.df.colLabel[3][3].set_text("Services (O)")
        self.df.colLabel[3][4].set_text("Services (A)")
        self.df.colLabel[3][5].set_text("Repairs  (A)")
        self.df.colLabel[3][6].set_text("Balance     ")

    def doPremises(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="rcaprm", where=[("rcp_cono", "=",
            self.conum), ("rcp_code", "=", w)], limit=1)
        if not acc:
            return "Invalid Premises"
        self.code = w
        self.owner = acc[self.sql.rcaprm_col.index("rcp_owner")]
        for num, fld in enumerate(acc):
            if num > 2:
                self.df.loadEntry("T", 1, (num-3), data=fld)

    def doAccount(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="rcatnm", where=[("rtn_cono", "=",
            self.conum), ("rtn_owner", "=", self.owner), ("rtn_code", "=",
            self.code), ("rtn_acno", "=", w)], limit=1)
        if not acc:
            return "Invalid Account"
        self.acno = w
        self.depdtw = self.sysdtw
        self.depdtd = self.sysdtd
        self.name = acc[self.sql.rcatnm_col.index("rtn_name")]
        con = self.sql.getRec(tables="rcacon", where=[("rcc_cono", "=",
            self.conum), ("rcc_owner", "=", self.owner), ("rcc_code", "=",
            self.code), ("rcc_acno", "=", self.acno)], order="rcc_cnum")
        if not con:
            return "Invalid Contract"
        self.df.loadEntry(frt, pag, p+1, data=self.name)
        for num, fld in enumerate(acc[5:]):
            self.df.loadEntry("T", 2, num, data=fld)
        self.cnum = con[-1:][0][4]
        self.df.topf[4][0][5] = self.cnum
        for num, fld in enumerate(con[-1:][0][5:]):
            self.df.loadEntry("T", 2, num+4, data=fld)
        self.loadBalances()
        self.mf.updateStatus("")

    def doEndTop(self):
        self.df.last[0] = [0, 0]
        self.df.setWidget(self.df.topEntry[0][0], state="disabled")

    def loadBalances(self):
        rtl = 0
        dep = 0
        fee = 0
        svo = 0
        sva = 0
        rep = 0
        bal = self.sql.getRec(tables="rcatnt", cols=["rtu_mtyp",
            "round(sum(rtu_tramt), 2)"], where=[("rtu_cono", "=", self.conum),
            ("rtu_owner", "=", self.owner), ("rtu_code", "=", self.code),
            ("rtu_acno", "=", self.acno)], group="rtu_mtyp")
        if bal:
            for a in bal:
                if a[0] == 1:
                    rtl = a[1]
                if a[0] == 2:
                    dep = a[1]
                if a[0] == 3:
                    fee = a[1]
                if a[0] == 4:
                    svo = a[1]
                if a[0] == 5:
                    sva = a[1]
                if a[0] == 6:
                    rep = a[1]
        bal = float(ASD(rtl) + ASD(fee) + ASD(svo) + ASD(sva) + ASD(rep))
        self.df.loadEntry("C", 3, 0, data=dep)
        self.df.loadEntry("C", 3, 1, data=rtl)
        self.df.loadEntry("C", 3, 2, data=fee)
        self.df.loadEntry("C", 3, 3, data=svo)
        self.df.loadEntry("C", 3, 4, data=sva)
        self.df.loadEntry("C", 3, 5, data=rep)
        self.df.loadEntry("C", 3, 6, data=bal)
        self.trn = self.sql.getRec(tables="rcatnt", where=[("rtu_cono", "=",
            self.conum), ("rtu_owner", "=", self.owner), ("rtu_code", "=",
            self.code), ("rtu_acno", "=", self.acno)],
            order="rtu_trdt, rtu_type")

    def doTagSelect(self):
        self.mf.updateStatus("")

    def doTrans1(self, widget=None):
        self.df.focusField("T", 4, 1)

    def doTrans2(self, frt, pag, r, c, p, i, w):
        tit = "Transactions for Account: %s %s %s - %s" % \
            (self.owner, self.code, self.acno, self.name)
        tab = ["rcatnt"]
        col = (
            ("rtu_trdt", "", 0, "   Date"),
            ("rtu_curdt", "", 0, "Curr-Dt"),
            ("rtu_batch", "", 0, "Batch"),
            ("rtu_type", ("XX", rctrtp), 3, "Typ"),
            ("rtu_mtyp", ("XX", rcmvtp), 3, "Mov"),
            ("rtu_refno", "", 0, "Reference", "Y"),
            ("rtu_tramt", "", 0, "     Amount"),
            ("rtu_taxamt", "", 0, " VAT-Amount"),
            ("rtu_desc", "", 0, "Details"))
        whr = [
            ("rtu_cono", "=", self.conum),
            ("rtu_owner", "=", self.owner),
            ("rtu_code", "=", self.code),
            ("rtu_acno", "=", self.acno)]
        if w:
            whr.append(("rtu_cnum", "=", w))
        ord = "rtu_trdt, rtu_type"
        state = self.df.disableButtons()
        rec = SRec(self.mf, screen=self.df.nb.Page3, title=tit, tables=tab,
            cols=col, where=whr, order=ord, rows=12)
        self.df.enableButtons(state=state)
        self.doTrans1()

    def doDeposit1(self, widget=None):
        self.df.focusField("T", 5, 1)

    def doDeposit2(self, frt, pag, r, c, p, i, w):
        self.depdtw = w
        self.depdtd = self.df.t_disp[pag][0][p]
        dat = getDeposit(self.mf, self.conum, self.depdtw, self.owner,
            self.code, self.acno)
        if not dat:
            return
        tit = "Deposit for Account: %s %s %s - %s as at %s" % (self.owner,
            self.code, self.acno, self.name, self.depdtd)
        col = (
            ("a", "D1", 10, "    Date"),
            ("b", "SD", 11.2, "  Deposits"),
            ("c", "UI", 5, " Days"),
            ("d", "UD", 5.2, " Rate"),
            ("e", "SD", 11.2, "  Interest"),
            ("f", "SD", 11.2, "   Balance"),
            ("g", "SD", 11.2, "     Admin"))
        state = self.df.disableButtons()
        rec = SRec(self.mf, screen=self.df.nb.Page4, title=tit, cols=col,
            where=dat, wtype="D")
        self.df.enableButtons(state=state)
        self.doDeposit1()

    def doNotes(self, widget=None):
        state = self.df.disableButtons()
        self.df.mstAlw.hide()
        nf = NotesCreate(self.mf, self.conum, self.conam, self.capnm, "RCA",
            "%7%7s%s" % (self.owner, self.code, self.acno))
        self.df.mstAlw.show()
        self.df.enableButtons(state=state)

    def doClear(self, widget=None):
        self.df.selPage("Premises")
        self.df.focusField("T", 0, 1)

    def doPrint(self, widget=None):
        titl = "Select the required print option!"
        data = (
            ("Info Only", "I"),
            ("Deposit Only", "D"),
            ("Transactions Only", "T"),
            ("Info and Transactions", "A"))
        ss = SChoice(self.mf, titl=titl, data=data)
        self.mf.updateStatus("")
        if not ss.selection:
            self.pr_opts = " "
        else:
            self.pr_opts = ss.selection[1]
        if self.pr_opts != " ":
            self.printDetail()
        self.df.selPage("Premises")
        self.df.focusField("T", 1, 1)

    def printDetail(self):
        self.head = ("%03u %-30s %63s %10s" % \
            (self.conum, self.conam, self.sysdttm, __name__))
        from tartanFunctions import doReportlab
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        self.pgnum = 0
        self.pglin = 99
        if self.pr_opts not in ("D", "T"):
            self.pageHeading()
            self.printInfo()
        if self.pr_opts == "I":
            self.c.drawText(self.t)
            self.c.showPage()
            self.c.save()
            doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                head=self.tit, view="V")
        elif self.pr_opts == "A":
            if not self.trn:
                self.c.drawText(self.t)
                self.c.showPage()
                self.c.save()
                doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                    head=self.tit, view="V")
            else:
                self.c.drawText(self.t)
                self.c.showPage()
                self.pageHeading()
                self.pageHeadingTrans()
                self.printTrans()
                self.c.drawText(self.t)
                self.c.showPage()
                self.c.save()
                doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                    head=self.tit, view="V")
        elif self.pr_opts == "D":
            all = getDeposit(self.mf, self.conum, self.depdtw, self.owner,
                self.code, self.acno)
            if all:
                self.pageHeading(deposit=True)
                self.pageHeadingDeposit()
                self.printDeposit(all)
                self.c.drawText(self.t)
                self.c.showPage()
                self.c.save()
                doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                    head=self.tit, view="V")
        elif self.pr_opts == "T":
            if self.trn:
                self.pageHeading()
                self.pageHeadingTrans()
                self.printTrans()
                self.c.drawText(self.t)
                self.c.showPage()
                self.c.save()
                doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                    head=self.tit, view="V")

    def printInfo(self):
        for x in range(0, len(self.df.topf[0])):
            self.t.textLine("%-26s%-25s %s" % ("",
                self.df.topf[0][x][3], self.df.t_disp[0][0][x]))
        self.t.textLine()
        for x in range(0, len(self.df.topf[1])):
            self.t.textLine("%-26s%-25s %s" % ("",
                self.df.topf[1][x][3], self.df.t_disp[1][0][x]))
        self.t.textLine()
        for x in range(0, len(self.df.topf[2])):
            self.t.textLine("%-26s%-25s %s" % ("",
                self.df.topf[2][x][3], self.df.t_disp[2][0][x]))
        self.t.textLine()
        self.pglin = self.pglin + 3 + len(self.df.topf[3])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.t.textLine("%26s%-34s %13s" % ("", "Details","Amount "))
        self.t.textLine("%26s%s" % ("", self.suc * 48))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        desc = (
            "Deposit     ",
            "Rentals     ",
            "Fees        ",
            "Services (O)",
            "Services (A)",
            "Repairs  (A)",
            "Balance     ")
        for n, d in enumerate(self.df.c_disp[3]):
            self.t.textLine("%26s%-34s %13s" % ("", desc[n], d[0]))

    def printDeposit(self, all):
        tot1 = 0
        tot2 = 0
        tot3 = 0
        for ct in all:
            date = CCD(ct[0], "D1", 10)
            move = CCD(ct[1], "SD", 13.2)
            days = CCD(ct[2], "UI", 5)
            rate = CCD(ct[3], "UD", 5.2)
            intr = CCD(ct[4], "SD", 13.2)
            bals = CCD(ct[5], "SD", 13.2)
            admn = CCD(ct[6], "SD", 13.2)
            tot1 = float(ASD(tot1) + ASD(move.work))
            tot2 = float(ASD(tot2) + ASD(intr.work))
            tot3 = float(ASD(tot3) + ASD(admn.work))
            if self.pglin > self.lpp:
                if self.pgnum != 0:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
                self.pageHeadingDeposit()
            self.t.textLine("%s %s %s %s %s %s %s" % (date.disp, move.disp,
                days.disp, rate.disp, intr.disp, bals.disp, admn.disp))
            self.pglin = self.pglin + 1
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.t.textLine("%s" % (self.suc * len(self.head)))
        move = CCD(tot1, "SD", 13.2)
        intr = CCD(tot2, "SD", 13.2)
        admn = CCD(tot3, "SD", 13.2)
        self.t.textLine("%-10s %s %11s %s %s %s" % ("Totals", move.disp, "",
            intr.disp, bals.disp, admn.disp))

    def printTrans(self):
        col = self.sql.rcatnt_col
        bal = 0
        for ct in self.trn:
            trdt = CCD(ct[col.index("rtu_trdt")], "D1", 10)
            refno = CCD(ct[col.index("rtu_refno")], "Na", 9)
            trtp = CCD(ct[col.index("rtu_type")], "UI", 1)
            mtyp = CCD(ct[col.index("rtu_mtyp")], "UI", 1)
            batch = CCD(ct[col.index("rtu_batch")], "Na", 7)
            desc = CCD(ct[col.index("rtu_desc")], "NA", 30)
            tramt = ct[col.index("rtu_tramt")]
            if tramt < 0:
                debit = CCD(0, "SD", 13.2)
                credit = CCD(tramt, "SD", 13.2)
            else:
                debit = CCD(tramt, "SD", 13.2)
                credit = CCD(0, "SD", 13.2)
            if mtyp.work != 2:
                bal = float(ASD(bal) + ASD(tramt))
            tot = CCD(bal, "SD", 13.2)
            taxamt = CCD(ct[col.index("rtu_taxamt")], "SD", 13.2)
            if self.pglin > self.lpp:
                if self.pgnum != 0:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
                self.pageHeadingTrans()
            self.t.textLine("%s %s %s %s %s %s %s %s %s %s" % (trdt.disp,
                refno.disp, rctrtp[(trtp.work - 1)][0],
                rcmvtp[(mtyp.work - 1)][0], batch.disp, desc.disp,
                debit.disp, credit.disp, tot.disp, taxamt.disp))
            self.pglin = self.pglin + 1

    def pageHeading(self, deposit=False):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        if deposit:
            self.t.textLine("%-42s %-10s %49s %5s" % \
                ("Rentals Ledger Deposit Interrogation as at", self.depdtd,
                "Page", self.pgnum))
        else:
            self.t.textLine("%-34s %-10s %57s %5s" % \
                ("Rentals Ledger Interrogation as at", self.sysdtd,
                "Page", self.pgnum))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 6

    def pageHeadingDeposit(self):
        self.t.textLine("%-16s %s %s %s %s %s %s %s %s" % ("",
            self.df.topf[0][0][3], self.df.t_disp[0][0][0], "",
            self.df.topf[0][1][3], self.df.t_disp[0][0][1], "",
            self.df.topf[0][2][3], self.df.t_disp[0][0][2]))
        self.t.textLine()
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.t.textLine("%-10s %-13s %-5s %-5s %-13s %-13s %-13s" % ("   Date",
            "    Deposits", " Days", " Rate", "    Interest", "     Balance",
            "       Admin"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 8

    def pageHeadingTrans(self):
        self.t.textLine("%-16s %s %s %s %s %s %s %s %s" % ("",
            self.df.topf[0][0][3], self.df.t_disp[0][0][0], "",
            self.df.topf[0][1][3], self.df.t_disp[0][0][1], "",
            self.df.topf[0][2][3], self.df.t_disp[0][0][2]))
        self.t.textLine()
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.t.textLine("%-10s %-9s %-3s %-3s %-7s %-30s %-13s %-13s %-13s" % \
            ("   Date", "Reference", "Typ", "Mov", "Batch", "Remarks",
            "       Debit", "      Credit",  "     Balance"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 8

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

if __name__ == "__main__":
    from TartanClasses import MainFrame
    mf = MainFrame()
    mf.dbm.openDbase()
    rc4020(mf, 1, "Company 1", "paul")
