#! /usr/bin/python

"""
SYNOPSIS
    CSV File Imports.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, TartanDialog, Sql
from tartanFunctions import askQuestion

class rc6020:
    def __init__(self, mf):
        self.mf = mf
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlmst", "rcaowt", "rcatnm", "rcacon",
            "rcatnt"], prog=__name__)
        self.commit = False
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]

    def mainProcess(self):
        tit = ("%s" % "CSV File Imports",)
        fle = {
            "stype": "F",
            "types": "fle",
            "ftype": (("CSV Files", "*.csv"),)}
        coy = {
            "stype": "R",
            "tables": ("ctlmst",),
            "cols": (
                ("ctm_cono", "", 0, "Coy"),
                ("ctm_name", "", 0, "Name", "Y"))}
        r1s = (("Owner", "O"), ("Tenants", "T"))
        r2s = (("Yes", "Y"), ("No", "N"))
        fld = (
            (("T",0,0,0),"INA",50,"File Name","File Name",
                "","N",self.doFile,fle,None,("fle",)),
            (("T",0,1,0),("IRB",r1s),0,"File Type","File Type",
                "O","N",self.doType,None,None,None),
            (("T",0,2,0),"IUI",3,"Company Number","Company Number",
                1,"N",self.doCono,coy,None,("notzero",)),
            (("T",0,3,0),("IRB",r2s),0,"Delete Existing","Delete Existing",
                "N","N",self.doDelete,None,None,None))
        tnd = ((self.doEnd,"y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tend=tnd,
            txit=txt)

    def doFile(self, frt, pag, r, c, p, i, w):
        self.file = w

    def doType(self, frt, pag, r, c, p, i, w):
        self.atyp = w

    def doCono(self, frt, pag, r, c, p, i, w):
        self.cono = w
        acc = self.sql.getRec(tables="ctlmst", where=[("ctm_cono", "=",
            self.cono)], limit=1)
        if not acc:
            return "Invalid Company"

    def doDelete(self, frt, pag, r, c, p, i, w):
        self.delete = w

    def doEnd(self):
        if self.delete == "Y":
            if self.atyp == "O":
                self.sql.delRec("rcaowt", where=[("rot_cono", "=", self.cono)])
            if self.atyp == "T":
                self.sql.delRec("rcatnt", where=[("rtu_cono", "=", self.cono)])
                self.sql.delRec("rcaowt", where=[("rot_cono", "=", self.cono),
                    ("rot_type", "=", 1)])
        data = open(self.file, "r")
        raw = data.readlines()
        for line in raw:
            for c in (10, 12, 13):
                line = line.replace(chr(c), "")
            line = line.split(",")
            if self.atyp == "O":
                if len(line) >  6:
                    yes = askQuestion(self.mf.body, "Owner Error",
                        "%s\n\nContinue??????" % line)
                    if yes == "no":
                        self.mf.dbm.rollbackDbase()
                        break
                else:
                    data = [self.cono]
                    data.extend(line)
                    data.extend(["","","","","","","","","",0])
                    self.sql.insRec("rcaowm", data=data)
                    self.commit = True
            elif self.atyp == "T":
                if len(line) !=  7:
                    yes = askQuestion(self.mf.body, "Tenant Error",
                        "%s\n\nContinue??????" % line)
                    if yes == "no":
                        self.mf.dbm.rollbackDbase()
                        break
                else:
                    data = [self.cono]
                    data.extend(line[:3])
                    data.append(1)
                    data.append(4)
                    data.append("Take/On")
                    data.append("")
                    data.append(20070720)
                    if line[3]:
                        self.sql.updRec("rcacon", cols=["rcc_deposit"],
                            data=[line[3]], where=[("rcc_cono", "=", self.cono),
                            ("rcc_owner", "=", line[0]), ("rcc_code", "=",
                            line[1]), ("rcc_acno", "=", line[2]), ("rcc_cnum",
                            "=", 1)])
                    if line[4]:
                        rec = [self.cono, line[0], line[1], line[2], 1, 1,
                            "Take/On", "", 20070720, 1, line[4], 0.0, 200707,
                            "Initial Takeon", "", "", self.sysdtw, 0]
                        self.sql.insRec("rcatnt", data=rec)
                        amt = float(ASD(0) - ASD(line[4]))
                        rec = [self.cono, line[0], 1, "Take/On", "", 20070720,
                            amt, 0.0, 200707, "Initial Takeon", "", "",
                            self.sysdtw, 0]
                        self.sql.insRec("rcaowt", data=rec)
                    if line[5]:
                        rec = [self.cono, line[0], line[1], line[2], 1, 4,
                            "Take/On", "", 20070720, 5, line[5], 0.0, 200707,
                            "Initial Takeon", "", "", self.sysdtw, 0]
                        self.sql.insRec("rcatnt", data=rec)
                    self.commit = True
        yes = askQuestion(self.mf.body, "More",
            "Would You Like To Import Another File?")
        if yes == "no":
            self.doExit()
        else:
            self.df.focusField("T", 0, 1)

    def doExit(self):
        if self.commit:
            self.mf.dbm.commitDbase(ask=True)
        self.df.closeProcess()
        self.mf.closeLoop()

if __name__ == "__main__":
    from TartanClasses import MainFrame
    mf = MainFrame()
    mf.dbm.openDbase()
    rpt = rc6020(mf)
