#! /usr/bin/python

"""
SYNOPSIS
    Rentals Missing Masterfile Records.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import ScrollText, Sql

class rc6030:
    def __init__(self, mf):
        self.mf = mf
        self.setVariables()
        self.mainProcess()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["rcaowm", "rcaprm", "rcatnm", "rcacon"],
            prog=__name__)
        self.chk = [
            [                                                       # 0
                ["rcaprm", ["rcp_cono", "rcp_owner"]],              # 0,0
                ["rcaowm", ["rom_cono", "rom_acno"]]],              # 0,1
            [                                                       # 1
                ["rcatnm", ["rtn_cono", "rtn_owner"]],              # 1,0
                ["rcaowm", ["rom_cono", "rom_acno"]]],              # 1,1
            [
                ["rcacon", ["rcc_cono", "rcc_owner"]],
                ["rcaowm", ["rom_cono", "rom_acno"]]],
            [
                ["rcaprm", ["rcp_cono", "rcp_owner", "rcp_code"]],
                ["rcatnm", ["rtn_cono", "rtn_owner", "rtn_code"]]],
            [
                ["rcatnm", ["rtn_cono", "rtn_owner", "rtn_code", "rtn_acno"]],
                ["rcacon", ["rcc_cono", "rcc_owner", "rcc_code", "rcc_acno"]]]]

    def mainProcess(self):
        miss = {}
        for c in self.chk:
            rec = self.sql.getRec(tables=c[0][0], cols=c[0][1])
            if not rec:
                continue
            for r in rec:
                whr = []
                for n, w in enumerate(c[1][1]):
                    whr.append((w, "=", r[n]))
                acc = self.sql.getRec(tables=c[1][0], where=whr, limit=1)
                if not acc:
                    if not c[1][0] in miss:
                        miss[c[1][0]] = [r]
                    elif not r in miss[c[1][0]]:
                        miss[c[1][0]].append(r)
        missing = ""
        for key in (("rcaowm", "Owner"), ("rcaprm", "Premises")):
            if key[0] in miss:
                for r in miss[key[0]]:
                    missing = missing + "Missing %s %s\n" % (key[1], r)
        st = ScrollText(scrn=self.mf.body, title="Missing Records",
            mess=missing)

    def closeProcess(self):
        self.mf.closeLoop()

if __name__ == "__main__":
    from TartanClasses import MainFrame
    mf = MainFrame()
    mf.dbm.openDbase()
    rc6030(mf)
