#! /usr/bin/env python

"""
SYNOPSIS
    Rentals Ledger Month End Routine.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import copy, time
from TartanClasses import ASD, CCD, GetCtl, Sql, TartanDialog
from tartanFunctions import askQuestion, mthendDate, projectDate, showError
from tartanWork import mthnam

class rcm010:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        gc = GetCtl(self.mf)
        rcactl = gc.getCtl("rcactl", self.conum)
        if not rcactl:
            return "no"
        self.lme = rcactl["cte_lme"]
        self.glint = rcactl["cte_glint"]
        if self.glint == "Y":
            self.glbnk = rcactl["cte_glbnk"]
            ctlctl = gc.getCtl("ctlctl", self.conum)
            if not ctlctl:
                return "no"
            ctls = ["rca_com", "rca_own", "rca_tnt", "vat_ctl"]
            if gc.chkRec(self.conum, ctlctl, ctls):
                return "no"
            self.rcacom = ctlctl["rca_com"]
            self.rcaown = ctlctl["rca_own"]
            self.rcatnt = ctlctl["rca_tnt"]
            self.convat = ctlctl["vat_ctl"]
        # Check for Company Record
        ctlmst = gc.getCtl("ctlmst", self.conum)
        if not ctlmst:
            return "no"
        # Set Company VAT Default
        self.taxdf = ctlmst["ctm_taxdf"]
        if not self.taxdf:
            self.taxdf = "N"
        self.sql = Sql(self.mf.dbm, ["ctlvtf", "gentrn", "rcactl", "rcaowm",
            "rcaowt", "rcatnm", "rcacon", "rcatnt"], prog=__name__)
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.tme = mthendDate(self.sysdtw)

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "Rentals Month End Routine (%s)" % __name__)
        fld = (
            (("T",0,0,0),"Od1",10,"Last Month End Date"),
            (("T",0,1,0),"ID1",10,"This Month End Date","This Month End Date",
                self.tme,"N",self.doTme,None,None,("efld",)))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.closeProcess,)
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tend=tnd,
            txit=txt, focus=False)
        self.df.loadEntry("T", 0, 0, data=self.lme)
        self.df.focusField("T", 0, 2)

    def doTme(self, frt, pag, r, c, p, i, w):
        if w <= self.lme:
            return "Invalid Month-End Date"
        self.tme = w
        self.curdt = w / 100
        yy = self.curdt / 100
        mm = (self.curdt % 100) + 1
        while mm > 12:
            yy += 1
            mm -= 12
        self.nxtdt = (yy * 10000) + (mm * 100) + 1
        self.nxtcd = self.nxtdt / 100
        self.batch = "R%06s" % self.nxtcd
        chk = self.sql.getRec(tables="rcatnt", cols=["count(*)"],
            where=[("rtu_cono", "=", self.conum), ("rtu_type", "=", 1),
            ("rtu_batch", "=", self.batch), ("rtu_trdt", "=", self.nxtdt),
            ("rtu_curdt", "=", self.nxtcd)], limit=1)
        if chk[0]:
            return "Month-End Already Executed"

    def doEnd(self):
        self.updateTables()

    def updateTables(self):
        self.df.closeProcess()
        all = self.sql.getRec(tables=["rcacon", "rcatnm"], cols=["rcc_cono",
            "rcc_owner", "rcc_code", "rcc_acno", "rcc_cnum", "rcc_payind",
            "rcc_start", "rcc_period", "rcc_rtlamt", "rcc_deposit", "rtn_name"],
            where=[("rcc_cono", "=", self.conum), ("rtn_cono=rcc_cono",),
            ("rtn_owner=rcc_owner",), ("rtn_code=rcc_code",),
            ("rtn_acno=rcc_acno",), ("rcc_status", "<>", "X")])
        if all:
            for num, self.con in enumerate(all):
                own = self.sql.getRec(tables="rcaowm", cols=["rom_vatdf"],
                    where=[("rom_cono", "=", self.conum), ("rom_acno", "=",
                    self.con[1])], limit=1)
                if not own:
                    showError(self.mf.body, "Missing Owner", "This Owner's "\
                        "Record (%s) Could Not Be Found!" % self.con[1])
                    continue
                self.vatdf = own[0]
                self.prm = self.sql.getRec(tables="rcaprm", cols=["rcp_crate"],
                    where=[("rcp_cono", "=", self.conum), ("rcp_owner", "=",
                    self.con[1]), ("rcp_code", "=", self.con[2])], limit=1)
                if not self.prm:
                    showError(self.mf.body, "Missing Premises", "This "\
                        "Premises's Record (%s %s) Could Not Be Found!" % \
                        (self.con[1], self.con[2]))
                    continue
                self.freq = self.con[5]
                strt = self.con[6]
                self.pers = self.con[7]
                self.amnt = self.con[8]
                self.depo = self.con[9]
                if self.freq == "M":
                    mths = 1 * self.pers
                elif self.freq == "3":
                    mths = 3 * self.pers
                elif self.freq == "6":
                    mths = 6 * self.pers
                else:
                    mths = 12 * self.pers

                yy = strt / 10000
                mm = ((strt % 10000) / 100) + mths - 1
                while mm < 1:
                    yy -= 1
                    mm += 12
                while mm > 12:
                    yy += 1
                    mm -= 12
                chkmth = copy.deepcopy(mthnam)
                if mm == 2:
                    if (yy % 4) == 0:
                        chkmth[2][2] = 29
                    else:
                        chkmth[2][2] = 28
                self.exdt = (yy * 10000) + (mm * 100) + chkmth[mm][2]
                if (self.exdt / 100) <= self.curdt:
                    self.sql.updRec("rcacon", cols=["rcc_status"], data=["X"],
                        where=[("rcc_cono", "=", self.con[0]), ("rcc_owner",
                        "=", self.con[1]), ("rcc_code", "=", self.con[2]),
                        ("rcc_acno", "=", self.con[3]), ("rcc_cnum", "=",
                        self.con[4])])
                    ok = askQuestion(self.mf.body, "Expired Contract",
                        "This contract has Expired, would you like to "\
                        "Renew it?\n\nOwner: %s\nPremises: %s\nAccount: %s\n"\
                        "Name: %s" % (self.con[1], self.con[2], self.con[3],
                        self.con[10]))
                    if ok == "no":
                        continue
                    self.doExpiredContract()
                self.doRaiseRental(num)
        self.sql.updRec("rcactl", cols=["cte_lme"], data=[self.tme],
            where=[("cte_cono", "=", self.conum)])
        self.mf.dbm.commitDbase(ask=True, mess="Do You Want To Save All "\
            "Changes?\n\nPlease Note That Once The Changes Have Been "\
            "Saved, There Is No Going Back Without Restoring From Backup!")
        self.mf.closeLoop()

    def doExpiredContract(self):
        tit = ("Renew Rental Contract", )
        r1s = (
            ("Monthly","M"),
            ("Quarterly","3"),
            ("Bi-Annually", "6"),
            ("Annually","A"))
        rndt = projectDate(self.exdt, 1)
        self.fld = (
            (("T",0,0,0),"OUA",7,"Owner Code"),
            (("T",0,1,0),"OUA",7,"Premises Code"),
            (("T",0,2,0),"ONA",7,"Account Code"),
            (("T",0,3,0),"ONA",30,"Tenant Name"),
            (("T",0,4,0),("IRB",r1s),0,"Payment Frequency","Payment Frequency",
                self.freq,"N",None,None,None,None),
            (("T",0,5,0),"ID1",10,"Start Date","Start Date",
                rndt,"N",None,None,None,("efld",)),
            (("T",0,6,0),"IUI",3,"Number of Periods","Number of Periods",
                self.pers,"N",None,None,None,("notzero",)),
            (("T",0,7,0),"IUD",12.2,"Rental Amount","Rental Amount",
                self.amnt,"N",None,None,None,("notzero",)),
            (("T",0,8,0),"IUD",12.2,"Deposit Amount","Deposit Amount",
                self.depo,"N",None,None,None,("efld",)))
        tnd = ((self.doExpireEnd,"y"), )
        txt = (self.doLoadFocus, )
        self.na = TartanDialog(self.mf, newscn=(0,1), tops=True, title=tit,
            eflds=self.fld, tend=tnd, txit=txt, focus=False)
        self.doLoadFocus()
        self.mf.startLoop()

    #def doDeposit(self, frt, pag, r, c, p, i, w):
    #    self.diff = float(ASD(w) - ASD(self.depo))

    def doExpireEnd(self):
        dat = self.con[:4]
        dat.append(self.con[4] + 1)
        for x in range(4, len(self.na.t_work[0][0])):
            dat.append(self.na.t_work[0][0][x])
        dat.append("C")
        self.sql.insRec("rcacon", data=dat)
        self.con[:9] = dat[:9]
        self.na.closeProcess()
        self.mf.closeLoop()

    def doRaiseRental(self, num):
        ref = CCD((num + 1), "Na", 9).work
        amt = CCD(self.con[8], "UD", 12.2).work
        acc = self.sql.getRec(tables=["ctlvmf"], cols=["vtm_new"],
            where=[("vtm_cono", "=", self.conum), ("vtm_code", "=",
            self.vatdf)], limit=1)
        if not acc:
            rate = 0
        else:
            rate = float(acc[0])
        vat = CCD(round((amt * rate / 100), 2), "SD", 11.2).work
        tot = float(ASD(amt) + ASD(vat))
        data = self.con[:5]
        data.extend([1, ref, self.batch, self.nxtdt, 1, tot, vat, self.nxtcd,
            "Rental Raised", self.vatdf, "", self.sysdtw, 0])
        self.sql.insRec("rcatnt", data=data)
        if self.glint == "Y":
            # Update Tenant Control
            gld = "%7s Rental Raised" % self.con[3]
            data = [self.conum, self.rcatnt, self.nxtcd, self.nxtdt, 4, ref,
                self.batch, tot, 0, gld, "", "", 0, self.sysdtw, 0]
            self.sql.insRec("gentrn", data=data)
        rta = float(ASD(0) - ASD(tot))
        rtv = float(ASD(0) - ASD(vat))
        data = self.con[:2]
        data.extend([1, ref, self.batch, self.nxtdt, rta, rtv, self.nxtcd,
            "Rental on %s Premises" % self.con[2], self.vatdf, "",
            self.sysdtw, 0])
        self.sql.insRec("rcaowt", data=data)
        if self.glint == "Y":
            # Update Owner Control
            gld = "%7s Rental Raised" % self.con[1]
            data = [self.conum, self.rcaown, self.nxtcd, self.nxtdt, 4, ref,
                self.batch, rta, 0, gld, "", "", 0, self.sysdtw, 0]
            self.sql.insRec("gentrn", data=data)
        if self.prm[0]:
            cma = round((tot * self.prm[0] / 100.0), 2)
            acc = self.sql.getRec(tables=["ctlvmf"], cols=["vtm_new"],
                where=[("vtm_cono", "=", self.conum), ("vtm_code", "=",
                self.taxdf)], limit=1)
            if not acc:
                rate = 0
            else:
                rate = float(acc[0])
            cmv = round((cma * rate / 100.0), 2)
            cmt = float(ASD(cma) + ASD(cmv))
            cma = float(ASD(0) - ASD(cma))
            tax = float(ASD(0) - ASD(cmv))
            des = "Commission @ %3.5s%s Inclusive" % (self.prm[0], "%")
            data = self.con[:2]
            data.extend([4, ref, self.batch, self.nxtdt, cmt, cmv, self.nxtcd,
                des, self.vatdf, "", self.sysdtw, 0])
            self.sql.insRec("rcaowt", data=data)
            if self.glint == "Y":
                # Update Owner Control
                gld = "%7s Commission Raised" % self.con[1]
                data = [self.conum, self.rcaown, self.nxtcd, self.nxtdt, 4,
                    ref, self.batch, cmt, 0, gld, "", "", 0, self.sysdtw, 0]
                self.sql.insRec("gentrn", data=data)
                # Update Commission Account
                data = [self.conum, self.rcacom, self.nxtcd, self.nxtdt, 4,
                    ref, self.batch, cma, tax, gld, "", "", 0, self.sysdtw, 0]
                self.sql.insRec("gentrn", data=data)
            if self.taxdf:
                # VAT Transaction (ctlvtf)
                data = [self.conum, self.taxdf, "O", self.nxtcd, "R", 1,
                    self.batch, ref, self.nxtdt, self.con[3], self.con[10],
                    cma, tax, self.sysdtw, 0, 0]
                self.sql.insRec("ctlvtf", data=data)
                if self.glint == "Y" and tax:
                    # Update VAT Control
                    data = [self.conum, self.convat, self.nxtcd, self.nxtdt, 4,
                        ref, self.batch, tax, 0, gld, "", "", 0, self.sysdtw, 0]
                    self.sql.insRec("gentrn", data=data)

    def doLoadFocus(self):
        self.na.loadEntry("T", 0, 0, data=self.con[1])
        self.na.loadEntry("T", 0, 0, data=self.con[2])
        self.na.loadEntry("T", 0, 1, data=self.con[3])
        self.na.loadEntry("T", 0, 2, data=self.con[10])
        self.na.focusField("T", 0, 4)

    def closeProcess(self):
        self.df.closeProcess()
        self.mf.closeLoop()

