#! /usr/bin/env python

"""
SYNOPSIS
    Rentals Ledger Master Listing.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import copy, time
from TartanClasses import ASD, CCD, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, getModName, doPrinter, showError
from tartanWork import mthnam

class rt3030:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["rtlmst", "rtlcon"], prog=__name__)
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdtd = "%i/%02i/%02i" % (t[0], t[1], t[2])
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Rentals Ledger Master Listing (%s)" % __name__)
        r1s = (("Yes","Y"),("No","N"))
        r2s = (("All","Y"),("Arrears","A"),("Expiring","E"),("Expired","X"))
        fld = (
            (("T",0,0,0),"ID1",10,"Report Date","Report Date",
                self.sysdtw,"Y",self.doDate,None,None,("efld",)),
            (("T",0,1,0),("IRB",r2s),0,"Contracts","Contracts to Print",
                "Y","Y",self.doType,None,None,None),
            (("T",0,2,0),"IUI",2,"Months to Expiry","Months to Expiry",
                1,"Y",self.doMonths,None,None,("notzero",)),
            (("T",0,3,0),("IRB",r1s),0,"Consolidate","Consolidate",
                "N","Y",self.doCons,None,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doDate(self, frt, pag, r, c, p, i, w):
        self.date = w
        self.datd = self.df.t_disp[0][0][0]

    def doType(self, frt, pag, r, c, p, i, w):
        self.styp = w
        if self.styp != "E":
            self.months = 0
            return "sk1"

    def doMonths(self, frt, pag, r, c, p, i, w):
        self.months = w
        if self.styp != "A":
            self.cons = "N"
            return "sk1"

    def doCons(self, frt, pag, r, c, p, i, w):
        self.cons = w

    def doEnd(self):
        self.df.closeProcess()
        ord = "rtc_code, rtc_acno, rtc_cnum"
        whr = [("rtc_cono", "=", self.conum), ("rtc_start", "<=", self.date)]
        if self.styp == "X":
            whr.append(("rtc_status", "=", "X"))
        if self.cons == "Y":
            whr.append(("rtc_cnum", "=", 1))
        all = self.sql.getRec(tables="rtlcon", where=whr, order=ord)
        if not all:
            showError(self.mf.body, "Selection Error",
            "No Records Selected")
        else:
            self.printReport(all)
        self.closeProcess()

    def printReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        self.head = ("%03u %-30s %103s %10s" % \
            (self.conum, self.conam, self.sysdttm, __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        self.pgnum = 0
        self.pglin = 99
        mst = self.sql.rtlmst_col
        con = self.sql.rtlcon_col
        self.tots = [0, 0, 0, 0, 0]
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                self.t.textLine()
                self.t.textLine("Print Job ABORTED")
                break
            code = CCD(dat[con.index("rtc_code")], "NA", 7)
            acno = CCD(dat[con.index("rtc_acno")], "NA", 7)
            cnum = CCD(dat[con.index("rtc_cnum")], "UI", 3)
            freq = CCD(dat[con.index("rtc_payind")], "UA", 1)
            strt = CCD(dat[con.index("rtc_start")], "D1", 10)
            prds = CCD(dat[con.index("rtc_period")], "UI", 3)
            stat = CCD(dat[con.index("rtc_status")], "UA", 1)
            if self.styp != "X":
                curdt = self.date / 100
                if self.months:
                    yy = curdt / 100
                    mm = (curdt % 100) + self.months
                    while mm > 12:
                        yy += 1
                        mm -= 12
                    curdt = (yy * 100) + mm
                if freq.work == "M":
                    mths = 1 * prds.work
                elif freq.work == "3":
                    mths = 3 * prds.work
                elif freq.work == "6":
                    mths = 6 * prds.work
                else:
                    mths = 12 * prds.work
                yy = strt.work / 10000
                mm = ((strt.work % 10000) / 100) + mths - 1
                while mm < 1:
                    yy -= 1
                    mm += 12
                while mm > 12:
                    yy += 1
                    mm -= 12
                chkmth = copy.deepcopy(mthnam)
                if mm == 2:
                    if (yy % 4) == 0:
                        chkmth[2][2] = 29
                    else:
                        chkmth[2][2] = 28
                exdt = CCD(((yy*10000)+(mm*100)+chkmth[mm][2]), "D1", 10)
                if self.styp == "E" and (exdt.work / 100) > curdt:
                    continue
            else:
                exdt = CCD(0, "d1", 10)
            acc = self.sql.getRec(tables="rtlmst", where=[("rtm_cono", "=",
                self.conum), ("rtm_code", "=", code.work), ("rtm_acno", "=",
                acno.work)], limit=1)
            name = CCD(acc[mst.index("rtm_name")], "NA", 30)
            if self.cons == "Y":
                trn = self.sql.getRec(tables="rtltrn", cols=["rtt_type",
                    "round(sum(rtt_tramt), 2)"], where=[("rtt_cono", "=",
                    self.conum), ("rtt_code", "=", code.work), ("rtt_acno",
                    "=", acno.work), ("rtt_trdt", "<=", self.date)],
                    group="rtt_type")
            else:
                trn = self.sql.getRec(tables="rtltrn", cols=["rtt_type",
                    "round(sum(rtt_tramt), 2)"], where=[("rtt_cono", "=",
                    self.conum), ("rtt_code", "=", code.work), ("rtt_acno",
                    "=", acno.work), ("rtt_cnum", "=", cnum.work), ("rtt_trdt",
                    "<=", self.date)], group="rtt_type")
            rtl = CCD(0, "SD", 13.2)
            rec = CCD(0, "SD", 13.2)
            pmt = CCD(0, "SD", 13.2)
            jnl = CCD(0, "SD", 13.2)
            if trn:
                for t in trn:
                    if t[0] == 1:
                        rtl = CCD(t[1], "SD", 13.2)
                    elif t[0] == 2:
                        rec = CCD(t[1], "SD", 13.2)
                    elif t[0] == 3:
                        pmt = CCD(t[1], "SD", 13.2)
                    elif t[0] == 4:
                        jnl = CCD(t[1], "SD", 13.2)
            bal = float(ASD(rtl.work)+ASD(rec.work)+ASD(pmt.work)+ASD(jnl.work))
            bal = CCD(bal, "SD", 13.2)
            if self.pglin > self.lpp:
                if self.pgnum:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
            self.t.textLine("%s %s %s %s %s %s %s %s %s %s %s %s %s %s" % \
                (code.disp, acno.disp, cnum.disp, name.disp, freq.disp,
                strt.disp, prds.disp, exdt.disp, stat.disp, rtl.disp,
                rec.disp, pmt.disp, jnl.disp, bal.disp))
            self.tots[0] = float(ASD(self.tots[0]) + ASD(rtl.work))
            self.tots[1] = float(ASD(self.tots[1]) + ASD(rec.work))
            self.tots[2] = float(ASD(self.tots[2]) + ASD(pmt.work))
            self.tots[3] = float(ASD(self.tots[3]) + ASD(jnl.work))
            self.tots[4] = float(ASD(self.tots[4]) + ASD(bal.work))
            self.pglin = self.pglin + 1
        for x in range(5):
            exec "t%s = CCD(self.tots[x], 'SD', 13.2).disp" % x
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.textLine("%19s %-30s %29s %-13s %-13s %-13s %-13s %-13s" % \
            ("", "Grand Totals", "", t0, t1, t2, t3, t4))
        p.closeProgress()
        if self.t:
            self.c.drawText(self.t)
            self.c.showPage()
            self.c.save()
            doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                head=self.tit, view=self.df.repprt[0][1],
                prtnam=self.df.repprt[0][2], mail=self.df.repprt[1][2])
        self.closeProcess()

    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-34s %-10s %97s %5s" % ("Rentals Ledger Master "\
            "Report as at", self.datd, "Page", self.pgnum))
        self.t.textLine()
        opts = "%-14s %-1s" % ("(Options: Type", self.styp)
        if self.styp == "E":
            opts = "%s %-6s %-2s)" % \
            (opts, "Months", self.months)
        elif self.cons == "Y":
            opts = "%s %-12s %-1s)" % \
            (opts, "Consolidated", self.cons)
        else:
            opts = "%s)" % opts
        self.t.textLine(opts)
        self.t.textLine()
        self.t.textLine("%-7s %-7s %-3s %-30s %-1s %-10s %-3s %-10s %-1s "\
            "%-13s %-13s %-13s %-13s %-13s" % ("Prm-Cod", "Acc-Num", "Seq",
            "Name", "F", "Start-Date", "Per", "Expir-Date", "S",
            "     Rentals ", "    Receipts ", "    Payments ",
            "    Journals ", "     Balance "))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 8

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

