#! /usr/bin/env python

"""
SYNOPSIS
    Rentals Ledger Interrogation.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, Balances, CCD, NotesCreate, SChoice, Sql
from TartanClasses import SRec, TartanDialog
from tartanFunctions import getModName, doPrinter
from tartanWork import rttrtp

class rt4010:
    def __init__(self, mf, coy, nam, usr):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.capnm = usr
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["rtlprm", "rtlmst", "rtlcon", "rtltrn"],
            prog=__name__)
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) +  (t[1] * 100) + t[2]
        self.sysdtd = "%i/%02i/%02i" % (t[0], t[1], t[2])
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
                "Rental Ledger Interrogation (%s)" % __name__)
        prm = {
            "stype": "R",
            "tables": ("rtlprm",),
            "cols": (
                ("rtp_code", "", 0, "Prm-Code"),
                ("rtp_desc", "", 0, "Description", "Y")),
            "where": [("rtp_cono", "=", self.conum)]}
        acc = {
            "stype": "R",
            "tables": ("rtlmst",),
            "cols": (
                ("rtm_acno", "", 0, "Acc-Num"),
                ("rtm_name", "", 0, "Name", "Y")),
            "where": [("rtm_cono", "=", self.conum)],
            "whera": (("T", "rtm_code", 0, 0),)}
        seq = {
            "stype": "R",
            "tables": ("rtlcon",),
            "cols": (
                ("rtc_cnum", "", 0, "Acc-Num"),
                ("rtc_payind", "", 0, "F"),
                ("rtc_start", "", 0, "Start-Date"),
                ("rtc_period", "", 0, "Per")),
            "where": [("rtc_cono", "=", self.conum)],
            "whera": [["T", "rtc_code", 0, 0], ["T", "rtc_acno", 1, 0]]}
        tag = (
            ("Premises",self.doTagSelect,("T",0,2),("T",0,1)),
            ("Tenant",self.doTagSelect,("T",0,2),("T",0,1)),
            ("Balances",self.doTagSelect,("T",0,2),("T",0,1)),
            ("Transactions",self.doTrans1,("T",0,2),("T",0,1)))
        r1s = (
            ("Monthly","M"),
            ("Quarterly","3"),
            ("Bi-Annually", "6"),
            ("Annually","A"))
        fld = (
            (("T",0,0,0),"INA",7,"Premises","Premises Code",
                "","Y",self.doPremises,prm,None,("notblank",)),
            (("T",0,0,0),"INA",7,"Account","Account Code",
                "","N",self.doAccount,acc,None,("notblank",)),
            (("T",0,0,0),"ONA",30,"Name"),
            (("T",1,0,0),"ONA",30,"Description"),
            (("T",1,1,0),"ONA",30,"Address-1"),
            (("T",1,2,0),"ONA",30,"Address-2"),
            (("T",1,3,0),"ONA",30,"Address-3"),
            (("T",1,4,0),"ONA",4,"Postal Code"),
            (("T",2,0,0),"ONA",30,"Address-1"),
            (("T",2,1,0),"ONA",30,"Address-2"),
            (("T",2,2,0),"ONA",30,"Address-3"),
            (("T",2,3,0),"ONA",4,"Postal Code"),
            (("T",2,4,0),"ONA",20,"Telephone Number"),
            (("T",2,5,0),"OTX",50,"E-Mail Address"),
            (("T",2,6,0),"OUA",1,"VAT Indicator"),
            (("T",2,7,0),"ONA",10,"VAT Number"),
            (("T",2,8,0),"OUA",1,"Payment Frequency"),
            (("T",2,9,0),"OD1",10,"Start Date"),
            (("T",2,10,0),"OUI",3,"Number of Periods"),
            (("T",2,11,0),"OUD",12.2,"Rental Amount"),
            (("T",2,12,0),"OUA",1,"Status"),
            (("C",3,0,0),"OSD",13.2,"Value","",
                "","N",None,None,None,None,("Details",5)),
            (("T",4,0,0),"IUI",3,"Sequence Number","Sequence Number",
                "N","N",self.doTrans2,seq,None,None))
        but = (
            ("Clear",None,self.doClear,0,("T",0,0),("T",0,1)),
            ("Notes",None,self.doNotes,0,("T",0,0),("T",0,1)),
            ("Print",None,self.doPrint,0,("T",0,0),("T",0,1)),
            ("Quit",None,self.doExit,1,None,None))
        tnd = ((self.doEndTop, "N"), None, None, None, None)
        txt = (self.doExit, None, None, None, self.doExit)
        self.df = TartanDialog(self.mf, title=self.tit, tags=tag, eflds=fld,
            butt=but, tend=tnd, txit=txt)
        self.df.colLabel[3][0].set_text("Rentals ")
        self.df.colLabel[3][1].set_text("Receipts")
        self.df.colLabel[3][2].set_text("Payments")
        self.df.colLabel[3][3].set_text("Journals")
        self.df.colLabel[3][4].set_text("Balance ")

    def doPremises(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="rtlprm", where=[("rtp_cono", "=",
            self.conum), ("rtp_code", "=", w)], limit=1)
        if not acc:
            return "Invalid Premises"
        self.code = w
        for num, fld in enumerate(acc):
            if num < 2 or num > 6:
                continue
            self.df.loadEntry("T", 1, (num-2), data=fld)

    def doAccount(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="rtlmst", where=[("rtm_cono", "=",
            self.conum), ("rtm_code", "=", self.code), ("rtm_acno", "=", w)],
            limit=1)
        if not acc:
            return "Invalid Account"
        self.acno = w
        self.name = acc[self.sql.rtlmst_col.index("rtm_name")]
        con = self.sql.getRec(tables="rtlcon", where=[("rtc_cono", "=",
            self.conum), ("rtc_code", "=", self.code), ("rtc_acno", "=",
            self.acno)], order="rtc_cnum")
        if not con:
            return "Invalid Contract"
        self.df.loadEntry(frt, pag, p+1, data=acc[3])
        for num, fld in enumerate(acc[4:]):
            self.df.loadEntry("T", 2, num, data=fld)
        self.cnum = con[-1:][0][3]
        self.df.topf[4][0][5] = self.cnum
        for num, fld in enumerate(con[-1:][0][4:]):
            self.df.loadEntry("T", 2, num+8, data=fld)
        self.loadBalances()
        self.mf.updateStatus("")

    def doEndTop(self):
        self.df.last[0] = [0, 0]
        self.df.setWidget(self.df.topEntry[0][0], state="disabled")

    def loadBalances(self):
        rtl = 0
        rec = 0
        pmt = 0
        jnl = 0
        bal = self.sql.getRec(tables="rtltrn", cols=["rtt_type",
            "round(sum(rtt_tramt), 2)"], where=[("rtt_cono", "=", self.conum),
            ("rtt_code", "=", self.code), ("rtt_acno", "=", self.acno)],
            group="rtt_type", order="rtt_type")
        if bal:
            for a in bal:
                if a[0] == 1:
                    rtl = a[1]
                if a[0] == 2:
                    rec = a[1]
                if a[0] == 3:
                    pmt = a[1]
                if a[0] == 4:
                    jnl = a[1]
        bal = float(ASD(rtl) + ASD(rec) + ASD(pmt) + ASD(jnl))
        self.df.loadEntry("C", 3, 0, data=rtl)
        self.df.loadEntry("C", 3, 1, data=rec)
        self.df.loadEntry("C", 3, 2, data=pmt)
        self.df.loadEntry("C", 3, 3, data=jnl)
        self.df.loadEntry("C", 3, 4, data=bal)
        self.trn = self.sql.getRec(tables="rtltrn", where=[("rtt_cono", "=",
            self.conum), ("rtt_code", "=", self.code), ("rtt_acno", "=",
            self.acno)], order="rtt_trdt, rtt_type")

    def doTagSelect(self):
        self.mf.updateStatus("")

    def doTrans1(self, widget=None):
        self.df.focusField("T", 4, 1)

    def doTrans2(self, frt, pag, r, c, p, i, w):
        tit = "Transactions for Account: %s %s - %s" % \
            (self.code, self.acno, self.name)
        tab = ["rtltrn"]
        col = (
            ("rtt_trdt", "", 0, "   Date"),
            ("rtt_curdt", "", 0, "Curr-Dt"),
            ("rtt_batch", "", 0, "Batch"),
            ("rtt_type", ("XX", rttrtp), 3, "Typ"),
            ("rtt_refno", "", 0, "Reference", "Y"),
            ("rtt_tramt", "", 0, "     Amount"),
            ("rtt_taxamt", "", 0, " VAT-Amount"),
            ("rtt_desc", "", 0, "Details"))
        whr = [("rtt_cono", "=", self.conum), ("rtt_code", "=", self.code),
            ("rtt_acno", "=", self.acno)]
        if w:
            whr.append(("rtt_cnum", "=", w))
        ord = "rtt_trdt, rtt_type"
        state = self.df.disableButtons()
        rec = SRec(self.mf, screen=self.df.nb.Page2, title=tit, tables=tab,
            cols=col, where=whr, order=ord, rows=12)
        self.df.enableButtons(state=state)
        self.doTrans1()

    def doNotes(self, widget=None):
        state = self.df.disableButtons()
        self.df.mstAlw.hide()
        nf = NotesCreate(self.mf, self.conum, self.conam, self.capnm, "RTL",
            "%7s%s" % (self.code, self.acno))
        self.df.mstAlw.show()
        self.df.enableButtons(state=state)

    def doClear(self, widget=None):
        self.df.selPage("Premises")
        self.df.focusField("T", 0, 1)

    def doPrint(self, widget=None):
        titl = "Select the required print option!"
        data = (
            ("Info Only", "I"),
            ("Transactions Only", "T"),
            ("Info and Transactions", "A"))
        ss = SChoice(self.mf, titl=titl, data=data)
        self.mf.updateStatus("")
        if not ss.selection:
            self.pr_opts = None
        else:
            self.pr_opts = ss.selection[1]
        if self.pr_opts:
            self.printDetail()
        if self.df.topq[self.df.pag]:
            self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def printDetail(self):
        self.head = ("%03u %-30s %59s %10s" % \
            (self.conum, self.conam, self.sysdttm, __name__))
        from tartanFunctions import doReportlab
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        self.pgnum = 0
        self.pglin = 99
        if self.pr_opts != "T":
            self.pageHeading()
            self.printInfo()
        if self.pr_opts == "I":
            self.c.drawText(self.t)
            self.c.showPage()
            self.c.save()
            doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                head=self.tit, view="V")
        elif self.pr_opts == "A":
            if not self.trn:
                self.c.drawText(self.t)
                self.c.showPage()
                self.c.save()
                doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                    head=self.tit, view="V")
            else:
                self.c.drawText(self.t)
                self.c.showPage()
                self.pageHeading()
                self.pageHeadingTrans()
                self.printTrans()
                self.c.drawText(self.t)
                self.c.showPage()
                self.c.save()
                doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                    head=self.tit, view="V")
        elif self.pr_opts == "T":
            if self.trn:
                self.pageHeading()
                self.pageHeadingTrans()
                self.printTrans()
                self.c.drawText(self.t)
                self.c.showPage()
                self.c.save()
                doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                    head=self.tit, view="V")

    def printInfo(self):
        for x in range(0, len(self.df.topf[0])):
            self.t.textLine("%-26s%-25s %s" % ("",
                self.df.topf[0][x][3], self.df.t_disp[0][0][x]))
        self.t.textLine()
        for x in range(0, len(self.df.topf[1])):
            self.t.textLine("%-26s%-25s %s" % ("",
                self.df.topf[1][x][3], self.df.t_disp[1][0][x]))
        self.t.textLine()
        for x in range(0, len(self.df.topf[2])):
            self.t.textLine("%-26s%-25s %s" % ("",
                self.df.topf[2][x][3], self.df.t_disp[2][0][x]))
        self.t.textLine()
        self.pglin = self.pglin + 3 + len(self.df.topf[3])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.t.textLine("%26s%-34s %13s" % ("", "Details","Amount "))
        self.t.textLine("%26s%s" % ("", self.suc * 48))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        desc = (
            "Rentals ",
            "Receipts",
            "Payments",
            "Journals",
            "Balance ")
        for n, d in enumerate(self.df.c_disp[3]):
            self.t.textLine("%26s%-34s %13s" % ("", desc[n], d[0]))

    def printTrans(self):
        col = self.sql.rtltrn_col
        bal = 0
        for ct in self.trn:
            trdt = CCD(ct[col.index("rtt_trdt")], "D1", 10)
            refno = CCD(ct[col.index("rtt_refno")], "Na", 9)
            trtp = CCD(ct[col.index("rtt_type")], "UI", 1)
            batch = CCD(ct[col.index("rtt_batch")], "Na", 7)
            desc = CCD(ct[col.index("rtt_desc")], "NA", 30)
            tramt = ct[col.index("rtt_tramt")]
            if tramt < 0:
                debit = CCD(0, "SD", 13.2)
                credit = CCD(tramt, "SD", 13.2)
            else:
                debit = CCD(tramt, "SD", 13.2)
                credit = CCD(0, "SD", 13.2)
            bal = float(ASD(bal) + ASD(tramt))
            tot = CCD(bal, "SD", 13.2)
            taxamt = CCD(ct[col.index("rtt_taxamt")], "SD", 13.2)
            if self.pglin > self.lpp:
                if self.pgnum != 0:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
                self.pageHeadingTrans()
            self.t.textLine("%s %s %s %s %s %s %s %s %s" % (trdt.disp,
                refno.disp, rttrtp[(trtp.work - 1)][0], batch.disp,
                desc.disp, debit.disp, credit.disp, tot.disp, taxamt.disp))
            self.pglin = self.pglin + 1

    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-34s %-10s %53s %5s" % \
            ("Rentals Ledger Interrogation as at", self.sysdtd, "Page",
            self.pgnum))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 6

    def pageHeadingTrans(self):
        self.t.textLine("%-16s %s %s %s %s %s %s %s %s" % ("",
            self.df.topf[0][0][3], self.df.t_disp[0][0][0], "",
            self.df.topf[0][1][3], self.df.t_disp[0][0][1], "",
            self.df.topf[0][2][3], self.df.t_disp[0][0][2]))
        self.t.textLine()
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.t.textLine("%-10s %-9s %-3s %-7s %-30s %-13s %-13s %-13s" % \
            ("   Date", "Reference", "Typ", "Batch", "Remarks",
            "       Debit", "      Credit",  "     Balance"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 8

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

