#! /usr/bin/python

"""
SYNOPSIS
    Rentals's Ledger Key Change.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN; if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import GetCtl, Sql, TartanDialog

class rt6010:
    def __init__(self, mf, conum, conam):
        self.mf = mf
        self.conum = conum
        self.conam = conam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.tables = (
            ("ctlnot", "not_key"),
            ("ctlvtf", "vtt_cono", "vtt_acno", "vtt_styp"),
            ("rtlmst", "rtm_cono", "rtm_code", "rtm_acno"),
            ("rtltrn", "rtt_cono", "rtt_code", "rtt_acno"))
        tabs = ["rtlprm"]
        for tab in self.tables:
            tabs.append(tab[0])
        self.sql = Sql(self.mf.dbm, tabs, prog=__name__)
        gc = GetCtl(self.mf)
        rtlctl = gc.getCtl("rtlctl", self.conum)
        if not rtlctl:
            return "no"

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "Rental's Ledger Key Changes")
        prm = {
            "stype": "R",
            "tables": ("rtlprm",),
            "cols": (
                ("rtp_code", "", 0, "Cod"),
                ("rtp_desc", "", 0, "Description", "Y")),
            "where": [("rtp_cono", "=", self.conum)]}
        rtm = {
            "stype": "R",
            "tables": ("rtlmst",),
            "cols": [
                ("rtm_code", "", 0, "Cod"),
                ("rtm_acno", "", 0, "Acc-Num"),
                ("rtm_name", "", 0, "Name", "Y")],
            "where": [("rtm_cono", "=", self.conum)],
            "whera": [["T", "rtm_code", 0]],
            "index": 1}
        fld = [
            (("T",0,0,0),"I@rtm_code",0,"Old Code","Old Premises Code",
                "","Y",self.doOldCod,prm,None,("notblank",)),
            (["T",0,0,12],"I@rtm_acno",0,"Old Account","Old Account Number",
                "","Y",self.doOldAcc,rtm,None,("notblank",)),
            (["T",0,0,30],"O@rtm_name",0,""),
            (("T",0,1,0),"I@rtm_code",0,"New Code","New Premises Code",
                "","Y",self.doNewCod,prm,None,("notblank",)),
            (["T",0,1,12],"I@rtm_acno",0,"New Account","New Account Number",
                "","Y",self.doNewAcc,None,None,("notblank",))]
        tnd = ((self.doProcess,"y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tend=tnd,
            txit=txt)

    def doOldCod(self, frt, pag, r, c, p, i, w):
        if w:
            acc = self.sql.getRec(tables="rtlprm", where=[("rtp_cono", "=",
                self.conum), ("rtp_code", "=", w)], limit=1)
            if not acc:
                return "Invalid Premises Number, Does Not exist"
        self.oldprm = w

    def doOldAcc(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="rtlmst", where=[("rtm_cono", "=",
            self.conum), ("rtm_code", "=", self.oldprm), ("rtm_acno", "=", w)],
            limit=1)
        if not acc:
            return "Invalid Account Number, Does Not exist"
        self.oldacc = w
        self.oldnot = "%7s%s" % (self.oldprm, w)
        self.df.loadEntry(frt, pag, p+1, data=acc[3])

    def doNewCod(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="rtlprm", where=[("rtp_cono", "=",
            self.conum), ("rtp_code", "=", w)], limit=1)
        if not acc:
            return "Invalid Premises Number, Does Not exist"
        self.newprm = w

    def doNewAcc(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="rtlmst", where=[("rtm_cono", "=",
            self.conum), ("rtm_code", "=", self.newprm), ("rtm_acno", "=", w)],
            limit=1)
        if acc:
            return "Invalid Account Number, Already Exists"
        self.newacc = w
        self.newnot = "%7s%s" % (self.newprm, w)

    def doProcess(self, focus=True):
        for tab in self.tables:
            if tab[0] == "ctlnot":
                whr = [("not_cono", "=", self.conum), ("not_sys", "=", "RTL"),
                    (tab[1], "=", self.oldnot)]
                dat = [self.newnot]
                col = [tab[1]]
            elif tab[0] == "ctlvtf":
                whr = [(tab[1], "=", self.conum), (tab[2], "=", self.oldacc),
                    (tab[3], "=", "R")]
                dat = [self.newacc]
                col = [tab[2]]
            else:
                whr = [(tab[1], "=", self.conum), (tab[2], "=", self.oldprm),
                    (tab[3], "=", self.oldacc)]
                dat = [self.newprm, self.newacc]
                col = [tab[2], tab[3]]
            self.sql.updRec(tab[0], where=whr, data=dat, cols=col)
        if focus:
            self.df.focusField("T", 0, 1)

    def doExit(self):
        self.df.closeProcess()
        self.mf.dbm.commitDbase(ask=True)
        self.mf.closeLoop()

