#! /usr/bin/env python

"""
SYNOPSIS
    Rentals Ledger Month End Routine.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, CCD, GetCtl, PwdConfirm, Sql, TartanDialog
from tartanFunctions import askQuestion, mthendDate

class rtm010:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlbat", "ctlvtf", "gentrn", "rtlctl",
            "rtlmst", "rtlcon", "rtltrn"], prog=__name__)
        gc = GetCtl(self.mf)
        rtlctl = gc.getCtl("rtlctl", self.conum)
        if not rtlctl:
            return "no"
        self.glint = rtlctl["ctr_glint"]
        self.lme = rtlctl["ctr_lme"]
        if self.glint == "Y":
            ctlctl = gc.getCtl("ctlctl", self.conum)
            if not ctlctl:
                return "no"
            if gc.chkRec(self.conum, ctlctl, ["vat_ctl"]):
                return "no"
            self.convat = ctlctl["vat_ctl"]
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.tme = mthendDate(self.sysdtw)

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "Rentals Month End Routine (%s)" % __name__)
        fld = (
            (("T",0,0,0),"Od1",10,"Last Month End Date"),
            (("T",0,1,0),"ID1",10,"This Month End Date","This Month End Date",
                self.tme,"N",self.doTme,None,None,("efld",)))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.closeProcess,)
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tend=tnd,
            txit=txt, focus=False)
        self.df.loadEntry("T", 0, 0, data=self.lme)
        self.df.focusField("T", 0, 2)

    def doTme(self, frt, pag, r, c, p, i, w):
        if w <= self.lme:
            return "Invalid Month-End Date"
        self.tme = w
        self.curdt = w / 100
        yy = self.curdt / 100
        mm = (self.curdt % 100) + 1
        while mm > 12:
            yy += 1
            mm -= 12
        self.nxtdt = (yy * 10000) + (mm * 100) + 1
        self.nxtcd = self.nxtdt / 100
        self.batch = "R%06s" % self.nxtcd
        chk = self.sql.getRec(tables="rtltrn", cols=["count(*)"],
            where=[("rtt_cono", "=", self.conum), ("rtt_type", "=", 1),
            ("rtt_batch", "=", self.batch), ("rtt_trdt", "=", self.nxtdt),
            ("rtt_curdt", "=", self.nxtcd)], limit=1)
        if chk[0]:
            return "Month-End Already Executed"

    def doEnd(self):
        all = self.sql.getRec(tables="ctlbat", cols=["count(*)"],
            where=[("btm_cono", "=", self.conum), ("btm_styp", "=", "RTL"),
            ("btm_ind", "=", "N"), ("btm_curdt", "=", self.curdt)], limit=1)
        if all[0]:
            ok = askQuestion(self.mf.body, "Unbalanced Batches Exist",
                "There are unbalanced batches for this month. You "\
                "SHOULD not continue, but print a batch error report, and "\
                "correct the errors. Continue, YES or NO ?", default="no")
            if ok == "yes":
                ok = PwdConfirm(self.mf, conum=self.conum, system="RTL",
                    code="UnbalBatch")
                if ok.flag =="no":
                    self.closeProcess()
                else:
                    self.updateTables()
            else:
                self.closeProcess()
        else:
            self.updateTables()

    def updateTables(self):
        self.df.closeProcess()
        all = self.sql.getRec(tables=["rtlcon", "rtlmst"], cols=["rtc_cono",
            "rtc_code", "rtc_acno", "rtc_cnum", "rtc_payind", "rtc_start",
            "rtc_period", "rtc_rtlamt", "rtm_name", "rtm_vatind"],
            where=[("rtc_cono", "=", self.conum), ("rtm_cono=rtc_cono",),
            ("rtm_acno=rtc_acno",), ("rtc_start", "<=", self.nxtdt),
            ("rtc_status", "<>", "X")])
        if all:
            for num, self.rec in enumerate(all):
                self.freq = self.rec[4]
                strt = self.rec[5]
                self.pers = self.rec[6]
                self.amnt = self.rec[7]
                if self.freq == "M":
                    mths = 1 * self.pers
                elif self.freq == "3":
                    mths = 3 * self.pers
                elif self.freq == "6":
                    mths = 6 * self.pers
                else:
                    mths = 12 * self.pers
                yy = strt / 10000
                mm = ((strt % 10000) / 100) + mths - 1
                while mm > 12:
                    yy += 1
                    mm -= 12
                dd = strt % 100
                self.exdt = (yy * 10000) + (mm * 100) + dd
                if (self.exdt / 100) <= self.curdt:
                    self.sql.updRec("rtlcon", cols=["rtc_status"], data=["X"],
                        where=[("rtc_cono", "=", self.rec[0]), ("rtc_code",
                        "=", self.rec[1]), ("rtc_acno", "=", self.rec[2]),
                        ("rtc_cnum", "=", self.rec[3])])
                    ok = askQuestion(self.mf.body, "Expired Contract",
                        "This contract has Expired, would you like to "\
                        "Renew it?\n\nPremises: %s\nAccount: %s\nName: %s" % \
                        (self.rec[1], self.rec[2], self.rec[8]))
                    if ok == "no":
                        continue
                    self.doExpiredContract()
                self.doRaiseRental(num)
        self.sql.updRec("rtlctl", cols=["ctr_lme"], data=[self.tme],
            where=[("ctr_cono", "=", self.conum)])
        self.mf.dbm.commitDbase(ask=True, mess="Do You Want To Save All "\
            "Changes?\n\nPlease Note That Once The Changes Have Been "\
            "Saved, There Is No Going Back Without Restoring From Backup!")
        self.mf.closeLoop()

    def doExpiredContract(self):
        tit = ("Renew Rental Contract", )
        r1s = (
            ("Monthly","M"),
            ("Quarterly","3"),
            ("Bi-Annually", "6"),
            ("Annually","A"))
        self.fld = (
            (("T",0,0,0),"OUA",3,"Premises Code"),
            (("T",0,1,0),"ONA",7,"Account Code"),
            (("T",0,2,0),"ONA",30,"Tenant Name"),
            (("T",0,3,0),("IRB",r1s),0,"Payment Frequency","Payment Frequency",
                self.freq,"N",None,None,None,None),
            (("T",0,4,0),"ID1",10,"Start Date","Start Date",
                self.exdt,"N",None,None,None,("efld",)),
            (("T",0,5,0),"IUI",3,"Number of Periods","Number of Periods",
                self.pers,"N",None,None,None,("notzero",)),
            (("T",0,6,0),"IUD",12.2,"Rental Amount","Rental Amount",
                self.amnt,"N",None,None,None,("notzero",)))
        tnd = ((self.doExpireEnd,"y"), )
        txt = (self.doLoadFocus, )
        self.na = TartanDialog(self.mf, newscn=(0,1), tops=True, title=tit,
            eflds=self.fld, tend=tnd, txit=txt, focus=False)
        self.doLoadFocus()
        self.mf.startLoop()

    def doExpireEnd(self):
        dat = self.rec[:3]
        dat.append(self.rec[3] + 1)
        for x in range(3, len(self.na.t_work[0][0])):
            dat.append(self.na.t_work[0][0][x])
        dat.append("C")
        self.sql.insRec("rtlcon", data=dat)
        self.rec[:8] = dat[:8]
        self.na.closeProcess()
        self.mf.closeLoop()

    def doRaiseRental(self, num):
        ref = CCD((num + 1), "Na", 9).work
        amt = CCD(self.rec[7], "UD", 12.2).work
        acc = self.sql.getRec(tables=["ctlvmf"], cols=["vtm_new"],
            where=[("vtm_cono", "=", self.conum), ("vtm_code", "=",
            self.rec[9])], limit=1)
        if not acc:
            rate = 0
        else:
            rate = float(acc[0])
        vat = CCD(round((amt * rate / 100), 2), "SD", 11.2).work
        tot = float(ASD(amt) + ASD(vat))
        data = self.rec[:4]
        data.extend([1, ref, self.batch, self.nxtdt, tot, vat, self.nxtcd,
            "Rental Raised", self.rec[9], "", self.sysdtw, 0])
        self.sql.insRec("rtltrn", data=data)
        if self.rec[9]:
            # VAT Transaction (ctlvtf)
            val = float(ASD(0) - ASD(amt))
            tax = float(ASD(0) - ASD(vat))
            data = [self.conum, self.rec[9], "O", self.nxtcd, "R", 1,
                self.batch, ref, self.nxtdt, self.rec[2], self.rec[8],
                val, tax, self.sysdtw, 0, 0]
            self.sql.insRec("ctlvtf", data=data)
        if self.glint == "N":
            return
        acc = self.sql.getRec(tables="rtlprm", cols=["rtp_rtlacc",
            "rtp_incacc"], where=[("rtp_cono", "=", self.conum),
            ("rtp_code", "=", self.rec[1])], limit=1)
        if not acc:
            return
        ref = "Month/End"
        # General Ledger Rental Control
        whr = [("glt_cono", "=", self.conum), ("glt_acno", "=", acc[0]),
            ("glt_batch", "=", self.batch), ("glt_curdt", "=", self.nxtcd),
            ("glt_trdt", "=", self.nxtdt), ("glt_type", "=", 1)]
        glt = self.sql.getRec(tables="gentrn", where=whr)
        if glt and len(glt) == 1:
            val = glt[0][self.sql.gentrn_col.index("glt_tramt")]
            val = float(ASD(val) + ASD(tot))
            glt[0][self.sql.gentrn_col.index("glt_tramt")] = val
            tax = glt[0][self.sql.gentrn_col.index("glt_taxamt")]
            tax = float(ASD(tax) + ASD(vat))
            glt[0][self.sql.gentrn_col.index("glt_taxamt")] = tax
            self.sql.updRec("gentrn", where=whr, data=glt[0])
        else:
            data = [self.conum, acc[0], self.nxtcd, self.nxtdt, 1, ref,
                self.batch, tot, vat, self.rec[8], self.rec[9], "", 0,
                self.sysdtw, 0]
            self.sql.insRec("gentrn", data=data)
        amt = float(ASD(0) - ASD(amt))
        vat = float(ASD(0) - ASD(vat))
        # General Ledger Income Account
        whr = [("glt_cono", "=", self.conum), ("glt_acno", "=", acc[1]),
            ("glt_batch", "=", self.batch), ("glt_curdt", "=", self.nxtcd),
            ("glt_trdt", "=", self.nxtdt), ("glt_type", "=", 1)]
        glt = self.sql.getRec(tables="gentrn", where=whr)
        if glt and len(glt) == 1:
            val = glt[0][self.sql.gentrn_col.index("glt_tramt")]
            val = float(ASD(val) + ASD(amt))
            glt[0][self.sql.gentrn_col.index("glt_tramt")] = val
            self.sql.updRec("gentrn", where=whr, data=glt[0])
        else:
            data = [self.conum, acc[1], self.nxtcd, self.nxtdt, 1, ref,
                self.batch, amt, 0.00, self.rec[8], "", "", 0,
                self.sysdtw, 0]
            self.sql.insRec("gentrn", data=data)
        if not vat:
            return
        # General Ledger VAT Account
        whr = [("glt_cono", "=", self.conum), ("glt_acno", "=", self.convat),
            ("glt_batch", "=", self.batch), ("glt_curdt", "=", self.nxtcd),
            ("glt_trdt", "=", self.nxtdt), ("glt_type", "=", 1)]
        glt = self.sql.getRec(tables="gentrn", where=whr)
        if glt and len(glt) == 1:
            val = glt[0][self.sql.gentrn_col.index("glt_tramt")]
            val = float(ASD(val) + ASD(vat))
            glt[0][self.sql.gentrn_col.index("glt_tramt")] = val
            self.sql.updRec("gentrn", where=whr, data=glt[0])
        elif vat:
            data = [self.conum, self.convat, self.nxtcd, self.nxtdt, 1, ref,
                self.batch, vat, 0.00, self.rec[8], "", "", 0,
                self.sysdtw, 0]
            self.sql.insRec("gentrn", data=data)

    def doLoadFocus(self):
        self.na.loadEntry("T", 0, 0, data=self.rec[1])
        self.na.loadEntry("T", 0, 1, data=self.rec[2])
        self.na.loadEntry("T", 0, 2, data=self.rec[8])
        self.na.focusField("T", 0, 4)

    def closeProcess(self):
        self.df.closeProcess()
        self.mf.closeLoop()

