#! /usr/bin/python2

"""
SYNOPSIS
    Sales Invoicing Data Capture.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import AgeAll, ASD, Balances, CCD, GetCtl, PrintInvoice
from TartanClasses import PwdConfirm, Sql, TartanDialog
from tartanFunctions import callModule, showPrinters, textFormat

class si2010:
    def __init__(self, mf, conum, conam, period, usr):
        self.mf = mf
        self.conum = conum
        self.conam = conam
        self.period = period
        self.capnm = usr
        no = self.setVariables()
        if not no:
            self.getPrinters()
            if self.prtnam != "cancel":
                self.mainProcess()
                self.mf.startLoop()

    def setVariables(self):
        # Set Table Fields
        tables = [
            "ctlsys", "ctlmst","ctlmes","ctlrep","ctlvtf",
            "drschn","drsmst","drstrn","drsdel",
            "genmst","gentrn",
            "strgrp","strmf1","strmf2","strprc","strrcp","strtrn","struoi",
            "slsctl","slsiv1","slsiv2","slsiv3"]
        self.sql = Sql(self.mf.dbm, tables, prog=__name__)
        # Set, Check Controls
        gc = GetCtl(self.mf)
        ctlsys = gc.getCtl("ctlsys")
        if ctlsys and ctlsys["sys_smtp"]:
            self.email = "Y"
        else:
            self.email = "N"
        drsctl = gc.getCtl("drsctl", self.conum)
        if not drsctl:
            return "no"
        self.drgl = drsctl["ctd_glint"]
        self.chns = drsctl["ctd_chain"]
        strctl = gc.getCtl("strctl", self.conum)
        if not strctl:
            return "no"
        self.stgl = strctl["cts_glint"]
        self.locs = strctl["cts_locs"]
        self.prices = strctl["cts_prices"]
        self.automu = strctl["cts_automu"]
        if self.drgl == "Y" or self.stgl == "Y":
            ctlctl = gc.getCtl("ctlctl", self.conum)
            if not ctlctl:
                return "no"
            if self.drgl == "Y":
                ctls = ["drs_ctl", "vat_ctl"]
                if gc.chkRec(self.conum, ctlctl, ctls):
                    return "no"
                self.dctl = ctlctl["drs_ctl"]
                self.vctl = ctlctl["vat_ctl"]
            if self.stgl == "Y":
                ctls = ["stk_soh", "stk_susp"]
                if gc.chkRec(self.conum, ctlctl, ctls):
                    return "no"
                self.soh = ctlctl["stk_soh"]
                self.ssp = ctlctl["stk_susp"]
        slsctl = gc.getCtl("slsctl", self.conum, error=False)
        if not slsctl:
            self.useadd = "P"
            self.delnos = "Y"
            self.delval = "Y"
        else:
            self.useadd = slsctl["ctv_useadd"]
            self.delnos = slsctl["ctv_delnos"]
            self.delval = slsctl["ctv_delval"]
        ####################################################################
        # Check for Password Requirement to Invoice and Credit Note Creation
        ####################################################################
        cf = PwdConfirm(self.mf, conum=self.conum, system="INV",
            code="Invoices")
        if cf.flag == "no":
            self.allow = "N"
        else:
            self.allow = "Y"
        ####################################################################
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]

    def getPrinters(self):
        tit = ("Date and Printer Selection",)
        self.printers = showPrinters()
        prt = {
            "stype": "C",
            "head": ("Name","Description"),
            "data": self.printers}
        r1s = (("Yes","Y"),("No","N"))
        r2s = (("Print","P"),("View","V"),("None","N"))
        fld = [
            (("T",0,0,0),"ID1",10,"Document Date","Document Date",
                self.sysdtw,"N",self.doInvDate,None,None,("efld",))]
        if self.delnos == "Y":
            fld.append(
                (("T",0,1,0),("IRB",r1s),0,"Delivery Notes","Delivery Notes",
                    "Y","N",self.doDelNote,None,None,None))
            idx = 2
        else:
            self.dnote = "N"
            idx = 1
        fld.extend([
            (("T",0,idx,0),("IRB",r2s),0,"Print Choice","Print Choice",
                "P","N",self.doPrtChoice,None,None,None),
            (("T",0,idx+1,0),"INA",30,"Printer Name","Printer Name",
                "Default","Y",self.doPrtNam,prt,None,None)])
        if self.email == "Y":
            fld.append(
            (("T",0,idx+2,0),("IRB",r1s),0,"E-Mail","E-Mail Document",
                "Y","N",self.doPrtMail,None,None,None))
        self.df = TartanDialog(self.mf, screen=self.mf.body, tops=True,
            title=tit, eflds=fld, tend=((self.doPrtEnd,"y"),),
            txit=(self.doPrtExit,))
        self.mf.startLoop()

    def doInvDate(self, frt, pag, r, c, p, i, w):
        if w < self.period[1][0] or w > self.period[2][0]:
            return "Invalid Date, Not In Financial Period" 
        self.trdt = w
        self.trdd = self.df.t_disp[0][0][0]
        self.curdt = self.trdt / 100
        self.batch = "S%s" % self.curdt

    def doDelNote(self, frt, pag, r, c, p, i, w):
        self.dnote = w

    def doPrtChoice(self, frt, pag, r, c, p, i, w):
        if w == "P":
            return
        if w == "V":
            self.prtnam = "view"
        else:
            self.prtnam = "none"
        self.df.loadEntry(frt, pag, p+1, data=self.prtnam)
        return "sk1"

    def doPrtNam(self, frt, pag, r, c, p, i, w):
        if w not in self.printers:
            return "Invalid Printer"
        self.prtnam = w

    def doPrtMail(self, frt, pag, r, c, p, i, w):
        self.email = w

    def doPrtEnd(self):
        self.doPrtClose()

    def doPrtExit(self):
        self.prtnam = "cancel"
        self.doPrtClose()

    def doPrtClose(self):
        self.df.closeProcess()
        self.mf.closeLoop()

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "Sales Invoicing as at %s" % self.trdd)
        rep = {
            "stype": "R",
            "tables": ("ctlrep",),
            "cols": (
                ("rpm_rep", "", 0, "Rep"),
                ("rpm_name", "", 0, "Name", "Y")),
            "where": [("rpm_cono", "=", self.conum)]}
        chn = {
            "stype": "R",
            "tables": ("drschn",),
            "cols": [
                ("chm_chain", "", 0, "Chn"),
                ("chm_name", "", 0, "Name", "Y")],
            "where": [("chm_cono", "=", self.conum)],
            "screen": self.mf.body}
        drm = {
            "stype": "R",
            "tables": ("drsmst",),
            "cols": [
                ("drm_chain", "", 0, "Chn"),
                ("drm_acno", "", 0, "Acc-Num"),
                ("drm_name", "", 0, "Name","Y"),
                ("drm_add1", "", 0, "Address Line 1")],
            "where": [("drm_cono", "=", self.conum)],
            "screen": self.mf.body,
            "index": 1}
        self.grps = {
            "stype": "R",
            "tables": ("strgrp",),
            "cols": (
                ("gpm_group", "", 0, "Grp"),
                ("gpm_desc", "", 0, "Description", "Y")),
            "where": [("gpm_cono", "=", self.conum)],
            "screen": self.mf.body}
        self.cods = {
            "stype": "R",
            "tables": ("strmf1",),
            "cols": (
                ("st1_group", "", 0, "Grp"),
                ("st1_code", "", 0, "Product-Code"),
                ("st1_type", "", 0, "T"),
                ("st1_desc", "", 0, "Description","Y")),
            "where": [],
            "screen": self.mf.body,
            "index": 1}
        loc = {
            "stype": "R",
            "tables": ("strloc",),
            "cols": (
                ("srl_loc", "", 0, "Grp"),
                ("srl_desc", "", 0, "Description", "Y")),
            "where": [("srl_cono", "=", self.conum)]}
        r1s = (
            ("Invoice","I"),
            ("Credit Note","C"),
            ("Sales Order","O"),
            ("Quotation","Q"))
        r2s = (("Account","A"),("Cash","C"))
        fld = [
            (("T",0,0,0),("IRB",r1s),0,"Type","Transaction Type",
                "I","Y",self.doTyp,None,None,None),
            (("T",0,1,0),("IRB",r2s),0,"Mode ","Mode of Payment",
                "A","Y",self.doMod,None,None,None),
            (("T",0,2,0),"IUI",3,"Chain","Chain Store",
                "","N",self.doChn,chn,None,None),
            (("T",0,2,0),"ONA",30,""),
            [["T",0,3,0],"INA",7,"Acc-Num","Account Number",
                "","Y",self.doAcc,drm,None,None],
            [["T",0,3,0],"ONA",30,""],
            [["T",0,4,0],"INa",3,"Rep","Rep Number",
                "","Y",self.doRep,rep,None,None],
            [["T",0,4,0],"ONA",30,""]]
        if self.chns == "N":
            self.chain = 0
            del drm["cols"][0]
            drm["index"] = 0
            del fld[2]
            del fld[2]
            fld[2][0][2] = 2
            fld[3][0][2] = 2
            fld[4][0][2] = 3
            fld[5][0][2] = 3
        else:
            drm["whera"] = [["T", "drm_chain", 2]]
        if self.prices == 1:
            self.level = 1
        else:
            r3s = [("One","1"),("Two","2")]
            if self.prices > 2:
                r3s.append(("Three","3"))
            if self.prices > 3:
                r3s.append(("Four","4"))
            if self.prices > 4:
                r3s.append(("Five","5"))
            if self.chns == "Y":
                fld.append([("T",0,5,0),("IRB",r3s),0,"Level",
                "Price Level","","Y",self.doLev,None,None,None])
            else:
                fld.append([("T",0,4,0),("IRB",r3s),0,"Level",
                "Price Level","","Y",self.doLev,None,None,None])
        fld.extend((
            (("C",0,0,0),"INA",3,"Grp","Product Group(noesc)",
                "r","Y",self.doGrp,self.grps.copy(),None,None),
            (("C",0,0,1),"INA",(10,20),"Code","Product Code",
                "","N",self.doCod,self.cods.copy(),None,None),
            (("C",0,0,2),"ITv",30,"Description","Description",
                "","N",self.doDes,None,None,None),
            (("C",0,0,3),"IUA",1,"L","Location",
                "r","N",self.doLoc,loc,None,None),
            (("C",0,0,4),"ISD",11.2,"Quantity","Quantity",
                1,"N",self.doQty,None,None,("notzero",)),
            (("C",0,0,5),"INA",1,"V","V.A.T. Code",
                "I","N",self.doVat,None,None,("notblank",)),
            (("C",0,0,6),"IUD",11.2,"Price","Price",
                "","Y",self.doPri,None,None,None),
            (("C",0,0,7),"IUD",6.2,"Dis-%","Discount Percentage",
                "","N",self.doDis,None,None,None),
            (("C",0,0,8),"OSD",11.2,"Value")))
        self.row = (15,)
        self.but = (
            ("Cancel",None,self.doCancel,1,("C",0,1),("T",0,1),
                "Cancel the Document"),
            ("DelAdd",None,self.doDelAdd,1,("C",0,1),("T",0,1),
                "Enter a Delivery Address"),
            ("Ribbon",None,self.doRibbon,1,("C",0,1),("T",0,1),
                "Enter Ribbon Line Details"),
            ("Message",None,self.doMessage,1,("C",0,1),("T",0,1),
                "Enter a Message"),
            ("Edit",None,self.doEdit,0,("C",0,1),("C",0,2),
                "Display All Lines and Allow Editing"),
            ("Reprint",None,self.doReprnt,0,("T",0,1),(("T",0,2),("T",0,0)),
                "Reprint Documents"),
            ("DrsMaint",None,self.doNewDrs,0,("T",0,3),("T",0,5),
                "Maintain Debtors Accounts"),
            ("DrsQuery",None,self.doQryDrs,1,None,None,
                "Interrogate Debtors Accounts"),
            ("StrMaint",None,self.doNewStr,0,("C",0,1),("C",0,2),
                "Maintain Stores Records"),
            ("StrQuery",None,self.doQryStk,1,None,None,
                "Interrogate Stores Records"),
            ("Exit",None,self.doTopExit,0,("T",0,1),("T",0,2),
                "Exit the Sales Invoicing Routine"),
            ("Accept",None,self.doAccept,0,("C",0,1),(("T",0,1),("C",0,2)),
                "Accept and Process the Document"))
        tnd = ((self.doTopEnd,"n"), )
        txt = (self.doTopExit, )
        cnd = ((self.doColEnd,"n"), )
        cxt = (None, )
        self.df = TartanDialog(self.mf, screen=self.mf.body, title=tit,
            eflds=fld, rows=self.row, butt=self.but, tend=tnd, txit=txt,
            cend=cnd, cxit=cxt, vport=True)

    def doTyp(self, frt, pag, r, c, p, i, w):
        if w in ("C", "I") and self.allow == "N":
            return "Unauthorised Type"
        self.typs = w
        # Initialise Variables
        self.accept = None
        self.crlimit = None
        self.dnum = ""
        self.onum = ""
        self.jnum = ""
        self.dadd = ""
        self.cnam = ""
        self.vatn = ""
        self.ribbon = None
        self.drsdel = None
        self.ctlmes = None
        # Initialise Action Variables
        self.actxit = False
        self.actinv = False
        self.acttyp = None
        #
        if self.typs == "C":
            fg = "red"
        else:
            fg = "blue"
        if self.typs in ("O", "Q"):
            self.doAction()
            if self.actxit:
                # Refocus on type field
                return "rf"
            if self.acttyp in ("A", "D"):
                err = self.doReadLoadDrs()
                if err:
                    return err
                self.df.loadEntry("T", 0, 1, data=self.mode)
                if self.chns == "Y":
                    chn = self.sql.getRec(tables="drschn", cols=["chm_name"],
                        where=[("chm_cono", "=", self.conum), ("chm_chain",
                        "=", self.chain)], limit=1)
                    if chn:
                        nam = chn[0]
                    else:
                        nam = ""
                    self.df.loadEntry("T", 0, 2, data=self.chain)
                    self.df.loadEntry("T", 0, 3, data=nam)
                    num = 4
                else:
                    num = 2
                self.df.loadEntry("T", 0, num, data=self.acno)
                self.df.loadEntry("T", 0, num+1, data=self.name)
                self.df.loadEntry("T", 0, num+2, data=self.repno)
                self.df.loadEntry("T", 0, num+3, data=self.repnm)
                if self.prices == 1:
                    return "nd"
                self.df.loadEntry("T", 0, num+4, data=str(self.level))
                return "nd"
            elif self.acttyp == "C":
                # Force focus to field 1
                return "ff1"

    def doAction(self):
        tit = ("Select Method",)
        doc = {
            "stype": "R",
            "tables": ("slsiv1", "drsmst"),
        "cols": [
            ("si1_docno", "", 0, "Doc-Num"),
            ("si1_chain", "", 0, "Chn"),
            ("si1_acno", "", 0, "Acc-Num"),
            ("drm_name", "", 0, "Name", "Y")],
        "where": [("si1_cono", "=", self.conum), ("si1_rtn", "=", self.typs),
            ("si1_invno", "=", ""), ("drm_cono=si1_cono",),
            ("drm_chain=si1_chain",), ("drm_acno=si1_acno",)],
        "screen": self.df.mstFrame}
        r1s = (
            ("New", "N"),
            ("Amend", "A"),
            ("Despatch", "D"),
            ("Cancel", "C"))
        fld = [
            [["T",0,0,0],("IRB",r1s),0,"Action","Action Type",
                "N","N",self.doActTyp,None,None,None],
            [["T",0,1,0],"IUI",9,"Document","Document Number",
                "","N",self.doActDoc,doc,None,("notzero",)]]
        state = self.df.disableButtons()
        self.at = TartanDialog(self.mf, title=tit, newscn=(0,1), tops=True,
            eflds=fld, tend=((self.doActEnd,"n"),), txit=(self.doActExit,))
        self.mf.startLoop()
        self.df.enableButtons(state=state)

    def doActTyp(self, frt, pag, r, c, p, i, w):
        if w == "D" and self.allow == "N":
            return "Unauthorised Selection"
        self.acttyp = w
        if self.acttyp == "N":
            return "sk1"

    def doActDoc(self, frt, pag, r, c, p, i, w):
        slsiv1 = self.sql.getRec(tables="slsiv1", where=[("si1_cono", "=",
            self.conum), ("si1_rtn", "=", self.typs), ("si1_docno", "=", w),
            ("si1_invno", "=", "")], limit=1)
        if not slsiv1:
            return "Invalid Document"
        self.docno = CCD(w, "UI", 9.0).work
        self.othno = CCD(w, "Na", 9.0).work
        self.repno = slsiv1[self.sql.slsiv1_col.index("si1_rep")]
        self.chain = slsiv1[self.sql.slsiv1_col.index("si1_chain")]
        self.acno = slsiv1[self.sql.slsiv1_col.index("si1_acno")]
        self.mode = slsiv1[self.sql.slsiv1_col.index("si1_mode")]
        self.level = slsiv1[self.sql.slsiv1_col.index("si1_level")]
        if not self.level:
            self.level = 1
        self.dnum = slsiv1[self.sql.slsiv1_col.index("si1_our_ord")]
        self.onum = slsiv1[self.sql.slsiv1_col.index("si1_cus_ord")]
        self.jnum = slsiv1[self.sql.slsiv1_col.index("si1_jobno")]
        self.cnam = slsiv1[self.sql.slsiv1_col.index("si1_contact")]
        self.vatn = slsiv1[self.sql.slsiv1_col.index("si1_vatno")]
        self.ribbon = [self.dnum, self.onum, self.jnum, self.cnam, self.vatn]
        if not slsiv1[self.sql.slsiv1_col.index("si1_add1")]:
            self.drsdel = None
        else:
            self.drsdel = ["",
                slsiv1[self.sql.slsiv1_col.index("si1_add1")],
                slsiv1[self.sql.slsiv1_col.index("si1_add2")],
                slsiv1[self.sql.slsiv1_col.index("si1_add3")],
                slsiv1[self.sql.slsiv1_col.index("si1_add4")]]
        if not slsiv1[self.sql.slsiv1_col.index("si1_mes1")]:
            self.ctlmes = None
        else:
            self.ctlmes = ["INV", 0,
                slsiv1[self.sql.slsiv1_col.index("si1_mes1")] + " " +
                slsiv1[self.sql.slsiv1_col.index("si1_mes2")] + " " +
                slsiv1[self.sql.slsiv1_col.index("si1_mes3")] + " " +
                slsiv1[self.sql.slsiv1_col.index("si1_mes4")] + " " +
                slsiv1[self.sql.slsiv1_col.index("si1_mes5")]]

    def doActEnd(self):
        if self.acttyp == "C":
            self.sql.updRec("slsiv1", cols=["si1_invno"], data=["cancel"],
                where=[("si1_cono", "=", self.conum), ("si1_rtn", "=",
                self.typs), ("si1_docno", "=", self.docno)])
            self.sql.delRec("slsiv2", where=[("si2_cono", "=", self.conum),
                ("si2_rtn", "=", self.typs), ("si2_docno", "=", self.docno)])
            self.sql.delRec("slsiv3", where=[("si3_cono", "=", self.conum),
                ("si3_rtn", "=", self.typs), ("si3_docno", "=", self.docno)])
            # Commit Deletetion
            self.mf.dbm.commitDbase()
        self.doActClose()

    def doActExit(self):
        self.doActClose()
        self.actxit = True

    def doActClose(self):
        self.at.closeProcess()
        self.mf.closeLoop()

    def doMod(self, frt, pag, r, c, p, i, w):
        self.mode = w
        if self.mode == "C":
            self.bal = 0
            self.chain = 0
            self.acno = "CASHSLS"
            self.drsmst = None
            while not self.drsmst:
                err = self.doReadLoadDrs()
                if not self.drsmst:
                    data = [self.conum, self.chain, self.acno,
                        "Cash Sales Account", "", "", "", "", "", "", "",
                        "", "", "", "", "", 0, 0, "", "", "", "", "", "",
                        0, 0, 0, 0, 0, 0, "N", 0, 0, ""]
                    self.sql.insRec("drsmst", data=data)
                elif err:
                    break
                    return err
            self.name = self.drsmst[self.sql.drsmst_col.index("drm_name")]
            if self.chns == "Y":
                idx = p + 3
            else:
                idx = p + 1
            self.df.loadEntry("T", 0, idx, data=self.acno)
            self.df.loadEntry("T", 0, idx+1, data=self.name)
            if self.chns == "Y":
                return "sk4"
            else:
                return "sk2"

    def doChn(self, frt, pag, r, c, p, i, w):
        self.chain = w
        if self.chain:
            chn = self.sql.getRec(tables="drschn", cols=["chm_name"],
                where=[("chm_cono", "=", self.conum), ("chm_chain", "=", w)],
                limit=1)
            if not chn:
                return "Invalid Chain"
            self.df.loadEntry(frt, pag, p+1, data=chn[0])

    def doAcc(self, frt, pag, r, c, p, i, w):
        self.acno = w
        err = self.doReadLoadDrs()
        if err:
            return err
        self.df.loadEntry(frt, pag, p+1, data=self.name)
        if self.chns == "Y":
            self.df.topf[0][6][5] = self.repno
        else:
            self.df.topf[0][4][5] = self.repno

    def doReadLoadDrs(self):
        self.drsmst = self.sql.getRec(tables="drsmst", where=[("drm_cono", "=",
            self.conum), ("drm_chain", "=", self.chain), ("drm_acno", "=",
            self.acno)], limit=1)
        if not self.drsmst:
            return "Invalid Account"
        if self.drsmst[self.sql.drsmst_col.index("drm_stop")] == "Y":
            return "Account Stopped"
        ref = int(self.drsmst[self.sql.drsmst_col.index("drm_rfterms")] / 30)
        rej = int(self.drsmst[self.sql.drsmst_col.index("drm_rjterms")] / 30)
        bal = Balances(self.mf, "DRS", self.conum, self.curdt,
            [self.chain, self.acno])
        obal, self.bal, age = bal.doAllBals()
        if self.bal > 0:
            if not self.crlimit and ref > 0:
                bal = 0
                for y in range(ref-1, 5):
                    bal = float(ASD(bal) + ASD(age[y]))
                if bal > 0:
                    state = self.df.disableButtons()
                    cf = PwdConfirm(self.mf, conum=self.conum, system="DRS",
                        code="RefLimit")
                    self.df.enableButtons(state=state)
                    if cf.flag == "no":
                        return "Referral Terms Exceeded"
                    self.crlimit = "ok"
            if not self.crlimit and rej > 0:
                bal = 0
                for y in range(rej-1, 5):
                    bal = float(ASD(bal) + ASD(age[y]))
                if bal > 0:
                    state = self.df.disableButtons()
                    cf = PwdConfirm(self.mf, conum=self.conum, system="DRS",
                        code="RejLimit")
                    self.df.enableButtons(state=state)
                    if cf.flag == "no":
                         return "Rejected Terms Exceeded"
                    self.crlimit = "ok"
        if self.acttyp and self.acttyp in ("A", "D"):
            pass
        else:
            self.repno = self.drsmst[self.sql.drsmst_col.index("drm_rep")]
        if self.repno:
            nam = self.sql.getRec(tables="ctlrep", cols=["rpm_name"],
                where=[("rpm_cono", "=", self.conum), ("rpm_rep", "=",
                self.repno)], limit=1)
            if nam:
                self.repnm = nam[0]
            else:
                self.repnm = ""
        else:
            self.repnm = ""
        self.name = self.drsmst[self.sql.drsmst_col.index("drm_name")]
        self.dadd = self.drsmst[self.sql.drsmst_col.index("drm_delivery")]
        self.dmes = self.drsmst[self.sql.drsmst_col.index("drm_invmes")]
        if self.typs in ("O", "Q") and self.acttyp in ("A", "D"):
            return
        self.drsdel = self.sql.getRec(tables="drsdel",
            where=[("del_code", "=", self.dadd)], limit=1)
        self.ctlmes = self.sql.getRec(tables="ctlmes", where=[("mss_system",
            "=", "INV"), ("mss_message", "=", self.dmes)], limit=1)
        self.cnam = self.drsmst[self.sql.drsmst_col.index("drm_sls")]
        self.vatn = self.drsmst[self.sql.drsmst_col.index("drm_vatno")]
        self.ribbon = [self.dnum, self.onum, self.jnum, self.cnam, self.vatn]

    def doRep(self, frt, pag, r, c, p, i, w):
        nam = self.sql.getRec(tables="ctlrep", cols=["rpm_name"],
            where=[("rpm_cono", "=", self.conum), ("rpm_rep", "=", w)], limit=1)
        if not nam:
            return "Invalid Rep Code"
        if self.repno and w != self.repno:
            state = self.df.disableButtons()
            cf = PwdConfirm(self.mf, conum=self.conum, system="INV",
                code="ChangeRep")
            self.df.enableButtons(state=state)
            if cf.flag == "no":
                return "rf"
        self.repno = w
        self.repnm = nam[0]
        self.df.loadEntry(frt, pag, p+1, data=self.repnm)
        if self.prices == 1:
            pass
        else:
            lev = str(self.drsmst[self.sql.drsmst_col.index("drm_prices")])
            if lev in ("", "0"):
                lev = "1"
            if self.chns == "Y":
                self.df.topf[0][8][5] = lev
            else:
                self.df.topf[0][6][5] = lev

    def doLev(self, frt, pag, r, c, p, i, w):
        self.level = int(w)

    def doTopEnd(self):
        if not self.drsmst:
            if self.chns == "Y":
                self.df.focusField("T", 0, 5)
            else:
                self.df.focusField("T", 0, 3)
        elif self.acttyp in ("A", "D"):
            self.doReadLoadStr()
            self.amend = False
        else:
            # Get Next Document Number
            self.doGetDocno(self.typs)
            # Create Document Transaction (Head)
            self.dad1, self.dad2, self.dad3, self.dad4 = "", "", "", ""
            if self.typs in ("I", "C"):
                date = self.trdt
            else:
                date = self.sysdtw
            data = [self.conum, self.typs, self.docno, self.mode,
                self.level, self.chain, self.acno, self.dad1, self.dad2,
                self.dad3, self.dad4, date, "", "", "", "", "", self.repno,
                "I", "", "", "", "", "", "", self.sysdtw]
            # Write and Commit the new document header
            self.sql.insRec("slsiv1", data=data)
            self.mf.dbm.commitDbase()
            # Clear Totals for document
            self.doClearTots()

    def doReadLoadStr(self):
        self.doClearTots()
        slsiv2 = self.sql.getRec(tables="slsiv2", where=[("si2_cono", "=",
            self.conum), ("si2_rtn", "=", self.typs), ("si2_docno", "=",
            self.docno)], order="si2_seq")
        if not slsiv2:
            return
        self.actinv = True
        for seq, line in enumerate(slsiv2):
            self.doExtractData(line)
            self.sql.updRec("slsiv2", cols=["si2_seq"], data=[seq],
                where=[("si2_cono", "=", self.conum), ("si2_rtn", "=",
                self.typs), ("si2_docno", "=", self.docno), ("si2_seq", "=",
                self.oldline)])
            self.sql.updRec("slsiv3", cols=["si3_seq"], data=[seq],
                where=[("si3_cono", "=", self.conum), ("si3_rtn", "=",
                self.typs), ("si3_docno", "=", self.docno), ("si3_seq", "=",
                self.oldline)])
            if seq >= self.row[0]:
                pos = (self.row[0] - 1) * 9
            else:
                pos = seq * 9
            # Load Values
            self.df.doInvoke(["C", 0, (pos+1), "rt"], self.grp)
            self.df.doInvoke(["C", 0, (pos+2), "rt"], self.code)
            self.df.doInvoke(["C", 0, (pos+3), "rt"], self.desc)
            self.df.doInvoke(["C", 0, (pos+4), "rt"], self.loc)
            self.df.doInvoke(["C", 0, (pos+5), "rt"], self.qty)
            self.df.doInvoke(["C", 0, (pos+6), "rt"], self.vatcod)
            self.df.doInvoke(["C", 0, (pos+7), "rt"], self.price)
            self.df.doInvoke(["C", 0, (pos+8), self.doDis], self.disrat)
        self.actinv = False

    def doTopExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

    def doClearTots(self):
        self.totvat = 0
        self.totinv = 0
        self.df.setViewPort(self.typs, self.totinv)
        self.df.clearFrame("C", 0)
        self.df.focusField("C", 0, 1)

    def doGrp(self, frt, pag, r, c, p, i, w):
        ########################################################################
        # Get Next Line Number
        ########################################################################
        seq = self.sql.getRec(tables="slsiv2", cols=["count(*)"],
            where=[("si2_cono", "=", self.conum), ("si2_rtn", "=", self.typs),
            ("si2_docno", "=", self.docno)], limit=1)
        self.newline = int(seq[0])
        ########################################################################
        self.grp = w
        self.strgrp = self.doReadGroup(self.grp)
        if not self.strgrp:
            return "Invalid Group Code"
        self.df.colf[pag][1][8]["where"] = [("st1_cono", "=", self.conum),
            ("st1_group", "=", self.grp)]

    def doCod(self, frt, pag, r, c, p, i, w):
        self.strmf1 = self.sql.getRec(tables="strmf1", where=[("st1_cono", "=",
            self.conum), ("st1_group", "=", self.grp), ("st1_code", "=", w)],
            limit=1)
        if not self.strmf1:
            return "Invalid Code"
        if self.stgl == "Y":
            gl = self.sql.getRec(tables="genmst", cols=["glm_desc"],
                where=[("glm_cono", "=", self.conum), ("glm_acno", "=",
                self.strmf1[self.sql.strmf1_col.index("st1_sls")])], limit=1)
            if not gl:
                return "Invalid GL Sales Code"
            gl = self.sql.getRec(tables="genmst", cols=["glm_desc"],
                where=[("glm_cono", "=", self.conum), ("glm_acno", "=",
                self.strmf1[self.sql.strmf1_col.index("st1_cos")])], limit=1)
            if not gl:
                return "Invalid GL Cost of Sales Code"
        self.code = w
        self.gtype = self.strmf1[self.sql.strmf1_col.index("st1_type")]
        desc = self.strmf1[self.sql.strmf1_col.index("st1_desc")]
        self.df.loadEntry(frt, pag, p+1, data=desc)

    def doReadGroup(self, group):
        strgrp = self.sql.getRec(tables="strgrp", where=[("gpm_cono", "=",
            self.conum), ("gpm_group", "=", group)], limit=1)
        return strgrp

    def doDes(self, frt, pag, r, c, p, i, w):
        self.desc = w
        if not r:
            if self.locs == "N":
                self.loc = "1"
                self.df.loadEntry(frt, pag, p+1, data=self.loc)
                return "sk1"
            else:
                acc = self.sql.getRec(tables="strloc", cols=["srl_loc"],
                    where=[("srl_cono", "=", self.conum)], limit=1)
                self.loc = acc[0]
                self.df.loadEntry(frt, pag, p+1, data=self.loc)

    def doLoc(self, frt, pag, r, c, p, i, w):
        self.strmf2 = self.sql.getRec(tables="strmf2", where=[("st2_cono", "=",
            self.conum), ("st2_group", "=", self.grp), ("st2_code", "=",
            self.code), ("st2_loc", "=", w)], limit=1)
        if not self.strmf2:
            return "Invalid Location"
        self.loc = w

    def doQty(self, frt, pag, r, c, p, i, w):
        self.qty = w
        if self.gtype == "R":
            ok = self.doRecipe(atype="I", lineno=self.newline)
            if ok == "NoItems":
                return "Invalid Recipe, No Items"
            elif ok == "NoQty":
                return "cl"
            elif ok == "Quit":
                return "cl"
            self.doRecipe(atype="S", lineno=self.newline)
        else:
            self.doCalSell()
            if self.typs in ("C", "O", "Q"):
                pass
            else:
                # Check for quantity on hand
                bal = Balances(self.mf, "STR", self.conum, self.curdt,
                    (self.grp, self.code, self.loc, ("P", self.period[0])))
                bals = bal.doStrBals()
                qty = CCD(bals[5][0], "SD", 11.2)
                ind = self.strmf1[self.sql.strmf1_col.index("st1_value_ind")]
                if ind != "N" and self.typs != "C" and self.qty > qty.work:
                    state = self.df.disableButtons()
                    cf = PwdConfirm(self.mf, conum=self.conum, system="INV",
                        code="ExQty")
                    self.df.enableButtons(state=state)
                    if cf.flag == "no":
                        return "cl"
        vat = self.strgrp[self.sql.strgrp_col.index("gpm_vatcode")]
        self.df.loadEntry(frt, pag, p+1, data=vat)

    def doRecipe(self, atype="I", lineno=0):
        """
        I = Load Recipe and Allow Changes
        C = Change Quantity
        U = Get Cost Prices
        S = Get Selling Price
        """
        self.lineno = lineno
        if atype == "I":
            try:
                self.sql.delRec("slsiv3", where=[("si3_cono", "=", self.conum),
                ("si3_rtn", "=", self.typs), ("si3_docno", "=", self.docno),
                ("si3_seq", "=", self.lineno)])
            except:
                pass
            recipe = self.sql.getRec("strrcp", where=[("srr_cono", "=",
                self.conum), ("srr_group", "=", self.grp), ("srr_code", "=",
                self.code), ("srr_loc", "=", self.loc)])
            if not recipe:
                return "NoItems"
            self.recipe = []
            needpwd = False
            for item in recipe:
                quant = item[6] * self.qty
                vals = self.doCalCost(item[4], item[5], self.loc, 1)
                if vals[0] != "N" and quant > vals[3]:
                    needpwd = True
                data = [self.conum, self.typs, self.docno, self.lineno,
                    item[4], item[5], item[6], vals[1], 0]
                self.recipe.append(data)
            if needpwd:
                state = self.df.disableButtons()
                cf = PwdConfirm(self.mf, conum=self.conum, system="INV",
                    code="ExQty")
                self.df.enableButtons(state=state)
                if cf.flag == "no":
                    return "NoQty"
            self.doRecChg()
            if self.rectyp == "quit":
                return "Quit"
            for item in self.recipe:
                self.sql.insRec("slsiv3", data=item)
        elif atype == "C":
            self.recipe = self.sql.getRec("slsiv3", where=[("si3_cono", "=",
                self.conum), ("si3_rtn", "=", self.typs), ("si3_docno", "=",
                self.docno), ("si3_seq", "=", self.lineno)])
            self.sql.delRec("slsiv3", where=[("si3_cono", "=", self.conum),
                ("si3_rtn", "=", self.typs), ("si3_docno", "=", self.docno),
                ("si3_seq", "=", self.lineno)])
            self.doRecChg()
            if self.rectyp == "quit":
                return "Quit"
            for item in self.recipe:
                self.sql.insRec("slsiv3", data=item)
        elif atype == "U":
            self.icost = 0
            self.tcost = 0
            recipe = self.sql.getRec("slsiv3", where=[("si3_cono", "=",
                self.conum), ("si3_rtn", "=", self.typs), ("si3_docno", "=",
                self.docno), ("si3_seq", "=", self.lineno)])
            for item in recipe:
                quant = item[6] * self.qty
                vals = self.doCalCost(item[4], item[5], self.loc, quant)
                self.tcost = float(ASD(self.tcost) + ASD(vals[2]))
            self.icost = round(self.tcost / self.qty, 2)
        elif atype == "S":
            self.doCalSell()

    def doRecChg(self):
        # Display recipe items and allow editing of quantities etc.
        data = []
        for num, item in enumerate(self.recipe):
            desc = self.sql.getRec(tables="strmf1", cols=["st1_desc"],
                where=[("st1_cono", "=", self.conum), ("st1_group", "=",
                item[4]), ("st1_code", "=", item[5])], limit=1)
            cost = self.doCalCost(item[4], item[5], self.loc, item[6])
            data.append([num, item[4], item[5], desc[0], item[6], cost[2]])
        titl = "Recipe Items"
        head = ("Seq","Grp","Product-Code", "Description", "  Quantity",
            "      Cost")
        lin = {
            "stype": "C",
            "titl": titl,
            "head": head,
            "typs": [
                ("UI", 2),
                ("NA", 3),
                ("NA", 20),
                ("NA", 30),
                ("SD", 11.2),
                ("SD", 11.2)],
            "data": data,
            "butt": [
                ("Add", self.doRecAdd),
                ("Accept", self.doRecAccept),
                ("Quit", self.doRecQuit)]}
        state = self.df.disableButtons()
        self.mf.updateStatus("Select a Product to Edit or an Action")
        self.rectyp = None
        chg = self.df.selChoice(lin)
        self.df.enableButtons(state=state)
        if chg and chg.selection:
            self.rectyp = "chg"
            self.recchg = chg.selection
        if self.rectyp in ("add", "chg"):
            self.doRecChanges()
            self.doRecChg()

    def doRecAdd(self, widget=None):
        self.rectyp = "add"

    def doRecAccept(self, widget=None):
        self.rectyp = "continue"

    def doRecQuit(self, widget=None):
        self.rectyp = "quit"

    def doRecChanges(self):
        fld = []
        if self.rectyp == "chg":
            tit = ("Change Item",)
            fld = (
                (("T",0,0,0),"ONA",3,"Group"),
                (("T",0,1,0),"ONA",20,"Code"),
                (("T",0,2,0),"ONA",30,"Description"),
                (("T",0,3,0),"ISD",11.2,"Quantity","Quantity",
                    "","N",self.doRecQty,None,None,("notzero",)))
            but = (("Delete",None,self.doRecDel,1,None,None),)
        else:
            tit = ("Add Item",)
            fld = (
                (("T",0,0,0),"INA",3,"Group","Product Group",
                    "","N",self.doRecGrp,self.grps.copy(),None,None),
                (("T",0,1,0),"INA",20,"Code", "Product Code",
                    "","N",self.doRecCod,self.cods.copy(),None,None),
                (("T",0,2,0),"ONA",30,"Description"),
                (("T",0,3,0),"ISD",11.2,"Quantity","Quantity",
                    "","N",self.doRecQty,None,None,("notzero",)))
            but = None
        state = self.df.disableButtons()
        self.rp = TartanDialog(self.mf, title=tit, newscn=(0,1),
            tops=True, eflds=fld, butt=but, tend=((self.doRecEnd,"n"),),
            txit=(self.doRecExit,))
        if self.rectyp == "chg":
            self.rp.loadEntry("T", 0, 0, data=self.recchg[1])
            self.rp.loadEntry("T", 0, 1, data=self.recchg[2])
            self.rp.loadEntry("T", 0, 2, data=self.recchg[3])
            self.rp.loadEntry("T", 0, 3, data=self.recchg[4])
            self.rp.focusField("T", 0, 4, clr=False)
        else:
            self.rp.focusField("T", 0, 1)
        self.mf.startLoop()
        self.df.enableButtons(state=state)

    def doRecGrp(self, frt, pag, r, c, p, i, w):
        strgrp = self.doReadGroup(w)
        if not strgrp:
            return "Invalid Group Code"
        self.recgrp = w
        self.rp.topf[pag][1][8]["where"] = [("st1_cono", "=", self.conum),
            ("st1_group", "=", self.recgrp), ("st1_type", "not", "in",
            ("J", "R"))]

    def doRecCod(self, frt, pag, r, c, p, i, w):
        strmf1 = self.sql.getRec(tables="strmf1", where=[("st1_cono", "=",
            self.conum), ("st1_group", "=", self.recgrp), ("st1_code", "=",
            w), ("st1_type", "not", "in", ("J", "R"))], limit=1)
        if not strmf1:
            return "Invalid Code"
        self.reccod = w
        desc = strmf1[self.sql.strmf1_col.index("st1_desc")]
        self.rp.loadEntry(frt, pag, p+1, data=desc)

    def doRecQty(self, frt, pag, r, c, p, i, w):
        self.recqty = w

    def doRecDel(self, widget=None):
        del self.recipe[int(self.recchg[0])]
        self.doRecExit()

    def doRecEnd(self):
        if self.rectyp == "chg":
            self.recipe[int(self.recchg[0])][6] = self.recqty
        else:
            quant = self.recqty * self.qty
            vals = self.doCalCost(self.recgrp, self.reccod, self.loc, 1)
            self.recipe.append([self.conum, self.typs, self.docno,
                self.lineno, self.recgrp, self.reccod, self.recqty,
                vals[1], 0])
        self.doRecExit()

    def doRecExit(self):
        self.rp.closeProcess()
        self.mf.closeLoop()

    def doVat(self, frt, pag, r, c, p, i, w):
        vat = self.sql.getRec(tables="ctlvmf", cols=["vtm_new"],
            where=[("vtm_cono", "=", self.conum), ("vtm_code", "=", w)],
            limit=1)
        if not vat:
            return "Invalid V.A.T Code"
        self.vatcod = w
        self.vatrat = float(vat[0])
        self.df.loadEntry(frt, pag, p+1, data=self.rrp)

    def doPri(self, frt, pag, r, c, p, i, w):
        if self.gtype == "R":
            self.doRecipe(atype="U", lineno=self.newline)
            icost = self.icost
            tcost = self.tcost
        else:
            ind, icost, tcost, cbal = self.doCalCost(self.grp, self.code,
                self.loc, self.qty)
        if w < icost:
            state = self.df.disableButtons()
            cf = PwdConfirm(self.mf, conum=self.conum, system="INV",
                code="BelowCost")
            self.df.enableButtons(state=state)
            if cf.flag == "no":
                return "rf"
        self.price = w
        unt = self.strmf1[self.sql.strmf1_col.index("st1_units")]
        if not unt:
            pri = self.price
        else:
            pri = round((self.price / unt), 2)
        rat = float(ASD(100.0) + ASD(self.vatrat))
        self.inc = round((pri * rat / 100.0), 4)
        self.exc = round((pri * 1), 2)
        dis = self.drsmst[self.sql.drsmst_col.index("drm_dis_per")]
        self.df.loadEntry(frt, pag, p+1, data=dis)

    def doDis(self, frt, pag, r, c, p, i, w):
        if self.actinv:
            pass
        else:
            self.disrat = w
            self.uoi = self.strmf1[self.sql.strmf1_col.index("st1_uoi")]
            self.unt = self.strmf1[self.sql.strmf1_col.index("st1_units")]
            dis = float(ASD(100.0) - ASD(self.disrat))
            self.excamt = round((self.qty * self.exc * dis / 100.0), 2)
            self.incamt = round((self.qty * self.inc * dis / 100.0), 2)
            self.vatamt = float(ASD(self.incamt) - ASD(self.excamt))
        self.df.loadEntry(frt, pag, p+1, data=self.incamt)
        self.totvat = float(ASD(self.totvat) + ASD(self.vatamt))
        self.totinv = float(ASD(self.totinv) + ASD(self.incamt))
        lim = self.drsmst[self.sql.drsmst_col.index("drm_limit")]
        bal = float(ASD(self.bal) + ASD(self.totinv))
        if not self.crlimit and lim > 0 and bal > lim:
            state = self.df.disableButtons()
            cf = PwdConfirm(self.mf, conum=self.conum, system="DRS",
                code="CrLimit")
            self.df.enableButtons(state=state)
            if cf.flag == "no":
                self.totvat = float(ASD(self.totvat) - ASD(self.vatamt))
                self.totinv = float(ASD(self.totinv) - ASD(self.incamt))
                return "cl"
            self.crlimit = "ok"

    def doColEnd(self):
        if self.actinv:
            pass
        else:
            # Create Sales Invoice Transaction (Body)
            data = [self.conum, self.typs, self.docno, self.newline,
                self.grp, self.code, self.loc, self.uoi, self.unt,
                self.desc, self.disrat, self.qty, self.price,
                self.vatcod, self.vatrat, self.sysdtw]
            self.sql.insRec("slsiv2", data=data)
            if self.acttyp == "A":
                self.amend = True
        self.df.advanceLine(0)
        self.df.setViewPort(self.typs, self.totinv)

    def doQryDrs(self, widget=None):
        callModule(self.mf, self.df, "dr4010", coy=(self.conum, self.conam),
            period=None, user=self.capnm)

    def doQryStk(self, widget=None):
        callModule(self.mf, self.df, "st4010", coy=(self.conum, self.conam),
            period=self.period, user=self.capnm)

    def doDelAdd(self, widget=None):
        tit = ("Delivery Address",)
        cod = {
            "stype": "R",
            "tables": ("drsdel",),
            "cols": [
                ("del_code", "", 0, "Del-Cod"),
                ("del_add1", "", 0, "Address Line 1"),
                ("del_add2", "", 0, "Address Line 2")],
            "screen": self.mf.body}
        fld = (
            (("T",0,0,0),"INa",7,"Del-Cod","Delivery Code",
                self.dadd,"N",self.doDelCod,cod,None,None),
            (("T",0,1,0),"INA",30,"Address-1","Address Line 1",
                "","N",None,None,None,None),
            (("T",0,2,0),"INA",30,"Address-2","Address Line 2",
                "","N",None,None,None,None),
            (("T",0,3,0),"INA",30,"Address-3","Address Line 3",
                "","N",None,None,None,None),
            (("T",0,4,0),"INA",30,"Address-4","Address Line 4",
                "","N",None,cod,None,None))
        state = self.df.disableButtons()
        self.tx = TartanDialog(self.mf, title=tit, newscn=(0,1), tops=True,
            eflds=fld, tend=((self.doDelEnd,"y"),), txit=(self.doDelExit,),
            focus=False)
        if self.drsdel:
            self.doLoadDel("y")
        else:
            self.tx.focusField("T", 0, 1)
        self.mf.startLoop()
        self.df.enableButtons(state=state)
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def doDelCod(self, frt, pag, r, c, p, i, w):
        if not w:
            return
        self.drsdel = self.sql.getRec(tables="drsdel", where=[("del_code",
            "=", w)], limit=1)
        if self.drsdel:
            self.doLoadDel()
            return "nd"

    def doLoadDel(self, focus=False):
        for n, m in enumerate(self.drsdel):
            self.tx.loadEntry("T", 0, n, data=m)
        if focus:
            self.tx.focusField("T", 0, 2, clr=False)

    def doDelEnd(self):
        self.drsdel = []
        for x in range(0, self.tx.topq[0]):
            self.drsdel.append(self.tx.t_work[0][0][x])
        self.doDelExit()

    def doDelExit(self):
        self.tx.closeProcess()
        self.mf.closeLoop()

    def doMessage(self, widget=None):
        tit = ("Invoice Message",)
        cod = {
            "stype": "R",
            "tables": ("ctlmes",),
            "cols": [
                ("mss_message", "",  0, "Cod"),
                ("mss_detail",  "", 60, "Details")],
            "where": [("mss_system", "=", "INV")],
            "screen": self.mf.body}
        fld = (
            (("T",0,0,0),"IUI",3,"Message Code","Message Code",
                "","N",self.doMesCod,cod,None,None),
            (("T",0,1,0),"INA",30,"Line-1","Message Line 1",
                "","N",None,None,None,None),
            (("T",0,2,0),"INA",30,"Line-2","Message Line 2",
                "","N",None,None,None,None),
            (("T",0,3,0),"INA",30,"Line-3","Message Line 3",
                "","N",None,None,None,None),
            (("T",0,4,0),"INA",30,"Line-4","Message Line 4",
                "","N",None,None,None,None),
            (("T",0,5,0),"INA",30,"Line-5","Message Line 5",
                "","N",None,None,None,None))
        state = self.df.disableButtons()
        self.tx = TartanDialog(self.mf, title=tit, newscn=(0,1), tops=True,
            eflds=fld, tend=((self.doMesEnd,"y"),), txit=(self.doMesExit,),
            focus=False)
        if self.ctlmes:
            self.doLoadMes("y")
        else:
            self.tx.focusField("T", 0, 1)
        self.mf.startLoop()
        self.df.enableButtons(state=state)
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def doMesCod(self, frt, pag, r, c, p, i, w):
        if w:
            self.ctlmes = self.sql.getRec(tables="ctlmes",
                where=[("mss_system", "=", "INV"), ("mss_message", "=", w)],
                limit=1)
            if not self.ctlmes:
                return "Invalid Message Code"
            self.doLoadMes()
            return "nd"

    def doLoadMes(self, focus=False):
        self.tx.clearFrame("T", 0)
        self.tx.loadEntry("T", 0, 0, data=self.ctlmes[1])
        fmt = textFormat(self.ctlmes[2], width=30)
        if len(fmt) > 5:
            l = 5
        else:
            l = len(fmt)
        for lin in range(0, l):
            self.tx.loadEntry("T", 0, 1+lin, data=fmt[lin])
        if focus:
            self.tx.focusField("T", 0, 2, clr=False)

    def doMesEnd(self):
        self.ctlmes = ["INV", self.tx.t_work[0][0][0]]
        mess = ""
        for m in self.tx.t_work[0][0][1:]:
            mess = mess + m + " "
        words = mess.split(" ")
        mess = ""
        for word in words:
            if word:
                mess = mess + word + " "
        self.ctlmes.append(mess[:-1])
        self.doMesExit()

    def doMesExit(self):
        self.tx.closeProcess()
        self.mf.closeLoop()

    def doEdit(self, widget=None):
        # Display document items and allow editing of desc, qty and price
        data = self.sql.getRec(tables="slsiv2", cols=["si2_seq", "si2_group",
            "si2_code", "si2_desc", "si2_loc", "si2_qty", "si2_vat_code",
            "si2_price", "si2_disc_per"], where=[("si2_cono", "=", self.conum),
            ("si2_rtn", "=", self.typs), ("si2_docno", "=", self.docno)],
            order="si2_seq")
        if data:
            titl = "Document Product Lines"
            head = ("Seq","Grp","Product-Code", "Description","L", "Quantity",
                "V","Price","Disc-%")
            lin = {
                "stype": "C",
                "titl": titl,
                "head": head,
                "data": data}
            state = self.df.disableButtons()
            self.mf.updateStatus("Select a Product to Edit")
            chg = self.df.selChoice(lin)
            if chg and chg.selection:
                self.change = chg.selection
                self.doChgChanges()
            self.df.enableButtons(state=state)
        self.df.focusField("C", 0, self.df.col)

    def doChgChanges(self):
        tit = ("Change Item",)
        fld = (
            (("T",0,0,0),"ONA",3,"Group"),
            (("T",0,1,0),"ONA",20,"Code"),
            (("T",0,2,0),"ITv",30,"Description","Description",
                "","N",self.doChgDes,None,None,("notblank",)),
            (("T",0,3,0),"ISD",11.2,"Quantity","Quantity",
                "","N",self.doChgQty,None,None,("notzero",)),
            (("T",0,4,0),"ISD",11.2,"Selling Price","Selling Price",
                "","N",self.doChgPrc,None,None,("notzero",)))
        but = [["Delete",None,self.doChgDel,1,None,None]]
        self.cg = TartanDialog(self.mf, title=tit, newscn=(0,1),
            tops=True, eflds=fld, butt=but, tend=((self.doChgEnd,"n"),),
            txit=(self.doChgExit,))
        acc = self.sql.getRec(tables="strmf1", cols=["st1_type"],
            where=[("st1_cono", "=", self.conum), ("st1_group", "=",
            self.change[1]), ("st1_code", "=", self.change[2])], limit=1)
        if acc:
            self.gtype = acc[0]
        else:
            self.gtype = "N"
        self.cg.loadEntry("T", 0, 0, data=self.change[1])
        self.cg.loadEntry("T", 0, 1, data=self.change[2])
        self.cg.loadEntry("T", 0, 2, data=self.change[3])
        self.cg.loadEntry("T", 0, 3, data=self.change[5])
        self.cg.loadEntry("T", 0, 4, data=self.change[7])
        self.cg.focusField("T", 0, 3, clr=False)
        self.mf.startLoop()

    def doChgDes(self, frt, pag, r, c, p, i, w):
        self.desc = w

    def doChgQty(self, frt, pag, r, c, p, i, w):
        self.qty = w
        if self.gtype == "R":
            self.cg.window.hide()
            self.doRecipe(atype="C", lineno=self.change[0])
            self.doRecipe(atype="S", lineno=self.change[0])
            self.cg.window.show_all()
            self.cg.loadEntry(frt, pag, p+1, data=self.rrp)
            self.cg.focusField(frt, pag, c)

    def doChgPrc(self, frt, pag, r, c, p, i, w):
        self.price = w
        if self.gtype == "R":
            self.doRecipe(atype="U", lineno=self.change[0])
            icost = self.icost
            tcost = self.tcost
        else:
            ind, icost, tcost, cbal = self.doCalCost(self.change[1],
                self.change[2], self.change[4], self.qty)
        if w < icost:
            state = self.df.disableButtons()
            cf = PwdConfirm(self.mf, conum=self.conum, system="INV",
                code="BelowCost")
            self.df.enableButtons(state=state)
            if cf.flag == "no":
                return "rf"

    def doChgDel(self, widget=None):
        seq = self.change[0]
        self.sql.delRec("slsiv2", where=[("si2_cono", "=", self.conum),
            ("si2_rtn", "=", self.typs), ("si2_docno", "=", self.docno),
            ("si2_seq", "=", seq)])
        self.sql.delRec("slsiv3", where=[("si3_cono", "=", self.conum),
            ("si3_rtn", "=", self.typs), ("si3_docno", "=", self.docno),
            ("si3_seq", "=", seq)])
        if self.acttyp == "A":
            self.amend = True
        self.doReadLoadStr()
        self.doChgExit()

    def doChgEnd(self):
        seq = self.change[0]
        self.sql.updRec("slsiv2", cols=["si2_desc", "si2_qty", "si2_price"],
            data=[self.desc, self.qty, self.price], where=[("si2_cono", "=",
            self.conum), ("si2_rtn", "=", self.typs), ("si2_docno", "=",
            self.docno), ("si2_seq", "=", seq)])
        if self.acttyp == "A":
            self.amend = True
        self.doReadLoadStr()
        self.doChgExit()

    def doChgExit(self):
        self.cg.closeProcess()
        self.mf.closeLoop()

    def doReprnt(self, widget=None):
        tit = ("Reprint Documents",)
        iv1 = {
            "stype": "R",
            "tables": ("slsiv1", "drsmst"),
            "cols": [
                ("si1_docno", "",  0, "Doc-Num"),
                ("si1_date", "",  0, "Date"),
                ("si1_chain", "",  0, "Chn"),
                ("si1_acno",  "", 0, "Acc-Num"),
                ("drm_name",  "", 0, "Name", "Y"),
                ("si1_invno",  "", 0, "Dis-Num")],
            "where": [("si1_cono", "=", self.conum), ("si1_cono=drm_cono",),
                ("si1_chain=drm_chain",), ("si1_acno=drm_acno",)],
            "whera": [["T", "si1_rtn", 1, 0]],
            "screen": self.mf.body}
        r1s = (("Copies", "C"), ("Originals", "O"))
        r2s = (
            ("Inv","I"),
            ("C-N","C"),
            ("S-O","O"),
            ("Qte","Q"))
        r3s = (("Yes", "Y"), ("No", "N"))
        fld = [
            (("T",0,0,0),("IRB",r1s),0,"Document Mode","Document Mode",
                "C","N",self.doMode,None,None,None),
            (("T",0,1,0),("IRB",r2s),0,"Document Type","Document Type",
                "I","N",self.doReprntType,None,None,None)]
        if self.delnos == "Y":
            fld.append(
                (("T",0,2,0),("IRB",r3s),0,"Delivery Note","Delivery Note",
                    self.dnote,"N",None,None,None,None))
            idx = 3
        else:
            idx = 2
        fld.extend([
            (("T",0,idx,0),"IUI",9,"From Number","From Document Number",
                "","N",self.doIv1,iv1,None,("notblank",)),
            (("T",0,idx+1,0),"IUI",9,"To   Number","To Document Number",
                "","N",None,iv1,None,("notblank",))])
        state = self.df.disableButtons()
        self.tx = TartanDialog(self.mf, title=tit, newscn=(0,1),
            tops=True, eflds=fld, tend=((self.doReprntEnd, "n"),),
            txit=(self.doReprntExit,), view=("N","V"), mail=("B","N"))
        self.mf.startLoop()
        self.df.enableButtons(state=state)
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def doMode(self, frt, pag, r, c, p, i, w):
        if w == "C":
            self.copy = "y"
        else:
            self.copy = "n"
            if self.delnos == "Y":
                idx = 5
            else:
                idx = 4
            self.tx.topf[pag][idx][5] = "P"
            self.tx.topf[pag][idx+2][5] = "Y"

    def doIv1(self, frt, pag, r, c, p, i, w):
        if self.delnos == "Y":
            idx = 3
        else:
            idx = 2
        self.tx.topf[pag][idx][5] = w

    def doReprntType(self, frt, pag, r, c, p, i, w):
        if w != "I":
            self.tx.loadEntry(frt, pag, p+1, data="N")
            return "sk1"

    def doReprntEnd(self):
        self.tx.closeProcess()
        self.df.mstAlw.hide()
        typ = self.tx.t_work[0][0][1]
        if self.delnos == "Y":
            dn = self.tx.t_work[0][0][2]
            idx = 3
        else:
            dn = "N"
            idx = 2
        frm = self.tx.t_work[0][0][idx]
        to = self.tx.t_work[0][0][idx+1]
        prt = self.tx.repprt[0][2]
        eml = self.tx.repprt[1][1:3]
        iv1 = self.sql.getRec(tables="slsiv1", cols=["si1_docno"],
            where=[("si1_cono", "=", self.conum), ("si1_rtn", "=", typ),
            ("si1_docno", ">=", frm), ("si1_docno", "<=", to)])
        for doc in iv1:
            PrintInvoice(self.mf, self.conum, typ, doc[0], prtnam=prt,
                email=eml, copy=self.copy, dnote=dn, dvals=self.delval,
                useadd=self.useadd)
        self.df.mstAlw.show()
        self.mf.closeLoop()

    def doReprntExit(self):
        self.tx.closeProcess()
        self.mf.closeLoop()

    def doNewDrs(self, widget=None):
        callModule(self.mf, self.df, "dr1010", coy=(self.conum, self.conam),
            period=None, user=self.capnm)

    def doNewStr(self, widget=None):
        callModule(self.mf, self.df, "st1010", coy=(self.conum, self.conam),
            period=None, user=self.capnm)

    def doAccept(self, widget=None):
        if self.df.col == 1:
            self.doCancel()
            return
        if self.typs in ("O", "Q") and self.acttyp == "D" and self.doChkItems():
            self.df.focusField(self.df.frt, self.df.pag, self.df.col)
            return
        if self.totinv >= 1000.00 and not self.ribbon[4]:
            self.accept = "y"
            self.doRibbon()
            return
        # Update Tables
        self.doUpdateTables()
        # Commit Transaction
        self.mf.dbm.commitDbase()
        # Print Document
        if self.acttyp == "A" and not self.amend:
            pass
        else:
            if self.typs == "O":
                email = "N"
            else:
                email = self.email
            if self.typs == "I":
                dnote = self.dnote
            else:
                dnote = "N"
            self.df.mstAlw.hide()
            PrintInvoice(self.mf, self.conum, self.typs, self.docno,
                prtnam=self.prtnam, email=email, dnote=dnote,
                dvals=self.delval, useadd=self.useadd)
            self.df.mstAlw.show()
        self.df.setViewPort(self.typs, 0)
        self.df.focusField("T", 0, 1)

    def doChkItems(self):
        trn = self.sql.getRec(tables="slsiv2", where=[("si2_cono", "=",
            self.conum), ("si2_rtn", "=", self.typs), ("si2_docno", "=",
            self.docno)])
        needpwd = False
        for line in trn:
            # Check for quantity on hand
            grp = line[self.sql.slsiv2_col.index("si2_group")]
            cod = line[self.sql.slsiv2_col.index("si2_code")]
            loc = line[self.sql.slsiv2_col.index("si2_loc")]
            qty = line[self.sql.slsiv2_col.index("si2_qty")]
            mf1 = self.sql.getRec(tables="strmf1", where=[("st1_cono", "=",
                self.conum), ("st1_group", "=", grp), ("st1_code", "=", cod)],
                limit=1)
            typ = mf1[self.sql.strmf1_col.index("st1_type")]
            ind = mf1[self.sql.strmf1_col.index("st1_value_ind")]
            if typ != "R" and ind != "N":
                bal = Balances(self.mf, "STR", self.conum, self.curdt,
                    (grp, cod, loc, ("P", self.period[0])))
                bals = bal.doStrBals()
                if qty > bals[5][0]:
                    needpwd = True
        if needpwd:
            state = self.df.disableButtons()
            cf = PwdConfirm(self.mf, conum=self.conum, system="INV",
                code="ExQty")
            self.df.enableButtons(state=state)
            if cf.flag == "no":
                return "NoQty"

    def doExtractData(self, line):
        self.oldline = line[self.sql.slsiv2_col.index("si2_seq")]
        self.grp = line[self.sql.slsiv2_col.index("si2_group")]
        self.code = line[self.sql.slsiv2_col.index("si2_code")]
        self.desc = line[self.sql.slsiv2_col.index("si2_desc")]
        self.loc = line[self.sql.slsiv2_col.index("si2_loc")]
        self.qty = line[self.sql.slsiv2_col.index("si2_qty")]
        self.uoi = line[self.sql.slsiv2_col.index("si2_uoi")]
        self.unt = line[self.sql.slsiv2_col.index("si2_units")]
        self.vatcod = line[self.sql.slsiv2_col.index("si2_vat_code")]
        self.vatrat = line[self.sql.slsiv2_col.index("si2_vat_rate")]
        self.price = line[self.sql.slsiv2_col.index("si2_price")]
        self.disrat = line[self.sql.slsiv2_col.index("si2_disc_per")]
        # Calculate Values
        if not self.unt:
            pri = self.price
        else:
            pri = round((self.price / self.unt), 2)
        rat = float(ASD(100) + ASD(self.vatrat))
        self.inc = round((pri * rat / 100), 4)
        self.exc = round((pri * 1), 2)
        dis = float(ASD(100) - ASD(self.disrat))
        self.excamt = round((self.qty * self.exc * dis / 100), 2)
        self.incamt = round((self.qty * self.inc * dis / 100), 2)
        self.vatamt = float(ASD(self.incamt) - ASD(self.excamt))

    def doAgeNormal(self):
        self.doAgeing("N")

    def doAgeCurrent(self):
        self.doAgeing("C")

    def doAgeing(self, atype):
        if atype == "N":
            state = self.df.disableButtons()
        self.age = AgeAll(self.mf, system="drs", agetyp=atype,
            agekey=[self.conum, self.chain, self.acno, self.rtn, self.othno,
            self.curdt, self.agetot, 0.0])
        if self.age.cancel:
            self.age.agetot = self.agetot
        if atype == "N":
            self.df.enableButtons(state=state)

    def doRibbon(self, widget=None):
        tit = ("Ribbon Line",)
        fld = (
            (("T",0,0,0),"INa",9,"Delivery Note Number",
                "Delivery Note Number",self.dnum,"N",None,None,None,None),
            (("T",0,1,0),"INA",14,"Customer Order Number",
                "Customer Order Number",self.onum,"N",None,None,None,None),
            (("T",0,2,0),"INa",7,"Job Number","Job Number",
                self.jnum,"N",None,None,None,None),
            (("T",0,3,0),"INA",30,"Contact Person","Contact Person",
                self.cnam,"N",None,None,None,None),
            (("T",0,4,0),"INA",10,"VAT Number","VAT Number",
                self.vatn,"N",self.doVatNum,None,None,None))
        state = self.df.disableButtons()
        self.tx = TartanDialog(self.mf, title=tit, newscn=(0,1), tops=True,
            eflds=fld, tend=((self.doRibEnd,"n"),), txit=(self.doRibExit,),
            focus=False)
        if self.ribbon:
            for x in range(5):
                self.tx.loadEntry("T", 0, x, data=self.ribbon[x])
        pos = 1
        if self.typs in ("O", "Q"):
            if self.typs == "O" or self.acttyp == "D":
                self.tx.loadEntry("T", 0, 0, data="")
                self.tx.skip[0] = [1]
                pos = 2
            else:
                self.tx.loadEntry("T", 0, 1, data="")
                self.tx.loadEntry("T", 0, 2, data="")
                self.tx.skip[0] = [1, 2, 3]
                pos = 4
        self.tx.focusField("T", 0, pos, clr=False)
        self.mf.startLoop()
        self.df.enableButtons(state=state)
        if self.accept:
            self.doAccept()
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def doVatNum(self, frt, pag, r, c, p, i, w):
        if self.totinv >= 1000.00 and not w:
            return "Invalid VAT Number"

    def doRibEnd(self):
        self.tx.closeProcess()
        self.ribbon = []
        for x in range(0, self.tx.topq[0]):
            self.ribbon.append(self.tx.t_work[0][0][x])
        self.mf.closeLoop()

    def doRibExit(self):
        self.tx.closeProcess()
        self.accept = None
        self.mf.closeLoop()

    def doUpdateTables(self):
        # Update slsiv1 with ribbon, delivery and message details and
        # Update drsmst with VAT number if available
        whr = [("si1_cono", "=", self.conum), ("si1_rtn", "=", self.typs),
            ("si1_docno", "=", self.docno), ("si1_chain", "=", self.chain),
            ("si1_acno", "=", self.acno)]
        if self.ribbon:
            self.sql.updRec("slsiv1", cols=["si1_our_ord", "si1_cus_ord",
                "si1_jobno", "si1_contact", "si1_vatno"],
                data=[self.ribbon[0], self.ribbon[1], self.ribbon[2],
                self.ribbon[3], self.ribbon[4]], where=whr)
            if self.mode == "A" and self.typs in ("I", "O") and not self.vatn and self.ribbon[4]:
                self.sql.updRec("drsmst", cols=["drm_vatno"],
                    data=[self.ribbon[4]], where=[("drm_cono", "=",
                    self.conum), ("drm_chain", "=", self.chain),
                    ("drm_acno", "=", self.acno)])
        if self.drsdel:
            self.sql.updRec("slsiv1", cols=["si1_add1", "si1_add2",
                "si1_add3", "si1_add4"], data=[self.drsdel[1],
                self.drsdel[2], self.drsdel[3], self.drsdel[4]],
                where=whr)
        if self.ctlmes:
            fmt = textFormat(self.ctlmes[2], width=30)
            for m in range(len(fmt), 5):
                fmt.append("")
            self.sql.updRec("slsiv1", cols=["si1_mes1", "si1_mes2",
                "si1_mes3", "si1_mes4", "si1_mes5"], data=[fmt[0], fmt[1],
                fmt[2], fmt[3], fmt[4]], where=whr)
        if self.typs in ("O", "Q") and self.acttyp in ("A", "N"):
            return
        # Dispatch of order or quotation - change slsiv1, slsiv2 and slsiv3 type
        if self.typs in ("O", "Q") and self.acttyp == "D":
            actdoc = self.docno
            self.doGetDocno("I")
            data = self.sql.getRec(tables="slsiv1", where=[("si1_cono", "=",
                self.conum), ("si1_rtn", "=", self.typs), ("si1_docno", "=",
                actdoc)], limit=1)
            data[self.sql.slsiv1_col.index("si1_rtn")] = "I"
            data[self.sql.slsiv1_col.index("si1_docno")] = self.docno
            data[self.sql.slsiv1_col.index("si1_date")] = self.trdt
            ordno = CCD(actdoc, "Na", 9).work
            data[self.sql.slsiv1_col.index("si1_our_ord")] = ordno
            data[self.sql.slsiv1_col.index("si1_capdt")] = self.sysdtw
            # Write and Commit the new invoice header
            self.sql.insRec("slsiv1", data=data)
            self.mf.dbm.commitDbase()
            # Update order/quote header by inserting the invoice number
            self.sql.updRec("slsiv1", cols=["si1_invno"], data=[self.docno],
                where=[("si1_cono", "=", self.conum), ("si1_rtn", "=",
                self.typs), ("si1_docno", "=", actdoc)])
            # Move the order/quote body lines to the new invoice
            self.sql.updRec("slsiv2", cols=["si2_rtn", "si2_docno"],
                data=["I", self.docno], where=[("si2_cono", "=", self.conum),
                ("si2_rtn", "=", self.typs), ("si2_docno", "=", actdoc)])
            self.sql.updRec("slsiv3", cols=["si3_rtn", "si3_docno"],
                data=["I", self.docno], where=[("si3_cono", "=", self.conum),
                ("si3_rtn", "=", self.typs), ("si3_docno", "=", actdoc)])
            self.typs = "I"
        # Create Stores Transactions
        trn = self.sql.getRec(tables="slsiv2", where=[("si2_cono", "=",
            self.conum), ("si2_rtn", "=", self.typs), ("si2_docno", "=",
            self.docno)])
        if not trn:
            return
        for line in trn:
            self.doExtractData(line)
            gtype = self.sql.getRec(tables="strmf1", cols=["st1_type"],
                where=[("st1_cono", "=", self.conum), ("st1_group", "=",
                self.grp), ("st1_code", "=", self.code)], limit=1)
            if gtype[0] == "R":
                self.doRecipe(atype="U", lineno=self.oldline)
                icost = self.icost
                tcost = self.tcost
            else:
                ind, icost, tcost, cbal = self.doCalCost(self.grp, self.code,
                    self.loc, self.qty)
            if self.typs == "I":
                rtn = 7
                qty = float(ASD(0) - ASD(self.qty))
                cst = float(ASD(0) - ASD(tcost))
                sel = float(ASD(0) - ASD(self.excamt))
                vat = float(ASD(0) - ASD(self.vatamt))
            elif self.typs == "C":
                rtn = 7
                qty = self.qty
                cst = tcost
                sel = self.excamt
                vat = self.vatamt
            if self.ribbon:
                ref2 = self.ribbon[0]
            else:
                ref2 = ""
            # Write record
            data = [self.conum, self.grp, self.code, self.loc, self.trdt, rtn,
                self.othno, self.batch, ref2, qty, cst, sel, self.curdt,
                self.name, self.chain, self.acno, self.repno, "INV",
                self.disrat, 0, 0, "", self.sysdtw, 0]
            self.sql.insRec("strtrn", data=data)
            if gtype[0] == "R":
                items = self.sql.getRec("slsiv3", where=[("si3_cono", "=",
                    self.conum), ("si3_rtn", "=", self.typs), ("si3_docno",
                    "=", self.othno), ("si3_seq", "=", self.oldline)])
                for item in items:
                    quan = item[6] * qty
                    cost = item[7] * quan
                    data = [self.conum, item[4], item[5], self.loc, self.trdt,
                        6, self.othno, self.batch, ref2, quan, cost, 0,
                        self.curdt, self.name, self.chain, self.acno,
                        self.repno, "INV", 0, 0, 0, "", self.sysdtw, 0]
                    self.sql.insRec("strtrn", data=data)
                quan = float(ASD(0) - ASD(qty))
                cost = float(ASD(0) - ASD(cst))
                data = [self.conum, self.grp, self.code, self.loc, self.trdt,
                    5, self.othno, self.batch, ref2, quan, cost, 0,
                    self.curdt, self.name, self.chain, self.acno, 0, "INV",
                    0, 0, 0, "", self.sysdtw, 0]
                self.sql.insRec("strtrn", data=data)
            # If Integrated Create GL Transaction (Sales & Cost of Sales)
            if self.stgl == "Y":
                col = ["glt_tramt", "glt_taxamt", "glt_seq"]
                # Stock on Hand Control
                if cst:
                    rec = self.sql.getRec(tables="gentrn", cols=col,
                        where=[("glt_cono", "=", self.conum), ("glt_acno", "=",
                        self.soh), ("glt_curdt", "=", self.curdt), ("glt_trdt",
                        "=", self.trdt), ("glt_type", "=", 1), ("glt_refno",
                        "=", self.othno), ("glt_batch", "=", self.batch),
                        ("glt_desc", "=", self.name)], limit=1)
                    if rec:
                        at1 = float(ASD(rec[0]) + ASD(cst))
                        self.sql.updRec("gentrn", cols=["glt_tramt"],
                            data=[at1], where=[("glt_seq", "=", rec[2])])
                    else:
                        data = (self.conum, self.soh, self.curdt, self.trdt,
                            1, self.othno, self.batch, cst, 0, self.name,
                            "", "", 0, self.sysdtw, 0)
                        self.sql.insRec("gentrn", data=data)
                # Sales and COS Accounts
                acc = self.sql.getRec(tables="strmf1", cols=["st1_sls",
                    "st1_cos"], where=[("st1_cono", "=", self.conum),
                    ("st1_group", "=", self.grp), ("st1_code", "=",
                    self.code)], limit=1)
                # Sales Account
                if sel or vat:
                    rec = self.sql.getRec(tables="gentrn", cols=col,
                        where=[("glt_cono", "=", self.conum), ("glt_acno", "=",
                        acc[0]), ("glt_curdt", "=", self.curdt), ("glt_trdt",
                        "=", self.trdt), ("glt_type", "=", 1), ("glt_refno",
                        "=", self.othno), ("glt_batch", "=", self.batch),
                        ("glt_desc", "=", self.name)], limit=1)
                    if rec:
                        at1 = float(ASD(rec[0]) + ASD(sel))
                        at2 = float(ASD(rec[1]) + ASD(vat))
                        self.sql.updRec("gentrn", cols=["glt_tramt",
                            "glt_taxamt"], data=[at1, at2], where=[("glt_seq",
                            "=", rec[2])])
                    else:
                        data = (self.conum, acc[0], self.curdt, self.trdt, 1,
                            self.othno, self.batch, sel, vat, self.name,
                            self.vatcod, "", 0, self.sysdtw, 0)
                        self.sql.insRec("gentrn", data=data)
                # Cost of Sales Account
                if cst:
                    cst = float(ASD(0) - ASD(cst))
                    rec = self.sql.getRec(tables="gentrn", cols=col,
                        where=[("glt_cono", "=", self.conum), ("glt_acno", "=",
                        acc[1]), ("glt_curdt", "=", self.curdt), ("glt_trdt",
                        "=", self.trdt), ("glt_type", "=", 1), ("glt_refno",
                        "=", self.othno), ("glt_batch", "=", self.batch),
                        ("glt_desc", "=", self.name)], limit=1)
                    if rec:
                        at1 = float(ASD(rec[0]) + ASD(cst))
                        self.sql.updRec("gentrn", cols=["glt_tramt"],
                            data=[at1], where=[("glt_seq", "=", rec[2])])
                    else:
                        data = (self.conum, acc[1], self.curdt, self.trdt, 1,
                            self.othno, self.batch, cst, 0, self.name, "",
                            "", 0, self.sysdtw, 0)
                        self.sql.insRec("gentrn", data=data)
            # Create VAT Transaction (ctlvtf)
            if self.typs in ("I", "C"):
                if self.typs == "I":
                    rtn = 1
                elif self.typs == "C":
                    rtn = 4
                whr = [("vtt_cono", "=", self.conum), ("vtt_code", "=",
                    self.vatcod), ("vtt_vtyp", "=", "O"), ("vtt_curdt", "=",
                    self.curdt), ("vtt_styp", "=", "D"), ("vtt_ttyp", "=",
                    rtn), ("vtt_batch", "=", self.batch), ("vtt_refno", "=",
                    self.othno), ("vtt_refdt", "=", self.trdt), ("vtt_acno",
                    "=", self.acno), ("vtt_desc", "=", self.name)]
                rec = self.sql.getRec(tables="ctlvtf", cols=["vtt_exc",
                    "vtt_tax"], where=whr, limit=1)
                if rec:
                    at1 = float(ASD(rec[0]) + ASD(sel))
                    at2 = float(ASD(rec[1]) + ASD(vat))
                    self.sql.updRec("ctlvtf", cols=["vtt_exc", "vtt_tax"],
                        data=[at1, at2], where=whr)
                else:
                    self.sql.insRec("ctlvtf", data=[self.conum, self.vatcod,
                        "O", self.curdt, "D", rtn, self.batch, self.othno,
                        self.trdt, self.acno, self.name, sel, vat, self.sysdtw,
                        0, 0])
        # Create Debtors Transaction
        if self.typs == "I":
            self.rtn = 1
            self.agetot = self.totinv
            vat = self.totvat
            self.doAgeCurrent()
        elif self.typs == "C":
            self.rtn = 4
            self.agetot = float(ASD(0) - ASD(self.totinv))
            vat = float(ASD(0) - ASD(self.totvat))
            self.doAgeNormal()
        if self.ribbon:
            ref2 = self.ribbon[0]
        else:
            ref2 = ""
        data = [self.conum, self.chain, self.acno, self.rtn, self.othno,
            self.batch, self.trdt, ref2, self.agetot, vat, self.curdt,
            self.name, self.vatcod, "", self.sysdtw, 0]
        self.sql.insRec("drstrn", data=data)
        # If Integrated Create GL Transaction (Drs Control & VAT)
        if self.drgl == "Y":
            # Debtors Control
            if self.totinv:
                if self.typs == "C":
                    amt = float(ASD(0) - ASD(self.totinv))
                else:
                    amt = self.totinv
                data = (self.conum, self.dctl, self.curdt, self.trdt, 1,
                    self.othno, self.batch, amt, 0, self.name, "", "", 0,
                    self.sysdtw, 0)
                self.sql.insRec("gentrn", data=data)
            # VAT Control
            if self.totvat:
                if self.typs == "C":
                    vat = self.totvat
                else:
                    vat = float(ASD(0) - ASD(self.totvat))
                data = (self.conum, self.vctl, self.curdt, self.trdt, 1,
                    self.othno, self.batch, vat, 0, self.name, "", "", 0,
                    self.sysdtw, 0)
                self.sql.insRec("gentrn", data=data)

    def doGetDocno(self, atype):
        # Get Next Document Number
        docno = self.sql.getRec(tables="slsiv1", cols=["max(si1_docno)"],
            where=[("si1_cono", "=", self.conum), ("si1_rtn", "=", atype)],
            limit=1)[0]
        if not docno:
            docno = 0
        if atype not in ("C", "I"):
            docno += 1
        else:
            check = True
            while check:
                docno += 1
                next = CCD(docno, "Na", 9).work
                check = self.sql.getRec(tables="drstrn", where=[("drt_cono",
                    "=", self.conum), ("drt_chain", "=", self.chain),
                    ("drt_acno", "=", self.acno), ("drt_ref1", "=", next)])
        self.docno = CCD(docno, "UI", 9).work
        self.othno = CCD(docno, "Na", 9).work

    def doCalCost(self, grp, code, loc, qty):
        # Calculate cost price
        ind = self.sql.getRec(tables="strmf1", cols=["st1_value_ind"],
            where=[("st1_cono", "=", self.conum), ("st1_group", "=", grp),
            ("st1_code", "=", code)], limit=1)
        bal = Balances(self.mf, "STR", self.conum, self.curdt,
            (grp, code, loc, ("P", self.period[0])))
        bals = bal.doStrBals()
        if ind[0] == "N":
            icost = 0.00
            tcost = 0.00
        elif ind[0] == "S":
            icost = bals[7]
            tcost = round((icost * qty), 2)
        else:
            if qty == bals[5][0]:
                tcost = bals[5][1]
                icost = round((tcost / qty), 2)
            else:
                icost = bals[6]
                tcost = round((icost * qty), 2)
            if not icost:
                icost = bals[7]
                tcost = round((icost * qty), 2)
        return (ind[0], icost, tcost, bals[5][0])

    def doCalSell(self):
        prc = self.sql.getRec(tables="strprc", cols=["stp_price"],
            where=[("stp_cono", "=", self.conum), ("stp_group", "=",
            self.grp), ("stp_code", "=", self.code), ("stp_loc", "=",
            self.loc), ("stp_level", "=", self.level)], limit=1)
        if prc and prc[0]:
            self.rrp = prc[0]
            return
        if self.gtype == "R":
            items = self.sql.getRec("slsiv3", cols=["si3_rgroup",
                "si3_rcode", "si3_rqty"], where=[("si3_cono", "=",
                self.conum), ("si3_rtn", "=", self.typs), ("si3_docno",
                "=", self.docno), ("si3_seq", "=", self.lineno)])
        else:
            items = [[self.grp, self.code, 1]]
        cst = 0
        self.rrp = 0
        for grp, cod, qty in items:
            bal = Balances(self.mf, "STR", self.conum, self.curdt,
                (grp, cod, self.loc, ("P", self.period[0])))
            bals = bal.doStrBals()
            if bals[7]:
                cst = float(ASD(qty * cst) + ASD(bals[7]))
            else:
                cst = float(ASD(qty * cst) + ASD(bals[6]))
            if self.gtype == "R":
                prc = self.sql.getRec(tables="strprc", cols=["stp_price"],
                    where=[("stp_cono", "=", self.conum), ("stp_group", "=",
                    grp), ("stp_code", "=", cod), ("stp_loc", "=", self.loc),
                    ("stp_level", "=", self.level)], limit=1)
            else:
                prc = None
            if prc and prc[0]:
                prc = round((qty * prc[0]), 2)
                self.rrp = float(ASD(self.rrp) + ASD(prc))
        if not self.rrp and self.automu == "Y":
            gpm = self.sql.getRec(tables="strgrp", cols=["gpm_markup"],
                where=[("gpm_cono", "=", self.conum), ("gpm_group", "=",
                self.grp)], limit=1)
            mkp = gpm[0]
            st2 = self.sql.getRec(tables="strmf2", cols=["st2_markup"],
                where=[("st2_cono", "=", self.conum), ("st2_group", "=",
                self.grp), ("st2_code", "=", self.code), ("st2_loc", "=",
                self.loc)], limit=1)
            if st2[0]:
                mkp = st2[0]
            if mkp:
                self.rrp = round((cst * (100 + mkp) / 100.0), 2)

    def doCancel(self, widget=None):
        self.mf.dbm.rollbackDbase()
        if self.acttyp not in ("A", "D"):
            self.sql.updRec("slsiv1", cols=["si1_invno"], data=["cancel"],
                where=[("si1_cono", "=", self.conum), ("si1_rtn", "=",
                self.typs), ("si1_docno", "=", self.docno)])
            self.mf.dbm.commitDbase()
        self.df.setViewPort(self.typs, 0)
        self.df.focusField("T", 0, 1)

