#! /usr/bin/env python

"""
SYNOPSIS
    Sales Orders Outstanding Report.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, CCD, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, doWriteExport, getModName, doPrinter
from tartanFunctions import showError

class si3010:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["drsmst", "slsiv1", "slsiv2"],
            prog=__name__)
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdtd = "%i/%02i/%02i" % (t[0], t[1], t[2])
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.t = None
        self.slstot = 0

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Sales Orders Outstanding (%s)" % __name__)
        si1 = {
            "stype": "R",
            "tables": ("slsiv1",),
            "cols": (
                ("si1_docno", "", 0, "Order-Num"),
                ("si1_chain", "", 0, "Chn"),
                ("si1_acno", "", 0, "Drs-Acc"),
                ("si1_date", "", 0, "Order-Date")),
            "where": [
                ("si1_cono", "=", self.conum),
                ("si1_rtn", "=", "O"),
                ("si1_invno", "=", "")]}
        r1s = (("List", "L"), ("Detail", "D"))
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"Report Type","Report Type",
                "L","Y",self.doType,None,None,None),
            (("T",0,1,0),"IUI",9,"From Order Number","From Order Number",
                "","Y",self.doOrd1,si1,None,None),
            (("T",0,2,0),"IUI",9,"To   Order Number","To Order Number",
                "","Y",self.doOrd2,si1,None,None))
        tnd = ((self.doEnd,"y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("Y","V"), mail=("Y","N"))

    def doType(self, frt, pag, r, c, p, i, w):
        self.rtype = w
        if self.rtype == "L":
            self.ord1 = 0
            self.ord2 = 999999999
            self.df.topEntry[0][3][2][0].show()
            self.df.topEntry[0][3][3][0].show()
            return "sk2"
        self.df.topEntry[0][3][2][0].hide()
        self.df.topEntry[0][3][3][0].hide()

    def doOrd1(self, frt, pag, r, c, p, i, w):
        if w:
            acc = self.sql.getRec(tables="slsiv1", where=[("si1_cono", "=",
                self.conum), ("si1_rtn", "=", "O"), ("si1_docno", "=", w),
                ("si1_invno", "=", "")], limit=1)
            if not acc:
                return "This is Not an Outstanding Order"
        self.ord1 = w

    def doOrd2(self, frt, pag, r, c, p, i, w):
        if w:
            if w < self.ord1:
                return "Invalid Last Order < First Order"
            acc = self.sql.getRec(tables="slsiv1", where=[("si1_cono", "=",
                self.conum), ("si1_rtn", "=", "O"), ("si1_docno", "=", w),
                ("si1_invno", "=", "")], limit=1)
            if not acc:
                return "This is Not an Outstanding Order"
            self.ord2 = w
        else:
            self.ord2 = 999999999

    def doEnd(self):
        self.df.closeProcess()
        whr = (
            ("si1_cono", "=", self.conum),
            ("si1_rtn", "=", "O"),
            ("si1_docno", "between", self.ord1, self.ord2),
            ("si1_invno", "=", ""))
        all = self.sql.getRec(tables="slsiv1", where=whr)
        if not all:
            showError(self.mf.body, "Selection Error",
                "No Orders Selected")
        elif self.df.repprt[0][2] == "export":
            self.exportReport(all)
        else:
            self.printReport(all)
            self.c.save()
            doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                head=self.tit, view=self.df.repprt[0][1],
                prtnam=self.df.repprt[0][2], mail=self.df.repprt[1][2])
        self.closeProcess()

    def exportReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        expnam = getModName(self.mf.wrkdir, __name__, self.conum)
        self.expheads = ["%03u %-30s %s" % (self.conum,self.conam,self.sysdttm)]
        self.expheads.append("Outstanding Sales Orders Report as at %s" % \
            self.sysdtd)
        self.expcolsh = [["Order-Num", "Order-Date", "Chn", "Acc-Num", "Name",
            "Rep", "Rep-Name", "Job-Num", "Excl-Value"]]
        self.expforms = [("Na", 9), ("D1", 10), ("UI", 3), ("NA", 7),
            ("NA", 30), ("Na", 3), ("NA", 30), ("Na", 7), ("SD", 13.2)]
        self.expdatas = []
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                p.closeProgress()
                return
            if not self.getValues(dat):
                continue
            self.getOrdTot()
            self.expdatas.append(["BODY", [self.ord.work, self.dte.work,
                self.chn.work, self.drs.work, self.drsnam, self.rep.work,
                self.repnam, self.job.work, self.ordtot]])
        self.expdatas.append(["BLANK"])
        self.expdatas.append(["TOTAL", [""] * 8 + [self.slstot]])
        p.closeProgress()
        doWriteExport(xtype=self.df.repprt[0][1], name=expnam, sheet=__name__,
            heads=self.expheads, colsh=self.expcolsh, forms=self.expforms,
            datas=self.expdatas, rcdic=self.mf.rcdic)

    def printReport(self, all):
        self.p = ProgressBar(self.mf.body, max=len(all), esc=True)
        if self.rtype == "L":
            self.head = ("%03u %-30s %73s %10s" % (self.conum, self.conam,
                self.sysdttm, __name__))
        else:
            self.head = ("%03u %-30s %44s %10s" % (self.conum, self.conam,
                self.sysdttm, __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        self.pgnum = 0
        self.pglin = 99
        for num, dat in enumerate(all):
            can = self.p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            if not self.getValues(dat):
                continue
            if self.rtype == "D":
                self.printHeader()
            self.printBody()
        if self.rtype == "D":
            self.c.drawText(self.t)
            self.c.showPage()
        elif self.rtype == "L":
            tot = CCD(self.slstot, "SD", 13.2)
            self.t.textLine()
            self.t.textLine("%106s %s" % ("", tot.disp))
            self.c.drawText(self.t)
            self.c.showPage()
        self.p.closeProgress()

    def getValues(self, data):
        col = self.sql.slsiv1_col
        self.ord = CCD(data[col.index("si1_docno")], "Na", 9)
        self.si2 = self.sql.getRec(tables="slsiv2", where=[("si2_cono",
            "=", self.conum), ("si2_rtn", "=", "O"), ("si2_docno", "=",
            self.ord.work)], order="si2_seq")
        if not self.si2:
            return
        self.mod = CCD(data[col.index("si1_mode")], "UA", 1)
        self.chn = CCD(data[col.index("si1_chain")], "UI", 3)
        self.drs = CCD(data[col.index("si1_acno")], "NA", 7)
        acc = self.sql.getRec(tables="drsmst", cols=["drm_name"],
            where=[("drm_cono", "=", self.conum), ("drm_chain", "=",
            self.chn.work), ("drm_acno", "=", self.drs.work)], limit=1)
        if not acc:
            self.drsnam = ""
        else:
            self.drsnam = acc[0]
        self.dte = CCD(data[col.index("si1_date")], "D1", 10)
        self.job = CCD(data[col.index("si1_jobno")], "Na", 7)
        self.rep = CCD(data[col.index("si1_rep")], "Na", 3)
        acc = self.sql.getRec(tables="ctlrep", cols=["rpm_name"],
            where=[("rpm_cono", "=", self.conum), ("rpm_rep", "=",
            self.rep.work)], limit=1)
        if not acc:
            self.repnam = ""
        else:
            self.repnam = acc[0]
        return True

    def printHeader(self):
        self.pageHeading()
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.t.textLine()
        if self.rtype == "L":
            self.t.textLine("%-9s %-10s %-3s %-7s %-30s %-3s %-30s %-7s %12s" \
                % ("Order-Num", "Order-Date", "Chn", "Acc-Num", "Name", "Rep",
                "Rep-Name", "Job-Num", "Excl-Value"))
            self.pglin = self.pglin + 3
        else:
            text = "Order Number: %-9s Date: %-10s Salesman: %-3s %-30s" % \
                (self.ord.disp, self.dte.disp, self.rep.disp, self.repnam)
            self.t.textLine(text)
            self.t.textLine()
            text = "Account Details: %-3s %-7s %-30s" % (self.chn.disp,
                self.drs.disp, self.drsnam)
            self.t.textLine(text)
            self.t.textLine()
            self.t.textLine("%-3s %-20s %-30s %-1s %10s  %-1s %11s %-6s" % \
                ("Grp", "Product-Code", "Description", "L", "Qty-Ord", "V",
                "U-Price", "Disc-%"))
            self.pglin = self.pglin + 7
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)

    def pageHeading(self):
        if self.pgnum:
            self.c.drawText(self.t)
            self.c.showPage()
        self.pgnum = self.pgnum + 1
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.t.textLine(self.head)
        self.t.textLine()
        if self.rtype == "L":
            self.t.textLine("%-37s %-10s %64s %5s" % \
                ("Outstanding Sales Orders Report as at", self.sysdtd,
                "Page", self.pgnum))
        else:
            self.t.textLine("%-37s %-10s %35s %5s" % \
                ("Outstanding Sales Orders Report as at", self.sysdtd,
                "Page", self.pgnum))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 3

    def printBody(self):
        if self.rtype == "L":
            if self.pglin > self.lpp:
                self.printHeader()
            self.getOrdTot()
            val = CCD(self.ordtot, "SD", 13.2)
            text = "%s %s %s %s %-30s %s %-30s %s %s" % (self.ord.disp,
                self.dte.disp, self.chn.disp, self.drs.disp, self.drsnam,
                self.rep.disp, self.repnam, self.job.disp, val.disp)
            self.t.textLine(text)
            self.pglin = self.pglin + 1
        else:
            col = self.sql.slsiv2_col
            for dat in self.si2:
                grp = CCD(dat[col.index("si2_group")], "UA", 3)
                cod = CCD(dat[col.index("si2_code")], "NA", 20)
                des = CCD(dat[col.index("si2_desc")], "NA", 30)
                loc = CCD(dat[col.index("si2_loc")], "UA", 1)
                qty = CCD(dat[col.index("si2_qty")], "SD", 11.2)
                vat = CCD(dat[col.index("si2_vat_code")], "UA", 1)
                prc = CCD(dat[col.index("si2_price")], "UD", 11.2)
                dis = CCD(dat[col.index("si2_disc_per")], "UD", 6.2)
                if self.pglin > self.lpp:
                    self.printHeader()
                self.t.textLine("%s %s %s %s %s %s %s %s" % (grp.disp, cod.disp,
                    des.disp, loc.disp, qty.disp, vat.disp, prc.disp, dis.disp))
                self.pglin = self.pglin + 1

    def getOrdTot(self):
        self.ordtot = 0
        col = self.sql.slsiv2_col
        for dat in self.si2:
            unt = CCD(dat[col.index("si2_units")], "UI", 7.0)
            qty = CCD(dat[col.index("si2_qty")], "SD", 13.2)
            pri = CCD(dat[col.index("si2_price")], "UD", 12.2)
            dis = CCD(dat[col.index("si2_disc_per")], "UD", 6.2)
            if unt.work == 0:
                price = pri.work
            else:
                price = round((pri.work / unt.work), 2)
            excpri = round((price * 1), 2)                  #S/Price
            net = float(ASD(100.0) - ASD(dis.work))
            excamt = round((qty.work * excpri * net / 100.0), 2)
            val = CCD(excamt, "SD", 13.2)
            self.ordtot = float(ASD(self.ordtot) + ASD(val.work))
            self.slstot = float(ASD(self.slstot) + ASD(val.work))

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

