#! /usr/bin/env python

"""
SYNOPSIS
    Salesmen Sales History Report.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, Balances, CCD, CreateChart, ProgressBar, Sql
from TartanClasses import TartanDialog
from tartanFunctions import doReportlab, getModName, doPrinter, showError
from tartanWork import mthnam

class si3070:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, "ctlrep", prog=__name__)
        if self.sql.error:
            return self.sql.error
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.t = None
        self.mchart = []

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Salesmans Sales History (%s)" % __name__)
        rep = {
            "stype": "R",
            "tables": ("ctlrep",),
            "cols": (
                ("rpm_rep", "", 0, "Rep"),
                ("rpm_name", "", 0, "Name", "Y")),
            "where": [("rpm_cono", "=", self.conum)]}
        fld = (
            (("T",0,0,0),"ID2",7,"Ending Period","Ending Period",
                (self.sysdtw/100),"Y",self.doDat,None,None,("efld",)),
            (("T",0,1,0),"INa",3,"First Rep Code",
                "First Rep Code","","Y",self.doRep1,rep,None,None),
            (("T",0,2,0),"INa",3,"Last Rep Code",
                "Last Rep Code","","Y",self.doRep2,rep,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doDat(self, frt, pag, r, c, p, i, w):
        self.end = w
        yr = w / 100 - 1
        mth = (w % 100) + 1
        if mth > 12:
            yr = yr + 1
            mth = mth - 12
        self.start = (yr * 100) + mth

    def doRep1(self, frt, pag, r, c, p, i, w):
        self.rep1 = w

    def doRep2(self, frt, pag, r, c, p, i, w):
        self.rep2 = w

    def doEnd(self):
        self.df.closeProcess()
        whr = [("rpm_cono", "=", self.conum)]
        if self.rep1:
            whr.append(("rpm_rep", ">=", self.rep1))
        if self.rep2:
            whr.append(("rpm_rep", "<=", self.rep2))
        all = self.sql.getRec(tables="ctlrep", cols=["rpm_rep", "rpm_name"],
            where=whr, order="rpm_rep")
        if not all:
            showError(self.mf.body, "Processing Error",
            "No Records Selected")
        else:
            self.mthhead = ""
            space = " " * 11
            y = self.end % 100
            for x in range(0, 12):
                y = y + 1
                if y == 13:
                    y = 1
                self.mthhead = self.mthhead + \
                    space[:(11 - len(mthnam[y][1]))] + mthnam[y][1]
            self.printReport(all)
        self.closeProcess()

    def printReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        self.head = ("%03u %-30s %48s %33s %48s %6s" % \
            (self.conum, self.conam, "", self.sysdttm, "", __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        self.stots = [0] * 13
        self.ctots = [0] * 13
        self.pgnum = 0
        self.pglin = 99
        for x in range(0, len(all)):
            can = p.displayProgress(x)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            rep = CCD(all[x][0], "Na", 3)
            name = CCD(all[x][1], "NA", 30)
            bals = Balances(self.mf, "STR", self.conum, self.end, (rep.work, ))
            this, hist = bals.doStrHist(self.start)
            if this == 0:
                continue
            costsd = ""
            salesd = ""
            proftd = ""
            prperd = ""
            prt = "no"
            mchart = ["T", name.work]
            for x in range(11,-1,-1):
                c = float(ASD(0) - ASD(hist[2][x][1]))
                c = CCD(round(c, 0), "SL", 11)
                s = float(ASD(0) - ASD(hist[2][x][2]))
                s = CCD(round(s, 0), "SL", 11)
                if c.work or s.work:
                    prt = "yes"
                prf = float(ASD(s.work) - ASD(c.work))
                prf = CCD(round(prf, 0), "SL", 11)
                if s.work == 0:
                    pcn = 0
                else:
                    pcn = round((prf.work * 100.0 / s.work), 2)
                pcn = CCD(pcn, "SD", 11.2)
                self.ctots[x] = self.ctots[x] + c.work
                self.stots[x] = self.stots[x] + s.work
                costsd = costsd + c.disp
                salesd = salesd + s.disp
                proftd = proftd + prf.disp
                prperd = prperd + pcn.disp
                mchart.append(s.work)
            if prt == "no":
                continue
            self.mchart.append(mchart)
            if self.pglin > (self.lpp - 4):
                if self.pgnum != 0:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
            self.t.textLine("%s %s %s %s" % (rep.disp, name.disp,
                "Sales ", salesd))
            self.t.textLine("%-34s %s %s" % ("", "Costs ", costsd))
            self.t.textLine("%-34s %s %s" % ("", "Profit", proftd))
            self.t.textLine("%-34s %s %s" % ("", "Prf-% ", prperd))
            self.t.textLine("%s" % (self.suc * len(self.head)))
            self.pglin = self.pglin + 4
        p.closeProgress()
        if self.t:
            if can != "yes":
                self.grandTotal()
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
            head=self.tit, view=self.df.repprt[0][1],
            prtnam=self.df.repprt[0][2], mail=self.df.repprt[1][2])
        CreateChart(self.mf, self.conum, self.conam, [self.start,
            self.end], [self.tit, "Values"], None, self.mchart)
            
    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        per = CCD(self.end, "D2", 7)
        self.t.textLine("%-29s %-7s %129s %5s" % \
            ("Salesmans Sales History up to", per.disp, "Page", self.pgnum))
        self.t.textLine()
        if self.rep1:
            opt = "(Options: From Rep %s" % self.rep1
        else:
            opt = "(Options: From First Rep"
        if self.rep2:
            opt = "%s To Rep %s)" % (opt, self.rep2)
        else:
            opt = "%s To Last Rep)" % opt
        self.t.textLine(opt)
        self.t.textLine()
        self.t.textLine("%-3s %-37s%-13s" % ("Rep", "Name", self.mthhead))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 8

    def grandTotal(self):
        costsd = ""
        salesd = ""
        proftd = ""
        prperd = ""
        mchart = ["T", "Grand Totals"]
        for x in range(11,-1,-1):
            c = CCD(self.ctots[x], "SL", 11)
            s = CCD(self.stots[x], "SL", 11)
            prf = float(ASD(s.work) - ASD(c.work))
            prf = CCD(prf, "SL", 11)
            if s.work == 0:
                pcn = 0
            else:
                pcn = round((prf.work * 100.0 / s.work), 2)
            pcn = CCD(pcn, "SD", 11.2)
            self.ctots[x] = self.ctots[x] + c.work
            self.stots[x] = self.stots[x] + s.work
            costsd = costsd + c.disp
            salesd = salesd + s.disp
            proftd = proftd + prf.disp
            prperd = prperd + pcn.disp
            mchart.append(s.work)
        self.mchart.append(mchart)
        self.t.textLine("%-3s %-6s %30s %s" % ("", "Totals", "Sales ", salesd))
        self.t.textLine("%-34s %s %s" % ("", "Costs ", costsd))
        self.t.textLine("%-34s %s %s" % ("", "Profit", proftd))
        self.t.textLine("%-34s %s %s" % ("", "Prf-% ", prperd))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.textLine()

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

