#! /usr/bin/python

"""
SYNOPSIS
    Salaries Staff Loans Raise Interest.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import CCD, GetCtl, LoanInterest, ProgressBar, Sql
from TartanClasses import TartanDialog

class sl2030:
    def __init__(self, mf, coy, nam, period):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.period = period
        no = self.setVariables()
        if not no:
            self.dataHeader()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, "waglmf", prog=__name__)
        gc = GetCtl(self.mf)
        wagctl = gc.getCtl("wagctl", self.conum)
        if not wagctl:
            return "no"
        glint = wagctl["ctw_glint"]
        if glint == "Y":
            ctlctl = gc.getCtl("ctlctl", self.conum)
            if not ctlctl:
                return "no"
            ctls = ["wag_slc", "wag_sli"]
            if gc.chkRec(self.conum, ctlctl, ctls):
                return "no"
            self.glctls = (ctlctl["wag_slc"], ctlctl["wag_sli"])
        else:
            self.glctls = None
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]

    def dataHeader(self):
        tit = ("%03i %s" % (self.conum, self.conam),
                "Staff Loans Raise Interest (%s)" % __name__)
        fld = (
            (("T",0,0,0),"ID1",10,"Transaction Date","Transaction Date",
                self.sysdtw,"N",self.doTrdate,None,None,None),)
        tnd = ((self.endPage0, "y"),)
        txt = (self.exitPage0,)
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tend=tnd,
            txit=txt)

    def doTrdate(self, frt, pag, r, c, p, i, w):
        self.trdate = w
        if w < self.period[1][0] or w > self.period[2][0]:
            return "Invalid Date, Not in Financial Period"

    def endPage0(self):
        self.df.closeProcess()
        all = self.sql.getRec(tables="waglmf", cols=["wlm_empno", "wlm_loan",
            "wlm_int_per", "wlm_start_date"], where=[("wlm_cono", "=",
            self.conum), ("wlm_int_per", ">", 0), ("wlm_start_date", "<",
            self.trdate)], order="wlm_empno, wlm_loan")
        p = ProgressBar(self.mf.body, max=len(all))
        for num, dat in enumerate(all):
            p.displayProgress(num)
            rate = CCD(dat[2], "UD", 6.0)
            start = CCD(dat[3], "d1", 10.0)
            curdt = int(self.trdate / 100)
            batch = "%03i%04i" % (self.conum, curdt % 10000)
            lint = LoanInterest(self.mf, self.period[1][0], self.conum,
                dat[0], dat[1], rate.work, start.work, self.trdate,
                batch, curdt, update="Y", glctls=self.glctls)
        p.closeProgress()
        self.mf.dbm.commitDbase()
        self.mf.closeLoop()

    def exitPage0(self):
        self.df.closeProcess()
        self.mf.closeLoop()

