#! /usr/bin/env python

"""
SYNOPSIS
    Staff Loans Transaction Audit Trail.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, CCD, GetCtl, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, doPrinter, getModName, showError
from tartanWork import sltrtp

class sl3020:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        gc = GetCtl(self.mf)
        wagctl = gc.getCtl("wagctl", self.conum)
        if not wagctl:
            return "no"
        lir = wagctl["ctw_i_date"]
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        if lir:
            fy = lir / 10000
            fm = (lir / 100) - (fy * 100) + 1
            if fm == 13:
                fy = fy + 1; fm = 1
            self.sdate = (fy * 100) + fm
        else:
            self.sdate = 0
        self.edate = self.sysdtw / 100
        self.totind = "N"
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Staff Loans Audit Trail (%s)" % __name__)
        data = ["All Types"]
        for x in range(1, len(sltrtp)+1):
            data.append(sltrtp[x-1][1])
        btt = {
            "stype": "C",
            "titl": "Select the Required Type",
            "data": data,
            "retn": "I"}
        btm = {
            "stype": "R",
            "tables": ("wagltf",),
            "cols": (
                ("wlt_batch", "", 0, "Bat-Num"),
                ("wlt_type", ("xx", sltrtp), 20, "Type"),
                ("wlt_curdt", "", 0, "Cur-Dat")),
            "where": [],
            "group": "wlt_batch, wlt_type, wlt_curdt",
            "order": "wlt_type, wlt_curdt, wlt_batch"}
        r1s = (("Yes","Y"),("No","N"))
        fld = (
            (("T",0,0,0),"Id2",7,"Starting Period","",
                self.sdate,"Y",self.doStartPer,None,None,("efld",)),
            (("T",0,1,0),"ID2",7,"Ending Period","",
                self.edate,"Y",self.doEndPer,None,None,("efld",)),
            (("T",0,2,0),"IUI",1,"Type","Transaction Type",
                "","Y",self.doBatTyp,btt,None,None),
            (("T",0,3,0),"INa",7,"Batch Number","",
                "","Y",self.doBatNum,btm,None,None),
            (("T",0,4,0),("IRB",r1s),0,"Totals Only","",
                "Y","Y",self.doTots,None,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doStartPer(self, frt, pag, r, c, p, i, w):
        if w > self.edate:
            return "Invalid Period"
        self.sdatw = w
        self.sdatd = CCD(self.sdatw, "d2", 7).disp

    def doEndPer(self, frt, pag, r, c, p, i, w):
        if not w:
            self.edatw = self.edate
        elif w < self.sdatw or w > self.edate:
            return "Invalid Period"
        else:
            self.edatw = w
        self.edatd = CCD(self.edatw, "d2", 7).disp

    def doBatTyp(self, frt, pag, r, c, p, i, w):
        if w > len(sltrtp):
            return "Invalid Batch Type"
        self.btype = w
        if self.btype:
            self.df.topf[pag][i+1][8]["whera"] = [["T","btm_rtyp",0]]
        else:
            self.df.topf[pag][i+1][8]["whera"] = []

    def doBatNum(self, frt, pag, r, c, p, i, w):
        self.batch = w

    def doTots(self, frt, pag, r, c, p, i, w):
        self.totsonly = w

    def doEnd(self):
        self.df.closeProcess()
        if not self.btype:
            self.styp = 0
            self.etyp = 99
        else:
            styp = etyp = self.btype
        if not self.batch:
            self.sbat = ""
            self.ebat = "zzzzzzz"
        else:
            self.sbat = self.ebat = self.batch
        self.sql = Sql(self.mf.dbm, ["wagltf"], prog=__name__)
        whr = [("wlt_cono", "=", self.conum), ("wlt_curdt", "between",
            self.sdatw, self.edatw), ("wlt_batch", "between", self.sbat,
            self.ebat), ("wlt_type", "between", self.styp, self.etyp)]
        ord = "wlt_type, wlt_batch, wlt_ref"
        all = self.sql.getRec(tables="wagltf", where=whr, order=ord)
        if not all:
            showError(self.mf.body, "Transaction Error",
            "No Transactions Selected")
        else:
            self.printReport(all)
        self.closeProcess()

    def printReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        if self.totsonly == "Y":
            self.head = ("%03u %-30s %38s %6s" % \
            (self.conum, self.conam, self.sysdttm, __name__))
        else:
            self.head = ("%03u %-30s %12s %33s %12s %6s" % \
            (self.conum, self.conam, "", self.sysdttm, "", __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        self.bqty = 0
        self.bamt = 0
        self.bded = 0
        self.tqty = 0
        self.tamt = 0
        self.tded = 0
        self.gqt = [0,0,0,0,0,0,0]
        self.gam = [0,0,0,0,0,0,0]
        self.gdd = [0,0,0,0,0,0,0]
        self.trtp = 0
        self.pgnum = 0
        self.pglin = 99
        col = self.sql.wagltf_col
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            empno = CCD(dat[col.index("wlt_empno")], "UI", 5)
            loan = CCD(dat[col.index("wlt_loan")], "UI", 2)
            batch = CCD(dat[col.index("wlt_batch")], "Na", 7)
            trtp = CCD(dat[col.index("wlt_type")], "UI", 2)
            trdt = CCD(dat[col.index("wlt_trdt")], "D1", 10)
            ref = CCD(dat[col.index("wlt_ref")], "Na", 9)
            amt = CCD(dat[col.index("wlt_amt")], "SD", 13.2)
            ded = CCD(dat[col.index("wlt_ded")], "SD", 13.2)
            int = CCD(dat[col.index("wlt_per")], "UD", 13.2)
            curdt = CCD(dat[col.index("wlt_curdt")], "D2", 7)
            desc = CCD(dat[col.index("wlt_desc")], "NA", 30)
            if not self.trtp:
                self.trtp = trtp.work
                self.batch = batch.work
            if trtp.work != self.trtp:
                self.batchTotal()
                self.typeTotal()
                self.trtp = trtp.work
                self.batch = batch.work
                self.pglin = 99
            if batch.work != self.batch:
                self.batchTotal()
                self.batch = batch.work
                if self.totsonly != "Y":
                    self.typeHeading()
            if self.pglin > self.lpp:
                if self.pgnum:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
            if self.totsonly != "Y":
                self.t.textLine("%s %s %s %s %s %s %s %s" % \
                    (empno.disp, loan.disp, ref.disp, trdt.disp,
                    desc.disp, int.disp, amt.disp, ded.disp))
                self.pglin = self.pglin + 1
            self.bqty = self.bqty + 1
            self.bamt = float(ASD(self.bamt) + ASD(amt.work))
            self.bded = float(ASD(self.bded) + ASD(ded.work))
            self.tqty = self.tqty + 1
            self.tamt = float(ASD(self.tamt) + ASD(amt.work))
            self.tded = float(ASD(self.tded) + ASD(ded.work))
            self.gqt[trtp.work -1] = self.gqt[trtp.work -1] + 1
            self.gam[trtp.work -1] = float(ASD(self.gam[trtp.work -1]) + \
                ASD(amt.work))
            self.gdd[trtp.work -1] = float(ASD(self.gdd[trtp.work -1]) + \
                ASD(ded.work))
        p.closeProgress()
        if self.t:
            if can != "yes":
                self.batchTotal()
                self.typeTotal()
                self.grandTotal()
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
            head=self.tit, view=self.df.repprt[0][1],
            prtnam=self.df.repprt[0][2], mail=self.df.repprt[1][2])
            
    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        if self.totsonly == "Y":
            self.t.textLine(
            "%-34s %-7s %2s %-7s %20s %5s" % \
            ("Staff Loans Audit Trail for Period",
            self.sdatd, "to", self.edatd, "Page", self.pgnum))
        else:
            self.t.textLine(
            "%-34s %-7s %-2s %-7s %41s %5s" % \
            ("Staff Loans Audit Trail for Period",
            self.sdatd, "to", self.edatd, "Page", self.pgnum))
        self.t.textLine()
        self.pglin = 4
        if self.totind == "N":
            self.typeHeading()
        else:
            self.t.textLine("%-14s" % "Totals Summary")
            self.t.textLine()
            if self.totsonly == "Y":
                self.t.textLine(
                "%-44s%-8s  %-13s %-13s" % \
                ("Document Type", "Quantity", "      Amount", "   Deduction"))
            else:
                self.t.textLine(
                "%-30s%-8s  %-13s %-13s" % \
                ("Document Type", "Quantity", "      Amount", "   Deduction"))
            self.t.textLine("%s" % (self.suc * len(self.head)))
            self.t.setFont("Tartan", self.cpi, self.ldg)
            self.pglin = self.pglin + 4

    def typeHeading(self):
        if self.totsonly == "N":
            batch = self.batch
        else:
            batch = "Various"
        if self.lpp - self.pglin < 7:
            self.c.drawText(self.t)
            self.c.showPage()
            self.pageHeading()
            return
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.t.textLine("%-7s %-7s %-10s %3s" % \
            ("Batch", batch, "    Type", sltrtp[self.trtp - 1][1]))
        self.t.textLine()
        if self.totsonly == "Y":
            self.t.textLine("%-53s %-13s %-13s" % \
            ("Details", "      Amount", "   Deduction"))
        else:
            self.t.textLine(
            "%-5s %-2s %-9s %-10s %-30s %-13s %-13s %-13s" % \
            ("Empno", "Ln", "Reference", "Trans-Date", "Remarks",
            "       I-Rate", "      Amount", "   Deduction"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = self.pglin + 4

    def batchTotal(self):
        j = CCD(self.bamt, "SD", 13.2)
        k = CCD(self.bded, "SD", 13.2)
        if self.totsonly == "Y":
            self.t.textLine("%-5s %-7s %-39s %13s %13s" % \
                ("Batch", self.batch, "Totals", j.disp, k.disp))
        else:
            self.t.textLine()
            self.pglin = self.pglin + 1
            self.t.textLine("%-29s %-44s %13s %13s" % \
                (" ", "Batch " + self.batch + " Totals", j.disp, k.disp))
        self.pglin = self.pglin + 1
        if self.totsonly == "N":
            self.t.textLine()
            self.pglin = self.pglin + 1
        self.bqty = 0
        self.bamt = 0
        self.bded = 0

    def typeTotal(self):
        j = CCD(self.tamt, "SD", 13.2)
        k = CCD(self.tded, "SD", 13.2)
        if self.totsonly == "Y":
            self.t.textLine()
            self.t.textLine("%-53s %13s %13s" % ("Type Totals", j.disp, k.disp))
        else:
            self.t.textLine("%-29s %-44s %13s %13s" % \
            (" ", "Type Totals", j.disp, k.disp))
        self.t.textLine()
        self.tqty = 0
        self.tamt = 0
        self.tded = 0
    
    def grandTotal(self):
        self.c.drawText(self.t)
        self.c.showPage()
        self.totind = "Y"
        self.pageHeading()
        tot = [0,0,0,0]
        for x in range(0, len(sltrtp)):
            qt = CCD(self.gqt[x], "SI", 8)
            am = CCD(self.gam[x], "SD", 13.2)
            dd = CCD(self.gdd[x], "SD", 13.2)
            if self.totsonly == "Y":
                self.t.textLine("%-44s %s %s %s" % \
                (sltrtp[x][1], qt.disp, am.disp, dd.disp))
            else:
                self.t.textLine("%-30s %s %s %s" % \
                (sltrtp[x][1], qt.disp, am.disp, dd.disp))
            tot[0] = tot[0] + qt.work
            tot[1] = float(ASD(tot[1]) + ASD(am.work))
            tot[2] = float(ASD(tot[2]) + ASD(dd.work))
        self.t.textLine()
        qt = CCD(tot[0], "SI", 8)
        am = CCD(tot[1], "SD", 13.2)
        dd = CCD(tot[2], "SD", 13.2)
        if self.totsonly == "Y":
            self.t.textLine("%-44s %s %s %s" % \
            ("Grand Totals", qt.disp, am.disp, dd.disp))
        else:
            self.t.textLine("%-30s %s %s %s" % \
            ("Grand Totals", qt.disp, am.disp, dd.disp))
        self.t.textLine()

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

