#! /usr/bin/env python

"""
SYNOPSIS
    Staff Loans Master Listing.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import CCD, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, getModName, doPrinter, showError

class sl3030:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, "waglmf", prog=__name__)
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdtd = "%i/%02i/%02i" % (t[0], t[1], t[2])
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Staff Loans Master Listing (%s)" % __name__)
        r1s = (("Number","N"),("Name","M"))
        r2s = (("Yes","Y"),("No","N"))
        fld = (
            (("T",0,0,0),("IRB",r1s),1,"Sort Order","Sort Order",
                "N","Y",self.doSort,None,None,None),
            (("T",0,1,0),("IRB",r2s),1,"Ignore Zero Balances",
                "Ignore Zero Balances","Y","Y",self.doZero,None,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doSort(self, frt, pag, r, c, p, i, w):
        self.sort = w

    def doZero(self, frt, pag, r, c, p, i, w):
        self.zero = w

    def doEnd(self):
        self.df.closeProcess()
        col = ["wlm_empno", "wgm_sname", "wlm_loan", "wlm_desc",
            "wlm_ded_code", "wlm_start_date", "wlm_int_per", "wlm_ded_amt"]
        whr = [("wlm_cono", "=", self.conum), ("wlm_cono=wgm_cono",),
            ("wlm_empno=wgm_empno",)]
        if self.sort == "N":
            ord = "wlm_empno, wlm_loan"
        else:
            ord = "wlm_sname"
        all = self.sql.getRec(tables=["waglmf", "wagmst"], cols=col,
            where=whr, order=ord)
        if not all:
            showError(self.mf.body, "Transaction Error",
            "No Transactions Selected")
        else:
            self.printReport(all)
        self.closeProcess()

    def printReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        self.head = ("%03u %-30s %63s %6s" % \
            (self.conum, self.conam, self.sysdttm, __name__))
        pdfnam = getModName(self.mf.wrkdir, __name__, self.conum, ext="pdf")
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, pdfnam, self.head)
        self.pgnum = 0
        self.pglin = 99
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            emp = CCD(dat[0], "UI", 5)
            nam = CCD(dat[1], "NA", 30)
            lon = CCD(dat[2], "UI", 2)
            des = CCD(dat[3], "NA", 30)
            cod = CCD(dat[4], "UI", 2)
            dte = CCD(dat[5], "d1", 10)
            per = CCD(dat[6], "SD", 10.2)
            ded = CCD(dat[7], "SD", 10.2)
            if self.pglin > self.lpp:
                if self.pgnum:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
            self.t.textLine("%s %s %s %s %s %s %s %s" % (emp.disp, nam.disp,
                lon.disp, des.disp, cod.disp, dte.disp, per.disp, ded.disp))
            self.pglin = self.pglin + 1
        p.closeProgress()
        if self.t:
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=pdfnam, head=self.tit,
            view=self.df.repprt[0][1], prtnam=self.df.repprt[0][2],
            mail=self.df.repprt[1][2])

    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-29s %-10s %58s %5s" % \
            ("Staff Loans Master List as at", self.sysdtd,
                "Page", self.pgnum))
        self.t.textLine()
        self.t.textLine("%-15s%-1s%-3s%-16s%-1s%-3s%-4s%-1s%-1s" % \
            ("(Options: Sort-", self.df.t_disp[0][0][0], "",
                "Ignore Zero Bal-", self.df.t_disp[0][0][1], "",
                "Csv-", self.df.t_disp[0][0][2], ")"))
        self.t.textLine()
        self.t.textLine("%-5s %-30s %-2s %-30s %-2s %-10s %-10s %-13s" % \
            ("EmpNo", "Name", "Ln", "Description", "Cd", "Start-Date",
            " Int Rate", "Deduction"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 8

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

