#! /usr/bin/env python

"""
SYNOPSIS
    Staff Loans Statements.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from tartanWork import sltrtp
from TartanClasses import ASD, CCD, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, doPrinter, getModName, getSingleRecords
from tartanFunctions import showError

class sl3040:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        ok = self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["wagmst", "waglmf", "wagltf"],
            prog=__name__)
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.head = ("%03u %-30s %80s %6s" % \
            (self.conum, self.conam, self.sysdttm, __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Staff Loans Statements (%s)" % __name__)
        r1s = (("Yes","Y"),("Singles","S"))
        r2s = (("Yes","Y"),("No","N"))
        fld = (
            (("T",0,0,0),"Id1",10,"Start Date","Start Date",
                "","Y",self.doFdate,None,None,("efld",)),
            (("T",0,1,0),"ID1",10,"Last Date","Last Date",
                self.sysdtw,"Y",self.doLdate,None,None,("efld",)),
            (("T",0,2,0),("IRB",r1s),0,"Whole File","Whole File",
                "S","Y",self.doWhole,None,None,None),
            (("T",0,3,0),("IRB",r2s),0,"Ignore Paid Ups","Ignore Paid Ups",
                "Y","Y",self.doPaidup,None,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doFdate(self, frt, pag, r, c, p, i, w):
        self.fdate = w

    def doLdate(self, frt, pag, r, c, p, i, w):
        self.ldate = w

    def doWhole(self, frt, pag, r, c, p, i, w):
        self.whole = w

    def doPaidup(self, frt, pag, r, c, p, i, w):
        self.paidup = w

    def doEnd(self):
        self.df.closeProcess()
        self.prtnam = self.df.repprt[0][2]
        whr = [("wlm_cono", "=", self.conum)]
        if self.whole == "S":
            all = getSingleRecords(self.mf, "waglmf",
                ("wlm_empno", "wlm_desc"), where=whr)
        else:
            ord = "wlm_empno, wlm_loan"
            all = self.sql.getRec(tables="waglmf", where=whr, order=ord)
            if not all:
                showError(self.mf.body, "Error", "No Accounts Selected")
        if all:
            p = ProgressBar(self.mf.body, max=len(all), esc=True)
            for num, emp in enumerate(all):
                can = p.displayProgress(num)
                if can == "yes":
                    if self.t:
                        self.t.textLine()
                        self.t.textLine("Print Job ABORTED")
                        break
                self.doProcess(emp)
            p.closeProgress()
            self.c.save()
            self.doPrint()
        self.mf.closeLoop()

    def doProcess(self, emp):
        self.pgnum = 0
        self.totbal = 0
        dic = self.sql.waglmf_dic
        self.emp = CCD(emp[dic["wlm_empno"][1]], "UI", 5)
        whr = [("wgm_cono", "=", self.conum), ("wgm_empno", "=", self.emp.work)]
        col = ("wgm_sname", "wgm_fname")
        mst = self.sql.getRec(tables="wagmst", where=whr, cols=col, limit=1)
        self.nam = CCD(mst[0] + ", " + mst[1], "NA", 60)
        self.lon = CCD(emp[dic["wlm_loan"][1]], "UI", 2)
        self.des = CCD(emp[dic["wlm_desc"][1]], "NA", 30)
        whr = [("wlt_cono", "=", self.conum), ("wlt_empno", "=", self.emp.work),
            ("wlt_loan", "=", self.lon.work)]
        col = ("wlt_trdt", "wlt_type", "wlt_ref", "wlt_batch", "wlt_per",
            "wlt_amt", "wlt_ded", "wlt_desc")
        ord = "wlt_trdt, wlt_type, wlt_ref"
        wlt = self.sql.getRec(tables="wagltf", where=whr, cols=col, order=ord)
        if wlt:
            self.loadBalances(wlt)
            if self.paidup == "Y" and not self.cbal:
                pass
            else:
                self.printHeader()
                self.printBody(wlt)
                self.printTrailer()

    def loadBalances(self, wlt):
        self.obal = 0.0
        self.cbal = 0.0
        for x in range(0, len(wlt)):
            dat = CCD(wlt[x][0], "d1", 10)
            typ = CCD(wlt[x][1], "UI", 2)
            amt = CCD(wlt[x][5], "SD", 13.2)
            if self.fdate and dat.work < self.fdate:
                self.obal = float(ASD(self.obal) + ASD(float(amt.work)))
            if dat.work <= self.ldate:
                self.cbal = float(ASD(self.cbal) + ASD(float(amt.work)))
            
    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-27s %-15s %2s %15s %53s %5s" % \
            ("Staff Loans Statements from", self.df.t_disp[0][0][0],
            "to", self.df.t_disp[0][0][1], "Page", self.pgnum))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 4

    def printHeader(self):
        self.pageHeading()
        self.pglin = self.pglin + 1
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        mess = "%-8s %-5s %-60s %-4s %-2s %-30s" % ("Employee",
            self.emp.disp, self.nam.disp, "Loan", self.lon.disp, self.des.disp)
        self.t.textLine(mess)
        self.pglin = self.pglin + 1
        self.t.textLine()
        self.pglin = self.pglin + 1
        mess = "%-10s %-3s %-9s %-7s %-6s %-13s %-13s %-13s %-30s" % \
            ("   Date", "Typ", "Reference", "Batch", "  Int-%",
            "      Amount", "   Deduction", "     Balance", "Remarks")
        self.t.textLine(mess)
        self.pglin = self.pglin + 1
        mess = "%s" % (self.suc * len(self.head))
        self.t.textLine(mess)
        self.pglin = self.pglin + 1
        self.t.setFont("Tartan", self.cpi, self.ldg)

    def printBody(self, wlt):
        for trn in wlt:
            dat = CCD(trn[0], "d1", 10)
            bat = CCD(trn[3], "Na", 7)
            typ = CCD(trn[1], "UI", 2)
            ref = CCD(trn[2], "Na", 9)
            int = CCD(trn[4], "UD", 7.2)
            amt = CCD(trn[5], "SD", 13.2)
            ded = CCD(trn[6], "SD", 13.2)
            des = CCD(trn[7], "NA", 30)
            if self.fdate and dat.work < self.fdate:
                continue
            if dat.work > self.ldate:
                continue
            if self.pglin > self.lpp:
                self.printTrailer()
                self.printHeader()
                bal = CCD(self.obal, "SD", 13.2)
                if bal.work:
                    mess = "%-69s%s%-30s" % ("", bal.disp, " Brought Forward")
                    self.t.textLine(mess)
                    self.pglin = self.pglin + 1
            self.obal=float(ASD(self.obal)+ASD(float(amt.work)))
            bal = CCD(self.obal, "SD", 13.2)
            mess = "%s %s %s %s %s %s %s %s %s" % \
                (dat.disp, sltrtp[typ.work - 1][0], ref.disp, bat.disp,
                int.disp, amt.disp, ded.disp, bal.disp, des.disp)
            self.t.textLine(mess)
            self.pglin = self.pglin + 1

    def printTrailer(self):
        self.c.drawText(self.t)
        self.c.showPage()

    def doPrint(self):
        if self.prtnam == "view":
            view = "Y"
        else:
            view = "N"
        doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
            head=self.tit, view=view, prtnam = self.prtnam)

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()

