#! /usr/bin/env python

"""
SYNOPSIS
    Staff Loans Interrogation.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, CCD, LoanInterest, SChoice, Sql
from TartanClasses import SRec, TartanDialog
from tartanFunctions import doReportlab, getModName, getPeriods,  doPrinter
from tartanWork import sltrtp

class sl4010:
    def __init__(self, mf, coy, nam, usr):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.capnm = usr
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlynd", "wagmst", "waglmf", "wagltf"],
            prog=__name__)
        if self.sql.error:
            return self.sql.error
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) +  (t[1] * 100) + t[2]
        self.sysdtd = "%i/%02i/%02i" % (t[0], t[1], t[2])
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        # Get the current period starting date
        period = self.sql.getRec(tables="ctlynd", cols=["max(cye_period)"],
            where=[("cye_cono", "=", self.conum)], limit=1)[0]
        self.period = getPeriods(self.mf, self.conum, period)[0].work

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Staff Loans Interrogation (%s)" % __name__)
        wgm = {
            "stype": "R",
            "tables": ("waglmf", "wagmst"),
            "cols": (
                ("wlm_empno", "", 0, "EmpNo"),
                ("wgm_sname", "", 0, "Surname", "Y"),
                ("wlm_loan", "", 0, "Ln")),
            "where": [("wlm_cono", "=", self.conum), ("wgm_cono=wlm_cono",),
                ("wgm_empno=wlm_empno",)]}
        lnm = {
            "stype": "R",
            "tables": ("waglmf",),
            "cols": (
                ("wlm_empno", "", 0, "EmpNo"),
                ("wlm_loan", "", 0, "Ln"),
                ("wlm_desc", "", 0, "Description", "Y")),
            "where": [("wlm_cono", "=", self.conum)],
            "whera": [["T", "wlm_empno", 0, 0]],
            "index": 1}
        r1s = (("Yes", "Y"), ("No", "N"))
        tag = (
            ("General", self.doGeneral, ("T",0,0), ("T",0,1)),
            ("Trans", self.doTrans1, ("T",0,0), ("T",0,1)))
        fld = (
            (("T",0,0,0),"IUI",5,"Emp-Num","Employee Number",
                "","N",self.doEmp,wgm,None,("notzero",)),
            (("T",0,0,0),"ONA",20,"Surname"),
            (("T",0,0,0),"IUI",5,"Loan-Num","Loan Number",
                "","N",self.doLoan,lnm,None,("notzero",)),
            (("T",0,0,0),"ONA",20,"Desc"),
            (("T",1,0,0),"ONA",30,"Description"),
            (("T",1,1,0),"OUI",3,"Deduction Code"),
            (("T",1,2,0),"OUD",6.2,"Interest Percentage"),
            (("T",1,3,0),"OD1",10,"Start Date"),
            (("T",1,4,0),"OSD",10.2,"Deduction Amount"),
            (("T",1,5,0),"OSD",10.2,"Total Advances"),
            (("T",1,6,0),"OSD",10.2,"Total Interest"),
            (("T",1,7,0),"OSD",10.2,"Total Repayments"),
            (("T",1,8,0),"OSD",10.2,"Total Adjustments"),
            (("T",1,9,0),"OSD",10.2,"Balance"),
            (("T",1,10,0),"Od1",10,"Last Interest Raised"),
            (("T",1,11,0),"Od1",10,"Last Payment Received"),
            (("T",1,12,0),"OSD",10.2,"Capital"),
            (("T",1,13,0),"OSD",10.2,"Int-Due"),
            (("T",2,0,0),("IRB",r1s),0,"History","",
                "Y","Y",self.doTrans2,None,None,None))
        tnd = ((self.doEndTop,"N"), None, None)
        txt = (self.doExit, None, None)
        cnd = (None, None, None)
        cxt = (None, None, None)
        but = (
            ("Clear",None,self.doClear,0,("T",0,0),("T",0,1)),
            ("Print",None,self.doPrint,0,("T",0,0),("T",0,1)),
            ("Quit",None,self.doExit,1,None,None))
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tags=tag,
            tend=tnd, txit=txt, cend=cnd, cxit=cxt, butt=but)

    def doEmp(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="wagmst", cols=["wgm_sname",
            "wgm_fname"], where=[("wgm_cono", "=", self.conum),
            ("wgm_empno", "=", w)], limit=1)
        if not acc:
            return "Invalid Employee Number"
        self.empno = w
        self.sname = acc[0]
        self.fname = acc[1]
        self.history = "Y"
        self.df.loadEntry("T", pag, p+1, data=self.sname)

    def doLoan(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="waglmf", cols=["wlm_desc",
            "wlm_ded_code", "wlm_int_per", "wlm_start_date", "wlm_ded_amt"],
            where=[("wlm_cono", "=", self.conum), ("wlm_empno", "=",
            self.empno), ("wlm_loan", "=", w)], limit=1)
        if not acc:
            return "Invalid Loan Number"
        self.loan = w
        self.desc = acc[0]
        self.df.loadEntry("T", pag, p+1, data=self.desc)
        d = 0
        for i in range(0, 5):
            self.df.loadEntry("T", 1, i, data=acc[d])
            d = d + 1
        self.lon = 0.0
        self.imt = 0.0
        self.pay = 0.0
        self.adj = 0.0
        self.bal = 0.0
        self.idt = 0
        self.pdt = 0
        self.cap = 0.0
        self.due = 0
        self.loadBalances()
        rate = self.df.t_work[1][0][2]
        start = self.df.t_work[1][0][3]
        lint = LoanInterest(self.mf, self.period, self.conum,
            self.empno, self.loan, rate, start, self.sysdtw, "Pending",
            (self.sysdtw/100), update="Y", glctls=(933, 933))
        self.bal = float(ASD(self.bal) + ASD(lint.imt))
        self.imt = float(ASD(self.imt) + ASD(lint.imt))
        self.due = float(ASD(self.due) + ASD(lint.imt))
        self.df.loadEntry("T", 1, 5, data=self.lon)
        self.df.loadEntry("T", 1, 6, data=self.imt)
        self.df.loadEntry("T", 1, 7, data=self.pay)
        self.df.loadEntry("T", 1, 8, data=self.adj)
        self.df.loadEntry("T", 1, 9, data=self.bal)
        self.df.loadEntry("T", 1, 10, data=self.idt)
        self.df.loadEntry("T", 1, 11, data=self.pdt)
        self.df.loadEntry("T", 1, 12, data=self.cap)
        self.df.loadEntry("T", 1, 13, data=self.due)
        self.mf.updateStatus("")

    def doHist(self, frt, pag, r, c, p, i, w):
        self.history = w

    def doEndTop(self):
        self.df.last[0] = [0, 0]
        self.df.setWidget(self.df.topEntry[0][0], state="disabled")

    def loadBalances(self):
        whr = [("wlt_cono", "=", self.conum), ("wlt_empno", "=", self.empno),
            ("wlt_loan", "=", self.loan)]
        odr = "wlt_trdt, wlt_type"
        self.wlt = self.sql.getRec(tables="wagltf", where=whr, order=odr)
        if self.wlt:
            for rec in self.wlt:
                dat = CCD(rec[self.sql.wagltf_col.index("wlt_trdt")], "d1", 10)
                typ = CCD(rec[self.sql.wagltf_col.index("wlt_type")], "UI", 2)
                amt = CCD(rec[self.sql.wagltf_col.index("wlt_amt")], "SD", 13.2)
                self.bal = float(ASD(self.bal) + ASD(amt.work))
                self.cap = float(ASD(self.cap) + ASD(amt.work))
                if typ.work == 1:
                    if dat.work >= self.period and \
                        dat.work >= self.df.t_work[1][0][3]:
                        self.cap = float(ASD(self.cap) - ASD(amt.work))
                        self.due = float(ASD(self.due) + ASD(amt.work))
                    self.imt = float(ASD(self.imt) + ASD(amt.work))
                    self.idt = dat.work
                elif typ.work in (2, 3):
                    self.lon = float(ASD(self.lon) + ASD(amt.work))
                elif typ.work == 4:
                    self.pay = float(ASD(self.pay) + ASD(amt.work))
                    self.pdt = dat.work
                elif typ.work == 5:
                    self.adj = float(ASD(self.adj) + ASD(amt.work))

    def doGeneral(self):
        self.mf.updateStatus("")

    def doTrans1(self, widget=None):
        self.df.focusField("T", 2, 1)

    def doTrans2(self, frt, pag, r, c, p, i, w):
        self.history = w
        tit = "Transactions for Loan: %s %s - %s" % \
            (self.empno, self.sname, self.desc)
        tab = ["wagltf"]
        col = (("wlt_trdt", "", 0, "   Date"),
                ("wlt_batch", "", 0, "Batch"),
                ("wlt_type", ("XX", sltrtp), 3, "Typ"),
                ("wlt_ref", "", 0, "Reference", "Y"),
                ("wlt_per", "", 0, " Int-%"),
                ("wlt_amt", "", 0, "      Amount"),
                ("wlt_ded", "", 0, "   Deduction"),
                ("wlt_desc", "", 0, "Remarks"))
        whr = [
            ("wlt_cono", "=", self.conum),
            ("wlt_empno", "=", self.empno),
            ("wlt_loan", "=", self.loan)]
        if self.history == "N":
            whr.append(("wlt_curdt", ">=", self.period / 100))
        odr = "wlt_trdt, wlt_type, wlt_ref"
        state = self.df.disableButtons()
        rec = SRec(self.mf, screen=self.df.nb.Page2, title=tit, tables=tab,
            cols=col, where=whr, order=odr, rows=12)
        self.df.enableButtons(state=state)
        self.df.focusField("T", 2, 1)

    def doClear(self, widget=None):
        self.df.selPage("General")
        self.df.focusField("T", 0, 1)

    def doPrint(self, widget=None):
        head = "Select the required print options"
        data = (
            ("Info Only", "I"),
            ("Transactions Only", "T"),
            ("Info and Transactions", "A"))
        state = self.df.disableButtons()
        ss = SChoice(self.mf, head=head, data=data)
        self.df.enableButtons(state=state)
        self.mf.updateStatus("")
        if ss.selection == [] or ss.selection == None:
            self.pr_opts = " "
        else:
            self.pr_opts = ss.selection[1]
        if self.pr_opts != " ":
            self.printDetail()
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def printDetail(self):
        self.head = ("%03u %-30s %47s %10s" % \
            (self.conum, self.conam, self.sysdttm, __name__))
        from tartanFunctions import doReportlab
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        self.pgnum = 0
        self.pglin = 99
        if self.pr_opts != "T":
            self.pageHeading()
            self.printInfo()
        if self.pr_opts == "I":
            self.c.drawText(self.t)
            self.c.showPage()
            self.c.save()
            doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                head=self.tit, view="V")
        elif self.pr_opts == "A":
            if not self.wlt:
                self.c.drawText(self.t)
                self.c.showPage()
                self.c.save()
                doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                    head=self.tit, view="V")
            else:
                self.c.drawText(self.t)
                self.c.showPage()
                self.acctot = 0
                self.pageHeading()
                self.pageHeadingTrans()
                self.printTrans()
                self.c.drawText(self.t)
                self.c.showPage()
                self.c.save()
                doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                    head=self.tit, view="V")
        elif self.pr_opts == "T":
            if self.wlt:
                self.acctot = 0
                self.pageHeading()
                self.pageHeadingTrans()
                self.printTrans()
                self.c.drawText(self.t)
                self.c.showPage()
                self.c.save()
                doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                    head=self.tit, view="V")

    def printInfo(self):
        self.t.textLine("%-5s %-20s %-25s %s" %    ("", "",
                self.df.topf[0][0][3], self.df.t_disp[0][0][0]))
        self.t.textLine("%-5s %-20s %-25s %s" %    ("", "",
                self.df.topf[0][1][3], self.df.t_disp[0][0][1]))
        self.t.textLine("%-5s %-20s %-25s %s" %    ("", "",
                self.df.topf[0][2][3], self.df.t_disp[0][0][2]))
        self.t.textLine()
        for x in range(0, len(self.df.topf[1])):
            self.t.textLine("%-5s %-20s %-25s %s" % ("", "",
                self.df.topf[1][x][3], self.df.t_disp[1][0][x]))
        self.t.textLine()

    def printTrans(self):
        whr = [
            ("wlt_cono", "=", self.conum),
            ("wlt_empno", "=", self.empno),
            ("wlt_loan", "=", self.loan)]
        odr = "wlt_trdt, wlt_type"
        if self.history == "N":
            whr.append(("wlt_curdt", ">=", self.period / 100))
        recs = self.sql.getRec(tables="wagltf", where=whr, order=odr)
        for rec in recs:
            trd = CCD(rec[self.sql.wagltf_col.index("wlt_trdt")], "D1", 10)
            typ = CCD(rec[self.sql.wagltf_col.index("wlt_type")], "UI", 1)
            ref = CCD(rec[self.sql.wagltf_col.index("wlt_ref")], "Na", 9)
            bat = CCD(rec[self.sql.wagltf_col.index("wlt_batch")], "Na", 7)
            rte = CCD(rec[self.sql.wagltf_col.index("wlt_per")], "NA", 7.2)
            amt = CCD(rec[self.sql.wagltf_col.index("wlt_amt")], "SD", 11.2)
            ded = CCD(rec[self.sql.wagltf_col.index("wlt_ded")], "SD", 11.2)
            det = CCD(rec[self.sql.wagltf_col.index("wlt_desc")], "NA", 30)
            if self.pglin > self.lpp:
                if self.pgnum != 0:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
                self.pageHeadingTrans()
            self.acctot = float(ASD(self.acctot) + ASD(amt.work))
            self.t.textLine("%s %s %s %s %s %s %s %s" % (trd.disp,
                sltrtp[(typ.work - 1)][0], ref.disp, bat.disp,
                rte.disp, amt.disp, ded.disp, det.disp))
            self.pglin = self.pglin + 1

    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-31s %-10s %44s %5s" % \
            ("Staff Loans Interrogation as at", self.sysdtd, "Page",
            self.pgnum))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 6

    def pageHeadingTrans(self):
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.t.textLine()
        self.t.textLine("%-1s %-7s %s %-2s %-7s %s %-2s %-8s %s " \
            "%-2s %-4s %s" %    ("",
            self.df.topf[0][0][3], self.df.t_disp[0][0][0], "",
            self.df.topf[0][1][3], self.df.t_disp[0][0][1], "",
            self.df.topf[0][2][3], self.df.t_disp[0][0][2], "",
            self.df.topf[0][3][3], self.df.t_disp[0][0][3]))
        self.t.textLine()
        self.t.textLine("%-10s %-3s %-9s %-7s %-7s %-10s %-12s %-7s" % \
            ("   Date", "Typ", "Reference", "Batch", "Int%",
            "    Amount", "  Deduction", "Remarks"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 8
        b = CCD(self.acctot / 100, "SD", 15.2)
        if self.acctot != 0:
            self.t.textLine("%-42s %13s %-30s" % \
                (" ", b.disp, "Brought Forward"))
            self.pglin = self.pglin + 1

    def doExit(self, widget=None):
        try:
            self.mf.dbm.rollbackDbase()
        except:
            pass
        self.df.closeProcess()
        self.mf.closeLoop()

