#! /usr/bin/python2

"""
SYNOPSIS
    Stores Purchase Orders Placing and Receiving.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, Balances, CCD, GetCtl, PrintOrder, PwdConfirm
from TartanClasses import Sql, TartanDialog
from tartanFunctions import callModule, showError, showPrinters, textFormat

class st2010:
    def __init__(self, mf, conum, conam, period, usr):
        self.mf = mf
        self.conum = conum
        self.conam = conam
        self.period = period
        self.capnm = usr
        no = self.setVariables()
        if not no:
            self.getPrinters()
            if self.prtnam != "cancel":
                self.mainProcess()
                self.mf.startLoop()
        elif no == "nocrs":
            showError(self.mf.body, "System Error",
                "This module requires you to have a Creditor's System")
        else:
            showError(self.mf.body, "Control Error",
                "Control Record %s Not Found, "\
                "Please Create It and then Retry." % no)

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlmst", "ctlmes", "ctlvmf", "crsmst",
            "gentrn", "strgrp", "strloc", "strmf1", "strmf2", "strpom",
            "strpot", "strtrn", "strprc"], prog=__name__)
        if self.sql.error:
            return self.sql.error
        gc = GetCtl(self.mf)
        ctlsys = gc.getCtl("ctlsys")
        if ctlsys and ctlsys["sys_smtp"]:
            self.email = "Y"
        else:
            self.email = "N"
        ctlmst = gc.getCtl("ctlmst", self.conum)
        mods = []
        for x in range(0, 21, 2):
            mods.append(ctlmst["ctm_modules"][x:x+2])
        if not "CR" in mods:
            return "nocrs"
        strctl = gc.getCtl("strctl", self.conum)
        if not strctl:
            return "strctl"
        self.stgl = strctl["cts_glint"]
        self.locs = strctl["cts_locs"]
        self.prices = strctl["cts_prices"]
        self.automu = strctl["cts_automu"]
        if self.stgl == "Y":
            ctlctl = gc.getCtl("ctlctl", self.conum)
            if not ctlctl:
                return "ctlctl"
            ctls = ["stk_soh", "stk_susp"]
            if gc.chkRec(self.conum, ctlctl, ctls):
                return "no"
            self.soh = ctlctl["stk_soh"]
            self.ssp = ctlctl["stk_susp"]
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]

    def getPrinters(self):
        tit = ("Printer Selection",)
        self.printers = showPrinters()
        prt = {
            "stype": "C",
            "titl": "Available Printers",
            "head": ("Name","Description"),
            "data": self.printers}
        r1s = (("Yes","Y"),("No","N"))
        r2s = (("Print","P"),("View","V"),("None","N"))
        fld = [
            (("T",0,0,0),"ID1",10,"Order Date","Order Date",
                self.sysdtw,"N",self.doOrdDate,None,None,("efld",)),
            (("T",0,1,0),("IRB",r1s),0,"Auto Markup","Automatic Markup",
                self.automu,"N",self.doAutoMkUp,None,None,None),
            (("T",0,2,0),("IRB",r2s),0,"Choice","Select Choice",
                "P","N",self.doPrtChoice,None,None,None),
            (("T",0,3,0),"INA",30,"Printer","Printer Name",
                "Default","Y",self.doPrtNam,prt,None,None)]
        if self.email == "Y":
            fld.append(
            (("T",0,4,0),("IRB",r1s),0,"E-Mail","E-Mail Document",
                "Y","N",self.doPrtMail,None,None,None))
        self.pr = TartanDialog(self.mf, screen=self.mf.body, tops=True,
            title=tit, eflds=fld, tend=((self.doPrtEnd,"y"),),
            txit=(self.doPrtExit,))
        self.mf.startLoop()

    def doOrdDate(self, frt, pag, r, c, p, i, w):
        if w < self.period[1][0] or w > self.period[2][0]:
            return "Invalid Date, Not In Financial Period" 
        self.trdt = w
        self.trdd = self.pr.t_disp[0][0][0]
        self.curdt = self.trdt / 100
        self.batch = "S%s" % self.curdt

    def doAutoMkUp(self, frt, pag, r, c, p, i, w):
        self.mkup = w

    def doPrtChoice(self, frt, pag, r, c, p, i, w):
        if w == "P":
            return "ok"
        if w == "V":
            self.prtnam = "view"
        else:
            self.prtnam = "none"
        return "sk1"

    def doPrtNam(self, frt, pag, r, c, p, i, w):
        if w not in self.printers:
            return "Invalid Printer"
        self.prtnam = w

    def doPrtMail(self, frt, pag, r, c, p, i, w):
        self.email = w

    def doPrtEnd(self):
        self.doPrtClose()

    def doPrtExit(self):
        self.prtnam = "cancel"
        self.doPrtClose()

    def doPrtClose(self):
        self.pr.closeProcess()
        self.mf.closeLoop()

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "Purchase Orders as at %s" % self.trdd)
        doc = {
            "stype": "R",
            "tables": ("strpom", "crsmst"),
            "cols": (
                ("pom_ordno", "", 0, "Ord-Num"),
                ("pom_acno", "", 0, "Acc-Num"),
                ("crm_name", "", 0, "Name", "Y")),
            "where": [("pom_cono", "=", self.conum), ("crm_cono=pom_cono",),
                ("crm_acno=pom_acno",), ("pom_delno", "=", "")]}
        loc = {
            "stype": "R",
            "tables": ("strloc",),
            "cols": (
                ("srl_loc", "", 0, "L"),
                ("srl_desc", "", 0, "Description", "Y")),
            "where": [("srl_cono", "=", self.conum)]}
        crm = {
            "stype": "R",
            "tables": ("crsmst",),
            "cols": [
                ("crm_acno", "", 0, "Acc-Num"),
                ("crm_name", "", 0, "Name","Y")],
            "where": [("crm_cono", "=", self.conum)],
            "index": 0}
        ord = {
            "stype": "R",
            "tables": ("ctlrep",),
            "cols": [
                ("rpm_rep", "", 0, "Ord"),
                ("rpm_name", "", 0, "Name","Y")],
            "where": [("rpm_cono", "=", self.conum)],
            "index": 0}
        grp = {
            "stype": "R",
            "tables": ("strgrp",),
            "cols": (
                ("gpm_group", "", 0, "Grp"),
                ("gpm_desc", "", 0, "Description", "Y")),
            "where": [("gpm_cono", "=", self.conum)]}
        cod = {
            "stype": "R",
            "tables": ("strmf1",),
            "cols": (
                ("st1_group", "", 0, "Grp"),
                ("st1_code", "", 0, "Product-Code"),
                ("st1_desc", "", 0, "Description","Y")),
            "where": [],
            "order": "st1_group, st1_code",
            "index": 1}
        r1s = (
            ("New", "N"),
            ("Amend", "A"),
            ("Receive", "R"),
            ("Cancel", "C"))
        r2s = (("Account","A"),("Cash","C"))
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"Action","Action",
                "N","Y",self.doActTyp,None,None,None),
            (("T",0,1,0),"IUI",9,"Ord-No","Order Number",
                "","N",self.doOrdNum,doc,None,("notblank",)),
            (("T",0,2,0),"IUA",1,"Loc","Location Code",
                "1","Y",self.doLoc,loc,None,None),
            (("T",0,2,0),"ONA",30,""),
            (("T",0,3,0),"INA",7,"Acc-No","Account Number",
                "","N",self.doCrsAcc,crm,None,None),
            (("T",0,3,0),"ONA",30,""),
            (("T",0,4,0),"INa",3,"Ord-By","Ordered By",
                "","N",self.doOrdBy,ord,None,None),
            (("T",0,4,0),"ONA",30,""),
            (("T",0,5,0),"INa",7,"Del-No","Delivery Note Number",
                "","N",self.doDelNum,None,None,("notblank",)),
            (("T",0,5,0),"ID1",10,"Date","Delivery Date",
                self.trdt,"N",self.doDelDate,None,None,("efld",)),
            (("C",0,0,0),"INA",3,"Grp","Product Group",
                "r","Y",self.doGrp,grp,None,None),
            (("C",0,0,1),"INA",20,"Product-Code","Product Code",
                "","N",self.doCod,cod,None,None),
            (("C",0,0,2),"INA",(20,30),"Description","Description",
                "","N",self.doDes,None,None,None),
            (("C",0,0,3),"ISD",10.2,"Quantity","Quantity",
                1,"N",self.doQty,None,None,("notzero",)),
            (("C",0,0,4),"INA",1,"V","V.A.T. Code",
                "I","N",self.doVat,None,None,("notblank",)),
            (("C",0,0,5),"IUD",9.2,"Price","Exclusive Price",
                "","N",self.doPri,None,None,None),
            (("C",0,0,6),"IUD",6.2,"Dis-%","Trade Discount",
                "","N",self.doDis,None,None,None),
            (("C",0,0,7),"OSD",10.2,"Value"))
        self.row = (15,)
        self.but = (
            ("Quit",None,self.doTopExit,1,None,None),
            ("QryCrs",None,self.doQryCrs,1,None,None),
            ("QryStr",None,self.doQryStk,1,None,None),
            ("DelAdd",None,self.doDelAdd,1,("C",0,1),("T",0,1)),
            ("Ribbon",None,self.doRibbon,1,("C",0,1),("T",0,1)),
            ("Cancel",None,self.doCancel,1,("C",0,1),("T",0,1)),
            ("Messag",None,self.doMessag,1,("C",0,1),("T",0,1)),
            ("Edit",None,self.doEdit,0,("C",0,1),("C",0,2)),
            ("Reprnt",None,self.doReprnt,0,("T",0,1),("T",0,2)),
            ("NewCrs",None,self.doNewCrs,0,("T",0,1),("T",0,2)),
            ("NewStr",None,self.doNewStr,0,("T",0,1),("T",0,2)),
            ("Accept",None,self.doAccept,0,("C",0,1),(("T",0,1),("C",0,2))))
        tnd = ((self.doTopEnd,"n"), )
        txt = (self.doTopExit, )
        cnd = ((self.doColEnd,"n"), )
        cxt = (None, )
        self.df = TartanDialog(self.mf, screen=self.mf.body, title=tit,
            eflds=fld, rows=self.row, butt=self.but, tend=tnd, txit=txt,
            cend=cnd, cxit=cxt)

    def doActTyp(self, frt, pag, r, c, p, i, w):
        # Initialise Action Variables
        self.acttyp = w
        self.actord = False
        # Initialise Variables
        self.dnum = ""
        self.onum = ""
        self.jnum = ""
        self.dadd = ""
        self.cnam = ""
        self.vatn = ""
        self.ribbon = None
        ctl = self.sql.getRec(tables="ctlmst", where=[("ctm_cono", "=",
            self.conum)], limit=1)
        self.deladd = [
            ctl[self.sql.ctlmst_col.index("ctm_sadd1")],
            ctl[self.sql.ctlmst_col.index("ctm_sadd2")],
            ctl[self.sql.ctlmst_col.index("ctm_sadd3")],
            ctl[self.sql.ctlmst_col.index("ctm_spcode")]] 
        self.ctlmes = None
        #
        for x in (3,4,6):
            if self.acttyp == "R":
                self.df.butt[x][3] = 0
                self.df.butt[x][4] = None
            else:
                self.df.butt[x][3] = 1
                self.df.butt[x][4] = ["C", 0, 1]
        if self.acttyp == "N":
            # Get and Display Next Document Number
            ordno = self.sql.getRec(tables="strpom", cols=["max(pom_ordno)"],
                where=[("pom_cono", "=", self.conum)], limit=1)
            if not ordno[0]:
                self.ordno = CCD(1, "UI", 9).work
                self.othno = CCD(1, "Na", 9).work
            else:
                self.ordno = CCD((int(ordno[0]) + 1), "UI", 9).work
                self.othno = CCD((int(ordno[0]) + 1), "Na", 9).work
            self.df.loadEntry(self.df.frt, self.df.pag, 1, data=self.ordno)
            # Location default
            if self.locs == "N":
                self.loc = "1"
                self.df.loadEntry(frt, pag, p+2, data=self.loc)
                self.df.loadEntry(frt, pag, p+3, data="Location One")
                return "sk2"
            return "sk1"

    def doOrdNum(self, frt, pag, r, c, p, i, w):
        strpom = self.sql.getRec(tables="strpom", where=[("pom_cono", "=",
            self.conum), ("pom_ordno", "=", w)], limit=1)
        if not strpom:
            return "Invalid Document"
        if strpom[self.sql.strpom_col.index("pom_delno")]:
            return "Order Already Received"
        self.ordno = CCD(w, "UI", 9.0).work
        self.othno = CCD(w, "Na", 9.0).work
        self.loc = strpom[self.sql.strpom_col.index("pom_loc")]
        self.acno = strpom[self.sql.strpom_col.index("pom_acno")]
        self.cnam = strpom[self.sql.strpom_col.index("pom_contact")]
        self.vatn = strpom[self.sql.strpom_col.index("pom_vatno")]
        self.ordby = strpom[self.sql.strpom_col.index("pom_ordby")]
        self.ribbon = ["", "", self.cnam, self.vatn]
        self.df.loadEntry(frt, pag, p+1, data=self.loc)
        self.doLoc(frt, pag, r, c+1, p+1, i+1, self.loc)
        self.df.loadEntry(frt, pag, p+3, data=self.acno)
        self.doCrsAcc(frt, pag, r, c+3, p+3, i+3, self.acno)
        self.df.loadEntry(frt, pag, p+5, data=self.ordby)
        self.doOrdBy(frt, pag, r, c+5, p+5, i+5, self.ordby)
        if self.acttyp == "R":
            return "sk6"
        else:
            return "nd"

    def doLoc(self, frt, pag, r, c, p, i, w):
        desc = self.sql.getRec(tables="strloc", cols=["srl_desc"],
            where=[("srl_cono", "=", self.conum), ("srl_loc", "=", w)],
            limit=1)
        if not desc:
            return "Invalid Location"
        self.loc = w
        self.df.loadEntry(frt, pag, p+1, data=desc[0])

    def doCrsAcc(self, frt, pag, r, c, p, i, w):
        self.crsmst = self.sql.getRec(tables="crsmst", where=[("crm_cono",
            "=", self.conum), ("crm_acno", "=", w)], limit=1)
        if not self.crsmst:
            return "Invalid Account"
        self.acno = w
        self.name = self.crsmst[self.sql.crsmst_col.index("crm_name")]
        self.df.loadEntry(frt, pag, p+1, data=self.name)

    def doOrdBy(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="ctlrep", cols=["rpm_name"],
            where=[("rpm_cono", "=", self.conum), ("rpm_rep", "=", w)],
            limit=1)
        if not acc:
            return "Invalid Code"
        self.ordby = w
        self.df.loadEntry(frt, pag, p+1, data=acc[0])
        return "sk3"

    def doDelNum(self, frt, pag, r, c, p, i, w):
        self.delno = w

    def doDelDate(self, frt, pag, r, c, p, i, w):
        self.deldt = w

    def doTopEnd(self):
        if self.acttyp == "N":
            # Create Document Transaction (Head)
            self.dad1, self.dad2, self.dad3, self.dad4 = "", "", "", ""
            data = [self.conum, self.ordno, self.loc, self.trdt, self.acno,
                self.dad1, self.dad2, self.dad3, self.dad4, "", "", "", "",
                self.ordby, "", "", "", "", "", self.sysdtw, "", 0]
            # Write and Commit Header
            self.sql.insRec("strpom", data=data)
            self.mf.dbm.commitDbase()
            self.doClearTots()
        elif self.acttyp == "C":
            # Print Deletetion
            PrintOrder(self.mf, self.conum, self.ordno, prtnam=self.prtnam,
                email=self.email, copy="d", commit=False)
            # Update Deletetion
            self.sql.updRec("strpom", cols=["pom_delno"], data=["cancel"],
                where=[("pom_cono", "=", self.conum), ("pom_ordno", "=",
                self.ordno)])
            self.sql.delRec("strpot", where=[("pot_cono", "=", self.conum),
                ("pot_ordno", "=", self.ordno)])
            # COMMIT Deletetion
            self.mf.dbm.commitDbase()
            self.doClearTots()
            self.df.focusField("T", 0, 1)
        else:
            self.doReadLoadStr()
            self.amend = False

    def doReadLoadStr(self):
        self.doClearTots()
        data = self.sql.getRec(tables="strpot", where=[("pot_cono", "=",
            self.conum), ("pot_ordno", "=", self.ordno)])
        if not data:
            return
        self.actord = True
        for seq, line in enumerate(data):
            self.doExtData(line)
            if seq >= self.row[0]:
                pos = (self.row[0] - 1) * 8
            else:
                pos = seq * 8
            # Load Values
            self.df.doInvoke(["C", 0, (pos+1), "rt"], self.grp)
            self.df.doInvoke(["C", 0, (pos+2), "rt"], self.code)
            self.df.doInvoke(["C", 0, (pos+3), "rt"], self.desc)
            self.df.doInvoke(["C", 0, (pos+4), "rt"], self.qty)
            self.df.doInvoke(["C", 0, (pos+5), "rt"], self.vatcod)
            self.df.doInvoke(["C", 0, (pos+6), "rt"], self.price)
            self.df.doInvoke(["C", 0, (pos+7), self.doDis], self.disrat)
        self.actord = False

    def doTopExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

    def doClearTots(self):
        self.vattot = 0
        self.ordtot = 0
        self.df.clearFrame("C", 0)
        self.df.focusField("C", 0, 1)

    def doGrp(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="strgrp", where=[("gpm_cono", "=",
            self.conum), ("gpm_group", "=", w)], limit=1)
        if not acc:
            return "Invalid Group Code"
        self.grp = w
        self.df.colf[pag][1][8]["where"] = [("st1_cono", "=", self.conum),
            ("st1_group", "=", self.grp), ("st1_type", "not", "in", ("J", "R")),
            ("st1_value_ind", "<>", "N")]

    def doCod(self, frt, pag, r, c, p, i, w):
        self.strmf1 = self.sql.getRec(tables="strmf1", where=[("st1_cono", "=",
            self.conum), ("st1_group", "=", self.grp), ("st1_code", "=", w)],
            limit=1)
        if not self.strmf1:
            return "Invalid Code"
        if self.strmf1[self.sql.strmf1_col.index("st1_type")] in ("J", "R"):
            return "Invalid Code (Group Item)"
        if self.strmf1[self.sql.strmf1_col.index("st1_value_ind")] == "N":
            return "Invalid Code (Value Indicator)"
        self.strmf2 = self.sql.getRec(tables="strmf2", where=[("st2_cono", "=",
            self.conum), ("st2_group", "=", self.grp), ("st2_code", "=", w),
            ("st2_loc", "=", self.loc)], limit=1)
        if not self.strmf2:
            return "Invalid Location"
        ord = self.sql.getRec(tables="strpot", cols=["count(*)"],
            where=[("pot_cono", "=", self.conum), ("pot_ordno", "=",
            self.ordno), ("pot_group", "=", self.grp), ("pot_code", "=", w)],
            limit=1)
        if ord[0]:
            return "Item Already On Order"
        self.code = w
        desc = self.strmf1[self.sql.strmf1_col.index("st1_desc")]
        self.df.loadEntry(frt, pag, p+1, data=desc)

    def doDes(self, frt, pag, r, c, p, i, w):
        self.desc = w

    def doQty(self, frt, pag, r, c, p, i, w):
        self.qty = w
        bal = Balances(self.mf, "STR", self.conum, self.curdt,
            (self.grp, self.code, self.loc, ("P", self.period[0])))
        bals = bal.doStrBals()
        qty = CCD(bals[5][0], "SD", 10.2)

    def doVat(self, frt, pag, r, c, p, i, w):
        vat = self.sql.getRec(tables="ctlvmf", cols=["vtm_new"],
            where=[("vtm_cono", "=", self.conum), ("vtm_code", "=", w)],
            limit=1)
        if not vat:
            return "Invalid V.A.T Code"
        self.vatcod = w
        self.vatrat = vat[0]
        # Last Cost Price
        bal = Balances(self.mf, "STR", self.conum, self.curdt,
            (self.grp, self.code, self.loc, ("P", self.period[0])))
        bals = bal.doStrBals()
        self.df.loadEntry(frt, pag, p+1, data=bals[7])

    def doPri(self, frt, pag, r, c, p, i, w):
        self.price = w
        unt = self.strmf1[self.sql.strmf1_col.index("st1_units")]
        if not unt:
            pri = self.price
        else:
            pri = round((self.price / unt), 2)
        self.inc = round((pri * self.vatrat / 100), 4)
        self.exc = round((pri * 1), 2)
        dis = self.crsmst[self.sql.crsmst_col.index("crm_trdis")]
        self.df.loadEntry(frt, pag, p+1, data=dis)

    def doDis(self, frt, pag, r, c, p, i, w):
        if self.actord:
            pass
        else:
            self.disrat = w
            self.uoi = self.strmf1[self.sql.strmf1_col.index("st1_uoi")]
            self.unt = self.strmf1[self.sql.strmf1_col.index("st1_units")]
            dis = float(ASD(100) - ASD(self.disrat))
            self.excamt = round((self.qty * self.exc * dis / 100), 2)
            self.incamt = round((self.qty * self.inc * dis / 100), 2)
            self.vatamt = float(ASD(self.incamt) - ASD(self.excamt))
        self.df.loadEntry(frt, pag, p+1, data=self.excamt)
        self.vattot = float(ASD(self.vattot) + ASD(self.vatamt))
        self.ordtot = float(ASD(self.ordtot) + ASD(self.excamt))

    def doColEnd(self):
        if self.actord:
            pass
        else:
            # Create Purchase Order Transaction (Body)
            data = [self.conum, self.ordno, self.grp, self.code, self.uoi,
                self.unt, self.desc, self.qty, self.price, self.disrat,
                self.vatcod, self.vatrat, self.sysdtw, 0]
            self.sql.insRec("strpot", data=data)
            if self.acttyp == "A":
                self.amend = True
        self.df.advanceLine(0)

    def doQryCrs(self, widget=None):
        callModule(self.mf, self.df, "cr4010", coy=(self.conum, self.conam),
            period=None, user=self.capnm)

    def doQryStk(self, widget=None):
        callModule(self.mf, self.df, "st4010", coy=(self.conum, self.conam),
            period=self.period, user=self.capnm)

    def doDelAdd(self, widget=None):
        tit = ("Delivery Address",)
        fld = (
            (("T",0,0,0),"INA",30,"Address-1","Address Line 1",
                "","N",None,None,None,None),
            (("T",0,1,0),"INA",30,"Address-2","Address Line 2",
                "","N",None,None,None,None),
            (("T",0,2,0),"INA",30,"Address-3","Address Line 3",
                "","N",None,None,None,None),
            (("T",0,3,0),"INA",30,"Address-4","Address Line 4",
                "","N",None,None,None,None))
        state = self.df.disableButtons()
        self.tx = TartanDialog(self.mf, title=tit, newscn=(0, 0), tops=True,
            eflds=fld, tend=((self.doDelEnd,"n"),), txit=(self.doDelExit,),
            focus=False)
        for x in range(0, len(self.deladd)):
            self.tx.loadEntry("T", 0, 0+x, data=self.deladd[x])
        self.tx.focusField("T", 0, 1, clr=False)
        self.mf.startLoop()
        self.df.enableButtons(state=state)
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def doDelEnd(self):
        self.deladd = []
        for x in range(0, self.tx.topq[0]):
            self.deladd.append(self.tx.t_work[0][0][x])
        self.doDelExit()

    def doDelExit(self):
        self.tx.closeProcess()
        self.mf.closeLoop()

    def doMessag(self, widget=None):
        tit = ("Order Message",)
        cod = {
            "stype": "R",
            "tables": ("ctlmes",),
            "cols": [
                ("mss_message", "",  0, "Cod"),
                ("mss_detail",  "", 60, "Details")],
            "where": [("mss_system", "=", "STR")],
            "screen": self.df.mstFrame}
        fld = (
            (("T",0,0,0),"IUI",3,"Message Code","Message Code",
                "","N",self.doMesCod,cod,None,None),
            (("T",0,1,0),"INA",30,"Line-1","Message Line 1",
                "","N",None,None,None,None),
            (("T",0,2,0),"INA",30,"Line-2","Message Line 2",
                "","N",None,None,None,None),
            (("T",0,3,0),"INA",30,"Line-3","Message Line 3",
                "","N",None,None,None,None),
            (("T",0,4,0),"INA",30,"Line-4","Message Line 4",
                "","N",None,None,None,None),
            (("T",0,5,0),"INA",30,"Line-5","Message Line 5",
                "","N",None,None,None,None))
        state = self.df.disableButtons()
        self.tx = TartanDialog(self.mf, title=tit, newscn=(0, 0), tops=True,
            eflds=fld, tend=((self.doMesEnd,"n"),), txit=(self.doMesExit,),
            focus=False)
        if self.ctlmes:
            self.doLoadMes("y")
        else:
            self.tx.focusField("T", 0, 1)
        self.mf.startLoop()
        self.df.enableButtons(state=state)
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def doMesCod(self, frt, pag, r, c, p, i, w):
        if w:
            self.ctlmes = self.sql.getRec(tables="ctlmes",
                where=[("mss_system", "=", "STR"), ("mss_message", "=", w)],
                limit=1)
            if not self.ctlmes:
                return "Invalid Message Code"
            self.doLoadMes()
            return "nd"

    def doLoadMes(self, focus=False):
        self.tx.loadEntry("T", 0, 0, data=self.ctlmes[1])
        fmt = textFormat(self.ctlmes[2], width=30)
        if len(fmt) > 5:
            l = 5
        else:
            l = len(fmt)
        for lin in range(0, l):
            self.tx.loadEntry("T", 0, 1+lin, data=fmt[lin])
        if focus:
            self.tx.focusField("T", 0, 2, clr=False)

    def doMesEnd(self):
        self.ctlmes = ["STR", self.tx.t_work[0][0][0]]
        mess = ""
        for x in range(1, self.tx.topq[0]):
            mess = mess + self.tx.t_work[0][0][x] + " "
        words = mess.split(" ")
        mess = ""
        for word in words:
            if word:
                mess = mess + word + " "
        self.ctlmes.append(mess[:-1])
        self.doMesExit()

    def doMesExit(self):
        self.tx.closeProcess()
        self.mf.closeLoop()

    def doEdit(self, widget=None):
        all = self.sql.getRec(tables="strpot", where=[("pot_cono", "=",
            self.conum), ("pot_ordno", "=", self.ordno)])
        if all:
            data = []
            for l in all:
                qty = CCD(l[self.sql.strpot_col.index("pot_qty")], "SD", 10.2)
                prc = CCD(l[self.sql.strpot_col.index("pot_price")], "UD", 9.2)
                dis = CCD(l[self.sql.strpot_col.index("pot_disper")], "SD", 6.2)
                data.append([
                    l[self.sql.strpot_col.index("pot_group")],
                    l[self.sql.strpot_col.index("pot_code")],
                    l[self.sql.strpot_col.index("pot_desc")],
                    qty.disp,
                    l[self.sql.strpot_col.index("pot_vatcod")],
                    prc.disp, dis.disp])
            head = ("Grp", "Product-Code", "Description", " Quantity",
                "V", "    Price", "Disc-%")
            if self.acttyp == "R":
                rtn = "R"
            else:
                rtn = "P"
            lin = {
                "stype": "C",
                "head": head,
                "data": data}
            state = self.df.disableButtons()
            chg = self.df.selChoice(lin)
            if chg and chg.selection:
                self.change = chg.selection
                self.doChanges()
            self.df.enableButtons(state=state)
        self.df.focusField("C", 0, self.df.col)

    def doChanges(self):
        tit = ("Change Items",)
        fld = (
            (("T",0,1,0),"ONA",3,"Group"),
            (("T",0,2,0),"ONA",20,"Code"),
            (("T",0,3,0),"INA",30,"Description","Description",
                "","N",self.doChgDes,None,None,("notblank",)),
            (("T",0,4,0),"ISD",10.2,"Quantity","Quantity",
                "","N",self.doChgQty,None,None,("notzero",)),
            (("T",0,5,0),"ISD",9.2,"Selling Price","Selling Price",
                "","N",self.doChgPrc,None,None,("notzero",)),
            (("T",0,6,0),"IUD",6.2,"Discount Percent","Discount Percent",
                "","N",self.doChgDis,None,None,("efld",)))
        but = (("Delete",None,self.doChgDel,1,None,None),)
        tnd=((self.doChgEnd,"n"),)
        txt=(self.doChgExit,)
        self.cg = TartanDialog(self.mf, title=tit, newscn=(0, 0), tops=True,
            eflds=fld, butt=but, tend=tnd, txit=txt)
        self.cg.loadEntry("T", 0, 0, data=self.change[0])
        self.cg.loadEntry("T", 0, 1, data=self.change[1])
        self.cg.loadEntry("T", 0, 2, data=self.change[2])
        self.cg.loadEntry("T", 0, 3, data=self.change[3])
        self.cg.loadEntry("T", 0, 4, data=self.change[5])
        self.cg.loadEntry("T", 0, 5, data=self.change[6])
        self.cg.focusField("T", 0, 3, clr=False)
        self.mf.startLoop()

    def doChgDes(self, frt, pag, r, c, p, i, w):
        self.desc = w

    def doChgQty(self, frt, pag, r, c, p, i, w):
        self.qty = w

    def doChgPrc(self, frt, pag, r, c, p, i, w):
        self.price = w

    def doChgDis(self, frt, pag, r, c, p, i, w):
        self.disrat = w

    def doChgDel(self, widget=None):
        grp = self.change[0]
        cod = self.change[1]
        self.sql.delRec("strpot", where=[("pot_cono", "=", self.conum),
            ("pot_ordno", "=", self.ordno), ("pot_group", "=", grp),
            ("pot_code", "=", cod)])
        if self.acttyp == "A":
            self.amend = True
        self.doReadLoadStr()
        self.doChgExit()

    def doChgEnd(self):
        grp = self.change[0]
        cod = self.change[1]
        self.sql.updRec("strpot",
            cols=["pot_desc", "pot_qty", "pot_price", "pot_disper"],
            data=[self.desc, self.qty, self.price, self.disrat],
            where=[("pot_cono", "=", self.conum), ("pot_ordno", "=",
            self.ordno), ("pot_group", "=", grp), ("pot_code", "=", cod)])
        if self.acttyp == "A":
            self.amend = True
        self.doReadLoadStr()
        self.doChgExit()

    def doChgExit(self):
        self.cg.closeProcess()
        self.mf.closeLoop()

    def doReprnt(self, widget=None):
        tit = ("Reprint Orders",)
        ord = {
            "stype": "R",
            "tables": ("strpom","crsmst"),
            "cols": [
                ("pom_ordno", "",  0, "Doc-Num"),
                ("pom_date", "",  0, "Date"),
                ("pom_acno",  "", 0, "Acc-Num"),
                ("crm_name",  "", 0, "Name", "Y")],
            "where": [("pom_cono", "=", self.conum), ("pom_cono=crm_cono",),
                ("pom_acno=crm_acno",)],
            "screen": self.mf.body}
        fld = (
            (("T",0,0,0),"IUI",9,"From Number","From Document Number",
                "","N",self.doOrd,ord,None,("notzero",)),
            [("T",0,1,0),"IUI",9,"To   Number","To Document Number",
                "","N",None,ord,None,("notzero",)])
        state = self.df.disableButtons()
        self.tx = TartanDialog(self.mf, title=tit, newscn=(0, 0),
            tops=True, eflds=fld, tend=((self.doReprntEnd,"n"),),
            txit=(self.doReprntExit,), view=("N","V"), mail=("N","N"))
        self.mf.startLoop()
        self.df.enableButtons(state=state)
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def doOrd(self, frt, pag, r, c, p, i, w):
        self.tx.topf[pag][1][5] = w

    def doReprntEnd(self):
        frm = self.tx.t_work[0][0][0]
        if not self.tx.t_work[0][0][1]:
            to = frm
        else:
            to = self.tx.t_work[0][0][1]
        prt = self.tx.repprt[0][2]
        eml = self.tx.repprt[1][1]
        ord = self.sql.getRec(tables="strpom", cols=["pom_ordno"],
            where=[("pom_cono", "=", self.conum), ("pom_ordno", ">=", frm),
            ("pom_ordno", "<=", to)])
        if ord:
            for doc in ord:
                PrintOrder(self.mf, self.conum, doc[0], prtnam=prt, email=eml,
                    copy="y")
        self.doReprntExit()

    def doReprntExit(self):
        self.tx.closeProcess()
        self.mf.closeLoop()

    def doNewCrs(self, widget=None):
        state = self.df.disableButtons()
        cf = PwdConfirm(self.mf, conum=self.conum, system="STR", code="NewCrs")
        if cf.flag != "no":
            callModule(self.mf, self.df, "cr1010", coy=(self.conum, self.conam),
                period=None, user=self.capnm)
        self.df.enableButtons(state=state)
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def doNewStr(self, widget=None):
        state = self.df.disableButtons()
        cf = PwdConfirm(self.mf, conum=self.conum, system="STR", code="NewStr")
        if cf.flag != "no":
            callModule(self.mf, self.df, "st1010", coy=(self.conum, self.conam),
                period=None, user=self.capnm)
        self.df.enableButtons(state=state)
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def doAccept(self, widget=None):
        if self.df.col == 1:
            self.doCancel()
        else:
            # Update Tables
            self.doUpdateTables()
            # COMMIT Transaction
            self.mf.dbm.commitDbase()
            # Print Document
            if self.acttyp == "N" or (self.acttyp == "A" and self.amend):
                PrintOrder(self.mf, self.conum, self.ordno, prtnam=self.prtnam,
                    email=self.email, copy=self.acttyp.lower())
            self.df.focusField("T", 0, 1)

    def doExtData(self, line):
        self.grp = line[self.sql.strpot_col.index("pot_group")]
        self.code = line[self.sql.strpot_col.index("pot_code")]
        self.uoi = line[self.sql.strpot_col.index("pot_uoi")]
        self.unt = line[self.sql.strpot_col.index("pot_units")]
        self.desc = line[self.sql.strpot_col.index("pot_desc")]
        self.qty = line[self.sql.strpot_col.index("pot_qty")]
        self.price = line[self.sql.strpot_col.index("pot_price")]
        self.disrat = line[self.sql.strpot_col.index("pot_disper")]
        self.vatcod = line[self.sql.strpot_col.index("pot_vatcod")]
        self.vatrat = line[self.sql.strpot_col.index("pot_vatrat")]
        # Calculate Values
        if not self.unt:
            pri = self.price
        else:
            pri = round((self.price / self.unt), 2)
        rat = float(ASD(100) + ASD(self.vatrat))
        inc = round((pri * rat / 100), 4)
        exc = round((pri * 1), 2)
        dis = float(ASD(100) - ASD(self.disrat))
        self.excamt = round((self.qty * exc * dis / 100), 2)
        self.incamt = round((self.qty * inc * dis / 100), 2)
        self.vatamt = float(ASD(self.incamt) - ASD(self.excamt))
        if self.mkup == "Y":
            udis = round((self.price * self.disrat / 100), 2)
            self.ucost = float(ASD(self.price) - ASD(udis))

    def doRibbon(self, widget=None):
        tit = ("Ribbon Line",)
        fld = (
            (("T",0,0,0),"INA",30,"Customer Order","Customer Order",
                self.cnam,"N",None,None,None,None),
            (("T",0,1,0),"INA",30,"Job Number","Job Number",
                self.cnam,"N",None,None,None,None),
            (("T",0,2,0),"INA",30,"Contact Person","Contact Person",
                self.cnam,"N",None,None,None,None),
            (("T",0,3,0),"INA",10,"VAT Number","VAT Number",
                self.vatn,"N",self.doVatNum,None,None,None))
        state = self.df.disableButtons()
        self.tx = TartanDialog(self.mf, title=tit, newscn=(0, 0), tops=True,
            eflds=fld, tend=((self.doRibEnd,"n"),), txit=(self.doRibExit,),
            focus=False)
        if self.ribbon:
            self.tx.loadEntry("T", 0, 0, data=self.ribbon[0])
            self.tx.loadEntry("T", 0, 1, data=self.ribbon[1])
            self.tx.loadEntry("T", 0, 2, data=self.ribbon[2])
            self.tx.loadEntry("T", 0, 3, data=self.ribbon[3])
        self.tx.focusField("T", 0, 1)
        self.mf.startLoop()
        self.df.enableButtons(state=state)
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def doVatNum(self, frt, pag, r, c, p, i, w):
        if self.ordtot >= 1000.00 and not w:
            return "Invalid VAT Number"

    def doRibEnd(self):
        self.ribbon = []
        for x in range(0, self.tx.topq[0]):
            self.ribbon.append(self.tx.t_work[0][0][x])
        self.doRibExit()

    def doRibExit(self):
        self.tx.closeProcess()
        self.mf.closeLoop()

    def doUpdateTables(self):
        # Update strpom with ribbon, delivery and message details
        whr = [("pom_cono", "=", self.conum), ("pom_ordno", "=", self.ordno),
            ("pom_acno", "=", self.acno)]
        if self.ribbon:
            self.sql.updRec("strpom", cols=["pom_cusord", "pom_jobnum",
                "pom_contact", "pom_vatno"], data=[self.ribbon[0],
                self.ribbon[1], self.ribbon[2], self.ribbon[3]], where=whr)
            if not self.vatn:
                self.sql.updRec("crsmst", cols=["crm_vatno"],
                    data=[self.ribbon[1]], where=[("crm_cono", "=", self.conum),
                    ("crm_acno", "=", self.acno)])
        if self.deladd:
            self.sql.updRec("strpom", cols=["pom_add1", "pom_add2", "pom_add3",
                "pom_add4"], data=[self.deladd[0], self.deladd[1],
                self.deladd[2], self.deladd[3]], where=whr)
        if self.ctlmes:
            fmt = textFormat(self.ctlmes[2], width=30)
            for m in range(len(fmt), 5):
                fmt.append("")
            self.sql.updRec("strpom", cols=["pom_mes1", "pom_mes2", "pom_mes3",
                "pom_mes4", "pom_mes5"], data=[fmt[0], fmt[1], fmt[2], fmt[3],
                fmt[4]], where=whr)
        if self.acttyp != "R":
            return
        # Receipt of Order
        # Create Stores Transactions and update strpot lines
        trn = self.sql.getRec(tables="strpot", where=[("pot_cono", "=",
            self.conum), ("pot_ordno", "=", self.ordno)])
        if not trn:
            return
        for line in trn:
            self.doExtData(line)
            curdt = self.deldt / 100
            # Update strpom record
            self.sql.updRec("strpom", cols=["pom_delno", "pom_deldt"],
                data=[self.delno, self.deldt], where=[("pom_cono", "=",
                self.conum), ("pom_ordno", "=", self.ordno)])
            # Write strtrn record
            cst = self.excamt
            data = [self.conum, self.grp, self.code, self.loc, self.deldt, 1,
                self.othno, self.batch, self.delno, self.qty, cst, 0, curdt,
                self.name, 0, self.acno, 0, "STR", self.disrat, 0, 0, "",
                self.sysdtw, 0]
            self.sql.insRec("strtrn", data=data)
            if self.mkup == "Y":
                # Create or Update Price Records
                gpm = self.sql.getRec(tables="strgrp", cols=["gpm_markup"],
                    where=[("gpm_cono", "=", self.conum), ("gpm_group", "=",
                    self.grp)], limit=1)
                mkp = gpm[0]
                st2 = self.sql.getRec(tables="strmf2", cols=["st2_markup"],
                    where=[("st2_cono", "=", self.conum), ("st2_group", "=",
                    self.grp), ("st2_code", "=", self.code), ("st2_loc", "=",
                    self.loc)], limit=1)
                if st2[0]:
                    mkp = st2[0]
                if mkp:
                    sell = round((self.ucost * (100 + mkp) / 100.0), 2)
                    stp = self.sql.getRec(tables="strprc", where=[("stp_cono",
                        "=", self.conum), ("stp_group", "=", self.grp),
                        ("stp_code", "=", self.code), ("stp_loc", "=",
                        self.loc), ("stp_level", "=", 1)], limit=1)
                    if stp:
                        self.sql.updRec("strprc", cols=["stp_price"],
                            data=[sell], where=[("stp_cono", "=", self.conum),
                            ("stp_group", "=", self.grp), ("stp_code", "=",
                            self.code), ("stp_loc", "=", self.loc),
                            ("stp_level", "=", 1)])
                    else:
                        self.sql.insRec("strprc", data=[self.conum, self.grp,
                            self.code, self.loc, 1, sell])
            # If Integrated Create GL Transaction (SOH and Recon Accounts)
            if self.stgl == "Y" and cst:
                # Stock on Hand Control
                data = (self.conum, self.soh, curdt, self.deldt, 5,
                    self.othno, self.batch, cst, 0, self.name, "",
                    "", 0, self.sysdtw, 0)
                self.sql.insRec("gentrn", data=data)
                cst = float(ASD(0) - ASD(cst))
                # Stock Reconciliation Control
                data = (self.conum, self.ssp, curdt, self.deldt, 5,
                    self.othno, self.batch, cst, 0, self.name, "",
                    "", 0, self.sysdtw, 0)
                self.sql.insRec("gentrn", data=data)

    def doCancel(self, widget=None):
        self.mf.dbm.rollbackDbase()
        if self.acttyp == "N":
            self.sql.updRec("strpom", cols=["pom_delno"], data=["cancel"],
                where=[("pom_cono", "=", self.conum), ("pom_ordno", "=",
                self.ordno), ("pom_acno", "=", self.acno)])
            self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

