#! /usr/bin/python

"""
SYNOPSIS
    Stores Goods Received Notes.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, GetCtl, RepPrt, Sql, TartanDialog
from tartanFunctions import callModule, showPrinters

class st2020:
    def __init__(self, mf, coy, nam, period, usr):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.period = period
        self.capnm = usr
        no = self.setVariables()
        if not no:
            self.setPrinter()
            if self.prtnam != "cancel":
                self.dataHeader()
                self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["gentrn", "strgrp", "strmf1",
            "strmf2", "strtrn", "strpom", "strprc"], prog=__name__)
        if self.sql.error:
            return self.sql.error
        gc = GetCtl(self.mf)
        ctlsys = gc.getCtl("ctlsys")
        if ctlsys and ctlsys["sys_smtp"]:
            self.email = "Y"
        else:
            self.email = "N"
        strctl = gc.getCtl("strctl", self.conum)
        if not strctl:
            return "no"
        self.glint = strctl["cts_glint"]
        self.locs = strctl["cts_locs"]
        self.automu = strctl["cts_automu"]
        if self.glint == "Y":
            ctlctl = gc.getCtl("ctlctl", self.conum)
            if not ctlctl:
                return "no"
            ctls = ["stk_soh", "stk_susp"]
            if gc.chkRec(self.conum, ctlctl, ctls):
                return "no"
            self.stk_soh = ctlctl["stk_soh"]
            self.stk_susp = ctlctl["stk_susp"]
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        if self.locs == "N":
            self.loc = "1"

    def setPrinter(self):
        tit = ("Printer Selection",)
        self.printers = showPrinters()
        prt = {
            "stype": "C",
            "titl": "Available Printers",
            "head": ("Name","Description"),
            "data": self.printers}
        r1s = (("Print","P"),("View","V"),("None","N"))
        r2s = (("Yes","Y"),("No","N"))
        fld = [
            (("T",0,0,0),("IRB",r1s),0,"Choice","Select Choice",
                "P","N",self.doPrtChoice,None,None,None),
            (("T",0,1,0),"INA",30,"Printer","Printer Name",
                "Default","Y",self.doPrtNam,prt,None,None)]
        if self.email == "Y":
            fld.extend([
            (("T",0,2,0),("IRB",r2s),0,"E-Mail","E-Mail Document",
                "N","N",self.doPrtMail,None,None,None),
            (("T",0,3,0),"ITX",50,"Address","E-Mail Address",
                "","N",self.doPrtAddr,None,None,("email",False))])
        self.pr = TartanDialog(self.mf, screen=self.mf.body, tops=True,
            title=tit, eflds=fld, tend=((self.doPrtEnd,"y"),),
            txit=(self.doPrtExit,))
        self.mf.startLoop()

    def doPrtChoice(self, frt, pag, r, c, p, i, w):
        self.view = "N"
        if w == "P":
            return "ok"
        if w == "V":
            self.view = "Y"
            self.prtnam = "view"
        else:
            self.prtnam = "none"
        return "sk1"

    def doPrtNam(self, frt, pag, r, c, p, i, w):
        if w not in self.printers:
            return "Invalid Printer"
        self.prtnam = w

    def doPrtMail(self, frt, pag, r, c, p, i, w):
        self.email = w
        if self.email == "N":
            self.emadd = ""
            return "sk1"

    def doPrtAddr(self, frt, pag, r, c, p, i, w):
        self.emadd = w

    def doPrtEnd(self):
        self.doPrtClose()

    def doPrtExit(self):
        self.prtnam = "cancel"
        self.doPrtClose()

    def doPrtClose(self):
        self.pr.closeProcess()
        self.mf.closeLoop()

    def dataHeader(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "Goods Received Notes (%s)" % __name__)
        gpm = {
            "stype": "R",
            "tables": ("strgrp",),
            "cols": (
                ("gpm_group", "", 0, "Grp"),
                ("gpm_desc", "", 0, "Description", "Y")),
            "where": [("gpm_cono", "=", self.conum)]}
        stm = {
            "stype": "R",
            "tables": ("strmf1",),
            "cols": (
                ("st1_group", "", 0, "Grp"),
                ("st1_code", "", 0, "Product Code"),
                ("st1_desc", "", 0, "Description", "Y")),
            "where": [("st1_cono", "=", self.conum), ("st1_type", "not",
                "in", ("J", "R")), ("st1_value_ind", "<>", "N")],
            "whera": [["C", "st1_group", 0, 0]],
            "order": "st1_group, st1_code",
            "index": 1}
        stl = {
            "stype": "R",
            "tables": ("strmf2","strmf1","strloc"),
            "cols": (
                ("st2_group", "", 0, "Grp"),
                ("st2_code", "", 0, "Product Code"),
                ("st1_desc", "", 0, "Description"),
                ("st2_loc", "", 0, "L"),
                ("srl_desc", "", 0, "Location", "Y")),
            "where": [("st1_cono=st2_cono",), ("st1_group=st2_group",),
                    ("st1_code=st2_code",), ("st2_cono=srl_cono",),
                    ("st2_loc=srl_loc",), ("st1_cono", "=", self.conum)],
            "whera": [["C", "st1_group", 0, 0], ["C", "st1_code", 1, 0]],
            "order": "st2_loc",
            "index": 3}
        fld = [
            (("T",0,0,0),"INa",9,"GRN Number","GRN Number",
                "","Y",self.doGrv,None,None,("notblank",)),
            (("T",0,0,0),"ID1",10,"Date","GRN Date",
                self.sysdtw,"N",self.doDte,None,None,("efld",)),
            (("T",0,0,0),"INa",7,"Order Number","Order Number",
                "","N",self.doOrd,None,None,("efld",)),
            (("T",0,0,0),"IUA",1,"Auto Markup","Automatic Markup",
                self.automu,"N",self.doMkUp,None,None,("in",("Y","N"))),
            (("C",0,0,0),"IUA",3,"Grp","Product Group",
                "r","Y",self.doGroup,gpm,None,("notblank",)),
            (("C",0,0,1),"INA",(15,20),"Product Code","Product Code",
                "","Y",self.doCode,stm,None,("notblank",)),
            (("C",0,0,2),"ONA",15,"Description"),
            (("C",0,0,3),"ONA",5,"U.O.I"),
            [("C",0,0,4),"IUA",1,"L","Location",
                "r","Y",self.doLoc,stl,None,("notblank",)],
            (("C",0,0,5),"ISD",9.2,"Quantity","Quantity",
                "","Y",self.doQty,None,None,("notzero",)),
            (("C",0,0,6),"IUD",9.2,"Unit-Cost","Unit Cost Price",
                "","Y",self.doUcost,None,None,("notzero",)),
            (("C",0,0,7),"IUD",5.2,"Dis-%","Discount Percentage",
                "","Y",self.doDisPer,None,None,("efld",)),
            (("C",0,0,8),"OSD",9.2,"Value"),
            (("C",0,0,9),"IUD",9.2,"Unit-Sell","Unit Selling Price",
                "","Y",self.doUsell,None,None,("efld",))]
        tnd = ((self.endPage0,"n"),)
        txt = (self.exitPage0,)
        cnd = ((self.endPage1,"y"),)
        cxt = (self.exitPage1,)
        but = (
            ("Interrogate",None,self.queryStr,0,("C",0,1),("C",0,2)),
            ("Reprint",None,self.doReprint,0,("T",0,1),None),
            ("Cancel",None,self.cancelPage,0,("C",0,1),("C",0,0)))
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tend=tnd,
            txit=txt, cend=cnd, cxit=cxt, butt=but)

    def doGrv(self, frt, pag, r, c, p, i, w):
        self.grv = w
        self.cancel = False

    def doDte(self, frt, pag, r, c, p, i, w):
        if w < self.period[1][0] or w > self.period[2][0]:
            return "Invalid Date, Not in Financial Period"
        self.dte = w
        self.dtw = self.df.t_disp[0][0][i]
        self.curdt = self.dte / 100
        self.batno = "S%s" % self.curdt

    def doOrd(self, frt, pag, r, c, p, i, w):
        self.ord = w

    def doMkUp(self, frt, pag, r, c, p, i, w):
        self.mkup = w

    def doGroup(self, frt, pag, r, c, p, i, w):
        self.group = w
        acc = self.sql.getRec(tables="strgrp", cols=["gpm_markup"],
            where=[("gpm_cono", "=", self.conum), ("gpm_group", "=", w)],
            limit=1)
        if not acc:
            return "Invalid Group"
        self.mkper = acc[0]

    def doCode(self, frt, pag, r, c, p, i, w):
        self.code = w
        acc = self.sql.getRec(tables="strmf1", cols=["st1_type", "st1_desc",
            "st1_uoi", "st1_value_ind"], where=[("st1_cono", "=", self.conum),
            ("st1_group", "=", self.group), ("st1_code", "=", w)], limit=1)
        if not acc:
            return "Invalid Code"
        if acc[0] in ("J", "R"):
            return "Invalid Code (Job/Recipe Item)"
        if acc[3] == "N":
            return "Invalid Code (Value Indicator)"
        self.desc = acc[1]
        self.uoi = acc[2]
        self.df.loadEntry("C", pag, p+1, data=self.desc)
        self.df.loadEntry("C", pag, p+2, data=self.uoi)
        if self.locs == "N":
            self.loc = "1"
            self.df.loadEntry("C", pag, p+3, data=self.loc)
            no = self.checkLoc()
            if no:
                return no
            return "sk3"

    def doLoc(self, frt, pag, r, c, p, i, w):
        self.loc = w
        no = self.checkLoc()
        if no:
            return no

    def checkLoc(self):
        acc = self.sql.getRec(tables="strmf2", cols=["st2_markup"],
            where=[("st2_cono", "=", self.conum), ("st2_group", "=",
            self.group), ("st2_code", "=", self.code), ("st2_loc", "=",
            self.loc)], limit=1)
        if not acc:
            return "Invalid Location For This Product"
        if acc[0]:
            self.mkper = acc[0]

    def doQty(self, frt, pag, r, c, p, i, w):
        self.qty = w

    def doUcost(self, frt, pag, r, c, p, i, w):
        self.ucost = w
        self.tcost = round((self.qty * self.ucost), 2)

    def doDisPer(self, frt, pag, r, c, p, i, w):
        self.trdis = w
        udis = round((self.ucost * self.trdis / 100), 2)
        tdis = round((self.tcost * self.trdis / 100), 2)
        self.ucost = float(ASD(self.ucost) - ASD(udis))
        self.tcost = float(ASD(self.tcost) - ASD(tdis))
        self.df.loadEntry(frt, pag, p+1, data=self.tcost)
        self.prc = self.sql.getRec(tables="strprc", where=[("stp_cono", "=",
            self.conum), ("stp_group", "=", self.group), ("stp_code", "=",
            self.code), ("stp_loc", "=", self.loc), ("stp_level", "=", 1)],
            limit=1)
        if self.mkup == "Y" and self.mkper:
            self.usell = round((self.ucost * (100 + self.mkper) / 100.0), 2)
        elif self.prc:
            self.usell = self.prc[self.sql.strprc_col.index("stp_price")]
        else:
            self.usell = 0.00
        self.df.loadEntry(frt, pag, p+2, data=self.usell)

    def doUsell(self, frt, pag, r, c, p, i, w):
        self.usell = w

    def endPage0(self):
        self.df.focusField("C", 0, 1)

    def endPage1(self):
        self.updateTables()
        self.df.advanceLine(0)

    def exitPage0(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

    def exitPage1(self):
        if not self.cancel and self.df.col != 1:
            self.mf.dbm.commitDbase()
            self.doPrintDoc(self.view, self.prtnam, self.emadd)
        else:
            self.mf.dbm.rollbackDbase()
        self.df.focusField("T", 0, 1)

    def cancelPage(self, widget=None):
        self.cancel = True
        self.exitPage1()

    def updateTables(self):
        # Stores Ledger Transaction
        data = [self.conum, self.group, self.code, self.loc, self.dte,
            1, self.grv, self.batno, self.ord, self.qty, self.tcost, 0,
            self.curdt, self.desc, 0, "", "", "STR", self.trdis, 0, 0,
            "N", self.sysdtw, 0]
        self.sql.insRec("strtrn", data=data)
        if self.usell:
            if self.prc:
                self.sql.updRec("strprc", cols=["stp_price"], data=[self.usell],
                    where=[("stp_cono", "=", self.conum), ("stp_group", "=",
                    self.group), ("stp_code", "=", self.code), ("stp_loc", "=",
                    self.loc), ("stp_level", "=", 1)])
            else:
                self.sql.insRec("strprc", data=[self.conum, self.group,
                    self.code, self.loc, 1, self.usell])
        if self.glint == "N":
            return
        # General Ledger Control Transaction (Stock On Hand)
        data = [self.conum, self.stk_soh, self.curdt, self.dte, 4, self.grv,
            self.batno, self.tcost, 0, self.code, "N" ,"" ,0, self.sysdtw, 0]
        self.sql.insRec("gentrn", data=data)
        # General Ledger Control Transaction (Stock Suspense)
        val = float(ASD(0) - ASD(self.tcost))
        data = [self.conum, self.stk_susp, self.curdt, self.dte, 4, self.grv,
            self.batno, val, 0, self.code, "N", "", 0, self.sysdtw, 0]
        self.sql.insRec("gentrn", data=data)

    def doReprint(self, widget=None):
        tit = ("Reprint Documents",)
        dat = self.sql.getRec(tables="strtrn", cols=["stt_loc", "stt_trdt",
            "stt_ref1", "stt_ref2"], where=[("stt_cono", "=", self.conum),
            ("stt_type", "=", 1)], group="stt_loc, stt_trdt, stt_ref1, "\
            "stt_ref2", order="stt_trdt, stt_ref1")
        data = []
        for d in dat:
            try:
                doc = int(d[2])
                chk = self.sql.getRec(tables="strpom", where=[("pom_cono", "=",
                    self.conum), ("pom_ordno", "=", doc), ("pom_loc", "=",
                    d[0]), ("pom_date", "=", d[1])])
                if chk:
                    continue
            except:
                pass
            data.append([d[2], d[1], d[3]])
        grv = {
            "stype": "R",
            "tables": ("strtrn", "strpom"),
            "cols": (
                ("stt_ref1", "", 0, "Reference"),
                ("stt_trdt", "", 0, "Date", "Y"),
                ("stt_ref2", "", 0, "Order-Num")),
            "wtype": "D",
            "where": data,
            "screen": self.mf.body,
            "comnd": self.doSelRec}
        grd = {
            "stype": "R",
            "tables": ("strtrn",),
            "cols": (
                ("stt_trdt", "", 0, "Date"),
                ("stt_ref2", "", 0, "Order-Num")),
            "where": [("stt_cono", "=", self.conum), ("stt_type", "=", 1)],
            "whera": (("T", "stt_ref1", 0, 0),),
            "group": "stt_trdt",
            "order": "stt_trdt"}
        fld = (
            (("T",0,0,0),"INa",9,"GRN Number","GRN Number",
                "","N",self.doRepGrv,grv,None,("notblank",)),
            (("T",0,1,0),"ID1",10,"Date","GRN Date",
                0,"N",self.doRepDte,grd,None,("notzero",)))
        state = self.df.disableButtons()
        self.tx = TartanDialog(self.mf, title=tit, newscn=(0,1),
            tops=True, eflds=fld, tend=((self.doRepEnd, "n"),),
            txit=(self.doRepExit,), view=("N","V"), mail=("Y","N"))
        self.mf.startLoop()
        self.df.enableButtons(state=state)
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def doSelRec(self, frt, pag, r, c, p, i, w):
        self.tx.doKeyPressed(frt, pag, p, w[0])
        self.tx.doKeyPressed(frt, pag, p+1, w[1])

    def doRepGrv(self, frt, pag, r, c, p, i, w):
        self.grv = w

    def doRepDte(self, frt, pag, r, c, p, i, w):
        self.dte = w
        self.dtw = self.tx.t_disp[0][0][i]
        acc = self.sql.getRec(tables="strtrn", cols=["count(*)"],
            where=[("stt_cono", "=", self.conum), ("stt_type", "=", 1),
            ("stt_ref1", "=", self.grv), ("stt_trdt", "=", self.dte)], limit=1)
        if not acc[0]:
            return "No Document Found"

    def doRepEnd(self):
        self.tx.closeProcess()
        self.doPrintDoc(self.tx.repprt[0][1], self.tx.repprt[0][2],
            self.tx.repprt[1][2])
        self.mf.closeLoop()

    def doRepExit(self):
        self.tx.closeProcess()
        self.mf.closeLoop()

    def doPrintDoc(self, view, prtnam, emadd):
        hds = ["Goods Received Notes",
            "GRN Number: %s GRN Date: %s" % (self.grv, self.dtw)]
        col = [
            ["stt_group", "UA", 3, "Grp", "y"],
            ["stt_code", "NA", 20, "Product-Code", "y"],
            ["stt_desc", "NA", 40, "Description", "y"],
            ["stt_loc", "UA", 1, "L", "y"],
            ["stt_qty", "SD", 11.2, "Quantity", "y"],
            ["stt_cost", "SD", 11.2, "Price", "y"]]
        gtt = ["stt_cost"]
        whr = [("stt_cono", "=", self.conum), ("stt_type", "=", 1),
            ("stt_ref1", "=", self.grv), ("stt_trdt", "=", self.dte)]
        ord = "stt_seq"
        self.df.mstAlw.hide()
        rpt = RepPrt(self.mf, conum=self.conum, conam=self.conam,
            name=__name__, tables=["strtrn"], heads=hds, cols=col, gtots=gtt,
            where=whr, order=ord, view=view, prtnam=prtnam, mail=emadd)
        self.df.mstAlw.show()

    def queryStr(self, widget=None):
        callModule(self.mf, self.df, "st4010", coy=(self.conum, self.conam),
            period=self.period, user=self.capnm)
