#! /usr/bin/python

"""
SYNOPSIS
    Stores Issues.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, Balances, GetCtl, PwdConfirm, RepPrt, Sql
from TartanClasses import TartanDialog
from tartanFunctions import chkCtlAcc, showError, showPrinters

class st2030:
    def __init__(self, mf, coy, nam, period):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.period = period
        no = self.setVariables()
        if not no:
            self.setPrinter()
            if self.prtnam != "cancel":
                self.dataHeader()
                self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["genmst", "gentrn", "strgrp", "strmf1",
            "strmf2", "strtrn", "strrcp"], prog=__name__)
        if self.sql.error:
            return self.sql.error
        gc = GetCtl(self.mf)
        strctl = gc.getCtl("strctl", self.conum)
        if not strctl:
            return "no"
        self.glint = strctl["cts_glint"]
        self.locs = strctl["cts_locs"]
        if self.glint == "Y":
            ctlctl = gc.getCtl("ctlctl", self.conum)
            if not ctlctl:
                return "no"
            ctls = ["stk_soh", "stk_susp"]
            if gc.chkRec(self.conum, ctlctl, ctls):
                return "no"
            self.stk_soh = ctlctl["stk_soh"]
            self.stk_susp = ctlctl["stk_susp"]
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        if self.locs == "N":
            self.loc = "1"
        self.glnum = 0

    def setPrinter(self):
        tit = ("Printer Selection",)
        self.printers = showPrinters()
        prt = {
            "stype": "C",
            "titl": "Available Printers",
            "head": ("Name","Description"),
            "data": self.printers}
        r1s = (("Print","P"),("View","V"),("None","N"))
        r2s = (("Yes","Y"),("No","N"))
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"Choice","Select Choice",
                "P","N",self.doPrtChoice,None,None,None),
            (("T",0,1,0),"INA",30,"Printer","Printer Name",
                "Default","Y",self.doPrtNam,prt,None,None))
        self.pr = TartanDialog(self.mf, screen=self.mf.body, tops=True,
            title=tit, eflds=fld, tend=((self.doPrtEnd,"y"),),
            txit=(self.doPrtExit,))
        self.mf.startLoop()

    def doPrtChoice(self, frt, pag, r, c, p, i, w):
        self.prtvwr = w
        if w == "P":
            return "ok"
        elif w == "V":
            self.prtnam = "view"
        else:
            self.prtnam = "none"
        return "sk1"

    def doPrtNam(self, frt, pag, r, c, p, i, w):
        if w not in self.printers:
            return "Invalid Printer"
        self.prtnam = w

    def doPrtEnd(self):
        self.doPrtClose()

    def doPrtExit(self):
        self.prtnam = "cancel"
        self.doPrtClose()

    def doPrtClose(self):
        self.pr.closeProcess()
        self.mf.closeLoop()

    def dataHeader(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "Stores Issues (%s)" % __name__)
        gpm = {
            "stype": "R",
            "tables": ("strgrp",),
            "cols": (
                ("gpm_group", "", 0, "Grp"),
                ("gpm_desc", "", 0, "Description", "Y")),
            "where": [("gpm_cono", "=", self.conum)]}
        stm = {
            "stype": "R",
            "tables": ("strmf1",),
            "cols": (
                ("st1_group", "", 0, "Grp"),
                ("st1_code", "", 0, "Product Code"),
                ("st1_type", "", 0, "T"),
                ("st1_desc", "", 0, "Description", "Y")),
            "where": [("st1_cono", "=", self.conum)],
            "whera": [["C", "st1_group", 0, 0]],
            "order": "st1_group, st1_code",
            "index": 1}
        stl = {
            "stype": "R",
            "tables": ("strmf2","strmf1","strloc"),
            "cols": (
                ("st2_group", "", 0, "Grp"),
                ("st2_code", "", 0, "Product Code"),
                ("st1_desc", "", 0, "Description"),
                ("st2_loc", "", 0, "L"),
                ("srl_desc", "", 0, "Location", "Y")),
            "where": [("st1_cono=st2_cono",), ("st1_group=st2_group",),
                    ("st1_code=st2_code",), ("st2_cono=srl_cono",),
                    ("st2_loc=srl_loc",), ("st1_cono", "=", self.conum)],
            "whera": [["C", "st1_group", 0, 0], ["C", "st1_code", 1, 0]],
            "order": "st2_loc",
            "index": 3}
        glm = {
            "stype": "R",
            "tables": ("genmst",),
            "cols": (
                ("glm_acno", "", 0, "Acc-Num"),
                ("glm_desc", "", 0, "Description", "Y")),
            "where": [("glm_cono", "=", self.conum)]}
        fld = [
            (("T",0,0,0),"INa",9,"Issue Number","Issue Number",
                "","N",self.doIss,None,None,("notblank",)),
            (("T",0,0,0),"ID1",10,"Date","Issue Date",
                self.sysdtw,"N",self.doDte,None,None,("efld",)),
            (("C",0,0,0),"IUA",3,"Grp","Product Group",
                "r","N",self.doGroup,gpm,None,("notblank",)),
            [("C",0,0,1),"INA",20,"Product Code","Product Code",
                "","N",self.doCode,stm,None,("notblank",)],
            (("C",0,0,2),"ONA",5,"U.O.I"),
            (("C",0,0,3),"IUA",1,"L","Location",
                "r","N",self.doLoc,stl,None,("notblank",)),
            (("C",0,0,4),"ISD",9.2,"Quantity","Quantity",
                "","N",self.doQty,None,None,("notzero",)),
            (("C",0,0,5),"OUD",9.2,"Unit-Cost"),
            (("C",0,0,6),"OSD",9.2,"Value")]
        if self.glint == "Y":
            fld.append((("C",0,0,7),"IUI",7,"G/L-Acc","G/L Account Number",
                self.stk_susp,"N",self.doGenAcc,glm,None,None))
            fld.append((("C",0,0,8),"ONA",10,"Desc"))
            fld.append((("C",0,0,9),"INA",(20,30),"Details","Transaction "\
                "Details","","N",self.doTrnDet,None,None,None))
        row = (15,)
        tnd = ((self.endPage0,"n"),)
        txt = (self.exitPage0,)
        cnd = ((self.endPage1,"y"),)
        cxt = (self.exitPage1,)
        but = (("Reprint",None,self.doReprint,0,("T",0,1),None),)
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, rows=row,
            tend=tnd, txit=txt, cend=cnd, cxit=cxt, butt=but)

    def doIss(self, frt, pag, r, c, p, i, w):
        self.iss = w

    def doDte(self, frt, pag, r, c, p, i, w):
        if w < self.period[1][0] or w > self.period[2][0]:
            return "Invalid Date, Not in Financial Period"
        self.dte = w
        self.ddt = self.df.t_disp[pag][0][p]
        self.curdt = self.dte / 100
        self.batno = "S%s" % self.curdt

    def doGroup(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="strgrp", cols=["gpm_desc"],
            where=[("gpm_cono", "=", self.conum), ("gpm_group", "=", w)],
            limit=1)
        if not acc:
            return "Invalid Group"
        self.group = w

    def doCode(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="strmf1", cols=["st1_desc", "st1_type",
            "st1_uoi"], where=[("st1_cono", "=", self.conum), ("st1_group",
            "=", self.group), ("st1_code", "=", w)], limit=1)
        if not acc:
            return "Invalid Code"
        self.code = w
        self.desc = acc[0]
        self.gtype = acc[1]
        self.df.loadEntry(frt, pag, p+1, data=acc[2])
        if self.locs == "N":
            self.loc = "1"
            self.df.loadEntry("C", pag, p+2, data=self.loc)
            no = self.checkLoc()
            if no:
                return no
            else:
                return "sk2"

    def doLoc(self, frt, pag, r, c, p, i, w):
        self.loc = w
        no = self.checkLoc()
        if no:
            return no

    def checkLoc(self):
        acc = self.sql.getRec(tables="strmf2", cols=["st2_bin"],
            where=[("st2_cono", "=", self.conum), ("st2_group", "=",
            self.group), ("st2_code", "=", self.code), ("st2_loc", "=",
            self.loc)], limit=1)
        if not acc:
            return "Invalid Location For This Product"

    def doQty(self, frt, pag, r, c, p, i, w):
        self.quant = w
        if self.gtype == "R":
            err = self.doRecipe()
            if err:
                return "ff2"
        if self.extractCost():
            return "rf"
        self.df.loadEntry("C", pag, p+1, data=self.ucost)
        self.df.loadEntry("C", pag, p+2, data=self.tcost)

    def doRecipe(self):
        self.recipe = self.sql.getRec(tables="strrcp", where=[("srr_cono",
            "=", self.conum), ("srr_group", "=", self.group), ("srr_code",
            "=", self.code), ("srr_loc", "=", self.loc)],
            order="srr_rgroup, srr_rcode")
        if not self.recipe:
            mess = "Invalid Recipe, No Items"
            showError(self.mf.body, "Recipe Error", mess)
            return mess
        else:
            return self.doRecChg()

    def doRecChg(self):
        # Display recipe items and allow editing of quantities etc.
        data = []
        for num, item in enumerate(self.recipe):
            desc = self.sql.getRec(tables="strmf1", cols=["st1_desc"],
                where=[("st1_cono", "=", self.conum), ("st1_group", "=",
                item[4]), ("st1_code", "=", item[5])], limit=1)
            if not desc:
                mess = "Invalid Stock Record in Recipe"
                showError(self.mf.body, "Recipe Error",
                    """%s!

Group: %s
Code:  %s""" % (mess, item[4], item[5]))
                return mess
            data.append([num, item[4], item[5], desc[0], item[6]])
        titl = "Recipe Items"
        head = ("Seq","Grp","Product-Code", "Description", "Quantity")
        lin = {
            "stype": "C",
            "titl": titl,
            "head": head,
            "typs": [
                ("UI", 2),
                ("NA", 3), 
                ("NA", 20), 
                ("NA", 30), 
                ("SD", 11.2)],
            "data": data}
        state = self.df.disableButtons()
        self.mf.updateStatus("Select a Product to Edit or Quit to Continue")
        chg = self.df.selChoice(lin)
        if chg and chg.selection:
            self.recchg = chg.selection
            self.doRecChanges()
            self.doRecChg()
        else:
            self.df.enableButtons(state=state)

    def doRecChanges(self):
        tit = ("Change Items",)
        fld = (
            (("T",0,0,0),"ONA",3,"Group"),
            (("T",0,1,0),"ONA",20,"Code"),
            (("T",0,2,0),"ONA",30,"Description"),
            (("T",0,3,0),"ISD",10.2,"Quantity","Quantity",
                "","N",self.doRecQty,None,None,('notzero',)))
        but = (("Delete",None,self.doRecDel,1,None,None),)
        self.rp = TartanDialog(self.mf, title=tit, newscn=(0, 0),
            tops=True, eflds=fld, butt=but, tend=((self.doRecEnd,"n"),),
            txit=(self.doRecExit,))
        self.rp.loadEntry("T", 0, 0, data=self.recchg[1])
        self.rp.loadEntry("T", 0, 1, data=self.recchg[2])
        self.rp.loadEntry("T", 0, 2, data=self.recchg[3])
        self.rp.loadEntry("T", 0, 3, data=self.recchg[4])
        self.rp.focusField("T", 0, 4, clr=False)
        self.mf.startLoop()

    def doRecQty(self, frt, pag, r, c, p, i, w):
        self.recqty = w

    def doRecDel(self, widget=None):
        del self.recipe[int(self.recchg[0])]
        self.doRecExit()

    def doRecEnd(self):
        self.recipe[int(self.recchg[0])][6] = self.recqty
        self.doRecExit()

    def doRecExit(self):
        self.rp.closeProcess()
        self.mf.closeLoop()

    def doGenAcc(self, frt, pag, r, c, p, i, w):
        self.glnum = w
        acc = self.sql.getRec(tables="genmst", cols=["glm_desc"],
            where=[("glm_cono", "=", self.conum), ("glm_acno", "=", w)],
            limit=1)
        if not acc:
            return "Invalid Account Code"
        chk = chkCtlAcc(self.mf, self.conum, self.glnum)
        if chk:
            return chk
        self.df.loadEntry("C", pag, p+1, data=acc[0])
        return "sk"

    def doTrnDet(self, frt, pag, r, c, p, i, w):
        self.trndet = w
        if not self.trndet:
            self.trndet = "Stores Issues"

    def endPage0(self):
        self.df.focusField("C", 0, 1)

    def endPage1(self):
        self.updateTables()
        self.mf.dbm.commitDbase()
        self.df.advanceLine(0)

    def exitPage0(self):
        self.df.closeProcess()
        self.mf.closeLoop()

    def exitPage1(self):
        if self.df.col != 1:
            self.doPrintDoc(self.prtvwr, self.prtnam)
        self.df.focusField("T", 0, 1)

    def extractCost(self):
        qbal = 0
        vbal = 0
        self.ucost = 0
        self.tcost = 0
        if self.gtype == "R":
            for item in self.recipe:
                quant = item[6] * self.quant
                vals = self.doCalCost(item[4], item[5], quant)
                if not vals:
                    return "error"
                self.tcost = float(ASD(self.tcost) + ASD(vals[2]))
        else:
            vals = self.doCalCost(self.group, self.code, self.quant)
            if not vals:
                return "error"
            self.tcost = vals[2]
        self.ucost = round(self.tcost / self.quant, 2)

    def updateTables(self):
        if self.gtype == "R":
            cost = 0
            # Issue individual items
            for item in self.recipe:
                acc = self.sql.getRec(tables="strmf1", cols=["st1_desc"],
                    where=[("st1_cono", "=", item[0]), ("st1_group", "=",
                    item[4]), ("st1_code", "=", item[5])], limit=1)
                if acc:
                    des = acc[0]
                else:
                    des = "Unknown Description"
                qty = item[6] * self.quant
                vals = self.doCalCost(item[4], item[5], qty, chk=False)
                qty = float(ASD(0) - ASD(qty))
                val = float(ASD(0) - ASD(vals[2]))
                data = [self.conum, item[4], item[5], self.loc, self.dte,
                    6, self.iss, self.batno, self.glnum, qty, val, 0,
                    self.curdt, des, 0, "", "", "STR", 0, 0, 0, "",
                    self.sysdtw, 0]
                self.sql.insRec("strtrn", data=data)
            # Receive recipe item
            data = [self.conum, self.group, self.code, self.loc, self.dte, 5,
                self.iss, self.batno, self.glnum, self.quant, self.tcost,
                0, self.curdt, self.desc, 0, "", "", "STR", 0, 0, 0, "",
                self.sysdtw, 0]
            self.sql.insRec("strtrn", data=data)
        # Stores Ledger Transaction
        qty = float(ASD(0) - ASD(self.quant))
        val = float(ASD(0) - ASD(self.tcost))
        data = [self.conum, self.group, self.code, self.loc, self.dte, 2,
            self.iss, self.batno, self.glnum, qty, val, 0, self.curdt,
            self.desc, 0, "", "", "STR", 0, 0, 0, "N", self.sysdtw, 0]
        self.sql.insRec("strtrn", data=data)
        if self.glint == "N":
            return
        col = self.sql.gentrn_col
        # General Ledger Control Transaction (Stock On Hand)
        acc = self.sql.getRec(tables="gentrn", where=[("glt_cono", "=",
            self.conum), ("glt_acno", "=", self.stk_soh), ("glt_curdt", "=",
            self.curdt), ("glt_trdt", "=", self.dte), ("glt_type", "=", 4),
            ("glt_refno", "=", self.iss), ("glt_batch", "=", self.batno)],
            limit=1)
        if acc:
            amnt = float(ASD(acc[col.index("glt_tramt")]) + ASD(val))
            self.sql.updRec("gentrn", cols=["glt_tramt"], data=[amnt],
                where=[("glt_seq", "=", acc[col.index("glt_seq")])])
        else:
            self.sql.insRec("gentrn", data=[self.conum, self.stk_soh,
            self.curdt, self.dte, 4, self.iss, self.batno, val, 0,
            self.trndet, "N", "", 0, self.sysdtw, 0])
        # General Ledger Transaction (Expense)
        acc = self.sql.getRec(tables="gentrn", where=[("glt_cono", "=",
            self.conum), ("glt_acno", "=", self.glnum), ("glt_curdt", "=",
            self.curdt), ("glt_trdt", "=", self.dte), ("glt_type", "=", 4),
            ("glt_refno", "=", self.iss), ("glt_batch", "=", self.batno)],
            limit=1)
        if acc:
            amnt = float(ASD(acc[col.index("glt_tramt")]) + ASD(self.tcost))
            self.sql.updRec("gentrn", cols=["glt_tramt"], data=[amnt],
                where=[("glt_seq", "=", acc[col.index("glt_seq")])])
        else:
            self.sql.insRec("gentrn", data=[self.conum, self.glnum,
            self.curdt, self.dte, 4, self.iss, self.batno, self.tcost,
            0, self.trndet, "N", "", 0, self.sysdtw, 0])

    def doCalCost(self, grp, code, qty, chk=True):
        # Calculate cost price
        acc = self.sql.getRec(tables="strmf1", cols=["st1_desc",
            "st1_value_ind"], where=[("st1_cono", "=", self.conum),
            ("st1_group", "=", grp), ("st1_code", "=", code)], limit=1)
        bal = Balances(self.mf, "STR", self.conum, self.curdt,
            (grp, code, self.loc, ("P", self.period[0])))
        bals = bal.doStrBals()
        if chk and qty > bals[5][0]:
            cf = PwdConfirm(self.mf, conum=self.conum, system="STR",
                code="ExQty", product=(grp, code, acc[0]))
            if cf.flag == "no":
                return
        if acc[1] == "N":
            icost = 0.00
            tcost = 0.00
        else:
            if qty == bals[5][0]:
                tcost = bals[5][1]
                icost = round((tcost / qty), 2)
            else:
                icost = bals[6]
                tcost = round((icost * qty), 2)
            if icost == 0:
                icost = bals[7]
                tcost = round((icost * qty), 2)
        return (acc[1], icost, tcost, bals[5][0])

    def doReprint(self, widget=None):
        tit = ("Reprint Documents",)
        data = self.sql.getRec(tables="strtrn", cols=["stt_ref1", "stt_trdt"],
            where=[("stt_cono", "=", self.conum), ("stt_type", "=", 2)],
            group="stt_ref1, stt_trdt", order="stt_trdt, stt_ref1")
        iss = {
            "stype": "R",
            "tables": ("strtrn", "strpom"),
            "cols": (
                ("stt_ref1", "", 0, "Reference"),
                ("stt_trdt", "", 0, "Date", "Y")),
            "wtype": "D",
            "where": data,
            "screen": self.mf.body,
            "comnd": self.doSelRec}
        isd = {
            "stype": "R",
            "tables": ("strtrn",),
            "cols": (
                ("stt_trdt", "", 0, "Date"),),
            "where": [("stt_cono", "=", self.conum), ("stt_type", "=", 2)],
            "whera": (("T", "stt_ref1", 0, 0),),
            "group": "stt_trdt",
            "order": "stt_trdt",
            "screen": self.mf.body}
        fld = (
            (("T",0,0,0),"INa",9,"Issue Number","Issue Number",
                "","N",self.doRepIss,iss,None,("notblank",)),
            (("T",0,1,0),"ID1",10,"Date","GRN Date",
                0,"N",self.doRepDte,isd,None,("notzero",)))
        state = self.df.disableButtons()
        self.tx = TartanDialog(self.mf, title=tit, newscn=(0,1),
            tops=True, eflds=fld, tend=((self.doRepEnd, "n"),),
            txit=(self.doRepExit,), view=("N","V"), mail=("Y","N"))
        self.mf.startLoop()
        self.df.enableButtons(state=state)
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def doSelRec(self, frt, pag, r, c, p, i, w):
        self.tx.doKeyPressed(frt, pag, p, w[0])
        self.tx.doKeyPressed(frt, pag, p+1, w[1])

    def doRepIss(self, frt, pag, r, c, p, i, w):
        self.iss = w

    def doRepDte(self, frt, pag, r, c, p, i, w):
        self.dte = w
        self.ddt = self.df.t_disp[pag][0][p]
        acc = self.sql.getRec(tables="strtrn", cols=["count(*)"],
            where=[("stt_cono", "=", self.conum), ("stt_type", "=", 2),
            ("stt_ref1", "=", self.iss), ("stt_trdt", "=", self.dte)], limit=1)
        if not acc[0]:
            return "No Document Found"

    def doRepEnd(self):
        self.tx.closeProcess()
        self.doPrintDoc(self.tx.repprt[0][1], self.tx.repprt[0][2])
        self.mf.closeLoop()

    def doRepExit(self):
        self.tx.closeProcess()
        self.mf.closeLoop()

    def doPrintDoc(self, prtvwr, prtnam):
        hds = ["Goods Issued Notes",
            "GIN Number: %s GIN Date: %s" % (self.iss, self.ddt)]
        tab = ["strmf1", "strtrn"]
        col = [
            ["stt_group", "UA", 3, "Grp", "y"],
            ["stt_code", "NA", 20, "Product-Code", "y"],
            ["stt_desc", "NA", 40, "Description", "y"],
            ["stt_loc", "UA", 1, "L", "y"],
            ["stt_qty", "SD", 11.2, "Quantity", "y", "y"],
            ["stt_cost", "SD", 11.2, "Price", "y", "y"]]
        gtt = ["stt_cost"]
        whr = [("stt_cono", "=", self.conum), ("stt_type", "in", (2,6)),
            ("stt_ref1", "=", self.iss), ("stt_trdt", "=", self.dte),
            ("st1_cono=stt_cono",), ("st1_group=stt_group",),
            ("st1_code=stt_code",), ("st1_type", "<>", "R")]
        ord = "stt_seq"
        self.df.mstAlw.hide()
        rpt = RepPrt(self.mf, conum=self.conum, conam=self.conam, name=__name__,
            tables=tab, heads=hds, cols=col, gtots=gtt, where=whr, order=ord,
            view=self.prtvwr, prtnam=prtnam)
        self.df.mstAlw.show()
