#! /usr/bin/env python

"""
SYNOPSIS
    Stores Ledger Transaction Audit Trail.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, CCD, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, getModName, doPrinter, showError
from tartanWork import sttrtp

class st3020:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, "strtrn", prog=__name__)
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.curdt = self.sysdtw / 100
        self.totind = "N"
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Stores Ledger Audit Trail (%s)" % __name__)
        data = ["All Types"]
        for x in range(1, len(sttrtp)+1):
            data.append(sttrtp[x-1][1])
        btt = {
            "stype": "C",
            "titl": "Select the Required Type",
            "data": data,
            "retn": "I"}
        btm = {
            "stype": "R",
            "tables": ("strtrn",),
            "cols": (
                ("stt_batch", "", 0, "Bat-Num"),
                ("stt_type", ("xx", sttrtp), 20, "Type"),
                ("stt_curdt", "", 0, "Cur-Dat")),
            "where": [],
            "group": "stt_batch, stt_type, stt_curdt",
            "order": "stt_type, stt_curdt, stt_batch"}
        r1s = (("Financial","F"),("Capture","C"))
        r2s = (("Yes","Y"),("No","N"))
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"Period Type","",
                "F","Y",self.doPtyp,None,None,None),
            (("T",0,1,0),"Id2",7,"Starting Period","",
                self.curdt,"Y",self.doStartPer,None,None,("efld",)),
            (("T",0,2,0),"Id2",7,"Ending Period","",
                self.curdt,"Y",self.doEndPer,None,None,("efld",)),
            (("T",0,3,0),"Id1",10,"Starting Date","",
                self.sysdtw,"Y",self.doStartDat,None,None,("efld",)),
            (("T",0,4,0),"Id1",10,"Ending Date","",
                self.sysdtw,"Y",self.doEndDat,None,None,("efld",)),
            (("T",0,5,0),"IUI",1,"Type","Transaction Type",
                "","Y",self.doBatTyp,btt,None,None),
            (("T",0,6,0),"INa",7,"Batch Number","",
                "","Y",self.doBatNum,btm,None,None),
            (("T",0,7,0),("IRB",r2s),0,"Totals Only","",
                "Y","Y",self.doTots,None,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doPtyp(self, frt, pag, r, c, p, i, w):
        self.ptyp = w
        if self.ptyp == "C":
            return "sk2"

    def doStartPer(self, frt, pag, r, c, p, i, w):
        if w > self.curdt:
            return "Invalid Period, After To-Day"
        else:
            self.sperw = w
        self.sperd = self.df.t_disp[pag][0][i]
        self.df.loadEntry(frt, pag, p, data=self.sperw)
        self.df.topf[pag][i+1][5] = self.sperw

    def doEndPer(self, frt, pag, r, c, p, i, w):
        if w == 0:
            self.eperw = self.sperw
        elif w < self.sperw:
            return "Invalid Period, Before Start"
        else:
            self.eperw = w
        self.eperd = self.df.t_disp[pag][0][i]
        self.df.loadEntry(frt, pag, p, self.eperw)
        return "sk2"

    def doStartDat(self, frt, pag, r, c, p, i, w):
        if w > self.sysdtw:
            return "Invalid Date, After To-Day"
        else:
            self.sdatw = w
        self.sperd = self.df.t_disp[pag][0][i]
        self.df.topf[pag][i+1][5] = self.sdatw

    def doEndDat(self, frt, pag, r, c, p, i, w):
        if w == 0:
            self.edatw = self.sdatw
        elif w < self.sdatw:
            return "Invalid Date, Before Start"
        else:
            self.edatw = w
        self.eperd = self.df.t_disp[pag][0][i]
        self.df.loadEntry(frt, pag, p, self.edatw)

    def doBatTyp(self, frt, pag, r, c, p, i, w):
        if w > len(sttrtp):
            return "Invalid Batch Type"
        self.whr = [("stt_cono", "=", self.conum)]
        if self.ptyp == "F":
            self.whr.append(("stt_curdt", "between", self.sperw, self.eperw))
        else:
            self.whr.append(("stt_capdt", "between", self.sdatw, self.edatw))
        if w:
            self.whr.append(("stt_type", "=", w))
        self.df.topf[pag][i+1][8]["where"] = self.whr

    def doBatNum(self, frt, pag, r, c, p, i, w):
        self.batch = w

    def doTots(self, frt, pag, r, c, p, i, w):
        self.totsonly = w

    def doEnd(self):
        self.df.closeProcess()
        tab = "strtrn"
        ord = "stt_type, stt_batch, stt_ref1"
        all = self.sql.getRec(tables=tab, where=self.whr, order=ord)
        if not all:
            showError(self.mf.body, "Processing Error",
            "No Stores Transactions Selected")
        else:
            self.printReport(all)
        self.mf.closeLoop()

    def printReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        if self.totsonly == "Y":
            self.head = ("%03u %-30s %1s %33s %1s %6s" % \
            (self.conum, self.conam, "", self.sysdttm, "", __name__))
        else:
            self.head = ("%03u %-30s %25s %33s %25s %6s" % \
            (self.conum, self.conam, "", self.sysdttm, "", __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        self.bcnt = 0
        self.bqty = 0
        self.bcst = 0
        self.bsel = 0
        self.tcnt = 0
        self.tqty = 0
        self.tcst = 0
        self.tsel = 0
        self.gcnt = [0,0,0,0,0,0,0,0]
        self.gqty = [0,0,0,0,0,0,0,0]
        self.gcst = [0,0,0,0,0,0,0,0]
        self.gsel = [0,0,0,0,0,0,0,0]
        self.trtp = 0
        self.pgnum = 0
        self.pglin = 99
        col = self.sql.strtrn_col
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            trtp = CCD(dat[col.index("stt_type")], "UI", 2)
            batch = CCD(dat[col.index("stt_batch")], "Na", 7)
            grp = CCD(dat[col.index("stt_group")], "UA", 3)
            code = CCD(dat[col.index("stt_code")], "NA", 20)
            loc = CCD(dat[col.index("stt_loc")], "UA", 1)
            ref1 = CCD(dat[col.index("stt_ref1")], "Na", 9)
            trdt = CCD(dat[col.index("stt_trdt")], "D1", 10)
            desc = CCD(dat[col.index("stt_desc")], "NA", 30)
            debtor = CCD(dat[col.index("stt_acno")], "NA", 7)
            qty = CCD(dat[col.index("stt_qty")], "SD", 13.2)
            cost = CCD(dat[col.index("stt_cost")], "SD", 13.2)
            sell = CCD(dat[col.index("stt_sell")], "SD", 13.2)
            if self.trtp == 0:
                self.trtp = trtp.work
                self.batch = batch.work
            if trtp.work != self.trtp:
                self.batchTotal()
                self.typeTotal()
                self.trtp = trtp.work
                self.batch = batch.work
                self.pglin = 99
            if batch.work != self.batch:
                self.batchTotal()
                self.batch = batch.work
                if self.totsonly != "Y":
                    self.typeHeading()
            if self.pglin > self.lpp:
                if self.pgnum != 0:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
            if self.totsonly != "Y":
                self.t.textLine("%s %s %s %s %s %s %s %s %s %s" % \
                    (grp.disp, code.disp, loc.disp, ref1.disp, trdt.disp,
                    desc.disp, debtor.disp, qty.disp, cost.disp, sell.disp))
                self.pglin = self.pglin + 1
            self.bcnt = self.bcnt + 1
            self.bqty = float(ASD(self.bqty) + ASD(qty.work))
            self.bcst = float(ASD(self.bcst) + ASD(cost.work))
            self.bsel = float(ASD(self.bsel) + ASD(sell.work))
            self.tcnt = self.tcnt + 1
            self.tqty = float(ASD(self.tqty) + ASD(qty.work))
            self.tcst = float(ASD(self.tcst) + ASD(cost.work))
            self.tsel = float(ASD(self.tsel) + ASD(sell.work))
            self.gcnt[trtp.work -1] = self.gcnt[trtp.work -1] + 1
            self.gqty[trtp.work -1] = float(ASD(self.gqty[trtp.work -1]) + \
                ASD(qty.work))
            self.gcst[trtp.work -1] = float(ASD(self.gcst[trtp.work -1]) + \
                ASD(cost.work))
            self.gsel[trtp.work -1] = float(ASD(self.gsel[trtp.work -1]) + \
                ASD(sell.work))
        p.closeProgress()
        if self.t:
            if can != "yes":
                self.batchTotal()
                self.typeTotal()
                self.grandTotal()
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
            head=self.tit, view=self.df.repprt[0][1],
            prtnam=self.df.repprt[0][2], mail=self.df.repprt[1][2])
            
    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        if self.totsonly == "Y":
            self.t.textLine("%-36s %-10s %-2s %-10s "\
            "%11s %5s" % ("Stores Ledger Audit Trail for Period",
            self.sperd, "to", self.eperd, "Page", self.pgnum))
        else:
            self.t.textLine("%-36s %-10s %-2s %-10s %59s %5s" % \
            ("Stores Ledger Audit Trail for Period",
            self.sperd, "to", self.eperd, "Page", self.pgnum))
        self.t.textLine()
        self.pglin = 4
        if self.totind == "N":
            self.typeHeading()
        else:
            self.t.textLine("%-14s" % "Totals Summary")
            self.t.textLine()
            self.t.textLine("%-29s %-8s %-13s %-13s %-13s" %
                ("Document Type", " Number", "    Quantity", "    Cost-Amt",
                "    Sell-Amt"))
            self.t.textLine("%s" % (self.suc * len(self.head)))
            self.t.setFont("Tartan", self.cpi, self.ldg)
            self.pglin = self.pglin + 4

    def typeHeading(self):
        if self.totsonly != "Y":
            batch = self.batch
        else:
            batch = "Various"
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.t.textLine("%-7s %7s %-10s %3s" % \
            ("Batch", batch, "    Type", sttrtp[self.trtp - 1][1]))
        self.t.textLine()
        if self.trtp == 1:
            drscrs = "Crs-Acc"
        elif self.trtp == 7:
            drscrs = "Drs-Acc"
        else:
            drscrs = ""
        if self.totsonly == "Y":
            self.t.textLine("%-30s %-7s %-13s %-13s "\
            "%-13s" % ("Remarks", "", "    Quantity", "    Cost-Amt",
            "    Sell-Amt"))
        else:
            self.t.textLine("%-3s %-20s %1s %-9s %-10s %-30s %-7s %-13s "\
            "%-13s %-13s" % ("Grp", "Product-Code", "L", "Reference", "   Date",
            "Remarks", drscrs, "    Quantity", "    Cost-Amt", "    Sell-Amt"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = self.pglin + 4

    def batchTotal(self):
        j = CCD(self.bcnt, "SI", 7)
        k = CCD(self.bqty, "SD", 13.2)
        l = CCD(self.bcst, "SD", 13.2)
        m = CCD(self.bsel, "SD", 13.2)
        if self.totsonly == "N":
            self.t.textLine()
            self.pglin = self.pglin + 1
        if self.totsonly == "Y":
            self.t.textLine("%-38s %13s %13s %13s" % \
            ("Batch " + self.batch + " Totals", k.disp, l.disp, m.disp))
        else:
            self.t.textLine("%-47s %-38s %13s %13s %13s" % \
            (" ", "Batch " + self.batch + " Totals", k.disp, l.disp, m.disp))
        self.pglin = self.pglin + 1
        if self.totsonly != "Y":
            self.t.textLine()
            self.pglin = self.pglin + 1
        self.bcnt = 0
        self.bqty = 0
        self.bcst = 0
        self.bsel = 0

    def typeTotal(self):
        j = CCD(self.tcnt, "SI", 7)
        k = CCD(self.tqty, "SD", 13.2)
        l = CCD(self.tcst, "SD", 13.2)
        m = CCD(self.tsel, "SD", 13.2)
        if self.totsonly == "Y":
            self.t.textLine()
            self.t.textLine("%-38s %13s %13s %13s" % \
                ("Type-Totals", k.disp, l.disp, m.disp))
        else:
            self.t.textLine("%-47s %-38s %13s %13s %13s" % \
            (" ", "Type-Totals", k.disp, l.disp, m.disp))
        self.t.textLine()
        self.tcnt = 0
        self.tqty = 0
        self.tcst = 0
        self.tsel = 0
    
    def grandTotal(self):
        self.c.drawText(self.t)
        self.c.showPage()
        self.totind = "Y"
        self.pageHeading()
        tot = [0,0,0,0]
        for x in range(0, len(sttrtp)):
            cnt = CCD(self.gcnt[x], "SI", 7)
            qty = CCD(self.gqty[x], "SD", 13.2)
            cst = CCD(self.gcst[x], "SD", 13.2)
            sel = CCD(self.gsel[x], "SD", 13.2)
            self.t.textLine("%-30s %s %s %s %s" % \
                (sttrtp[x][1], cnt.disp, qty.disp, cst.disp, sel.disp))
            tot[0] = tot[0] + cnt.work
            tot[1] = float(ASD(tot[1]) + ASD(qty.work))
            tot[2] = float(ASD(tot[2]) + ASD(cst.work))
            tot[3] = float(ASD(tot[3]) + ASD(sel.work))
        self.t.textLine()
        cnt = CCD(tot[0], "SI", 8)
        qty = CCD(tot[1], "SD", 13.2)
        cst = CCD(tot[2], "SD", 13.2)
        sel = CCD(tot[3], "SD", 13.2)
        self.t.textLine("%-29s %s %s %s %s" % \
            ("Grand Totals", cnt.disp, qty.disp, cst.disp, sel.disp))
        self.t.textLine()

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

