#! /usr/bin/env python

"""
SYNOPSIS
    Stores Ledger Selling And Cost Price List.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import Balances, CCD, GetCtl, ProgressBar, Sql
from TartanClasses import TartanDialog
from tartanFunctions import doReportlab, doWriteExport, getModName, doPrinter
from tartanFunctions import showError

class st3030:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlvmf", "strloc", "strgrp",
            "strprc", "strmf1", "strmf2"], prog=__name__)
        gc = GetCtl(self.mf)
        strctl = gc.getCtl("strctl", self.conum)
        if not strctl:
            return "no"
        self.locs = strctl["cts_locs"]
        self.prcs = strctl["cts_prices"]
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdtd = "%i/%02i/%02i" % (t[0], t[1], t[2])
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i) %6s" % (t[0],
            t[1], t[2], t[3], t[4], __name__)
        self.head = ("%03u %-30s %s" % (self.conum, self.conam, "%s"))
        self.stot = [0] * 6
        self.gtot = [0] * 6
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Stores Selling Price List (%s)" % __name__)
        loc = {
            "stype": "R",
            "tables": ("strloc",),
            "cols": (
                ("srl_loc", "", 0, "L"),
                ("srl_desc", "", 0, "Description", "Y")),
            "where": [("srl_cono", "=", self.conum)]}
        grp = {
            "stype": "R",
            "tables": ("strgrp",),
            "cols": (
                ("gpm_group", "", 0, "Grp"),
                ("gpm_desc", "", 0, "Description", "Y")),
            "where": [("gpm_cono", "=", self.conum)]}
        r1s = (("Yes","Y"),("No","N"))
        r2s = (("Selling Price","S"),("Cost Price","C"))
        r3s = (("Levels","L"),("Mark Up","M"))
        if self.locs == "N":
            self.loc = "1"
            fld = []
        else:
            fld = [(("T",0,0,0),"IUA",1,"Location","Location",
                "1","Y",self.doLoc,loc,None,("efld",))]
        fld.extend([
            (("T",0,1,0),"IUA",3,"Product Group","Product Group",
                "","Y",self.doGroup,grp,None,None),
            (("T",0,2,0),("IRB",r1s),0,"Ignore Out of Stock","Ignore Out "\
                "of Stock","Y","Y",self.doNoStock,None,None,None),
            (("T",0,3,0),("IRB",r2s),0,"Report Type","Report Type",
                "S","Y",self.doType,None,None,None),
            (("T",0,4,0),("IRB",r3s),0,"Price Type","Price Type",
                "L","Y",self.doPriceType,None,None,None),
            (("T",0,5,0),"IUI",1,"Price Level","Price Level",
                0,"Y",self.doLvl,None,None,("between",0,self.prcs)),
            (("T",0,6,0),("IRB",r1s),0,"Ignore Unpriced","Ignore Unpriced",
                "Y","Y",self.doNoPrice,None,None,None),
            (("T",0,7,0),("IRB",r1s),0,"Use Last S/P","Use Last S/P if "\
                "Unpriced","Y","Y",self.doLastSP,None,None,None),
            (("T",0,8,0),("IRB",r1s),0,"V.A.T Inclusive","V.A.T Inclusive",
                "Y","Y",self.doVat,None,None,None),
            (("T",0,9,0),("IRB",r1s),0,"Show Cost Price","Show Cost Price",
                "N","Y",self.doCost,None,None,None)])
        tnd = ((self.doEnd,"Y"),)
        txt = (self.doExit,)
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("Y","V"), mail=("Y","N"))

    def doLoc(self, frt, pag, r, c, p, i, w):
        acc = self.getLoc(w)
        if not acc:
            return "Invalid Location"
        self.loc = w

    def doGroup(self, frt, pag, r, c, p, i, w):
        if not w:
            self.group = ""
        else:
            acc = self.sql.getRec(tables="strgrp", cols=["gpm_desc"],
                where=[("gpm_cono", "=", self.conum), ("gpm_group", "=", w)],
                limit=1)
            if not acc:
                return "Invalid Group"
            self.group = w

    def doNoStock(self, frt, pag, r, c, p, i, w):
        self.nostock = w

    def doType(self, frt, pag, r, c, p, i, w):
        self.rtype = w
        if self.rtype == "C":
            return "sk6"

    def doPriceType(self, frt, pag, r, c, p, i, w):
        self.ptype = w
        if self.ptype == "M":
            return "sk1"

    def doLvl(self, frt, pag, r, c, p, i, w):
        self.level = w

    def doNoPrice(self, frt, pag, r, c, p, i, w):
        self.noprice = w
        if self.noprice == "Y":
            self.lasts = "N"
            return "sk1"

    def doLastSP(self, frt, pag, r, c, p, i, w):
        self.lasts = w

    def doVat(self, frt, pag, r, c, p, i, w):
        self.vatinc = w

    def doCost(self, frt, pag, r, c, p, i, w):
        self.costs = w

    def doEnd(self):
        self.df.closeProcess()
        whr = [("st2_cono", "=", self.conum)]
        if self.group:
            whr.append(("st2_group", "=", self.group))
        whr.append(("st2_loc", "=", self.loc))
        whr.extend([("st1_cono=st2_cono",), ("st1_group=st2_group",),
            ("st1_code=st2_code",)])
        ord = "st2_loc, st2_group, st2_code"
        all = self.sql.getRec(tables=["strmf1", "strmf2"], cols=["st2_group",
            "st2_code", "st1_desc", "st1_uoi", "st1_vatcode"], where=whr,
            order=ord)
        if not all:
            showError(self.mf.body, "Processing Error",
                "No Records Selected")
        elif self.df.repprt[0][2] == "export":
            self.exportReport(all)
        elif self.rtype == "S":
            self.printSell(all)
        else:
            self.printCost(all)
        self.closeProcess()

    def exportReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        expnam = getModName(self.mf.wrkdir, __name__, self.conum)
        expheads = [self.head % self.sysdttm]
        if self.rtype == "S":
            expheads.append("Selling Price List as at %s" % self.sysdtd)
            mess = "Options:- Ignore Out of Stock: %s  VAT Inclusive: %s  "\
                "Ignore Unpriced: %s" % (self.nostock,self.vatinc,self.noprice)
            if self.noprice == "N":
                mess = "%s  Use Last S/P for Unpriced: %s" % (mess, self.lasts)
            expheads.append(mess)
            expcolsh = [["Grp", "Product-Code", "Description", "U.O.I",
                "V", "Qty-Balance", "Price-1", "Price-2", "Price-3", "Price-4",
                "Price-5"]]
            expforms = [("UA", 3), ("NA", 20), ("NA", 30), ("NA", 10),
                ("UA", 1), ("SD", 10.2), ("UD", 9.2), ("UD", 9.2),
                ("UD", 9.2), ("UD", 9.2), ("UD", 9.2)]
            if self.costs == "Y":
                expcolsh[0].extend(["Avg-Cost", "Lst-Cost"])
                expforms.extend([("SD", 10.2), ("SD", 10.2)])
        else:
            expheads.append("Cost Price List as at %s" % self.sysdtd)
            mess = "Options:- Ignore Out of Stock: %s" % self.nostock
            expheads.append(mess)
            expcolsh = [["Grp", "Product-Code", "Description", "U.O.I",
                    "Qty-Balance", "Avg-Cost", "Lst-Cost"]]
            expforms = [("UA", 3), ("NA", 20), ("NA", 30), ("NA", 10),
                ("SD", 10.2), ("SD", 10.2), ("SD", 10.2)]
        expdatas = []
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                p.closeProgress()
                return
            vals = self.getValues(dat)
            if not vals:
                continue
            if self.rtype == "S":
                vvv = [vals[0].work, vals[1].work, vals[2].work, vals[3].work,
                    vals[4].work, vals[5].work, vals[8][0].work,
                    vals[8][1].work, vals[8][2].work, vals[8][3].work,
                    vals[8][4].work]
                if self.costs == "Y":
                    vvv.extend([vals[6].work, vals[7].work])
                expdatas.append(["BODY", vvv])
            else:
                expdatas.append(["BODY", [vals[0].work, vals[1].work,
                    vals[2].work, vals[3].work, vals[4].work, vals[5].work,
                    vals[6].work]])
        p.closeProgress()
        doWriteExport(xtype=self.df.repprt[0][1], name=expnam, sheet=__name__,
            heads=expheads, colsh=expcolsh, forms=expforms, datas=expdatas,
            rcdic=self.mf.rcdic)

    def printSell(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        if self.costs == "Y":
            spc1 = " " * 35; spc2 = " " * 36
        else:
            spc1 = " " * 25; spc2 = " " * 25
        self.head = ("%03u %-30s %s %33s %s %6s" % \
            (self.conum, self.conam, spc1, self.sysdttm, spc2, __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        old_grp = ""
        self.pgnum = 0
        self.pglin = 99
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            vals = self.getValues(dat)
            if not vals:
                continue
            grp, code, desc, uoi, vat, qty, acst, lcst, pr = vals
            for nm, sp in enumerate(pr):
                if not nm:
                    continue
                if not sp.work and not sp.disp == "*        ":
                    pr[nm].disp = "         "
            if old_grp and old_grp != grp.work:
                self.pglin = 99
            if self.pglin > self.lpp:
                if self.pgnum:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
            if self.costs != "Y":
                self.t.textLine("%s %s %s %s %s %s %s %s %s %s" % (code.disp,
                    desc.disp, uoi.disp, vat.disp, qty.disp, pr[0].disp,
                    pr[1].disp, pr[2].disp, pr[3].disp, pr[4].disp))
            else:
                self.t.textLine("%s %s %s %s %s %s %s %s %s %s %s %s" % \
                    (code.disp, desc.disp, uoi.disp, vat.disp, qty.disp,
                    pr[0].disp, pr[1].disp, pr[2].disp, pr[3].disp,
                    pr[4].disp, acst.disp, lcst.disp))
            self.pglin = self.pglin + 1
            old_grp = grp.work
        p.closeProgress()
        if self.t:
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam, head=self.tit,
            view=self.df.repprt[0][1], prtnam=self.df.repprt[0][2],
            mail=self.df.repprt[1][2])

    def printCost(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        self.head = ("%03u %-30s %10s %30s %9s %6s" % \
            (self.conum, self.conam, "", self.sysdttm, "", __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        old_grp = ""
        self.pgnum = 0
        self.pglin = 99
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            vals = self.getValues(dat)
            if not vals:
                continue
            grp, code, desc, uoi, qty, acst, lcst = vals
            if old_grp and old_grp != grp.work:
                self.pglin = 99
            if self.pglin > self.lpp:
                if self.pgnum:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
            self.t.textLine("%s %s %s %s %s %s" % (code.disp, desc.disp,
                uoi.disp, qty.disp, acst.disp, lcst.disp))
            self.pglin = self.pglin + 1
            old_grp = grp.work
        p.closeProgress()
        if self.t:
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam, head=self.tit,
            view=self.df.repprt[0][1], prtnam=self.df.repprt[0][2],
            mail=self.df.repprt[1][2])

    def getValues(self, data):
        grp = CCD(data[0], "UA", 3)
        self.groupd = grp.disp
        code = CCD(data[1], "NA", 20)
        desc = CCD(data[2], "UA", 30)
        uoi = CCD(data[3], "NA", 10)
        # Balances and Cost Prices
        bals = Balances(self.mf, "STR", self.conum, (self.sysdtw / 100),
            keys=(grp.work, code.work, self.loc))
        bal = bals.doStrBals(0, markup=True)
        qty = CCD(bal[5][0], "SD", 12.2)
        if self.nostock == "Y" and not qty.work > 0:
            return
        acst = CCD(bal[6], "SD", 10.2)
        lcst = CCD(bal[7], "SD", 10.2)
        if self.rtype == "C":
            return (grp, code, desc, uoi, qty, acst, lcst)
        # Selling Prices
        prs = [0] * 5
        vat = CCD(data[4], "NA", 1)
        if self.vatinc == "Y":
            acc = self.sql.getRec(tables="ctlvmf", cols=["vtm_new"],
                where=[("vtm_cono", "=", self.conum), ("vtm_code", "=",
                vat.work)], limit=1)
            if not acc:
                vatper = 0
            else:
                vatper = acc[0]
        else:
            vatper = 0
        if self.ptype == "L":
            whr = [("stp_cono", "=", self.conum), ("stp_group", "=", grp.work),
                ("stp_code", "=", code.work), ("stp_loc", "=", self.loc)]
            if self.level:
                whr.append(("stp_level", "=", self.level))
            prc = self.sql.getRec(tables="strprc", cols=["stp_level",
                "stp_price"], where=whr, order="stp_level")
            if prc:
                for lev, pr in (prc):
                    prs[lev-1] = round((pr * ((100 + vatper) / 100.0)),2)
        elif bal[8]:
            # Markup Percentage Based on Last Cost
            pr = round((lcst.work * (1 + (bal[8] / 100.0))), 2)
            prs[0] = round((pr * ((100 + vatper) / 100.0)),2)
        pr = []
        for y in range(0, 5):
            pr.append(CCD(prs[y], "UD", 9.2))
        if not pr[0].work and not pr[1].work and not pr[2].work and \
                                    not pr[3].work and not pr[4].work:
            if self.noprice == "Y":
                return
            if self.lasts == "Y":
                lsp = self.sql.getRec(tables="strtrn", cols=["stt_sell"],
                    where=[("stt_cono", "=", self.conum), ("stt_group",
                    "=", grp.work), ("stt_code", "=", code.work),
                    ("stt_loc", "=", self.loc), ("stt_type", "=", 7)],
                    order="stt_trdt desc", limit=1)
                if not lsp or not lsp[0] > 0:
                    lsp = CCD(0, "UD", 9.2)
                else:
                    lsp = CCD(lsp[0], "UD", 9.2)
                pr[0] = lsp
                if lsp.work:
                    pr[1].disp = "*        "
        return (grp, code, desc, uoi, vat, qty, acst, lcst, pr)

    def pageHeading(self):
        locd = self.getLoc(self.loc)[0]
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        if self.rtype == "S":
            if self.costs == "Y":
                pag = (" " * 102) + "Page"
            else:
                pag = (" " * 81) + "Page"
            self.t.textLine("%-24s %-10s %s %5s" % ("Selling Price List as at",
                self.sysdtd, pag, self.pgnum))
            self.t.textLine()
            mess = "%-31s %1s %-14s %1s %-16s %s" % (
                "(Options:- Ignore Out of Stock:", self.nostock,
                "VAT Inclusive:", self.vatinc,
                "Ignore Unpriced:", self.noprice)
            if self.noprice == "N":
                mess = "%s Use Last S/P for Unpriced: %s" % (mess, self.lasts)
            self.t.textLine("%s%s" % (mess, ")"))
        else:
            self.t.textLine("%-21s %-10s %57s %5s" % \
                ("Cost Price List as at", self.sysdtd, "Page", self.pgnum))
            self.t.textLine()
            self.t.textLine("%-31s %1s%1s" % (
                "(Options:- Ignore Out of Stock:", self.nostock, ")"))
        self.t.textLine()
        self.t.textLine("%-5s %3s  %-8s %s %s" % \
            ("Group", self.groupd, "Location", self.loc, locd))
        self.t.textLine()
        cost = "Avg-Cost"
        last = "Lst-Cost"
        if self.rtype == "S":
            if self.costs == "N":
                cost = ""
                last = ""
            self.t.textLine("%-20s %-30s %-10s %1s %-14s %-9s %-9s %-9s " \
                "%-9s %-9s%-10s %-10s" % ("Product-Code", "Description",
                "U.O.I", "V", "Qty-Balance", "Price-1", "Price-2",
                "Price-3", "Price-4", "Price-5", cost, last))
        else:
            self.t.textLine("%-20s %-30s %-10s %-12s  %-10s %-10s" % \
                ("Product-Code", "Description", "U.O.I", "Qty-Balance",
                cost, last))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 10

    def getLoc(self, loc):
        acc = self.sql.getRec(tables="strloc", cols=["srl_desc"],
            where=[("srl_cono", "=", self.conum), ("srl_loc", "=", loc)],
            limit=1)
        return acc

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

