#! /usr/bin/env python

"""
SYNOPSIS
    Stores Ledger Master Code List.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import CCD, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, getModName, doPrinter, showError

class st3040:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["strgrp", "strmf1", "struoi"],
            prog=__name__)
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdtd = "%i/%02i/%02i" % (t[0], t[1], t[2])
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Stores Master Code List (%s)" % __name__)
        r1s = (("All", "A"), ("Normal", "N"), ("Recipe", "R"))
        grp = {
            "stype": "R",
            "tables": ("strgrp",),
            "cols": (
                ("gpm_group", "", 0, "Grp"),
                ("gpm_desc", "", 0, "Description", "Y")),
            "where": [("gpm_cono", "=", self.conum)]}
        fld = (
            (("T",0,0,0),"IUA",3,"Product Group","Product Group",
                "","Y",self.doGroup,grp,None,("efld",)),
            (("T",0,1,0),("IRB",r1s),0,"Product Type","Product Type",
                "A","Y",self.doType,None,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doGroup(self, frt, pag, r, c, p, i, w):
        if not w:
            self.group = ""
        else:
            acc = self.sql.getRec(tables="strgrp", cols=["gpm_desc"],
                where=[("gpm_cono", "=", self.conum), ("gpm_group", "=", w)],
                limit=1)
            if not acc:
                return "Invalid Group"
            self.group = w

    def doType(self, frt, pag, r, c, p, i, w):
        self.itype = w

    def doEnd(self):
        self.df.closeProcess()
        if not self.group:
            self.sgrp = ""
            self.egrp = "zzz"
        else:
            self.sgrp = self.group; self.egrp = self.group
        whr = [("st1_cono", "=", self.conum), ("st1_group", ">=", self.sgrp),
            ("st1_group", "<=", self.egrp)]
        if not self.itype == "A":
            whr.append(("st1_type", "=", self.itype))
        whr.extend([("unm_cono=st1_cono",), ("unm_unit=st1_uoi",)])
        all = self.sql.getRec(tables=["strmf1", "struoi"], cols=["st1_group",
            "st1_code", "st1_type", "st1_desc", "st1_uoi", "unm_desc"],
            where=whr, order="st1_group, st1_code")
        if not all:
            showError(self.mf.body, "Processing Error",
            "No Records Selected")
        else:
            self.printReport(all)
        self.closeProcess()

    def printReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        self.head = ("%03u %-30s %8s %33s %8s %6s" % \
            (self.conum, self.conam, "", self.sysdttm, "", __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        old_grp = ""
        self.pgnum = 0
        self.pglin = 99
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            grp = CCD(dat[0], "UA", 3)
            self.groupd = grp.disp
            code = CCD(dat[1], "NA", 20)
            itype = CCD(dat[2], "UA", 1)
            desc = CCD(dat[3], "UA", 30)
            uoi = CCD(dat[4], "NA", 10)
            umd = CCD(dat[5], "NA", 30)
            if old_grp and old_grp != grp.work:
                self.pglin = 99
            if self.pglin > self.lpp:
                if self.pgnum:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
            self.t.textLine("%s %s %s %s %s" % (code.disp, itype.disp,
                desc.disp, uoi.disp, umd.disp))
            self.pglin = self.pglin + 1
            old_grp = grp.work
        p.closeProgress()
        if self.t:
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
            head=self.tit, view=self.df.repprt[0][1],
            prtnam=self.df.repprt[0][2], mail=self.df.repprt[1][2])
            
    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-29s %-10s %45s %5s" % \
            ("Stores Master Code List as at", self.sysdtd, "Page", self.pgnum))
        self.t.textLine()
        self.t.textLine("%-5s %3s" % ("Group", self.groupd))
        self.t.textLine()
        self.t.textLine("%-20s %-1s %-30s %-39s" % \
            ("Product-Code", "T", "Description", "Unit of Issue"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 8

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

