#! /usr/bin/env python

"""
SYNOPSIS
    Purchase Orders Report.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import CCD, GetCtl, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, getModName, doPrinter, showError

class st3070:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["crsmst", "strloc", "strpom", "strpot"],
            prog=__name__)
        gc = GetCtl(self.mf)
        strctl = gc.getCtl("strctl", self.conum)
        if not strctl:
            return "strctl"
        self.locs = strctl["cts_locs"]
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdtd = "%i/%02i/%02i" % (t[0], t[1], t[2])
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.t = None
        self.maxlines = 20

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Purchase Orders (%s)" % __name__)
        loc = {
            "stype": "R",
            "tables": ("strloc",),
            "cols": (
                ("srl_loc", "", 0, "L"),
                ("srl_desc", "", 0, "Description", "Y")),
            "where": [("srl_cono", "=", self.conum)]}
        self.orm = {
            "stype": "R",
            "tables": ("strpom",),
            "cols": (
                ("pom_ordno", "", 0, "OrderNo"),
                ("pom_acno", "", 0, "Crs-Num"),
                ("pom_date", "", 0, "Order-Date")),
            "where": []}
        r1s = (("Yes", "Y"), ("No", "N"))
        if self.locs == "N":
            fld = [
            (("T",0,0,0),"OUA",1,"Location","Location",
                "1","Y",self.doLoc,loc,None,None)]
        else:
            fld = [
            (("T",0,0,0),"IUA",1,"Location","Location",
                "","Y",self.doLoc,loc,None,None)]
        fld.extend([
            (("T",0,1,0),("IRB",r1s),0,"Oustanding Only","Oustanding Only",
                "Y","Y",self.doTyp,None,None,None),
            (("T",0,2,0),"IUI",9,"From Order Number","From Order Number",
                "","Y",self.doOrd1,self.orm,None,None),
            (("T",0,3,0),"IUI",9,"To   Order Number","To Order Number",
                "","Y",self.doOrd2,self.orm,None,None)])
        tnd = ((self.doEnd,"y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doLoc(self, frt, pag, r, c, p, i, w):
        self.locw = w
        if not w:
            self.locd = "ALL"
        else:
            acc = self.sql.getRec(tables="strloc", cols=["srl_desc"],
                where=[("srl_cono", "=", self.conum), ("srl_loc", "=", w)],
                limit=1)
            if not acc:
                return "Invalid Location"
            self.locd = self.locw

    def doTyp(self, frt, pag, r, c, p, i, w):
        self.outs = w
        if self.outs == "Y":
            self.orm["where"] = [
                ("pom_cono", "=", self.conum),
                ("pom_delno", "=", "")]
        else:
            self.orm["where"] = [
                ("pom_cono", "=", self.conum)]

    def doOrd1(self, frt, pag, r, c, p, i, w):
        self.ord1 = w

    def doOrd2(self, frt, pag, r, c, p, i, w):
        if w:
            if w < self.ord1:
                return "Invalid Last Order < First Order"
            self.ord2 = w
        else:
            self.ord2 = 999999999

    def doEnd(self):
        self.df.closeProcess()
        whr = [("pom_cono", "=", self.conum)]
        if self.locw:
            whr.append(("pom_loc", "=", self.locw))
        if self.outs == "Y":
            whr.append(("pom_delno", "=", ""))
        else:
            whr.append(("pom_delno", "<>", "cancel"))
        whr.extend([("pom_ordno", ">=", self.ord1),
            ("pom_ordno", "<=", self.ord2)])
        all = self.sql.getRec(tables="strpom", cols=["pom_ordno", "pom_loc",
            "pom_date", "pom_acno", "pom_contact", "pom_deldt"], where=whr)
        if not all:
            showError(self.mf.body, "Selection Error",
            "No Orders Selected")
        else:
            self.printSetup(all)
            self.processRecords(all)
            self.c.save()
            doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
                head=self.tit, view=self.df.repprt[0][1],
                prtnam=self.df.repprt[0][2], mail=self.df.repprt[1][2])
        self.closeProcess()

    def printSetup(self, all):
        self.p = ProgressBar(self.mf.body, max=len(all), esc=True)
        self.head = ("%03u %-30s %46s %10s" % \
            (self.conum, self.conam, self.sysdttm, __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        self.pgnum = 0
        self.pglin = 99

    def processRecords(self, all):
        for num, dat in enumerate(all):
            can = self.p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            self.ord = CCD(dat[0], "Na", 9)
            self.pot = self.sql.getRec(tables="strpot", cols=["pot_group",
                "pot_code", "pot_uoi", "pot_desc", "pot_qty", "pot_price",
                "pot_disper"], where=[("pot_cono", "=", self.conum),
                ("pot_ordno", "=", self.ord.work)], order="pot_group, pot_code")
            self.loc = CCD(dat[1], "UA", 1)
            self.dto = CCD(dat[2], "d1", 10)
            self.crs = CCD(dat[3], "NA", 7)
            acc = self.sql.getRec(tables="crsmst", cols=["crm_name"],
                where=[("crm_cono", "=", self.conum), ("crm_acno", "=",
                self.crs.work)], limit=1)
            if not acc:
                self.nam = ""
            else:
                self.nam = acc[0]
            self.per = CCD(dat[4], "NA", 20)
            self.dte = CCD(dat[5], "d1", 10)
            self.printHeader()
            if self.pot:
                self.printBody()
            else:
                self.t.textLine("No Products on Order - Please Amend or Cancel")
            self.c.drawText(self.t)
            self.c.showPage()

        self.p.closeProgress()
            
    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        if self.outs == "Y":
            self.t.textLine("%-39s %3s %-5s %-10s %25s %5s" % \
            ("Outstanding Orders Report for Location", self.locd, "as at",
            self.sysdtd, "Page", self.pgnum))
        else:
            self.t.textLine("%-27s %3s %-5s %-10s %37s %5s" % \
            ("Orders Report for Location", self.locd, "as at",
            self.sysdtd, "Page", self.pgnum))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 3

    def printHeader(self):
        self.pageHeading()
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.t.textLine()
        text = "Order-Number: %-9s Supplier: %-7s %-30s Date: %-10s" % \
            (self.ord.disp, self.crs.disp, self.nam, self.dto.disp)
        if self.outs == "N" and self.dte.work:
            text = "%s Received: %s" % (text, self.dte.disp)
        self.t.textLine(text)
        self.t.textLine()
        self.t.textLine("%-3s %-20s %-1s %-30s %-10s %11s %10s" % ("Grp",
            "Product-Code", "L", "Description", "U.O.I", "U-Price", "Qty-Ord"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = self.pglin + 5

    def printBody(self):
        for dat in self.pot:
            grp = CCD(dat[0], "UA", 3)
            cod = CCD(dat[1], "NA", 20)
            uoi = CCD(dat[2], "NA", 10)
            des = CCD(dat[3], "NA", 30)
            qty = CCD(dat[4], "SD", 11.2)
            prc = CCD(dat[5], "UD", 11.2)
            if self.pglin > self.lpp:
                self.printHeader()
            self.t.textLine("%s %s %s %s %s %s %s" % (grp.disp, cod.disp,
                self.loc.disp, des.disp, uoi.disp, prc.disp, qty.disp))
            self.pglin = self.pglin + 1

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

