#! /usr/bin/env python

"""
SYNOPSIS
    Stores Stock On Hand Report.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, CCD, GetCtl, ProgressBar, Sql, TartanDialog
from tartanFunctions import doReportlab, doWriteExport, getModName, doPrinter
from tartanFunctions import mthendDate, showError

class st3080:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["strloc", "strgrp", "strmf1"],
            prog=__name__)
        gc = GetCtl(self.mf)
        strctl = gc.getCtl("strctl", self.conum)
        if not strctl:
            return "strctl"
        self.locs = strctl["cts_locs"]
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdtd = "%i/%02i/%02i" % (t[0], t[1], t[2])
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i) %6s" % (t[0],
            t[1], t[2], t[3], t[4], __name__)
        self.head = ("%03u %-30s %s" % (self.conum, self.conam, "%s"))
        self.colsh = ["Grp", "Product-Code", "Description", "U.O.I",
            "Qty-Balance", "Val-Balance"]
        self.forms = [("UA", 3), ("NA", 20), ("NA", 30), ("NA", 10),
            ("SD", 12.2), ("SD", 12.2)]
        self.gtot = 0
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Stock On Hand Report (%s)" % __name__)
        loc = {
            "stype": "R",
            "tables": ("strloc",),
            "cols": (
                ("srl_loc", "", 0, "L"),
                ("srl_desc", "", 0, "Description", "Y")),
            "where": [("srl_cono", "=", self.conum)]}
        grp = {
            "stype": "R",
            "tables": ("strgrp",),
            "cols": (
                ("gpm_group", "", 0, "Grp"),
                ("gpm_desc", "", 0, "Description", "Y")),
            "where": [("gpm_cono", "=", self.conum)]}
        r1s = (("Yes","Y"), ("No","N"))
        fld = [
            (("T",0,0,0),"ID2",7,"Reporting Period","Reporting Period",
                (self.sysdtw/100),"Y",self.doPeriod,None,None,("efld",)),
            [("T",0,1,0),"IUA",1,"Location","Location",
                "","Y",self.doLoc,loc,None,("efld",)],
            (("T",0,2,0),"IUA",3,"Product Group","Product Group",
                "","Y",self.doGroup,grp,None,None),
            (("T",0,3,0),("IRB",r1s),0,"Ignore Zero Balances",
                "Ignore Zero Balances","Y","Y",self.doZero,None,None,None)]
        if self.locs == "N":
            fld[1][1] = "OUA"
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("Y","V"), mail=("Y","N"))

    def doPeriod(self, frt, pag, r, c, p, i, w):
        if w > (self.sysdtw/100):
            return "Invalid Period"
        self.curdt = w
        self.date = CCD(mthendDate(w*100), "D1", 10)
        if self.locs == "N":
            self.df.loadEntry(frt, pag, p+1, data="1")
            no = self.doLoc(frt, pag, r, c+1, p+1, i+1, "1")
            if no:
                return no
            return "sk1"

    def doLoc(self, frt, pag, r, c, p, i, w):
        if not w:
            self.loc = ""
            self.locd = "ALL"
        else:
            acc = self.sql.getRec(tables="strloc", cols=["srl_desc"],
                where=[("srl_cono", "=", self.conum), ("srl_loc", "=", w)],
                limit=1)
            if not acc:
                return "Invalid Location"
            self.loc = w
            self.locd = acc[0]

    def doGroup(self, frt, pag, r, c, p, i, w):
        if not w:
            self.group = ""
        else:
            acc = self.sql.getRec(tables="strgrp", cols=["gpm_desc"],
                where=[("gpm_cono", "=", self.conum), ("gpm_group", "=", w)],
                limit=1)
            if not acc:
                return "Invalid Group"
            self.group = w

    def doZero(self, frt, pag, r, c, p, i, w):
        self.zero = w

    def doEnd(self):
        self.df.closeProcess()
        if not self.group:
            self.sgrp = ""
            self.egrp = "zzz"
        else:
            self.sgrp = self.group; self.egrp = self.group
        all = self.sql.getRec(tables="strmf1", cols=["st1_group", "st1_code",
            "st1_desc", "st1_uoi"], where=[("st1_cono", "=", self.conum),
            ("st1_group", ">=", self.sgrp), ("st1_group", "<=", self.egrp)],
            order="st1_group, st1_code")
        if not all:
            showError(self.mf.body, "Processing Error",
                "No Records Selected")
        elif self.df.repprt[0][2] == "export":
            self.exportReport(all)
        else:
            self.printReport(all)
        self.closeProcess()

    def exportReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        expnam = getModName(self.mf.wrkdir, __name__, self.conum)
        self.expheads = [self.head % self.sysdttm]
        self.expheads.append("Stock On Hand Report as at %s" % self.date.disp)
        self.expcolsh = [self.colsh]
        self.expforms = self.forms
        self.expdatas = []
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                p.closeProgress()
                return
            vals = self.getValues(dat)
            if not vals:
                continue
            self.expdatas.append(["BODY", [vals[0].work, vals[1].work,
                vals[2].work, vals[3].work, vals[4].work, vals[5].work]])
        p.closeProgress()
        self.grandTotal()
        doWriteExport(xtype=self.df.repprt[0][1], name=expnam, sheet=__name__,
            heads=self.expheads, colsh=self.expcolsh, forms=self.expforms,
            datas=self.expdatas, rcdic=self.mf.rcdic)

    def printReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        self.head = ("%03u %-30s %5s %33s %5s %6s" % (self.conum, self.conam,
            "", self.sysdttm, "", __name__))
        pdfnam = getModName(self.mf.wrkdir, __name__, self.conum, ext="pdf")
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, pdfnam, self.head)
        self.stot = 0
        old_grp = ""
        self.pgnum = 0
        self.pglin = 99
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            vals = self.getValues(dat)
            if not vals:
                continue
            grp, code, desc, uoi, qbal, vbal = vals
            if old_grp and old_grp != grp.work:
                self.groupTotal()
                self.pglin = 99
            if self.pglin > self.lpp:
                if self.pgnum:
                    self.c.drawText(self.t)
                    self.c.showPage()
                self.pageHeading()
            self.t.textLine("%s %s %s %s %s" % \
                (code.disp, desc.disp, uoi.disp, qbal.disp, vbal.disp))
            self.pglin = self.pglin + 1
            self.stot = float(ASD(self.stot) + ASD(vbal.work))
            old_grp = grp.work
        p.closeProgress()
        if self.t:
            if can != "yes":
                self.groupTotal()
                self.grandTotal()
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=pdfnam, head=self.tit,
            view=self.df.repprt[0][1], prtnam=self.df.repprt[0][2],
            mail=self.df.repprt[1][2])

    def getValues(self, data):
        grp = CCD(data[0], "UA", 3)
        self.groupd = grp.disp
        code = CCD(data[1], "NA", 20)
        desc = CCD(data[2], "NA", 30)
        uoi = CCD(data[3], "NA", 10)
        bal = self.sql.getRec(tables="strtrn",
            cols=["round(sum(stt_qty), 2)", "round(sum(stt_cost), 2)"],
            where=[("stt_cono", "=", self.conum), ("stt_group", "=",
            grp.work), ("stt_code", "=", code.work), ("stt_loc", "=",
            self.loc), ("stt_curdt", "<=", self.curdt)], limit=1)
        if not bal[0]:
            bal[0] = 0
        if not bal[1]:
            bal[1] = 0
        qbal = CCD(bal[0], "SD", 12.2)
        vbal = CCD(bal[1], "SD", 12.2)
        if self.zero == "Y" and qbal.work == 0 and vbal.work == 0:
            return
        self.gtot = float(ASD(self.gtot) + ASD(vbal.work))
        return (grp, code, desc, uoi, qbal, vbal)
            
    def pageHeading(self):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-26s %-10s %43s %5s" % ("Stock On Hand Report as at",
            self.date.disp, "Page", self.pgnum))
        self.t.textLine()
        self.t.textLine("%-5s %3s      %-8s %s  %s" % ("Group", self.groupd,
            "Location", self.loc, self.locd))
        self.t.textLine()
        self.t.textLine("%-20s %-30s %-10s %11s %12s" % ("Product-Code",
            "Description", "U.O.I", "Qty-Balance", "Val-Balance"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 8

    def groupTotal(self):
        val = CCD(self.stot, "SD", 12.2)
        self.t.textLine()
        self.t.textLine("%-20s %-41s %12s %12s" % \
            ("", "Group Totals", "", val.disp))
        self.t.textLine()
        self.stot = 0

    def grandTotal(self):
        if self.df.repprt[0][2] == "export":
            self.expdatas.append(["ULINES"])
            self.expdatas.append(["TOTAL", ["", "", "Grand Totals", "", "",
                self.gtot]])
            self.expdatas.append(["ULINED"])
            return
        val = CCD(self.gtot, "SD", 12.2)
        self.t.textLine("%-20s %-41s %12s %12s" % \
            (" ", "Grand Totals", "", val.disp))

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

