#! /usr/bin/env python

"""
SYNOPSIS
    Stores Account Statements.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, Balances, CCD, GetCtl, ProgressBar, Sql
from TartanClasses import TartanDialog
from tartanFunctions import doReportlab, getModName, doPrinter, showError
from tartanWork import sttrtp

class st3090:
    def __init__(self, mf, coy, nam, period):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.period = period
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["strloc", "strgrp", "strmf1"],
            prog=__name__)
        gc = GetCtl(self.mf)
        strctl = gc.getCtl("strctl", self.conum)
        if not strctl:
            return "strctl"
        self.locs = strctl["cts_locs"]
        self.pdfnam = getModName(self.mf.wrkdir, __name__, self.conum,
            ext="pdf")
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdtd = "%i/%02i/%02i" % (t[0], t[1], t[2])
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])
        self.t = None

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Stock Account Statements (%s)" % __name__)
        loc = {
            "stype": "R",
            "tables": ("strloc",),
            "cols": (
                ("srl_loc", "", 0, "L"),
                ("srl_desc", "", 0, "Description", "Y")),
            "where": [("srl_cono", "=", self.conum)]}
        grp = {
            "stype": "R",
            "tables": ("strgrp",),
            "cols": (
                ("gpm_group", "", 0, "Grp"),
                ("gpm_desc", "", 0, "Description", "Y")),
            "where": [("gpm_cono", "=", self.conum)]}
        r1s = (("Yes","Y"),("No","N"))
        fld = [
            (("T",0,0,0),"ID2",7,"Start Period","Start Period",
                (self.sysdtw/100),"Y",self.doSdate,None,None,("efld",)),
            (("T",0,1,0),"ID2",7,"End Period","End Period",
                (self.sysdtw/100),"Y",self.doEdate,None,None,("efld",)),
            [("T",0,2,0),"IUA",1,"Location","Location",
                "","Y",self.doLoc,loc,None,("efld",)],
            (("T",0,3,0),"IUA",3,"Product Group","Product Group",
                "","Y",self.doGroup,grp,None,None),
            (("T",0,4,0),("IRB",r1s),0,"New Account on New Page",
                "New Account on New Page","N","Y",self.doPage,None,None,None)]
        if self.locs == "N":
            fld[2][1] = "OUA"
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doSdate(self, frt, pag, r, c, p, i, w):
        self.sdate = w

    def doEdate(self, frt, pag, r, c, p, i, w):
        self.edate = w
        if self.locs == "N":
            self.df.loadEntry(frt, pag, p+1, data="1")
            no = self.doLoc(frt, pag, r, c+1, p+1, i+1, "1")
            if no:
                return no
            return "sk1"

    def doLoc(self, frt, pag, r, c, p, i, w):
        if not w:
            self.loc = ""
            self.locd = "ALL"
        else:
            acc = self.sql.getRec(tables="strloc", cols=["srl_desc"],
                where=[("srl_cono", "=", self.conum), ("srl_loc", "=", w)],
                limit=1)
            if not acc:
                return "Invalid Location"
            self.loc = w
            self.locd = acc[0]

    def doGroup(self, frt, pag, r, c, p, i, w):
        if not w:
            self.group = ""
        else:
            acc = self.sql.getRec(tables="strgrp", cols=["gpm_desc"],
                where=[("gpm_cono", "=", self.conum), ("gpm_group", "=", w)],
                limit=1)
            if not acc:
                return "Invalid Group"
            self.group = w

    def doPage(self, frt, pag, r, c, p, i, w):
        self.npag = w

    def doEnd(self):
        self.df.closeProcess()
        if not self.group:
            self.sgrp = ""
            self.egrp = "zzz"
        else:
            self.sgrp = self.group; self.egrp = self.group
        all = self.sql.getRec(tables="strmf1", cols=["st1_group", "st1_code",
            "st1_desc"], where=[("st1_cono", "=", self.conum), ("st1_group",
            ">=", self.sgrp), ("st1_group", "<=", self.egrp)],
            order="st1_group, st1_code")
        if not all:
            showError(self.mf.body, "Processing Error",
            "No Records Selected")
        else:
            self.printReport(all)
        self.closeProcess()

    def printReport(self, all):
        p = ProgressBar(self.mf.body, max=len(all), esc=True)
        self.head = ("%03u %-30s %7s %33s %7s %6s" % \
            (self.conum, self.conam, "", self.sysdttm, "", __name__))
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, self.pdfnam, self.head)
        lstgrp = ""
        lstcod = ""
        self.pgnum = 0
        self.pglin = 99
        for num, dat in enumerate(all):
            can = p.displayProgress(num)
            if can == "yes":
                if self.t:
                    self.t.textLine()
                    self.t.textLine("Print Job ABORTED")
                break
            grp = CCD(dat[0], "UA", 3)
            cod = CCD(dat[1], "NA", 20)
            desc = CCD(dat[2], "UA", 30)
            bals = Balances(self.mf, "STR", self.conum, self.edate,
                keys=(grp.work, cod.work, self.loc, ("P", self.period[0])))
            m_ob, m_mv, m_cb, y_ob, y_mv, y_cb, ac, lc, stt_rslt = \
                bals.doStrBals(self.sdate, trans="Y")
            self.qfwd = y_ob[0]
            self.vfwd = y_ob[1]
            trs = stt_rslt[0]
            dic = stt_rslt[1]
            if self.qfwd or self.vfwd or trs:
                if lstgrp and self.npag == "Y":
                    self.c.drawText(self.t)
                    self.c.showPage()
                    self.pageHeading(grp.disp, cod.disp, desc.disp)
                elif lstgrp or self.qfwd or self.vfwd:
                    self.accountHeading(grp.disp, cod.disp, desc.disp)
                lstgrp = grp.work
                lstcod = cod.work
            if trs:
                for z in trs:
                    dt  = CCD(z[dic["stt_trdt"][1]],  "d1", 10)
                    qt  = CCD(z[dic["stt_qty"][1]],   "SD", 12.2)
                    cs  = CCD(z[dic["stt_cost"][1]],  "SD", 12.2)
                    loc = CCD(z[dic["stt_loc"][1]],   "NA", 1)
                    bt  = CCD(z[dic["stt_batch"][1]], "Na", 7)
                    r1  = CCD(z[dic["stt_ref1"][1]],  "Na", 9)
                    r2  = CCD(z[dic["stt_ref1"][1]],  "Na", 7)
                    tp  = CCD(z[dic["stt_type"][1]],  "UI", 1)
                    if self.pglin > (self.lpp - 7):
                        if self.pgnum:
                            self.c.drawText(self.t)
                            self.c.showPage()
                        self.pageHeading(grp.disp, cod.disp, desc.disp)
                    self.qfwd = float(ASD(self.qfwd) + ASD(qt.work))
                    self.vfwd = float(ASD(self.vfwd) + ASD(cs.work))
                    qb = CCD(self.qfwd, "SD", 12.2)
                    vb = CCD(self.vfwd, "SD", 12.2)
                    self.t.textLine("%s %s %s %s %s %s %s %s %s" % \
                        (bt.disp, r1.disp, r2.disp, sttrtp[tp.work-1][0],
                        dt.disp, qt.disp, cs.disp, qb.disp, vb.disp))
                    self.pglin = self.pglin + 1
        p.closeProgress()
        if self.t and can != "yes":
            self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=self.pdfnam,
            head=self.tit, view=self.df.repprt[0][1],
            prtnam=self.df.repprt[0][2], mail=self.df.repprt[1][2])
            
    def pageHeading(self, grp, cod, desc):
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.pgnum = self.pgnum + 1
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-30s %-10s %43s %5s" % \
            ("Stock Account Statements as at", self.sysdtd, "Page", self.pgnum))
        self.t.textLine()
        self.t.textLine("%-21s%-7s%-3s%-9s%-7s%-1s" % \
            ("(Options: Start Date-", self.df.t_disp[0][0][0], "",
                "End Date-", self.df.t_disp[0][0][1], ")"))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.pglin = 5
        self.accountHeading(grp, cod, desc)

    def accountHeading(self, grp, cod, desc):
        if  self.pglin > (self.lpp - 10):
            if self.pgnum:
                if self.t:
                    self.c.drawText(self.t)
                self.c.showPage()
            self.pageHeading(grp, cod, desc)
        else:
            self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
            self.t.textLine("%s" % (self.suc * len(self.head)))
            self.t.textLine("%-5s %3s      %-7s %s      %-8s %s  %s" % \
                ("Group", grp, "Product", cod, "Location", self.loc, self.locd))
            self.t.textLine()
            self.t.textLine("%-7s %-9s %-7s %-3s %-10s %11s %12s %12s %12s" % \
                ("BatchNo", "Reference", "Refno-2", "Typ", "   Date",
                "Quantity", "Cost-Amt", "Qty-Balance", "Cst-Balance"))
            self.t.textLine("%s" % (self.suc * len(self.head)))
            self.pglin = self.pglin + 5
            self.t.setFont("Tartan", self.cpi, self.ldg)
            if not self.qfwd and not self.vfwd:
                return
            qb = CCD(self.qfwd, "SD", 12.2)
            vb = CCD(self.vfwd, "SD", 12.2)
            self.t.textLine("%-25s %-40s %s %s" % ("", "Fwd", qb.disp, vb.disp))
            self.pglin += 1

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

