#! /usr/bin/python

"""
SYNOPSIS
    Stores Revalue at Last Cost and Eradicate All Minus Balances.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, Balances, CCD, GetCtl, ProgressBar, Sql
from TartanClasses import TartanDialog
from tartanFunctions import showError

class st6020:
    def __init__(self, mf, coy, nam, period):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.period = period
        no = self.checkControls()
        if not no:
            self.setVariables()
            self.dataHeader()
            self.mf.startLoop()

    def checkControls(self):
        gc = GetCtl(self.mf)
        strctl = gc.getCtl("strctl", self.conum)
        if not strctl:
            return "no"
        self.glint = strctl["cts_glint"]
        self.locs = strctl["cts_locs"]
        if self.glint == "Y":
            ctlctl = gc.getCtl("ctlctl", self.conum)
            if not ctlctl:
                return "no"
            ctls = ["stk_soh", "stk_susp"]
            if gc.chkRec(self.conum, ctlctl, ctls):
                return "no"
            self.stk_soh = ctlctl["stk_soh"]
            self.stk_susp = ctlctl["stk_susp"]

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["gentrn", "strmf1", "strmf2", "strtrn"],
            prog=__name__)
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]

    def dataHeader(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "Stores Ledger Revaluation (%s)" % __name__)
        r1s = (("Average", "A"), ("Last", "L"))
        r2s = (("Yes", "Y"), ("No", "N"))
        fld = [
            (("T",0,0,0),"ID1",10,"Effective Date","",
                self.sysdtw,"N",self.doDate,None,None,("efld",)),
            (("T",0,1,0),("IRB",r1s),0,"Cost Method","",
                "L","N",self.doMethod,None,None,None),
            (("T",0,2,0),("IRB",r2s),0,"Clear Minuses","",
                "Y","N",self.doMinus,None,None,None)]
        tnd = ((self.endPage,"y"),)
        txt = (self.exitPage,)
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tend=tnd,
            txit=txt)

    def doDate(self, frt, pag, r, c, p, i, w):
        self.date = w
        self.curdt = w / 100

    def doMethod(self, frt, pag, r, c, p, i, w):
        self.method = w

    def doMinus(self, frt, pag, r, c, p, i, w):
        self.minus = w

    def endPage(self):
        self.df.closeProcess()
        sel = "Select stt_group, stt_code, stt_loc"
        dat = (self.conum,)
        all = self.sql.getRec(tables=["strmf1", "strmf2"], cols=["st2_group",
            "st2_code", "st2_loc"], where=[("st2_cono", "=", self.conum),
            ("st1_cono=st2_cono",), ("st1_group=st2_group",),
            ("st1_code=st2_code",), ("st1_value_ind", "<>", "N")],
            order="st2_group, st2_code")
        if not all:
            showError(self.mf.body, "Processing Error",
            "No Stock Records")
        else:
            self.cnt = 0
            p = ProgressBar(self.mf.body, max=len(all),
                typ="Valuation of Stock at Last Cost")
            for num, rec in enumerate(all):
                p.displayProgress(num)
                self.updateTables(rec)
            p.closeProgress()
            self.mf.dbm.commitDbase(ask=True)
        self.mf.closeLoop()

    def exitPage(self):
        self.df.closeProcess()
        self.mf.closeLoop()

    def updateTables(self, rec):
        bals = Balances(self.mf, "STR", self.conum, (self.sysdtw / 100),
            keys=(rec[0], rec[1], rec[2], ("P", self.period[0])))
        m_ob, m_mv, m_cb, y_ob, y_mv, y_cb, ac, lc = bals.doStrBals()
        if self.minus == "N" and not ac and not lc:
            return
        fqty = CCD(y_cb[0], "SD", 12.2)
        fval = CCD(y_cb[1], "SD", 12.2)
        if self.minus == "Y" and fqty.work < 0:
            nqty = 0
        else:
            nqty = fqty.work
        if self.method == "L" and lc:
            nval = nqty * lc
        elif self.method == "A" and ac:
            nval = nqty * ac
        elif lc:
            nval = nqty * lc
        else:
            nval = nqty * ac
        dqty = float(ASD(nqty) - ASD(fqty.work))
        dval = float(ASD(nval) - ASD(fval.work))
        if not dqty and not dval:
            return
        # Stores Ledger Transaction
        if  dqty >= 0:
            rtn = 5
        else:
            rtn = 6
        self.sql.insRec("strtrn", data=[self.conum, rec[0], rec[1], rec[2],
            self.date, rtn, self.cnt, "ST-RVAL", "", dqty, dval, 0, self.curdt,
            "", 0, "", "", "STR", 0, 0, 0, "", self.sysdtw, 0])
        # General Ledger Control Transaction (Stock On Hand)
        self.sql.insRec("gentrn", data=[self.conum, self.stk_soh, self.curdt,
            self.date, 4, self.cnt, "ST-RVAL", dval, 0, "", "N", "", 0,
            self.sysdtw, 0])
        # General Ledger Control Transaction (Stock Suspense)
        val = float(ASD(0) - ASD(dval))
        self.sql.insRec("gentrn", data=[self.conum, self.stk_susp, self.curdt,
            self.date, 4, self.cnt, "ST-RVAL", val, 0, "", "N", "", 0,
            self.sysdtw, 0])
        self.cnt = self.cnt + 1

