#! /usr/bin/env python

"""
SYNOPSIS
    Units Of Issue File Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import Sql, TartanDialog

class stc210:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, "struoi", prog=__name__)

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
                "Units Of Issue File Maintenance (%s)" % __name__)
        unm = {
            "stype": "R",
            "tables": ("struoi",),
            "cols": (
                ("unm_unit", "", 0, "Unit"),
                ("unm_desc", "", 0, "Description", "Y")),
            "where": [("unm_cono", "=", self.conum)]}
        r1s = (("Price","P"), ("Value","V"))
        self.fld = (
            (("T",0,0,0),"INA",10,"Unit Of Issue","Unit Of Issue",
                "","N",self.doUnit,unm,None,("notblank",)),
            (("T",0,1,0),"INA",30,"Description","Description",
                "","N",None,None,self.doDelete,("notblank",)))
        but = (
            ("Accept",None,self.doAccept,0,("T",0,2),("T",0,0)),
            ("Cancel",None,self.doCancel,0,("T",0,2),("T",0,0)))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.closeProcess, )
        abt = None
        self.df = TartanDialog(self.mf, title=tit, eflds=self.fld, butt=but,
            tend=tnd, txit=txt)

    def doUnit(self, frt, pag, r, c, p, i, w):
        self.uoi = w
        acc = self.sql.getRec(tables="struoi", cols=["unm_desc"],
            where=[("unm_cono", "=", self.conum), ("unm_unit", "=", w)],
            limit=1)
        if not acc:
            self.new = "Y"
        else:
            self.new = "N"
            for x in range(1, self.df.topq[pag]):
                self.df.loadEntry(frt, pag, p+x, data=acc[x-1])

    def doDelete(self):
        self.sql.delRec("struoi", where=[("unm_cono", "=", self.conum),
            ("unm_unit", "=", self.uoi)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doEnd(self):
        data = [self.conum]
        for x in range(0, len(self.df.t_work[0][0])):
            data.append(self.df.t_work[0][0][x])
        if self.new == "Y":
            self.sql.insRec("struoi", data=data)
        else:
            self.sql.updRec("struoi", data=data, where=[("unm_cono", "=",
            self.conum), ("unm_unit", "=", self.uoi)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doAccept(self, widget=None):
        frt, pag, col, mes = self.df.doCheckFields()
        if mes:
            self.df.focusField(frt, pag, (col+1), err=mes)
        else:
            self.df.doEndFrame("T", 0, cnf="N")

    def doCancel(self, widget=None):
        self.mf.dbm.rollbackDbase()
        self.df.focusField("T", 0, 1)

    def closeProcess(self):
        self.df.closeProcess()
        self.mf.closeLoop()

