#! /usr/bin/env python

"""
SYNOPSIS
    Product Groups File Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import GetCtl, Sql, TartanDialog

class stc310:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlvmf", "genmst", "strmf1", "strgrp"],
            prog=__name__)
        gc = GetCtl(self.mf)
        strctl = gc.getCtl("strctl", self.conum)
        if not strctl:
            return "no"
        self.glint = strctl["cts_glint"]

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
                "Product Groups File Maintenance (%s)" % __name__)
        gpm = {
            "stype": "R",
            "tables": ("strgrp",),
            "cols": (("gpm_group", "", 0, "Grp"),
                ("gpm_desc", "", 0, "Description", "Y")),
            "where": [("gpm_cono", "=", self.conum)]}
        vtm = {
            "stype": "R",
            "tables": ("ctlvmf",),
            "cols": (
                ("vtm_code", "", 0, "C"),
                ("vtm_desc", "", 0, "Description", "Y")),
            "where": [("vtm_cono", "=", self.conum)]}
        glm = {
            "stype": "R",
            "tables": ("genmst",),
            "cols": (
                ("glm_acno", "", 0, "Acc-Num"),
                ("glm_desc", "", 0, "Description", "Y")),
            "where": [("glm_cono", "=", self.conum)]}
        self.fld = [
            (("T",0,0,0),"IUA",3,"Product Group","Product Group",
                "","N",self.doGroup,gpm,None,("notblank",)),
            (("T",0,1,0),"INA",30,"Description","Description",
                "","N",None,None,self.doDelete,("notblank",)),
            (("T",0,2,0),"IUA",1,"Vat Code","Vat Code",
                "","N",self.doVat,vtm,None,("notblank",))]
        if self.glint == "Y":
            self.fld.append((("T",0,3,0),"IUI",7,"Sales Account",
                "Sales Account","","N",self.doSales,glm,None,("notzero",)))
            self.fld.append((("T",0,3,0),"ONA",30,""))
            self.fld.append((("T",0,4,0),"IUI",7,"COS Account",
                "COS Account","","N",self.doCos,glm,None,("notzero",)))
            self.fld.append((("T",0,4,0),"ONA",30,""))
            self.fld.append((("T",0,5,0),"IUD",5.1,"Mark-Up Percent",
                "Mark-Up Percent","","N",None,None,None,("efld",)))
            self.fld.append((("T",0,6,0),"IUD",5.1,"Reorder Factor",
                "Reorder Factor","","N",None,None,None,("efld",)))
        else:
            self.fld.append((("T",0,3,0),"IUD",5.1,"Mark-Up Percent",
                "Mark-Up Percent","","N",None,None,None,("efld",)))
            self.fld.append((("T",0,4,0),"IUD",5.1,"Reorder Factor",
                "Reorder Factor","","N",None,None,None,("efld",)))
        but = (
            ("Accept",None,self.doAccept,0,("T",0,2),("T",0,0)),
            ("Cancel",None,self.doCancel,0,("T",0,2),("T",0,0)))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doCloseProcess, )
        abt = None
        self.df = TartanDialog(self.mf, title=tit, eflds=self.fld, butt=but,
            tend=tnd, txit=txt)

    def doGroup(self, frt, pag, r, c, p, i, w):
        self.group = w
        acc = self.sql.getRec(tables="strgrp", where=[("gpm_cono", "=",
            self.conum), ("gpm_group", "=", self.group)], limit=1)
        if not acc:
            self.new = "Y"
        else:
            self.new = "N"
            col = self.sql.strgrp_col
            self.df.loadEntry(frt, pag, p+1, data=acc[col.index("gpm_desc")])
            self.df.loadEntry(frt, pag, p+2, data=acc[col.index("gpm_vatcode")])
            p = 3
            if self.glint == "Y":
                self.df.loadEntry(frt,pag,p,data=acc[col.index("gpm_sales")])
                des = self.getGenDesc(acc[col.index("gpm_sales")])
                if not des:
                    self.df.loadEntry(frt,pag,p+1,data="Invalid Sales Code")
                else:
                    self.df.loadEntry(frt,pag,p+1,data=des[0])
                self.df.loadEntry(frt,pag,p+2,data=acc[col.index("gpm_costs")])
                des = self.getGenDesc(acc[col.index("gpm_costs")])
                if not des:
                    self.df.loadEntry(frt,pag,p+3,data="Invalid Costs Code")
                else:
                    self.df.loadEntry(frt,pag,p+3,data=des[0])
                p = 7
            self.df.loadEntry(frt, pag, p, data=acc[col.index("gpm_markup")])
            self.df.loadEntry(frt, pag, p+1, data=acc[col.index("gpm_reorder")])

    def doVat(self, frt, pag, r, c, p, i, w):
        vat = self.sql.getRec(tables="ctlvmf", cols=["vtm_desc"],
            where=[("vtm_cono", "=", self.conum), ("vtm_code", "=", w)],
            limit=1)
        if not vat:
            return "Invalid VAT Code"
        self.code = w

    def doSales(self, frt, pag, r, c, p, i, w):
        acc = self.getGenDesc(w)
        if not acc:
            return "Invalid Sales Code"
        self.sales = w
        self.df.loadEntry("T", 0, 4, data=acc[0])

    def doCos(self, frt, pag, r, c, p, i, w):
        acc = self.getGenDesc(w)
        if not acc:
            return "Invalid COS Code"
        self.cos = w
        self.df.loadEntry("T", 0, 6, data=acc[0])

    def doDelete(self):
        st1 = self.sql.getRec(tables="strmf1", cols=["count(*)"],
            where=[("st1_cono", "=", self.conum), ("st1_group", "=",
            self.group)], limit=1)
        if st1[0]:
            return "Records Exist for this Group, Not Deleted"
        self.sql.delRec("strgrp", where=[("gpm_cono", "=", self.conum),
            ("gpm_group", "=", self.group)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doEnd(self):
        data = [self.conum]
        for x in range(0, len(self.df.t_work[0][0])):
            if self.glint == "Y" and x in (4, 6):
                continue
            data.append(self.df.t_work[0][0][x])
            if self.glint == "N" and x == 2:
                data.append(0)
                data.append(0)
        if self.new == "Y":
            self.sql.insRec("strgrp", data=data)
        else:
            self.sql.updRec("strgrp", data=data, where=[("gpm_cono", "=",
            self.conum), ("gpm_group", "=", self.group)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doAccept(self, widget=None):
        frt, pag, col, mes = self.df.doCheckFields()
        if mes:
            self.df.focusField(frt, pag, (col+1), err=mes)
        else:
            self.df.doEndFrame("T", 0, cnf="N")

    def getGenDesc(self, acno):
        return self.sql.getRec(tables="genmst", cols=["glm_desc"],
            where=[("glm_cono", "=", self.conum), ("glm_acno", "=", acno)],
            limit=1)

    def doCancel(self, widget=None):
        self.mf.dbm.rollbackDbase()
        self.df.focusField("T", 0, 1)

    def doCloseProcess(self):
        self.df.closeProcess()
        self.mf.closeLoop()

