#! /usr/bin/env python

"""
SYNOPSIS
    Product Groups Listing.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import GetCtl, RepPrt, Sql, TartanDialog

class stc320:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["genmst", "strgrp"], prog=__name__)
        gc = GetCtl(self.mf)
        strctl = gc.getCtl("strctl", self.conum)
        if not strctl:
            return "strctl"
        self.glint = strctl["cts_glint"]
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "Product Groups Master Listing (%s)" % __name__)
        r1s = (("Yes","Y"),("No","N"))
        fld = ()
        tnd = ((self.doEnd,"Y"), )
        txt = (self.closeProcess, )
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"))

    def doEnd(self):
        tables = ["strgrp"]
        heads = ["Product Groups Master Listing"]
        grps = self.sql.getRec(tables="strgrp", cols=["gpm_group",
            "gpm_desc", "gpm_vatcode", "gpm_sales", "gpm_costs",
            "gpm_reorder"], where=[("gpm_cono", "=", self.conum)],
            order="gpm_group")
        data = []
        for grp in grps:
            if self.glint == "Y":
                ac1 = self.getGenDesc(grp[3])
                if not ac1:
                    ac1 = [""]
                ac2 = self.getGenDesc(grp[4])
                if not ac2:
                    ac2 = [""]
            else:
                ac1 = ac2 = [""]
            gp = grp[:4]
            gp.extend([ac1[0], grp[4], ac2[0], grp[5]])
            data.append(gp)
        cols = [
            ["a","UA",3.0,"Grp"],
            ["b","NA",30.0,"Description"],
            ["c","UA",1.0,"V"],
            ["d","UI",7.0,"SLS-Acc"],
            ["e","NA",30.0,"Description"],
            ["f","UI",7.0,"COS-Acc"],
            ["g","NA",30.0,"Description"],
            ["h","UD",5.1,"Reord"]]
        state = self.df.disableButtons()
        rp = RepPrt(self.mf, name=__name__, tables=data, heads=heads,
            cols=cols, conum=self.conum, conam=self.conam, ttype="D",
            view=self.df.repprt[0][1], prtnam=self.df.repprt[0][2])
        self.df.enableButtons(state=state)
        self.closeProcess()

    def getGenDesc(self, acc):
        des = self.sql.getRec(tables="genmst", cols=["glm_desc"],
            where=[("glm_cono", "=", self.conum), ("glm_acno", "=", acc)],
            limit=1)
        return des

    def closeProcess(self):
        self.df.closeProcess()
        self.mf.closeLoop()

