"""
SYNOPSIS
    These are various constants and variables used in TARTAN.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

#--------
# Systems
#--------
finsys = {
    "ASS": ["Asset Register", "AR", "Y"],
    "BWL": ["Bowling Clubs", "BC", "Y"],
    "CRS": ["Creditors Ledger", "CR", "Y"],
    "DRS": ["Debtors Ledger", "DR", "Y"],
    "GEN": ["General Ledger", "GL", "Y"],
    "INV": ["Sales Invoicing", "SI", "Y"],
    "MEM": ["Members Ledger", "ML", "Y"],
    "MST": ["Master Control", "MS", "Y"],
    "RCA": ["Rentals Ledger (Extended)", "RC", "Y"],
    "RTL": ["Rentals Ledger (Basic)", "RT", "Y"],
    "SLN": ["Staff Loans", "SL", "Y"],
    "STR": ["Stores Ledger", "ST", "Y"],
    "TEL": ["Telehone Directory", "TD", "N"],
    "WAG": ["Salaries and Wages", "WG", "Y"]}
#---------------------
# Month Names and Days
#---------------------
mthnam = [(),
    ("Jan", "January", 31),
    ["Feb", "February", 28],
    ("Mar", "March", 31),
    ("Apr", "April", 30),
    ("May", "May", 31),
    ("Jun", "June", 30),
    ("Jul", "July", 31),
    ("Aug", "August", 31),
    ("Sep", "September", 30),
    ("Oct", "October", 31),
    ("Nov", "November", 30),
    ("Dec", "December", 31)]
#-------------
# Record Types
#-------------
# General Ledger
#---------------
gltrtp = [
    ("Sls", "Sale"),
    ("Pmt", "Payment"),
    ("P/C", "Petty Cash"),
    ("J/E", "Journal Entry"),
    ("Pur", "Purchase"),
    ("Rec", "Receipt"),
    ("Bnk", "Bank Statement")]
#
# Creditor"s Ledger
#------------------
crtrtp = [
    ("Inv", "Invoice"),
    ("Rec", "Receipt"),
    ("J/E", "Journal"),
    ("C/N", "Credit Note"),
    ("Pay", "Payment"),
    ("Dis", "Set-Discount"),
    ("Inc", "Inc-Discount"),
    ("Adv", "Adv-Discount")]
#
# Debtor"s Ledger
#----------------
drtrtp = [
    ("Inv", "Invoice"),
    ("Rec", "Receipt"),
    ("J/E", "Journal"),
    ("C/N", "Credit Note"),
    ("Pay", "Payment"),
    ("Dis", "Set-Discount")]
#
# Stores Ledger
#--------------
sttrtp = [
    ("Rec", "Receipt"),
    ("Iss", "Issue"),
    ("Tfi", "Transfer In"),
    ("Tfo", "Transfer Out"),
    ("Jli", "Journal In"),
    ("Jlo", "Journal Out"),
    ("Sal", "Sale")]
#
# Assets Register
#----------------
artrtp = [
    ("Pmt", "Payment"),
    ("Rec", "Receipt"),
    ("J/E", "Journal Entry"),
    ("Dep", "Depreciation")]
#
armvtp = [
    ("Pur", "New Purchase"),
    ("Imp", "Improvement"),
    ("W/O", "Write Off"),
    ("Dep", "Depreciation"),
    ("Sal", "Sale of Asset")]
#
# Rental Collection Agency
#-------------------------
rctrtp = [
    ("Rtl", "Rental"),
    ("Rec", "Receipt"),
    ("Pmt", "Payment"),
    ("J/E", "Journal Entry")]
rcmvtp = [
    ("Rtl", "Rental"),
    ("Dep", "Deposit"),
    ("Fee", "Contract Fee"),
    ("Srv", "Services (Owner)"),
    ("Srv", "Services (Agency)"),
    ("Rep", "Repairs  (Agency)")]
#
# Rentals Ledger
#---------------
rttrtp = [
    ("Rtl", "Rental"),
    ("Rec", "Receipt"),
    ("Pmt", "Payment"),
    ("J/E", "Journal Entry")]
#
# Staff Loans
#------------
sltrtp = [
    ("Int", "Interest"),
    ("Lon", "Loans"),
    ("Adv", "Advance"),
    ("Pay", "Payment"),
    ("Adj", "Adjustment")]
#
# Member's Ledger
#----------------
mltrtp = [
    ("Inv", "Invoice"),
    ("Pmt", "Payment"),
    ("J/E", "Journal Entry"),
    ("C/N", "Credit Note"),
    ("Rec", "Receipt"),
    ("Dis", "Set-Discount")]
#-----------------------------
# Password Controlled Routines
#-----------------------------
pwctrl = (
    ("MST", "ChgBatch", "Allow Changes to Batch Totals"),
    ("MST", "MultiDate", "Allow Multiple Date Allocations"),
    ("MST", "Super", "Supervisor Password"),
    ("MST", "TarBck", "Allow Database Backup"),
    ("MST", "TarRes", "Allow Database Restore"),
    ("MST", "UnbalBatch", "Allow Override of Unbalanced Batches"),
    ("MST", "YearEnd", "Allow Year End to be Run for a Finalised Period"),
    ("GEN", "CtlAcc", "Allow Postings to Control Accounts"),
    ("GEN", "NoPost", "Allow Postings to Locked Accounts"),
    ("GEN", "Super", "Supervisor Password"),
    ("CRS", "NewAcc", "Allow Capture of New Account"),
    ("CRS", "Super", "Supervisor Password"),
    ("DRS", "CrLimit", "Allow Overide of Credit Limit"),
    ("DRS", "NewAcc", "Allow Capture of New Account"),
    ("DRS", "RefLimit", "Allow Override of Referral Limit"),
    ("DRS", "RejLimit", "Allow Override of Rejection Limit"),
    ("DRS", "Super", "Supervisor Password"),
    ("MEM", "PropSec", "Blank Proposer and/or Seconder"),
    ("MEM", "FinPer", "Override Financial Period Dates"),
    ("MEM", "Super", "Supervisor Password"),
    ("RCA", "Overpaid", "Allow Override of Overpayment"),
    ("RTL", "NewAcc", "Allow Capture of New Account"),
    ("RTL", "Super", "Supervisor Password"),
    ("STR", "ExQty", "Allow Overide of Quantity Limit"),
    ("STR", "NewAcc", "Allow Capture of New Account"),
    ("STR", "Super", "Supervisor Password"),
    ("INV", "BelowCost", "Allow Sale Below Cost"),
    ("INV", "ChangeRep", "Allow Overide of Rep on Debtor Record"),
    ("INV", "ExQty", "Allow Overide of Quantity Limit"),
    ("INV", "Invoices", "Allow Invoices and Credit Notes"),
    ("INV", "Super", "Supervisor Password"))
#-----------
# Data Types
#-----------
dattyp = (
    ("D0", "Date (CCYYMMDD)"),
    ("D1", "Date (CCYYMMDD)"),
    ("d1", "Date (CCYYMMDD) Alowing Zero"),
    ("D2", "Current Period Date (CCYYMM)"),
    ("d2", "Current Period Date (CCYYMM) Allowing Zero"),
    ("D3", "Date (DD-MMM-YYYY)"),
    ("DT", "Date and Time (DD-MMM-YYYY-HH:MM)"),
    ("FF", "File or Directory Format"),
    ("HA", "Normal Alphanumeric Hidden as for Passwords"),
    ("NA", "Normal Alphanumeric"),
    ("Na", "Normal Alphanumeric Right Justified"),
    ("RW", "Raw Data, No formatting or checking"),
    ("SD", "Signed Decimal"),
    ("Sd", "Convert Integer to Signed Decimal"),
    ("SI", "Signed Integer"),
    ("SL", "Signed Long Integer"),
    ("TM", "Time (HH:MM)"),
    ("TS", "Timestamp (CCYY-MM-DD HH:MM:SS)"),
    ("TV", "Text or View"),
    ("TX", "Text or Blob Alphanumeric Variable Length"),
    ("UA", "Upper Case Alphanumeric"),
    ("Ua", "Upper Case Alphanumeric Right Justified"),
    ("UD", "Unsigned Decimal"),
    ("Ud", "Convert Integer to Unsigned Decimal"),
    ("UI", "Unsigned Integer"),
    ("UL", "Unsigned Long Integer"),
    ("US", "Unsigned Serial"))
#----------
# Tax rates
#----------
payeRates = {
    "2004": {
        "rates": [
            [270000, 78070, 40],
            [195000, 49570, 38],
            [155000, 35570, 35],
            [115000, 23570, 30],
            [74000, 13320, 25],
            [0, 0, 18]],
        "allow": [5800, 3200, 30000]},
    "2005": {
        "rates": [
            [270000, 78070, 40],
            [195000, 49570, 38],
            [155000, 35570, 35],
            [115000, 23570, 30],
            [74000, 13320, 25],
            [0, 0, 18]],
        "allow": [5800, 3200, 30000]},
    "2006": {
        "rates": [
            [300000, 86000, 40],
            [230000, 59400, 38],
            [180000, 41900, 35],
            [130000, 26900, 30],
            [80000, 14400, 25],
            [0, 0, 18]],
        "allow": [6300, 4500, 30000]},
    "2007": {
        "rates": [
            [400000, 117000, 40],
            [300000, 79000, 38],
            [220000, 51000, 35],
            [160000, 33000, 30],
            [100000, 18000, 25],
            [0, 0, 18]],
        "allow": [7200, 4500, 30000]},
    "2008": {
        "rates": [
            [450000, 131125, 40],
            [350000, 93125, 38],
            [250000, 58125, 35],
            [180000, 37125, 30],
            [112500, 20250, 25],
            [0, 0, 18]],
        "allow": [7740, 4680, 30000]},
    "2009": {
        "rates": [
            [490000, 143010, 40],
            [380000, 101210, 38],
            [270000, 62710, 35],
            [195000, 40210, 30],
            [122000, 21960, 25],
            [0, 0, 18]],
        "allow": [8280, 5040, 30000]},
    "2010": {
        "rates": [
            [525000, 152960, 40],
            [410000, 109260, 38],
            [290000, 67260, 35],
            [210000, 43260, 30],
            [132000, 23760, 25],
            [0, 0, 18]],
        "allow": [9756, 5400, 30000]},
    "2011": {
        "rates": [
            [552000, 160730, 40],
            [431000, 114750, 38],
            [305000, 70650, 35],
            [221000, 45450, 30],
            [140000, 25200, 25],
            [0, 0, 18]],
        "allow": [10260, 5675, 30000]},
    "2012": {
        "rates": [
            [580000, 168250, 40],
            [455000, 120750, 38],
            [325000, 75250, 35],
            [235000, 48250, 30],
            [150000, 27000, 25],
            [0, 0, 18]],
        "allow": [10755, 6012, 2000, 0]},
    "2013": {
        "rates": [
            [617000, 178940, 40],
            [484000, 128400, 38],
            [346000, 80100, 35],
            [250000, 51300, 30],
            [160000, 28800, 25],
            [0, 0, 18]],
        "allow": [11440, 6390, 2130, 0]}}
siteRates = {
    "2004": 60000,
    "2005": 60000,
    "2006": 60000,
    "2007": 60000,
    "2008": 60000,
    "2009": 60000,
    "2010": 60000,
    "2011": 60000,
    "2012": 60000,
    "2012": 60000}
#==========
# Countries
#==========
countries = {
    "":   ("", "Unknown"),
    "AF": ("AFG", "Afghanistan"),
    "AX": ("ALA", "Aland Islands"),
    "AL": ("ALB", "Albania"),
    "DZ": ("DZA", "Algeria"),
    "AS": ("ASM", "American Samoa"),
    "AD": ("AND", "Andorra"),
    "AO": ("AGO", "Angola"),
    "AI": ("AIA", "Anguilla"),
    "AQ": ("ATA", "Antarctica"),
    "AG": ("ATG", "Antigua and Barbuda"),
    "AR": ("ARG", "Argentina"),
    "AM": ("ARM", "Armenia"),
    "AW": ("ABW", "Aruba"),
    "AU": ("AUS", "Australia"),
    "AT": ("AUT", "Austria"),
    "AZ": ("AZE", "Azerbaijan"),
    "BS": ("BHS", "Bahamas"),
    "BH": ("BHR", "Bahrain"),
    "BD": ("BGD", "Bangladesh"),
    "BB": ("BRB", "Barbados"),
    "BY": ("BLR", "Belarus"),
    "BE": ("BEL", "Belgium"),
    "BZ": ("BLZ", "Belize"),
    "BJ": ("BEN", "Benin"),
    "BM": ("BMU", "Bermuda"),
    "BT": ("BTN", "Bhutan"),
    "BO": ("BOL", "Bolivia"),
    "BA": ("BIH", "Bosnia and Herzegovina"),
    "BW": ("BWA", "Botswana"),
    "BV": ("BVT", "Bouvet Island"),
    "BR": ("BRA", "Brazil"),
    "IO": ("IOT", "British Indian Ocean Territory"),
    "BN": ("BRN", "Brunei Darussalam"),
    "BG": ("BGR", "Bulgaria"),
    "BF": ("BFA", "Burkina Faso"),
    "BI": ("BDI", "Burundi"),
    "KH": ("KHM", "Cambodia"),
    "CM": ("CMR", "Cameroon"),
    "CA": ("CAN", "Canada"),
    "CV": ("CPV", "Cape Verde"),
    "KY": ("CYM", "Cayman Islands"),
    "CF": ("CAF", "Central African Republic"),
    "TD": ("TCD", "Chad"),
    "CL": ("CHL", "Chile"),
    "CN": ("CHN", "China"),
    "CX": ("CXR", "Christmas Island"),
    "CC": ("CCK", "Cocos (Keeling) Islands"),
    "CO": ("COL", "Colombia"),
    "KM": ("COM", "Comoros"),
    "CG": ("COG", "Congo"),
    "CD": ("COD", "Congo, The Democratic Republic of the"),
    "CK": ("COK", "Cook Islands"),
    "CR": ("CRI", "Costa Rica"),
    "CI": ("CIV", "Cote d'Ivoire"),
    "HR": ("HRV", "Croatia"),
    "CU": ("CUB", "Cuba"),
    "CY": ("CYP", "Cyprus"),
    "CZ": ("CZE", "Czech Republic"),
    "DK": ("DNK", "Denmark"),
    "DJ": ("DJI", "Djibouti"),
    "DM": ("DMA", "Dominica"),
    "DO": ("DOM", "Dominican Republic"),
    "EC": ("ECU", "Ecuador"),
    "EG": ("EGY", "Egypt"),
    "SV": ("SLV", "El Salvador"),
    "GQ": ("GNQ", "Equatorial Guinea"),
    "ER": ("ERI", "Eritrea"),
    "EE": ("EST", "Estonia"),
    "ET": ("ETH", "Ethiopia"),
    "FK": ("FLK", "Falkland Islands (Malvinas)"),
    "FO": ("FRO", "Faroe Islands"),
    "FJ": ("FJI", "Fiji"),
    "FI": ("FIN", "Finland"),
    "FR": ("FRA", "France"),
    "GF": ("GUF", "French Guiana"),
    "PF": ("PYF", "French Polynesia"),
    "TF": ("ATF", "French Southern Territories"),
    "GA": ("GAB", "Gabon"),
    "GM": ("GMB", "Gambia"),
    "GE": ("GEO", "Georgia"),
    "DE": ("DEU", "Germany"),
    "GH": ("GHA", "Ghana"),
    "GI": ("GIB", "Gibraltar"),
    "GR": ("GRC", "Greece"),
    "GL": ("GRL", "Greenland"),
    "GD": ("GRD", "Grenada"),
    "GP": ("GLP", "Guadeloupe"),
    "GU": ("GUM", "Guam"),
    "GT": ("GTM", "Guatemala"),
    "GG": ("GGY", "Guernsey"),
    "GN": ("GIN", "Guinea"),
    "GW": ("GNB", "Guinea-Bissau"),
    "GY": ("GUY", "Guyana"),
    "HT": ("HTI", "Haiti"),
    "HM": ("HMD", "Heard Island and McDonald Islands"),
    "VA": ("VAT", "Holy See (Vatican City State)"),
    "HN": ("HND", "Honduras"),
    "HK": ("HKG", "Hong Kong"),
    "HU": ("HUN", "Hungary"),
    "IS": ("ISL", "Iceland"),
    "IN": ("IND", "India"),
    "ID": ("IDN", "Indonesia"),
    "IR": ("IRN", "Iran, Islamic Republic of"),
    "IQ": ("IRQ", "Iraq"),
    "IE": ("IRL", "Ireland"),
    "IM": ("IMN", "Isle of Man"),
    "IL": ("ISR", "Israel"),
    "IT": ("ITA", "Italy"),
    "JM": ("JAM", "Jamaica"),
    "JP": ("JPN", "Japan"),
    "JE": ("JEY", "Jersey"),
    "JO": ("JOR", "Jordan"),
    "KZ": ("KAZ", "Kazakhstan"),
    "KE": ("KEN", "Kenya"),
    "KI": ("KIR", "Kiribati"),
    "KP": ("PRK", "Korea, Democratic People's Republic of"),
    "KR": ("KOR", "Korea, Republic of"),
    "KW": ("KWT", "Kuwait"),
    "KG": ("KGZ", "Kyrgyzstan"),
    "LA": ("LAO", "Lao People's Democratic Republic"),
    "LV": ("LVA", "Latvia"),
    "LB": ("LBN", "Lebanon"),
    "LS": ("LSO", "Lesotho"),
    "LR": ("LBR", "Liberia"),
    "LY": ("LBY", "Libyan Arab Jamahiriya"),
    "LI": ("LIE", "Liechtenstein"),
    "LT": ("LTU", "Lithuania"),
    "LU": ("LUX", "Luxembourg"),
    "MO": ("MAC", "Macao"),
    "MK": ("MKD", "Macedonia, Republic of"),
    "MG": ("MDG", "Madagascar"),
    "MW": ("MWI", "Malawi"),
    "MY": ("MYS", "Malaysia"),
    "MV": ("MDV", "Maldives"),
    "ML": ("MLI", "Mali"),
    "MT": ("MLT", "Malta"),
    "MH": ("MHL", "Marshall Islands"),
    "MQ": ("MTQ", "Martinique"),
    "MR": ("MRT", "Mauritania"),
    "MU": ("MUS", "Mauritius"),
    "YT": ("MYT", "Mayotte"),
    "MX": ("MEX", "Mexico"),
    "FM": ("FSM", "Micronesia, Federated States of"),
    "MD": ("MDA", "Moldova"),
    "MC": ("MCO", "Monaco"),
    "MN": ("MNG", "Mongolia"),
    "ME": ("MNE", "Montenegro"),
    "MS": ("MSR", "Montserrat"),
    "MA": ("MAR", "Morocco"),
    "MZ": ("MOZ", "Mozambique"),
    "MM": ("MMR", "Myanmar"),
    "NA": ("NAM", "Namibia"),
    "NR": ("NRU", "Nauru"),
    "NP": ("NPL", "Nepal"),
    "NL": ("NLD", "Netherlands"),
    "AN": ("ANT", "Netherlands Antilles"),
    "NC": ("NCL", "New Caledonia"),
    "NZ": ("NZL", "New Zealand"),
    "NI": ("NIC", "Nicaragua"),
    "NE": ("NER", "Niger"),
    "NG": ("NGA", "Nigeria"),
    "NU": ("NIU", "Niue"),
    "NF": ("NFK", "Norfolk Island"),
    "MP": ("MNP", "Northern Mariana Islands"),
    "NO": ("NOR", "Norway"),
    "OM": ("OMN", "Oman"),
    "PK": ("PAK", "Pakistan"),
    "PW": ("PLW", "Palau"),
    "PS": ("PSE", "Palestinian Territory, Occupied"),
    "PA": ("PAN", "Panama"),
    "PG": ("PNG", "Papua New Guinea"),
    "PY": ("PRY", "Paraguay"),
    "PE": ("PER", "Peru"),
    "PH": ("PHL", "Philippines"),
    "PN": ("PCN", "Pitcairn"),
    "PL": ("POL", "Poland"),
    "PT": ("PRT", "Portugal"),
    "PR": ("PRI", "Puerto Rico"),
    "QA": ("QAT", "Qatar"),
    "RE": ("REU", "Reunion"),
    "RO": ("ROU", "Romania"),
    "RU": ("IRB", "Russian Federation"),
    "RW": ("RWA", "Rwanda"),
    "BL": ("BLM", "Saint Barthelemy"),
    "SH": ("SHN", "Saint Helena"),
    "KN": ("KNA", "Saint Kitts and Nevis"),
    "LC": ("LCA", "Saint Lucia"),
    "MF": ("MAF", "Saint Martin (French part)"),
    "PM": ("SPM", "Saint Pierre and Miquelon"),
    "VC": ("VCT", "Saint Vincent and the Grenadines"),
    "WS": ("WSM", "Samoa"),
    "SM": ("SMR", "San Marino"),
    "ST": ("STP", "Sao Tome and Principe"),
    "SA": ("SAU", "Saudi Arabia"),
    "SN": ("SEN", "Senegal"),
    "RS": ("SRB", "Serbia"),
    "SC": ("SYC", "Seychelles"),
    "SL": ("SLE", "Sierra Leone"),
    "SG": ("SGP", "Singapore"),
    "SK": ("SVK", "Slovakia"),
    "SI": ("SVN", "Slovenia"),
    "SB": ("SLB", "Solomon Islands"),
    "SO": ("SOM", "Somalia"),
    "ZA": ("ZAF", "South Africa"),
    "GS": ("SGS", "South Georgia and the South Sandwich Islands"),
    "ES": ("ESP", "Spain"),
    "LK": ("LKA", "Sri Lanka"),
    "SD": ("SDN", "Sudan"),
    "SR": ("SUR", "Suriname"),
    "SJ": ("SJM", "Svalbard and Jan Mayen"),
    "SZ": ("SWZ", "Swaziland"),
    "SE": ("SWE", "Sweden"),
    "CH": ("CHE", "Switzerland"),
    "SY": ("SYR", "Syrian Arab Republic"),
    "TW": ("TWN", "Taiwan, Province of China"),
    "TJ": ("TJK", "Tajikistan"),
    "TZ": ("TZA", "Tanzania, United Republic of"),
    "TH": ("THA", "Thailand"),
    "TL": ("TLS", "Timor-Leste"),
    "TG": ("TGO", "Togo"),
    "TK": ("TKL", "Tokelau"),
    "TO": ("TON", "Tonga"),
    "TT": ("TTO", "Trinidad and Tobago"),
    "TN": ("TUN", "Tunisia"),
    "TR": ("TUR", "Turkey"),
    "TM": ("TKM", "Turkmenistan"),
    "TC": ("TCA", "Turks and Caicos Islands"),
    "TV": ("TUV", "Tuvalu"),
    "UG": ("UGA", "Uganda"),
    "UA": ("UKR", "Ukraine"),
    "AE": ("ARE", "United Arab Emirates"),
    "GB": ("GBR", "United Kingdom"),
    "US": ("USA", "United States"),
    "UM": ("UMI", "United States Minor Outlying Islands"),
    "UY": ("URY", "Uruguay"),
    "UZ": ("UZB", "Uzbekistan"),
    "VU": ("VUT", "Vanuatu"),
    "VE": ("VEN", "Venezuela"),
    "VN": ("VNM", "Viet Nam"),
    "VG": ("VGB", "Virgin Islands, British"),
    "VI": ("VIR", "Virgin Islands, U.S."),
    "WF": ("WLF", "Wallis and Futuna"),
    "EH": ("ESH", "Western Sahara"),
    "YE": ("YEM", "Yemen"),
    "ZM": ("ZMB", "Zambia"),
    "ZW": ("ZWE", "Zimbabwe")}
# END
