#! /usr/bin/python

"""
SYNOPSIS
    Tartan Systems Table Creation GUI.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import os
from TartanClasses import FileDialog, RepPrt, Sql, TartanDialog
from tartanFunctions import askQuestion, makeArray, showError
from tartanWork import dattyp

class tb1010:
    def __init__(self, mf, sys):
        self.mf = mf
        self.sys = sys
        self.mf.dbm.openDbase()
        no = self.setVariables()
        if not no:
            self.doProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ftable", "ffield"], prog=__name__)
        if self.sql.error:
            return self.sql.error

    def doProcess(self):
        tit = ("Tartan Systems Table Maintenance",)
        fft = {
            "stype": "R",
            "tables": ("ftable",),
            "cols": (
                ("ft_tabl", "", 0, "Table"),
                ("ft_desc", "", 0, "Description", "Y")),
            "where": [("ft_seq", "=", 1)]}
        ffl = {
            "stype": "R",
            "tables": ("ffield",),
            "cols": (
                ("ff_seq", "", 0, "Seq"),
                ("ff_name", "", 0, "Name"),
                ("ff_desc", "", 0, "Description", "Y"),
                ("ff_type", "", 0, "Tp"),
                ("ff_size", "", 0, "Size")),
            "whera": (("T", "ff_tabl", 0, 0),),
            "index": 0}
        ffi = {
            "stype": "R",
            "tables": ("ftable",),
            "cols": (
                ("ft_seq", "", 0, "Seq"),
                ("ft_type", "", 0, "T"),
                ("ft_key0", "", 0, "Column-1"),
                ("ft_key1", "", 0, "Column-2"),
                ("ft_key2", "", 0, "Column-3"),
                ("ft_key3", "", 0, "Column-4"),
                ("ft_key4", "", 0, "Column-5"),
                ("ft_key4", "", 0, "Column-6"),
                ("ft_key4", "", 0, "Column-7")),
            "whera": (("T", "ft_tabl", 0, 0),),
            "comnd": self.doIdxCmd}
        typ = {
            "stype": "C",
            "titl": "Select the Required Data Type",
            "data": dattyp,
            "index": 0}
        valtyp = []
        for t in dattyp:
            valtyp.append(t[0])
        self.col = {
            "stype": "C",
            "titl": "Select the Required Column",
            "data": []}
        fld = [
            (("T",0,0,0),"INA",20,"Table ","Table Name",
                "","Y",self.doTable,fft,None,("notin",("","ftable","ffield"))),
            (("T",0,0,0),"INA",30," Description ","Description",
                "","N",self.doDesc,None,None,None),
            (("C",1,0,0),"IUI",3,"Seq","Column Sequence",
                "i","N",self.doColSeq,ffl,None,("between",0,999)),
            (("C",1,0,1),"INA",20,"Column","Column Name",
                "","N",None,None,None,("notblank",)),
            (("C",1,0,2),"INA",2,"Tp","Data Type",
                "","N",None,typ,None,("in", valtyp)),
            (("C",1,0,3),"IUD",6.1,"Size","Field Size",
                "","N",None,None,None,("notzero",)),
            (("C",1,0,4),"INA",30,"Description","Description",
                "","N",None,None,None,None),
            (("C",1,0,5),"INA",30,"Heading","Report Heading",
                "","N",None,None,None,("notblank",)),
            (("C",2,0,0),"IUI",1,"S","Sequence",
                "i","N",self.doIdxSeq,ffi,None,("efld",)),
            (("C",2,0,1),"IUA",1,"T","Type (U/N)",
                "","N",None,None,None,("in", ("U","N"))),
            (("C",2,0,2),"INA",(14,20),"Column-0","Column-0",
                "","N",self.doIdxCol,self.col,None,("notblank",)),
            (("C",2,0,3),"INA",(14,20),"Column-1","Column-1",
                "","N",self.doIdxCol,self.col,None,("efld",)),
            (("C",2,0,4),"INA",(14,20),"Column-2","Column-2",
                "","N",self.doIdxCol,self.col,None,("efld",)),
            (("C",2,0,5),"INA",(14,20),"Column-3","Column-3",
                "","N",self.doIdxCol,self.col,None,("efld",)),
            (("C",2,0,6),"INA",(14,20),"Column-4","Column-4",
                "","N",self.doIdxCol,self.col,None,("efld",)),
            (("C",2,0,7),"INA",(5,20),"Col-5","Column-5",
                "","N",self.doIdxCol,self.col,None,("efld",)),
            (("C",2,0,8),"INA",(5,20),"Col-6","Column-6",
                "","N",self.doIdxCol,self.col,None,("efld",)),
            (("C",2,0,9),"INA",(5,20),"Col-7","Column-7",
                "","N",self.doIdxCol,self.col,None,("efld",)),
            (("C",2,0,10),"INA",(5,20),"Col-8","Column-8",
                "","N",self.doIdxCol,self.col,None,("efld",)),
            (("C",2,0,11),"INA",(5,20),"Col-9","Column-9",
                "","N",self.doIdxCol,self.col,None,("efld",))]
        tag = (
            ("Columns",None,("C",1,1),("T",0,1)),
            ("Indexes",None,("C",1,1),("T",0,1)))
        row = (0,15,5)
        tnd = ((self.endTop, "y"),)
        txt = (self.exitTop,)
        cnd = ((None,"n"), (self.endPage1,"y"), (self.endPage2,"y"))
        cxt = (None, self.exitPage1, self.exitPage2)
        but = (
            ("Files",None,self.fileBrowser,0,("T",0,1),None),
            ("Print",None,self.doPrint,0,("T",0,2),("T",0,1)),
            ("Cancel",None,self.doRestart,0,("T",0,2),("T",0,1)))
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tags=tag,
            rows=row, tend=tnd, txit=txt, cend=cnd, cxit=cxt, butt=but)

    def doTable(self, frt, pag, r, c, p, i, w):
        self.tab = w
        tab = self.sql.getRec(tables="ftable", cols=["ft_tabl", "ft_desc"],
            where=[("ft_tabl", "=", w), ("ft_seq", "=", 1)], limit=1)
        if not tab:
            self.new = "y"
        else:
            self.new = "n"
            self.df.loadEntry(frt, pag, p+1, data=tab[1])
            yn = askQuestion(screen=self.mf.body, head="Table Exists",
                mess="Changing This Record Could Result In Loss of Data, "\
                "Are You Sure You Want to Continue?", default="no")
            if yn == "no":
                return yn

    def doDesc(self, frt, pag, r, c, p, i, w):
        self.desc = w

    def endTop(self):
        self.fld = makeArray(6,100,1,"S")
        self.idx = makeArray(12,20,1,"S")
        if self.new == "n":
            fld = self.sql.getRec(tables="ffield", where=[("ff_tabl", "=",
                self.tab)], order="ff_seq")
            for n, f in enumerate(fld):
                self.fld[n] = f[1:]
            idx = self.sql.getRec(tables="ftable", where=[("ft_tabl", "=",
                self.tab)], order="ft_seq")
            for n, f in enumerate(idx):
                self.idx[n] = f[2:]
                self.idx[n][0] = n
            self.doPopulate("fld")
        else:
            self.df.focusField("C", 1, 1)

    def exitTop(self):
        self.df.mstAlw.destroy()
        self.mf.closeLoop()

    def doColSeq(self, frt, pag, r, c, p, i, w):
        self.colseq = w
        for n, c in enumerate(self.fld[self.colseq][1:
           ]):
            self.df.loadEntry("C", 1, p+n+1, data=c)

    def endPage1(self):
        pos = (self.df.col / self.df.colq[1]) - 1
        self.fld[self.colseq] = self.df.c_work[1][pos][:]
        self.doPopulate("fld")

    def exitPage1(self):
        self.createCols()
        self.df.selPage("Indexes")
        self.doPopulate("idx")

    def createCols(self):
        self.valcol = []
        self.col["data"] = []
        for fld in self.fld:
            self.valcol.append(fld[1])
            self.col["data"].append([fld[1], fld[4]])

    def doIdxCmd(self, frt, pag, r, c, p, i, w):
        idxseq = int(w[0]) - 1
        self.doIdxSeq(frt, pag, r, c, p, i, idxseq)
        self.df.focusField(frt, pag, c+1)

    def doIdxSeq(self, frt, pag, r, c, p, i, w):
        self.idxseq = w
        for n, c in enumerate(self.idx[self.idxseq][1:
           ]):
            self.df.loadEntry("C", 2, p+n+1, data=c)

    def doIdxCol(self, frt, pag, r, c, p, i, w):
        if w and w not in self.valcol:
            return "Invalid Column"

    def endPage2(self):
        pos = (self.df.col / self.df.colq[2]) - 1
        self.idx[self.idxseq] = self.df.c_work[2][pos][:]
        self.doPopulate("idx")

    def exitPage2(self):
        if self.new == "y" and self.df.col == 1:
            self.df.selPage("Columns")
        else:
            fld = os.path.join(self.mf.prgdir, "csv", self.tab + "_fld.csv")
            if os.path.exists(fld):
                if os.path.exists("%s~" % fld):
                    os.remove("%s~" % fld)
                os.rename(fld, "%s~" % fld)
            csv = open(fld, "w")
            for dat in self.fld:
                if dat == [""] * 6:
                    break
                csv.write('%s,"%s","%s","%s","%s","%s"\n' % tuple(dat))
            csv.close()
            idx = os.path.join(self.mf.prgdir, "csv", self.tab + "_idx.csv")
            if os.path.exists(idx):
                if os.path.exists("%s~" % idx):
                    os.remove("%s~" % idx)
                os.rename(idx, "%s~" % idx)
            csv = open(idx, "w")
            for num, dat in enumerate(self.idx):
                if dat == [""] * 12:
                    break
                if num == 0:
                    dat.insert(0, self.df.t_work[0][0][1])
                else:
                    dat.insert(0, "")
                dat[1] = num + 1
                csv.write('"%s",%s,"%s","%s","%s","%s","%s","%s","%s",'\
                    '"%s","%s","%s","%s"\n' % tuple(dat))
            csv.close()
            yn = askQuestion(screen=self.mf.body, head="Update Database",
                mess="Would You like to Update the Database Now?")
            if yn == "yes":
                self.tarUpd()
            self.doRestart()

    def doPopulate(self, ftyp=None):
        if ftyp == "fld":
            self.df.clearFrame("C", 1)
            for num, dat in enumerate(self.fld):
                if dat == [""] * 6:
                    break
                if num > 14:
                    self.df.scrollScreen(1)
                    pos = 14
                else:
                    pos = num
                self.df.focusField("C", 1, (pos*6)+1)
                for n, c in enumerate(dat):
                    self.df.loadEntry("C", 1, (pos*6)+n, data=c)
            self.df.advanceLine(1)
            return
        self.df.clearFrame("C", 2)
        for num, dat in enumerate(self.idx):
            if dat == [""] * 12:
                break
            if num > 4:
                self.df.scrollScreen(2)
                pos = 4
            else:
                pos = num
            self.df.focusField("C", 2, (pos*12)+1)
            for n, c in enumerate(dat):
                self.df.loadEntry("C", 2, (pos*12)+n, data=c)
        self.df.advanceLine(2)

    def fileBrowser(self, widget=None):
        dialog = FileDialog(initd = os.path.join(self.mf.prgdir, "csv"),
            ftype = [("comma seperated", "*_idx.csv")])
        self.file = dialog.askopenfilename()
        if self.file:
            chk = self.sql.getRec(tables="ftable", cols=["count(*)"],
                where=[("ft_tabl", "=", self.file)], limit=1)
            if chk[0]:
                showError(self.mf.body, "Error", "Table Already Exists")
            else:
                self.importFile()

    def importFile(self):
        table = os.path.basename(self.file).split("_")[0]
        try:
            fle = open(self.file, "r")
            key = fle.readlines()
            fle.close()
        except:
            key = None
        try:
            fle = open(self.file.replace("_idx", "_fld"), "r")
            dat = fle.readlines()
            fle.close()
        except:
            dat = None
        if not key or not dat:
            showError(self.mf.body, "Error", "Missing key or fld File")
            self.df.focusField("T", 0, 1)
        else:
            self.new = "i"
            self.fld = makeArray(6,100,1,"S")
            self.idx = makeArray(12,20,1,"S")
            self.tab = os.path.basename(self.file.split("_")[0])
            for n, k in enumerate(key):
                k = k.replace(chr(10),"").replace(chr(13),"").replace('"',"")
                cols = k.split(",")
                if not n:
                    self.desc = cols[0]
                cols[1] = n
                self.idx[n] = cols[1:]
            fld = []
            for n, d in enumerate(dat):
                d = d.replace(chr(10),"").replace(chr(13),"").replace('"',"")
                cols = d.split(",")
                self.fld[n] = cols[:]
            self.df.loadEntry("T", 0, 0, data=self.tab)
            self.df.loadEntry("T", 0, 1, data=self.desc)
            self.doPopulate("fld")

    def doRestart(self, widget=None):
        self.df.last[0] = [0, 0]
        self.df.selPage("Columns")
        self.df.focusField("T", 0, 1)

    def doPrint(self, widget=None):
        state = self.df.disableButtons()
        cols = [["ff_seq","UI",3.0,"Seq"],
            ["ff_name","NA",20.0,"Field Name"],
            ["ff_type","NA",2.0,"Tp"],
            ["ff_size","NA",6.1,"Size"],
            ["ff_desc","NA",30.0,"Description"],
            ["ff_head","NA",20.0,"Heading"]]
        whr = [("ff_tabl", "=", self.tab)]
        rp = RepPrt(self.mf, name=self.tab + "_fld", tables=["ffield"],
            heads=["Table Fields for Table %s" % self.tab,], cols=cols,
            where=whr)
        cols = [["ft_desc","NA",30.0,"Table Description"],
            ["ft_seq","UI",2.0,"Sq"],
            ["ft_type","NA",1.0,"T"],
            ["ft_key0","NA",10.0,"1st-Key"],
            ["ft_key1","NA",10.0,"2nd-Key"],
            ["ft_key2","NA",10.0,"3rd-Key"],
            ["ft_key3","NA",10.0,"4th-Key"],
            ["ft_key4","NA",10.0,"5th-Key"],
            ["ft_key5","NA",10.0,"6th-Key"],
            ["ft_key6","NA",10.0,"7th-Key"],
            ["ft_key7","NA",10.0,"8th-Key"],
            ["ft_key8","NA",10.0,"9th-Key"],
            ["ft_key9","NA",10.0,"10th-Key"]]
        whr = [("ft_tabl", "=", self.tab)]
        rp = RepPrt(self.mf, name=self.tab+ "_idx", tables=["ftable"],
            heads=["Table Keys for Table %s" % self.tab,], cols=cols,
            where=whr)
        self.df.enableButtons(state=state)
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def tarUpd(self):
        self.df.mstAlw.hide()
        dir = ("-d", os.path.join(self.mf.prgdir, "csv"))
        usr = ("-u", self.mf.dbuser)
        pwd = ("-p", self.mf.dbpwd)
        sys = ("-s", self.sys)
        tab = ("-t", self.tab)
        if self.new in ("i", "y"):
            from TartanClasses import DBCreate
            opts = [("-c", "t"), dir, usr, pwd, tab, ("-x", False)]
            DBCreate(mf=self.mf, opts=opts)
            fle = open(os.path.join(dir[1], sys[1] + ".sys"), "a")
            fle.write("%s\n" % self.tab)
            fle.close()
        else:
            from tb1020 import tb1020
            upd = tb1020(opts=[("-m", self.mf), dir, usr, pwd, sys, tab])
        self.df.mstAlw.show()

if __name__ == "__main__":
    from TartanClasses import MainFrame
    mf = MainFrame()
    tb1010(mf, "tartan")
