#! /usr/bin/python2

"""
SYNOPSIS
    Tables Alteration comparing New csv file with existing data in database.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import os, sys, time
from TartanClasses import ASD, CCD, DBCreate, MainFrame, ProgressBar
from TartanClasses import SplashScreen, Sql
from tartanFunctions import getTrn, loadRcFile, showError

class tb1020:
    def __init__(self, opts=None):
        self.mf = None
        self.gui = False
        self.pth = None
        self.usr = ""
        self.pwd = ""
        self.rcf = None
        self.sys = None
        self.ver = None
        self.tab = None
        self.fmt = False
        self.cln = True
        for o, v in opts:
            if o == "-h":
                print """
This is used to change tables in a database.

Usage: python tb1020.py [options]

    -c  = Clean the DataBase i.e. remove unwanted tables etc
    -d  = Directory where .csv files reside, defaults to "~/prg/csv"
    -f  = Force reformat of all fields
    -g  = GUI y/n
    -h  = Display Usage
    -m  = MainFrame Module
    -p  = Database Admin Password
    -r  = tartanrc file to use, defaults to ~/.tartanrc
    -s  = system file to use, defaults to tartan
    -t  = String of comma separated Table Names referring to ".csv" files e.g.
            strmf1,strmf2,strtrn
    -u  = Database Admin User Name
    -v  = The New version number
"""
                sys.exit()
            elif o == "-m":
                self.mf = v
            elif o == "-g":
                self.gui = True
            elif o == "-d":
                self.pth = v
            elif o == "-u":
                self.usr = v
            elif o == "-p":
                self.pwd = v
            elif o == "-r":
                self.rcf = v
            elif o == "-s":
                self.sys = v
            elif o == "-v":
                self.ver = v
            elif o == "-t":
                self.tab = []
                try:
                    tb = v.split(",")
                    for t in tb:
                        self.tab.append(t.strip())
                except:
                    pass
            elif o == "-f":
                if v.lower() == "y":
                    self.fmt = True
                else:
                    self.fmt = False
            elif o == "-c":
                if v.lower() == "y":
                    self.cln = True
                else:
                    self.cln = False
        no = self.setVariables()
        if no:
            return
        if self.oldver[0]:
            if self.oldver < (2, 5, 5):
                showError(self.mf.body, "Version Error", "Your Current "\
                    "Version Must Be 2.5.5 or Later Before You Can Update "\
                    "to This Version!")
                return
            if self.oldver[0] < 3:
                self.doCvtAgeing()
            if self.oldver[0] < 4:
                if self.doMergeJournals():
                    return
        self.doProcessTables()
        if self.cln:
            self.doCleanDatabase()
        if self.oldver[0] and self.oldver[0] == 3:
            if self.oldver[1] in (0, 1):
                self.doFixRef()
            elif self.oldver[1] == 2 and self.oldver[2] < 4:
                self.doFixRef()
        if self.ver:
            sql = Sql(self.mf.dbm, "verupd", prog=__name__)
            sql.delRec("verupd")
            sql.insRec("verupd", data=[self.ver, self.sysdtw])
            self.mf.dbm.commitDbase()

    def setVariables(self):
        t = time.localtime()
        self.sysdtw = ((t[0] * 10000) + (t[1] * 100) + t[2])
        if not self.mf:
            if self.rcf:
                from tartanFunctions import loadRcFile
                rcdic = loadRcFile(rcfile=self.rcf)
                if rcdic == "error":
                    return rcdic
                self.mf = MainFrame(rcdic=rcdic, display=self.gui)
                self.mf.dbm.openDbase()
            else:
                showError(None, "RC File Error", "RC File Not Found")
                return "error"
        self.sql = Sql(self.mf.dbm, ["ffield", "ftable", "verupd"],
            prog=__name__, errs=False)
        if "ffield" in self.sql.error or "ftable" in self.sql.error:
            sys.exit()
        if "verupd" in self.sql.error:
            self.oldver = (0, 0, 0)
        else:
            v = self.sql.getRec(tables="verupd", cols=["max(ver_version)"],
                limit=1)
            if v[0]:
                ov = v[0].split(".")
                self.oldver = (int(ov[0]), int(ov[1]), int(ov[2]))
            else:
                self.oldver = (0, 0, 0)
        if not self.pth:
            self.pth = os.path.join(self.mf.prgdir, "csv")
        if not self.tab:
            self.doGetTables()
        self.tab.sort()
        self.opts = [("-u", self.usr), ("-c", "t"), ("-d", self.pth),
            ("-i", False), ("-w", False), ("-x", False)]
        if self.pwd:
            self.opts.append(("-p", self.pwd))
        self.tabord = {
            "asstrn": ["ast_capdt", "ast_seq"],
            "chglog": ["chg_dte", "chg_seq"],
            "crsage": ["cra_curdt", "cra_seq"],
            "crstrn": ["crt_capdt", "crt_seq"],
            "cshana": ["can_date", "can_seq"],
            "ctlnot": ["not_date", "not_seq"],
            "ctlvtf": ["vtt_capdt", "vtt_seq"],
            "drsage": ["dra_curdt", "dra_seq"],
            "drstrn": ["drt_capdt", "drt_seq"],
            "genrct": ["grt_seq", "grt_date"],
            "gentrn": ["glt_capdt", "glt_seq"],
            "memage": ["mta_curdt", "mta_seq"],
            "memsta": ["mls_date", "mls_seq"],
            "memtrn": ["mlt_capdt", "mlt_seq"],
            "memtrs": ["mst_trdt", "mst_seq"],
            "rcaowt": ["rot_capdt", "rot_seq"],
            "rcatnt": ["rtt_capdt", "rtt_seq"],
            "rtltrn": ["rtt_capdt", "rtt_seq"],
            "slsiv3": ["si3_seqnum"],
            "strpot": ["pot_capdt", "pot_seq"],
            "strtrn": ["stt_capdt", "stt_seq"],
            "wagcap": ["wcp_capdt", "wcp_seq"],
            "wagltf": ["wlt_capdt", "wlt_seq"],
            "wagtf1": ["wt1_capdt", "wt1_seq"]}

    def doCvtAgeing(self):
        if self.mf.display:
            sp = SplashScreen(self.mf.body,
            "Converting Files to Version 3\n\nPlease Wait...")
        else:
            print "Converting Files to Version 3, Please Wait..."
        age = []
        if "crstrn" in self.tab:
            age.append("crsage")
        if "drstrn" in self.tab:
            age.append("drsage")
        for table in age:
            opts = self.opts[:]
            opts.append(("-t", table))
            DBCreate(mf=self.mf, opts=opts)
        if not "crsrtf" in self.tab:
            sql = Sql(self.mf.dbm, ["drsctl", "drstrn", "drsage", "crsctl",
                "crstrn", "crsage"], prog=__name__)
            # Discount - Debtors
            typ = "Debtors Discount"
            all = sql.getRec(tables="drstrn", where=[("drt_disamt", "<>", 0)])
            if not self.mf.display:
                print typ
            else:
                pb = ProgressBar(self.mf.body, max=len(all), typ=typ)
            for num, rec in enumerate(all):
                if self.mf.display:
                    pb.displayProgress(num)
                rec[4] = 6
                rec[9] = rec[10]
                rec[10] = 0
                rec[11] = 0
                rec[12] = 0
                rec[13] = 0
                rec[20] = 0
                sql.insRec("drstrn", data=rec, format=False)
            if self.mf.display:
                pb.closeProgress()
            # Balancing Amount - Debtors
            typ = "Debtors Balancing"
            sql.delRec("crstrn", where=[("crt_batch", "=", "Imp/Bat"),
                ("crt_ref1", "=", "Imp/Amt")])
            all = sql.getRec(tables="drstrn", cols=["drt_cono", "drt_chain",
                "drt_acno", "round(sum(drt_tramt), 2)",
                "round(sum(drt_trbal), 2)"],
                group="drt_cono, drt_chain, drt_acno")
            if not self.mf.display:
                print typ
            else:
                pb = ProgressBar(self.mf.body, max=len(all), typ=typ)
            for num, rec in enumerate(all):
                cono, chain, acno, tramt, trbal = rec
                if self.mf.display:
                    pb.displayProgress(num)
                diff = float(ASD(tramt) - ASD(trbal))
                if diff:
                    dte = sql.getRec(tables="drstrn", cols=["min(drt_trdt)"],
                        where=[("drt_cono", "=", cono), ("drt_chain", "=",
                        chain), ("drt_acno", "=", acno)], limit=1)[0]
                    diff = float(ASD(0) - ASD(diff))
                    sql.insRec("drstrn", data=[cono, chain, acno, dte, 3,
                        "Bal/Ref", "Bal/Bat", dte, "", diff, 0, 0, 0, 0,
                        dte/100, "Balancing Amount", "N", "", "Y",
                        self.sysdtw, 0], format=False)
            if self.mf.display:
                pb.closeProgress()
            # Ageing - Debtors
            typ = "Debtors Ageing"
            all = sql.getRec(tables="drstrn",
                where=[("drt_trbal <> drt_tramt",)])
            if not self.mf.display:
                print typ
            else:
                pb = ProgressBar(self.mf.body, max=len(all), typ=typ)
            for num, rec in enumerate(all):
                if self.mf.display:
                    pb.displayProgress(num)
                data = rec[:3]
                data.extend(rec[4:6])
                data.extend([rec[14], 2, "Cvt/Ref"])
                data.append(float(ASD(rec[9]) - ASD(rec[12])))
                data.append(0)
                sql.insRec("drsage", data=data, format=False)
            if self.mf.display:
                pb.closeProgress()
            # Discount - Creditors
            typ = "Creditors Discount"
            all = sql.getRec(tables="crstrn", where=[("crt_disamt", "<>", 0)])
            if not self.mf.display:
                print typ
            else:
                pb = ProgressBar(self.mf.body, max=len(all), typ=typ)
            for num, rec in enumerate(all):
                if self.mf.display:
                    pb.displayProgress(num)
                rec[3] = 6
                rec[8] = rec[9]
                rec[9] = 0
                rec[10] = 0
                rec[11] = 0
                rec[12] = 0
                rec[13] = 0
                rec[15] = 0
                rec[17] = 0
                rec[23] = 0
                sql.insRec("crstrn", data=rec, format=False)
            if self.mf.display:
                pb.closeProgress()
        else:
            sql = Sql(self.mf.dbm, ["crsctl", "crstrn", "crsrtf", "crsage"],
                prog=__name__)
            # Discount - Creditors
            typ = "Creditors Discount"
            all = sql.getRec(tables="crstrn", where=[("crt_type", "=", 5)])
            if not self.mf.display:
                print typ
            else:
                pb = ProgressBar(self.mf.body, max=len(all), typ=typ)
            for num, rec in enumerate(all):
                if self.mf.display:
                    pb.displayProgress(num)
                rec[9] = 0
                rec[11] = 0
                rec[12] = 0
                rec[13] = 0
                rec[15] = 0
                rec[17] = 0
                rec[23] = 0
                whr = [("rtf_cono", "=", rec[0]), ("rtf_acno", "=", rec[1]),
                    ("rtf_type", "=", rec[3]), ("rtf_ref1", "=", rec[4])]
                rtf = sql.getRec(tables="crsrtf", where=whr, limit=1)
                if rtf:
                    if rtf[6]:
                        rec[3] = 6
                        rec[8] = rtf[6]
                        rec[10] = rtf[7]
                        sql.insRec("crstrn", data=rec, format=False)
                    if rtf[8]:
                        rec[3] = 7
                        rec[8] = rtf[8]
                        rec[10] = rtf[9]
                        sql.insRec("crstrn", data=rec, format=False)
                    if rtf[10]:
                        rec[3] = 8
                        rec[8] = rtf[10]
                        rec[10] = rtf[11]
                        sql.insRec("crstrn", data=rec, format=False)
                    sql.delRec("crsrtf", where=whr)
            if self.mf.display:
                pb.closeProgress()
        # Balancing Amount - Creditors
        typ = "Creditors Balancing"
        sql.delRec("crstrn", where=[("crt_batch", "=", "Imp/Bat"),
            ("crt_ref1", "=", "Imp/Amt")])
        all = sql.getRec(tables="crstrn", cols=["crt_cono", "crt_acno",
            "round(sum(crt_tramt), 2)", "round(sum(crt_trbal), 2)"],
            group="crt_cono, crt_acno")
        if not self.mf.display:
            print typ
        else:
            pb = ProgressBar(self.mf.body, max=len(all), typ=typ)
        for num, rec in enumerate(all):
            cono, acno, tramt, trbal  = rec
            if self.mf.display:
                pb.displayProgress(num)
            diff = float(ASD(tramt) - ASD(trbal))
            if diff:
                dte = sql.getRec(tables="crstrn", cols=["min(crt_trdt)"],
                    where=[("crt_cono", "=", cono), ("crt_acno", "=", acno)],
                    limit=1)[0]
                diff = float(ASD(0) - ASD(diff))
                sql.insRec("crstrn", data=[cono, acno, dte, 3, "Bal/Ref",
                    "Bal/Bat", dte, "", diff, 0, 0, 0, 0, 0, dte/100, 0,
                    "", 0, "Balancing Amount", "N", "", "Y", self.sysdtw, 0],
                    format=False)
        if self.mf.display:
            pb.closeProgress()
        # Ageing - Creditors
        typ = "Creditors Ageing"
        all = sql.getRec(tables="crstrn", where=[("crt_trbal <> crt_tramt",)])
        if not self.mf.display:
            print typ
        else:
            pb = ProgressBar(self.mf.body, max=len(all), typ=typ)
        for num, rec in enumerate(all):
            if self.mf.display:
                pb.displayProgress(num)
            data = rec[:2]
            data.extend(rec[3:5])
            cd = sql.getRec(tables="crstrn", cols=["crt_curdt"],
                where=[("crt_cono", "=", rec[0]), ("crt_acno", "=", rec[1]),
                ("crt_type", "=", 5), ("crt_ref1", "=", rec[7])], limit=1)
            if cd:
                data.append(cd[0])
            else:
                data.append(rec[14])
            data.extend([5, "Cvt/Ref"])
            data.append(float(ASD(rec[8]) - ASD(rec[11])))
            data.append(0)
            sql.insRec("crsage", data=data, format=False)
        if self.mf.display:
            pb.closeProgress()
            sp.closeSplash()
            self.mf.updateStatus("")
        self.mf.dbm.commitDbase()

    def doMergeJournals(self):
        typ = ("G", "Merging Journals")
        if not self.mf.display:
            print typ[1]
        else:
            pb = ProgressBar(self.mf.body, typ=typ, cnt=1000)
        crs = True
        rtf = True
        drs = True
        sql = Sql(self.mf.dbm, tables=["crstrn", "crsrtf", "crsage", "ctlbat",
            "ctlvtf", "drstrn", "drsage"], errs=False)
        if sql.error:
            if "crstrn" in sql.error:
                crs = False
            elif "crsrtf" in sql.error:
                rtf = False
            if "drstrn" in sql.error:
                drs = False
        if crs:
            # Creditor's
            chk = sql.getRec(tables="crstrn", cols=["max(crt_type)"], limit=1)
            if chk and chk[0] > 5:
                if self.mf.display:
                    pb.displayProgress()
                for rtyp in ((6, 3), (7, 6), (8, 7), (9, 8)):
                    all = sql.getRec(tables="crstrn", where=[("crt_type", "=",
                        rtyp[0])])
                    for a in all:
                        if self.mf.display:
                            pb.displayProgress()
                        acc = sql.getRec(tables="crstrn", where=[("crt_cono",
                            "=", a[0]), ("crt_acno", "=", a[1]), ("crt_type",
                            "=", rtyp[1]), ("crt_ref1", "=", a[3])], limit=1)
                        if acc:
                            amt = float(ASD(acc[7]) + ASD(a[7]))
                            tax = float(ASD(acc[8]) + ASD(a[8]))
                            sql.delRec("crstrn", where=[("crt_seq", "=",
                                a[18])])
                            sql.updRec("crstrn", cols=["crt_tramt",
                                "crt_taxamt"], data=[amt, tax],
                                where=[("crt_seq", "=", acc[18])])
                        else:
                            sql.updRec("crstrn", cols=["crt_type"],
                                data=[rtyp[1]], where=[("crt_seq", "=",
                                a[18])])
                    if rtf:
                        all = sql.getRec(tables="crsrtf", where=[("rtf_type",
                            "=", rtyp[0])])
                        for a in all:
                            if self.mf.display:
                                pb.displayProgress()
                            acc = sql.getRec(tables="crsrtf",
                                where=[("rtf_cono", "=", a[0]), ("rtf_acno",
                                "=", a[1]), ("rtf_type", "=", rtyp[1]),
                                ("rtf_ref1", "=", a[3])], limit=1)
                            if acc:
                                sql.delRec("crsrtf", data=a)
                                amt1 = float(ASD(acc[6]) + ASD(a[6]))
                                tax1 = float(ASD(acc[7]) + ASD(a[7]))
                                amt2 = float(ASD(acc[8]) + ASD(a[8]))
                                tax2 = float(ASD(acc[9]) + ASD(a[9]))
                                amt3 = float(ASD(acc[10]) + ASD(a[10]))
                                tax3 = float(ASD(acc[11]) + ASD(a[11]))
                                sql.updRec("crsrtf", cols=["rtf_disamt",
                                    "rtf_distax", "rtf_incamt", "rtf_inctax",
                                    "rtf_advamt", "rtf_advtax"], data=[amt1,
                                    tax1, amt2, tax2, amt3, tax3],
                                    where=[("rtf_cono", "=", acc[0]),
                                    ("rtf_acno", "=", acc[1]), ("rtf_type",
                                    "=", acc[2]), ("rtf_ref1", "=", acc[3])])
                            else:
                                sql.updRec("crsrtf", cols=["rtf_type"],
                                    data=[rtyp[1]], where=[("rtf_cono",
                                    "=", a[0]), ("rtf_acno", "=", a[1]),
                                    ("rtf_type", "=", a[2]), ("rtf_ref1",
                                    "=", a[3])])
                    sql.updRec("crsage", cols=["cra_type"], data=[rtyp[1]],
                        where=[("cra_type", "=", rtyp[0])])
                    sql.updRec("crsage", cols=["cra_atyp"], data=[rtyp[1]],
                        where=[("cra_atyp", "=", rtyp[0])])
                    sql.updRec("ctlbat", cols=["btm_rtyp"], data=[rtyp[1]],
                        where=[("btm_styp", "=", "CRS"), ("btm_rtyp", "=",
                        rtyp[0])])
                    sql.updRec("ctlvtf", cols=["vtt_ttyp"], data=[rtyp[1]],
                        where=[("vtt_styp", "=", "C"), ("vtt_ttyp", "=",
                        rtyp[0])])
        if drs:
            # Debtor's
            chk = sql.getRec(tables="drstrn", cols=["max(drt_type)"], limit=1)
            if chk and chk[0] > 5:
                for rtyp in ((6, 3), (7, 6)):
                    all = sql.getRec(tables="drstrn", where=[("drt_type", "=",
                        rtyp[0])])
                    for a in all:
                        if self.mf.display:
                            pb.displayProgress()
                        acc = sql.getRec(tables="drstrn", where=[("drt_cono",
                            "=", a[0]), ("drt_chain", "=", a[1]), ("drt_acno",
                            "=", a[2]), ("drt_type", "=", rtyp[1]), ("drt_ref1",
                            "=", a[4])], limit=1)
                        if acc:
                            amt = float(ASD(acc[8]) + ASD(a[8]))
                            tax = float(ASD(acc[9]) + ASD(a[9]))
                            sql.delRec("drstrn", where=[("drt_seq",
                                "=", a[15])])
                            sql.updRec("drstrn", cols=["drt_tramt",
                                "drt_taxamt"], data=[amt, tax],
                                where=[("drt_seq", "=", acc[15])])
                        else:
                            sql.updRec("drstrn", cols=["drt_type"],
                                data=[rtyp[1]], where=[("drt_seq", "=", a[15])])
                    sql.updRec("drsage", cols=["dra_type"], data=[rtyp[1]],
                        where=[("dra_type", "=", rtyp[0])])
                    sql.updRec("drsage", cols=["dra_atyp"], data=[rtyp[1]],
                        where=[("dra_atyp", "=", rtyp[0])])
                    sql.updRec("ctlbat", cols=["btm_rtyp"], data=[rtyp[1]],
                        where=[("btm_styp", "=", "DRS"), ("btm_rtyp", "=",
                        rtyp[0])])
                    sql.updRec("ctlvtf", cols=["vtt_ttyp"], data=[rtyp[1]],
                        where=[("vtt_styp", "=", "D"), ("vtt_ttyp", "=",
                        rtyp[0])])
            if self.mf.display:
                pb.closeProgress()
        self.mf.dbm.commitDbase()

    def doProcessTables(self):
        if self.mf.display:
            typ = "Updating Table Details"
            pb = ProgressBar(self.mf.body, max=len(self.tab), typ=typ)
        for num, self.table in enumerate(self.tab):
            if self.mf.display:
                self.mf.updateStatus("Checking Table %s" % self.table)
                pb.displayProgress(num)
            else:
                print "Checking Table %s" % self.table
            if self.table in ("frelat", "wagrcv"):
                sql = Sql(self.mf.dbm, self.table, prog=__name__)
                sql.sqlRec("Drop table %s" % self.table, excp=True)
                self.doRemoveTable(self.table)
                chk = "new"
            elif self.table == "memctp":
                sql = Sql(self.mf.dbm, self.table, prog=__name__)
                sql.sqlRec("Update memctp set mcp_prorata = '1' where "\
                    "mcp_prorata = 'Y'")
            else:
                chk = self.doReadTables("idx")
                if not chk:
                    chk = self.doReadTables("fld")
            if chk:
                if chk == "new":
                    continue
                if self.mf.display:
                    self.mf.updateStatus("Creating Table %s" % self.table)
                else:
                    print "Creating Table %s" % self.table
                all = True
                self.doNewTable()
            elif self.fmt:
                if self.mf.display:
                    self.mf.updateStatus("Updating Table %s (fmt)" % self.table)
                else:
                    print "Updating Table %s (fmt)" % self.table
                all = True
                self.doNewTable()
            elif self.oldfld != self.newfld:
                all = False
                if len(self.oldfld) != len(self.newfld):
                    all = True
                else:
                    for seq, fld in enumerate(self.oldfld):
                        for s, f in enumerate(fld):
                            if f != self.newfld[seq][s]:
                                if s in (1,2,3,4):
                                    all = True
                                    break
                        if all:
                            break
                if self.mf.display:
                    self.mf.updateStatus("Updating Table %s" % self.table)
                else:
                    print "Updating Table %s" % self.table
                if all:
                    self.doNewTable()
                else:
                    self.doUpdateTable()
            else:
                all = False
            if not all and self.oldidx != self.newidx:
                if self.mf.display:
                    self.mf.updateStatus("Dropping Indexes for %s" % self.table)
                old = self.sql.getRec(tables="ftable", where=[("ft_tabl", "=",
                    self.table)], order="ft_seq")
                for rec in old:
                    nam = "%s_key%s" % (self.table, str(rec[2]))
                    if self.doCheckIndex(self.table, nam):
                        self.sql.sqlRec("Drop index %s" % nam)
                        if self.mf.dbname == "FBird":
                            self.mf.dbm.commitDbase()
                if self.mf.display:
                    self.mf.updateStatus("Vacuuming %s" % self.table)
                self.mf.dbm.vacuumDbase(self.table)
                if self.mf.display:
                    self.mf.updateStatus("Updating ftable and ffield")
                DBCreate(mf=self.mf, opts=[("-u", self.usr), ("-c", "p"),
                    ("-d", self.pth), ("-i", False), ("-t", self.table),
                    ("-w", False), ("-x", False)])
                self.doCreateIndexes(self.table, False, False)
        if self.mf.display:
            self.mf.updateStatus("")
            pb.closeProgress()

    def doGetTables(self):
        self.tab = []
        tab = self.sql.getRec(tables="ftable", where=[("ft_seq", "=", 1)])
        for t in tab:
            if t[0] not in ("ftable", "ffield"):
                self.tab.append(t[0])
        self.tab.sort()

    def doReadTables(self, tipe=None):
        if tipe == "idx":
            old = self.sql.getRec(tables="ftable", where=[("ft_tabl", "=",
                self.table)], order="ft_seq")
        else:
            old = self.sql.getRec(tables="ffield", where=[("ff_tabl", "=",
                self.table)], order="ff_seq")
        if not old:
            if tipe == "idx":
                tab = "ftable"
            else:
                tab = "ffield"
            if self.mf.display:
                self.mf.updateStatus("Table %s Not Found in %s" % \
                    (self.table, tab))
            else:
                print "Table %s Not Found in %s" % (self.table, tab)
            return tipe
        exec "self.old%s = old" % tipe
        csv = os.path.join(self.pth, "%s_%s.csv" % (self.table, tipe))
        if os.path.isfile(csv) == 0:
            if self.mf.display:
                self.mf.updateStatus("Csv for %s Table Not Found" % csv)
            else:
                print "Csv for %s Table Not Found" % csv
            return "new"
        new = []
        of = open(csv, "r")
        ot = of.readlines()
        for t in ot:
            t = t.replace(chr(10), "")
            t = t.replace(chr(13), "")
            t = t.replace(chr(34), "")
            t = t.split(",")
            t.insert(0, self.table)
            if tipe == "idx":
                t[2] = int(t[2])
                for x in range(len(t), 14):
                    t.append("")
            else:
                t[1] = int(t[1])
                t[4] = float(t[4])
            new.append(t)
        exec "self.new%s = new" % tipe

    def doUpdateTable(self):
        for seq, fld in enumerate(self.newidx):
            self.sql.updRec("ftable", where=[("ft_tabl", "=", fld[0]),
                ("ft_seq", "=", fld[2])], cols=["ft_desc"], data=[fld[1]])
        for seq, fld in enumerate(self.newfld):
            self.sql.updRec("ffield", where=[("ff_tabl", "=", fld[0]),
                ("ff_seq", "=", fld[1])], cols=["ff_desc", "ff_head"],
                data=[fld[5], fld[6]])
        self.mf.dbm.commitDbase()

    def doNewTable(self):
        if self.table == "ctlvtf":
            # Delete old VAT Transaction records
            sql = Sql(self.mf.dbm, ["ctlynd", self.table], prog=__name__)
            ynd = sql.getRec(tables="ctlynd", cols=["cye_cono", "cye_start"],
                where=[("cye_period", "=", 0)], order="cye_cono")
            for rec in ynd:
                if rec[0] and rec[1]:
                    sql.delRec("ctlvtf", where=[("vtt_cono", "=", rec[0]),
                        ("vtt_curdt", "<", rec[1] / 100)])
        else:
            sql = Sql(self.mf.dbm, self.table, prog=__name__)
        # Read Existing Table
        if self.table == "ctlnot" and self.oldver[0] == 2 and self.oldver[1] < 5:
            data = sql.getRec(tables=self.table,
                order="not_cono, not_sys, not_key, not_date, not_seq")
        else:
            order = ""
            if self.table in self.tabord:
                exec "cols = sql.%s_col" % self.table
                ords = self.tabord[self.table]
                for fld in ords:
                    if fld in cols:
                        if not order:
                            order = fld
                        else:
                            order = "%s, %s" % (order, fld)
            data = sql.getRec(tables=self.table, order=order)
            if self.table == "ctlmst" and "ctm_ehost" in sql.ctlmst_col:
                if data:
                    self.smtp = data[0][sql.ctlmst_col.index("ctm_ehost")]
                else:
                    self.smtp = ""
        # Create New Table
        opts = self.opts[:]
        opts.append(("-t", self.table))
        DBCreate(mf=self.mf, opts=opts)
        if self.table == "ctlnot" and self.oldver[0] == 2 and self.oldver[1] < 5:
            sql = Sql(self.mf.dbm, ["ctlnot", "telnot"], prog=__name__)
            if data:
                self.doNoteConvert(sql, data)
            data = sql.getRec(tables="telnot",
                order="tdn_name, tdn_date, tdn_seq")
            if data:
                self.doTelnoteConvert(sql, data)
        elif self.table in ("assgrp", "assmst") and self.oldver[0] and self.oldver < (3, 4, 18):
            if not "assdep" in self.tab:
                opts = self.opts[:]
                opts.append(("-t", "assdep"))
                DBCreate(mf=self.mf, opts=opts)
            if self.table == "assgrp":
                sql = Sql(self.mf.dbm, ["assdep", "assgrp"], prog=__name__)
                for dat in data:
                    if dat[3] == "D":
                        desc = "Balance @ %s%s and" % (dat[4], "%")
                    else:
                        desc = "Fixed @ %s%s and" % (dat[4], "%")
                    if dat[5] == "D":
                        desc = "%s Balance @ %s%s" % (desc, dat[6], "%")
                    else:
                        desc = "%s Fixed @ %s%s" % (desc, dat[6], "%")
                    chk = sql.getRec(tables="assdep", where=[("asd_cono",
                        "=", dat[0]), ("asd_desc", "=", desc)], limit=1)
                    if not chk:
                        nxt = sql.getRec(tables="assdep",
                            cols=["max(asd_code)"], where=[("asd_cono",
                            "=", dat[0])], limit=1)
                        if not nxt or not nxt[0]:
                            cod = 1
                        else:
                            cod = int(nxt[0]) + 1
                        ndat = [dat[0], cod, desc, dat[3], dat[5], dat[4],
                            dat[6], 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
                        sql.insRec("assdep", data=ndat)
                    else:
                        cod = chk[1]
                    ndat = [dat[0], dat[1], dat[2], cod, dat[7], dat[8], dat[9]]
                    sql.insRec("assgrp", data=ndat)
            elif self.table == "assmst":
                sql = Sql(self.mf.dbm, ["assdep", "assmst"], prog=__name__)
                for dat in data:
                    if dat[3] == "D":
                        desc = "Balance @ %s%s and" % (dat[5], "%")
                    else:
                        desc = "Fixed @ %s%s and" % (dat[5], "%")
                    if dat[5] == "D":
                        desc = "%s Balance @ %s%s" % (desc, dat[7], "%")
                    else:
                        desc = "%s Fixed @ %s%s" % (desc, dat[7], "%")
                    chk = sql.getRec(tables="assdep", where=[("asd_cono",
                        "=", dat[0]), ("asd_desc", "=", desc)], limit=1)
                    if not chk:
                        nxt = sql.getRec(tables="assdep",
                            cols=["max(asd_code)"], where=[("asd_cono",
                            "=", dat[0])], limit=1)
                        if not nxt or not nxt[0]:
                            cod = 1
                        else:
                            cod = int(nxt[0]) + 1
                        ndat = [dat[0], cod, desc, dat[4], dat[6], dat[5],
                            dat[7], 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
                        sql.insRec("assdep", data=ndat)
                    else:
                        cod = chk[1]
                    ndat = [dat[0], dat[1], dat[2], dat[3], cod]
                    sql.insRec("assmst", data=ndat)
        elif self.table not in ("frelat", "wagrcv") and data not in (None, Exception, "Exception"):
            sql = Sql(self.mf.dbm, self.table, prog=__name__)
            # Create Dictionaries
            olddic = {}
            for f in self.oldfld:
                olddic[f[2]] = f[1:]
            newdic = {}
            for f in self.newfld:
                newdic[f[2]] = f[1:]
            # Populate New Table
            if self.mf.display:
                typ = "Updating Database Contents"
                pb = ProgressBar(self.mf.body, max=len(data), typ=typ)
            if self.table in ("genrpc", "genstr"):
                seq = 0
            for num, old in enumerate(data):
                if self.mf.display:
                    pb.displayProgress(num)
                dat = []
                for ind, csv in enumerate(self.newfld):
                    typ = newdic[csv[2]][2]
                    siz = newdic[csv[2]][3]
                    if csv[2] in olddic:
                        new = old[olddic[csv[2]][0]]
                        if self.table == "ctlnot" and csv[2] == "not_sys":
                            if new == "AR":
                                new = "ASS"
                            if new == "DR":
                                new = "DRS"
                            if new == "CR":
                                new = "CRS"
                            if new == "GL":
                                new = "GEN"
                            if new == "SL":
                                new = "SLN"
                            if new == "ST":
                                new = "STR"
                            if new == "WG":
                                new = "WAG"
                        # Change G/L Import Control Record Type
                        elif self.table == "genrcc" and csv[2] == "grc_rtn":
                            if new == "J": new = "P"
                        # Change G/L Reconciliation Flag to YYYYMM
                        elif self.table == "gentrn" and csv[2] == "glt_recon":
                            if type(new) == int:
                                pass
                            elif new == "Y":
                                new = 200101
                            else:
                                new = 0
                    elif self.table == "assctl" and csv[2] == "cta_rordp":
                        new = "Y"
                    elif self.table == "bwlflm" and csv[2] == "bfm_fmat":
                        new = 1
                    elif self.table == "bwlflo" and csv[2] == "bfo_fmat":
                        new = 1
                    elif self.table == "bwlfls":
                        if csv[2] == "bfs_fmat":
                            new = 1
                        elif csv[2] == "bfs_active":
                            new = "Y"
                    elif self.table == "bwlflt" and csv[2] == "bft_fmat":
                        new = 1
                    elif self.table == "ctlctl" and "ctl_qifdte" in olddic and csv[2] == "ctl_dtefmt":
                        new = old[olddic["ctl_qifdte"][0]]
                    elif self.table == "ctlmst" and csv[2] == "ctm_logo":
                        new = os.path.join(self.mf.prgdir, "img",
                            "logo_%03i.jpg" % old[olddic["ctm_cono"][0]])
                        if not os.path.exists(new):
                            new = ""
                    elif self.table == "ctlmst" and csv[2] == "ctm_flip":
                        new = "N"
                    elif self.table == "ctlsys" and csv[2] == "sys_smtp":
                        new = self.smtp
                    elif self.table == "drsctl" and csv[2] == "ctd_glinp":
                        new = "E"
                    elif self.table == "drsctl" and csv[2] == "ctd_stype":
                        new = "N"
                    elif self.table == "drsctl" and csv[2] == "ctd_sinv":
                        new = "N"
                    elif self.table == "drsctl" and csv[2] == "ctd_sage":
                        new = "Y"
                    elif self.table == "drsmst" and csv[2] in ("drm_mgr", "drm_acc", "drm_sls"):
                        new = old[olddic["drm_contact"][0]]
                    elif self.table == "drsmst" and csv[2] in ("drm_mgr_email", "drm_acc_email", "drm_sls_email"):
                        new = old[olddic["drm_email"][0]]
                    elif self.table == "drsrcm" and csv[2] == "dcm_day":
                        new = 1
                    elif self.table == "crsctl" and csv[2] == "ctc_glinp":
                        new = "E"
                    elif self.table == "crsmst" and csv[2] in ("crm_mgr", "crm_acc", "crm_ord"):
                        new = old[olddic["crm_contact"][0]]
                    elif self.table == "crsmst" and csv[2] in ("crm_mgr_email", "crm_acc_email", "crm_ord_email"):
                        new = old[olddic["crm_email"][0]]
                    elif self.table == "genrpc" and csv[2] == "glc_cusno":
                        seq += 1
                        new = seq
                    elif self.table == "genstr" and csv[2] == "gls_seq":
                        new = seq
                        seq += 1
                    elif self.table == "memctc":
                        if csv[2] == "mcc_age_l":
                            new = old[olddic["mcc_limit"][0]]
                        elif csv[2] == "mcc_and_s":
                            if old[olddic["mcc_limit"][0]] == 60:
                                new = 15
                            else:
                                new = 0
                        elif csv[2] == "mcc_or_s":
                            new = 0
                        elif csv[2] == "mcc_rgrp":
                            if old[olddic["mcc_type"][0]] == "C":
                                if old[olddic["mcc_code"][0]] in (1, 2):
                                    new = "BS"
                                elif old[olddic["mcc_code"][0]] in (3, 4):
                                    new = "CS"
                                elif old[olddic["mcc_code"][0]] == 5:
                                    new = "GS"
                                elif old[olddic["mcc_code"][0]] in (6, 7):
                                    new = "HS"
                                elif old[olddic["mcc_code"][0]] in (8, 19):
                                    new = "OC"
                                elif old[olddic["mcc_code"][0]] == 9:
                                    new = "RS"
                                elif old[olddic["mcc_code"][0]] in (10, 11, 12):
                                    new = "SS"
                                elif old[olddic["mcc_code"][0]] in (13, 14, 15):
                                    new = "TS"
                                else:
                                    new = "NA"
                            else:
                                new = "NA"
                        elif typ[1].upper() in ("A", "X"):
                            new = ""
                        else:
                            new = 0
                    elif self.table == "memctk" and csv[2] == "mck_type":
                        if int(old[olddic["mck_code"][0]]) == 1:
                            new = "H"
                        elif int(old[olddic["mck_code"][0]]) == 2:
                            new = "W"
                        elif int(old[olddic["mck_code"][0]]) == 3:
                            new = "M"
                        elif int(old[olddic["mck_code"][0]]) == 4:
                            new = "F"
                        elif int(old[olddic["mck_code"][0]]) in (5, 6):
                            new = "E"
                        else:
                            new = "T"
                    elif self.table == "rcactl" and "ctr_cono" in olddic:
                        new =  old[ind]
                    elif self.table == "rcatnm" and "rtm_cono" in olddic:
                        new =  old[ind]
                    elif self.table == "rcatnt" and "rtt_cono" in olddic:
                        new =  old[ind]
                    elif self.table == "strctl" and csv[2] == "cts_automu":
                        new = "N"
                    elif self.table == "strmf1" and csv[2] == "st1_type":
                        new = "N"
                    elif self.table == "strtrn" and csv[2] == "stt_styp":
                        new = old[olddic["stt_systype"][0]]
                    elif self.table == "strtrn" and csv[2] == "stt_acno":
                        new = old[olddic["stt_debtor"][0]]
                    elif self.table == "telmst" and csv[2] == "tdm_name":
                        new = old[olddic["ph_name"][0]]
                    elif self.table == "telmst" and csv[2] == "tdm_adr1":
                        new = old[olddic["ph_adr1"][0]]
                    elif self.table == "telmst" and csv[2] == "tdm_adr2":
                        new = old[olddic["ph_adr2"][0]]
                    elif self.table == "telmst" and csv[2] == "tdm_adr3":
                        new = old[olddic["ph_adr3"][0]]
                    elif self.table == "telmst" and csv[2] == "tdm_pcode":
                        new = old[olddic["ph_pcode"][0]]
                    elif self.table == "telmst" and csv[2] == "tdm_telcd":
                        new = old[olddic["ph_telcd"][0]]
                    elif self.table == "telmst" and csv[2] == "tdm_telno":
                        new = old[olddic["ph_telno"][0]]
                    elif self.table == "telmst" and csv[2] == "tdm_faxno":
                        new = old[olddic["ph_faxno"][0]]
                    elif self.table == "wagtf2" and csv[2] == "wt2_taxcode":
                        sq = Sql(self.mf.dbm, "wagedc", prog=__name__)
                        cono = old[olddic["wt2_cono"][0]]
                        tipe = old[olddic["wt2_type"][0]]
                        code = old[olddic["wt2_code"][0]]
                        tc = sq.getRec(tables="wagedc", cols=["ced_taxcode"],
                            where=[("ced_cono", "=", cono), ("ced_type", "=",
                            tipe), ("ced_code", "=", code)], limit=1)
                        if not tc:
                            new = "N"
                        else:
                            new = tc[0]
                    elif self.table == "wagtf2" and csv[2] == "wt2_taxportion":
                        sq = Sql(self.mf.dbm, "wagedc", prog=__name__)
                        cono = old[olddic["wt2_cono"][0]]
                        tipe = old[olddic["wt2_type"][0]]
                        code = old[olddic["wt2_code"][0]]
                        tc = sq.getRec(tables="wagedc", cols=["ced_taxportion"],
                            where=[("ced_cono", "=", cono), ("ced_type", "=",
                            tipe), ("ced_code", "=", code)], limit=1)
                        if not tc:
                            new = 0
                        else:
                            new = tc[0]
                    elif typ[1].upper() in ("A", "X"):
                        new = ""
                    else:
                        new = 0
                    new = CCD(new, typ, siz)
                    dat.append(new.work)
                ### Work around unique keys
                if self.table == "drstrn":
                    unique = "drt_ref1"
                elif self.table == "crstrn":
                    unique = "crt_ref1"
                else:
                    unique = None
                ###
                sql.insRec(self.table, data=dat, unique=unique, format=False)
                if self.table == "crsrmf" and "rmf_bibt" in olddic:
                    sq = Sql(self.mf.dbm, "crsmst", prog=__name__)
                    cono = old[olddic["rmf_cono"][0]]
                    acno = old[olddic["rmf_acno"][0]]
                    bibt = old[olddic["rmf_bibt"][0]]
                    bacc = old[olddic["rmf_bacc"][0]]
                    sq.updRec("crsmst", cols=["crm_bibt", "crm_bacc"],
                        data=[bibt, bacc], where=[("crm_cono", "=", cono),
                        ("crm_acno", "=", acno)])
                if self.table == "telmst" and "ph_name" in olddic and "tdm_name" in newdic and old[olddic["ph_contact"][0]]:
                    sq = Sql(self.mf.dbm, "telcon", prog=__name__)
                    chk = self.sql.getRec(tables="ftable",
                        where=[("ft_tabl", "=", "telcon")])
                    if not chk:
                        opts = self.opts[:]
                        opts.append(("-t", "telcon"))
                        DBCreate(mf=self.mf, opts=opts)
                    sq.insRec("telcon", data=[
                        old[olddic["ph_name"][0]],
                        old[olddic["ph_contact"][0]], "", "",
                        old[olddic["ph_celno"][0]], ""], format=False)
            if self.mf.display:
                pb.closeProgress()
        self.mf.dbm.commitDbase()

    def doNoteConvert(self, sql, all):
        lstcoy = 0
        lstsys = ""
        lstkey = ""
        lstdat = 0
        data = ""
        for dat in all:
            if not data:
                lstcoy = dat[0]
                lstsys = dat[1]
                lstkey = dat[2]
                lstdat = dat[4]
                user   = dat[5]
                data   = dat[6]
                flag   = dat[7]
                date   = dat[8]
            if lstcoy == dat[0] and lstsys == dat[1] and lstkey == dat[2] and lstdat == dat[4]:
                data = "%s %s" % (data, dat[6])
            else:
                if data:
                    if   lstsys == "AR":
                        lstsys = "ASS"
                    elif lstsys == "DR":
                        lstsys = "DRS"
                    elif lstsys == "CR":
                        lstsys = "CRS"
                    elif lstsys == "GL":
                        lstsys = "GEN"
                    elif lstsys == "SL":
                        lstsys = "SLN"
                    elif lstsys == "ST":
                        lstsys = "STR"
                    elif lstsys == "WG":
                        lstsys = "WAG"
                    sql.insRec("ctlnot", data=[lstcoy, lstsys, lstkey,
                        lstdat, user, data, flag, date, 0], format=False)
                lstcoy = dat[0]
                lstsys = dat[1]
                lstkey = dat[2]
                lstdat = dat[4]
                data   = dat[6]
                user   = dat[5]
                data   = dat[6]
                flag   = dat[7]
                date   = dat[8]
        if data:
            sql.insRec("ctlnot", data=[lstcoy, lstsys, lstkey,
            lstdat, user, data, flag, date, 0], format=False)

    def doTelnoteConvert(self, sql, all):
        lstkey = ""
        lstdat = 0
        data = ""
        for dat in all:
            if not data:
                lstkey = dat[0]
                lstdat = dat[2]
                user   = dat[3]
                data   = dat[4]
                flag   = dat[5]
                date   = dat[6]
            if lstkey == dat[0] and lstdat == dat[2]:
                data = "%s %s" % (data, dat[4])
            else:
                if data:
                    sql.insRec("ctlnot", data=[0, "TEL", lstkey,
                    lstdat, user, data, flag, date, 0], format=False)
                lstkey = dat[0]
                lstdat = dat[2]
                data   = dat[4]
                user   = dat[3]
                data   = dat[4]
                flag   = dat[5]
                date   = dat[6]
        if data:
            sql.insRec("ctlnot", data=[0, "TEL", lstkey, lstdat,
            user, data, flag, date, 0], format=False)

    def doCleanDatabase(self):
        typ = "Cleaning Database ... Please Wait"
        if self.mf.display:
            self.mf.updateStatus(typ)
        else:
            print typ
        self.doGetTables()
        if self.sys:
            fle = os.path.join(self.pth, self.sys + ".sys")
        else:
            fle = os.path.join(self.pth, "tartan.sys")
        if os.path.exists(fle):
            fle = open(fle, "r")
        else:
            if not self.mf.display:
                scrn = None
            else:
                scrn = self.mf.body
            showError(scrn, "System File Error",
                "System File %s Not Found" % fle)
            return
        lin = fle.readlines()
        fle.close()
        csv = []
        for l in lin:
            l = l.replace(chr(10), "")
            l = l.replace(chr(13), "")
            l = l.replace(chr(34), "")
            if l in ("ftable", "ffield"):
                continue
            csv.append(l)
        if self.mf.display:
            max = len(self.tab) + len(csv)
            pb = ProgressBar(self.mf.body, max=max, typ=typ)
        self.regdte = 0
        for n1, t in enumerate(self.tab):
            if self.mf.display:
                self.mf.updateStatus(t)
                pb.displayProgress(n1)
            sql = Sql(self.mf.dbm, t, prog=__name__)
            if t not in csv:
                if t == "reglog":
                    try:
                        reg = sql.getRec(tables=t, limit=1)
                        self.regdte = int(reg[0].split()[0].replace("-", ""))
                    except:
                        pass
                sql.sqlRec("Drop table %s" % t, excp=True)
                self.doRemoveTable(t)
            elif self.mf.dbase not in ("PgSQL", "SQLite", "MsSQL"):
                tab = sql.sqlRec("Select count(*) from %s" % t, limit=1)
            else:
                if self.mf.dbase == "PgSQL":
                    column = "relname"
                    table = "pg_class"
                    where = "relname = '%s'" % t
                elif self.mf.dbase == "SQLite":
                    column = "name"
                    table = "SQLITE_MASTER"
                    where = "name = '%s' and type = 'table'" % t
                elif self.mf.dbase == "MsSQL":
                    column = "*"
                    table = "INFORMATION_SCHEMA.tables"
                    where = "where table_name = '%s'"
                tab = sql.sqlRec("Select %s from %s where %s" % (column,
                    table, where), limit=1)
                if not tab:
                    self.doRemoveTable(t)
        if self.mf.display:
            self.mf.updateStatus("")
        for n2, t in enumerate(csv):
            if self.mf.display:
                self.mf.updateStatus(t)
                pb.displayProgress(n1+n2)
            if t not in self.tab:
                opts = self.opts[:]
                opts.append(("-t", t))
                DBCreate(mf=self.mf, opts=opts)
                self.mf.dbm.commitDbase()
            else:
                self.doCreateIndexes(t, True, True)
        if self.mf.display:
            self.mf.updateStatus("")
            pb.closeProgress()

    def doCreateIndexes(self, table, check, commit):
        if (sys.platform == "win32" or self.mf.dbase == "FBird"):
            if table in ("memkon", "emllog"):
                return
        if self.mf.display:
            self.mf.updateStatus("Creating New Indexes for %s" % table)
        keys = self.sql.getRec(tables="ftable", where=[("ft_tabl", "=", table)])
        if not keys:
            return
        for key in keys:
            nam = "%s_key%s" % (table, str(key[2]))
            if check and self.doCheckIndex(table, nam):
                continue
            idx = ""
            for k in key[4:]:
                if k:
                    if idx:
                        idx = "%s,%s" % (idx, k)
                    else:
                        idx = k
            if key[3] == "N":
                sql = "Create index"
            else:
                sql = "Create unique index"
            try:
                self.mf.sqlRec("%s %s on %s (%s)" % (sql, nam, table, idx))
            except:
                pass
        if commit:
            self.mf.dbm.commitDbase()

    def doCheckIndex(self, table, index):
        if self.mf.dbase == "PgSQL":
            sel = "Select * from pg_indexes where schemaname='public' "\
                "and tablename='%s' and indexname='%s'" % (table, index)
            return self.mf.sqlRec(sel, limit=1)
        elif self.mf.dbase == "SQLite":
            sel = "Select * from sqlite_master where type='index' "\
                "and tbl_name='%s' and name='%s'" % (table, index)
            return self.mf.sqlRec(sel, limit=1)
        elif self.mf.dbase == "FBird":
            sel = "Select * from rdb$indices where rdb$relation_name in "\
                "('%s', '%s') and rdb$index_name in ('%s', '%s')" % \
                (table.lower(), table.upper(), index.lower(), index.upper())
            return self.mf.sqlRec(sel, limit=1)

    def doRemoveTable(self, t):
        self.sql.delRec("ftable", cols=["ft_tabl"], data=[t])
        self.sql.delRec("ffield", cols=["ff_tabl"], data=[t])
        self.mf.dbm.commitDbase()

    def doFixRef(self):
        if not "drstrn" in self.tab:
            return
        sql = Sql(self.mf.dbm, ["drstrn", "drsage"], prog=__name__)
        all = sql.getRec(tables="drstrn", cols=["drt_ref1", "drt_seq"])
        if all:
            for ref1, seq in all:
                ref1 = CCD(ref1, "Na", 9).work
                sql.updRec("drstrn", cols=["drt_ref1"], data=[ref1],
                    where=[("drt_seq", "=", seq)])
        all = sql.getRec(tables="drsage", cols=["dra_ref1", "dra_aref",
            "dra_seq"])
        if all:
            for ref1, aref, seq in all:
                ref1 = CCD(ref1, "Na", 9).work
                aref = CCD(aref, "Na", 9).work
                sql.updRec("drsage", cols=["dra_ref1", "dra_aref"],
                    data=[ref1, aref], where=[("dra_seq", "=", seq)])

if __name__ == "__main__":
    import getopt
    opts, args = getopt.getopt(sys.argv[1:], "hm:gd:u:p:r:s:t:f:c:")
    tb1020(opts)
