#! /usr/bin/python2

"""
SYNOPSIS
    Tables Data Maintenance i.e. Facility to change information in the
    database.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import CCD, SChoice, Sql, TartanDialog

class tb1030:
    def __init__(self, mf):
        self.mf = mf
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.doReset()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, "ftable", prog=__name__)
        self.doGetTable()
        if not self.table:
            return "no"
        self.sql = Sql(self.mf.dbm, ["ftable", self.table], prog=__name__)
        self.insrec = Sql(self.mf.dbm, self.table, prog=__name__)
        self.read = "N"

    def doGetTable(self):
        tit = ("Table to Fix/Amend",)
        tbl = {
            "stype": "R",
            "tables": ("ftable",),
            "cols": (
                ("ft_tabl", "", 0, "Table"),
                ("ft_desc", "", 0, "Description", "Y")),
            "where": [("ft_seq", "=", 1)],
            "screen": self.mf.body}
        fld = (
            (("T",0,0,0),"I@ft_tabl",0,"","",
                "","N",self.doTabNam,tbl,None,None),)
        tnd = ((self.doTabEnd,"y"), )
        txt = (self.doTabExit, )
        self.tb = TartanDialog(self.mf, tops=True, title=tit, eflds=fld,
            tend=tnd, txit=txt)
        self.mf.startLoop()

    def doTabNam(self, frt, pag, r, c, p, i, w):
        desc = self.sql.getRec(tables="ftable", cols=["ft_desc"],
            where=[("ft_tabl", "=", w)], limit=1)
        if not desc:
            return "Invalid Table"
        self.table = w
        self.desc = desc[0]

    def doTabExit(self):
        self.table = None
        self.doTabEnd()

    def doTabEnd(self):
        self.tb.closeProcess()
        self.mf.closeLoop()

    def mainProcess(self):
        exec "self.dics = self.sql.%s_dic" % self.table
        exec "self.lcol = self.sql.%s_col" % self.table
        self.pgs = ((len(self.lcol) - 1) / 15) + 1
        tit = [self.desc]
        fld = []
        for num, col in enumerate(self.lcol):
            if self.pgs == 1:
                pag = 0
            else:
                pag = int(num / 15) + 1
            if self.dics[col][2][0] == "D":
                typ = "I%s" % self.dics[col][2].lower()
            else:
                typ = "I%s" % self.dics[col][2]
            siz = self.dics[col][3]
            if siz > 50:
                siz = (50, siz)
            fld.append([["T", pag, num, 0], typ, siz, self.dics[col][4],
                self.dics[col][4], "", "N", None, None, None, None])
        but = (
            ("Display",None,self.doDisplay,None,None,None),
            ("Update",None,self.doUpdate,None,None,None),
            ("Delete",None,self.doDelete,None,None,None),
            ("Cancel",None,self.doReset,None,None,None),
            ("Exit",None,self.doExit,1,None,None))
        if self.pgs == 1:
            tag = None
            tnd = ((self.doEnd, "n"), )
            txt = (self.doExit, )
        else:
            tag = []
            tnd = [None]
            txt = [None]
            for pag in range(1, (self.pgs + 1)):
                tag.append(["Page-_%s" % pag, self.chgPage, ("T", 1, 1), None])
                tnd.append([self.doEnd, "n"])
                txt.append(self.doExit)
        if self.pgs == 1:
            self.df = TartanDialog(self.mf, tops=True, title=tit, eflds=fld,
                tags=tag, tend=tnd, txit=txt, butt=but, clicks=self.doClick)
        else:
            self.df = TartanDialog(self.mf, title=tit, eflds=fld, tags=tag,
                tend=tnd, txit=txt, butt=but, clicks=self.doClick)
        self.click = []

    def doClick(self, *opts):
        if self.click:
            self.df.loadEntry("T", self.click[0], self.click[1],
                data=self.df.t_work[self.click[0]][0][self.click[1]])
        self.click = opts[2]
        self.df.focusField("T", self.click[0], (self.click[1] + 1))

    def chgPage(self):
        self.df.focusField("T", self.df.pag, 1, clr=False)

    def doDisplay(self, widget=None):
        whr = []
        hed = []
        pos = 0
        for pag in range(self.df.pgs + 1):
            for num, fld in enumerate(self.df.t_work[pag][0]):
                hed.append(self.dics[self.lcol[pos]][5])
                if fld:
                    if type(fld) == str and fld.count("%"):
                        eql = "like"
                    else:
                        eql = "="
                    if self.dics[self.lcol[pos]][2][1] in ("A", "a"):
                        whr.append((self.lcol[pos], eql, fld))
                    else:
                        whr.append((self.lcol[pos], eql, fld))
                pos += 1
        keys = self.sql.getRec(tables="ftable", where=[("ft_tabl", "=",
            self.table), ("ft_seq", "=", 1)], limit=1)
        ord = ""
        for x in range(4, 14):
            if keys[x]:
                ord = "%s%s," % (ord, keys[x])
        ord = ord[:-1]
        recs = self.sql.getRec(tables=self.table, where=whr, order=ord)
        if recs:
            titl = "Available Records"
            dat = []
            for rec in recs:
                a = []
                for n, c in enumerate(self.lcol):
                    d = CCD(rec[n], self.dics[c][2], self.dics[c][3])
                    a.append(d.disp)
                dat.append(a)
            state = self.df.disableButtons()
            sc = SChoice(self.mf, scrn=self.mf.body, titl=titl, head=hed,
                data=dat, retn="I", mode="S")
            self.df.enableButtons(state=state)
            if sc.selection:
                self.read = "Y"
                self.olddata = recs[sc.selection[0]]
                for num, dat in enumerate(self.olddata):
                    if self.pgs == 1:
                        pag = 0
                        idx = num
                    else:
                        pag = int(num / 15) + 1
                        idx = num - ((pag - 1) * 15)
                    self.df.loadEntry("T", pag, idx, data=dat)
                self.df.setWidget(self.df.B0, "disabled")
                self.df.setWidget(self.df.B1, "normal")
                self.df.setWidget(self.df.B2, "normal")
                self.df.setWidget(self.df.B3, "normal")
            else:
                self.read = "N"
            frt, pag, row, col = self.df.first
            self.df.focusField(frt, pag, 1, clr=False)

    def doUpdate(self, widget=None):
        if self.read == "N":
            return
        self.sql.delRec(self.table, data=self.olddata)
        self.olddata = []
        newdata = []
        for pag in range((self.df.pgs + 1)):
            newdata.extend(self.df.t_work[pag][0])
        self.insrec.insRec(self.table, data=newdata)
        self.doReset()

    def doDelete(self, widget=None):
        if self.read == "N":
            return
        self.sql.delRec(self.table, data=self.olddata)
        self.olddata = []
        self.doReset()

    def doEnd(self):
        self.df.focusField(self.df.frt, self.df.pag, 1, clr=False)
        #if self.df.pag not in (0, self.pgs):
        #    self.df.selPage(self.df.tags[self.df.pag][0])
        #else:
        #    self.doUpdate()
        #    self.doReset()

    def doReset(self, widget=None):
        self.read = "N"
        if self.pgs == 1:
            self.df.focusField("T", 0, 1)
        else:
            self.df.selPage(self.df.tags[0][0])
            self.df.focusField("T", 1, 1)
        self.df.setWidget(self.df.B0, "normal")
        self.df.setWidget(self.df.B1, "disabled")
        self.df.setWidget(self.df.B2, "disabled")
        self.df.setWidget(self.df.B3, "disabled")
        self.click = []
        
    def doExit(self, widget=None):
        self.mf.dbm.commitDbase(ask=True)
        self.df.closeProcess()
        self.mf.closeLoop()

