#! /usr/bin/python2

"""
SYNOPSIS
    Utility to copy Tables from one database to another using different
    rcfiles.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import os, sys
from TartanClasses import DBCreate, MainFrame, ProgressBar, Sql, TartanDialog
from tartanFunctions import getSingleRecords, loadRcFile, showError

class tb1040:
    def __init__(self, mf, opts=None):
        self.mf = mf
        self.setVariables()
        if not opts:
            self.doDialog()
            self.mf.startLoop()
        else:
            self.rcf = None
            self.tables = None
            for o, v in opts:
                if o == "-r":
                    self.rcf = v
                elif o == "-t":
                    self.tables = []
                    tb = v.split(",")
                    for t in tb:
                        self.tables.append([t.strip()])
            if self.rcf and self.tables:
                check = self.doFrom("T", 0, 0, 1, 0, 0, self.rcf)
                if check:
                    print check
                else:
                    self.doCopyTables()
                    self.mff.dbm.closeDbase()

    def setVariables(self):
        sql = Sql(self.mf.dbm, ["verupd"], prog=__name__)
        self.vera = sql.getRec(tables="verupd")
        if self.vera:
            self.vera = self.vera[0][0]

    def doDialog(self):
        tit = ("Database File Copies",)
        self.tab = {
            "stype": "R",
            "tables": ("ftable",),
            "cols": (
                ("ft_tabl","",0,"TabNam"),
                ("ft_desc","",0,"Description")),
            "where": [("ft_seq", "=", 1)]}
        fle = {
            "stype": "F",
            "types": "fle",
            "initd": None,
            "ftype": [["RC Files", "*"]],
            "showh": "Y"}                        # Show hidden files
        r1s = (("Yes", "Y"), ("No", "N"))
        fld = (
            (("T",0,0,0),"IFF",50,"RC File From  ","From RC File",
                "","Y",self.doFrom,fle,None,None),
            (("T",0,1,0),("IRB",r1s),0,"Whole Database","Whole Database",
                "N","N",self.doWhole,None,None,None))
        tnd = ((self.doProcess,"y"), )
        txt = (self.doCloseProcess, )
        self.df = TartanDialog(self.mf, tops=True, title=tit, eflds=fld,
            tend=tnd, txit=txt)

    def doFrom(self, frt, pag, r, c, p, i, w):
        if sys.platform == "win32":
            if w.lower() == self.mf.rcdic["name"].lower():
                return "Invalid RC File (1)"
        elif w == self.mf.rcdic["name"]:
            return "Invalid RC File (2)"
        rcdic = loadRcFile(w)
        if rcdic == "error":
            return rcdic
        elif not rcdic:
            return "Invalid RC File (3)"
        try:
            self.mff = MainFrame(rcdic, display=False)
            self.mff.dbm.openDbase()
            self.sql = Sql(self.mff.dbm, ["verupd", "ftable", "ffield"],
                prog=__name__)
            verb = self.sql.getRec(tables="verupd")
            if verb:
                verb = verb[0][0]
            if verb != self.vera:
                if not self.mff.display:
                    return "Version Error"
                showError(self.mf.body, "Version Error",
                    "The version (%s) of the source database does not agree "\
                    "with the version (%s) of the receiving database.\n\n"\
                    "You must ensure that these version do agree before "\
                    "attempting to copy tables." % (verb, self.vera))
                return "Version Error"
        except:
            return "Invalid RC File (4)"
        if not self.mff.dbm.db:
            return "Invalid RC File (5)"

    def doWhole(self, frt, pag, r, c, p, i, w):
        self.whole = w

    def doProcess(self):
        self.df.closeProcess()
        if self.whole == "Y":
            self.tables = self.sql.getRec(tables="ftable", cols=["ft_tabl"],
                where=[("ft_seq", "=", 1)], order="ft_tabl")
        else:
            self.tables = getSingleRecords(self.mf, "ftable", ("ft_tabl",
                "ft_desc"), where=[("ft_seq", "=", 1)], order="ft_tabl")
        if not self.tables:
            showError(self.mf.body, "Table Error",
                 "No Tables Selected for Copying %s" % self.mf.dbname)
        else:
            self.doCopyTables()
        self.doCloseProcess()

    def doCopyTables(self):
        dir = os.path.join(self.mf.prgdir, "csv")
        if self.mf.display:
            p1 = ProgressBar(self.mf.body, max=len(self.tables),
                typ="Copying Database")
        ok = None
        for num, tab in enumerate(self.tables):
            if self.mf.display:
                p1.displayProgress(num)
            fle = os.path.join(dir, tab[0])
            if tab[0] == "tmpfle":
                continue
            if tab[0] not in ("ftable", "ffield"):
                if self.mf.display:
                    self.mf.updateStatus("Adding Table %s" % tab[0])
                else:
                    print "Adding Table %s" % tab[0]
                if os.path.isfile("%s_fld.csv" % fle) == 0:
                    print "NO", fle
                    continue
                opts = [("-d", dir), ("-t", tab[0]), ("-x", False)]
                if tab[0] in ("frelat", "wagrcv"):
                    opts.append(("-r", False))
                DBCreate(mf=self.mf, opts=opts)
            if self.mf.display:
                self.mf.updateStatus("Populating Table %s" % tab[0])
            else:
                print "Populating Table %s" % tab[0]
            sqt = Sql(self.mf.dbm, tab[0], prog=__name__)
            try:
                sqt.delRec(tab[0])
            except:
                pass
            sqf = Sql(self.mff.dbm, tab[0], prog=__name__)
            dat = sqf.getRec(tab[0])
            if dat:
                if self.mf.display:
                    p2 = ProgressBar(self.mf.body, max=len(dat),
                        typ="Populating Table %s" % tab[0])
                for n, d in enumerate(dat):
                    if self.mf.display:
                        p2.displayProgress(n)
                    no = sqt.insRec(tab[0], data=d)
                    if no:
                        if self.mf.display:
                            showError(self.mf.body, "Table Error", ok)
                        else:
                            print "Table Error", ok
                if self.mf.display:
                    p2.closeProgress()
                if no:
                    pass
                else:
                    self.mf.dbm.commitDbase()
        if self.mf.display:
            p1.closeProgress()

    def doCloseProcess(self):
        self.df.closeProcess()
        try:
            self.mff.dbm.closeDbase()
        except:
            pass
        self.mf.closeLoop()

if __name__ == "__main__":
    import getopt
    from TartanClasses import MainFrame
    opts, args = getopt.getopt(sys.argv[1:], "r:t:")
    mf = MainFrame(display=False)
    mf.dbm.openDbase()
    tb1040(mf, opts)
