#! /usr/bin/python

"""
SYNOPSIS
    This utility creates _fld.csv and _idx.csv files in the wrk directory
    for all tables in the database as well as pdf files.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import os, time
from pyPdf import PdfFileWriter, PdfFileReader
from TartanClasses import ProgressBar, RepPrt, Sql, TartanDialog
from tartanFunctions import doPrinter, getSingleRecords
from tartanWork import finsys

class tb3010:
    def __init__(self, mf):
        self.mf = mf
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()
        #self.printReport()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, "ftable", prog=__name__)
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])

    def mainProcess(self):
        tit = ("Table Details Listing",)
        dats = [["ALL", "All Systems"]]
        syss = finsys.keys()
        syss.sort()
        for s in syss:
            dats.append((s, finsys[s][0]))
        syss.append("ALL")
        sts = {
            "stype": "C",
            "titl": "Select the Required System",
            "data": dats}
        r1s = (("All", "A"), ("System", "S"), ("Singles", "X"))
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"Tables","Tables",
                "X","Y",self.doTab,None,None,None),
            (("T",0,1,0),"IUA",3,"System","System",
                "ALL","Y",self.doSys,sts,None,("in", syss)))
        tnd = ((self.doEnd,"y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doTab(self, frt, pag, r, c, p, i, w):
        self.seltab = w
        if self.seltab != "S":
            self.sys = None
            return "sk1"

    def doSys(self, frt, pag, r, c, p, i, w):
        if w == "MST":
            self.sys = "ctl"
        else:
            self.sys = w

    def doEnd(self):
        self.view = self.df.repprt[0][1]
        self.prtnam = self.df.repprt[0][2]
        self.mail = self.df.repprt[1][2]
        self.df.closeProcess()
        self.printReport()
        self.mf.closeLoop()

    def printReport(self):
        if self.seltab == "X":
            tabs = getSingleRecords(self.mf, "ftable",
            ("ft_tabl", "ft_desc"), where=[("ft_seq", "=", 1)])
        else:
            whr = [("ft_seq", "=", 1)]
            if self.seltab == "S":
                whr.append(("ft_tabl", "like", self.sys.lower() + "%"))
            tabs = self.sql.getRec(tables=["ftable"], cols=["ft_tabl",
                "ft_desc"], where=whr)
        if not tabs:
            return
        rep = os.path.join(self.mf.rcdic["wrkdir"], "%s.pdf" % __name__)
        # Create Output Class
        new = PdfFileWriter()
        p = ProgressBar(self.mf.body, max=len(tabs), esc=True)
        for num, tab in enumerate(tabs):
            can = p.displayProgress(num)
            if can == "yes":
                break
            cols = [
                ["ff_seq", "UI", 3.0,"Seq"],
                ["ff_name","NA",20.0,"Field Name"],
                ["ff_type","NA", 2.0,"Tp"],
                ["ff_size","NA", 6.1,"Size"],
                ["ff_desc","NA",30.0,"Description"],
                ["ff_head","NA",30.0,"Heading"]]
            whr = [("ff_tabl", "=", tab[0])]
            ord = "ff_seq"
            fld = tab[0] + "_fld"
            rp = RepPrt(self.mf, name=fld, tables=["ffield"],
                heads=["Table Fields for Table %s" % tab[0],], pbar=None,
                cols=cols, where=whr, order=ord, view="N", pdffl=fld)
            # Report Fld
            inp = PdfFileReader(file(rp.pdfnam, "rb"))
            for page in range(inp.getNumPages()):
                new.addPage(inp.getPage(page))
            cols = [["ft_desc","NA",30.0,"Table Description"],
                ["ft_seq","UI",2.0,"Sq"],
                ["ft_type","NA",1.0,"T"],
                ["ft_key0","NA",10.0,"1st-Key"],
                ["ft_key1","NA",10.0,"2nd-Key"],
                ["ft_key2","NA",10.0,"3rd-Key"],
                ["ft_key3","NA",10.0,"4th-Key"],
                ["ft_key4","NA",10.0,"5th-Key"],
                ["ft_key5","NA",10.0,"6th-Key"]]
            whr = [("ft_tabl", "=", tab[0])]
            ord = "ft_seq"
            idx = tab[0] + "_idx"
            rp = RepPrt(self.mf, name=idx, tables=["ftable"],
                heads=["Table Keys for Table %s" % tab[0],], pbar=None,
                cols=cols, where=whr, order=ord, view="N", pdffl=idx)
            # Report Idx
            inp = PdfFileReader(file(rp.pdfnam, "rb"))
            for page in range(inp.getNumPages()):
                new.addPage(inp.getPage(page))
        # Write Output File
        out = file(rep, "wb")
        new.write(out)
        out.close()
        doPrinter(mf=self.mf, conum=1, rep=rep, head="Table Fields "\
            "and Indexes for Table %s" % tab[0], view=self.view,
            prtnam=self.prtnam, mail=self.mail)
        p.closeProgress()

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()

if __name__ == "__main__":
    from TartanClasses import MainFrame
    mf = MainFrame()
    mf.dbm.openDbase()
    tb3010(mf)
