#! /usr/bin/env python

"""
SYNOPSIS
    Telephone Directory Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import CCD, NotesCreate, NotesPrint, ProgressBar, RepPrt
from TartanClasses import SChoice, Sql, TabPrt, TartanDialog
from tartanFunctions import showWarning, textFormat

class td1010:
    def __init__(self, mf, usr):
        self.mf = mf
        self.capnm = usr
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlnot", "telmst", "telcon"],
            prog=__name__)
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]

    def mainProcess(self):
        tit = ("Telephone Directory",)
        acc = self.sql.getRec(tables="telmst", cols=["count(*)"], limit=1)
        if acc[0] > 500:
            autoc = False
        else:
            autoc = True
        tdm = {
            "stype": "R",
            "tables": ("telmst",),
            "cols": (
                ("tdm_name", "", 30, "Name","Y"),
                ("tdm_telcd", "", 10, "Code"),
                ("tdm_telno", "", 20, "Telephone")),
            "order": "tdm_name",
            "autoc": autoc,
            "sort": "n"}
        self.fld = (
            (("T",0,0,0),"I@tdm_name",0,"","",
                "","Y",self.doName,tdm,None,("notblank",)),
            (("T",0,1,0),"I@tdm_adr1",0,"","",
                "","N",None,None,self.doDelAll,("efld",)),
            (("T",0,2,0),"I@tdm_adr2",0,"","",
                "","N",None,None,None,("efld",)),
            (("T",0,3,0),"I@tdm_adr3",0,"","",
                "","N",None,None,None,("efld",)),
            (("T",0,4,0),"I@tdm_pcode",0,"","",
                "","N",None,None,None,("efld",)),
            (("T",0,5,0),"I@tdm_telcd",0,"","",
                "","N",None,None,None,("efld",)),
            (("T",0,6,0),"I@tdm_telno",0,"","",
                "","N",None,None,None,("efld",)),
            (("T",0,7,0),"I@tdm_faxno",0,"","",
                "","N",None,None,None,("efld",)),
            (("T",0,8,0),"I@tdm_mobile",0,"","",
                "","N",None,None,None,("efld",)),
            (("T",0,9,0),"I@tdm_email",0,"","",
                "","N",None,None,None,("efld",)))
        but = (
            ("Go-Back",None,self.doUp,1,None,("T",0,1),"",1),
            ("Go-Forward",None,self.doDn,1,None,("T",0,1),"",1),
            ("Import",None,self.doImport,1,("T",0,1),("T",0,2),
                "Import Names, Addresses and Contact Details from "\
                "Creditor, Debtor and Member Ledgers, if Available",1),
            ("Notes",None,self.doNotes,1,("T",0,2),("T",0,1),"",1),
            ("Print",None,self.doPrint,1,None,None,"",1),
            ("Apply",None,self.doApply,0,("T",0,2),("T",0,1),"",1),
            ("Contacts",None,self.doContacts,0,("T",0,2),("T",0,1),"",2),
            ("Cancel",None,self.doCancel,0,("T",0,2),("T",0,1),"",2),
            ("Quit",None,self.doTopExit,1,None,None,"",2))
        tnd = ((self.doTopEnd,"y"),)
        txt = (self.doTopExit,)
        self.df = TartanDialog(self.mf, title=tit, eflds=self.fld, butt=but,
            tend=tnd, txit=txt)

    def doDetails(self):
        try:
            self.df.setWidget(self.df.B%i, "disabled")
        except:
            pass
        self.mf.updateStatus("")
        self.df.focusField("T", 0, self.df.last[1][0])

    def doName(self, frt, pag, r, c, p, i, w):
        self.name = w
        acc = self.sql.getRec(tables="telmst", where=[("tdm_name", "like",
            self.name + "%")], order="tdm_name asc", limit=1)
        if not acc:
            self.new = "y"
            self.con = 0
        else:
            self.new = "n"
            self.df.setWidget(self.df.B0, "normal")
            self.df.setWidget(self.df.B1, "normal")
            self.doLoadDetail(acc)

    def doDelAll(self):
        self.sql.delRec("telmst", where=[("tdm_name", "=", self.name)])
        self.sql.delRec("telcon", where=[("tdc_name", "=", self.name)])
        self.sql.delRec("ctlnot", where=[("not_cono", "=", 0),
            ("not_sys", "=", "TEL"), ("not_key", "=", self.name)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doTopEnd(self):
        if self.new == "y":
            self.sql.insRec("telmst", self.df.t_work[0][0])
        else:
            self.sql.updRec("telmst", data=self.df.t_work[0][0],
            where=[("tdm_name", "=", self.name)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doTopExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

    def doUp(self, widget=None):
        acc = self.sql.getRec(tables="telmst", where=[("tdm_name", "<",
            self.name)], order="tdm_name desc", limit=1)
        if acc:
            self.doLoadDetail(acc)
            self.df.focusField("T", 0, 2)
        else:
            showWarning(self.mf.body, "Beginning of File",
            "This is the beginning of the file.")

    def doDn(self, widget=None):
        acc = self.sql.getRec(tables="telmst", where=[("tdm_name", ">",
            self.name)], order="tdm_name asc", limit=1)
        if acc:
            self.doLoadDetail(acc)
            self.df.focusField("T", 0, 2, "N")
        else:
            showWarning(self.mf.body, "End of File",
            "You have reached the end of the file.")

    def doContacts(self, widget=None):
        # Display contacts and allow adding, editing etc.
        all = self.sql.getRec(tables="telcon", cols=["tdc_contact", "tdc_desig",
            "tdc_telno", "tdc_celno", "tdc_email"], where=[("tdc_name", "=",
            self.name)], order="tdc_contact")
        if not all:
            all = ["", "", "", "", ""]
        titl = "Contacts"
        head = ("Name", "Designation", "Telephone", "Mobile", "Email-Address")
        lin = {
            "stype": "C",
            "titl": titl,
            "head": head,
            "deco": True,
            "typs": [("NA",30,"Y"), ("NA",20), ("NA",20), ("NA",20), ("TX",50)],
            "data": all,
            "butt": [
                ("Add", self.doConAdd),
                ("Exit", self.doConExit)]}
        state = self.df.disableButtons()
        self.mf.updateStatus("Select a Contact to Edit or an Action")
        self.contyp = None
        chg = self.df.selChoice(lin)
        self.df.enableButtons(state=state)
        if chg and chg.selection:
            self.contyp = "chg"
            self.conchg = chg.selection
        if self.contyp in ("add", "chg"):
            self.doConChanges()
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def doConAdd(self, widget=None):
        self.contyp = "add"

    def doConExit(self, widget=None):
        self.contyp = "quit"

    def doConChanges(self):
        fld = []
        if self.contyp == "chg":
            tit = ("Change Contact",)
            but = (
                ("Delete",None,self.doChgDelete,1,None,None),
                ("Apply",None,self.doChgEnd,1,None,None),
                ("Quit",None,self.doChgExit,1,None,None))
            self.contact = self.conchg[0]
        else:
            tit = ("Add Contact",)
            but = None
        fld = (
            (("T",0,0,0),"I@tdc_contact",0,"","",
                "","N",None,None,None,("notblank",)),
            (("T",0,1,0),"I@tdc_desig",0,"","",
                "","N",None,None,None,("efld",)),
            (("T",0,2,0),"I@tdc_telno",0,"","",
                "","N",None,None,None,("efld",)),
            (("T",0,3,0),"I@tdc_celno",0,"","",
                "","N",None,None,None,("efld",)),
            (("T",0,4,0),"I@tdc_email",0,"","",
                "","N",None,None,None,("efld",)))
        state = self.df.disableButtons()
        self.rp = TartanDialog(self.mf, title=tit, newscn=(1,1),
            tops=True, eflds=fld, butt=but, tend=((self.doChgEnd,"n"),),
            txit=(self.doChgExit,))
        if self.contyp == "chg":
            for x in range(5):
                self.rp.loadEntry("T", 0, x, data=self.conchg[x])
            self.rp.focusField("T", 0, 1, clr=False)
        else:
            self.rp.focusField("T", 0, 1)
        self.mf.startLoop()
        self.df.enableButtons(state=state)

    def doChgDelete(self, widget=None):
        self.sql.delRec("telcon", where=[("tdc_name", "=", self.name),
            ("tdc_contact", "=", self.conchg[0])])
        self.mf.dbm.commitDbase()
        self.doChgExit()

    def doChgEnd(self, widget=None):
        data = [self.name]
        data.extend(self.rp.t_work[0][0])
        if self.contyp == "quit":
            self.doChgExit
        elif self.contyp == "add":
            self.sql.insRec("telcon", data=data)
        elif self.contyp == "chg":
            self.sql.updRec("telcon", data=data, where=[("tdc_name", "=",
                self.name), ("tdc_contact", "=", self.contact)])
        self.mf.dbm.commitDbase()
        self.doChgExit()

    def doChgExit(self, widget=None):
        self.rp.closeProcess()
        self.mf.closeLoop()

    def doImport(self, widget=None):
        tables = []
        commit = False
        sql = Sql(self.mf.dbm, tables=["ftable"], prog=__name__)
        crs = sql.getRec(tables="ftable", where=[("ft_tabl", "=", "crsmst")],
            cols=["count(*)"], limit=1)
        if crs[0]:
            commit = True
            sql = Sql(self.mf.dbm, tables="crsmst", prog=__name__)
            col = sql.crsmst_col
            all = sql.getRec(tables="crsmst")
            for rec in all:
                nam = rec[col.index("crm_name")]
                chk = self.sql.getRec(tables="telmst", cols=["count(*)"],
                    where=[("tdm_name", "=", nam)], limit=1)
                if not chk[0]:
                    data = [
                        nam,
                        rec[col.index("crm_add1")],
                        rec[col.index("crm_add2")],
                        rec[col.index("crm_add3")],
                        rec[col.index("crm_pcod")], "",
                        rec[col.index("crm_tel")],
                        rec[col.index("crm_fax")], "", ""]
                    self.sql.insRec("telmst", data=data)
                    con = rec[col.index("crm_mgr")]
                    eml = rec[col.index("crm_mgr_email")]
                    if con:
                        chk = self.sql.getRec(tables="telcon",
                            cols=["count(*)"], where=[("tdc_name", "=", nam),
                            ("tdc_contact", "=", con)])
                        if not chk[0][0]:
                            data = [nam, con, "Manager", "", "", eml]
                            self.sql.insRec("telcon", data=data)
                    con = rec[col.index("crm_acc")]
                    eml = rec[col.index("crm_acc_email")]
                    if con:
                        chk = self.sql.getRec(tables="telcon",
                            cols=["count(*)"], where=[("tdc_name", "=", nam),
                            ("tdc_contact", "=", con)])
                        if not chk[0][0]:
                            data = [nam, con, "Accounts", "", "", eml]
                            self.sql.insRec("telcon", data=data)
                    con = rec[col.index("crm_ord")]
                    eml = rec[col.index("crm_ord_email")]
                    if con:
                        chk = self.sql.getRec(tables="telcon",
                            cols=["count(*)"], where=[("tdc_name", "=", nam),
                            ("tdc_contact", "=", con)])
                        if not chk[0][0]:
                            data = [nam, con, "Orders", "", "", eml]
                            self.sql.insRec("telcon", data=data)
        drs = sql.getRec(tables="ftable", where=[("ft_tabl", "=", "drsmst")],
            cols=["count(*)"], limit=1)
        if drs[0]:
            commit = True
            sql = Sql(self.mf.dbm, tables="drsmst", prog=__name__)
            col = sql.drsmst_col
            all = sql.getRec(tables="drsmst")
            for rec in all:
                nam = rec[col.index("drm_name")]
                chk = self.sql.getRec(tables="telmst", cols=["count(*)"],
                    where=[("tdm_name", "=", nam)], limit=1)
                if not chk[0]:
                    data = [
                        nam,
                        rec[col.index("drm_add1")],
                        rec[col.index("drm_add2")],
                        rec[col.index("drm_add3")],
                        rec[col.index("drm_pcod")], "",
                        rec[col.index("drm_tel")],
                        rec[col.index("drm_fax")], "", ""]
                    self.sql.insRec("telmst", data=data)
                    con = rec[col.index("drm_mgr")]
                    eml = rec[col.index("drm_mgr_email")]
                    if con:
                        chk = self.sql.getRec(tables="telcon",
                            cols=["count(*)"], where=[("tdc_name", "=", nam),
                            ("tdc_contact", "=", con)])
                        if not chk[0][0]:
                            data = [nam, con, "Manager", "", "", eml]
                            self.sql.insRec("telcon", data=data)
                    con = rec[col.index("drm_acc")]
                    eml = rec[col.index("drm_acc_email")]
                    if con:
                        chk = self.sql.getRec(tables="telcon",
                            cols=["count(*)"], where=[("tdc_name", "=", nam),
                            ("tdc_contact", "=", con)])
                        if not chk[0][0]:
                            data = [nam, con, "Accounts", "", "", eml]
                            self.sql.insRec("telcon", data=data)
                    con = rec[col.index("drm_sls")]
                    eml = rec[col.index("drm_sls_email")]
                    if con:
                        chk = self.sql.getRec(tables="telcon",
                            cols=["count(*)"], where=[("tdc_name", "=", nam),
                            ("tdc_contact", "=", con)])
                        if not chk[0][0]:
                            data = [nam, con, "Orders", "", "", eml]
                            self.sql.insRec("telcon", data=data)
        mem = sql.getRec(tables="ftable", where=[("ft_tabl", "=", "memmst")],
            cols=["count(*)"], limit=1)
        if mem[0]:
            commit = True
            sql = Sql(self.mf.dbm, tables=["memmst", "memadd", "memkon"],
                prog=__name__)
            mm = sql.memmst_col
            ma = sql.memadd_col
            mk = sql.memkon_col
            all = sql.getRec(tables="memmst", where=[("mlm_state", "=", "A")])
            for rec in all:
                coy = rec[mm.index("mlm_cono")]
                num = rec[mm.index("mlm_memno")]
                nam = "%s, %s" % (rec[mm.index("mlm_surname")],
                    rec[mm.index("mlm_names")])
                chk = self.sql.getRec(tables="telmst", cols=["count(*)"],
                    where=[("tdm_name", "=", nam)], limit=1)
                if not chk[0]:
                    add = sql.getRec(tables="memadd", where=[("mla_cono",
                        "=", coy), ("mla_memno", "=", num), ("mla_type", "=",
                        "P")], limit=1)
                    if not add:
                        add = [coy, num, "P", "", "", "", "", "", "", ""]
                    add3 = add[ma.index("mla_add3")]
                    city = add[ma.index("mla_city")]
                    coun = add[ma.index("mla_country")]
                    if not add3:
                        if city:
                            add3 = city
                            if coun:
                                add3 = "%s, %s" % (add3, coun)
                        elif coun:
                            add3 = coun
                    kon = sql.getRec(tables=["memctk", "memkon"],
                        cols=["mck_type", "mlk_detail"], where=[("mlk_cono",
                        "=", coy), ("mlk_memno", "=", num),
                        ("mck_code=mlk_code",)])
                    tel = ""
                    fax = ""
                    cel = ""
                    eml = ""
                    for k in kon:
                        if k[0] == "E" and not eml:
                            eml = k[1]
                        elif k[0] == "H":
                            tel = k[1]
                        elif k[0] == "W" and not tel:
                            tel = k[1]
                        elif k[0] == "M":
                            cel = k[1]
                    data = [
                        nam,
                        add[ma.index("mla_add1")],
                        add[ma.index("mla_add2")],
                        add3,
                        add[ma.index("mla_code")],
                        "", tel, fax, cel, eml]
                    self.sql.insRec("telmst", data=data)
        if commit:
            self.mf.dbm.commitDbase(ask=True)

    def doLoadDetail(self, data):
        self.name = data[0]
        for x in range(0, self.df.topq[self.df.pag]):
            self.df.loadEntry(self.df.frt, self.df.pag, x, data=data[x])

    def doNotes(self, widget=None):
        state = self.df.disableButtons()
        self.df.mstAlw.hide()
        nf = NotesCreate(self.mf, 0, "", self.capnm, "TEL", self.name)
        self.df.mstAlw.show()
        self.df.enableButtons(state=state)
        self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def doPrint(self, widget=None):
        titl = "Select the required print options"
        if self.df.last[0][0] != 1:
            data = [
                ("1", "Print Current Card's Details"),
                ("2", "Print Current Card's Contacts")]
        else:
            data = []
        data.extend([
            ("3", "Print All Cards"),
            ("4", "Print All Notes")])
        ss = SChoice(self.mf, titl=titl, data=data)
        self.mf.updateStatus("")
        if ss.selection:
            state = self.df.disableButtons()
            self.df.mstAlw.hide()
            table = "telmst"
            if ss.selection[0] == "1" and self.df.last[0][0] != 1:
                whr = [("tdm_name", "=", self.name)]
                rp = TabPrt(self.mf, name=__name__, tabs=table, wher=whr)
            elif ss.selection[0] == "2" and self.df.last[0][0] != 1:
                head = ["Contacts for %s" % self.name]
                rp = RepPrt(self.mf, name=__name__, tables=["telcon"],
                    heads=head, cols=["tdc_contact", "tdc_desig", "tdc_telno",
                    "tdc_celno", "tdc_email"], where=[("tdc_name", "=",
                    self.name)], order="tdc_contact")
            elif ss.selection[0] == "3":
                head = ["Telephone Directory"]
                rp = RepPrt(self.mf, name=__name__, tables=[table], heads=head,
                    cols=["tdm_name", "tdm_telcd", "tdm_telno", "tdm_faxno",
                    "tdm_mobile", "tdm_email"], order="tdm_name")
            elif ss.selection[0] == "4":
                self.notes = NotesPrint(self.mf, 0, "", "TEL")
                if not self.notes.data:
                    pass
                else:
                    data = []
                    p = ProgressBar(self.mf.body, typ="Generating the Report",
                        max=len(self.notes.data))
                    for num, dat in enumerate(self.notes.data):
                        can = p.displayProgress(num)
                        if can == "yes":
                            break
                        desc = textFormat(dat[5], width=50)
                        for n, d in enumerate(desc):
                            if not n:
                                data.append([dat[2], dat[4],
                                CCD(dat[3], "d1", 10).disp, dat[6],
                                CCD(dat[7], "d1", 10).disp, d])
                            else:
                                data.append(["", "", "", "", "", d])
                    p.closeProgress()
                    if can != "yes":
                        name = __name__
                        head = ["Telephone Directory Notes Listing"]
                        cols = [
                            ["a", "NA", 30, "Name",      "y"],
                            ["b", "NA", 20, "User-Name", "y"],
                            ["c", "NA", 10, "Cap-Date",  "y"],
                            ["d", "UA",  1, "F",         "y"],
                            ["e", "NA", 10, "Act-Date",  "y"],
                            ["f", "NA", 50, "Details",   "y"]]
                        RepPrt(self.mf, name=name, tables=data, heads=head,
                            cols=cols, ttype="D", view=self.notes.view,
                            prtnam=self.notes.pnam, mail=self.notes.mail)
            self.df.mstAlw.show()
            self.df.enableButtons(state=state)
            self.df.focusField(self.df.frt, self.df.pag, self.df.col)

    def doApply(self, widget=None):
        if self.df.frt == "T":
            if self.new == "y":
                self.sql.insRec("telmst", self.df.t_work[0][0])
            else:
                self.sql.updRec("telmst", data=self.df.t_work[0][0],
                where=[("tdm_name", "=", self.name)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doCancel(self, widget=None):
        self.mf.dbm.rollbackDbase()
        self.df.focusField("T", 0, 1)

