#! /usr/bin/python2

"""
SYNOPSIS
    Salaries and Wages Master Report.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import RepPrt, TartanDialog

class wg3010:
    def __init__(self, mf, conum, conam):
        self.mf = mf
        self.conum = conum
        self.conam = conam
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdtt = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "Salaries and Wages Master Report")
        bch = {
            "stype": "R",
            "tables": ("ctlbra",),
            "cols": (
                ("bra_code", "", 0, "Cod"),
                ("bra_name", "", 0, "Name", "Y")),
            "where": [("bra_cono", "=", self.conum)]}
        dep = {
            "stype": "R",
            "tables": ("ctldep",),
            "cols": (
                ("dep_code", "", 0, "Cod"),
                ("dep_name", "", 0, "Name", "Y")),
            "where": [("dep_cono", "=", self.conum)],
            "whera": [["T", "dep_branch", 0, 0]]}
        fld = (
            (("T",0,0,0),"I@bra_code",0,"Branch Code","",
                "","N",self.doBchCod,bch,None,None),
            (("T",0,1,0),"I@dep_code",0,"Department Code","",
                "","N",self.doDepCod,dep,None,None,None))
        tnd = ((self.doEnd,"y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doBchCod(self, frt, pag, r, c, p, i, w):
        self.bch = w

    def doDepCod(self, frt, pag, r, c, p, i, w):
        self.dep = w

    def doEnd(self):
        self.df.closeProcess()
        name = __name__
        tables = ["wagmst"]
        heads = ["Salaries and Wages Employee Master Report"]
        cols = [
            ["wgm_empno",  "UI",  5, "EmpNo", "y"],
            ["wgm_sname",  "NA", 30, "Surname", "y"],
            ["wgm_fname",  "NA", 30, "Names", "y"],
            ["wgm_start",  "D1", 10, "Comm-Date", "y"],
            ["wgm_status", "NA",  3, "Sta", "y"],
            ["wgm_term",   "d1", 10, "Term-Date", "y"]]
        whr = [("wgm_cono", "=", self.conum)]
        opts = ""
        if self.bch:
            whr.append(("wgm_branch", "=", self.bch))
            opts = "Branch: %s" % self.bch
        if self.dep:
            whr.append(("wgm_dept", "=", self.dep))
            if not opts:
                opts = "Dept: %s" % self.dep
            else:
                opts = "%s Dept: %s" % (opts, self.dep)
        ord = "wgm_empno"
        if not opts:
            opts = None
        state = self.df.disableButtons()
        rpt = RepPrt(self.mf, conum=self.conum, conam=self.conam, name=name,
            tables=tables, heads=heads, cols=cols, where=whr, order=ord,
            opts=opts, view=self.df.repprt[0][1], prtnam=self.df.repprt[0][2],
            mail=self.df.repprt[1][2])
        self.df.enableButtons(state=state)
        self.closeProcess()

    def doExit(self):
        self.df.closeProcess()
        self.closeProcess()

    def closeProcess(self):
        self.mf.closeLoop()

