#! /usr/bin/python2

"""
SYNOPSIS
    Salaries and Wages EMP201 Report.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import ASD, CCD, Sql, TartanDialog
from tartanFunctions import doPrinter, doReportlab, getModName

class wg3030:
    def __init__(self, mf, conum, conam):
        self.mf = mf
        self.conum = conum
        self.conam = conam
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlbra", "ctldep", "wagmst", "wagtf1",
            "wagtf2"], prog=__name__)
        t = time.localtime()
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])

    def mainProcess(self):
        self.tit = ("%03i %s" % (self.conum, self.conam),
            "Salaries and Wages SARS EMP201 Report")
        fld = (
            (("T",0,0,0),"ID1",10,"Start Date","Start Date",
                "","N",self.doSDate,None,None,None),
            (("T",0,1,0),"ID1",10,"End Date","End Date",
                "","N",self.doEDate,None,None,None))
        tnd = ((self.doEnd,"y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=self.tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doSDate(self, frt, pag, r, c, p, i, w):
        self.sdatew = w
        self.sdated = self.df.t_disp[pag][0][i]

    def doEDate(self, frt, pag, r, c, p, i, w):
        if w < self.sdatew:
            return "Invalid Date"
        self.edatew = w
        self.edated = self.df.t_disp[pag][0][i]

    def doEnd(self):
        self.df.closeProcess()
        self.head = ("%03u %-30s %38s %10s" % (self.conum, self.conam,
            self.sysdttm, __name__))
        pdfnam = getModName(self.mf.wrkdir, __name__, self.conum, ext="pdf")
        self.c, self.ogn, self.cpi, self.ldg, self.lpp, self.suc, self.duc = \
            doReportlab(self.mf.prgdir, pdfnam, self.head)
        cod = self.sql.getRec(tables="wagedc", cols=["ced_code"],
            where=[("ced_cono", "=", self.conum), ("ced_type", "=", "E"),
            ("ced_taxcode", "<>", "N")])
        txbl = 0
        for c in cod:
            pay = self.sql.getRec(tables="wagtf2", cols=["sum(wt2_eamt)"],
                where=[("wt2_cono", "=", self.conum), ("wt2_type", "=", "E"),
                ("wt2_code", "=", c[0]), ("wt2_date", "between", self.sdatew,
                self.edatew)], limit=1)
            if pay and pay[0]:
                txbl = float(ASD(txbl) + ASD(pay[0]))
        amt = self.sql.getRec(tables="wagtf2", cols=["sum(wt2_eamt)"],
            where=[("wt2_cono", "=", self.conum), ("wt2_type", "=", "D"),
            ("wt2_code", "=", 1), ("wt2_date", "between", self.sdatew,
            self.edatew)], limit=1)
        if amt and amt[0]:
            paye = amt[0]
        else:
            paye = 0
        cod = self.sql.getRec(tables="wagedc", cols=["ced_code",
            "ced_sdlportion"], where=[("ced_cono", "=", self.conum),
            ("ced_type", "=", "E"), ("ced_sdlportion", "<>", 0)])
        sdl = 0
        for c in cod:
            pay = self.sql.getRec(tables="wagtf2", cols=["sum(wt2_eamt)"],
                where=[("wt2_cono", "=", self.conum), ("wt2_type", "=", "E"),
                ("wt2_code", "=", c[0]), ("wt2_date", "between", self.sdatew,
                self.edatew)], limit=1)
            if pay and pay[0]:
                sdl = float(ASD(sdl) + ASD(pay[0] * c[1] / 100.0))
        uifr = 0
        uifa = 0
        cod = self.sql.getRec(tables="wagedc", cols=["ced_code", "ced_eamt",
            "ced_ramt"], where=[("ced_cono", "=", self.conum), ("ced_type",
            "=", "D"), ("ced_ebase", "=", 6)], limit=1)
        if cod:
            uifr = float(ASD(cod[1]) + ASD(cod[2]))
            amt = self.sql.getRec(tables="wagtf2", cols=["sum(wt2_eamt)",
                "sum(wt2_ramt)"], where=[("wt2_cono", "=", self.conum),
                ("wt2_type", "=", "D"), ("wt2_code", "=", cod[0]),
                ("wt2_date", "between", self.sdatew, self.edatew)], limit=1)
            if amt and amt[0]:
                uifa = float(ASD(uifa) + ASD(amt[0]) + ASD(amt[1]))
        txbl = CCD(txbl, "SD", 13.2)
        paye = CCD(paye, "SD", 13.2)
        sdlp = CCD(sdl, "SD", 13.2)
        sdla = CCD((int(sdl)/100.0), "SD", 13.2)
        uifa = CCD(uifa, "SD", 13.2)
        uifp = CCD(int(uifa.work*100.0)/uifr, "SD", 13.2)
        # Print the Report
        self.t = self.c.beginText(self.ogn[0], self.ogn[1])
        self.t.setFont("Tartan-Bold", self.cpi, self.ldg)
        self.t.textLine(self.head)
        self.t.textLine()
        self.t.textLine("%-29s %-10s %-2s %-10s" % \
            ("SARS EMP201 Report for period", self.sdated, "to", self.edated))
        self.t.textLine()
        self.t.textLine("%-43s %12s %13s %13s" % \
            ("Details", "PAYE", "SDL", "UIF"))
        self.t.textLine("%s" % (self.suc * len(self.head)))
        self.t.setFont("Tartan", self.cpi, self.ldg)
        self.t.textLine("%-43s %13s %13s %13s" % \
            ("Remuneration/Leviable Amount", txbl.disp, sdlp.disp, uifp.disp))
        self.t.textLine()
        self.t.textLine("%-43s %13s %13s %13s" % \
            ("Amounts Payable", paye.disp, sdla.disp, uifa.disp))
        self.t.textLine()
        totl = CCD(float(ASD(paye.work) + ASD(sdla.work) + ASD(uifa.work)),
            "SD", 13.2)
        self.t.textLine("%-71s %13s" % ("Control Total", totl.disp))
        self.c.drawText(self.t)
        self.c.showPage()
        self.c.save()
        doPrinter(mf=self.mf, conum=self.conum, rep=pdfnam, head=self.tit,
            view=self.df.repprt[0][1], prtnam=self.df.repprt[0][2],
            mail=self.df.repprt[1][2])
        self.mf.closeLoop()
    
    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()
