#! /usr/bin/env python

"""
SYNOPSIS
    Wages Control File Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import Sql, TartanDialog

class wgc110:
    def __init__(self, mf, coy, nam, usr):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.user = usr
        no = self.setVariables()
        if not no:
            self.drawDialog()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["wagctl", "chglog"], prog=__name__)
        if self.sql.error:
            return "no"
        self.acc = self.sql.getRec("wagctl", where=[("ctw_cono", "=",
            self.conum)], limit=1)
        if not self.acc:
            self.new = True
            self.acc = [self.conum, "", 0, 0, 0, "N", "N", 0, 0, "", 61]
        else:
            self.new = False

    def drawDialog(self):
        tit = ("%03i %s" % (self.conum, self.conam),
                "Wages Control File Maintenance (%s)" % __name__)
        r1s = (("Yes","Y"),("No","N"))
        self.fld = (
            (("T",0,0,0),"INA",10,"Registration Number","",
                self.acc[1],"N",None,None,None,("notblank",)),
            (("T",0,1,0),"IUD",9.5,"Monthly Hours","",
                self.acc[2],"N",None,None,None,("notzero",)),
            (("T",0,2,0),"IUD",9.5,"Weekly Hours","",
                self.acc[3],"N",None,None,None,("notzero",)),
            (("T",0,3,0),"IUD",9.5,"Daily Hours","",
                self.acc[4],"N",None,None,None,("notzero",)),
            (("T",0,4,0),("IRB",r1s),0,"Diplomatic Immunity","",
                self.acc[5],"N",None,None,None,None),
            (("T",0,5,0),("IRB",r1s),0,"G/L Integration","",
                self.acc[6],"N",None,None,None,None),
            (("T",0,6,0),"IUD",6.2,"S/L Interest Rate","",
                self.acc[7],"N",None,None,None,("efld",)),
            (("T",0,7,0),"Id1",10,"Last Interest Date","",
                self.acc[8],"N",None,None,None,("efld",)),
            (("T",0,8,0),"INA",4,"Best Account Code","",
                self.acc[9],"N",None,None,None,("efld",)),
            (("T",0,9,0),"IUI",2,"Best Account Type","",
                self.acc[10],"N",None,None,None,("efld",)))
        but = (
            ("Accept",None,self.doAccept,0,("T",0,1),("T",0,0)),
            ("Quit",None,self.doExit,1,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=tit, eflds=self.fld, butt=but,
            tend=tnd, txit=txt)
        if not self.new:
            for n, f in enumerate(self.acc[1:]):
                self.df.loadEntry("T", 0, n, data=f)
            self.df.focusField("T", 0, 1, clr=False)

    def doEnd(self):
        data = [self.conum]
        for x in range(0, len(self.df.t_work[0][0])):
            data.append(self.df.t_work[0][0][x])
        if self.new:
            self.sql.insRec("wagctl", data=data)
        elif data != self.acc:
            self.sql.updRec("wagctl", data=data,
                where=[("ctw_cono", "=", self.conum)])
            dte = long("%04i%02i%02i%02i%02i%02i" % time.localtime()[:-3])
            for num, dat in enumerate(self.acc):
                if dat != data[num]:
                    self.sql.insRec("chglog", data=["wagctl", "U",
                        "%03i" % self.conum, self.sql.wagctl_col[num],
                        dte, self.user, str(dat), str(data[num]), 0])
        self.mf.dbm.commitDbase()
        self.doExit()

    def doAccept(self, widget=None):
        frt, pag, col, mes = self.df.doCheckFields()
        if mes:
            self.df.focusField(frt, pag, (col+1), err=mes)
        else:
            self.df.doEndFrame("T", 0, cnf="N")

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

