#! /usr/bin/env python

"""
SYNOPSIS
    Departments File Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import GetCtl, Sql, TartanDialog

class wgc310:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlmst", "ctlbra", "ctldep", "genmst"],
            prog=__name__)
        gc = GetCtl(self.mf)
        wagctl = gc.getCtl("wagctl", self.conum)
        if not wagctl:
            return "no"
        self.glint = wagctl["ctw_glint"]
        if self.glint == "N":
            self.co1 = 0
            self.sy1 = 0
            self.cm1 = 0
            self.co2 = 0
            self.sy2 = 0
            self.cm2 = 0

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
                "Departments File Maintenance (%s)" % __name__)
        bch = {
            "stype": "R",
            "tables": ("ctlbra",),
            "cols": (("bra_code", "", 0, "Cod"),
                ("bra_name", "", 0, "Name", "Y")),
            "where": [("bra_cono", "=", self.conum)]}
        dep = {
            "stype": "R",
            "tables": ("ctldep",),
            "cols": (("dep_code", "", 0, "Cod"),
                ("dep_name", "", 0, "Name", "Y")),
            "where": [("dep_cono", "=", self.conum)],
            "whera": [["T", "dep_branch", 0]]}
        coy = {
            "stype": "R",
            "tables": ("ctlmst",),
            "cols": (
                ("ctm_cono", "", 0, "Coy"),
                ("ctm_name", "", 0, "Name", "Y"))}
        glm = {
            "stype": "R",
            "tables": ("genmst",),
            "cols": (
                ("glm_acno", "", 0, "Acc-Num"),
                ("glm_desc", "", 0, "Description", "Y")),
            "where": [("glm_cono", "=", self.conum)]}
        self.fld = [
            (("T",0,0,0),"INA",3,"Branch","Branch Code",
                "","N",self.doBranch,bch,None,("efld",)),
            (("T",0,1,0),"IUI",1,"Department","Department Code",
                "","N",self.doDepartment,dep,None,("notzero",)),
            (("T",0,2,0),"INA",30,"Name","Name",
                "","N",self.doName,None,self.doDelete,("notblank",))]
        if self.glint == "Y":
            self.fld.extend([
            (("T",0,3,0),"IUI",3,"Debit  Company","Debit Company",
                self.conum,"N",self.doCoy,coy,None,("notzero",)),
            (("T",0,4,0),"IUI",7,"       Salary","Debit Salary",
                "","N",self.doSal,glm,None,("notzero",)),
            (("T",0,5,0),"IUI",7,"       Commission","Debit Commission",
                "","N",self.doCom,glm,None,("efld",)),
            (("T",0,6,0),"IUI",3,"Credit Company","Debit Company",
                self.conum,"N",self.doCoy,coy,None,("efld",)),
            (("T",0,7,0),"IUI",7,"       Salary","Credit Salary",
                "","N",self.doSal,glm,None,("efld",)),
            (("T",0,8,0),"IUI",7,"       Commission","Credit Commission",
                "","N",self.doCom,glm,None,("efld",))])
        but = (
            ("Accept",None,self.doAccept,0,("T",0,2),("T",0,0)),
            ("Cancel",None,self.doCancel,0,("T",0,2),("T",0,0)),
            ("Quit",None,self.doExit,1,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        abt = None
        self.df = TartanDialog(self.mf, title=tit, eflds=self.fld, butt=but,
            tend=tnd, txit=txt)

    def doBranch(self, frt, pag, r, c, p, i, w):
        acc = self.sql.getRec(tables="ctlbra", where=[("bra_cono", "=",
            self.conum), ("bra_code", "=", w)], limit=1)
        if not acc:
            return "Invalid Branch Code"
        self.branch = w

    def doDepartment(self, frt, pag, r, c, p, i, w):
        self.code = w
        acc = self.sql.getRec(tables="ctldep", where=[("dep_cono", "=",
            self.conum), ("dep_branch", "=", self.branch), ("dep_code",
            "=", self.code)], limit=1)
        if not acc:
            self.new = "Y"
            self.name = ""
            self.co1 = 0
            self.sl1 = 0
            self.cm1 = 0
            self.co2 = 0
            self.sl2 = 0
            self.cm2 = 0
        else:
            self.new = "N"
            self.name = acc[self.sql.ctldep_col.index("dep_name")]
            self.co1 = acc[self.sql.ctldep_col.index("dep_dr_coy")]
            self.sl1 = acc[self.sql.ctldep_col.index("dep_dr_sal")]
            self.cm1 = acc[self.sql.ctldep_col.index("dep_dr_com")]
            self.co2 = acc[self.sql.ctldep_col.index("dep_cr_coy")]
            self.sl2 = acc[self.sql.ctldep_col.index("dep_cr_sal")]
            self.cm2 = acc[self.sql.ctldep_col.index("dep_cr_com")]
            for x in range(1, (self.df.topq[pag] - 1)):
                self.df.loadEntry(frt, pag, p+x, data=acc[x+2])

    def doName(self, frt, pag, r, c, p, i, w):
        self.name = w

    def doCoy(self, frt, pag, r, c, p, i, w):
        coy = self.sql.getRec(tables="ctlmst", cols=["ctm_name "],
            where=[("ctm_cono", "=", w)], limit=1)
        if not coy:
            return "Invalid Company Number"
        if c == 4:
            self.co1 = w
        else:
            self.co2 = w

    def doSal(self, frt, pag, r, c, p, i, w):
        if w != 0:
            if c == 5:
                co = self.co1
            else:
                co = self.co2
            acc = self.sql.getRec(tables="genmst", cols=["glm_desc"],
                where=[("glm_cono", "=", co), ("glm_acno", "=", w)], limit=1)
            if not acc:
                return "Invalid Account Number"
        if c == 5:
            self.sl1 = w
        else:
            self.sl2 = w

    def doCom(self, frt, pag, r, c, p, i, w):
        if w != 0:
            if c == 6:
                co = self.co1
            else:
                co = self.co2
            acc = self.sql.getRec(tables="genmst", cols=["glm_desc"],
                where=[("glm_cono", "=", co), ("glm_acno", "=", w)], limit=1)
            if not acc:
                return "Invalid Account Number"
        if c == 6:
            self.cm1 = w
        else:
            self.cm2 = w

    def doDelete(self):
        dat = (self.conum, self.code)
        self.sql.delRec("ctldep", where=[("dep_cono", "=", self.conum),
            ("dep_branch", "=", self.branch), ("dep_code", "=", self.code)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doEnd(self):
        dat = [self.conum, self.branch, self.code, self.name, self.co1,
            self.sl1, self.cm1, self.co2, self.sl2, self.cm2]
        if self.new == "Y":
            self.sql.insRec("ctldep", data=dat)
        else:
            self.sql.updRec("ctldep", data=dat, where=[("dep_cono", "=",
            self.conum), ("dep_branch", "=", self.branch), ("dep_code",
            "=", self.code)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doAccept(self, widget=None):
        frt, pag, col, mes = self.df.doCheckFields()
        if mes:
            self.df.focusField(frt, pag, (col+1), err=mes)
        else:
            self.df.doEndFrame("T", 0, cnf="N")

    def doCancel(self, widget=None):
        self.mf.dbm.rollbackDbase()
        self.df.focusField("T", 0, 1)

    def doExit(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

