#! /usr/bin/env python

"""
SYNOPSIS
    Wages Earnings and Deductions File Maintenance.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

from TartanClasses import GetCtl, Sql, TartanDialog

class wgc510:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        no = self.setVariables()
        if not no:
            self.mainProcess()
            self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, ["ctlmst", "genmst", "wagrcv",
            "wagedc", "wagtf2"], prog=__name__)
        gc = GetCtl(self.mf)
        wagctl = gc.getCtl("wagctl", self.conum)
        if not wagctl:
            return "no"
        self.glint = wagctl["ctw_glint"]

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
                "Earnings and Deductions File Maintenance (%s)" % __name__)
        ced = {
            "stype": "R",
            "tables": ("wagedc",),
            "cols": (
                ("ced_code", "", 0, "Cde"),
                ("ced_desc", "", 0, "Description", "Y")),
            "where": [("ced_cono", "=", self.conum)],
            "whera": [["T", "ced_type", 0]],
            "order": "ced_code"}
        bas = {
            "stype": "C",
            "titl": "Available Base Rates",
            "head": ("C", "Description"),
            "data": (
                (1, "X * Normal Rate of Pay * Factor"),
                (2, "Normal Pay * Factor"),
                (3, "Normal Pay * Factor / 100"),
                (4, "X * Daily Rate of Pay * Factor"),
                (5, "X * Hourly Rate of Pay * Factor"),
                (6, "U.I.F. Pay * Factor"))}
        coy = {
            "stype": "R",
            "tables": ("ctlmst",),
            "cols": (
                ("ctm_cono", "", 0, "Coy"),
                ("ctm_name", "", 0, "Name", "Y"))}
        glm = {
            "stype": "R",
            "tables": ("genmst",),
            "cols": (
                ("glm_acno", "", 0, "Acc-Num"),
                ("glm_desc", "", 0, "Description", "Y")),
            "where": [("glm_cono", "=", self.conum)]}
        rcv = {
            "stype": "R",
            "tables": ("wagrcv",),
            "cols": (
                ("rcv_code", "", 0, "Code"),
                ("rcv_desc", "", 0, "Details", "Y"))}
        r1s = (("Variable","V"),("Fixed","F"))
        r2s = (("Amount","A"),("Rate","R"))
        r3s = (("Salary","S"),("Commission","C"))
        r4s = (("Amount","A"),("Rate","R"),("None","N"))
        r5s = (("Yes","Y"),("No","N"),("Notional","T"),
                ("One Time","O"),("Retrench","R"))
        r6s = (("Yes","Y"),("No","N"))
        self.fld = (
            (("T",0,0,0),"IUA",1,"Type","Earning/Deduction Type",
                "E","N",self.doType,None,None,("in", ("E", "D"))),
            (("T",0,0,0),"IUI",3,"Code","Code Number",
                "","N",self.doCode,ced,None,("notzero",)),
            (("T",0,0,0),"ONA",30,"Description"),
            (("T",1,0,0),"INA",30,"Description","Description",
                "","N",self.doDesc,None,self.doDelete,("notblank",)),
            (("T",1,1,0),("IRB",r1s),0,"Type","Type",
                "V","N",None,None,None,None),
            (("T",1,2,0),("IRB",r2s),0,"Employee Portion","Employee Portion",
                "A","N",None,None,None,None),
            (("T",1,3,0),"IUI",3,"         Base","Base",
                "","N",None,bas,None,("between",0,999)),
            (("T",1,4,0),"IUD",8.2,"         Amount","Amount",
                "","N",self.doAmount,None,None,("efld",)),
            (("T",1,5,0),"IUD",8.2,"         Limit","Limit",
                "","N",self.doElim,None,None,("efld",)),
            (("T",1,6,0),"IUI",3,"         GL/Cono","GL/Cono",
                "","N",self.doEglco,coy,None,("efld",)),
            (("T",1,6,0),"IUI",7,"         GL/Acno","GL/Acno",
                "","N",self.doEglno,glm,None,("efld",)),
            (("T",1,7,0),("IRB",r3s),0,"Earnings Type","Earnings Type",
                "S","N",self.doEarnType,None,None,None),
            (("T",1,8,0),("IRB",r4s),0,"Employer Portion","Employer Portion",
                "N","N",self.doEmployerPortion,None,None,None),
            (("T",1,9,0),"IUI",3,"         Base","Base",
                "","N",self.doEmployerBase,None,None,("between",0,999)),
            (("T",1,10,0),"IUD",8.2,"         Amount","Amount",
                "","N",None,None,None,("efld",)),
            (("T",1,11,0),"IUD",8.2,"         Limit","Limit",
                "","N",self.doRlim,None,None,("efld",)),
            (("T",1,12,0),"IUI",3,"         GL/Cono","GL/Cono",
                "","N",self.doRglco,coy,None,("efld",)),
            (("T",1,12,0),"IUI",7,"         GL/Acno","GL/Acno",
                "","N",self.doRglno,glm,None,("efld",)),
            (("T",1,13,0),("IRB",r5s),0,"Tax Code","Tax Code",
                "N","N",self.doTaxcode,None,None,None),
            (("T",1,14,0),"IUD",6.2,"Tax Portion","Tax Portion",
                "","N",self.doTaxportion,None,None,("efld",)),
            (("T",2,0,0),"IUI",4,"Rec Of Rev Code","Rec Of Rev Code",
                "","N",self.doRcv,rcv,None,("notzero",)),
            (("T",2,0,0),"ONA",30,""),
            (("T",2,1,0),("IRB",r6s),0,"Union Report","Include in Union Report",
                "N","N",self.doUniRep,None,None,None),
            (("T",2,2,0),("IRB",r6s),0,"Must Pay","Must Pay",
                "N","N",None,None,None,None),
            (("T",2,3,0),"IUI",2,"Balance Number","Balance Number",
                "","N",self.doBalNo,None,None,("between",0,3)),
            (("T",2,4,0),"IUD",5.2,"Hourly Limit","Hourly Limit",
                "","N",self.doLimit,None,None,("efld",)),
            (("T",2,5,0),("IRB",r6s),0,"Monthly Deduction","Monthly Deduction",
                "N","N",None,None,None,None),
            (("T",2,6,0),"IUD",6.2,"UIF Percentage","UIF Percentage",
                "","N",None,None,None,("efld",)),
            (("T",2,7,0),"IUD",6.2,"SDL Percentage","SDL Percentage",
                "","N",None,None,None,("efld",)))
        but = (
            ("Accept",None,self.doAccept,0,("T",1,1),("T",0,0)),
            ("Cancel",None,self.doCancel,0,("T",1,1),("T",0,0)),
            ("Quit",None,self.doExit0,1,None,None))
        tag = (
            ("Detail-_1",None,("T",0,0),("T",0,1)),
            ("Detail-_2",None,("T",0,0),("T",0,1)))
        tnd = ((self.doEnd0,"N"), (self.doEnd1,"N"), (self.doEnd2,"Y"))
        txt = (self.doExit0, self.doExit1, self.doExit2)
        self.df = TartanDialog(self.mf, title=tit, eflds=self.fld, tags=tag,
            butt=but, tend=tnd, txit=txt)

    def doType(self, frt, pag, r, c, p, i, w):
        self.rtype = w
        if self.rtype == "E":
            self.df.topf[1][15][5] = "Y"
        else:
            self.df.topf[1][15][5] = "Y"

    def doCode(self, frt, pag, r, c, p, i, w):
        self.code = w
        acc = self.sql.getRec(tables="wagedc", where=[("ced_cono", "=",
            self.conum), ("ced_type", "=", self.rtype), ("ced_code", "=",
            self.code)], limit=1)
        if not acc:
            self.new = True
        else:
            self.new = False
            d = 3
            self.df.loadEntry(frt, pag, p+1, data=acc[d])
            for pag in range(1, self.df.pgs+1):
                for x in range(0, self.df.topq[pag]):
                    if pag == 2 and x == 1:
                        continue
                    self.df.loadEntry("T", pag, x, data=acc[d])
                    if pag == 2 and not x:
                        ror = self.sql.getRec(tables="wagrcv", limit=1,
                            cols=["rcv_desc"], where=[("rcv_code", "=",
                            acc[d])])
                        if not ror:
                            desc = ""
                        else:
                            desc = ror[0]
                        self.df.loadEntry("T", 2, 1, data=desc)
                    d = d + 1

    def doDesc(self, frt, pag, r, c, p, i, w):
        self.df.loadEntry("T", 0, 2, data=self.df.t_disp[1][0][0])
        if self.rtype == "E" and self.code < 6:
            self.df.loadEntry("T", 1, p+1, data="V")
            self.df.loadEntry("T", 1, p+2, data="R")
            if self.code == 5:
                self.df.loadEntry("T", 1, p+3, data=4)
            else:
                self.df.loadEntry("T", 1, p+3, data=1)
            if self.code in (1, 5):
                self.df.loadEntry("T", 1, p+4, data=1)
                self.df.loadEntry("T", 1, p+5, data=0)
                if self.glint != "Y":
                    self.df.loadEntry("T", 1, p+6, data=0)
                    self.df.loadEntry("T", 1, p+7, data=0)
                    self.df.loadEntry("T", 1, p+8, data="S")
                    return "sk8"
                else:
                    return "sk5"
            else:
                return "sk3"

    def doDelete(self):
        t = self.sql.getRec(tables="wagtf2", cols=["count(*)"],
            where=[("wt2_cono", "=", self.conum), ("wt2_type", "=",
            self.rtype), ("wt2_code", "=", self.code)], limit=1)
        if t[0]:
            return "Transactions Exist, Not Deleted"
        self.sql.delRec("wagedc", where=[("ced_cono", "=", self.conum),
            ("ced_type", "=", self.rtype), ("ced_code", "=", self.code)])
        self.mf.dbm.commitDbase()
        self.df.focusField("T", 0, 1)

    def doAmount(self, frt, pag, r, c, p, i, w):
        if self.rtype == "E" and self.code < 6:
            self.df.loadEntry("T", 1, p+1, data=0)
            if self.glint != "Y":
                self.df.loadEntry("T", 1, p+2, data=0)
                self.df.loadEntry("T", 1, p+3, data=0)
                self.df.loadEntry("T", 1, p+4, data="S")
                return "sk3"
            else:
                return "sk1"

    def doElim(self, frt, pag, r, c, p, i, w):
        if self.glint != "Y":
            self.df.loadEntry("T", 1, p+1, data=0)
            self.df.loadEntry("T", 1, p+2, data=0)
            if self.rtype == "E":
                return "sk2"
            else:
                self.df.loadEntry("T", 1, p+3, data="")
                return "sk3"

    def doEglco(self, frt, pag, r, c, p, i, w):
        if self.rtype == "E" and self.code < 6 and not w:
            # Use department code for integration
            self.eglco = w
            self.df.loadEntry("T", 1, p+1, data=0)
            self.df.loadEntry("T", 1, p+2, data="S")
            self.df.loadEntry("T", 1, p+9, data="Y")
            self.df.loadEntry("T", 1, p+10, data=100.00)
            return "sk10"
        if w:
            self.eglco = w
        else:
            self.eglco = self.conum
        name = self.sql.getRec(tables="ctlmst", cols=["ctm_name"],
            where=[("ctm_cono", "=", self.eglco)], limit=1)
        if not name:
            return "Invalid Company"

    def doEglno(self, frt, pag, r, c, p, i, w):
        self.eglno = w
        if self.eglno:
            if not self.sql.getRec(tables="genmst", cols=["glm_desc"],
                where=[("glm_cono", "=", self.eglco), ("glm_acno", "=", w)],
                limit=1):
                return "Invalid Account"
        if self.rtype == "D":
            return "sk1"
        if self.code < 6:
            self.df.loadEntry("T", 1, p+1, data="S")
            self.df.loadEntry("T", 1, p+8, data="Y")
            self.df.loadEntry("T", 1, p+9, data=100.00)
            return "sk9"

    def doEarnType(self, frt, pag, r, c, p, i, w):
        if self.rtype == "E":
            return "sk6"

    def doEmployerPortion(self, frt, pag, r, c, p, i, w):
        if w == "N":
            for x in range(1, 6):
                self.df.loadEntry(frt, pag, p+x, data=0)
            return "sk5"

    def doEmployerBase(self, frt, pag, r, c, p, i, w):
        if not w:
            for x in range(1, 5):
                self.df.loadEntry(frt, pag, p+x, data=0)
            return "sk4"

    def doRlim(self, frt, pag, r, c, p, i, w):
        if self.glint != "Y":
            return "sk2"

    def doRglco(self, frt, pag, r, c, p, i, w):
        if not w:
            w = self.conum
        name = self.sql.getRec(tables="ctlmst", cols=["ctm_name"],
            where=[("ctm_cono", "=", w)], limit=1)
        if not name:
            return "Invalid Company"
        self.rglco = w

    def doRglno(self, frt, pag, r, c, p, i, w):
        if w:
            desc = self.sql.getRec(tables="genmst", cols=["glm_desc"],
                where=[("glm_cono", "=", self.rglco), ("glm_acno", "=", w)],
                limit=1)
            if not desc:
                return "Invalid Account"

    def doTaxcode(self, frt, pag, r, c, p, i, w):
        if self.rtype == "D" and w not in ("Y", "T", "N"):
            return "Invalid Selection"
        if w == "N":
            self.df.loadEntry("T", pag, p+1, data=0)
            return "sk1"
        if self.glint == "Y" and not self.eglno and w != "T":
            return "ff8|Invalid G/L Account Number"

    def doTaxportion(self, frt, pag, r, c, p, i, w):
        if not w:
            return "Invalid Tax Portion"

    def doRcv(self, frt, pag, r, c, p, i, w):
        if w == 4101:
            return "SITE Code Not Allowed"
        acc = self.sql.getRec(tables="wagrcv", cols=["rcv_desc"],
            where=[("rcv_code", "=", w)], limit=1)
        if not acc:
            return "Invalid Rec of Rev Code"
        else:
            self.df.loadEntry(frt, pag, p+1, data=acc[0])

    def doUniRep(self, frt, pag, r, c, p, i, w):
        self.df.loadEntry(frt, pag, p+1, data="N")
        if self.rtype == "E":
            if self.code in (1, 2, 3, 4, 5):
                self.df.loadEntry(frt, pag, p+2, data=0)
                self.df.loadEntry(frt, pag, p+3, data=0)
                self.df.loadEntry(frt, pag, p+4, data="N")
                self.df.loadEntry(frt, pag, p+5, data=100.00)
                return "sk5"
            return "sk4"
        return "sk1"

    def doBalNo(self, frt, pag, r, c, p, i, w):
        self.df.loadEntry(frt, pag, p+1, data=0)
        self.df.loadEntry(frt, pag, p+2, data="N")
        self.df.loadEntry(frt, pag, p+3, data="N")
        return "sk3"

    def doLimit(self, frt, pag, r, c, p, i, w):
        pass

    def doEnd0(self):
        self.df.focusField("T", 1, 1, clr=self.new)

    def doEnd1(self):
        self.df.selPage("Detail-_2")
        self.df.focusField("T", 2, 1, clr=self.new)

    def doEnd2(self):
        dat = [self.conum]
        for x in range(0, 2):
            dat.append(self.df.t_work[0][0][x])
        for x in range(0, len(self.df.t_work[1][0])):
            dat.append(self.df.t_work[1][0][x])
        for x in range(0, len(self.df.t_work[2][0])):
            if x in (0,2,3,4,5,6,7,8):
                dat.append(self.df.t_work[2][0][x])
        if self.new:
            self.sql.insRec("wagedc", data=dat)
        else:
            self.sql.updRec("wagedc", data=dat, where=[("ced_cono", "=",
            self.conum), ("ced_type", "=", self.rtype), ("ced_code", "=",
            self.code)])
        self.mf.dbm.commitDbase()
        self.df.last[0] = [0, 0]
        self.df.selPage("Detail-_1")
        self.df.focusField("T", 0, 1)

    def doAccept(self, widget=None):
        if self.rtype == "D":
            frt, pag, col, mes = self.df.doCheckFields(("T",0,None))
            if not mes:
                frt, pag, col, mes = self.df.doCheckFields(("T",1,(0,1,2,3,4,5,6,7,9,10,11,12,13,14)))
                if not mes:
                    if self.df.t_work[1][0][15] not in ("Y","N","T","O","R"):
                        frt = "T"
                        pag = 1
                        col = 15
                        mes = "Invalid Tax Code Selection"
                    if not mes:
                        frt, pag, col, mes = self.df.doCheckFields(("T",1,(16,)))
                        if not mes:
                            frt, pag, col, mes = self.df.doCheckFields(("T",2,(0,2,3,4,5,6)))
        else:
            frt, pag, col, mes = self.df.doCheckFields(("T",0,None))
            if not mes:
                frt, pag, col, mes = self.df.doCheckFields(("T",1,(0,1,2,3,4,5,6,7,8,15,16)))
                if not mes:
                    frt, pag, col, mes = self.df.doCheckFields(("T",2,(0,2,3)))
        if mes:
            if pag > 0 and pag != self.df.pag:
                if frt == "T":
                    self.df.last[pag][0] = col+1
                else:
                    self.df.last[pag][1] = col+1
                self.df.selPage(self.df.tags[pag - 1][0])
            self.df.focusField(frt, pag, (col+1), err=mes)
        else:
            self.doEnd2()

    def doCancel(self, widget=None):
        self.mf.dbm.rollbackDbase()
        self.df.focusField("T", 0, 1)

    def doExit0(self, widget=None):
        self.df.closeProcess()
        self.mf.closeLoop()

    def doExit1(self):
        self.df.focusField("T", 0, 1)

    def doExit2(self):
        self.df.selPage("Detail-_1")

