#! /usr/bin/env python

"""
SYNOPSIS
    Earnings and Deduction Codes Listing.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import RepPrt, Sql, TartanDialog
from tartanFunctions import showError

class wgc520:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, "wagedc", prog=__name__)
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
            "Earnings and Deductions Codes Listing (%s)" % __name__)
        r1s = (("Earnings","E"),("Deductions","D"),("Both","B"))
        r2s = (("Code Number","N"),("Description","D"))
        fld = (
            (("T",0,0,0),("IRB",r1s),0,"Report Type     ","Report Type",
                "E","Y",self.doType,None,None,None),
            (("T",0,1,0),("IRB",r2s),0,"Sort Order      ","Sort Order",
                "N","Y",self.doSort,None,None,None))
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"), mail=("Y","N"))

    def doType(self, frt, pag, r, c, p, i, w):
        self.rtyp = w

    def doSort(self, frt, pag, r, c, p, i, w):
        self.sort = w

    def doEnd(self):
        self.df.closeProcess()
        if self.sort == "N":
            ord = "ced_type desc, ced_code"
        else:
            ord = "ced_type desc, ced_desc"
        if self.rtyp in ("E", "D"):
            all = self.sql.getRec(tables="wagedc",
            where=[("ced_cono", "=", self.conum), ("ced_type", "=", self.rtyp)],
            order=ord)
        else:
            all = self.sql.getRec(tables="wagedc", where=[("ced_cono", "=",
            self.conum)], order=ord)
        if not all:
            showError(self.mf.body, "Transaction Error",
            "No Transactions Selected")
        else:
            self.printReport(all)
        self.mf.closeLoop()

    def printReport(self, all):
        descs = ["", "Type", "Code Number", "Description", "Fixed or Variable",
                "EMPLOYEE Rate or Amount", "         Based On",
                "         Amount",   "         Limit",
                "         G/L Cono", "         G/L Number", 
                "         Earnings Type",
                "EMPLOYER Rate or Amount", "         Based On",
                "         Amount", "         Limit", "         G/L Cono",
                "         G/L Number", "Tax Code", "Taxable Portion",
                "Receiver of Revenue Code", "Include in Union Report",
                "Must Pay", "Balance Number", "Hourly Limit",
                "Monthly Deduction", "UIF Portion", "SDL Portion"]
        tables = []
        for x in range(0, len(all)):
            for y in range(1, len(all[0])):
                lin = []
                lin.append("")
                lin.append(descs[y])
                lin.append(all[x][y])
                tables.append(lin)
        if self.rtyp == "E":
            typ = "Earnings"
        elif self.rtyp == "D":
            typ = "Deductions"
        else:
            typ = "Earnings and Deductions"
        heads = ["%s Codes Listing" % typ]
        cols = [["a","NA",15.0," "],
                ["b","RW",30.0,"Field Name"],
                ["c","NA",30.0,"Details"]]
        state = self.df.disableButtons()
        rp = RepPrt(self.mf, name=__name__, tables=tables, heads=heads,
            cols=cols, conum=self.conum, ttype="D", conam=self.conam,
            lines=27, view=self.df.repprt[0][1], prtnam=self.df.repprt[0][2],
            mail=self.df.repprt[1][2])
        self.df.enableButtons(state=state)

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()

