#! /usr/bin/env python

"""
SYNOPSIS
    Unions Codes Listing.

    This file is part of Tartan Systems (TARTAN).

AUTHOR
    Written by Paul Malherbe, <paul@tartan.co.za>

COPYING
    Copyright (C) 2004-2012 Paul Malherbe. Free use of this software is
    granted under the terms of the GNU General Public License (GPL) as
    published by the Free Software Foundation, either version 2 of the
    License, or (at your option) any later version.

    TARTAN is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY, without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License along
    with TARTAN, if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA or view/download
    it at http://www.gnu.org/licenses/gpl.txt
"""

import time
from TartanClasses import RepPrt, Sql, TartanDialog
from tartanFunctions import showError

class wgc620:
    def __init__(self, mf, coy, nam):
        self.mf = mf
        self.conum = coy
        self.conam = nam
        self.setVariables()
        self.mainProcess()
        self.mf.startLoop()

    def setVariables(self):
        self.sql = Sql(self.mf.dbm, "waguni", prog=__name__)
        t = time.localtime()
        self.sysdtw = (t[0] * 10000) + (t[1] * 100) + t[2]
        self.sysdttm = "(Printed on: %i/%02i/%02i at %02i:%02i)" % \
            (t[0], t[1], t[2], t[3], t[4])

    def mainProcess(self):
        tit = ("%03i %s" % (self.conum, self.conam),
                "Unions Listing (%s)" % __name__)
        r1s = (("Code Number","N"),("Name","D"))
        r2s = (("Yes","Y"),("No","N"))
        fld = (
            (("T",0,1,0),("IRB",r1s),0,"Sort Order","Sort Order",
                "N","Y",self.doSort,None,None,None),)
        tnd = ((self.doEnd,"Y"), )
        txt = (self.doExit, )
        self.df = TartanDialog(self.mf, title=tit, eflds=fld, tend=tnd,
            txit=txt, view=("N","V"))

    def doSort(self, frt, pag, r, c, p, i, w):
        self.sort = w

    def doEnd(self):
        self.df.closeProcess()
        if self.sort == "N":
            ord = "wgu_code"
        else:
            ord = "wgu_name"
        all = self.sql.getRec(tables="waguni", order=ord)
        if not all:
            showError(self.mf.body, "Transaction Error",
            "No Transactions Selected")
        else:
            self.printReport(all)
        self.mf.closeLoop()

    def printReport(self, all):
        descs = ["Code Number", "Name", "Address 1", "Address 2",
            "Address 3", "Pcod", "Contact", "Telephone Number", "Fax Number"]
        tables = []
        for x in range(0, len(all)):
            for y in range(0, len(all[0])-1):
                lin = []
                lin.append("")
                lin.append(descs[y])
                lin.append(all[x][y+1])
                tables.append(lin)
                if y == len(all[0]) - 2:
                    lin = []
                    for z in range(0, 3):
                        lin.append("")
                    for z in range(0, 3):
                        tables.append(lin)
                if x%6 == 5:
                    pass
        heads = ["Unions Master Listing"]
        cols = [["a","NA",15.0," "],
                ["b","NA",30.0,"Field Name"],
                ["c","NA",30.0,"Description"]]
        state = self.df.disableButtons()
        rp = RepPrt(self.mf, name=__name__, tables=tables, heads=heads,
            cols=cols, opts=opts, conum=self.conum, ttype="D",
            conam=self.conam, view=self.df.repprt[0][1],
            prtnam=self.df.repprt[0][2])
        self.df.enableButtons(state=state)

    def doExit(self):
        self.df.closeProcess()
        self.mf.closeLoop()

